@@ -739,7 +739,7 @@ BOOL WINAPI ReportEventW( HANDLE hEventLog, WORD wType, WORD wCategory, DWORD dw
  *  GuidCount          [I]   Number of elements in the TraceGuidReg array
  *  TraceGuidReg       [I/O] Array of TRACE_GUID_REGISTRATION structures
  *  MofImagePath       [I]   not supported, set to NULL
- *  MofResourceNmae    [I]   not supported, set to NULL
+ *  MofResourceName    [I]   not supported, set to NULL
  *  RegistrationHandle [O]   Provider's registration handle
  *
  * RETURNS
@@ -834,7 +834,18 @@ ULONG WINAPI UnregisterTraceGuids( TRACEHANDLE RegistrationHandle )
 ULONG WINAPI EventRegister( LPCGUID provider, PENABLECALLBACK callback, PVOID context, PREGHANDLE handle )
 {
     FIXME("%s, %p, %p, %p\n", debugstr_guid(provider), callback, context, handle);
-    return ERROR_CALL_NOT_IMPLEMENTED;
+
+    *handle = 0xdeadbeef;
+    return ERROR_SUCCESS;
+}
+
+/******************************************************************************
+ * EventUnregister [ADVAPI32.@]
+ */
+ULONG WINAPI EventUnregister( REGHANDLE handle )
+{
+    FIXME("%s: stub\n", wine_dbgstr_longlong(handle));
+    return ERROR_SUCCESS;
 }
 
 /******************************************************************************
@@ -847,6 +858,16 @@ BOOLEAN WINAPI EventEnabled( REGHANDLE handle, PCEVENT_DESCRIPTOR descriptor )
     return FALSE;
 }
 
+/******************************************************************************
+ * EventWrite [ADVAPI32.@]
+ */
+ULONG WINAPI EventWrite( REGHANDLE handle, PCEVENT_DESCRIPTOR descriptor, ULONG count,
+                         PEVENT_DATA_DESCRIPTOR data )
+{
+    FIXME("%s, %p, %u, %p: stub\n", wine_dbgstr_longlong(handle), descriptor, count, data);
+    return ERROR_SUCCESS;
+}
+
 /******************************************************************************
  * QueryTraceW [ADVAPI32.@]
  */
