@@ -1113,11 +1113,11 @@ LSTATUS WINAPI RegDeleteKeyExA( HKEY hkey, LPCSTR name, REGSAM access, DWORD res
     {
         if (!is_version_nt()) /* win95 does recursive key deletes */
         {
-            CHAR name[MAX_PATH];
+            CHAR sub[MAX_PATH];
 
-            while(!RegEnumKeyA(tmp, 0, name, sizeof(name)))
+            while(!RegEnumKeyA(tmp, 0, sub, sizeof(sub)))
             {
-                if(RegDeleteKeyExA(tmp, name, access, reserved))  /* recurse */
+                if(RegDeleteKeyExA(tmp, sub, access, reserved))  /* recurse */
                     break;
             }
         }
@@ -2486,7 +2486,7 @@ LSTATUS WINAPI RegConnectRegistryW( LPCWSTR lpMachineName, HKEY hKey,
 {
     LONG ret;
 
-    TRACE("(%s,%p,%p): stub\n",debugstr_w(lpMachineName),hKey,phkResult);
+    TRACE("(%s,%p,%p)\n",debugstr_w(lpMachineName), hKey, phkResult);
 
     if (!lpMachineName || !*lpMachineName) {
         /* Use the local machine name */
@@ -2679,7 +2679,7 @@ LSTATUS WINAPI RegLoadMUIStringW(HKEY hKey, LPCWSTR pwszValue, LPWSTR pwszBuffer
     LONG result;
         
     TRACE("(hKey = %p, pwszValue = %s, pwszBuffer = %p, cbBuffer = %d, pcbData = %p, "
-          "dwFlags = %d, pwszBaseDir = %s) stub\n", hKey, debugstr_w(pwszValue), pwszBuffer, 
+          "dwFlags = %d, pwszBaseDir = %s)\n", hKey, debugstr_w(pwszValue), pwszBuffer,
           cbBuffer, pcbData, dwFlags, debugstr_w(pwszBaseDir));
 
     /* Parameter sanity checks. */
@@ -2909,3 +2909,13 @@ LSTATUS WINAPI RegDeleteTreeA(HKEY hKey, LPCSTR lpszSubKey)
     RtlFreeUnicodeString( &lpszSubKeyW );
     return ret;
 }
+
+/******************************************************************************
+ * RegDisableReflectionKey [ADVAPI32.@]
+ *
+ */
+LONG WINAPI RegDisableReflectionKey(HKEY base)
+{
+    FIXME("%p: stub\n", base);
+    return ERROR_SUCCESS;
+}
