@@ -242,7 +242,7 @@ static HANDLE service_open_pipe(void)
             break;
         if (GetLastError() != ERROR_PIPE_BUSY)
             break;
-    } while (WaitNamedPipeW(szPipe, NMPWAIT_WAIT_FOREVER));
+    } while (WaitNamedPipeW(szPipe, NMPWAIT_USE_DEFAULT_WAIT));
     HeapFree(GetProcessHeap(), 0, szPipe);
 
     return handle;
@@ -366,7 +366,7 @@ static DWORD WINAPI service_control_dispatcher(LPVOID arg)
 
     if (pipe==INVALID_HANDLE_VALUE)
     {
-        ERR("failed to create control pipe error = %d\n", GetLastError());
+        WARN("failed to create control pipe error = %d\n", GetLastError());
         return 0;
     }
 
@@ -430,8 +430,7 @@ static DWORD WINAPI service_control_dispatcher(LPVOID arg)
                     !(service->full_access_handle = OpenServiceW( manager, data, GENERIC_READ|GENERIC_WRITE )))
                     FIXME( "failed to open service %s\n", debugstr_w(data) );
             }
-            result = service_handle_start(service, data + info.name_size,
-                                          data_size / sizeof(WCHAR) - info.name_size );
+            result = service_handle_start(service, data, data_size / sizeof(WCHAR));
             break;
         case WINESERV_SENDCONTROL:
             result = service_handle_control(service, info.control);
