@@ -184,13 +184,19 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetState(IAMMultiMediaStream* ifac
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_SetState(IAMMultiMediaStream* iface, STREAM_STATE NewState)
+static HRESULT WINAPI IAMMultiMediaStreamImpl_SetState(IAMMultiMediaStream* iface, STREAM_STATE new_state)
 {
     IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
+    HRESULT hr = E_INVALIDARG;
 
-    FIXME("(%p/%p)->() stub!\n", This, iface);
+    TRACE("(%p/%p)->(%u)\n", This, iface, new_state);
 
-    return E_NOTIMPL;
+    if (new_state == STREAMSTATE_RUN)
+        hr = IMediaControl_Run(This->media_control);
+    else if (new_state == STREAMSTATE_STOP)
+        hr = IMediaControl_Stop(This->media_control);
+
+    return hr;
 }
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetTime(IAMMultiMediaStream* iface, STREAM_TIME* pCurrentTime)
@@ -211,13 +217,13 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetDuration(IAMMultiMediaStream* i
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_Seek(IAMMultiMediaStream* iface, STREAM_TIME SeekTime)
+static HRESULT WINAPI IAMMultiMediaStreamImpl_Seek(IAMMultiMediaStream* iface, STREAM_TIME seek_time)
 {
     IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
 
-    FIXME("(%p/%p)->() stub!\n", This, iface);
+    TRACE("(%p/%p)->(%s)\n", This, iface, wine_dbgstr_longlong(seek_time));
 
-    return E_NOTIMPL;
+    return IMediaSeeking_SetPositions(This->media_seeking, &seek_time, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
 }
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetEndOfStream(IAMMultiMediaStream* iface, HANDLE* phEOS)
@@ -310,10 +316,19 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilterGraph(IAMMultiMediaStream
 static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilter(IAMMultiMediaStream* iface, IMediaStreamFilter** ppFilter)
 {
     IAMMultiMediaStreamImpl *This = impl_from_IAMMultiMediaStream(iface);
+    HRESULT hr = S_OK;
 
-    FIXME("(%p/%p)->(%p) stub!\n", This, iface, ppFilter); 
+    TRACE("(%p/%p)->(%p)\n", This, iface, ppFilter);
 
-    return E_NOTIMPL;
+    if (!ppFilter)
+        return E_POINTER;
+
+    *ppFilter = NULL;
+
+    if (This->media_stream_filter)
+        hr = IBaseFilter_QueryInterface(This->media_stream_filter, &IID_IMediaStreamFilter, (LPVOID*)ppFilter);
+
+    return hr;
 }
 
 static HRESULT WINAPI IAMMultiMediaStreamImpl_AddMediaStream(IAMMultiMediaStream* iface, IUnknown* pStreamObject, const MSPID* PurposeId,
@@ -324,9 +339,39 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_AddMediaStream(IAMMultiMediaStream
     IMediaStream* pStream;
     IMediaStream** pNewStreams;
 
-    FIXME("(%p/%p)->(%p,%s,%x,%p) partial stub!\n", This, iface, pStreamObject, debugstr_guid(PurposeId), dwFlags, ppNewStream);
+    TRACE("(%p/%p)->(%p,%s,%x,%p)\n", This, iface, pStreamObject, debugstr_guid(PurposeId), dwFlags, ppNewStream);
 
-    hr = mediastream_create((IMultiMediaStream*)iface, PurposeId, This->StreamType, &pStream);
+    if (!IsEqualGUID(PurposeId, &MSPID_PrimaryVideo) && !IsEqualGUID(PurposeId, &MSPID_PrimaryAudio))
+        return MS_E_PURPOSEID;
+
+    if (dwFlags & AMMSF_ADDDEFAULTRENDERER)
+    {
+        if (IsEqualGUID(PurposeId, &MSPID_PrimaryVideo))
+        {
+            /* Default renderer not supported by video stream */
+            return MS_E_PURPOSEID;
+        }
+        else
+        {
+            IBaseFilter* dsoundrender_filter;
+
+            /* Create the default renderer for audio */
+            hr = CoCreateInstance(&CLSID_DSoundRender, NULL, CLSCTX_INPROC_SERVER, &IID_IBaseFilter, (LPVOID*)&dsoundrender_filter);
+            if (SUCCEEDED(hr))
+            {
+                 hr = IGraphBuilder_AddFilter(This->pFilterGraph, dsoundrender_filter, NULL);
+                 IBaseFilter_Release(dsoundrender_filter);
+            }
+
+            /* No media stream created when the default renderer is used */
+            return hr;
+        }
+    }
+
+    if (IsEqualGUID(PurposeId, &MSPID_PrimaryVideo))
+        hr = ddrawmediastream_create((IMultiMediaStream*)iface, PurposeId, This->StreamType, &pStream);
+    else
+        hr = audiomediastream_create((IMultiMediaStream*)iface, PurposeId, This->StreamType, &pStream);
     if (SUCCEEDED(hr))
     {
         pNewStreams = CoTaskMemRealloc(This->pStreams, (This->nbStreams+1) * sizeof(IMediaStream*));
@@ -343,6 +388,12 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_AddMediaStream(IAMMultiMediaStream
             *ppNewStream = pStream;
     }
 
+    if (SUCCEEDED(hr))
+    {
+        /* Add stream to the media stream filter */
+        IMediaStreamFilter_AddMediaStream((IMediaStreamFilter*)This->media_stream_filter, (IAMMediaStream*)pStream);
+    }
+
     return hr;
 }
 
