@@ -27,8 +27,7 @@
 #include "winreg.h"
 #include "objbase.h"
 #include "oaidl.h"
-
-#define ATL_INITGUID
+#include "rpcproxy.h"
 #include "atliface.h"
 #include "atlbase.h"
 
@@ -78,7 +77,7 @@ typedef struct rep_list_str {
 } rep_list;
 
 typedef struct {
-    const IRegistrarVtbl *lpVtbl;
+    IRegistrar IRegistrar_iface;
     LONG ref;
     rep_list *rep;
 } Registrar;
@@ -89,6 +88,11 @@ typedef struct {
     DWORD len;
 } strbuf;
 
+static inline Registrar *impl_from_IRegistrar(IRegistrar *iface)
+{
+    return CONTAINING_RECORD(iface, Registrar, IRegistrar_iface);
+}
+
 static void strbuf_init(strbuf *buf)
 {
     buf->str = HeapAlloc(GetProcessHeap(), 0, 128*sizeof(WCHAR));
@@ -549,7 +553,7 @@ static HRESULT WINAPI Registrar_QueryInterface(IRegistrar *iface, REFIID riid, v
 
 static ULONG WINAPI Registrar_AddRef(IRegistrar *iface)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
     TRACE("(%p) ->%d\n", This, ref);
     return ref;
@@ -557,7 +561,7 @@ static ULONG WINAPI Registrar_AddRef(IRegistrar *iface)
 
 static ULONG WINAPI Registrar_Release(IRegistrar *iface)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
 
     TRACE("(%p) ->%d\n", This, ref);
@@ -571,7 +575,7 @@ static ULONG WINAPI Registrar_Release(IRegistrar *iface)
 
 static HRESULT WINAPI Registrar_AddReplacement(IRegistrar *iface, LPCOLESTR Key, LPCOLESTR item)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     int len;
     rep_list *new_rep;
 
@@ -595,7 +599,7 @@ static HRESULT WINAPI Registrar_AddReplacement(IRegistrar *iface, LPCOLESTR Key,
 
 static HRESULT WINAPI Registrar_ClearReplacements(IRegistrar *iface)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     rep_list *iter, *iter2;
 
     TRACE("(%p)\n", This);
@@ -619,7 +623,7 @@ static HRESULT WINAPI Registrar_ClearReplacements(IRegistrar *iface)
 static HRESULT WINAPI Registrar_ResourceRegisterSz(IRegistrar* iface, LPCOLESTR resFileName,
                 LPCOLESTR szID, LPCOLESTR szType)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     TRACE("(%p)->(%s %s %s)\n", This, debugstr_w(resFileName), debugstr_w(szID), debugstr_w(szType));
     return resource_register(This, resFileName, szID, szType, TRUE);
 }
@@ -627,35 +631,35 @@ static HRESULT WINAPI Registrar_ResourceRegisterSz(IRegistrar* iface, LPCOLESTR
 static HRESULT WINAPI Registrar_ResourceUnregisterSz(IRegistrar* iface, LPCOLESTR resFileName,
                 LPCOLESTR szID, LPCOLESTR szType)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     TRACE("(%p)->(%s %s %s)\n", This, debugstr_w(resFileName), debugstr_w(szID), debugstr_w(szType));
     return resource_register(This, resFileName, szID, szType, FALSE);
 }
 
 static HRESULT WINAPI Registrar_FileRegister(IRegistrar* iface, LPCOLESTR fileName)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     TRACE("(%p)->(%s)\n", This, debugstr_w(fileName));
     return file_register(This, fileName, TRUE);
 }
 
 static HRESULT WINAPI Registrar_FileUnregister(IRegistrar* iface, LPCOLESTR fileName)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     FIXME("(%p)->(%s)\n", This, debugstr_w(fileName));
     return file_register(This, fileName, FALSE);
 }
 
 static HRESULT WINAPI Registrar_StringRegister(IRegistrar* iface, LPCOLESTR data)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     TRACE("(%p)->(%s)\n", This, debugstr_w(data));
     return string_register(This, data, TRUE);
 }
 
 static HRESULT WINAPI Registrar_StringUnregister(IRegistrar* iface, LPCOLESTR data)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     TRACE("(%p)->(%s)\n", This, debugstr_w(data));
     return string_register(This, data, FALSE);
 }
@@ -663,7 +667,7 @@ static HRESULT WINAPI Registrar_StringUnregister(IRegistrar* iface, LPCOLESTR da
 static HRESULT WINAPI Registrar_ResourceRegister(IRegistrar* iface, LPCOLESTR resFileName,
                 UINT nID, LPCOLESTR szType)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     TRACE("(%p)->(%s %d %s)\n", iface, debugstr_w(resFileName), nID, debugstr_w(szType));
     return resource_register(This, resFileName, MAKEINTRESOURCEW(nID), szType, TRUE);
 }
@@ -671,7 +675,7 @@ static HRESULT WINAPI Registrar_ResourceRegister(IRegistrar* iface, LPCOLESTR re
 static HRESULT WINAPI Registrar_ResourceUnregister(IRegistrar* iface, LPCOLESTR resFileName,
                 UINT nID, LPCOLESTR szType)
 {
-    Registrar *This = (Registrar*)iface;
+    Registrar *This = impl_from_IRegistrar(iface);
     TRACE("(%p)->(%s %d %s)\n", This, debugstr_w(resFileName), nID, debugstr_w(szType));
     return resource_register(This, resFileName, MAKEINTRESOURCEW(nID), szType, FALSE);
 }
@@ -700,7 +704,7 @@ static HRESULT Registrar_create(const IUnknown *pUnkOuter, REFIID riid, void **p
         return E_NOINTERFACE;
 
     ret = HeapAlloc(GetProcessHeap(), 0, sizeof(Registrar));
-    ret->lpVtbl = &RegistrarVtbl;
+    ret->IRegistrar_iface.lpVtbl = &RegistrarVtbl;
     ret->ref = 1;
     ret->rep = NULL;
     *ppvObject = ret;
@@ -815,19 +819,6 @@ static HRESULT do_register_dll_server(IRegistrar *pRegistrar, LPCOLESTR wszDll,
     return hres;
 }
 
-static HRESULT do_register_server(BOOL do_register)
-{
-    static const WCHAR CLSID_RegistrarW[] =
-            {'C','L','S','I','D','_','R','e','g','i','s','t','r','a','r',0};
-    static const WCHAR atl_dllW[] = {'a','t','l','.','d','l','l',0};
-
-    WCHAR clsid_str[40];
-    const struct _ATL_REGMAP_ENTRY reg_map[] = {{CLSID_RegistrarW, clsid_str}, {NULL,NULL}};
-
-    StringFromGUID2(&CLSID_Registrar, clsid_str, sizeof(clsid_str)/sizeof(WCHAR));
-    return do_register_dll_server(NULL, atl_dllW, MAKEINTRESOURCEW(101), do_register, reg_map);
-}
-
 /***********************************************************************
  *           AtlModuleUpdateRegistryFromResourceD         [ATL.@]
  *
@@ -858,8 +849,7 @@ HRESULT WINAPI AtlModuleUpdateRegistryFromResourceD(_ATL_MODULEW* pM, LPCOLESTR
  */
 HRESULT WINAPI DllRegisterServer(void)
 {
-    TRACE("\n");
-    return do_register_server(TRUE);
+    return __wine_register_resources( hInst );
 }
 
 /***********************************************************************
@@ -867,8 +857,7 @@ HRESULT WINAPI DllRegisterServer(void)
  */
 HRESULT WINAPI DllUnregisterServer(void)
 {
-    TRACE("\n");
-    return do_register_server(FALSE);
+    return __wine_unregister_resources( hInst );
 }
 
 /***********************************************************************
