@@ -24,43 +24,45 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d10core);
 
+static void STDMETHODCALLTYPE d3d10_null_wined3d_object_destroyed(void *parent) {}
+
+const struct wined3d_parent_ops d3d10_null_wined3d_parent_ops =
+{
+    d3d10_null_wined3d_object_destroyed,
+};
+
 /* Inner IUnknown methods */
 
-static inline struct d3d10_device *d3d10_device_from_inner_unknown(IUnknown *iface)
+static inline struct d3d10_device *impl_from_IUnknown(IUnknown *iface)
 {
-    return (struct d3d10_device *)((char*)iface - FIELD_OFFSET(struct d3d10_device, inner_unknown_vtbl));
+    return CONTAINING_RECORD(iface, struct d3d10_device, IUnknown_inner);
 }
 
-static HRESULT STDMETHODCALLTYPE d3d10_device_inner_QueryInterface(IUnknown *iface, REFIID riid, void **object)
+static HRESULT STDMETHODCALLTYPE d3d10_device_inner_QueryInterface(IUnknown *iface, REFIID riid,
+        void **ppv)
 {
-    struct d3d10_device *This = d3d10_device_from_inner_unknown(iface);
+    struct d3d10_device *This = impl_from_IUnknown(iface);
 
-    TRACE("iface %p, riid %s, object %p\n", iface, debugstr_guid(riid), object);
+    TRACE("iface %p, riid %s, ppv %p\n", iface, debugstr_guid(riid), ppv);
 
-    if (IsEqualGUID(riid, &IID_IUnknown)
-            || IsEqualGUID(riid, &IID_ID3D10Device))
+    if (IsEqualGUID(riid, &IID_IUnknown) || IsEqualGUID(riid, &IID_ID3D10Device))
+        *ppv = &This->ID3D10Device_iface;
+    else if (IsEqualGUID(riid, &IID_IWineDXGIDeviceParent))
+        *ppv = &This->IWineDXGIDeviceParent_iface;
+    else
     {
-        IUnknown_AddRef((IUnknown *)This);
-        *object = This;
-        return S_OK;
+        WARN("%s not implemented, returning E_NOINTERFACE\n", debugstr_guid(riid));
+        *ppv = NULL;
+        return E_NOINTERFACE;
     }
 
-    if (IsEqualGUID(riid, &IID_IWineDXGIDeviceParent))
-    {
-        IWineDXGIDeviceParent_AddRef(&This->IWineDXGIDeviceParent_iface);
-        *object = &This->IWineDXGIDeviceParent_iface;
-        return S_OK;
-    }
-
-    WARN("%s not implemented, returning E_NOINTERFACE\n", debugstr_guid(riid));
-
-    *object = NULL;
-    return E_NOINTERFACE;
+    IUnknown_AddRef((IUnknown*)*ppv);
+    return S_OK;
 }
 
 static ULONG STDMETHODCALLTYPE d3d10_device_inner_AddRef(IUnknown *iface)
 {
-    struct d3d10_device *This = d3d10_device_from_inner_unknown(iface);
+    struct d3d10_device *This = impl_from_IUnknown(iface);
     ULONG refcount = InterlockedIncrement(&This->refcount);
 
     TRACE("%p increasing refcount to %u\n", This, refcount);
@@ -70,7 +72,7 @@ static ULONG STDMETHODCALLTYPE d3d10_device_inner_AddRef(IUnknown *iface)
 
 static ULONG STDMETHODCALLTYPE d3d10_device_inner_Release(IUnknown *iface)
 {
-    struct d3d10_device *This = d3d10_device_from_inner_unknown(iface);
+    struct d3d10_device *This = impl_from_IUnknown(iface);
     ULONG refcount = InterlockedDecrement(&This->refcount);
 
     TRACE("%p decreasing refcount to %u\n", This, refcount);
@@ -86,25 +88,28 @@ static ULONG STDMETHODCALLTYPE d3d10_device_inner_Release(IUnknown *iface)
 
 /* IUnknown methods */
 
-static HRESULT STDMETHODCALLTYPE d3d10_device_QueryInterface(ID3D10Device *iface, REFIID riid, void **object)
+static inline struct d3d10_device *impl_from_ID3D10Device(ID3D10Device *iface)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
-    TRACE("Forwarding to outer IUnknown\n");
-    return IUnknown_QueryInterface(This->outer_unknown, riid, object);
+    return CONTAINING_RECORD(iface, struct d3d10_device, ID3D10Device_iface);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d10_device_QueryInterface(ID3D10Device *iface, REFIID riid,
+        void **ppv)
+{
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    return IUnknown_QueryInterface(This->outer_unk, riid, ppv);
 }
 
 static ULONG STDMETHODCALLTYPE d3d10_device_AddRef(ID3D10Device *iface)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
-    TRACE("Forwarding to outer IUnknown\n");
-    return IUnknown_AddRef(This->outer_unknown);
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    return IUnknown_AddRef(This->outer_unk);
 }
 
 static ULONG STDMETHODCALLTYPE d3d10_device_Release(ID3D10Device *iface)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
-    TRACE("Forwarding to outer IUnknown\n");
-    return IUnknown_Release(This->outer_unknown);
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    return IUnknown_Release(This->outer_unk);
 }
 
 /* ID3D10Device methods */
@@ -123,10 +128,11 @@ static void STDMETHODCALLTYPE d3d10_device_PSSetShaderResources(ID3D10Device *if
             iface, start_slot, view_count, views);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_PSSetShader(ID3D10Device *iface, ID3D10PixelShader *shader)
+static void STDMETHODCALLTYPE d3d10_device_PSSetShader(ID3D10Device *iface,
+        ID3D10PixelShader *shader)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
-    struct d3d10_pixel_shader *ps = (struct d3d10_pixel_shader *)shader;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_pixel_shader *ps = unsafe_impl_from_ID3D10PixelShader(shader);
 
     TRACE("iface %p, shader %p\n", iface, shader);
 
@@ -140,20 +146,21 @@ static void STDMETHODCALLTYPE d3d10_device_PSSetSamplers(ID3D10Device *iface,
             iface, start_slot, sampler_count, samplers);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_VSSetShader(ID3D10Device *iface, ID3D10VertexShader *shader)
+static void STDMETHODCALLTYPE d3d10_device_VSSetShader(ID3D10Device *iface,
+        ID3D10VertexShader *shader)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
-    struct d3d10_vertex_shader *vs = (struct d3d10_vertex_shader *)shader;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_vertex_shader *vs = unsafe_impl_from_ID3D10VertexShader(shader);
 
     TRACE("iface %p, shader %p\n", iface, shader);
 
     wined3d_device_set_vertex_shader(This->wined3d_device, vs ? vs->wined3d_shader : NULL);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_DrawIndexed(ID3D10Device *iface,
-        UINT index_count, UINT start_index_location, INT base_vertex_location)
+static void STDMETHODCALLTYPE d3d10_device_DrawIndexed(ID3D10Device *iface, UINT index_count,
+        UINT start_index_location, INT base_vertex_location)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
 
     TRACE("iface %p, index_count %u, start_index_location %u, base_vertex_location %d.\n",
             iface, index_count, start_index_location, base_vertex_location);
@@ -162,10 +169,10 @@ static void STDMETHODCALLTYPE d3d10_device_DrawIndexed(ID3D10Device *iface,
     wined3d_device_draw_indexed_primitive(This->wined3d_device, start_index_location, index_count);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_Draw(ID3D10Device *iface,
-        UINT vertex_count, UINT start_vertex_location)
+static void STDMETHODCALLTYPE d3d10_device_Draw(ID3D10Device *iface, UINT vertex_count,
+        UINT start_vertex_location)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
 
     TRACE("iface %p, vertex_count %u, start_vertex_location %u\n",
             iface, vertex_count, start_vertex_location);
@@ -180,21 +187,22 @@ static void STDMETHODCALLTYPE d3d10_device_PSSetConstantBuffers(ID3D10Device *if
             iface, start_slot, buffer_count, buffers);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_IASetInputLayout(ID3D10Device *iface, ID3D10InputLayout *input_layout)
+static void STDMETHODCALLTYPE d3d10_device_IASetInputLayout(ID3D10Device *iface,
+        ID3D10InputLayout *input_layout)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_input_layout *layout = unsafe_impl_from_ID3D10InputLayout(input_layout);
 
     TRACE("iface %p, input_layout %p\n", iface, input_layout);
 
     wined3d_device_set_vertex_declaration(This->wined3d_device,
-            input_layout ? ((struct d3d10_input_layout *)input_layout)->wined3d_decl : NULL);
+            layout ? layout->wined3d_decl : NULL);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *iface,
-        UINT start_slot, UINT buffer_count, ID3D10Buffer *const *buffers,
-        const UINT *strides, const UINT *offsets)
+static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *iface, UINT start_slot,
+        UINT buffer_count, ID3D10Buffer *const *buffers, const UINT *strides, const UINT *offsets)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
     unsigned int i;
 
     TRACE("iface %p, start_slot %u, buffer_count %u, buffers %p, strides %p, offsets %p\n",
@@ -202,22 +210,24 @@ static void STDMETHODCALLTYPE d3d10_device_IASetVertexBuffers(ID3D10Device *ifac
 
     for (i = 0; i < buffer_count; ++i)
     {
+        struct d3d10_buffer *buffer = unsafe_impl_from_ID3D10Buffer(buffers[i]);
+
         wined3d_device_set_stream_source(This->wined3d_device, start_slot,
-                buffers[i] ? ((struct d3d10_buffer *)buffers[i])->wined3d_buffer : NULL,
-                offsets[i], strides[i]);
+                buffer ? buffer->wined3d_buffer : NULL, offsets[i], strides[i]);
     }
 }
 
 static void STDMETHODCALLTYPE d3d10_device_IASetIndexBuffer(ID3D10Device *iface,
         ID3D10Buffer *buffer, DXGI_FORMAT format, UINT offset)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_buffer *buffer_impl = unsafe_impl_from_ID3D10Buffer(buffer);
 
     TRACE("iface %p, buffer %p, format %s, offset %u.\n",
             iface, buffer, debug_dxgi_format(format), offset);
 
     wined3d_device_set_index_buffer(This->wined3d_device,
-            buffer ? ((struct d3d10_buffer *)buffer)->wined3d_buffer : NULL,
+            buffer_impl ? buffer_impl->wined3d_buffer : NULL,
             wined3dformat_from_dxgi_format(format));
     if (offset) FIXME("offset %u not supported.\n", offset);
 }
@@ -254,13 +264,14 @@ static void STDMETHODCALLTYPE d3d10_device_GSSetShader(ID3D10Device *iface, ID3D
     else WARN("iface %p, shader %p stub!\n", iface, shader);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_IASetPrimitiveTopology(ID3D10Device *iface, D3D10_PRIMITIVE_TOPOLOGY topology)
+static void STDMETHODCALLTYPE d3d10_device_IASetPrimitiveTopology(ID3D10Device *iface,
+        D3D10_PRIMITIVE_TOPOLOGY topology)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
 
     TRACE("iface %p, topology %s\n", iface, debug_d3d10_primitive_topology(topology));
 
-    wined3d_device_set_primitive_type(This->wined3d_device, (WINED3DPRIMITIVETYPE)topology);
+    wined3d_device_set_primitive_type(This->wined3d_device, (enum wined3d_primitive_type)topology);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_VSSetShaderResources(ID3D10Device *iface,
@@ -373,15 +384,14 @@ static void STDMETHODCALLTYPE d3d10_device_UpdateSubresource(ID3D10Device *iface
 static void STDMETHODCALLTYPE d3d10_device_ClearRenderTargetView(ID3D10Device *iface,
         ID3D10RenderTargetView *render_target_view, const FLOAT color_rgba[4])
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
-    struct wined3d_rendertarget_view *wined3d_view;
-    const WINED3DCOLORVALUE color = {color_rgba[0], color_rgba[1], color_rgba[2], color_rgba[3]};
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
+    struct d3d10_rendertarget_view *view = unsafe_impl_from_ID3D10RenderTargetView(render_target_view);
+    const struct wined3d_color color = {color_rgba[0], color_rgba[1], color_rgba[2], color_rgba[3]};
 
     TRACE("iface %p, render_target_view %p, color_rgba [%f %f %f %f]\n",
             iface, render_target_view, color_rgba[0], color_rgba[1], color_rgba[2], color_rgba[3]);
 
-    wined3d_view = ((struct d3d10_rendertarget_view *)render_target_view)->wined3d_view;
-    wined3d_device_clear_rendertarget_view(This->wined3d_device, wined3d_view, &color);
+    wined3d_device_clear_rendertarget_view(This->wined3d_device, view->wined3d_view, &color);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_ClearDepthStencilView(ID3D10Device *iface,
@@ -474,13 +484,14 @@ static void STDMETHODCALLTYPE d3d10_device_GSGetShader(ID3D10Device *iface, ID3D
     FIXME("iface %p, shader %p stub!\n", iface, shader);
 }
 
-static void STDMETHODCALLTYPE d3d10_device_IAGetPrimitiveTopology(ID3D10Device *iface, D3D10_PRIMITIVE_TOPOLOGY *topology)
+static void STDMETHODCALLTYPE d3d10_device_IAGetPrimitiveTopology(ID3D10Device *iface,
+        D3D10_PRIMITIVE_TOPOLOGY *topology)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
 
     TRACE("iface %p, topology %p\n", iface, topology);
 
-    wined3d_device_get_primitive_type(This->wined3d_device, (WINED3DPRIMITIVETYPE *)topology);
+    wined3d_device_get_primitive_type(This->wined3d_device, (enum wined3d_primitive_type *)topology);
 }
 
 static void STDMETHODCALLTYPE d3d10_device_VSGetShaderResources(ID3D10Device *iface,
@@ -621,7 +632,7 @@ static void STDMETHODCALLTYPE d3d10_device_Flush(ID3D10Device *iface)
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateBuffer(ID3D10Device *iface,
         const D3D10_BUFFER_DESC *desc, const D3D10_SUBRESOURCE_DATA *data, ID3D10Buffer **buffer)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
     struct d3d10_buffer *object;
     HRESULT hr;
 
@@ -642,7 +653,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateBuffer(ID3D10Device *iface,
         return hr;
     }
 
-    *buffer = (ID3D10Buffer *)object;
+    *buffer = &object->ID3D10Buffer_iface;
 
     TRACE("Created ID3D10Buffer %p\n", object);
 
@@ -658,9 +669,10 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture1D(ID3D10Device *ifac
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture2D(ID3D10Device *iface,
-        const D3D10_TEXTURE2D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data, ID3D10Texture2D **texture)
+        const D3D10_TEXTURE2D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data,
+        ID3D10Texture2D **texture)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
     struct d3d10_texture2d *object;
     HRESULT hr;
 
@@ -689,9 +701,10 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture2D(ID3D10Device *ifac
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateTexture3D(ID3D10Device *iface,
-        const D3D10_TEXTURE3D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data, ID3D10Texture3D **texture)
+        const D3D10_TEXTURE3D_DESC *desc, const D3D10_SUBRESOURCE_DATA *data,
+        ID3D10Texture3D **texture)
 {
-    struct d3d10_device *device = (struct d3d10_device *)iface;
+    struct d3d10_device *device = impl_from_ID3D10Device(iface);
     struct d3d10_texture3d *object;
     HRESULT hr;
 
@@ -771,7 +784,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateRenderTargetView(ID3D10Devic
     }
 
     TRACE("Created rendertarget view %p.\n", object);
-    *view = (ID3D10RenderTargetView *)object;
+    *view = &object->ID3D10RenderTargetView_iface;
 
     return S_OK;
 }
@@ -806,10 +819,11 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateDepthStencilView(ID3D10Devic
 }
 
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *iface,
-        const D3D10_INPUT_ELEMENT_DESC *element_descs, UINT element_count, const void *shader_byte_code,
-        SIZE_T shader_byte_code_length, ID3D10InputLayout **input_layout)
+        const D3D10_INPUT_ELEMENT_DESC *element_descs, UINT element_count,
+        const void *shader_byte_code, SIZE_T shader_byte_code_length,
+        ID3D10InputLayout **input_layout)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
     struct d3d10_input_layout *object;
     HRESULT hr;
 
@@ -835,7 +849,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *if
     }
 
     TRACE("Created input layout %p.\n", object);
-    *input_layout = (ID3D10InputLayout *)object;
+    *input_layout = &object->ID3D10InputLayout_iface;
 
     return S_OK;
 }
@@ -843,7 +857,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateInputLayout(ID3D10Device *if
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateVertexShader(ID3D10Device *iface,
         const void *byte_code, SIZE_T byte_code_length, ID3D10VertexShader **shader)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
     struct d3d10_vertex_shader *object;
     HRESULT hr;
 
@@ -866,7 +880,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateVertexShader(ID3D10Device *i
     }
 
     TRACE("Created vertex shader %p.\n", object);
-    *shader = (ID3D10VertexShader *)object;
+    *shader = &object->ID3D10VertexShader_iface;
 
     return S_OK;
 }
@@ -874,7 +888,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateVertexShader(ID3D10Device *i
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreateGeometryShader(ID3D10Device *iface,
         const void *byte_code, SIZE_T byte_code_length, ID3D10GeometryShader **shader)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
     struct d3d10_geometry_shader *object;
     HRESULT hr;
 
@@ -916,7 +930,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreateGeometryShaderWithStreamOutp
 static HRESULT STDMETHODCALLTYPE d3d10_device_CreatePixelShader(ID3D10Device *iface,
         const void *byte_code, SIZE_T byte_code_length, ID3D10PixelShader **shader)
 {
-    struct d3d10_device *This = (struct d3d10_device *)iface;
+    struct d3d10_device *This = impl_from_ID3D10Device(iface);
     struct d3d10_pixel_shader *object;
     HRESULT hr;
 
@@ -939,7 +953,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_device_CreatePixelShader(ID3D10Device *if
     }
 
     TRACE("Created pixel shader %p.\n", object);
-    *shader = (ID3D10PixelShader *)object;
+    *shader = &object->ID3D10PixelShader_iface;
 
     return S_OK;
 }
@@ -1293,22 +1307,22 @@ static inline struct d3d10_device *device_from_dxgi_device_parent(IWineDXGIDevic
 }
 
 static HRESULT STDMETHODCALLTYPE dxgi_device_parent_QueryInterface(IWineDXGIDeviceParent *iface,
-        REFIID riid, void **object)
+        REFIID riid, void **ppv)
 {
     struct d3d10_device *device = device_from_dxgi_device_parent(iface);
-    return d3d10_device_QueryInterface((ID3D10Device *)device, riid, object);
+    return IUnknown_QueryInterface(device->outer_unk, riid, ppv);
 }
 
 static ULONG STDMETHODCALLTYPE dxgi_device_parent_AddRef(IWineDXGIDeviceParent *iface)
 {
     struct d3d10_device *device = device_from_dxgi_device_parent(iface);
-    return d3d10_device_AddRef((ID3D10Device *)device);
+    return IUnknown_AddRef(device->outer_unk);
 }
 
 static ULONG STDMETHODCALLTYPE dxgi_device_parent_Release(IWineDXGIDeviceParent *iface)
 {
     struct d3d10_device *device = device_from_dxgi_device_parent(iface);
-    return d3d10_device_Release((ID3D10Device *)device);
+    return IUnknown_Release(device->outer_unk);
 }
 
 static struct wined3d_device_parent * STDMETHODCALLTYPE dxgi_device_parent_get_wined3d_device_parent(
@@ -1344,61 +1358,40 @@ static void CDECL device_parent_wined3d_device_created(struct wined3d_device_par
     device->wined3d_device = wined3d_device;
 }
 
-static HRESULT CDECL device_parent_create_surface(struct wined3d_device_parent *device_parent,
+static void CDECL device_parent_mode_changed(struct wined3d_device_parent *device_parent)
+{
+    TRACE("device_parent %p.\n", device_parent);
+}
+
+static HRESULT CDECL device_parent_create_texture_surface(struct wined3d_device_parent *device_parent,
         void *container_parent, UINT width, UINT height, enum wined3d_format_id format, DWORD usage,
-        WINED3DPOOL pool, UINT level, WINED3DCUBEMAP_FACES face, struct wined3d_surface **surface)
+        enum wined3d_pool pool, UINT level, enum wined3d_cubemap_face face, struct wined3d_surface **surface)
 {
     struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
-    struct d3d10_texture2d *texture;
-    D3D10_TEXTURE2D_DESC desc;
-    HRESULT hr;
 
-    FIXME("device_parent %p, container_parent %p, width %u, height %u, format %#x, usage %#x,\n"
-            "\tpool %#x, level %u, face %u, surface %p partial stub!\n",
+    TRACE("device_parent %p, container_parent %p, width %u, height %u, format %#x, usage %#x,\n"
+            "\tpool %#x, level %u, face %u, surface %p.\n",
             device_parent, container_parent, width, height, format, usage, pool, level, face, surface);
 
-    FIXME("Implement DXGI<->wined3d usage conversion\n");
-
-    desc.Width = width;
-    desc.Height = height;
-    desc.MipLevels = 1;
-    desc.ArraySize = 1;
-    desc.Format = dxgi_format_from_wined3dformat(format);
-    desc.SampleDesc.Count = 1;
-    desc.SampleDesc.Quality = 0;
-    desc.Usage = usage;
-    desc.BindFlags = 0;
-    desc.CPUAccessFlags = 0;
-    desc.MiscFlags = 0;
-
-    hr = d3d10_device_CreateTexture2D((ID3D10Device *)device, &desc, NULL, (ID3D10Texture2D **)&texture);
-    if (FAILED(hr))
-    {
-        ERR("CreateTexture2D failed, returning %#x\n", hr);
-        return hr;
-    }
-
-    *surface = texture->wined3d_surface;
-    wined3d_surface_incref(*surface);
-    ID3D10Texture2D_Release(&texture->ID3D10Texture2D_iface);
-
-    return S_OK;
+    return wined3d_surface_create(device->wined3d_device, width, height, format, level,
+            usage, pool, WINED3D_MULTISAMPLE_NONE, 0, WINED3D_SURFACE_TYPE_OPENGL, 0, container_parent,
+            &d3d10_null_wined3d_parent_ops, surface);
 }
 
-static HRESULT CDECL device_parent_create_rendertarget(struct wined3d_device_parent *device_parent,
-        void *container_parent, UINT width, UINT height, enum wined3d_format_id format,
-        WINED3DMULTISAMPLE_TYPE multisample_type, DWORD multisample_quality, BOOL lockable,
-        struct wined3d_surface **surface)
+static HRESULT CDECL device_parent_create_swapchain_surface(struct wined3d_device_parent *device_parent,
+        void *container_parent, UINT width, UINT height, enum wined3d_format_id format_id, DWORD usage,
+        enum wined3d_multisample_type multisample_type, DWORD multisample_quality, struct wined3d_surface **surface)
 {
     struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
+    struct wined3d_resource *sub_resource;
     struct d3d10_texture2d *texture;
     D3D10_TEXTURE2D_DESC desc;
     HRESULT hr;
 
-    FIXME("device_parent %p, container_parent %p, width %u, height %u, format %#x, multisample_type %#x,\n"
-            "\tmultisample_quality %u, lockable %u, surface %p partial stub!\n",
-            device_parent, container_parent, width, height, format, multisample_type,
-            multisample_quality, lockable, surface);
+    FIXME("device_parent %p, container_parent %p, width %u, height %u, format_id %#x, usage %#x,\n"
+            "\tmultisample_type %#x, multisample_quality %u, surface %p partial stub!\n",
+            device_parent, container_parent, width, height, format_id, usage,
+            multisample_type, multisample_quality, surface);
 
     FIXME("Implement DXGI<->wined3d usage conversion\n");
 
@@ -1406,7 +1399,7 @@ static HRESULT CDECL device_parent_create_rendertarget(struct wined3d_device_par
     desc.Height = height;
     desc.MipLevels = 1;
     desc.ArraySize = 1;
-    desc.Format = dxgi_format_from_wined3dformat(format);
+    desc.Format = dxgi_format_from_wined3dformat(format_id);
     desc.SampleDesc.Count = multisample_type ? multisample_type : 1;
     desc.SampleDesc.Quality = multisample_quality;
     desc.Usage = D3D10_USAGE_DEFAULT;
@@ -1414,55 +1407,16 @@ static HRESULT CDECL device_parent_create_rendertarget(struct wined3d_device_par
     desc.CPUAccessFlags = 0;
     desc.MiscFlags = 0;
 
-    hr = d3d10_device_CreateTexture2D((ID3D10Device *)device, &desc, NULL, (ID3D10Texture2D **)&texture);
+    hr = d3d10_device_CreateTexture2D(&device->ID3D10Device_iface, &desc, NULL,
+            (ID3D10Texture2D **)&texture);
     if (FAILED(hr))
     {
         ERR("CreateTexture2D failed, returning %#x\n", hr);
         return hr;
     }
 
-    *surface = texture->wined3d_surface;
-    wined3d_surface_incref(*surface);
-    ID3D10Texture2D_Release(&texture->ID3D10Texture2D_iface);
-
-    return S_OK;
-}
-
-static HRESULT CDECL device_parent_create_depth_stencil(struct wined3d_device_parent *device_parent,
-        UINT width, UINT height, enum wined3d_format_id format, WINED3DMULTISAMPLE_TYPE multisample_type,
-        DWORD multisample_quality, BOOL discard, struct wined3d_surface **surface)
-{
-    struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
-    struct d3d10_texture2d *texture;
-    D3D10_TEXTURE2D_DESC desc;
-    HRESULT hr;
-
-    FIXME("device_parent %p, width %u, height %u, format %#x, multisample_type %#x,\n"
-            "\tmultisample_quality %u, discard %u, surface %p partial stub!\n",
-            device_parent, width, height, format, multisample_type, multisample_quality, discard, surface);
-
-    FIXME("Implement DXGI<->wined3d usage conversion\n");
-
-    desc.Width = width;
-    desc.Height = height;
-    desc.MipLevels = 1;
-    desc.ArraySize = 1;
-    desc.Format = dxgi_format_from_wined3dformat(format);
-    desc.SampleDesc.Count = multisample_type ? multisample_type : 1;
-    desc.SampleDesc.Quality = multisample_quality;
-    desc.Usage = D3D10_USAGE_DEFAULT;
-    desc.BindFlags = D3D10_BIND_RENDER_TARGET;
-    desc.CPUAccessFlags = 0;
-    desc.MiscFlags = 0;
-
-    hr = d3d10_device_CreateTexture2D((ID3D10Device *)device, &desc, NULL, (ID3D10Texture2D **)&texture);
-    if (FAILED(hr))
-    {
-        ERR("CreateTexture2D failed, returning %#x\n", hr);
-        return hr;
-    }
-
-    *surface = texture->wined3d_surface;
+    sub_resource = wined3d_texture_get_sub_resource(texture->wined3d_texture, 0);
+    *surface = wined3d_surface_from_resource(sub_resource);
     wined3d_surface_incref(*surface);
     ID3D10Texture2D_Release(&texture->ID3D10Texture2D_iface);
 
@@ -1471,7 +1425,7 @@ static HRESULT CDECL device_parent_create_depth_stencil(struct wined3d_device_pa
 
 static HRESULT CDECL device_parent_create_volume(struct wined3d_device_parent *device_parent,
         void *container_parent, UINT width, UINT height, UINT depth, enum wined3d_format_id format,
-        WINED3DPOOL pool, DWORD usage, struct wined3d_volume **volume)
+        enum wined3d_pool pool, DWORD usage, struct wined3d_volume **volume)
 {
     HRESULT hr;
 
@@ -1492,22 +1446,23 @@ static HRESULT CDECL device_parent_create_volume(struct wined3d_device_parent *d
 }
 
 static HRESULT CDECL device_parent_create_swapchain(struct wined3d_device_parent *device_parent,
-        WINED3DPRESENT_PARAMETERS *present_parameters, struct wined3d_swapchain **swapchain)
+        struct wined3d_swapchain_desc *desc, struct wined3d_swapchain **swapchain)
 {
     struct d3d10_device *device = device_from_wined3d_device_parent(device_parent);
     IWineDXGIDevice *wine_device;
     HRESULT hr;
 
-    TRACE("device_parent %p, present_parameters %p, swapchain %p\n", device_parent, present_parameters, swapchain);
+    TRACE("device_parent %p, desc %p, swapchain %p\n", device_parent, desc, swapchain);
 
-    hr = d3d10_device_QueryInterface((ID3D10Device *)device, &IID_IWineDXGIDevice, (void **)&wine_device);
+    hr = d3d10_device_QueryInterface(&device->ID3D10Device_iface, &IID_IWineDXGIDevice,
+            (void **)&wine_device);
     if (FAILED(hr))
     {
         ERR("Device should implement IWineDXGIDevice\n");
         return E_FAIL;
     }
 
-    hr = IWineDXGIDevice_create_swapchain(wine_device, present_parameters, swapchain);
+    hr = IWineDXGIDevice_create_swapchain(wine_device, desc, swapchain);
     IWineDXGIDevice_Release(wine_device);
     if (FAILED(hr))
     {
@@ -1521,19 +1476,20 @@ static HRESULT CDECL device_parent_create_swapchain(struct wined3d_device_parent
 static const struct wined3d_device_parent_ops d3d10_wined3d_device_parent_ops =
 {
     device_parent_wined3d_device_created,
-    device_parent_create_surface,
-    device_parent_create_rendertarget,
-    device_parent_create_depth_stencil,
+    device_parent_mode_changed,
+    device_parent_create_swapchain_surface,
+    device_parent_create_texture_surface,
     device_parent_create_volume,
     device_parent_create_swapchain,
 };
 
 void d3d10_device_init(struct d3d10_device *device, void *outer_unknown)
 {
-    device->vtbl = &d3d10_device_vtbl;
-    device->inner_unknown_vtbl = &d3d10_device_inner_unknown_vtbl;
+    device->ID3D10Device_iface.lpVtbl = &d3d10_device_vtbl;
+    device->IUnknown_inner.lpVtbl = &d3d10_device_inner_unknown_vtbl;
     device->IWineDXGIDeviceParent_iface.lpVtbl = &d3d10_dxgi_device_parent_vtbl;
     device->device_parent.ops = &d3d10_wined3d_device_parent_ops;
     device->refcount = 1;
-    device->outer_unknown = outer_unknown;
+    /* COM aggregation always takes place */
+    device->outer_unk = outer_unknown;
 }
