@@ -37,7 +37,7 @@ static struct wined3d_resource *wined3d_resource_from_resource(ID3D10Resource *r
             return wined3d_buffer_get_resource(((struct d3d10_buffer *)resource)->wined3d_buffer);
 
         case D3D10_RESOURCE_DIMENSION_TEXTURE2D:
-            return wined3d_surface_get_resource(((struct d3d10_texture2d *)resource)->wined3d_surface);
+            return wined3d_texture_get_resource(((struct d3d10_texture2d *)resource)->wined3d_texture);
 
         default:
             FIXME("Unhandled resource dimension %#x.\n", dimension);
@@ -291,6 +291,11 @@ HRESULT d3d10_depthstencil_view_init(struct d3d10_depthstencil_view *view)
     return S_OK;
 }
 
+static inline struct d3d10_rendertarget_view *impl_from_ID3D10RenderTargetView(ID3D10RenderTargetView *iface)
+{
+    return CONTAINING_RECORD(iface, struct d3d10_rendertarget_view, ID3D10RenderTargetView_iface);
+}
+
 /* IUnknown methods */
 
 static HRESULT STDMETHODCALLTYPE d3d10_rendertarget_view_QueryInterface(ID3D10RenderTargetView *iface,
@@ -316,7 +321,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_rendertarget_view_QueryInterface(ID3D10Re
 
 static ULONG STDMETHODCALLTYPE d3d10_rendertarget_view_AddRef(ID3D10RenderTargetView *iface)
 {
-    struct d3d10_rendertarget_view *This = (struct d3d10_rendertarget_view *)iface;
+    struct d3d10_rendertarget_view *This = impl_from_ID3D10RenderTargetView(iface);
     ULONG refcount = InterlockedIncrement(&This->refcount);
 
     TRACE("%p increasing refcount to %u\n", This, refcount);
@@ -326,7 +331,7 @@ static ULONG STDMETHODCALLTYPE d3d10_rendertarget_view_AddRef(ID3D10RenderTarget
 
 static ULONG STDMETHODCALLTYPE d3d10_rendertarget_view_Release(ID3D10RenderTargetView *iface)
 {
-    struct d3d10_rendertarget_view *This = (struct d3d10_rendertarget_view *)iface;
+    struct d3d10_rendertarget_view *This = impl_from_ID3D10RenderTargetView(iface);
     ULONG refcount = InterlockedDecrement(&This->refcount);
 
     TRACE("%p decreasing refcount to %u\n", This, refcount);
@@ -378,7 +383,7 @@ static HRESULT STDMETHODCALLTYPE d3d10_rendertarget_view_SetPrivateDataInterface
 static void STDMETHODCALLTYPE d3d10_rendertarget_view_GetResource(ID3D10RenderTargetView *iface,
         ID3D10Resource **resource)
 {
-    struct d3d10_rendertarget_view *This = (struct d3d10_rendertarget_view *)iface;
+    struct d3d10_rendertarget_view *This = impl_from_ID3D10RenderTargetView(iface);
     struct wined3d_resource *wined3d_resource;
     IUnknown *parent;
     HRESULT hr;
@@ -408,7 +413,7 @@ static void STDMETHODCALLTYPE d3d10_rendertarget_view_GetResource(ID3D10RenderTa
 static void STDMETHODCALLTYPE d3d10_rendertarget_view_GetDesc(ID3D10RenderTargetView *iface,
         D3D10_RENDER_TARGET_VIEW_DESC *desc)
 {
-    struct d3d10_rendertarget_view *This = (struct d3d10_rendertarget_view *)iface;
+    struct d3d10_rendertarget_view *This = impl_from_ID3D10RenderTargetView(iface);
 
     TRACE("iface %p, desc %p\n", iface, desc);
 
@@ -438,7 +443,7 @@ HRESULT d3d10_rendertarget_view_init(struct d3d10_rendertarget_view *view,
     struct wined3d_resource *wined3d_resource;
     HRESULT hr;
 
-    view->vtbl = &d3d10_rendertarget_view_vtbl;
+    view->ID3D10RenderTargetView_iface.lpVtbl = &d3d10_rendertarget_view_vtbl;
     view->refcount = 1;
 
     if (!desc)
@@ -468,6 +473,15 @@ HRESULT d3d10_rendertarget_view_init(struct d3d10_rendertarget_view *view,
     return S_OK;
 }
 
+struct d3d10_rendertarget_view *unsafe_impl_from_ID3D10RenderTargetView(ID3D10RenderTargetView *iface)
+{
+    if (!iface)
+        return NULL;
+    assert(iface->lpVtbl == &d3d10_rendertarget_view_vtbl);
+
+    return impl_from_ID3D10RenderTargetView(iface);
+}
+
 static inline struct d3d10_shader_resource_view *impl_from_ID3D10ShaderResourceView(ID3D10ShaderResourceView *iface)
 {
     return CONTAINING_RECORD(iface, struct d3d10_shader_resource_view, ID3D10ShaderResourceView_iface);
