@@ -17,14 +17,13 @@
  */
 
 #include "config.h"
-#include <assert.h>
 #include "d3d8_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d8);
 
-static inline IDirect3DVertexBuffer8Impl *impl_from_IDirect3DVertexBuffer8(IDirect3DVertexBuffer8 *iface)
+static inline struct d3d8_vertexbuffer *impl_from_IDirect3DVertexBuffer8(IDirect3DVertexBuffer8 *iface)
 {
-    return CONTAINING_RECORD(iface, IDirect3DVertexBuffer8Impl, IDirect3DVertexBuffer8_iface);
+    return CONTAINING_RECORD(iface, struct d3d8_vertexbuffer, IDirect3DVertexBuffer8_iface);
 }
 
 static HRESULT WINAPI d3d8_vertexbuffer_QueryInterface(IDirect3DVertexBuffer8 *iface, REFIID riid, void **object)
@@ -48,16 +47,16 @@ static HRESULT WINAPI d3d8_vertexbuffer_QueryInterface(IDirect3DVertexBuffer8 *i
 
 static ULONG WINAPI d3d8_vertexbuffer_AddRef(IDirect3DVertexBuffer8 *iface)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
-    ULONG refcount = InterlockedIncrement(&buffer->ref);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    ULONG refcount = InterlockedIncrement(&buffer->refcount);
 
     TRACE("%p increasing refcount to %u.\n", iface, refcount);
 
     if (refcount == 1)
     {
-        IDirect3DDevice8_AddRef(buffer->parentDevice);
+        IDirect3DDevice8_AddRef(buffer->parent_device);
         wined3d_mutex_lock();
-        wined3d_buffer_incref(buffer->wineD3DVertexBuffer);
+        wined3d_buffer_incref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
     }
 
@@ -66,17 +65,17 @@ static ULONG WINAPI d3d8_vertexbuffer_AddRef(IDirect3DVertexBuffer8 *iface)
 
 static ULONG WINAPI d3d8_vertexbuffer_Release(IDirect3DVertexBuffer8 *iface)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
-    ULONG refcount = InterlockedDecrement(&buffer->ref);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    ULONG refcount = InterlockedDecrement(&buffer->refcount);
 
     TRACE("%p decreasing refcount to %u.\n", iface, refcount);
 
     if (!refcount)
     {
-        IDirect3DDevice8 *device = buffer->parentDevice;
+        IDirect3DDevice8 *device = buffer->parent_device;
 
         wined3d_mutex_lock();
-        wined3d_buffer_decref(buffer->wineD3DVertexBuffer);
+        wined3d_buffer_decref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
 
         /* Release the device last, as it may cause the device to be destroyed. */
@@ -89,11 +88,11 @@ static ULONG WINAPI d3d8_vertexbuffer_Release(IDirect3DVertexBuffer8 *iface)
 static HRESULT WINAPI d3d8_vertexbuffer_GetDevice(IDirect3DVertexBuffer8 *iface,
         IDirect3DDevice8 **device)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = buffer->parentDevice;
+    *device = buffer->parent_device;
     IDirect3DDevice8_AddRef(*device);
 
     TRACE("Returning device %p.\n", *device);
@@ -104,7 +103,7 @@ static HRESULT WINAPI d3d8_vertexbuffer_GetDevice(IDirect3DVertexBuffer8 *iface,
 static HRESULT WINAPI d3d8_vertexbuffer_SetPrivateData(IDirect3DVertexBuffer8 *iface,
         REFGUID guid, const void *data, DWORD data_size, DWORD flags)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
@@ -112,7 +111,7 @@ static HRESULT WINAPI d3d8_vertexbuffer_SetPrivateData(IDirect3DVertexBuffer8 *i
             iface, debugstr_guid(guid), data, data_size, flags);
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DVertexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_set_private_data(resource, guid, data, data_size, flags);
     wined3d_mutex_unlock();
 
@@ -122,7 +121,7 @@ static HRESULT WINAPI d3d8_vertexbuffer_SetPrivateData(IDirect3DVertexBuffer8 *i
 static HRESULT WINAPI d3d8_vertexbuffer_GetPrivateData(IDirect3DVertexBuffer8 *iface,
         REFGUID guid, void *data, DWORD *data_size)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
@@ -130,7 +129,7 @@ static HRESULT WINAPI d3d8_vertexbuffer_GetPrivateData(IDirect3DVertexBuffer8 *i
             iface, debugstr_guid(guid), data, data_size);
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DVertexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_get_private_data(resource, guid, data, data_size);
     wined3d_mutex_unlock();
 
@@ -139,14 +138,14 @@ static HRESULT WINAPI d3d8_vertexbuffer_GetPrivateData(IDirect3DVertexBuffer8 *i
 
 static HRESULT WINAPI d3d8_vertexbuffer_FreePrivateData(IDirect3DVertexBuffer8 *iface, REFGUID guid)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
     TRACE("iface %p, guid %s.\n", iface, debugstr_guid(guid));
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DVertexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_free_private_data(resource, guid);
     wined3d_mutex_unlock();
 
@@ -155,13 +154,13 @@ static HRESULT WINAPI d3d8_vertexbuffer_FreePrivateData(IDirect3DVertexBuffer8 *
 
 static DWORD WINAPI d3d8_vertexbuffer_SetPriority(IDirect3DVertexBuffer8 *iface, DWORD priority)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
     DWORD previous;
 
     TRACE("iface %p, priority %u.\n", iface, priority);
 
     wined3d_mutex_lock();
-    previous = wined3d_buffer_set_priority(buffer->wineD3DVertexBuffer, priority);
+    previous = wined3d_buffer_set_priority(buffer->wined3d_buffer, priority);
     wined3d_mutex_unlock();
 
     return previous;
@@ -169,13 +168,13 @@ static DWORD WINAPI d3d8_vertexbuffer_SetPriority(IDirect3DVertexBuffer8 *iface,
 
 static DWORD WINAPI d3d8_vertexbuffer_GetPriority(IDirect3DVertexBuffer8 *iface)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
     DWORD priority;
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    priority = wined3d_buffer_get_priority(buffer->wineD3DVertexBuffer);
+    priority = wined3d_buffer_get_priority(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 
     return priority;
@@ -183,12 +182,12 @@ static DWORD WINAPI d3d8_vertexbuffer_GetPriority(IDirect3DVertexBuffer8 *iface)
 
 static void WINAPI d3d8_vertexbuffer_PreLoad(IDirect3DVertexBuffer8 *iface)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    wined3d_buffer_preload(buffer->wineD3DVertexBuffer);
+    wined3d_buffer_preload(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 }
 
@@ -202,14 +201,14 @@ static D3DRESOURCETYPE WINAPI d3d8_vertexbuffer_GetType(IDirect3DVertexBuffer8 *
 static HRESULT WINAPI d3d8_vertexbuffer_Lock(IDirect3DVertexBuffer8 *iface, UINT offset, UINT size,
         BYTE **data, DWORD flags)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
     HRESULT hr;
 
     TRACE("iface %p, offset %u, size %u, data %p, flags %#x.\n",
             iface, offset, size, data, flags);
 
     wined3d_mutex_lock();
-    hr = wined3d_buffer_map(buffer->wineD3DVertexBuffer, offset, size, data, flags);
+    hr = wined3d_buffer_map(buffer->wined3d_buffer, offset, size, data, flags);
     wined3d_mutex_unlock();
 
     return hr;
@@ -217,12 +216,12 @@ static HRESULT WINAPI d3d8_vertexbuffer_Lock(IDirect3DVertexBuffer8 *iface, UINT
 
 static HRESULT WINAPI d3d8_vertexbuffer_Unlock(IDirect3DVertexBuffer8 *iface)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    wined3d_buffer_unmap(buffer->wineD3DVertexBuffer);
+    wined3d_buffer_unmap(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 
     return D3D_OK;
@@ -231,19 +230,19 @@ static HRESULT WINAPI d3d8_vertexbuffer_Unlock(IDirect3DVertexBuffer8 *iface)
 static HRESULT WINAPI d3d8_vertexbuffer_GetDesc(IDirect3DVertexBuffer8 *iface,
         D3DVERTEXBUFFER_DESC *desc)
 {
-    IDirect3DVertexBuffer8Impl *buffer = impl_from_IDirect3DVertexBuffer8(iface);
+    struct d3d8_vertexbuffer *buffer = impl_from_IDirect3DVertexBuffer8(iface);
     struct wined3d_resource_desc wined3d_desc;
     struct wined3d_resource *wined3d_resource;
 
     TRACE("iface %p, desc %p.\n", iface, desc);
 
     wined3d_mutex_lock();
-    wined3d_resource = wined3d_buffer_get_resource(buffer->wineD3DVertexBuffer);
+    wined3d_resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     wined3d_resource_get_desc(wined3d_resource, &wined3d_desc);
     wined3d_mutex_unlock();
 
     desc->Type = D3DRTYPE_VERTEXBUFFER;
-    desc->Usage = wined3d_desc.usage;
+    desc->Usage = wined3d_desc.usage & WINED3DUSAGE_MASK;
     desc->Pool = wined3d_desc.pool;
     desc->Size = wined3d_desc.size;
     desc->FVF = buffer->fvf;
@@ -283,18 +282,18 @@ static const struct wined3d_parent_ops d3d8_vertexbuffer_wined3d_parent_ops =
     d3d8_vertexbuffer_wined3d_object_destroyed,
 };
 
-HRESULT vertexbuffer_init(IDirect3DVertexBuffer8Impl *buffer, IDirect3DDevice8Impl *device,
+HRESULT vertexbuffer_init(struct d3d8_vertexbuffer *buffer, struct d3d8_device *device,
         UINT size, DWORD usage, DWORD fvf, D3DPOOL pool)
 {
     HRESULT hr;
 
     buffer->IDirect3DVertexBuffer8_iface.lpVtbl = &Direct3DVertexBuffer8_Vtbl;
-    buffer->ref = 1;
+    buffer->refcount = 1;
     buffer->fvf = fvf;
 
     wined3d_mutex_lock();
     hr = wined3d_buffer_create_vb(device->wined3d_device, size, usage & WINED3DUSAGE_MASK,
-            (WINED3DPOOL)pool, buffer, &d3d8_vertexbuffer_wined3d_parent_ops, &buffer->wineD3DVertexBuffer);
+            (enum wined3d_pool)pool, buffer, &d3d8_vertexbuffer_wined3d_parent_ops, &buffer->wined3d_buffer);
     wined3d_mutex_unlock();
     if (FAILED(hr))
     {
@@ -302,13 +301,13 @@ HRESULT vertexbuffer_init(IDirect3DVertexBuffer8Impl *buffer, IDirect3DDevice8Im
         return hr;
     }
 
-    buffer->parentDevice = &device->IDirect3DDevice8_iface;
-    IUnknown_AddRef(buffer->parentDevice);
+    buffer->parent_device = &device->IDirect3DDevice8_iface;
+    IUnknown_AddRef(buffer->parent_device);
 
     return D3D_OK;
 }
 
-IDirect3DVertexBuffer8Impl *unsafe_impl_from_IDirect3DVertexBuffer8(IDirect3DVertexBuffer8 *iface)
+struct d3d8_vertexbuffer *unsafe_impl_from_IDirect3DVertexBuffer8(IDirect3DVertexBuffer8 *iface)
 {
     if (!iface)
         return NULL;
@@ -317,9 +316,9 @@ IDirect3DVertexBuffer8Impl *unsafe_impl_from_IDirect3DVertexBuffer8(IDirect3DVer
     return impl_from_IDirect3DVertexBuffer8(iface);
 }
 
-static inline IDirect3DIndexBuffer8Impl *impl_from_IDirect3DIndexBuffer8(IDirect3DIndexBuffer8 *iface)
+static inline struct d3d8_indexbuffer *impl_from_IDirect3DIndexBuffer8(IDirect3DIndexBuffer8 *iface)
 {
-    return CONTAINING_RECORD(iface, IDirect3DIndexBuffer8Impl, IDirect3DIndexBuffer8_iface);
+    return CONTAINING_RECORD(iface, struct d3d8_indexbuffer, IDirect3DIndexBuffer8_iface);
 }
 
 static HRESULT WINAPI d3d8_indexbuffer_QueryInterface(IDirect3DIndexBuffer8 *iface, REFIID riid, void **object)
@@ -343,16 +342,16 @@ static HRESULT WINAPI d3d8_indexbuffer_QueryInterface(IDirect3DIndexBuffer8 *ifa
 
 static ULONG WINAPI d3d8_indexbuffer_AddRef(IDirect3DIndexBuffer8 *iface)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
-    ULONG refcount = InterlockedIncrement(&buffer->ref);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    ULONG refcount = InterlockedIncrement(&buffer->refcount);
 
     TRACE("%p increasing refcount to %u.\n", iface, refcount);
 
     if (refcount == 1)
     {
-        IDirect3DDevice8_AddRef(buffer->parentDevice);
+        IDirect3DDevice8_AddRef(buffer->parent_device);
         wined3d_mutex_lock();
-        wined3d_buffer_incref(buffer->wineD3DIndexBuffer);
+        wined3d_buffer_incref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
     }
 
@@ -361,17 +360,17 @@ static ULONG WINAPI d3d8_indexbuffer_AddRef(IDirect3DIndexBuffer8 *iface)
 
 static ULONG WINAPI d3d8_indexbuffer_Release(IDirect3DIndexBuffer8 *iface)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
-    ULONG refcount = InterlockedDecrement(&buffer->ref);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    ULONG refcount = InterlockedDecrement(&buffer->refcount);
 
     TRACE("%p decreasing refcount to %u.\n", iface, refcount);
 
     if (!refcount)
     {
-        IDirect3DDevice8 *device = buffer->parentDevice;
+        IDirect3DDevice8 *device = buffer->parent_device;
 
         wined3d_mutex_lock();
-        wined3d_buffer_decref(buffer->wineD3DIndexBuffer);
+        wined3d_buffer_decref(buffer->wined3d_buffer);
         wined3d_mutex_unlock();
 
         /* Release the device last, as it may cause the device to be destroyed. */
@@ -384,11 +383,11 @@ static ULONG WINAPI d3d8_indexbuffer_Release(IDirect3DIndexBuffer8 *iface)
 static HRESULT WINAPI d3d8_indexbuffer_GetDevice(IDirect3DIndexBuffer8 *iface,
         IDirect3DDevice8 **device)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
 
     TRACE("iface %p, device %p.\n", iface, device);
 
-    *device = buffer->parentDevice;
+    *device = buffer->parent_device;
     IDirect3DDevice8_AddRef(*device);
 
     TRACE("Returning device %p.\n", *device);
@@ -399,7 +398,7 @@ static HRESULT WINAPI d3d8_indexbuffer_GetDevice(IDirect3DIndexBuffer8 *iface,
 static HRESULT WINAPI d3d8_indexbuffer_SetPrivateData(IDirect3DIndexBuffer8 *iface,
         REFGUID guid, const void *data, DWORD data_size, DWORD flags)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
@@ -407,7 +406,7 @@ static HRESULT WINAPI d3d8_indexbuffer_SetPrivateData(IDirect3DIndexBuffer8 *ifa
             iface, debugstr_guid(guid), data, data_size, flags);
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DIndexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_set_private_data(resource, guid, data, data_size, flags);
     wined3d_mutex_unlock();
 
@@ -417,7 +416,7 @@ static HRESULT WINAPI d3d8_indexbuffer_SetPrivateData(IDirect3DIndexBuffer8 *ifa
 static HRESULT WINAPI d3d8_indexbuffer_GetPrivateData(IDirect3DIndexBuffer8 *iface,
         REFGUID guid, void *data, DWORD *data_size)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
@@ -425,7 +424,7 @@ static HRESULT WINAPI d3d8_indexbuffer_GetPrivateData(IDirect3DIndexBuffer8 *ifa
             iface, debugstr_guid(guid), data, data_size);
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DIndexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_get_private_data(resource, guid, data, data_size);
     wined3d_mutex_unlock();
 
@@ -434,14 +433,14 @@ static HRESULT WINAPI d3d8_indexbuffer_GetPrivateData(IDirect3DIndexBuffer8 *ifa
 
 static HRESULT WINAPI d3d8_indexbuffer_FreePrivateData(IDirect3DIndexBuffer8 *iface, REFGUID guid)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
     struct wined3d_resource *resource;
     HRESULT hr;
 
     TRACE("iface %p, guid %s.\n", iface, debugstr_guid(guid));
 
     wined3d_mutex_lock();
-    resource = wined3d_buffer_get_resource(buffer->wineD3DIndexBuffer);
+    resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     hr = wined3d_resource_free_private_data(resource, guid);
     wined3d_mutex_unlock();
 
@@ -450,13 +449,13 @@ static HRESULT WINAPI d3d8_indexbuffer_FreePrivateData(IDirect3DIndexBuffer8 *if
 
 static DWORD WINAPI d3d8_indexbuffer_SetPriority(IDirect3DIndexBuffer8 *iface, DWORD priority)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
     DWORD previous;
 
     TRACE("iface %p, priority %u.\n", iface, priority);
 
     wined3d_mutex_lock();
-    previous = wined3d_buffer_set_priority(buffer->wineD3DIndexBuffer, priority);
+    previous = wined3d_buffer_set_priority(buffer->wined3d_buffer, priority);
     wined3d_mutex_unlock();
 
     return previous;
@@ -464,13 +463,13 @@ static DWORD WINAPI d3d8_indexbuffer_SetPriority(IDirect3DIndexBuffer8 *iface, D
 
 static DWORD WINAPI d3d8_indexbuffer_GetPriority(IDirect3DIndexBuffer8 *iface)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
     DWORD priority;
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    priority = wined3d_buffer_get_priority(buffer->wineD3DIndexBuffer);
+    priority = wined3d_buffer_get_priority(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 
     return priority;
@@ -478,12 +477,12 @@ static DWORD WINAPI d3d8_indexbuffer_GetPriority(IDirect3DIndexBuffer8 *iface)
 
 static void WINAPI d3d8_indexbuffer_PreLoad(IDirect3DIndexBuffer8 *iface)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    wined3d_buffer_preload(buffer->wineD3DIndexBuffer);
+    wined3d_buffer_preload(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 }
 
@@ -497,14 +496,14 @@ static D3DRESOURCETYPE WINAPI d3d8_indexbuffer_GetType(IDirect3DIndexBuffer8 *if
 static HRESULT WINAPI d3d8_indexbuffer_Lock(IDirect3DIndexBuffer8 *iface, UINT offset, UINT size,
         BYTE **data, DWORD flags)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
     HRESULT hr;
 
     TRACE("iface %p, offset %u, size %u, data %p, flags %#x.\n",
             iface, offset, size, data, flags);
 
     wined3d_mutex_lock();
-    hr = wined3d_buffer_map(buffer->wineD3DIndexBuffer, offset, size, data, flags);
+    hr = wined3d_buffer_map(buffer->wined3d_buffer, offset, size, data, flags);
     wined3d_mutex_unlock();
 
     return hr;
@@ -512,12 +511,12 @@ static HRESULT WINAPI d3d8_indexbuffer_Lock(IDirect3DIndexBuffer8 *iface, UINT o
 
 static HRESULT WINAPI d3d8_indexbuffer_Unlock(IDirect3DIndexBuffer8 *iface)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
 
     TRACE("iface %p.\n", iface);
 
     wined3d_mutex_lock();
-    wined3d_buffer_unmap(buffer->wineD3DIndexBuffer);
+    wined3d_buffer_unmap(buffer->wined3d_buffer);
     wined3d_mutex_unlock();
 
     return D3D_OK;
@@ -526,20 +525,20 @@ static HRESULT WINAPI d3d8_indexbuffer_Unlock(IDirect3DIndexBuffer8 *iface)
 static HRESULT WINAPI d3d8_indexbuffer_GetDesc(IDirect3DIndexBuffer8 *iface,
         D3DINDEXBUFFER_DESC *desc)
 {
-    IDirect3DIndexBuffer8Impl *buffer = impl_from_IDirect3DIndexBuffer8(iface);
+    struct d3d8_indexbuffer *buffer = impl_from_IDirect3DIndexBuffer8(iface);
     struct wined3d_resource_desc wined3d_desc;
     struct wined3d_resource *wined3d_resource;
 
     TRACE("iface %p, desc %p.\n", iface, desc);
 
     wined3d_mutex_lock();
-    wined3d_resource = wined3d_buffer_get_resource(buffer->wineD3DIndexBuffer);
+    wined3d_resource = wined3d_buffer_get_resource(buffer->wined3d_buffer);
     wined3d_resource_get_desc(wined3d_resource, &wined3d_desc);
     wined3d_mutex_unlock();
 
     desc->Format = d3dformat_from_wined3dformat(buffer->format);
     desc->Type = D3DRTYPE_INDEXBUFFER;
-    desc->Usage = wined3d_desc.usage;
+    desc->Usage = wined3d_desc.usage & WINED3DUSAGE_MASK;
     desc->Pool = wined3d_desc.pool;
     desc->Size = wined3d_desc.size;
 
@@ -577,18 +576,18 @@ static const struct wined3d_parent_ops d3d8_indexbuffer_wined3d_parent_ops =
     d3d8_indexbuffer_wined3d_object_destroyed,
 };
 
-HRESULT indexbuffer_init(IDirect3DIndexBuffer8Impl *buffer, IDirect3DDevice8Impl *device,
+HRESULT indexbuffer_init(struct d3d8_indexbuffer *buffer, struct d3d8_device *device,
         UINT size, DWORD usage, D3DFORMAT format, D3DPOOL pool)
 {
     HRESULT hr;
 
     buffer->IDirect3DIndexBuffer8_iface.lpVtbl = &d3d8_indexbuffer_vtbl;
-    buffer->ref = 1;
+    buffer->refcount = 1;
     buffer->format = wined3dformat_from_d3dformat(format);
 
     wined3d_mutex_lock();
     hr = wined3d_buffer_create_ib(device->wined3d_device, size, usage & WINED3DUSAGE_MASK,
-            (WINED3DPOOL)pool, buffer, &d3d8_indexbuffer_wined3d_parent_ops, &buffer->wineD3DIndexBuffer);
+            (enum wined3d_pool)pool, buffer, &d3d8_indexbuffer_wined3d_parent_ops, &buffer->wined3d_buffer);
     wined3d_mutex_unlock();
     if (FAILED(hr))
     {
@@ -596,13 +595,13 @@ HRESULT indexbuffer_init(IDirect3DIndexBuffer8Impl *buffer, IDirect3DDevice8Impl
         return hr;
     }
 
-    buffer->parentDevice = &device->IDirect3DDevice8_iface;
-    IUnknown_AddRef(buffer->parentDevice);
+    buffer->parent_device = &device->IDirect3DDevice8_iface;
+    IUnknown_AddRef(buffer->parent_device);
 
     return D3D_OK;
 }
 
-IDirect3DIndexBuffer8Impl *unsafe_impl_from_IDirect3DIndexBuffer8(IDirect3DIndexBuffer8 *iface)
+struct d3d8_indexbuffer *unsafe_impl_from_IDirect3DIndexBuffer8(IDirect3DIndexBuffer8 *iface)
 {
     if (!iface)
         return NULL;
