@@ -27,7 +27,7 @@
 #include "d3d9types.h"
 #include "d3dcompiler_private.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(asmshader);
+WINE_DEFAULT_DEBUG_CHANNEL(bytecodewriter);
 
 /****************************************************************
  * General assembler shader construction helper routines follow *
@@ -44,17 +44,17 @@ WINE_DEFAULT_DEBUG_CHANNEL(asmshader);
  *  NULL in case of an allocation failure
  */
 struct instruction *alloc_instr(unsigned int srcs) {
-    struct instruction *ret = asm_alloc(sizeof(*ret));
+    struct instruction *ret = d3dcompiler_alloc(sizeof(*ret));
     if(!ret) {
         ERR("Failed to allocate memory for an instruction structure\n");
         return NULL;
     }
 
     if(srcs) {
-        ret->src = asm_alloc(srcs * sizeof(*ret->src));
+        ret->src = d3dcompiler_alloc(srcs * sizeof(*ret->src));
         if(!ret->src) {
             ERR("Failed to allocate memory for instruction registers\n");
-            asm_free(ret);
+            d3dcompiler_free(ret);
             return NULL;
         }
         ret->num_srcs = srcs;
@@ -80,14 +80,14 @@ BOOL add_instruction(struct bwriter_shader *shader, struct instruction *instr) {
     if(!shader) return FALSE;
 
     if(shader->instr_alloc_size == 0) {
-        shader->instr = asm_alloc(sizeof(*shader->instr) * INSTRARRAY_INITIAL_SIZE);
+        shader->instr = d3dcompiler_alloc(sizeof(*shader->instr) * INSTRARRAY_INITIAL_SIZE);
         if(!shader->instr) {
             ERR("Failed to allocate the shader instruction array\n");
             return FALSE;
         }
         shader->instr_alloc_size = INSTRARRAY_INITIAL_SIZE;
     } else if(shader->instr_alloc_size == shader->num_instrs) {
-        new_instructions = asm_realloc(shader->instr,
+        new_instructions = d3dcompiler_realloc(shader->instr,
                                        sizeof(*shader->instr) * (shader->instr_alloc_size) * 2);
         if(!new_instructions) {
             ERR("Failed to grow the shader instruction array\n");
@@ -110,7 +110,7 @@ BOOL add_constF(struct bwriter_shader *shader, DWORD reg, float x, float y, floa
 
     if(shader->num_cf) {
         struct constant **newarray;
-        newarray = asm_realloc(shader->constF,
+        newarray = d3dcompiler_realloc(shader->constF,
                                sizeof(*shader->constF) * (shader->num_cf + 1));
         if(!newarray) {
             ERR("Failed to grow the constants array\n");
@@ -118,14 +118,14 @@ BOOL add_constF(struct bwriter_shader *shader, DWORD reg, float x, float y, floa
         }
         shader->constF = newarray;
     } else {
-        shader->constF = asm_alloc(sizeof(*shader->constF));
+        shader->constF = d3dcompiler_alloc(sizeof(*shader->constF));
         if(!shader->constF) {
             ERR("Failed to allocate the constants array\n");
             return FALSE;
         }
     }
 
-    newconst = asm_alloc(sizeof(*newconst));
+    newconst = d3dcompiler_alloc(sizeof(*newconst));
     if(!newconst) {
         ERR("Failed to allocate a new constant\n");
         return FALSE;
@@ -146,7 +146,7 @@ BOOL add_constI(struct bwriter_shader *shader, DWORD reg, INT x, INT y, INT z, I
 
     if(shader->num_ci) {
         struct constant **newarray;
-        newarray = asm_realloc(shader->constI,
+        newarray = d3dcompiler_realloc(shader->constI,
                                sizeof(*shader->constI) * (shader->num_ci + 1));
         if(!newarray) {
             ERR("Failed to grow the constants array\n");
@@ -154,14 +154,14 @@ BOOL add_constI(struct bwriter_shader *shader, DWORD reg, INT x, INT y, INT z, I
         }
         shader->constI = newarray;
     } else {
-        shader->constI = asm_alloc(sizeof(*shader->constI));
+        shader->constI = d3dcompiler_alloc(sizeof(*shader->constI));
         if(!shader->constI) {
             ERR("Failed to allocate the constants array\n");
             return FALSE;
         }
     }
 
-    newconst = asm_alloc(sizeof(*newconst));
+    newconst = d3dcompiler_alloc(sizeof(*newconst));
     if(!newconst) {
         ERR("Failed to allocate a new constant\n");
         return FALSE;
@@ -182,7 +182,7 @@ BOOL add_constB(struct bwriter_shader *shader, DWORD reg, BOOL x) {
 
     if(shader->num_cb) {
         struct constant **newarray;
-        newarray = asm_realloc(shader->constB,
+        newarray = d3dcompiler_realloc(shader->constB,
                                sizeof(*shader->constB) * (shader->num_cb + 1));
         if(!newarray) {
             ERR("Failed to grow the constants array\n");
@@ -190,14 +190,14 @@ BOOL add_constB(struct bwriter_shader *shader, DWORD reg, BOOL x) {
         }
         shader->constB = newarray;
     } else {
-        shader->constB = asm_alloc(sizeof(*shader->constB));
+        shader->constB = d3dcompiler_alloc(sizeof(*shader->constB));
         if(!shader->constB) {
             ERR("Failed to allocate the constants array\n");
             return FALSE;
         }
     }
 
-    newconst = asm_alloc(sizeof(*newconst));
+    newconst = d3dcompiler_alloc(sizeof(*newconst));
     if(!newconst) {
         ERR("Failed to allocate a new constant\n");
         return FALSE;
@@ -228,7 +228,7 @@ BOOL record_declaration(struct bwriter_shader *shader, DWORD usage,
     }
 
     if(*num == 0) {
-        *decl = asm_alloc(sizeof(**decl));
+        *decl = d3dcompiler_alloc(sizeof(**decl));
         if(!*decl) {
             ERR("Error allocating declarations array\n");
             return FALSE;
@@ -242,7 +242,7 @@ BOOL record_declaration(struct bwriter_shader *shader, DWORD usage,
             }
         }
 
-        newdecl = asm_realloc(*decl,
+        newdecl = d3dcompiler_realloc(*decl,
                               sizeof(**decl) * ((*num) + 1));
         if(!newdecl) {
             ERR("Error reallocating declarations array\n");
@@ -267,7 +267,7 @@ BOOL record_sampler(struct bwriter_shader *shader, DWORD samptype, DWORD mod, DW
     if(!shader) return FALSE;
 
     if(shader->num_samplers == 0) {
-        shader->samplers = asm_alloc(sizeof(*shader->samplers));
+        shader->samplers = d3dcompiler_alloc(sizeof(*shader->samplers));
         if(!shader->samplers) {
             ERR("Error allocating samplers array\n");
             return FALSE;
@@ -284,7 +284,7 @@ BOOL record_sampler(struct bwriter_shader *shader, DWORD samptype, DWORD mod, DW
             }
         }
 
-        newarray = asm_realloc(shader->samplers,
+        newarray = d3dcompiler_realloc(shader->samplers,
                                sizeof(*shader->samplers) * (shader->num_samplers + 1));
         if(!newarray) {
             ERR("Error reallocating samplers array\n");
@@ -310,13 +310,13 @@ BOOL record_sampler(struct bwriter_shader *shader, DWORD samptype, DWORD mod, DW
 static struct bytecode_buffer *allocate_buffer(void) {
     struct bytecode_buffer *ret;
 
-    ret = asm_alloc(sizeof(*ret));
+    ret = d3dcompiler_alloc(sizeof(*ret));
     if(!ret) return NULL;
 
     ret->alloc_size = BYTECODEBUFFER_INITIAL_SIZE;
-    ret->data = asm_alloc(sizeof(DWORD) * ret->alloc_size);
+    ret->data = d3dcompiler_alloc(sizeof(DWORD) * ret->alloc_size);
     if(!ret->data) {
-        asm_free(ret);
+        d3dcompiler_free(ret);
         return NULL;
     }
     ret->state = S_OK;
@@ -329,7 +329,7 @@ static void put_dword(struct bytecode_buffer *buffer, DWORD value) {
     if(buffer->alloc_size == buffer->size) {
         DWORD *newarray;
         buffer->alloc_size *= 2;
-        newarray = asm_realloc(buffer->data,
+        newarray = d3dcompiler_realloc(buffer->data,
                                sizeof(DWORD) * buffer->alloc_size);
         if(!newarray) {
             ERR("Failed to grow the buffer data memory\n");
@@ -341,6 +341,236 @@ static void put_dword(struct bytecode_buffer *buffer, DWORD value) {
     buffer->data[buffer->size++] = value;
 }
 
+/* bwriter -> d3d9 conversion functions. */
+static DWORD d3d9_swizzle(DWORD bwriter_swizzle)
+{
+    /* Currently a NOP, but this allows changing the internal definitions
+     * without side effects. */
+    DWORD ret = 0;
+
+    if ((bwriter_swizzle & BWRITERVS_X_X) == BWRITERVS_X_X) ret |= D3DVS_X_X;
+    if ((bwriter_swizzle & BWRITERVS_X_Y) == BWRITERVS_X_Y) ret |= D3DVS_X_Y;
+    if ((bwriter_swizzle & BWRITERVS_X_Z) == BWRITERVS_X_Z) ret |= D3DVS_X_Z;
+    if ((bwriter_swizzle & BWRITERVS_X_W) == BWRITERVS_X_W) ret |= D3DVS_X_W;
+
+    if ((bwriter_swizzle & BWRITERVS_Y_X) == BWRITERVS_Y_X) ret |= D3DVS_Y_X;
+    if ((bwriter_swizzle & BWRITERVS_Y_Y) == BWRITERVS_Y_Y) ret |= D3DVS_Y_Y;
+    if ((bwriter_swizzle & BWRITERVS_Y_Z) == BWRITERVS_Y_Z) ret |= D3DVS_Y_Z;
+    if ((bwriter_swizzle & BWRITERVS_Y_W) == BWRITERVS_Y_W) ret |= D3DVS_Y_W;
+
+    if ((bwriter_swizzle & BWRITERVS_Z_X) == BWRITERVS_Z_X) ret |= D3DVS_Z_X;
+    if ((bwriter_swizzle & BWRITERVS_Z_Y) == BWRITERVS_Z_Y) ret |= D3DVS_Z_Y;
+    if ((bwriter_swizzle & BWRITERVS_Z_Z) == BWRITERVS_Z_Z) ret |= D3DVS_Z_Z;
+    if ((bwriter_swizzle & BWRITERVS_Z_W) == BWRITERVS_Z_W) ret |= D3DVS_Z_W;
+
+    if ((bwriter_swizzle & BWRITERVS_W_X) == BWRITERVS_W_X) ret |= D3DVS_W_X;
+    if ((bwriter_swizzle & BWRITERVS_W_Y) == BWRITERVS_W_Y) ret |= D3DVS_W_Y;
+    if ((bwriter_swizzle & BWRITERVS_W_Z) == BWRITERVS_W_Z) ret |= D3DVS_W_Z;
+    if ((bwriter_swizzle & BWRITERVS_W_W) == BWRITERVS_W_W) ret |= D3DVS_W_W;
+
+    return ret;
+}
+
+static DWORD d3d9_writemask(DWORD bwriter_writemask)
+{
+    DWORD ret = 0;
+
+    if (bwriter_writemask & BWRITERSP_WRITEMASK_0) ret |= D3DSP_WRITEMASK_0;
+    if (bwriter_writemask & BWRITERSP_WRITEMASK_1) ret |= D3DSP_WRITEMASK_1;
+    if (bwriter_writemask & BWRITERSP_WRITEMASK_2) ret |= D3DSP_WRITEMASK_2;
+    if (bwriter_writemask & BWRITERSP_WRITEMASK_3) ret |= D3DSP_WRITEMASK_3;
+
+    return ret;
+}
+
+static DWORD d3d9_srcmod(DWORD bwriter_srcmod)
+{
+    switch (bwriter_srcmod)
+    {
+        case BWRITERSPSM_NONE:       return D3DSPSM_NONE;
+        case BWRITERSPSM_NEG:        return D3DSPSM_NEG;
+        case BWRITERSPSM_BIAS:       return D3DSPSM_BIAS;
+        case BWRITERSPSM_BIASNEG:    return D3DSPSM_BIASNEG;
+        case BWRITERSPSM_SIGN:       return D3DSPSM_SIGN;
+        case BWRITERSPSM_SIGNNEG:    return D3DSPSM_SIGNNEG;
+        case BWRITERSPSM_COMP:       return D3DSPSM_COMP;
+        case BWRITERSPSM_X2:         return D3DSPSM_X2;
+        case BWRITERSPSM_X2NEG:      return D3DSPSM_X2NEG;
+        case BWRITERSPSM_DZ:         return D3DSPSM_DZ;
+        case BWRITERSPSM_DW:         return D3DSPSM_DW;
+        case BWRITERSPSM_ABS:        return D3DSPSM_ABS;
+        case BWRITERSPSM_ABSNEG:     return D3DSPSM_ABSNEG;
+        case BWRITERSPSM_NOT:        return D3DSPSM_NOT;
+        default:
+            FIXME("Unhandled BWRITERSPSM token %#x.\n", bwriter_srcmod);
+            return 0;
+    }
+}
+
+static DWORD d3d9_dstmod(DWORD bwriter_mod)
+{
+    DWORD ret = 0;
+
+    if (bwriter_mod & BWRITERSPDM_SATURATE)         ret |= D3DSPDM_SATURATE;
+    if (bwriter_mod & BWRITERSPDM_PARTIALPRECISION) ret |= D3DSPDM_PARTIALPRECISION;
+    if (bwriter_mod & BWRITERSPDM_MSAMPCENTROID)    ret |= D3DSPDM_MSAMPCENTROID;
+
+    return ret;
+}
+
+static DWORD d3d9_comparetype(DWORD asmshader_comparetype)
+{
+    switch (asmshader_comparetype)
+    {
+        case BWRITER_COMPARISON_GT:     return D3DSPC_GT;
+        case BWRITER_COMPARISON_EQ:     return D3DSPC_EQ;
+        case BWRITER_COMPARISON_GE:     return D3DSPC_GE;
+        case BWRITER_COMPARISON_LT:     return D3DSPC_LT;
+        case BWRITER_COMPARISON_NE:     return D3DSPC_NE;
+        case BWRITER_COMPARISON_LE:     return D3DSPC_LE;
+        default:
+            FIXME("Unexpected BWRITER_COMPARISON type %#x.\n", asmshader_comparetype);
+            return 0;
+    }
+}
+
+static DWORD d3d9_sampler(DWORD bwriter_sampler)
+{
+    if (bwriter_sampler == BWRITERSTT_UNKNOWN)  return D3DSTT_UNKNOWN;
+    if (bwriter_sampler == BWRITERSTT_1D)       return D3DSTT_1D;
+    if (bwriter_sampler == BWRITERSTT_2D)       return D3DSTT_2D;
+    if (bwriter_sampler == BWRITERSTT_CUBE)     return D3DSTT_CUBE;
+    if (bwriter_sampler == BWRITERSTT_VOLUME)   return D3DSTT_VOLUME;
+    FIXME("Unexpected BWRITERSAMPLER_TEXTURE_TYPE type %#x.\n", bwriter_sampler);
+
+    return 0;
+}
+
+static DWORD d3d9_register(DWORD bwriter_register)
+{
+    if (bwriter_register == BWRITERSPR_TEMP)        return D3DSPR_TEMP;
+    if (bwriter_register == BWRITERSPR_INPUT)       return D3DSPR_INPUT;
+    if (bwriter_register == BWRITERSPR_CONST)       return D3DSPR_CONST;
+    if (bwriter_register == BWRITERSPR_ADDR)        return D3DSPR_ADDR;
+    if (bwriter_register == BWRITERSPR_TEXTURE)     return D3DSPR_TEXTURE;
+    if (bwriter_register == BWRITERSPR_RASTOUT)     return D3DSPR_RASTOUT;
+    if (bwriter_register == BWRITERSPR_ATTROUT)     return D3DSPR_ATTROUT;
+    if (bwriter_register == BWRITERSPR_TEXCRDOUT)   return D3DSPR_TEXCRDOUT;
+    if (bwriter_register == BWRITERSPR_OUTPUT)      return D3DSPR_OUTPUT;
+    if (bwriter_register == BWRITERSPR_CONSTINT)    return D3DSPR_CONSTINT;
+    if (bwriter_register == BWRITERSPR_COLOROUT)    return D3DSPR_COLOROUT;
+    if (bwriter_register == BWRITERSPR_DEPTHOUT)    return D3DSPR_DEPTHOUT;
+    if (bwriter_register == BWRITERSPR_SAMPLER)     return D3DSPR_SAMPLER;
+    if (bwriter_register == BWRITERSPR_CONSTBOOL)   return D3DSPR_CONSTBOOL;
+    if (bwriter_register == BWRITERSPR_LOOP)        return D3DSPR_LOOP;
+    if (bwriter_register == BWRITERSPR_MISCTYPE)    return D3DSPR_MISCTYPE;
+    if (bwriter_register == BWRITERSPR_LABEL)       return D3DSPR_LABEL;
+    if (bwriter_register == BWRITERSPR_PREDICATE)   return D3DSPR_PREDICATE;
+
+    FIXME("Unexpected BWRITERSPR %#x.\n", bwriter_register);
+    return ~0U;
+}
+
+static DWORD d3d9_opcode(DWORD bwriter_opcode)
+{
+    switch (bwriter_opcode)
+    {
+        case BWRITERSIO_NOP:         return D3DSIO_NOP;
+        case BWRITERSIO_MOV:         return D3DSIO_MOV;
+        case BWRITERSIO_ADD:         return D3DSIO_ADD;
+        case BWRITERSIO_SUB:         return D3DSIO_SUB;
+        case BWRITERSIO_MAD:         return D3DSIO_MAD;
+        case BWRITERSIO_MUL:         return D3DSIO_MUL;
+        case BWRITERSIO_RCP:         return D3DSIO_RCP;
+        case BWRITERSIO_RSQ:         return D3DSIO_RSQ;
+        case BWRITERSIO_DP3:         return D3DSIO_DP3;
+        case BWRITERSIO_DP4:         return D3DSIO_DP4;
+        case BWRITERSIO_MIN:         return D3DSIO_MIN;
+        case BWRITERSIO_MAX:         return D3DSIO_MAX;
+        case BWRITERSIO_SLT:         return D3DSIO_SLT;
+        case BWRITERSIO_SGE:         return D3DSIO_SGE;
+        case BWRITERSIO_EXP:         return D3DSIO_EXP;
+        case BWRITERSIO_LOG:         return D3DSIO_LOG;
+        case BWRITERSIO_LIT:         return D3DSIO_LIT;
+        case BWRITERSIO_DST:         return D3DSIO_DST;
+        case BWRITERSIO_LRP:         return D3DSIO_LRP;
+        case BWRITERSIO_FRC:         return D3DSIO_FRC;
+        case BWRITERSIO_M4x4:        return D3DSIO_M4x4;
+        case BWRITERSIO_M4x3:        return D3DSIO_M4x3;
+        case BWRITERSIO_M3x4:        return D3DSIO_M3x4;
+        case BWRITERSIO_M3x3:        return D3DSIO_M3x3;
+        case BWRITERSIO_M3x2:        return D3DSIO_M3x2;
+        case BWRITERSIO_CALL:        return D3DSIO_CALL;
+        case BWRITERSIO_CALLNZ:      return D3DSIO_CALLNZ;
+        case BWRITERSIO_LOOP:        return D3DSIO_LOOP;
+        case BWRITERSIO_RET:         return D3DSIO_RET;
+        case BWRITERSIO_ENDLOOP:     return D3DSIO_ENDLOOP;
+        case BWRITERSIO_LABEL:       return D3DSIO_LABEL;
+        case BWRITERSIO_DCL:         return D3DSIO_DCL;
+        case BWRITERSIO_POW:         return D3DSIO_POW;
+        case BWRITERSIO_CRS:         return D3DSIO_CRS;
+        case BWRITERSIO_SGN:         return D3DSIO_SGN;
+        case BWRITERSIO_ABS:         return D3DSIO_ABS;
+        case BWRITERSIO_NRM:         return D3DSIO_NRM;
+        case BWRITERSIO_SINCOS:      return D3DSIO_SINCOS;
+        case BWRITERSIO_REP:         return D3DSIO_REP;
+        case BWRITERSIO_ENDREP:      return D3DSIO_ENDREP;
+        case BWRITERSIO_IF:          return D3DSIO_IF;
+        case BWRITERSIO_IFC:         return D3DSIO_IFC;
+        case BWRITERSIO_ELSE:        return D3DSIO_ELSE;
+        case BWRITERSIO_ENDIF:       return D3DSIO_ENDIF;
+        case BWRITERSIO_BREAK:       return D3DSIO_BREAK;
+        case BWRITERSIO_BREAKC:      return D3DSIO_BREAKC;
+        case BWRITERSIO_MOVA:        return D3DSIO_MOVA;
+        case BWRITERSIO_DEFB:        return D3DSIO_DEFB;
+        case BWRITERSIO_DEFI:        return D3DSIO_DEFI;
+
+        case BWRITERSIO_TEXCOORD:    return D3DSIO_TEXCOORD;
+        case BWRITERSIO_TEXKILL:     return D3DSIO_TEXKILL;
+        case BWRITERSIO_TEX:         return D3DSIO_TEX;
+        case BWRITERSIO_TEXBEM:      return D3DSIO_TEXBEM;
+        case BWRITERSIO_TEXBEML:     return D3DSIO_TEXBEML;
+        case BWRITERSIO_TEXREG2AR:   return D3DSIO_TEXREG2AR;
+        case BWRITERSIO_TEXREG2GB:   return D3DSIO_TEXREG2GB;
+        case BWRITERSIO_TEXM3x2PAD:  return D3DSIO_TEXM3x2PAD;
+        case BWRITERSIO_TEXM3x2TEX:  return D3DSIO_TEXM3x2TEX;
+        case BWRITERSIO_TEXM3x3PAD:  return D3DSIO_TEXM3x3PAD;
+        case BWRITERSIO_TEXM3x3TEX:  return D3DSIO_TEXM3x3TEX;
+        case BWRITERSIO_TEXM3x3SPEC: return D3DSIO_TEXM3x3SPEC;
+        case BWRITERSIO_TEXM3x3VSPEC:return D3DSIO_TEXM3x3VSPEC;
+        case BWRITERSIO_EXPP:        return D3DSIO_EXPP;
+        case BWRITERSIO_LOGP:        return D3DSIO_LOGP;
+        case BWRITERSIO_CND:         return D3DSIO_CND;
+        case BWRITERSIO_DEF:         return D3DSIO_DEF;
+        case BWRITERSIO_TEXREG2RGB:  return D3DSIO_TEXREG2RGB;
+        case BWRITERSIO_TEXDP3TEX:   return D3DSIO_TEXDP3TEX;
+        case BWRITERSIO_TEXM3x2DEPTH:return D3DSIO_TEXM3x2DEPTH;
+        case BWRITERSIO_TEXDP3:      return D3DSIO_TEXDP3;
+        case BWRITERSIO_TEXM3x3:     return D3DSIO_TEXM3x3;
+        case BWRITERSIO_TEXDEPTH:    return D3DSIO_TEXDEPTH;
+        case BWRITERSIO_CMP:         return D3DSIO_CMP;
+        case BWRITERSIO_BEM:         return D3DSIO_BEM;
+        case BWRITERSIO_DP2ADD:      return D3DSIO_DP2ADD;
+        case BWRITERSIO_DSX:         return D3DSIO_DSX;
+        case BWRITERSIO_DSY:         return D3DSIO_DSY;
+        case BWRITERSIO_TEXLDD:      return D3DSIO_TEXLDD;
+        case BWRITERSIO_SETP:        return D3DSIO_SETP;
+        case BWRITERSIO_TEXLDL:      return D3DSIO_TEXLDL;
+        case BWRITERSIO_BREAKP:      return D3DSIO_BREAKP;
+
+        case BWRITERSIO_PHASE:       return D3DSIO_PHASE;
+        case BWRITERSIO_COMMENT:     return D3DSIO_COMMENT;
+        case BWRITERSIO_END:         return D3DSIO_END;
+
+        case BWRITERSIO_TEXLDP:      return D3DSIO_TEX | D3DSI_TEXLD_PROJECT;
+        case BWRITERSIO_TEXLDB:      return D3DSIO_TEX | D3DSI_TEXLD_BIAS;
+
+        default:
+            FIXME("Unhandled BWRITERSIO token %#x.\n", bwriter_opcode);
+            return ~0U;
+    }
+}
+
 /******************************************************
  * Implementation of the writer functions starts here *
  ******************************************************/
@@ -598,7 +828,7 @@ static void ps_1_x_header(struct bc_writer *This, const struct bwriter_shader *s
 
     /* Declare the shader type and version */
     put_dword(buffer, This->version);
-    write_constF(shader, buffer, TRUE);
+    write_constF(shader, buffer, FALSE);
 }
 
 static void ps_1_4_header(struct bc_writer *This, const struct bwriter_shader *shader, struct bytecode_buffer *buffer) {
@@ -619,7 +849,7 @@ static void ps_1_4_header(struct bc_writer *This, const struct bwriter_shader *s
 
     /* Declare the shader type and version */
     put_dword(buffer, This->version);
-    write_constF(shader, buffer, TRUE);
+    write_constF(shader, buffer, FALSE);
 }
 
 static void end(struct bc_writer *This, const struct bwriter_shader *shader, struct bytecode_buffer *buffer) {
@@ -2175,7 +2405,7 @@ static void init_ps30_dx9_writer(struct bc_writer *writer) {
 }
 
 static struct bc_writer *create_writer(DWORD version, DWORD dxversion) {
-    struct bc_writer *ret = asm_alloc(sizeof(*ret));
+    struct bc_writer *ret = d3dcompiler_alloc(sizeof(*ret));
 
     if(!ret) {
         WARN("Failed to allocate a bytecode writer instance\n");
@@ -2287,7 +2517,7 @@ static struct bc_writer *create_writer(DWORD version, DWORD dxversion) {
     return ret;
 
 fail:
-    asm_free(ret);
+    d3dcompiler_free(ret);
     return NULL;
 }
 
@@ -2313,22 +2543,8 @@ static HRESULT call_instr_handler(struct bc_writer *writer,
     return E_INVALIDARG;
 }
 
-/* SlWriteBytecode (wineshader.@)
- *
- * Writes shader version specific bytecode from the shader passed in.
- * The returned bytecode can be passed to the Direct3D runtime like
- * IDirect3DDevice9::Create*Shader.
- *
- * Parameters:
- *  shader: Shader to translate into bytecode
- *  version: Shader version to generate(d3d version token)
- *  dxversion: DirectX version the code targets
- *  result: the resulting shader bytecode
- *
- * Return values:
- *  S_OK on success
- */
-DWORD SlWriteBytecode(const struct bwriter_shader *shader, int dxversion, DWORD **result) {
+HRESULT SlWriteBytecode(const struct bwriter_shader *shader, int dxversion, DWORD **result, DWORD *size)
+{
     struct bc_writer *writer;
     struct bytecode_buffer *buffer = NULL;
     HRESULT hr;
@@ -2380,21 +2596,17 @@ DWORD SlWriteBytecode(const struct bwriter_shader *shader, int dxversion, DWORD
         goto error;
     }
 
-    /* Cut off unneeded memory from the result buffer */
-    *result = asm_realloc(buffer->data,
-                         sizeof(DWORD) * buffer->size);
-    if(!*result) {
-        *result = buffer->data;
-    }
+    *size = buffer->size * sizeof(DWORD);
+    *result = buffer->data;
     buffer->data = NULL;
     hr = S_OK;
 
 error:
     if(buffer) {
-        asm_free(buffer->data);
-        asm_free(buffer);
+        d3dcompiler_free(buffer->data);
+        d3dcompiler_free(buffer);
     }
-    asm_free(writer);
+    d3dcompiler_free(writer);
     return hr;
 }
 
@@ -2404,30 +2616,30 @@ void SlDeleteShader(struct bwriter_shader *shader) {
     TRACE("Deleting shader %p\n", shader);
 
     for(i = 0; i < shader->num_cf; i++) {
-        asm_free(shader->constF[i]);
+        d3dcompiler_free(shader->constF[i]);
     }
-    asm_free(shader->constF);
+    d3dcompiler_free(shader->constF);
     for(i = 0; i < shader->num_ci; i++) {
-        asm_free(shader->constI[i]);
+        d3dcompiler_free(shader->constI[i]);
     }
-    asm_free(shader->constI);
+    d3dcompiler_free(shader->constI);
     for(i = 0; i < shader->num_cb; i++) {
-        asm_free(shader->constB[i]);
+        d3dcompiler_free(shader->constB[i]);
     }
-    asm_free(shader->constB);
+    d3dcompiler_free(shader->constB);
 
-    asm_free(shader->inputs);
-    asm_free(shader->outputs);
-    asm_free(shader->samplers);
+    d3dcompiler_free(shader->inputs);
+    d3dcompiler_free(shader->outputs);
+    d3dcompiler_free(shader->samplers);
 
     for(i = 0; i < shader->num_instrs; i++) {
         for(j = 0; j < shader->instr[i]->num_srcs; j++) {
-            asm_free(shader->instr[i]->src[j].rel_reg);
+            d3dcompiler_free(shader->instr[i]->src[j].rel_reg);
         }
-        asm_free(shader->instr[i]->src);
-        asm_free(shader->instr[i]);
+        d3dcompiler_free(shader->instr[i]->src);
+        d3dcompiler_free(shader->instr[i]);
     }
-    asm_free(shader->instr);
+    d3dcompiler_free(shader->instr);
 
-    asm_free(shader);
+    d3dcompiler_free(shader);
 }
