@@ -41,6 +41,7 @@ struct mem_file_desc
 
 static struct mem_file_desc current_shader;
 static ID3DInclude *current_include;
+static const char *initial_filename;
 
 #define INCLUDES_INITIAL_CAPACITY 4
 
@@ -143,7 +144,7 @@ static void wpp_warning(const char *file, int line, int col, const char *near,
     wpp_write_message_var("\n");
 }
 
-static char *wpp_lookup_mem(const char *filename, const char *parent_name,
+static char *wpp_lookup_mem(const char *filename, int type, const char *parent_name,
                             char **include_path, int include_path_count)
 {
     /* Here we return always ok. We will maybe fail on the next wpp_open_mem */
@@ -179,7 +180,7 @@ static void *wpp_open_mem(const char *filename, int type)
     struct mem_file_desc *desc;
     HRESULT hr;
 
-    if(filename[0] == '\0') /* "" means to load the initial shader */
+    if(!strcmp(filename, initial_filename))
     {
         current_shader.pos = 0;
         return &current_shader;
@@ -193,7 +194,7 @@ static void *wpp_open_mem(const char *filename, int type)
         return NULL;
     }
     hr = ID3DInclude_Open(current_include,
-                          type ? D3D_INCLUDE_SYSTEM : D3D_INCLUDE_LOCAL,
+                          type ? D3D_INCLUDE_LOCAL : D3D_INCLUDE_SYSTEM,
                           filename, parent_include, (LPCVOID *)&desc->buffer,
                           &desc->size);
     if(FAILED(hr))
@@ -310,7 +311,7 @@ static int wpp_close_output(void)
     return 1;
 }
 
-static HRESULT preprocess_shader(const void *data, SIZE_T data_size,
+static HRESULT preprocess_shader(const void *data, SIZE_T data_size, const char *filename,
         const D3D_SHADER_MACRO *defines, ID3DInclude *include, ID3DBlob **error_messages)
 {
     int ret;
@@ -347,8 +348,9 @@ static HRESULT preprocess_shader(const void *data, SIZE_T data_size,
     wpp_messages = NULL;
     current_shader.buffer = data;
     current_shader.size = data_size;
+    initial_filename = filename ? filename : "";
 
-    ret = wpp_parse("", NULL);
+    ret = wpp_parse(initial_filename, NULL);
     if (!wpp_close_output())
         ret = 1;
     if (ret)
@@ -359,7 +361,7 @@ static HRESULT preprocess_shader(const void *data, SIZE_T data_size,
             int size;
             ID3DBlob *buffer;
 
-            TRACE("Preprocessor messages:\n%s", debugstr_a(wpp_messages));
+            TRACE("Preprocessor messages:\n%s\n", debugstr_a(wpp_messages));
 
             if (error_messages)
             {
@@ -396,9 +398,8 @@ static HRESULT assemble_shader(const char *preproc_shader,
     struct bwriter_shader *shader;
     char *messages = NULL;
     HRESULT hr;
-    DWORD *res;
-    LPD3DBLOB buffer;
-    int size;
+    DWORD *res, size;
+    ID3DBlob *buffer;
     char *pos;
 
     shader = SlAssembleShader(preproc_shader, &messages);
@@ -406,7 +407,7 @@ static HRESULT assemble_shader(const char *preproc_shader,
     if (messages)
     {
         TRACE("Assembler messages:\n");
-        TRACE("%s", messages);
+        TRACE("%s\n", debugstr_a(messages));
 
         TRACE("Shader source:\n");
         TRACE("%s\n", debugstr_a(preproc_shader));
@@ -443,7 +444,7 @@ static HRESULT assemble_shader(const char *preproc_shader,
         return D3DXERR_INVALIDDATA;
     }
 
-    hr = SlWriteBytecode(shader, 9, &res);
+    hr = SlWriteBytecode(shader, 9, &res, &size);
     SlDeleteShader(shader);
     if (FAILED(hr))
     {
@@ -453,7 +454,6 @@ static HRESULT assemble_shader(const char *preproc_shader,
 
     if (shader_blob)
     {
-        size = HeapSize(GetProcessHeap(), 0, res);
         hr = D3DCreateBlob(size, &buffer);
         if (FAILED(hr))
         {
@@ -483,7 +483,7 @@ HRESULT WINAPI D3DAssemble(const void *data, SIZE_T datasize, const char *filena
     if (shader) *shader = NULL;
     if (error_messages) *error_messages = NULL;
 
-    hr = preprocess_shader(data, datasize, defines, include, error_messages);
+    hr = preprocess_shader(data, datasize, filename, defines, include, error_messages);
     if (SUCCEEDED(hr))
         hr = assemble_shader(wpp_output, shader, error_messages);
 
@@ -492,21 +492,138 @@ HRESULT WINAPI D3DAssemble(const void *data, SIZE_T datasize, const char *filena
     return hr;
 }
 
+static HRESULT compile_shader(const char *preproc_shader, const char *target, const char *entrypoint,
+        ID3DBlob **shader_blob, ID3DBlob **error_messages)
+{
+    struct bwriter_shader *shader;
+    char *messages = NULL;
+    HRESULT hr;
+    DWORD *res, size, major, minor;
+    ID3DBlob *buffer;
+    char *pos;
+    enum shader_type shader_type;
+
+    TRACE("Preprocessed shader source: %s\n", debugstr_a(preproc_shader));
+
+    TRACE("Parsing compilation target %s.\n", debugstr_a(target));
+    if (strlen(target) != 6 || target[1] != 's' || target[2] != '_' || target[4] != '_')
+    {
+        FIXME("Unknown compilation target %s.\n", debugstr_a(target));
+        return D3DERR_INVALIDCALL;
+    }
+
+    if (target[0] == 'v')
+        shader_type = ST_VERTEX;
+    else if (target[0] == 'p')
+        shader_type = ST_PIXEL;
+    else
+    {
+        FIXME("Unsupported shader target type %s.\n", debugstr_a(target));
+        return D3DERR_INVALIDCALL;
+    }
+
+    major = target[3] - '0';
+    if (major == 0 || major > 5)
+    {
+        FIXME("Unsupported shader target major version %d.\n", major);
+        return D3DERR_INVALIDCALL;
+    }
+    minor = target[5] - '0';
+    if (minor > 1 || (minor == 1 && (shader_type != ST_VERTEX || major > 1)))
+    {
+        FIXME("Unsupported shader target minor version %d.\n", minor);
+        return D3DERR_INVALIDCALL;
+    }
+    shader = parse_hlsl_shader(preproc_shader, shader_type, major, minor, entrypoint, &messages);
+
+    if (messages)
+    {
+        TRACE("Compiler messages:\n");
+        TRACE("%s\n", debugstr_a(messages));
+
+        TRACE("Shader source:\n");
+        TRACE("%s\n", debugstr_a(preproc_shader));
+
+        if (error_messages)
+        {
+            const char *preproc_messages = *error_messages ? ID3D10Blob_GetBufferPointer(*error_messages) : NULL;
+
+            size = strlen(messages) + (preproc_messages ? strlen(preproc_messages) : 0) + 1;
+            hr = D3DCreateBlob(size, &buffer);
+            if (FAILED(hr))
+            {
+                HeapFree(GetProcessHeap(), 0, messages);
+                if (shader) SlDeleteShader(shader);
+                return hr;
+            }
+            pos = ID3D10Blob_GetBufferPointer(buffer);
+            if (preproc_messages)
+            {
+                memcpy(pos, preproc_messages, strlen(preproc_messages) + 1);
+                pos += strlen(preproc_messages);
+            }
+            memcpy(pos, messages, strlen(messages) + 1);
+
+            if (*error_messages) ID3D10Blob_Release(*error_messages);
+            *error_messages = buffer;
+        }
+        HeapFree(GetProcessHeap(), 0, messages);
+    }
+
+    if (!shader)
+    {
+        ERR("HLSL shader parsing failed.\n");
+        return D3DXERR_INVALIDDATA;
+    }
+
+    hr = SlWriteBytecode(shader, 9, &res, &size);
+    SlDeleteShader(shader);
+    if (FAILED(hr))
+    {
+        ERR("SlWriteBytecode failed with error 0x%08x.\n", hr);
+        return D3DXERR_INVALIDDATA;
+    }
+
+    if (shader_blob)
+    {
+        hr = D3DCreateBlob(size, &buffer);
+        if (FAILED(hr))
+        {
+            HeapFree(GetProcessHeap(), 0, res);
+            return hr;
+        }
+        memcpy(ID3D10Blob_GetBufferPointer(buffer), res, size);
+        *shader_blob = buffer;
+    }
+
+    HeapFree(GetProcessHeap(), 0, res);
+
+    return S_OK;
+}
+
 HRESULT WINAPI D3DCompile(const void *data, SIZE_T data_size, const char *filename,
         const D3D_SHADER_MACRO *defines, ID3DInclude *include, const char *entrypoint,
         const char *target, UINT sflags, UINT eflags, ID3DBlob **shader, ID3DBlob **error_messages)
 {
+    HRESULT hr;
+
     FIXME("data %p, data_size %lu, filename %s, defines %p, include %p, entrypoint %s,\n"
             "target %s, sflags %#x, eflags %#x, shader %p, error_messages %p stub!\n",
             data, data_size, debugstr_a(filename), defines, include, debugstr_a(entrypoint),
             debugstr_a(target), sflags, eflags, shader, error_messages);
 
-    TRACE("Shader source:\n%s\n", debugstr_an(data, data_size));
+    if (shader) *shader = NULL;
+    if (error_messages) *error_messages = NULL;
 
-    if (error_messages)
-        D3DCreateBlob(1, error_messages); /* zero fill used as string end */
+    EnterCriticalSection(&wpp_mutex);
 
-    return D3DERR_INVALIDCALL;
+    hr = preprocess_shader(data, data_size, filename, defines, include, error_messages);
+    if (SUCCEEDED(hr))
+        hr = compile_shader(wpp_output, target, entrypoint, shader, error_messages);
+
+    HeapFree(GetProcessHeap(), 0, wpp_output);
+    LeaveCriticalSection(&wpp_mutex);
+    return hr;
 }
 
 HRESULT WINAPI D3DPreprocess(const void *data, SIZE_T size, const char *filename,
@@ -524,7 +641,7 @@ HRESULT WINAPI D3DPreprocess(const void *data, SIZE_T size, const char *filename
     if (shader) *shader = NULL;
     if (error_messages) *error_messages = NULL;
 
-    hr = preprocess_shader(data, size, defines, include, error_messages);
+    hr = preprocess_shader(data, size, filename, defines, include, error_messages);
 
     if (SUCCEEDED(hr))
     {
