@@ -30,7 +30,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(ddraw);
  * Helper functions
  *****************************************************************************/
 
-static void update_clip_space(IDirect3DDeviceImpl *device,
+static void update_clip_space(struct d3d_device *device,
         struct wined3d_vec3 *scale, struct wined3d_vec3 *offset)
 {
     D3DMATRIX clip_space =
@@ -529,7 +529,7 @@ static HRESULT WINAPI d3d_viewport_LightElements(IDirect3DViewport3 *iface,
 /*****************************************************************************
  * IDirect3DViewport3::SetBackground
  *
- * Sets tje background material
+ * Sets the background material
  *
  * Params:
  *  hMat: Handle from a IDirect3DMaterial interface
