@@ -570,7 +570,7 @@ int find_property(const DataFormat *df, LPCDIPROPHEADER ph)
     return -1;
 }
 
-DWORD semantic_to_obj_id(IDirectInputDeviceImpl* This, DWORD dwSemantic)
+static DWORD semantic_to_obj_id(IDirectInputDeviceImpl* This, DWORD dwSemantic)
 {
     DWORD type = (0x0000ff00 & dwSemantic) >> 8;
     DWORD offset = 0x000000ff & dwSemantic;
@@ -605,7 +605,7 @@ HRESULT _build_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf,
 
     for (i=0; i < lpdiaf->dwNumActions; i++)
     {
-        /* Don't touch an user configured action */
+        /* Don't touch a user configured action */
         if (lpdiaf->rgoAction[i].dwHow == DIAH_USERCONFIG) continue;
 
         if ((lpdiaf->rgoAction[i].dwSemantic & devMask) == devMask)
@@ -619,7 +619,7 @@ HRESULT _build_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf,
             if (type == DIDFT_PSHBUTTON) type = DIDFT_BUTTON;
             if (type == DIDFT_RELAXIS) type = DIDFT_AXIS;
 
-            /* Assure that the object exists */
+            /* Make sure the object exists */
             odf = dataformat_to_odf_by_type(df, inst, type);
 
             if (odf != NULL)
@@ -632,7 +632,7 @@ HRESULT _build_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf,
         }
         else if (!(dwFlags & DIDBAM_PRESERVE))
         {
-            /* we must clear action data belonging to other devices */
+            /* We must clear action data belonging to other devices */
             memset(&lpdiaf->rgoAction[i].guidInstance, 0, sizeof(GUID));
             lpdiaf->rgoAction[i].dwHow = DIAH_UNMAPPED;
         }
@@ -783,6 +783,8 @@ HRESULT WINAPI IDirectInputDevice2WImpl_Acquire(LPDIRECTINPUTDEVICE8W iface)
     IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
     HRESULT res;
 
+    TRACE("(%p)\n", This);
+
     if (!This->data_format.user_df) return DIERR_INVALIDPARAM;
     if (This->dwCoopLevel & DISCL_FOREGROUND && This->win != GetForegroundWindow())
         return DIERR_OTHERAPPHASPRIO;
@@ -791,10 +793,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_Acquire(LPDIRECTINPUTDEVICE8W iface)
     res = This->acquired ? S_FALSE : DI_OK;
     This->acquired = 1;
     if (res == DI_OK)
-    {
-        This->queue_head = This->queue_tail = This->overflow = 0;
         check_dinput_hooks(iface);
-    }
     LeaveCriticalSection(&This->crit);
 
     return res;
@@ -816,6 +815,8 @@ HRESULT WINAPI IDirectInputDevice2WImpl_Unacquire(LPDIRECTINPUTDEVICE8W iface)
     IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
     HRESULT res;
 
+    TRACE("(%p)\n", This);
+
     EnterCriticalSection(&This->crit);
     res = !This->acquired ? DI_NOEFFECT : DI_OK;
     This->acquired = 0;
@@ -881,6 +882,8 @@ HRESULT WINAPI IDirectInputDevice2WImpl_SetCooperativeLevel(LPDIRECTINPUTDEVICE8
         (dwflags & (DISCL_FOREGROUND | DISCL_BACKGROUND)) == (DISCL_FOREGROUND | DISCL_BACKGROUND))
         return DIERR_INVALIDPARAM;
 
+    if (hwnd && GetWindowLongW(hwnd, GWL_STYLE) & WS_CHILD) return E_HANDLE;
+
     if (dwflags == (DISCL_NONEXCLUSIVE | DISCL_BACKGROUND))
         hwnd = GetDesktopWindow();
 
@@ -933,9 +936,10 @@ HRESULT WINAPI IDirectInputDevice2AImpl_SetEventNotification(LPDIRECTINPUTDEVICE
 ULONG WINAPI IDirectInputDevice2WImpl_Release(LPDIRECTINPUTDEVICE8W iface)
 {
     IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
-    ULONG ref;
+    ULONG ref = InterlockedDecrement(&(This->ref));
+
+    TRACE("(%p) releasing from %d\n", This, ref + 1);
 
-    ref = InterlockedDecrement(&(This->ref));
     if (ref) return ref;
 
     IDirectInputDevice_Unacquire(iface);
@@ -1286,10 +1290,14 @@ HRESULT WINAPI IDirectInputDevice2WImpl_GetDeviceData(LPDIRECTINPUTDEVICE8W ifac
     TRACE("(%p) %p -> %p(%d) x%d, 0x%08x\n",
           This, dod, entries, entries ? *entries : 0, dodsize, flags);
 
-    if (!This->acquired)
-        return DIERR_NOTACQUIRED;
+    if (This->dinput->dwVersion == 0x0800 || dodsize == sizeof(DIDEVICEOBJECTDATA_DX3))
+    {
+        if (!This->queue_len) return DIERR_NOTBUFFERED;
+        if (!This->acquired) return DIERR_NOTACQUIRED;
+    }
+
     if (!This->queue_len)
-        return DIERR_NOTBUFFERED;
+        return DI_OK;
     if (dodsize < sizeof(DIDEVICEOBJECTDATA_DX3))
         return DIERR_INVALIDPARAM;
 
@@ -1312,7 +1320,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_GetDeviceData(LPDIRECTINPUTDEVICE8W ifac
     }
     *entries = len;
 
-    if (This->overflow)
+    if (This->overflow && This->dinput->dwVersion == 0x0800)
         ret = DI_BUFFEROVERFLOW;
 
     if (!(flags & DIGDD_PEEK))
