@@ -31,6 +31,7 @@ struct enum_data {
     LPDIACTIONFORMAT lpdiaf;
     LPDIRECTINPUTDEVICE8 keyboard;
     LPDIRECTINPUTDEVICE8 mouse;
+    const char* username;
     int ndevices;
 };
 
@@ -69,12 +70,13 @@ static void test_device_input(
     HRESULT hr;
     DIDEVICEOBJECTDATA obj_data;
     DWORD data_size = 1;
+    int i;
 
     hr = IDirectInputDevice8_Acquire(lpdid);
     ok (SUCCEEDED(hr), "Failed to acquire device hr=%08x\n", hr);
 
     if (event_type == INPUT_KEYBOARD)
-        keybd_event( event, 0, 0, 0);
+        keybd_event( event, DIK_SPACE, 0, 0);
 
     if (event_type == INPUT_MOUSE)
         mouse_event( event, 0, 0, 0, 0);
@@ -89,6 +91,28 @@ static void test_device_input(
     }
 
     ok (obj_data.uAppData == expected, "Retrieval of action failed uAppData=%lu expected=%d\n", obj_data.uAppData, expected);
+
+    /* Check for buffer owerflow */
+    for (i = 0; i < 17; i++)
+        if (event_type == INPUT_KEYBOARD)
+        {
+            keybd_event( VK_SPACE, DIK_SPACE, 0, 0);
+            keybd_event( VK_SPACE, DIK_SPACE, KEYEVENTF_KEYUP, 0);
+        }
+        else if (event_type == INPUT_MOUSE)
+        {
+            mouse_event(MOUSEEVENTF_LEFTDOWN, 1, 1, 0, 0);
+            mouse_event(MOUSEEVENTF_LEFTUP, 1, 1, 0, 0);
+        }
+
+    IDirectInputDevice8_Poll(lpdid);
+
+    data_size = 1;
+    hr = IDirectInputDevice8_GetDeviceData(lpdid, sizeof(obj_data), &obj_data, &data_size, 0);
+    ok(hr == DI_BUFFEROVERFLOW, "GetDeviceData() failed: %08x\n", hr);
+    data_size = 1;
+    hr = IDirectInputDevice8_GetDeviceData(lpdid, sizeof(obj_data), &obj_data, &data_size, 0);
+    ok(hr == DI_OK && data_size == 1, "GetDeviceData() failed: %08x cnt:%d\n", hr, data_size);
 }
 
 static void test_build_action_map(
@@ -133,11 +157,26 @@ static BOOL CALLBACK enumeration_callback(
     HRESULT hr;
     DIPROPDWORD dp;
     DIPROPRANGE dpr;
+    DIPROPSTRING dps;
+    WCHAR usernameW[MAX_PATH];
+    DWORD username_size = MAX_PATH;
     struct enum_data *data = pvRef;
+    DWORD cnt;
+    DIDEVICEOBJECTDATA buffer[5];
+
     if (!data) return DIENUM_CONTINUE;
 
     data->ndevices++;
 
+    /* Convert username to WCHAR */
+    if (data->username != NULL)
+    {
+        username_size = MultiByteToWideChar(CP_ACP, 0, data->username, -1, usernameW, 0);
+        MultiByteToWideChar(CP_ACP, 0, data->username, -1, usernameW, username_size);
+    }
+    else
+        GetUserNameW(usernameW, &username_size);
+
     /* collect the mouse and keyboard */
     if (IsEqualGUID(&lpddi->guidInstance, &GUID_SysKeyboard))
     {
@@ -164,12 +203,23 @@ static BOOL CALLBACK enumeration_callback(
     hr = IDirectInputDevice8_Acquire(lpdid);
     ok (hr == DIERR_INVALIDPARAM, "Device was acquired before SetActionMap hr=%08x\n", hr);
 
-    hr = IDirectInputDevice8_SetActionMap(lpdid, data->lpdiaf, NULL, 0);
+    hr = IDirectInputDevice8_SetActionMap(lpdid, data->lpdiaf, data->username, 0);
     ok (SUCCEEDED(hr), "SetActionMap failed hr=%08x\n", hr);
 
     /* Some joysticks may have no suitable actions and thus should not be tested */
     if (hr == DI_NOEFFECT) return DIENUM_CONTINUE;
 
+    /* Test username after SetActionMap */
+    dps.diph.dwSize = sizeof(dps);
+    dps.diph.dwHeaderSize = sizeof(DIPROPHEADER);
+    dps.diph.dwObj = 0;
+    dps.diph.dwHow  = DIPH_DEVICE;
+    dps.wsz[0] = '\0';
+
+    hr = IDirectInputDevice_GetProperty(lpdid, DIPROP_USERNAME, &dps.diph);
+    todo_wine ok (SUCCEEDED(hr), "GetProperty failed hr=%08x\n", hr);
+    todo_wine ok (!lstrcmpW(usernameW, dps.wsz), "Username not set correctly expected=%s, got=%s\n", wine_dbgstr_wn(usernameW, -1), wine_dbgstr_wn(dps.wsz, -1));
+
     /* Test buffer size */
     memset(&dp, 0, sizeof(dp));
     dp.diph.dwSize = sizeof(dp);
@@ -180,6 +230,10 @@ static BOOL CALLBACK enumeration_callback(
     ok (SUCCEEDED(hr), "GetProperty failed hr=%08x\n", hr);
     ok (dp.dwData == data->lpdiaf->dwBufferSize, "SetActionMap must set the buffer, buffersize=%d\n", dp.dwData);
 
+    cnt = 1;
+    hr = IDirectInputDevice_GetDeviceData(lpdid, sizeof(buffer[0]), buffer, &cnt, 0);
+    ok(hr == DIERR_NOTACQUIRED, "GetDeviceData() failed hr=%08x\n", hr);
+
     /* Test axis range */
     memset(&dpr, 0, sizeof(dpr));
     dpr.diph.dwSize = sizeof(dpr);
@@ -198,6 +252,10 @@ static BOOL CALLBACK enumeration_callback(
     hr = IDirectInputDevice8_Acquire(lpdid);
     ok (SUCCEEDED(hr), "Acquire failed hr=%08x\n", hr);
 
+    cnt = 1;
+    hr = IDirectInputDevice_GetDeviceData(lpdid, sizeof(buffer[0]), buffer, &cnt, 0);
+    ok(hr == DI_OK, "GetDeviceData() failed hr=%08x\n", hr);
+
     /* SetActionMap should not work on an acquired device */
     hr = IDirectInputDevice8_SetActionMap(lpdid, data->lpdiaf, NULL, 0);
     ok (hr == DIERR_ACQUIRED, "SetActionMap succeeded with an acquired device hr=%08x\n", hr);
@@ -205,39 +263,15 @@ static BOOL CALLBACK enumeration_callback(
     return DIENUM_CONTINUE;
 }
 
-/*  A simpler callback function used to count and check
-    the enumeration of devices.
-*/
-static BOOL CALLBACK counting_callback(
-    LPCDIDEVICEINSTANCE lpddi,
-    LPDIRECTINPUTDEVICE8 lpdid,
-    DWORD dwFlags,
-    DWORD dwRemaining,
-    LPVOID pvRef)
-{
-    struct enum_data *data = pvRef;
-    if (!data) return DIENUM_CONTINUE;
-
-    data->ndevices++;
-    if (IsEqualGUID(&lpddi->guidInstance, &GUID_SysKeyboard))
-        data->keyboard = lpdid;
-
-    if (IsEqualGUID(&lpddi->guidInstance, &GUID_SysMouse))
-        data->mouse = lpdid;
-
-    return DIENUM_CONTINUE;
-}
-
 static void test_action_mapping(void)
 {
     HRESULT hr;
     HINSTANCE hinst = GetModuleHandle(NULL);
     LPDIRECTINPUT8 pDI = NULL;
     DIACTIONFORMAT af;
-    struct enum_data data =  {pDI, &af, NULL, NULL, 0};
-    struct enum_data count = {pDI, &af, NULL, NULL, 0};
+    struct enum_data data =  {pDI, &af, NULL, NULL, NULL, 0};
 
-    hr = CoCreateInstance(&CLSID_DirectInput8, 0, 1, &IID_IDirectInput8A, (LPVOID*)&pDI);
+    hr = CoCreateInstance(&CLSID_DirectInput8, 0, CLSCTX_INPROC_SERVER, &IID_IDirectInput8A, (LPVOID*)&pDI);
     if (hr == DIERR_OLDDIRECTINPUTVERSION ||
         hr == DIERR_BETADIRECTINPUTVERSION ||
         hr == REGDB_E_CLASSNOTREG)
@@ -267,42 +301,15 @@ static void test_action_mapping(void)
     af.dwGenre = 0x01000000; /* DIVIRTUAL_DRIVING_RACE */
     af.dwBufferSize = 32;
 
-    /* Test enumerating all attached and installed devices */
-    count.keyboard = NULL;
-    count.mouse = NULL;
-    count.ndevices = 0;
-    hr = IDirectInput8_EnumDevicesBySemantics(pDI, 0, &af, counting_callback, &count, DIEDBSFL_ATTACHEDONLY);
-    ok (count.ndevices > 0, "EnumDevicesBySemantics did not call the callback hr=%08x\n", hr);
-    ok (count.keyboard != NULL, "EnumDevicesBySemantics should enumerate the keyboard\n");
-    ok (count.mouse != NULL, "EnumDevicesBySemantics should enumerate the mouse\n");
-
-    /* Enumerate Force feedback devices. We should get no mouse nor keyboard */
-    count.keyboard = NULL;
-    count.mouse = NULL;
-    hr = IDirectInput8_EnumDevicesBySemantics(pDI, 0, &af, counting_callback, &count, DIEDBSFL_FORCEFEEDBACK);
-    ok (SUCCEEDED(hr), "EnumDevicesBySemantics failed hr=%08x\n", hr);
-    ok (count.keyboard == NULL, "Keyboard should not be enumerated when asking for forcefeedback\n");
-    ok (count.mouse == NULL, "Mouse should not be enumerated when asking for forcefeedback\n");
-
-    /* Enumerate available devices. That is devices with not owned by any user.
-       Before setting the action map for all devices we still have them available.
-    */
-    count.ndevices = 0;
-    hr = IDirectInput8_EnumDevicesBySemantics(pDI, 0, &af, counting_callback, &count, DIEDBSFL_AVAILABLEDEVICES);
-    ok (SUCCEEDED(hr), "EnumDevicesBySemantics failed hr=%08x\n", hr);
-    ok (count.ndevices > 0, "There should be devices available before action mapping available=%d\n", count.ndevices);
-
     /* This enumeration builds and sets the action map for all devices */
     hr = IDirectInput8_EnumDevicesBySemantics(pDI, 0, &af, enumeration_callback, &data, DIEDBSFL_ATTACHEDONLY);
     ok (SUCCEEDED(hr), "EnumDevicesBySemantics failed: hr=%08x\n", hr);
 
-    /* After a successful action mapping we should have no devices available */
-    count.ndevices = 0;
-    hr = IDirectInput8_EnumDevicesBySemantics(pDI, 0, &af, counting_callback, &count, DIEDBSFL_AVAILABLEDEVICES);
-    ok (SUCCEEDED(hr), "EnumDevicesBySemantics failed hr=%08x\n", hr);
-    todo_wine ok (count.ndevices == 0, "No device should be available after action mapping available=%d\n", count.ndevices);
+    /* Repeat tests with a non NULL user */
+    data.username = "Ninja Brian";
+    hr = IDirectInput8_EnumDevicesBySemantics(pDI, NULL, &af, enumeration_callback, &data, DIEDBSFL_ATTACHEDONLY);
+    ok (SUCCEEDED(hr), "EnumDevicesBySemantics failed: hr=%08x\n", hr);
 
-    /* Use the devices we collect for some tests */
     if (data.keyboard != NULL)
     {
         /* Test keyboard BuildActionMap */
@@ -333,11 +340,6 @@ static void test_action_mapping(void)
 
         test_device_input(data.mouse, INPUT_MOUSE, MOUSEEVENTF_LEFTDOWN, 3);
     }
-
-    /* The call fails with a zeroed GUID */
-    memset(&af.guidActionMap, 0, sizeof(GUID));
-    hr = IDirectInput8_EnumDevicesBySemantics(pDI, 0, &af, enumeration_callback, 0, 0);
-    todo_wine ok(FAILED(hr), "EnumDevicesBySemantics succeeded with invalid GUID hr=%08x\n", hr);
 }
 
 START_TEST(device)
