@@ -1,4 +1,5 @@
-/* DirectMusic Private Include
+/*
+ * DirectMusic Private Include
  *
  * Copyright (C) 2003-2004 Rok Mandeljc
  *
@@ -75,38 +76,46 @@ typedef struct DMUSIC_PRIVATE_CHANNEL_GROUP_ {
 /*****************************************************************************
  * ClassFactory
  */
-extern HRESULT WINAPI DMUSIC_CreateDirectMusicImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
-extern HRESULT WINAPI DMUSIC_CreateDirectMusicBufferImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
-extern HRESULT WINAPI DMUSIC_CreateDirectMusicDownloadedInstrumentImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
-extern HRESULT WINAPI DMUSIC_CreateDirectMusicDownloadImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
-extern HRESULT WINAPI DMUSIC_CreateReferenceClockImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
 
-extern HRESULT WINAPI DMUSIC_CreateDirectMusicCollectionImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
-extern HRESULT WINAPI DMUSIC_CreateDirectMusicInstrumentImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
+/* CLSID */
+extern HRESULT WINAPI DMUSIC_CreateDirectMusicImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
+extern HRESULT WINAPI DMUSIC_CreateDirectMusicCollectionImpl(LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
+
+/* Internal */
+extern HRESULT DMUSIC_CreateDirectMusicBufferImpl(LPDMUS_BUFFERDESC desc, LPVOID* ret_iface) DECLSPEC_HIDDEN;
+extern HRESULT DMUSIC_CreateDirectMusicDownloadedInstrumentImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
+extern HRESULT DMUSIC_CreateDirectMusicDownloadImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
+extern HRESULT DMUSIC_CreateReferenceClockImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
+extern HRESULT DMUSIC_CreateDirectMusicInstrumentImpl (LPCGUID lpcGUID, LPVOID* ppobj, LPUNKNOWN pUnkOuter) DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IDirectMusic8Impl implementation structure
  */
 struct IDirectMusic8Impl {
-  /* IUnknown fields */
-  const IDirectMusic8Vtbl *lpVtbl;
-  LONG           ref;
+    /* IUnknown fields */
+    IDirectMusic8 IDirectMusic8_iface;
+    LONG ref;
 
-  /* IDirectMusicImpl fields */
-  IReferenceClockImpl* pMasterClock;
-  IDirectMusicPort** ppPorts;
-  int nrofports;
+    /* IDirectMusicImpl fields */
+    IReferenceClockImpl* pMasterClock;
+    IDirectMusicPort** ppPorts;
+    int nrofports;
 };
 
 /*****************************************************************************
  * IDirectMusicBufferImpl implementation structure
  */
 struct IDirectMusicBufferImpl {
-  /* IUnknown fields */
-  const IDirectMusicBufferVtbl *lpVtbl;
-  LONG           ref;
+    /* IUnknown fields */
+    IDirectMusicBuffer IDirectMusicBuffer_iface;
+    LONG ref;
 
-  /* IDirectMusicBufferImpl fields */
+    /* IDirectMusicBufferImpl fields */
+    GUID format;
+    DWORD size;
+    LPBYTE data;
+    DWORD write_pos;
+    REFERENCE_TIME start_time;
 };
 
 /*****************************************************************************
@@ -135,38 +144,36 @@ struct IDirectMusicDownloadImpl {
  * IDirectMusicPortImpl implementation structure
  */
 struct IDirectMusicPortImpl {
-  /* IUnknown fields */
-  const IDirectMusicPortVtbl *lpVtbl;
-  const IDirectMusicPortDownloadVtbl *lpDownloadVtbl;
-  const IDirectMusicThruVtbl *lpThruVtbl;
-  LONG           ref;
+    /* IUnknown fields */
+    IDirectMusicPort IDirectMusicPort_iface;
+    IDirectMusicPortDownload IDirectMusicPortDownload_iface;
+    IDirectMusicThru IDirectMusicThru_iface;
+    LONG ref;
 
-  /* IDirectMusicPortImpl fields */
-  IDirectSound* pDirectSound;
-  IReferenceClock* pLatencyClock;
-  BOOL fActive;
-  DMUS_PORTCAPS caps;
-  DMUS_PORTPARAMS params;
-  int nrofgroups;
-  DMUSIC_PRIVATE_CHANNEL_GROUP group[1];
+    /* IDirectMusicPortImpl fields */
+    IDirectSound* pDirectSound;
+    IReferenceClock* pLatencyClock;
+    BOOL fActive;
+    DMUS_PORTCAPS caps;
+    DMUS_PORTPARAMS params;
+    int nrofgroups;
+    DMUSIC_PRIVATE_CHANNEL_GROUP group[1];
 };
 
-extern HRESULT WINAPI IDirectMusicPortImpl_Activate (LPDIRECTMUSICPORT iface, BOOL fActive) DECLSPEC_HIDDEN;
-
 /** Internal factory */
-extern HRESULT WINAPI DMUSIC_CreateDirectMusicPortImpl (LPCGUID lpcGUID, LPVOID *ppobj, LPUNKNOWN pUnkOuter, LPDMUS_PORTPARAMS pPortParams, LPDMUS_PORTCAPS pPortCaps) DECLSPEC_HIDDEN;
+extern HRESULT DMUSIC_CreateDirectMusicPortImpl (LPCGUID lpcGUID, LPVOID *ppobj, LPUNKNOWN pUnkOuter, LPDMUS_PORTPARAMS pPortParams, LPDMUS_PORTCAPS pPortCaps) DECLSPEC_HIDDEN;
 
 /*****************************************************************************
  * IReferenceClockImpl implementation structure
  */
 struct IReferenceClockImpl {
-  /* IUnknown fields */
-  const IReferenceClockVtbl *lpVtbl;
-  LONG           ref;
+    /* IUnknown fields */
+    IReferenceClock IReferenceClock_iface;
+    LONG ref;
 
-  /* IReferenceClockImpl fields */
-  REFERENCE_TIME rtTime;
-  DMUS_CLOCKINFO pClockInfo;
+    /* IReferenceClockImpl fields */
+    REFERENCE_TIME rtTime;
+    DMUS_CLOCKINFO pClockInfo;
 };
 
 typedef struct _DMUS_PRIVATE_INSTRUMENT_ENTRY {
@@ -182,25 +189,24 @@ typedef struct _DMUS_PRIVATE_POOLCUE {
  * IDirectMusicCollectionImpl implementation structure
  */
 struct IDirectMusicCollectionImpl {
-  /* IUnknown fields */
-  const IUnknownVtbl *UnknownVtbl;
-  const IDirectMusicCollectionVtbl *CollectionVtbl;
-  const IDirectMusicObjectVtbl *ObjectVtbl;
-  const IPersistStreamVtbl *PersistStreamVtbl;
-  LONG           ref;
+    /* IUnknown fields */
+    IDirectMusicCollection IDirectMusicCollection_iface;
+    IDirectMusicObject IDirectMusicObject_iface;
+    IPersistStream IPersistStream_iface;
+    LONG ref;
 
-  /* IDirectMusicCollectionImpl fields */
-  IStream *pStm; /* stream from which we load collection and later instruments */
-  LARGE_INTEGER liCollectionPosition; /* offset in a stream where collection was loaded from */
-  LARGE_INTEGER liWavePoolTablePosition; /* offset in a stream where wave pool table can be found */
-  LPDMUS_OBJECTDESC pDesc;
-  CHAR* szCopyright; /* FIXME: should probably placed somewhere else */
-  LPDLSHEADER pHeader;
-  /* pool table */
-  LPPOOLTABLE pPoolTable;
-  LPPOOLCUE pPoolCues;
-  /* instruments */
-  struct list Instruments;
+    /* IDirectMusicCollectionImpl fields */
+    IStream *pStm; /* stream from which we load collection and later instruments */
+    LARGE_INTEGER liCollectionPosition; /* offset in a stream where collection was loaded from */
+    LARGE_INTEGER liWavePoolTablePosition; /* offset in a stream where wave pool table can be found */
+    LPDMUS_OBJECTDESC pDesc;
+    CHAR* szCopyright; /* FIXME: should probably placed somewhere else */
+    LPDLSHEADER pHeader;
+    /* pool table */
+    LPPOOLTABLE pPoolTable;
+    LPPOOLCUE pPoolCues;
+    /* instruments */
+    struct list Instruments;
 };
 
 /*****************************************************************************
@@ -208,8 +214,7 @@ struct IDirectMusicCollectionImpl {
  */
 struct IDirectMusicInstrumentImpl {
   /* IUnknown fields */
-  const IUnknownVtbl *UnknownVtbl;
-  const IDirectMusicInstrumentVtbl *InstrumentVtbl;
+  IDirectMusicInstrument IDirectMusicInstrument_iface;
   LONG           ref;
 
   /* IDirectMusicInstrumentImpl fields */
@@ -220,8 +225,13 @@ struct IDirectMusicInstrumentImpl {
   /* instrument data */
 };
 
+static inline IDirectMusicInstrumentImpl *impl_from_IDirectMusicInstrument(IDirectMusicInstrument *iface)
+{
+    return CONTAINING_RECORD(iface, IDirectMusicInstrumentImpl, IDirectMusicInstrument_iface);
+}
+
 /* custom :) */
-extern HRESULT WINAPI IDirectMusicInstrumentImpl_Custom_Load (LPDIRECTMUSICINSTRUMENT iface, LPSTREAM pStm) DECLSPEC_HIDDEN;
+extern HRESULT IDirectMusicInstrumentImpl_Custom_Load (LPDIRECTMUSICINSTRUMENT iface, LPSTREAM pStm) DECLSPEC_HIDDEN;
 
 /**********************************************************************
  * Dll lifetime tracking declaration for dmusic.dll
@@ -276,7 +286,9 @@ extern int even_or_odd (DWORD number) DECLSPEC_HIDDEN;
 extern const char *debugstr_fourcc (DWORD fourcc) DECLSPEC_HIDDEN;
 /* returns name of given GUID */
 extern const char *debugstr_dmguid (const GUID *id) DECLSPEC_HIDDEN;
-/* dump whole DMUS_OBJECTDESC struct */
-extern const char *debugstr_DMUS_OBJECTDESC (LPDMUS_OBJECTDESC pDesc) DECLSPEC_HIDDEN;
+/* Dump whole DMUS_OBJECTDESC struct */
+extern void dump_DMUS_OBJECTDESC(LPDMUS_OBJECTDESC desc) DECLSPEC_HIDDEN;
+/* Dump whole DMUS_PORTPARAMS struct */
+extern void dump_DMUS_PORTPARAMS(LPDMUS_PORTPARAMS params) DECLSPEC_HIDDEN;
 
-#endif	/* __WINE_DMUSIC_PRIVATE_H */
+#endif /* __WINE_DMUSIC_PRIVATE_H */
