@@ -67,10 +67,25 @@ static const WCHAR szRevisionId[] = {'s','z','R','e','v','i','s','i','o','n','I'
 static const WCHAR dwRefreshRate[] = {'d','w','R','e','f','r','e','s','h','R','a','t','e',0};
 static const WCHAR szManufacturer[] = {'s','z','M','a','n','u','f','a','c','t','u','r','e','r',0};
 
-/* IDxDiagProvider IUnknown parts follow: */
-static HRESULT WINAPI IDxDiagProviderImpl_QueryInterface(PDXDIAGPROVIDER iface, REFIID riid, LPVOID *ppobj)
+struct IDxDiagProviderImpl
 {
-    IDxDiagProviderImpl *This = (IDxDiagProviderImpl *)iface;
+  IDxDiagProvider IDxDiagProvider_iface;
+  LONG ref;
+  BOOL init;
+  DXDIAG_INIT_PARAMS params;
+  IDxDiagContainerImpl_Container *info_root;
+};
+
+static inline IDxDiagProviderImpl *impl_from_IDxDiagProvider(IDxDiagProvider *iface)
+{
+     return CONTAINING_RECORD(iface, IDxDiagProviderImpl, IDxDiagProvider_iface);
+}
+
+/* IDxDiagProvider IUnknown parts follow: */
+static HRESULT WINAPI IDxDiagProviderImpl_QueryInterface(IDxDiagProvider *iface, REFIID riid,
+        void **ppobj)
+{
+    IDxDiagProviderImpl *This = impl_from_IDxDiagProvider(iface);
 
     if (!ppobj) return E_INVALIDARG;
 
@@ -86,8 +101,9 @@ static HRESULT WINAPI IDxDiagProviderImpl_QueryInterface(PDXDIAGPROVIDER iface,
     return E_NOINTERFACE;
 }
 
-static ULONG WINAPI IDxDiagProviderImpl_AddRef(PDXDIAGPROVIDER iface) {
-    IDxDiagProviderImpl *This = (IDxDiagProviderImpl *)iface;
+static ULONG WINAPI IDxDiagProviderImpl_AddRef(IDxDiagProvider *iface)
+{
+    IDxDiagProviderImpl *This = impl_from_IDxDiagProvider(iface);
     ULONG refCount = InterlockedIncrement(&This->ref);
 
     TRACE("(%p)->(ref before=%u)\n", This, refCount - 1);
@@ -97,8 +113,9 @@ static ULONG WINAPI IDxDiagProviderImpl_AddRef(PDXDIAGPROVIDER iface) {
     return refCount;
 }
 
-static ULONG WINAPI IDxDiagProviderImpl_Release(PDXDIAGPROVIDER iface) {
-    IDxDiagProviderImpl *This = (IDxDiagProviderImpl *)iface;
+static ULONG WINAPI IDxDiagProviderImpl_Release(IDxDiagProvider *iface)
+{
+    IDxDiagProviderImpl *This = impl_from_IDxDiagProvider(iface);
     ULONG refCount = InterlockedDecrement(&This->ref);
 
     TRACE("(%p)->(ref before=%u)\n", This, refCount + 1);
@@ -114,8 +131,10 @@ static ULONG WINAPI IDxDiagProviderImpl_Release(PDXDIAGPROVIDER iface) {
 }
 
 /* IDxDiagProvider Interface follow: */
-static HRESULT WINAPI IDxDiagProviderImpl_Initialize(PDXDIAGPROVIDER iface, DXDIAG_INIT_PARAMS* pParams) {
-    IDxDiagProviderImpl *This = (IDxDiagProviderImpl *)iface;
+static HRESULT WINAPI IDxDiagProviderImpl_Initialize(IDxDiagProvider *iface,
+        DXDIAG_INIT_PARAMS *pParams)
+{
+    IDxDiagProviderImpl *This = impl_from_IDxDiagProvider(iface);
     HRESULT hr;
 
     TRACE("(%p,%p)\n", iface, pParams);
@@ -140,8 +159,10 @@ static HRESULT WINAPI IDxDiagProviderImpl_Initialize(PDXDIAGPROVIDER iface, DXDI
     return S_OK;
 }
 
-static HRESULT WINAPI IDxDiagProviderImpl_GetRootContainer(PDXDIAGPROVIDER iface, IDxDiagContainer** ppInstance) {
-  IDxDiagProviderImpl *This = (IDxDiagProviderImpl *)iface;
+static HRESULT WINAPI IDxDiagProviderImpl_GetRootContainer(IDxDiagProvider *iface,
+        IDxDiagContainer **ppInstance)
+{
+  IDxDiagProviderImpl *This = impl_from_IDxDiagProvider(iface);
 
   TRACE("(%p,%p)\n", iface, ppInstance);
 
@@ -150,7 +171,7 @@ static HRESULT WINAPI IDxDiagProviderImpl_GetRootContainer(PDXDIAGPROVIDER iface
   }
 
   return DXDiag_CreateDXDiagContainer(&IID_IDxDiagContainer, This->info_root,
-                                      (IDxDiagProvider *)This, (void **)ppInstance);
+          &This->IDxDiagProvider_iface, (void **)ppInstance);
 }
 
 static const IDxDiagProviderVtbl DxDiagProvider_Vtbl =
@@ -172,9 +193,9 @@ HRESULT DXDiag_CreateDXDiagProvider(LPCLASSFACTORY iface, LPUNKNOWN punkOuter, R
 
   provider = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(IDxDiagProviderImpl));
   if (NULL == provider) return E_OUTOFMEMORY;
-  provider->lpVtbl = &DxDiagProvider_Vtbl;
+  provider->IDxDiagProvider_iface.lpVtbl = &DxDiagProvider_Vtbl;
   provider->ref = 0; /* will be inited with QueryInterface */
-  return IDxDiagProviderImpl_QueryInterface ((PDXDIAGPROVIDER)provider, riid, ppobj);
+  return IDxDiagProviderImpl_QueryInterface(&provider->IDxDiagProvider_iface, riid, ppobj);
 }
 
 static void free_property_information(IDxDiagContainerImpl_Property *prop)
@@ -577,6 +598,7 @@ static HRESULT build_systeminfo_tree(IDxDiagContainerImpl_Container *node)
     OSVERSIONINFOW info;
     DWORD count, usedpage_mb, availpage_mb;
     WCHAR buffer[MAX_PATH], computer_name[MAX_COMPUTERNAME_LENGTH + 1], print_buf[200], localized_pagefile_fmt[200];
+    DWORD_PTR args[2];
 
     hr = add_ui4_property(node, dwDirectXVersionMajor, 9);
     if (FAILED(hr))
@@ -657,7 +679,11 @@ static HRESULT build_systeminfo_tree(IDxDiagContainerImpl_Container *node)
     usedpage_mb = (DWORD)((msex.ullTotalPageFile - msex.ullAvailPageFile) / (1024 * 1024));
     availpage_mb = (DWORD)(msex.ullAvailPageFile / (1024 * 1024));
     LoadStringW(dxdiagn_instance, IDS_PAGE_FILE_FORMAT, localized_pagefile_fmt, sizeof(localized_pagefile_fmt)/sizeof(WCHAR));
-    snprintfW(print_buf, sizeof(print_buf)/sizeof(WCHAR), localized_pagefile_fmt, usedpage_mb, availpage_mb);
+    args[0] = usedpage_mb;
+    args[1] = availpage_mb;
+    FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ARGUMENT_ARRAY,
+                   localized_pagefile_fmt, 0, 0, print_buf,
+                   sizeof(print_buf)/sizeof(*print_buf), (__ms_va_list*)args);
 
     hr = add_bstr_property(node, szPageFileLocalized, print_buf);
     if (FAILED(hr))
@@ -1297,6 +1323,7 @@ static HRESULT fill_filter_data_information(IDxDiagContainerImpl_Container *subc
     HRESULT hr;
     IFilterMapper2 *pFileMapper = NULL;
     IAMFilterData *pFilterData = NULL;
+    BYTE *ppRF = NULL;
     REGFILTER2 *pRF = NULL;
     WCHAR bufferW[10];
     ULONG j;
@@ -1312,9 +1339,10 @@ static HRESULT fill_filter_data_information(IDxDiagContainerImpl_Container *subc
     if (FAILED(hr))
         goto cleanup;
 
-    hr = IAMFilterData_ParseFilterData(pFilterData, pData, cb, (BYTE **)&pRF);
+    hr = IAMFilterData_ParseFilterData(pFilterData, pData, cb, (BYTE **)&ppRF);
     if (FAILED(hr))
         goto cleanup;
+    pRF = ((REGFILTER2**)ppRF)[0];
 
     snprintfW(bufferW, sizeof(bufferW)/sizeof(bufferW[0]), szVersionFormat, pRF->dwVersion);
     hr = add_bstr_property(subcont, szVersionW, bufferW);
@@ -1323,16 +1351,16 @@ static HRESULT fill_filter_data_information(IDxDiagContainerImpl_Container *subc
 
     if (pRF->dwVersion == 1)
     {
-        for (j = 0; j < pRF->u.s.cPins; j++)
-            if (pRF->u.s.rgPins[j].bOutput)
+        for (j = 0; j < pRF->u.s1.cPins; j++)
+            if (pRF->u.s1.rgPins[j].bOutput)
                 dwNOutputs++;
             else
                 dwNInputs++;
     }
     else if (pRF->dwVersion == 2)
     {
-        for (j = 0; j < pRF->u.s1.cPins2; j++)
-            if (pRF->u.s1.rgPins2[j].dwFlags & REG_PINFLAG_B_OUTPUT)
+        for (j = 0; j < pRF->u.s2.cPins2; j++)
+            if (pRF->u.s2.rgPins2[j].dwFlags & REG_PINFLAG_B_OUTPUT)
                 dwNOutputs++;
             else
                 dwNInputs++;
