@@ -189,9 +189,10 @@ static HRESULT STDMETHODCALLTYPE dxgi_device_CreateSurface(IWineDXGIDevice *ifac
         struct wined3d_surface *wined3d_surface;
         IUnknown *parent;
 
-        hr = device_parent->ops->create_surface(device_parent, NULL, desc->Width, desc->Height,
-                wined3dformat_from_dxgi_format(desc->Format), usage, WINED3D_POOL_DEFAULT, 0,
-                WINED3D_CUBEMAP_FACE_POSITIVE_X, &wined3d_surface);
+        hr = device_parent->ops->create_swapchain_surface(device_parent, NULL,
+                desc->Width, desc->Height, wined3dformat_from_dxgi_format(desc->Format), usage,
+                desc->SampleDesc.Count > 1 ? desc->SampleDesc.Count : WINED3D_MULTISAMPLE_NONE,
+                desc->SampleDesc.Quality, &wined3d_surface);
         if (FAILED(hr))
         {
             ERR("CreateSurface failed, returning %#x\n", hr);
@@ -284,7 +285,7 @@ static HRESULT STDMETHODCALLTYPE dxgi_device_create_surface(IWineDXGIDevice *ifa
     }
 
     TRACE("Created IDXGISurface %p\n", object);
-    *surface = outer ? (void *)&object->inner_unknown_vtbl : object;
+    *surface = outer ? &object->IUnknown_iface : (IUnknown *)&object->IDXGISurface_iface;
 
     return S_OK;
 }
