@@ -146,7 +146,7 @@ static VOID *assembly_data_offset(ASSEMBLY *assembly, ULONG offset)
 #define MAX_TABLES_3BIT_ENCODE 8191
 #define MAX_TABLES_5BIT_ENCODE 2047
 
-static inline ULONG get_table_size(ASSEMBLY *assembly, DWORD index)
+static inline ULONG get_table_size(const ASSEMBLY *assembly, DWORD index)
 {
     DWORD size;
     INT tables;
@@ -567,7 +567,7 @@ static HRESULT parse_metadata_header(ASSEMBLY *assembly, DWORD *hdrsz)
     size = FIELD_OFFSET(METADATAHDR, Version);
     memcpy(assembly->metadatahdr, metadatahdr, size);
 
-    /* we don't care about the version string */
+    assembly->metadatahdr->Version = (LPSTR)&metadatahdr->Version;
 
     ofs = FIELD_OFFSET(METADATAHDR, Flags);
     ptr += FIELD_OFFSET(METADATAHDR, Version) + metadatahdr->VersionLength + 1;
@@ -631,14 +631,18 @@ static HRESULT parse_pe_header(ASSEMBLY *assembly)
     if (!assembly->nthdr)
         return E_FAIL;
 
-    if (assembly->nthdr->FileHeader.Machine == IMAGE_FILE_MACHINE_AMD64)
+    if (assembly->nthdr->OptionalHeader.Magic == IMAGE_NT_OPTIONAL_HDR64_MAGIC)
     {
         IMAGE_OPTIONAL_HEADER64 *opthdr =
                 (IMAGE_OPTIONAL_HEADER64 *)&assembly->nthdr->OptionalHeader;
         datadirs = opthdr->DataDirectory;
     }
     else
-        datadirs = assembly->nthdr->OptionalHeader.DataDirectory;
+    {
+        IMAGE_OPTIONAL_HEADER32 *opthdr =
+                (IMAGE_OPTIONAL_HEADER32 *)&assembly->nthdr->OptionalHeader;
+        datadirs = opthdr->DataDirectory;
+    }
 
     if (!datadirs)
         return E_FAIL;
@@ -727,11 +731,11 @@ HRESULT assembly_release(ASSEMBLY *assembly)
     return S_OK;
 }
 
-static LPWSTR assembly_dup_str(ASSEMBLY *assembly, DWORD index)
+static LPWSTR assembly_dup_str(const ASSEMBLY *assembly, DWORD index)
 {
     int len;
     LPWSTR cpy;
-    LPSTR str = (LPSTR)&assembly->strings[index];
+    LPCSTR str = (LPCSTR)&assembly->strings[index];
 
     len = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
 
@@ -757,9 +761,9 @@ HRESULT assembly_get_name(ASSEMBLY *assembly, LPWSTR *name)
 
     ptr += FIELD_OFFSET(ASSEMBLYTABLE, PublicKey) + assembly->blobsz;
     if (assembly->stringsz == sizeof(DWORD))
-        stridx = *((DWORD *)ptr);
+        stridx = *(DWORD *)ptr;
     else
-        stridx = *((WORD *)ptr);
+        stridx = *(WORD *)ptr;
 
     *name = assembly_dup_str(assembly, stridx);
     if (!*name)
@@ -768,7 +772,7 @@ HRESULT assembly_get_name(ASSEMBLY *assembly, LPWSTR *name)
     return S_OK;
 }
 
-HRESULT assembly_get_path(ASSEMBLY *assembly, LPWSTR *path)
+HRESULT assembly_get_path(const ASSEMBLY *assembly, LPWSTR *path)
 {
     LPWSTR cpy = HeapAlloc(GetProcessHeap(), 0, (strlenW(assembly->path) + 1) * sizeof(WCHAR));
     *path = cpy;
@@ -807,29 +811,36 @@ HRESULT assembly_get_version(ASSEMBLY *assembly, LPWSTR *version)
     return S_OK;
 }
 
-HRESULT assembly_get_architecture(ASSEMBLY *assembly, DWORD fixme)
+PEKIND assembly_get_architecture(ASSEMBLY *assembly)
 {
-    /* FIXME */
-    return S_OK;
+    if ((assembly->corhdr->MajorRuntimeVersion == 2) && (assembly->corhdr->MinorRuntimeVersion == 0))
+        return peNone; /* .NET 1.x assembly */
+
+    if (assembly->nthdr->OptionalHeader.Magic == IMAGE_NT_OPTIONAL_HDR64_MAGIC)
+        return peAMD64; /* AMD64/IA64 assembly */
+
+    if ((assembly->corhdr->Flags & COMIMAGE_FLAGS_ILONLY) && !(assembly->corhdr->Flags & COMIMAGE_FLAGS_32BITREQUIRED))
+        return peMSIL; /* MSIL assembly */
+
+    return peI386; /* x86 assembly */
 }
 
-static BYTE *assembly_get_blob(ASSEMBLY *assembly, WORD index, ULONG *size)
+static BYTE *assembly_get_blob(ASSEMBLY *assembly, DWORD index, ULONG *size)
 {
     return GetData(&assembly->blobs[index], size);
 }
 
 HRESULT assembly_get_pubkey_token(ASSEMBLY *assembly, LPWSTR *token)
 {
-    ASSEMBLYTABLE *asmtbl;
     ULONG i, size;
     LONG offset;
-    BYTE *hashdata;
+    BYTE *hashdata, *pubkey, *ptr;
     HCRYPTPROV crypt;
     HCRYPTHASH hash;
-    BYTE *pubkey;
     BYTE tokbytes[BYTES_PER_TOKEN];
     HRESULT hr = E_FAIL;
     LPWSTR tok;
+    DWORD idx;
 
     *token = NULL;
 
@@ -837,11 +848,17 @@ HRESULT assembly_get_pubkey_token(ASSEMBLY *assembly, LPWSTR *token)
     if (offset == -1)
         return E_FAIL;
 
-    asmtbl = assembly_data_offset(assembly, offset);
-    if (!asmtbl)
+    ptr = assembly_data_offset(assembly, offset);
+    if (!ptr)
         return E_FAIL;
 
-    pubkey = assembly_get_blob(assembly, asmtbl->PublicKey, &size);
+    ptr += FIELD_OFFSET(ASSEMBLYTABLE, PublicKey);
+    if (assembly->blobsz == sizeof(DWORD))
+        idx = *(DWORD *)ptr;
+    else
+        idx = *(WORD *)ptr;
+
+    pubkey = assembly_get_blob(assembly, idx, &size);
 
     if (!CryptAcquireContextA(&crypt, NULL, NULL, PROV_RSA_FULL,
                               CRYPT_VERIFYCONTEXT))
@@ -889,3 +906,58 @@ done:
 
     return hr;
 }
+
+HRESULT assembly_get_runtime_version(ASSEMBLY *assembly, LPSTR *version)
+{
+    *version = assembly->metadatahdr->Version;
+    return S_OK;
+}
+
+HRESULT assembly_get_external_files(ASSEMBLY *assembly, LPWSTR **files, DWORD *count)
+{
+    LONG offset;
+    INT i, num_rows;
+    WCHAR **ret;
+    BYTE *ptr;
+    DWORD idx;
+
+    *count = 0;
+
+    offset = assembly->tables[TableFromToken(mdtFile)].offset;
+    if (offset == -1)
+        return S_OK;
+
+    ptr = assembly_data_offset(assembly, offset);
+    if (!ptr)
+        return S_OK;
+
+    num_rows = assembly->tables[TableFromToken(mdtFile)].rows;
+    if (num_rows <= 0)
+        return S_OK;
+
+    ret = HeapAlloc(GetProcessHeap(), 0, num_rows * sizeof(WCHAR *));
+    if (!ret)
+        return E_OUTOFMEMORY;
+
+    for (i = 0; i < num_rows; i++)
+    {
+        ptr += sizeof(DWORD); /* skip Flags field */
+        if (assembly->stringsz == sizeof(DWORD))
+            idx = *(DWORD *)ptr;
+        else
+            idx = *(WORD *)ptr;
+
+        ret[i] = assembly_dup_str(assembly, idx);
+        if (!ret[i])
+        {
+            for (; i >= 0; i--) HeapFree(GetProcessHeap(), 0, ret[i]);
+            HeapFree(GetProcessHeap(), 0, ret);
+            return E_OUTOFMEMORY;
+        }
+        ptr += assembly->stringsz; /* skip Name field */
+        ptr += assembly->blobsz; /* skip Hash field */
+    }
+    *count = num_rows;
+    *files = ret;
+    return S_OK;
+}
