@@ -175,8 +175,6 @@ static HRESULT GAMEUX_updateStatisticsFile(struct GAMEUX_STATS *stats)
             hr = E_OUTOFMEMORY;
     }
 
-    if(SUCCEEDED(hr))
-
     if(SUCCEEDED(hr))
         for(i=0; i<MAX_CATEGORIES; ++i)
         {
@@ -390,17 +388,18 @@ static HRESULT GAMEUX_getAppIdFromGDFPath(
         /* game is registered, let's read it's application id from registry */
         hr = GAMEUX_buildGameRegistryPath(installScope, &instanceId, &lpRegistryPath);
 
-    if(SUCCEEDED(hr))
+    if(SUCCEEDED(hr)) {
         hr = HRESULT_FROM_WIN32(RegOpenKeyExW(HKEY_LOCAL_MACHINE,
                 lpRegistryPath, 0, KEY_READ | KEY_WOW64_64KEY, &hKey));
-
-    if(SUCCEEDED(hr))
-        hr = HRESULT_FROM_WIN32(RegGetValueW(hKey,
-                NULL, sApplicationId, RRF_RT_REG_SZ,
-                NULL, lpApplicationId, &dwLength));
+        if(SUCCEEDED(hr)) {
+            hr = HRESULT_FROM_WIN32(RegGetValueW(hKey,
+                    NULL, sApplicationId, RRF_RT_REG_SZ,
+                    NULL, lpApplicationId, &dwLength));
+            RegCloseKey(hKey);
+        }
+    }
 
     HeapFree(GetProcessHeap(), 0, lpRegistryPath);
-    RegCloseKey(hKey);
 
     TRACE("found app id: %s, return: %#x\n", debugstr_w(lpApplicationId), hr);
     return hr;
@@ -931,7 +930,7 @@ static HRESULT WINAPI GameStatisticsImpl_SetStatistic(
                   value, dwValueLen+1);
     }
     else
-        /* Windows allows to pass NULL as value */
+        /* Windows allows passing NULL as value */
         This->stats.categories[categoryIndex].stats[statIndex].sValue[0] = 0;
 
     return hr;
@@ -946,7 +945,7 @@ static HRESULT WINAPI GameStatisticsImpl_Save(
 
     TRACE("(%p, %d)\n", This, trackChanges);
 
-    if(trackChanges == TRUE)
+    if(trackChanges)
         FIXME("tracking changes not yet implemented\n");
 
     hr = GAMEUX_updateStatisticsFile(&This->stats);
@@ -1083,7 +1082,7 @@ static HRESULT STDMETHODCALLTYPE GameStatisticsMgrImpl_GetGameStatistics(
 {
     HRESULT hr;
     WCHAR lpApplicationId[49];
-    GameStatisticsImpl *statisticsImpl;
+    GameStatisticsImpl *statisticsImpl = NULL;
     IGameStatistics *output_iface;
 
     TRACE("(%p, %s, 0x%x, %p, %p)\n", iface, debugstr_w(GDFBinaryPath), openType, pOpenResult, ppiStats);
