@@ -21,7 +21,7 @@
 #include "config.h"
 
 #include <stdarg.h>
-
+#include <limits.h>
 #include <math.h>
 #ifdef HAVE_FLOAT_H
 #include <float.h>
@@ -47,73 +47,16 @@ static inline void swap_ints( int *i, int *j )
     *j = tmp;
 }
 
-static BOOL get_vis_rectangles( DC *dc_dst, struct bitblt_coords *dst,
-                                DC *dc_src, struct bitblt_coords *src )
+BOOL intersect_vis_rectangles( struct bitblt_coords *dst, struct bitblt_coords *src )
 {
-    RECT rect, clip;
-
-    /* get the destination visible rectangle */
-
-    rect.left   = dst->log_x;
-    rect.top    = dst->log_y;
-    rect.right  = dst->log_x + dst->log_width;
-    rect.bottom = dst->log_y + dst->log_height;
-    LPtoDP( dc_dst->hSelf, (POINT *)&rect, 2 );
-    dst->x      = rect.left;
-    dst->y      = rect.top;
-    dst->width  = rect.right - rect.left;
-    dst->height = rect.bottom - rect.top;
-    if (dst->layout & LAYOUT_RTL && dst->layout & LAYOUT_BITMAPORIENTATIONPRESERVED)
-    {
-        swap_ints( &rect.left, &rect.right );
-        dst->x = rect.left;
-        dst->width = rect.right - rect.left;
-    }
-    if (rect.left > rect.right) { swap_ints( &rect.left, &rect.right ); rect.left++; rect.right++; }
-    if (rect.top > rect.bottom) { swap_ints( &rect.top, &rect.bottom ); rect.top++; rect.bottom++; }
-
-    if (get_clip_box( dc_dst, &clip ))
-        intersect_rect( &dst->visrect, &rect, &clip );
-    else
-        dst->visrect = rect;
-
-    /* get the source visible rectangle */
-
-    if (!src) return !is_rect_empty( &dst->visrect );
-
-    rect.left   = src->log_x;
-    rect.top    = src->log_y;
-    rect.right  = src->log_x + src->log_width;
-    rect.bottom = src->log_y + src->log_height;
-    LPtoDP( dc_src->hSelf, (POINT *)&rect, 2 );
-    src->x      = rect.left;
-    src->y      = rect.top;
-    src->width  = rect.right - rect.left;
-    src->height = rect.bottom - rect.top;
-    if (src->layout & LAYOUT_RTL && src->layout & LAYOUT_BITMAPORIENTATIONPRESERVED)
-    {
-        swap_ints( &rect.left, &rect.right );
-        src->x = rect.left;
-        src->width = rect.right - rect.left;
-    }
-    if (rect.left > rect.right) { swap_ints( &rect.left, &rect.right ); rect.left++; rect.right++; }
-    if (rect.top > rect.bottom) { swap_ints( &rect.top, &rect.bottom ); rect.top++; rect.bottom++; }
-
-    /* source is not clipped */
-    if (dc_src->header.type == OBJ_MEMDC)
-        intersect_rect( &src->visrect, &rect, &dc_src->vis_rect );
-    else
-        src->visrect = rect;  /* FIXME: clip to device size */
-
-    if (is_rect_empty( &src->visrect )) return FALSE;
-    if (is_rect_empty( &dst->visrect )) return FALSE;
+    RECT rect;
 
     /* intersect the rectangles */
 
     if ((src->width == dst->width) && (src->height == dst->height)) /* no stretching */
     {
         offset_rect( &src->visrect, dst->x - src->x, dst->y - src->y );
-        intersect_rect( &rect, &src->visrect, &dst->visrect );
+        if (!intersect_rect( &rect, &src->visrect, &dst->visrect )) return FALSE;
         src->visrect = dst->visrect = rect;
         offset_rect( &src->visrect, src->x - dst->x, src->y - dst->y );
     }
@@ -158,22 +101,75 @@ static BOOL get_vis_rectangles( DC *dc_dst, struct bitblt_coords *dst,
     return TRUE;
 }
 
+static BOOL get_vis_rectangles( DC *dc_dst, struct bitblt_coords *dst,
+                                DC *dc_src, struct bitblt_coords *src )
+{
+    RECT rect;
+
+    /* get the destination visible rectangle */
+
+    rect.left   = dst->log_x;
+    rect.top    = dst->log_y;
+    rect.right  = dst->log_x + dst->log_width;
+    rect.bottom = dst->log_y + dst->log_height;
+    LPtoDP( dc_dst->hSelf, (POINT *)&rect, 2 );
+    dst->x      = rect.left;
+    dst->y      = rect.top;
+    dst->width  = rect.right - rect.left;
+    dst->height = rect.bottom - rect.top;
+    if (dst->layout & LAYOUT_RTL && dst->layout & LAYOUT_BITMAPORIENTATIONPRESERVED)
+    {
+        dst->x += dst->width;
+        dst->width = -dst->width;
+    }
+    get_bounding_rect( &rect, dst->x, dst->y, dst->width, dst->height );
+
+    clip_visrect( dc_dst, &dst->visrect, &rect );
+
+    /* get the source visible rectangle */
+
+    if (!src) return !is_rect_empty( &dst->visrect );
+
+    rect.left   = src->log_x;
+    rect.top    = src->log_y;
+    rect.right  = src->log_x + src->log_width;
+    rect.bottom = src->log_y + src->log_height;
+    LPtoDP( dc_src->hSelf, (POINT *)&rect, 2 );
+    src->x      = rect.left;
+    src->y      = rect.top;
+    src->width  = rect.right - rect.left;
+    src->height = rect.bottom - rect.top;
+    if (src->layout & LAYOUT_RTL && src->layout & LAYOUT_BITMAPORIENTATIONPRESERVED)
+    {
+        src->x += src->width;
+        src->width = -src->width;
+    }
+    get_bounding_rect( &rect, src->x, src->y, src->width, src->height );
+
+    if (!clip_device_rect( dc_src, &src->visrect, &rect )) return FALSE;
+    if (is_rect_empty( &dst->visrect )) return FALSE;
+
+    return intersect_vis_rectangles( dst, src );
+}
+
 void free_heap_bits( struct gdi_image_bits *bits )
 {
     HeapFree( GetProcessHeap(), 0, bits->ptr );
 }
 
-static DWORD convert_bits( const BITMAPINFO *src_info, struct bitblt_coords *src,
-                           BITMAPINFO *dst_info, struct gdi_image_bits *bits )
+DWORD convert_bits( const BITMAPINFO *src_info, struct bitblt_coords *src,
+                    BITMAPINFO *dst_info, struct gdi_image_bits *bits, BOOL add_alpha )
 {
     void *ptr;
     DWORD err;
 
     dst_info->bmiHeader.biWidth = src->visrect.right - src->visrect.left;
-    if (!(ptr = HeapAlloc( GetProcessHeap(), 0, get_dib_image_size( dst_info ))))
+    dst_info->bmiHeader.biSizeImage = get_dib_image_size( dst_info );
+
+    if (!(ptr = HeapAlloc( GetProcessHeap(), 0, dst_info->bmiHeader.biSizeImage )))
         return ERROR_OUTOFMEMORY;
 
-    err = convert_bitmapinfo( src_info, bits->ptr, src, dst_info, ptr );
+    err = convert_bitmapinfo( src_info, bits->ptr, src, dst_info, ptr, add_alpha );
     if (bits->free) bits->free( bits );
     bits->ptr = ptr;
     bits->is_copy = TRUE;
@@ -181,17 +177,19 @@ static DWORD convert_bits( const BITMAPINFO *src_info, struct bitblt_coords *src
     return err;
 }
 
-static DWORD stretch_bits( const BITMAPINFO *src_info, struct bitblt_coords *src,
-                           BITMAPINFO *dst_info, struct bitblt_coords *dst,
-                           struct gdi_image_bits *bits, int mode )
+DWORD stretch_bits( const BITMAPINFO *src_info, struct bitblt_coords *src,
+                    BITMAPINFO *dst_info, struct bitblt_coords *dst,
+                    struct gdi_image_bits *bits, int mode )
 {
     void *ptr;
     DWORD err;
 
     dst_info->bmiHeader.biWidth = dst->visrect.right - dst->visrect.left;
     dst_info->bmiHeader.biHeight = dst->visrect.bottom - dst->visrect.top;
+    dst_info->bmiHeader.biSizeImage = get_dib_image_size( dst_info );
+
     if (src_info->bmiHeader.biHeight < 0) dst_info->bmiHeader.biHeight = -dst_info->bmiHeader.biHeight;
-    if (!(ptr = HeapAlloc( GetProcessHeap(), 0, get_dib_image_size( dst_info ))))
+    if (!(ptr = HeapAlloc( GetProcessHeap(), 0, dst_info->bmiHeader.biSizeImage )))
         return ERROR_OUTOFMEMORY;
 
     err = stretch_bitmapinfo( src_info, bits->ptr, src, dst_info, ptr, dst, mode );
@@ -202,6 +200,45 @@ static DWORD stretch_bits( const BITMAPINFO *src_info, struct bitblt_coords *src
     return err;
 }
 
+static DWORD blend_bits( const BITMAPINFO *src_info, const struct gdi_image_bits *src_bits,
+                         struct bitblt_coords *src, BITMAPINFO *dst_info,
+                         struct gdi_image_bits *dst_bits, struct bitblt_coords *dst, BLENDFUNCTION blend )
+{
+    if (!dst_bits->is_copy)
+    {
+        int size = dst_info->bmiHeader.biSizeImage;
+        void *ptr = HeapAlloc( GetProcessHeap(), 0, size );
+        if (!ptr) return ERROR_OUTOFMEMORY;
+        memcpy( ptr, dst_bits->ptr, size );
+        if (dst_bits->free) dst_bits->free( dst_bits );
+        dst_bits->ptr = ptr;
+        dst_bits->is_copy = TRUE;
+        dst_bits->free = free_heap_bits;
+    }
+    return blend_bitmapinfo( src_info, src_bits->ptr, src, dst_info, dst_bits->ptr, dst, blend );
+}
+
+/* helper to retrieve either both colors or only the background color for monochrome blits */
+static void get_mono_dc_colors( HDC hdc, BITMAPINFO *info, int count )
+{
+    COLORREF color = GetBkColor( hdc );
+
+    info->bmiColors[count - 1].rgbRed      = GetRValue( color );
+    info->bmiColors[count - 1].rgbGreen    = GetGValue( color );
+    info->bmiColors[count - 1].rgbBlue     = GetBValue( color );
+    info->bmiColors[count - 1].rgbReserved = 0;
+
+    if (count > 1)
+    {
+        color = GetTextColor( hdc );
+        info->bmiColors[0].rgbRed      = GetRValue( color );
+        info->bmiColors[0].rgbGreen    = GetGValue( color );
+        info->bmiColors[0].rgbBlue     = GetBValue( color );
+        info->bmiColors[0].rgbReserved = 0;
+    }
+    info->bmiHeader.biClrUsed = count;
+}
+
 /***********************************************************************
  *           null driver fallback implementations
  */
@@ -219,60 +256,52 @@ BOOL nulldrv_StretchBlt( PHYSDEV dst_dev, struct bitblt_coords *dst,
 
     if (!(dc_src = get_dc_ptr( src_dev->hdc ))) return FALSE;
     src_dev = GET_DC_PHYSDEV( dc_src, pGetImage );
-    err = src_dev->funcs->pGetImage( src_dev, 0, src_info, &bits, src );
-    release_dc_ptr( dc_src );
-    if (err) return FALSE;
+    if (src_dev->funcs->pGetImage( src_dev, src_info, &bits, src ))
+    {
+        release_dc_ptr( dc_src );
+        return FALSE;
+    }
 
     dst_dev = GET_DC_PHYSDEV( dc_dst, pPutImage );
-    memcpy( dst_info, src_info, FIELD_OFFSET( BITMAPINFO, bmiColors[256] ));
-    err = dst_dev->funcs->pPutImage( dst_dev, 0, 0, dst_info, &bits, src, dst, rop );
+    copy_bitmapinfo( dst_info, src_info );
+    err = dst_dev->funcs->pPutImage( dst_dev, 0, dst_info, &bits, src, dst, rop );
     if (err == ERROR_BAD_FORMAT)
     {
+        DWORD dst_colors = dst_info->bmiHeader.biClrUsed;
+
         /* 1-bpp source without a color table uses the destination DC colors */
         if (src_info->bmiHeader.biBitCount == 1 && !src_info->bmiHeader.biClrUsed)
+            get_mono_dc_colors( dst_dev->hdc, src_info, 2 );
+
+        if (dst_info->bmiHeader.biBitCount == 1 && !dst_colors)
         {
-            COLORREF color = GetTextColor( dst_dev->hdc );
-            src_info->bmiColors[0].rgbRed      = GetRValue( color );
-            src_info->bmiColors[0].rgbGreen    = GetGValue( color );
-            src_info->bmiColors[0].rgbBlue     = GetBValue( color );
-            src_info->bmiColors[0].rgbReserved = 0;
-            color = GetBkColor( dst_dev->hdc );
-            src_info->bmiColors[1].rgbRed      = GetRValue( color );
-            src_info->bmiColors[1].rgbGreen    = GetGValue( color );
-            src_info->bmiColors[1].rgbBlue     = GetBValue( color );
-            src_info->bmiColors[1].rgbReserved = 0;
-            src_info->bmiHeader.biClrUsed = 2;
+            /* 1-bpp destination without a color table requires a fake 1-entry table
+             * that contains only the background color; except with a 1-bpp source,
+             * in which case it uses the source colors */
+            if (src_info->bmiHeader.biBitCount > 1)
+                get_mono_dc_colors( src_dev->hdc, dst_info, 1 );
+            else
+                get_mono_dc_colors( src_dev->hdc, dst_info, 2 );
         }
 
-        /* 1-bpp destination without a color table requires a fake 1-entry table
-         * that contains only the background color */
-        if (dst_info->bmiHeader.biBitCount == 1 && !dst_info->bmiHeader.biClrUsed)
+        if (!(err = convert_bits( src_info, src, dst_info, &bits, FALSE )))
         {
-            COLORREF color = GetBkColor( src_dev->hdc );
-            dst_info->bmiColors[0].rgbRed      = GetRValue( color );
-            dst_info->bmiColors[0].rgbGreen    = GetGValue( color );
-            dst_info->bmiColors[0].rgbBlue     = GetBValue( color );
-            dst_info->bmiColors[0].rgbReserved = 0;
-            dst_info->bmiHeader.biClrUsed = 1;
-        }
-
-        if (!(err = convert_bits( src_info, src, dst_info, &bits )))
-        {
-            /* get rid of the fake 1-bpp table */
-            if (dst_info->bmiHeader.biClrUsed == 1) dst_info->bmiHeader.biClrUsed = 0;
-            err = dst_dev->funcs->pPutImage( dst_dev, 0, 0, dst_info, &bits, src, dst, rop );
+            /* get rid of the fake destination table */
+            dst_info->bmiHeader.biClrUsed = dst_colors;
+            err = dst_dev->funcs->pPutImage( dst_dev, 0, dst_info, &bits, src, dst, rop );
         }
     }
 
     if (err == ERROR_TRANSFORM_NOT_SUPPORTED &&
         ((src->width != dst->width) || (src->height != dst->height)))
     {
-        memcpy( src_info, dst_info, FIELD_OFFSET( BITMAPINFO, bmiColors[256] ));
+        copy_bitmapinfo( src_info, dst_info );
         err = stretch_bits( src_info, src, dst_info, dst, &bits, GetStretchBltMode( dst_dev->hdc ));
-        if (!err) err = dst_dev->funcs->pPutImage( dst_dev, 0, 0, dst_info, &bits, src, dst, rop );
+        if (!err) err = dst_dev->funcs->pPutImage( dst_dev, 0, dst_info, &bits, src, dst, rop );
     }
 
     if (bits.free) bits.free( &bits );
+    release_dc_ptr( dc_src );
     return !err;
 }
 
@@ -290,46 +319,193 @@ BOOL nulldrv_AlphaBlend( PHYSDEV dst_dev, struct bitblt_coords *dst,
 
     if (!(dc_src = get_dc_ptr( src_dev->hdc ))) return FALSE;
     src_dev = GET_DC_PHYSDEV( dc_src, pGetImage );
-    err = src_dev->funcs->pGetImage( src_dev, 0, src_info, &bits, src );
-    release_dc_ptr( dc_src );
-    if (err) return FALSE;
+    err = src_dev->funcs->pGetImage( src_dev, src_info, &bits, src );
+    if (err) goto done;
 
     dst_dev = GET_DC_PHYSDEV( dc_dst, pBlendImage );
-    memcpy( dst_info, src_info, FIELD_OFFSET( BITMAPINFO, bmiColors[256] ));
+    copy_bitmapinfo( dst_info, src_info );
     err = dst_dev->funcs->pBlendImage( dst_dev, dst_info, &bits, src, dst, func );
     if (err == ERROR_BAD_FORMAT)
     {
-        /* 1-bpp source without a color table uses black & white */
-        if (src_info->bmiHeader.biBitCount == 1 && !src_info->bmiHeader.biClrUsed)
-        {
-            src_info->bmiColors[0].rgbRed      = 0;
-            src_info->bmiColors[0].rgbGreen    = 0;
-            src_info->bmiColors[0].rgbBlue     = 0;
-            src_info->bmiColors[0].rgbReserved = 0;
-            src_info->bmiColors[1].rgbRed      = 0xff;
-            src_info->bmiColors[1].rgbGreen    = 0xff;
-            src_info->bmiColors[1].rgbBlue     = 0xff;
-            src_info->bmiColors[1].rgbReserved = 0;
-            src_info->bmiHeader.biClrUsed = 2;
-        }
-
-        err = convert_bits( src_info, src, dst_info, &bits );
+        err = convert_bits( src_info, src, dst_info, &bits, TRUE );
         if (!err) err = dst_dev->funcs->pBlendImage( dst_dev, dst_info, &bits, src, dst, func );
     }
 
     if (err == ERROR_TRANSFORM_NOT_SUPPORTED &&
         ((src->width != dst->width) || (src->height != dst->height)))
     {
-        memcpy( src_info, dst_info, FIELD_OFFSET( BITMAPINFO, bmiColors[256] ));
+        copy_bitmapinfo( src_info, dst_info );
         err = stretch_bits( src_info, src, dst_info, dst, &bits, COLORONCOLOR );
         if (!err) err = dst_dev->funcs->pBlendImage( dst_dev, dst_info, &bits, src, dst, func );
     }
 
     if (bits.free) bits.free( &bits );
+done:
+    release_dc_ptr( dc_src );
+    if (err) SetLastError( err );
     return !err;
 }
 
 
+DWORD nulldrv_BlendImage( PHYSDEV dev, BITMAPINFO *info, const struct gdi_image_bits *bits,
+                          struct bitblt_coords *src, struct bitblt_coords *dst, BLENDFUNCTION blend )
+{
+    char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
+    BITMAPINFO *dst_info = (BITMAPINFO *)buffer;
+    struct gdi_image_bits dst_bits;
+    struct bitblt_coords orig_dst;
+    DC *dc = get_nulldrv_dc( dev );
+    DWORD err;
+
+    if (info->bmiHeader.biPlanes != 1) goto update_format;
+    if (info->bmiHeader.biBitCount != 32) goto update_format;
+    if (info->bmiHeader.biCompression == BI_BITFIELDS)
+    {
+        DWORD *masks = (DWORD *)info->bmiColors;
+        if (masks[0] != 0xff0000 || masks[1] != 0x00ff00 || masks[2] != 0x0000ff)
+            goto update_format;
+    }
+
+    if (!bits) return ERROR_SUCCESS;
+    if ((src->width != dst->width) || (src->height != dst->height)) return ERROR_TRANSFORM_NOT_SUPPORTED;
+
+    dev = GET_DC_PHYSDEV( dc, pGetImage );
+    orig_dst = *dst;
+    err = dev->funcs->pGetImage( dev, dst_info, &dst_bits, dst );
+    if (err) return err;
+
+    dev = GET_DC_PHYSDEV( dc, pPutImage );
+    err = blend_bits( info, bits, src, dst_info, &dst_bits, dst, blend );
+    if (!err) err = dev->funcs->pPutImage( dev, 0, dst_info, &dst_bits, dst, &orig_dst, SRCCOPY );
+
+    if (dst_bits.free) dst_bits.free( &dst_bits );
+    return err;
+
+update_format:
+    if (blend.AlphaFormat & AC_SRC_ALPHA)  /* source alpha requires A8R8G8B8 format */
+        return ERROR_INVALID_PARAMETER;
+
+    info->bmiHeader.biPlanes      = 1;
+    info->bmiHeader.biBitCount    = 32;
+    info->bmiHeader.biCompression = BI_RGB;
+    info->bmiHeader.biClrUsed     = 0;
+    return ERROR_BAD_FORMAT;
+}
+
+BOOL nulldrv_GradientFill( PHYSDEV dev, TRIVERTEX *vert_array, ULONG nvert,
+                           void * grad_array, ULONG ngrad, ULONG mode )
+{
+    DC *dc = get_nulldrv_dc( dev );
+    char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
+    BITMAPINFO *info = (BITMAPINFO *)buffer;
+    struct bitblt_coords src, dst;
+    struct gdi_image_bits bits;
+    unsigned int i;
+    POINT *pts;
+    BOOL ret = FALSE;
+    DWORD err;
+    HRGN rgn;
+
+    if (!(pts = HeapAlloc( GetProcessHeap(), 0, nvert * sizeof(*pts) ))) return FALSE;
+    for (i = 0; i < nvert; i++)
+    {
+        pts[i].x = vert_array[i].x;
+        pts[i].y = vert_array[i].y;
+    }
+    LPtoDP( dev->hdc, pts, nvert );
+
+    /* compute bounding rect of all the rectangles/triangles */
+    reset_bounds( &dst.visrect );
+    for (i = 0; i < ngrad * (mode == GRADIENT_FILL_TRIANGLE ? 3 : 2); i++)
+    {
+        ULONG v = ((ULONG *)grad_array)[i];
+        dst.visrect.left   = min( dst.visrect.left,   pts[v].x );
+        dst.visrect.top    = min( dst.visrect.top,    pts[v].y );
+        dst.visrect.right  = max( dst.visrect.right,  pts[v].x );
+        dst.visrect.bottom = max( dst.visrect.bottom, pts[v].y );
+    }
+
+    dst.x = dst.visrect.left;
+    dst.y = dst.visrect.top;
+    dst.width = dst.visrect.right - dst.visrect.left;
+    dst.height = dst.visrect.bottom - dst.visrect.top;
+    if (!clip_visrect( dc, &dst.visrect, &dst.visrect )) goto done;
+
+    /* query the bitmap format */
+    info->bmiHeader.biSize          = sizeof(info->bmiHeader);
+    info->bmiHeader.biPlanes        = 1;
+    info->bmiHeader.biBitCount      = 0;
+    info->bmiHeader.biCompression   = BI_RGB;
+    info->bmiHeader.biXPelsPerMeter = 0;
+    info->bmiHeader.biYPelsPerMeter = 0;
+    info->bmiHeader.biClrUsed       = 0;
+    info->bmiHeader.biClrImportant  = 0;
+    info->bmiHeader.biWidth         = dst.visrect.right - dst.visrect.left;
+    info->bmiHeader.biHeight        = dst.visrect.bottom - dst.visrect.top;
+    info->bmiHeader.biSizeImage     = 0;
+    dev = GET_DC_PHYSDEV( dc, pPutImage );
+    err = dev->funcs->pPutImage( dev, 0, info, NULL, NULL, NULL, 0 );
+    if (err && err != ERROR_BAD_FORMAT) goto done;
+
+    info->bmiHeader.biSizeImage = get_dib_image_size( info );
+    if (!(bits.ptr = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, info->bmiHeader.biSizeImage )))
+        goto done;
+    bits.is_copy = TRUE;
+    bits.free = free_heap_bits;
+
+    /* make src and points relative to the bitmap */
+    src = dst;
+    src.x -= dst.visrect.left;
+    src.y -= dst.visrect.top;
+    offset_rect( &src.visrect, -dst.visrect.left, -dst.visrect.top );
+    for (i = 0; i < nvert; i++)
+    {
+        pts[i].x -= dst.visrect.left;
+        pts[i].y -= dst.visrect.top;
+    }
+
+    rgn = CreateRectRgn( 0, 0, 0, 0 );
+    gradient_bitmapinfo( info, bits.ptr, vert_array, nvert, grad_array, ngrad, mode, pts, rgn );
+    OffsetRgn( rgn, dst.visrect.left, dst.visrect.top );
+    ret = !dev->funcs->pPutImage( dev, rgn, info, &bits, &src, &dst, SRCCOPY );
+
+    if (bits.free) bits.free( &bits );
+    DeleteObject( rgn );
+
+done:
+    HeapFree( GetProcessHeap(), 0, pts );
+    return ret;
+}
+
+COLORREF nulldrv_GetPixel( PHYSDEV dev, INT x, INT y )
+{
+    DC *dc = get_nulldrv_dc( dev );
+    char buffer[FIELD_OFFSET( BITMAPINFO, bmiColors[256] )];
+    BITMAPINFO *info = (BITMAPINFO *)buffer;
+    struct bitblt_coords src;
+    struct gdi_image_bits bits;
+    COLORREF ret;
+
+    src.visrect.left = x;
+    src.visrect.top  = y;
+    LPtoDP( dev->hdc, (POINT *)&src.visrect, 1 );
+    src.visrect.right  = src.visrect.left + 1;
+    src.visrect.bottom = src.visrect.top + 1;
+    src.x = src.visrect.left;
+    src.y = src.visrect.top;
+    src.width = src.height = 1;
+
+    if (!clip_visrect( dc, &src.visrect, &src.visrect )) return CLR_INVALID;
+
+    dev = GET_DC_PHYSDEV( dc, pGetImage );
+    if (dev->funcs->pGetImage( dev, info, &bits, &src )) return CLR_INVALID;
+
+    ret = get_pixel_bitmapinfo( info, bits.ptr, &src );
+    if (bits.free) bits.free( &bits );
+    return ret;
+}
+
+
 /***********************************************************************
  *           PatBlt    (GDI32.@)
  */
@@ -762,17 +938,27 @@ BOOL WINAPI GdiAlphaBlend(HDC hdcDst, int xDst, int yDst, int widthDst, int heig
               blendFunction.SourceConstantAlpha, blendFunction.AlphaFormat );
 
         if (src.x < 0 || src.y < 0 || src.width < 0 || src.height < 0 ||
-            (dcSrc->header.type == OBJ_MEMDC &&
-             (src.width > dcSrc->vis_rect.right - dcSrc->vis_rect.left - src.x ||
-              src.height > dcSrc->vis_rect.bottom - dcSrc->vis_rect.top - src.y)))
+            src.log_width < 0 || src.log_height < 0 ||
+            (!is_rect_empty( &dcSrc->device_rect ) &&
+             (src.width > dcSrc->device_rect.right - dcSrc->vis_rect.left - src.x ||
+              src.height > dcSrc->device_rect.bottom - dcSrc->vis_rect.top - src.y)))
         {
             WARN( "Invalid src coords: (%d,%d), size %dx%d\n", src.x, src.y, src.width, src.height );
             SetLastError( ERROR_INVALID_PARAMETER );
             ret = FALSE;
         }
-        else if (dst.width < 0 || dst.height < 0)
+        else if (dst.log_width < 0 || dst.log_height < 0)
         {
-            WARN( "Invalid dst coords: (%d,%d), size %dx%d\n", dst.x, dst.y, dst.width, dst.height );
+            WARN( "Invalid dst coords: (%d,%d), size %dx%d\n",
+                  dst.log_x, dst.log_y, dst.log_width, dst.log_height );
+            SetLastError( ERROR_INVALID_PARAMETER );
+            ret = FALSE;
+        }
+        else if (dcSrc == dcDst && src.x + src.width > dst.x && src.x < dst.x + dst.width &&
+                 src.y + src.height > dst.y && src.y < dst.y + dst.height)
+        {
+            WARN( "Overlapping coords: (%d,%d), %dx%d and (%d,%d), %dx%d\n",
+                  src.x, src.y, src.width, src.height, dst.x, dst.y, dst.width, dst.height );
             SetLastError( ERROR_INVALID_PARAMETER );
             ret = FALSE;
         }
