@@ -114,6 +114,8 @@ DC *alloc_dc_ptr( WORD magic )
     dc->xformVport2World    = dc->xformWorld2Wnd;
     dc->vport2WorldValid    = TRUE;
 
+    reset_bounds( &dc->bounds );
+
     if (!(dc->hSelf = alloc_gdi_handle( &dc->header, magic, &dc_funcs )))
     {
         HeapFree( GetProcessHeap(), 0, dc );
@@ -154,11 +156,10 @@ void free_dc_ptr( DC *dc )
 
     while (dc->physDev != &dc->nulldrv)
     {
-        PHYSDEV physdev = pop_dc_driver( &dc->physDev );
+        PHYSDEV physdev = dc->physDev;
+        dc->physDev = physdev->next;
         physdev->funcs->pDeleteDC( physdev );
-        if (physdev == dc->dibdrv) dc->dibdrv = NULL;
     }
-    if (dc->dibdrv) dc->dibdrv->funcs->pDeleteDC( dc->dibdrv );
     free_gdi_handle( dc->hSelf );
     free_dc_state( dc );
 }
@@ -244,6 +245,8 @@ void DC_InitDC( DC* dc )
     SelectObject( dc->hSelf, dc->hFont );
     update_dc_clipping( dc );
     SetVirtualResolution( dc->hSelf, 0, 0, 0, 0 );
+    physdev = GET_DC_PHYSDEV( dc, pSetBoundsRect );
+    physdev->funcs->pSetBoundsRect( physdev, &dc->bounds, dc->bounds_enabled ? DCB_ENABLE : DCB_DISABLE );
 }
 
 
@@ -336,7 +339,6 @@ INT nulldrv_SaveDC( PHYSDEV dev )
     DC *newdc, *dc = get_nulldrv_dc( dev );
 
     if (!(newdc = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*newdc )))) return 0;
-    newdc->flags            = dc->flags;
     newdc->layout           = dc->layout;
     newdc->hPen             = dc->hPen;
     newdc->hBrush           = dc->hBrush;
@@ -379,7 +381,6 @@ INT nulldrv_SaveDC( PHYSDEV dev )
     newdc->vportExtY        = dc->vportExtY;
     newdc->virtual_res      = dc->virtual_res;
     newdc->virtual_size     = dc->virtual_size;
-    newdc->BoundsRect       = dc->BoundsRect;
     newdc->gdiFont          = dc->gdiFont;
 
     /* Get/SetDCState() don't change hVisRgn field ("Undoc. Windows" p.559). */
@@ -428,7 +429,6 @@ BOOL nulldrv_RestoreDC( PHYSDEV dev, INT level )
 
     if (!PATH_RestorePath( dc, dcs )) return FALSE;
 
-    dc->flags            = dcs->flags;
     dc->layout           = dcs->layout;
     dc->hDevice          = dcs->hDevice;
     dc->ROPmode          = dcs->ROPmode;
@@ -456,8 +456,6 @@ BOOL nulldrv_RestoreDC( PHYSDEV dev, INT level )
     dc->xformWorld2Vport = dcs->xformWorld2Vport;
     dc->xformVport2World = dcs->xformVport2World;
     dc->vport2WorldValid = dcs->vport2WorldValid;
-    dc->BoundsRect       = dcs->BoundsRect;
-
     dc->wndOrgX          = dcs->wndOrgX;
     dc->wndOrgY          = dcs->wndOrgY;
     dc->wndExtX          = dcs->wndExtX;
@@ -699,6 +697,7 @@ HDC WINAPI CreateCompatibleDC( HDC hdc )
     dc->vis_rect.top    = 0;
     dc->vis_rect.right  = 1;
     dc->vis_rect.bottom = 1;
+    dc->device_rect = dc->vis_rect;
 
     ret = dc->hSelf;
 
@@ -708,6 +707,15 @@ HDC WINAPI CreateCompatibleDC( HDC hdc )
         free_dc_ptr( dc );
         return 0;
     }
+
+    if (!dib_driver.pCreateDC( &dc->physDev, NULL, NULL, NULL, NULL ))
+    {
+        free_dc_ptr( dc );
+        return 0;
+    }
+    physDev = GET_DC_PHYSDEV( dc, pSelectBitmap );
+    physDev->funcs->pSelectBitmap( physDev, dc->hBitmap );
+
     DC_InitDC( dc );
     release_dc_ptr( dc );
     return ret;
@@ -1237,8 +1245,12 @@ BOOL WINAPI GetDeviceGammaRamp(HDC hDC, LPVOID ptr)
     TRACE("%p, %p\n", hDC, ptr);
     if( dc )
     {
-        PHYSDEV physdev = GET_DC_PHYSDEV( dc, pGetDeviceGammaRamp );
-        ret = physdev->funcs->pGetDeviceGammaRamp( physdev, ptr );
+        if (GetObjectType( hDC ) != OBJ_MEMDC)
+        {
+            PHYSDEV physdev = GET_DC_PHYSDEV( dc, pGetDeviceGammaRamp );
+            ret = physdev->funcs->pGetDeviceGammaRamp( physdev, ptr );
+        }
+        else SetLastError( ERROR_INVALID_PARAMETER );
 	release_dc_ptr( dc );
     }
     return ret;
@@ -1255,8 +1267,12 @@ BOOL WINAPI SetDeviceGammaRamp(HDC hDC, LPVOID ptr)
     TRACE("%p, %p\n", hDC, ptr);
     if( dc )
     {
-        PHYSDEV physdev = GET_DC_PHYSDEV( dc, pSetDeviceGammaRamp );
-        ret = physdev->funcs->pSetDeviceGammaRamp( physdev, ptr );
+        if (GetObjectType( hDC ) != OBJ_MEMDC)
+        {
+            PHYSDEV physdev = GET_DC_PHYSDEV( dc, pSetDeviceGammaRamp );
+            ret = physdev->funcs->pSetDeviceGammaRamp( physdev, ptr );
+        }
+        else SetLastError( ERROR_INVALID_PARAMETER );
 	release_dc_ptr( dc );
     }
     return ret;
@@ -1314,24 +1330,43 @@ HCOLORSPACE WINAPI SetColorSpace( HDC hDC, HCOLORSPACE hColorSpace )
  */
 UINT WINAPI GetBoundsRect(HDC hdc, LPRECT rect, UINT flags)
 {
-    UINT ret = 0;
+    PHYSDEV physdev;
+    RECT device_rect;
+    UINT ret;
     DC *dc = get_dc_ptr( hdc );
 
     if ( !dc ) return 0;
 
+    physdev = GET_DC_PHYSDEV( dc, pGetBoundsRect );
+    ret = physdev->funcs->pGetBoundsRect( physdev, &device_rect, DCB_RESET );
+    if (!ret)
+    {
+        release_dc_ptr( dc );
+        return 0;
+    }
+    if (dc->bounds_enabled && ret == DCB_SET) add_bounds_rect( &dc->bounds, &device_rect );
+
     if (rect)
     {
-        *rect = dc->BoundsRect;
-        ret = is_rect_empty( rect ) ? DCB_RESET : DCB_SET;
+        if (is_rect_empty( &dc->bounds ))
+        {
+            rect->left = rect->top = rect->right = rect->bottom = 0;
+            ret = DCB_RESET;
+        }
+        else
+        {
+            *rect = dc->bounds;
+            rect->left   = max( rect->left, 0 );
+            rect->top    = max( rect->top, 0 );
+            rect->right  = min( rect->right, dc->vis_rect.right - dc->vis_rect.left );
+            rect->bottom = min( rect->bottom, dc->vis_rect.bottom - dc->vis_rect.top );
+            ret = DCB_SET;
+        }
         DPtoLP( hdc, (POINT *)rect, 2 );
     }
-    if (flags & DCB_RESET)
-    {
-        dc->BoundsRect.left   = 0;
-        dc->BoundsRect.top    = 0;
-        dc->BoundsRect.right  = 0;
-        dc->BoundsRect.bottom = 0;
-    }
+    else ret = 0;
+
+    if (flags & DCB_RESET) reset_bounds( &dc->bounds );
     release_dc_ptr( dc );
     return ret;
 }
@@ -1342,43 +1377,36 @@ UINT WINAPI GetBoundsRect(HDC hdc, LPRECT rect, UINT flags)
  */
 UINT WINAPI SetBoundsRect(HDC hdc, const RECT* rect, UINT flags)
 {
+    PHYSDEV physdev;
     UINT ret;
     DC *dc;
 
     if ((flags & DCB_ENABLE) && (flags & DCB_DISABLE)) return 0;
     if (!(dc = get_dc_ptr( hdc ))) return 0;
 
-    ret = ((dc->flags & DC_BOUNDS_ENABLE) ? DCB_ENABLE : DCB_DISABLE) |
-           (is_rect_empty( &dc->BoundsRect ) ? DCB_RESET : DCB_SET);
-
-    if (flags & DCB_RESET)
+    physdev = GET_DC_PHYSDEV( dc, pSetBoundsRect );
+    ret = physdev->funcs->pSetBoundsRect( physdev, &dc->bounds, flags );
+    if (!ret)
     {
-        dc->BoundsRect.left   = 0;
-        dc->BoundsRect.top    = 0;
-        dc->BoundsRect.right  = 0;
-        dc->BoundsRect.bottom = 0;
+        release_dc_ptr( dc );
+        return 0;
     }
 
+    ret = (dc->bounds_enabled ? DCB_ENABLE : DCB_DISABLE) |
+          (is_rect_empty( &dc->bounds ) ? ret & DCB_SET : DCB_SET);
+
+    if (flags & DCB_RESET) reset_bounds( &dc->bounds );
+
     if ((flags & DCB_ACCUMULATE) && rect)
     {
         RECT rc = *rect;
 
         LPtoDP( hdc, (POINT *)&rc, 2 );
-        if (!is_rect_empty( &rc ))
-        {
-            if (!is_rect_empty( &dc->BoundsRect))
-            {
-                dc->BoundsRect.left   = min( dc->BoundsRect.left, rc.left );
-                dc->BoundsRect.top    = min( dc->BoundsRect.top, rc.top );
-                dc->BoundsRect.right  = max( dc->BoundsRect.right, rc.right );
-                dc->BoundsRect.bottom = max( dc->BoundsRect.bottom, rc.bottom );
-            }
-            else dc->BoundsRect = rc;
-        }
+        add_bounds_rect( &dc->bounds, &rc );
     }
 
-    if (flags & DCB_ENABLE) dc->flags |= DC_BOUNDS_ENABLE;
-    if (flags & DCB_DISABLE) dc->flags &= ~DC_BOUNDS_ENABLE;
+    if (flags & DCB_ENABLE) dc->bounds_enabled = TRUE;
+    if (flags & DCB_DISABLE) dc->bounds_enabled = FALSE;
 
     release_dc_ptr( dc );
     return ret;
