@@ -68,11 +68,15 @@ static void init_bit_fields(dib_info *dib, const DWORD *bit_fields)
 }
 
 static void init_dib_info(dib_info *dib, const BITMAPINFOHEADER *bi, const DWORD *bit_fields,
-                          const RGBQUAD *color_table, void *bits, enum dib_info_flags flags)
+                          const RGBQUAD *color_table, void *bits)
 {
     dib->bit_count    = bi->biBitCount;
     dib->width        = bi->biWidth;
     dib->height       = bi->biHeight;
+    dib->rect.left    = 0;
+    dib->rect.top     = 0;
+    dib->rect.right   = bi->biWidth;
+    dib->rect.bottom  = abs( bi->biHeight );
     dib->compression  = bi->biCompression;
     dib->stride       = get_dib_stride( dib->width, dib->bit_count );
     dib->bits.ptr     = bits;
@@ -141,11 +145,6 @@ static void init_dib_info(dib_info *dib, const BITMAPINFOHEADER *bi, const DWORD
         dib->color_table = color_table;
         dib->color_table_size = bi->biClrUsed;
     }
-    else if (flags & default_color_table)
-    {
-        dib->color_table = get_default_color_table( dib->bit_count );
-        dib->color_table_size = dib->color_table ? (1 << dib->bit_count) : 0;
-    }
     else
     {
         dib->color_table = NULL;
@@ -153,12 +152,12 @@ static void init_dib_info(dib_info *dib, const BITMAPINFOHEADER *bi, const DWORD
     }
 }
 
-void init_dib_info_from_bitmapinfo(dib_info *dib, const BITMAPINFO *info, void *bits, enum dib_info_flags flags)
+void init_dib_info_from_bitmapinfo(dib_info *dib, const BITMAPINFO *info, void *bits)
 {
-    init_dib_info( dib, &info->bmiHeader, (const DWORD *)info->bmiColors, info->bmiColors, bits, flags );
+    init_dib_info( dib, &info->bmiHeader, (const DWORD *)info->bmiColors, info->bmiColors, bits );
 }
 
-BOOL init_dib_info_from_bitmapobj(dib_info *dib, BITMAPOBJ *bmp, enum dib_info_flags flags)
+BOOL init_dib_info_from_bitmapobj(dib_info *dib, BITMAPOBJ *bmp)
 {
     if (!is_bitmapobj_dib( bmp ))
     {
@@ -172,16 +171,15 @@ BOOL init_dib_info_from_bitmapobj(dib_info *dib, BITMAPOBJ *bmp, enum dib_info_f
                                               bmp->dib.dsBm.bmHeight * width_bytes );
             if (!bmp->dib.dsBm.bmBits) return FALSE;
         }
-        init_dib_info_from_bitmapinfo( dib, &info, bmp->dib.dsBm.bmBits, flags );
+        init_dib_info_from_bitmapinfo( dib, &info, bmp->dib.dsBm.bmBits );
     }
     else init_dib_info( dib, &bmp->dib.dsBmih, bmp->dib.dsBitfields,
-                        bmp->color_table, bmp->dib.dsBm.bmBits, flags );
+                        bmp->color_table, bmp->dib.dsBm.bmBits );
     return TRUE;
 }
 
 static void clear_dib_info(dib_info *dib)
 {
-    dib->color_table = NULL;
     dib->bits.ptr    = NULL;
     dib->bits.free   = NULL;
     dib->bits.param  = NULL;
@@ -221,12 +219,12 @@ DWORD convert_bitmapinfo( const BITMAPINFO *src_info, void *src_bits, struct bit
     dib_info src_dib, dst_dib;
     DWORD ret;
 
-    init_dib_info_from_bitmapinfo( &src_dib, src_info, src_bits, default_color_table );
-    init_dib_info_from_bitmapinfo( &dst_dib, dst_info, dst_bits, default_color_table );
+    init_dib_info_from_bitmapinfo( &src_dib, src_info, src_bits );
+    init_dib_info_from_bitmapinfo( &dst_dib, dst_info, dst_bits );
 
     __TRY
     {
-        dst_dib.funcs->convert_to( &dst_dib, &src_dib, &src->visrect );
+        dst_dib.funcs->convert_to( &dst_dib, &src_dib, &src->visrect, FALSE );
         ret = TRUE;
     }
     __EXCEPT_PAGE_FAULT
@@ -265,10 +263,11 @@ int get_clipped_rects( const dib_info *dib, const RECT *rc, HRGN clip, struct cl
 
     init_clipped_rects( clip_rects );
 
-    rect.left   = 0;
-    rect.top    = 0;
-    rect.right  = dib->width;
-    rect.bottom = dib->height;
+    rect.left   = max( 0, -dib->rect.left );
+    rect.top    = max( 0, -dib->rect.top );
+    rect.right  = min( dib->rect.right, dib->width ) - dib->rect.left;
+    rect.bottom = min( dib->rect.bottom, dib->height ) - dib->rect.top;
+    if (is_rect_empty( &rect )) return 0;
     if (rc && !intersect_rect( &rect, &rect, rc )) return 0;
 
     if (!clip)
@@ -298,6 +297,26 @@ int get_clipped_rects( const dib_info *dib, const RECT *rc, HRGN clip, struct cl
     return clip_rects->count;
 }
 
+void add_clipped_bounds( dibdrv_physdev *dev, const RECT *rect, HRGN clip )
+{
+    const WINEREGION *region;
+    RECT rc;
+
+    if (!dev->bounds) return;
+    if (clip)
+    {
+        if (!(region = get_wine_region( clip ))) return;
+        if (!rect) rc = region->extents;
+        else intersect_rect( &rc, rect, &region->extents );
+        release_wine_region( clip );
+    }
+    else rc = *rect;
+
+    if (is_rect_empty( &rc )) return;
+    offset_rect( &rc, dev->dib.rect.left, dev->dib.rect.top );
+    add_bounds_rect( dev->bounds, &rc );
+}
+
 /**********************************************************************
  *	     dibdrv_CreateDC
  */
@@ -327,22 +346,6 @@ static BOOL dibdrv_DeleteDC( PHYSDEV dev )
     return TRUE;
 }
 
-/***********************************************************************
- *           dibdrv_CopyBitmap
- */
-static BOOL dibdrv_CopyBitmap( HBITMAP src, HBITMAP dst )
-{
-    return nulldrv_CopyBitmap( src, dst );
-}
-
-/***********************************************************************
- *           dibdrv_DeleteBitmap
- */
-static BOOL dibdrv_DeleteBitmap( HBITMAP bitmap )
-{
-    return TRUE;
-}
-
 /***********************************************************************
  *           dibdrv_SelectBitmap
  */
@@ -356,7 +359,7 @@ static HBITMAP dibdrv_SelectBitmap( PHYSDEV dev, HBITMAP bitmap )
 
     if (!bmp) return 0;
 
-    if(!init_dib_info_from_bitmapobj(&dib, bmp, default_color_table))
+    if (!init_dib_info_from_bitmapobj(&dib, bmp))
     {
         GDI_ReleaseObj( bitmap );
         return 0;
@@ -379,183 +382,15 @@ static void dibdrv_SetDeviceClipping( PHYSDEV dev, HRGN rgn )
 }
 
 /***********************************************************************
- *           dibdrv_ChoosePixelFormat
+ *           dibdrv_SetBoundsRect
  */
-static INT dibdrv_ChoosePixelFormat( PHYSDEV dev, const PIXELFORMATDESCRIPTOR *pfd )
+static UINT dibdrv_SetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
 {
-    FIXME( "Not supported on DIB section\n" );
-    return 0;
-}
+    dibdrv_physdev *pdev = get_dibdrv_pdev( dev );
 
-/***********************************************************************
- *           dibdrv_DescribePixelFormat
- */
-static INT dibdrv_DescribePixelFormat( PHYSDEV dev, INT fmt, UINT size, PIXELFORMATDESCRIPTOR *pfd )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return 0;
-}
-
-/***********************************************************************
- *           dibdrv_ExtEscape
- */
-static INT dibdrv_ExtEscape( PHYSDEV dev, INT escape, INT in_size, const void *in_data,
-                             INT out_size, void *out_data )
-{
-    return 0;
-}
-
-/***********************************************************************
- *           dibdrv_GetDeviceGammaRamp
- */
-static BOOL dibdrv_GetDeviceGammaRamp( PHYSDEV dev, void *ramp )
-{
-    SetLastError( ERROR_INVALID_PARAMETER );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_GetPixelFormat
- */
-static INT dibdrv_GetPixelFormat( PHYSDEV dev )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return 0;
-}
-
-/***********************************************************************
- *           dibdrv_SetDeviceGammaRamp
- */
-static BOOL dibdrv_SetDeviceGammaRamp( PHYSDEV dev, void *ramp )
-{
-    SetLastError( ERROR_INVALID_PARAMETER );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_SetPixelFormat
- */
-static BOOL dibdrv_SetPixelFormat( PHYSDEV dev, INT fmt, const PIXELFORMATDESCRIPTOR *pfd )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_SwapBuffers
- */
-static BOOL dibdrv_SwapBuffers( PHYSDEV dev )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_wglCopyContext
- */
-static BOOL dibdrv_wglCopyContext( HGLRC src, HGLRC dst, UINT mask )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_wglCreateContext
- */
-static HGLRC dibdrv_wglCreateContext( PHYSDEV dev )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return 0;
-}
-
-/***********************************************************************
- *           dibdrv_wglCreateContextAttribsARB
- */
-static HGLRC dibdrv_wglCreateContextAttribsARB( PHYSDEV dev, HGLRC ctx, const int *attribs )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return 0;
-}
-
-/***********************************************************************
- *           dibdrv_wglDeleteContext
- */
-static BOOL dibdrv_wglDeleteContext( HGLRC ctx )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_wglGetPbufferDCARB
- */
-static HDC dibdrv_wglGetPbufferDCARB( PHYSDEV dev, void *buffer )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return 0;
-}
-
-/***********************************************************************
- *           dibdrv_wglGetProcAddress
- */
-static PROC dibdrv_wglGetProcAddress( LPCSTR name )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return NULL;
-}
-
-/***********************************************************************
- *           dibdrv_wglMakeContextCurrentARB
- */
-static BOOL dibdrv_wglMakeContextCurrentARB( PHYSDEV draw_dev, PHYSDEV read_dev, HGLRC ctx )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_wglMakeCurrent
- */
-static BOOL dibdrv_wglMakeCurrent( PHYSDEV dev, HGLRC ctx )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_wglSetPixelFormatWINE
- */
-static BOOL dibdrv_wglSetPixelFormatWINE( PHYSDEV dev, INT fmt, const PIXELFORMATDESCRIPTOR *pfd )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_wglShareLists
- */
-static BOOL dibdrv_wglShareLists( HGLRC ctx1, HGLRC ctx2 )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_wglUseFontBitmapsA
- */
-static BOOL dibdrv_wglUseFontBitmapsA( PHYSDEV dev, DWORD first, DWORD count, DWORD base )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
-}
-
-/***********************************************************************
- *           dibdrv_wglUseFontBitmapsW
- */
-static BOOL dibdrv_wglUseFontBitmapsW( PHYSDEV dev, DWORD first, DWORD count, DWORD base )
-{
-    FIXME( "Not supported on DIB section\n" );
-    return FALSE;
+    if (flags & DCB_DISABLE) pdev->bounds = NULL;
+    else if (flags & DCB_ENABLE) pdev->bounds = rect;
+    return DCB_RESET;  /* we don't have device-specific bounds */
 }
 
 const struct gdi_dc_funcs dib_driver =
@@ -568,17 +403,13 @@ const struct gdi_dc_funcs dib_driver =
     dibdrv_ArcTo,                       /* pArcTo */
     NULL,                               /* pBeginPath */
     dibdrv_BlendImage,                  /* pBlendImage */
-    dibdrv_ChoosePixelFormat,           /* pChoosePixelFormat */
     dibdrv_Chord,                       /* pChord */
     NULL,                               /* pCloseFigure */
-    dibdrv_CopyBitmap,                  /* pCopyBitmap */
-    NULL,                               /* pCreateBitmap */
     NULL,                               /* pCreateCompatibleDC */
     dibdrv_CreateDC,                    /* pCreateDC */
-    dibdrv_DeleteBitmap,                /* pDeleteBitmap */
     dibdrv_DeleteDC,                    /* pDeleteDC */
     NULL,                               /* pDeleteObject */
-    dibdrv_DescribePixelFormat,         /* pDescribePixelFormat */
+    NULL,                               /* pDescribePixelFormat */
     NULL,                               /* pDeviceCapabilities */
     dibdrv_Ellipse,                     /* pEllipse */
     NULL,                               /* pEndDoc */
@@ -588,7 +419,7 @@ const struct gdi_dc_funcs dib_driver =
     NULL,                               /* pEnumICMProfiles */
     NULL,                               /* pExcludeClipRect */
     NULL,                               /* pExtDeviceMode */
-    dibdrv_ExtEscape,                   /* pExtEscape */
+    NULL,                               /* pExtEscape */
     dibdrv_ExtFloodFill,                /* pExtFloodFill */
     NULL,                               /* pExtSelectClipRgn */
     dibdrv_ExtTextOut,                  /* pExtTextOut */
@@ -599,11 +430,12 @@ const struct gdi_dc_funcs dib_driver =
     NULL,                               /* pFrameRgn */
     NULL,                               /* pGdiComment */
     NULL,                               /* pGdiRealizationInfo */
+    NULL,                               /* pGetBoundsRect */
     NULL,                               /* pGetCharABCWidths */
     NULL,                               /* pGetCharABCWidthsI */
     NULL,                               /* pGetCharWidth */
     NULL,                               /* pGetDeviceCaps */
-    dibdrv_GetDeviceGammaRamp,          /* pGetDeviceGammaRamp */
+    NULL,                               /* pGetDeviceGammaRamp */
     NULL,                               /* pGetFontData */
     NULL,                               /* pGetFontUnicodeRanges */
     NULL,                               /* pGetGlyphIndices */
@@ -614,7 +446,6 @@ const struct gdi_dc_funcs dib_driver =
     dibdrv_GetNearestColor,             /* pGetNearestColor */
     NULL,                               /* pGetOutlineTextMetrics */
     dibdrv_GetPixel,                    /* pGetPixel */
-    dibdrv_GetPixelFormat,              /* pGetPixelFormat */
     NULL,                               /* pGetSystemPaletteEntries */
     NULL,                               /* pGetTextCharsetInfo */
     NULL,                               /* pGetTextExtentExPoint */
@@ -660,16 +491,17 @@ const struct gdi_dc_funcs dib_driver =
     NULL,                               /* pSetArcDirection */
     NULL,                               /* pSetBkColor */
     NULL,                               /* pSetBkMode */
+    dibdrv_SetBoundsRect,               /* pSetBoundsRect */
     dibdrv_SetDCBrushColor,             /* pSetDCBrushColor */
     dibdrv_SetDCPenColor,               /* pSetDCPenColor */
     NULL,                               /* pSetDIBitsToDevice */
     dibdrv_SetDeviceClipping,           /* pSetDeviceClipping */
-    dibdrv_SetDeviceGammaRamp,          /* pSetDeviceGammaRamp */
+    NULL,                               /* pSetDeviceGammaRamp */
     NULL,                               /* pSetLayout */
     NULL,                               /* pSetMapMode */
     NULL,                               /* pSetMapperFlags */
     dibdrv_SetPixel,                    /* pSetPixel */
-    dibdrv_SetPixelFormat,              /* pSetPixelFormat */
+    NULL,                               /* pSetPixelFormat */
     NULL,                               /* pSetPolyFillMode */
     NULL,                               /* pSetROP2 */
     NULL,                               /* pSetRelAbs */
@@ -689,19 +521,9 @@ const struct gdi_dc_funcs dib_driver =
     NULL,                               /* pStretchDIBits */
     NULL,                               /* pStrokeAndFillPath */
     NULL,                               /* pStrokePath */
-    dibdrv_SwapBuffers,                 /* pSwapBuffers */
+    NULL,                               /* pSwapBuffers */
     NULL,                               /* pUnrealizePalette */
     NULL,                               /* pWidenPath */
-    dibdrv_wglCopyContext,              /* pwglCopyContext */
-    dibdrv_wglCreateContext,            /* pwglCreateContext */
-    dibdrv_wglCreateContextAttribsARB,  /* pwglCreateContextAttribsARB */
-    dibdrv_wglDeleteContext,            /* pwglDeleteContext */
-    dibdrv_wglGetPbufferDCARB,          /* pwglGetPbufferDCARB */
-    dibdrv_wglGetProcAddress,           /* pwglGetProcAddress */
-    dibdrv_wglMakeContextCurrentARB,    /* pwglMakeContextCurrentARB */
-    dibdrv_wglMakeCurrent,              /* pwglMakeCurrent */
-    dibdrv_wglSetPixelFormatWINE,       /* pwglSetPixelFormatWINE */
-    dibdrv_wglShareLists,               /* pwglShareLists */
-    dibdrv_wglUseFontBitmapsA,          /* pwglUseFontBitmapsA */
-    dibdrv_wglUseFontBitmapsW,          /* pwglUseFontBitmapsW */
+    NULL,                               /* wine_get_wgl_driver */
+    GDI_PRIORITY_DIB_DRV                /* priority */
 };
