@@ -26,6 +26,20 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(dib);
 
+/* paint a region with the brush (note: the region can be modified) */
+static BOOL brush_region( dibdrv_physdev *pdev, HRGN region )
+{
+    if (pdev->clip) CombineRgn( region, region, pdev->clip, RGN_AND );
+    return brush_rect( pdev, &pdev->brush, NULL, region, GetROP2( pdev->dev.hdc ));
+}
+
+/* paint a region with the pen (note: the region can be modified) */
+static BOOL pen_region( dibdrv_physdev *pdev, HRGN region )
+{
+    if (pdev->clip) CombineRgn( region, region, pdev->clip, RGN_AND );
+    return brush_rect( pdev, &pdev->pen_brush, NULL, region, GetROP2( pdev->dev.hdc ));
+}
+
 static RECT get_device_rect( HDC hdc, int left, int top, int right, int bottom, BOOL rtl_correction )
 {
     RECT rect;
@@ -72,6 +86,51 @@ static BOOL get_pen_device_rect( dibdrv_physdev *dev, RECT *rect, int left, int
     return TRUE;
 }
 
+static void add_pen_lines_bounds( dibdrv_physdev *dev, int count, const POINT *points, HRGN rgn )
+{
+    const WINEREGION *region;
+    RECT bounds, rect;
+    int width = 0;
+
+    if (!dev->bounds) return;
+    reset_bounds( &bounds );
+
+    if (dev->pen_uses_region)
+    {
+        /* Windows uses some heuristics to estimate the distance from the point that will be painted */
+        width = dev->pen_width + 2;
+        if (dev->pen_join == PS_JOIN_MITER)
+        {
+            width *= 5;
+            if (dev->pen_endcap == PS_ENDCAP_SQUARE) width = (width * 3 + 1) / 2;
+        }
+        else
+        {
+            if (dev->pen_endcap == PS_ENDCAP_SQUARE) width -= width / 4;
+            else width = (width + 1) / 2;
+        }
+
+        /* in case the heuristics are wrong, add the actual region too */
+        if ((region = get_wine_region( rgn )))
+        {
+            add_bounds_rect( &bounds, &region->extents );
+            release_wine_region( rgn );
+        }
+    }
+
+    while (count-- > 0)
+    {
+        rect.left   = points->x - width;
+        rect.top    = points->y - width;
+        rect.right  = points->x + width + 1;
+        rect.bottom = points->y + width + 1;
+        add_bounds_rect( &bounds, &rect );
+        points++;
+    }
+
+    add_clipped_bounds( dev, &bounds, dev->clip );
+}
+
 /* compute the points for the first quadrant of an ellipse, counterclockwise from the x axis */
 /* 'data' must contain enough space, (width+height)/2 is a reasonable upper bound */
 static int ellipse_first_quadrant( int width, int height, POINT *data )
@@ -285,6 +344,7 @@ static BOOL draw_arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
 
     reset_dash_origin( pdev );
     pdev->pen_lines( pdev, count, points, extra_lines > 0, outline );
+    add_pen_lines_bounds( pdev, count, points, outline );
 
     if (interior)
     {
@@ -370,23 +430,29 @@ static inline void get_text_bkgnd_masks( dibdrv_physdev *pdev, rop_mask *mask )
 
 static void draw_glyph( dibdrv_physdev *pdev, const POINT *origin, const GLYPHMETRICS *metrics,
                         const struct gdi_image_bits *image, DWORD text_color,
-                        const struct intensity_range *ranges, const struct clipped_rects *clipped_rects )
+                        const struct intensity_range *ranges, const struct clipped_rects *clipped_rects,
+                        RECT *bounds )
 {
     int i;
     RECT rect, clipped_rect;
     POINT src_origin;
     dib_info glyph_dib;
 
-    glyph_dib.bit_count = 8;
-    glyph_dib.width     = metrics->gmBlackBoxX;
-    glyph_dib.height    = metrics->gmBlackBoxY;
-    glyph_dib.stride    = get_dib_stride( metrics->gmBlackBoxX, 8 );
-    glyph_dib.bits      = *image;
+    glyph_dib.bit_count   = 8;
+    glyph_dib.width       = metrics->gmBlackBoxX;
+    glyph_dib.height      = metrics->gmBlackBoxY;
+    glyph_dib.rect.left   = 0;
+    glyph_dib.rect.top    = 0;
+    glyph_dib.rect.right  = metrics->gmBlackBoxX;
+    glyph_dib.rect.bottom = metrics->gmBlackBoxY;
+    glyph_dib.stride      = get_dib_stride( metrics->gmBlackBoxX, 8 );
+    glyph_dib.bits        = *image;
 
     rect.left   = origin->x  + metrics->gmptGlyphOrigin.x;
     rect.top    = origin->y  - metrics->gmptGlyphOrigin.y;
     rect.right  = rect.left  + metrics->gmBlackBoxX;
     rect.bottom = rect.top   + metrics->gmBlackBoxY;
+    add_bounds_rect( bounds, &rect );
 
     for (i = 0; i < clipped_rects->count; i++)
     {
@@ -407,7 +473,7 @@ static const int padding[4] = {0, 3, 2, 1};
 /***********************************************************************
  *         get_glyph_bitmap
  *
- * Retrieve a 17-level bitmap for the appropiate glyph.
+ * Retrieve a 17-level bitmap for the appropriate glyph.
  *
  * For non-antialiased bitmaps convert them to the 17-level format
  * using only values 0 or 16.
@@ -492,7 +558,7 @@ BOOL render_aa_text_bitmapinfo( HDC hdc, BITMAPINFO *info, struct gdi_image_bits
 
     assert( info->bmiHeader.biBitCount > 8 ); /* mono and indexed formats don't support anti-aliasing */
 
-    init_dib_info_from_bitmapinfo( &dib, info, bits->ptr, 0 );
+    init_dib_info_from_bitmapinfo( &dib, info, bits->ptr );
 
     fg = make_rgb_colorref( hdc, &dib, GetTextColor( hdc ), &got_pixel, &fg_pixel);
     if (!got_pixel) fg_pixel = dib.funcs->colorref_to_pixel( &dib, fg );
@@ -525,11 +591,15 @@ BOOL render_aa_text_bitmapinfo( HDC hdc, BITMAPINFO *info, struct gdi_image_bits
             POINT src_origin;
             dib_info glyph_dib;
 
-            glyph_dib.bit_count = 8;
-            glyph_dib.width     = metrics.gmBlackBoxX;
-            glyph_dib.height    = metrics.gmBlackBoxY;
-            glyph_dib.stride    = get_dib_stride( metrics.gmBlackBoxX, 8 );
-            glyph_dib.bits      = image;
+            glyph_dib.bit_count   = 8;
+            glyph_dib.width       = metrics.gmBlackBoxX;
+            glyph_dib.height      = metrics.gmBlackBoxY;
+            glyph_dib.rect.left   = 0;
+            glyph_dib.rect.top    = 0;
+            glyph_dib.rect.right  = metrics.gmBlackBoxX;
+            glyph_dib.rect.bottom = metrics.gmBlackBoxY;
+            glyph_dib.stride      = get_dib_stride( metrics.gmBlackBoxX, 8 );
+            glyph_dib.bits        = image;
 
             rect.left   = x + metrics.gmptGlyphOrigin.x;
             rect.top    = y - metrics.gmptGlyphOrigin.y;
@@ -576,15 +646,18 @@ BOOL dibdrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags,
     struct clipped_rects clipped_rects;
     UINT aa_flags, i;
     POINT origin;
+    RECT bounds;
     DWORD text_color, err;
     struct intensity_range ranges[17];
 
     init_clipped_rects( &clipped_rects );
+    reset_bounds( &bounds );
 
     if (flags & ETO_OPAQUE)
     {
         rop_mask bkgnd_color;
         get_text_bkgnd_masks( pdev, &bkgnd_color );
+        add_bounds_rect( &bounds, rect );
         get_clipped_rects( &pdev->dib, rect, pdev->clip, &clipped_rects );
         pdev->dib.funcs->solid_rects( &pdev->dib, clipped_rects.count, clipped_rects.rects,
                                       bkgnd_color.and, bkgnd_color.xor );
@@ -602,7 +675,7 @@ BOOL dibdrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags,
         free_clipped_rects( &clipped_rects );
         get_clipped_rects( &pdev->dib, NULL, pdev->clip, &clipped_rects );
     }
-    if (!clipped_rects.count) return TRUE;
+    if (!clipped_rects.count) goto done;
 
     text_color = get_pixel_color( pdev, GetTextColor( pdev->dev.hdc ), TRUE );
     get_aa_ranges( pdev->dib.funcs->pixel_to_colorref( &pdev->dib, text_color ), ranges );
@@ -618,7 +691,9 @@ BOOL dibdrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags,
         err = get_glyph_bitmap( dev->hdc, (UINT)str[i], aa_flags, &metrics, &image );
         if (err) continue;
 
-        if (image.ptr) draw_glyph( pdev, &origin, &metrics, &image, text_color, ranges, &clipped_rects );
+        if (image.ptr)
+            draw_glyph( pdev, &origin, &metrics, &image, text_color, ranges, &clipped_rects, &bounds );
+
         if (image.free) image.free( &image );
 
         if (dx)
@@ -639,6 +714,7 @@ BOOL dibdrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags,
     }
 
 done:
+    add_clipped_bounds( pdev, &bounds, pdev->clip );
     free_clipped_rects( &clipped_rects );
     return TRUE;
 }
@@ -715,13 +791,15 @@ static inline void do_next_row( dib_info *dib, HRGN clip, const RECT *row, int o
 static void fill_row( dib_info *dib, HRGN clip, RECT *row, DWORD pixel, UINT type, HRGN rgn )
 {
     while (row->left > 0 && is_interior( dib, clip, row->left - 1, row->top, pixel, type)) row->left--;
-    while (row->right < dib->width && is_interior( dib, clip, row->right, row->top, pixel, type)) row->right++;
+    while (row->right < dib->rect.right - dib->rect.left &&
+           is_interior( dib, clip, row->right, row->top, pixel, type))
+        row->right++;
+
     add_rect_to_region( rgn, row );
 
     if (row->top > 0) do_next_row( dib, clip, row, -1, pixel, type, rgn );
-    if (row->top < dib->height - 1) do_next_row( dib, clip, row, 1, pixel, type, rgn );
-
-    return;
+    if (row->top < dib->rect.bottom - dib->rect.top - 1)
+        do_next_row( dib, clip, row, 1, pixel, type, rgn );
 }
 
 /***********************************************************************
@@ -738,7 +816,7 @@ BOOL dibdrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT type )
 
     if (!is_interior( &pdev->dib, pdev->clip, x, y, pixel, type )) return FALSE;
 
-    rgn = CreateRectRgn( 0, 0, 0, 0 );
+    if (!(rgn = CreateRectRgn( 0, 0, 0, 0 ))) return FALSE;
     row.left = x;
     row.right = x + 1;
     row.top = y;
@@ -746,6 +824,7 @@ BOOL dibdrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT type )
 
     fill_row( &pdev->dib, pdev->clip, &row, pixel, type, rgn );
 
+    add_clipped_bounds( pdev, NULL, rgn );
     brush_region( pdev, rgn );
 
     DeleteObject( rgn );
@@ -781,8 +860,8 @@ COLORREF dibdrv_GetPixel( PHYSDEV dev, INT x, INT y )
     pt.y = y;
     LPtoDP( dev->hdc, &pt, 1 );
 
-    if (pt.x < 0 || pt.x >= pdev->dib.width ||
-        pt.y < 0 || pt.y >= pdev->dib.height)
+    if (pt.x < 0 || pt.x >= pdev->dib.rect.right - pdev->dib.rect.left ||
+        pt.y < 0 || pt.y >= pdev->dib.rect.bottom - pdev->dib.rect.top)
         return CLR_INVALID;
 
     pixel = pdev->dib.funcs->get_pixel( &pdev->dib, pt.x, pt.y );
@@ -810,10 +889,10 @@ BOOL dibdrv_LineTo( PHYSDEV dev, INT x, INT y )
     reset_dash_origin(pdev);
 
     ret = pdev->pen_lines(pdev, 2, pts, FALSE, region);
+    add_pen_lines_bounds( pdev, 2, pts, region );
 
     if (region)
     {
-        if (pdev->clip) CombineRgn( region, region, pdev->clip, RGN_AND );
         ret = pen_region( pdev, region );
         DeleteObject( region );
     }
@@ -840,6 +919,7 @@ BOOL dibdrv_PatBlt( PHYSDEV dev, struct bitblt_coords *dst, DWORD rop )
 
     TRACE("(%p, %d, %d, %d, %d, %06x)\n", dev, dst->x, dst->y, dst->width, dst->height, rop);
 
+    add_clipped_bounds( pdev, &dst->visrect, 0 );
     return brush_rect( pdev, &pdev->brush, &dst->visrect, pdev->clip, get_rop2_from_rop(rop) );
 }
 
@@ -851,10 +931,12 @@ BOOL dibdrv_PaintRgn( PHYSDEV dev, HRGN rgn )
     dibdrv_physdev *pdev = get_dibdrv_pdev(dev);
     const WINEREGION *region;
     int i;
-    RECT rect;
+    RECT rect, bounds;
 
     TRACE("%p, %p\n", dev, rgn);
 
+    reset_bounds( &bounds );
+
     region = get_wine_region( rgn );
     if(!region) return FALSE;
 
@@ -862,10 +944,12 @@ BOOL dibdrv_PaintRgn( PHYSDEV dev, HRGN rgn )
     {
         rect = get_device_rect( dev->hdc, region->rects[i].left, region->rects[i].top,
                                 region->rects[i].right, region->rects[i].bottom, FALSE );
+        add_bounds_rect( &bounds, &rect );
         brush_rect( pdev, &pdev->brush, &rect, pdev->clip, GetROP2( dev->hdc ) );
     }
 
     release_wine_region( rgn );
+    add_clipped_bounds( pdev, &bounds, pdev->clip );
     return TRUE;
 }
 
@@ -914,6 +998,7 @@ BOOL dibdrv_PolyPolygon( PHYSDEV dev, const POINT *pt, const INT *counts, DWORD
         pdev->pen_lines( pdev, counts[i], points + pos, TRUE, outline );
         pos += counts[i];
     }
+    add_pen_lines_bounds( pdev, total, points, outline );
 
     if (interior)
     {
@@ -936,19 +1021,21 @@ BOOL dibdrv_PolyPolygon( PHYSDEV dev, const POINT *pt, const INT *counts, DWORD
 BOOL dibdrv_PolyPolyline( PHYSDEV dev, const POINT* pt, const DWORD* counts, DWORD polylines )
 {
     dibdrv_physdev *pdev = get_dibdrv_pdev(dev);
-    DWORD max_points = 0, i;
+    DWORD total, pos, i;
     POINT *points;
     BOOL ret = TRUE;
     HRGN outline = 0;
 
-    for (i = 0; i < polylines; i++)
+    for (i = total = 0; i < polylines; i++)
     {
         if (counts[i] < 2) return FALSE;
-        max_points = max( counts[i], max_points );
+        total += counts[i];
     }
 
-    points = HeapAlloc( GetProcessHeap(), 0, max_points * sizeof(*pt) );
+    points = HeapAlloc( GetProcessHeap(), 0, total * sizeof(*pt) );
     if (!points) return FALSE;
+    memcpy( points, pt, total * sizeof(*pt) );
+    LPtoDP( dev->hdc, points, total );
 
     if (pdev->pen_uses_region && !(outline = CreateRectRgn( 0, 0, 0, 0 )))
     {
@@ -956,19 +1043,16 @@ BOOL dibdrv_PolyPolyline( PHYSDEV dev, const POINT* pt, const DWORD* counts, DWO
         return FALSE;
     }
 
-    for (i = 0; i < polylines; i++)
+    for (i = pos = 0; i < polylines; i++)
     {
-        memcpy( points, pt, counts[i] * sizeof(*pt) );
-        pt += counts[i];
-        LPtoDP( dev->hdc, points, counts[i] );
-
         reset_dash_origin( pdev );
-        pdev->pen_lines( pdev, counts[i], points, FALSE, outline );
+        pdev->pen_lines( pdev, counts[i], points + pos, FALSE, outline );
+        pos += counts[i];
     }
+    add_pen_lines_bounds( pdev, total, points, outline );
 
     if (outline)
     {
-        if (pdev->clip) CombineRgn( outline, outline, pdev->clip, RGN_AND );
         ret = pen_region( pdev, outline );
         DeleteObject( outline );
     }
@@ -1037,6 +1121,7 @@ BOOL dibdrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bottom )
     }
 
     pdev->pen_lines(pdev, 4, pts, TRUE, outline);
+    add_pen_lines_bounds( pdev, 4, pts, outline );
 
     if (outline)
     {
@@ -1157,6 +1242,7 @@ BOOL dibdrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
 
     reset_dash_origin( pdev );
     pdev->pen_lines( pdev, count, points, TRUE, outline );
+    add_pen_lines_bounds( pdev, count, points, outline );
 
     if (interior)
     {
@@ -1202,6 +1288,7 @@ COLORREF dibdrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color )
     rect.top =  pt.y;
     rect.right = rect.left + 1;
     rect.bottom = rect.top + 1;
+    add_clipped_bounds( pdev, &rect, pdev->clip );
 
     /* SetPixel doesn't do the 1bpp massaging like other fg colors */
     pixel = get_pixel_color( pdev, color, FALSE );
