@@ -50,6 +50,7 @@ static struct list drivers = LIST_INIT( drivers );
 static struct graphics_driver *display_driver;
 
 const struct gdi_dc_funcs *font_driver = NULL;
+static const struct wgl_funcs null_wgl_driver;
 
 static CRITICAL_SECTION driver_section;
 static CRITICAL_SECTION_DEBUG critsect_debug =
@@ -198,22 +199,12 @@ static BOOL nulldrv_Arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
     return TRUE;
 }
 
-static INT nulldrv_ChoosePixelFormat( PHYSDEV dev, const PIXELFORMATDESCRIPTOR *descr )
-{
-    return 0;
-}
-
 static BOOL nulldrv_Chord( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
                            INT xstart, INT ystart, INT xend, INT yend )
 {
     return TRUE;
 }
 
-static BOOL nulldrv_CreateBitmap( PHYSDEV dev, HBITMAP bitmap )
-{
-    return TRUE;
-}
-
 static BOOL nulldrv_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
 {
     if (!display_driver || !display_driver->funcs->pCreateCompatibleDC) return TRUE;
@@ -227,11 +218,6 @@ static BOOL nulldrv_CreateDC( PHYSDEV *dev, LPCWSTR driver, LPCWSTR device,
     return FALSE;
 }
 
-static BOOL nulldrv_DeleteBitmap( HBITMAP bitmap )
-{
-    return TRUE;
-}
-
 static BOOL nulldrv_DeleteDC( PHYSDEV dev )
 {
     assert(0);  /* should never be called */
@@ -311,6 +297,11 @@ static BOOL nulldrv_GdiRealizationInfo( PHYSDEV dev, void *info )
     return FALSE;
 }
 
+static UINT nulldrv_GetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
+{
+    return DCB_RESET;
+}
+
 static BOOL nulldrv_GetCharABCWidths( PHYSDEV dev, UINT first, UINT last, LPABC abc )
 {
     return FALSE;
@@ -380,6 +371,12 @@ static BOOL nulldrv_GetICMProfile( PHYSDEV dev, LPDWORD size, LPWSTR filename )
     return FALSE;
 }
 
+static DWORD nulldrv_GetImage( PHYSDEV dev, BITMAPINFO *info, struct gdi_image_bits *bits,
+                               struct bitblt_coords *src )
+{
+    return ERROR_NOT_SUPPORTED;
+}
+
 static DWORD nulldrv_GetKerningPairs( PHYSDEV dev, DWORD count, LPKERNINGPAIR pairs )
 {
     return 0;
@@ -390,7 +387,7 @@ static UINT nulldrv_GetOutlineTextMetrics( PHYSDEV dev, UINT size, LPOUTLINETEXT
     return 0;
 }
 
-static INT nulldrv_GetPixelFormat( PHYSDEV dev )
+static INT nulldrv_GetPixelFormat( HDC hdc )
 {
     return 0;
 }
@@ -491,6 +488,13 @@ static BOOL nulldrv_Polyline( PHYSDEV dev, const POINT *points, INT count )
     return PolyPolyline( dev->hdc, points, counts, 1 );
 }
 
+static DWORD nulldrv_PutImage( PHYSDEV dev, HRGN clip, BITMAPINFO *info,
+                               const struct gdi_image_bits *bits, struct bitblt_coords *src,
+                               struct bitblt_coords *dst, DWORD rop )
+{
+    return ERROR_SUCCESS;
+}
+
 static UINT nulldrv_RealizeDefaultPalette( PHYSDEV dev )
 {
     return 0;
@@ -557,6 +561,11 @@ static INT nulldrv_SetBkMode( PHYSDEV dev, INT mode )
     return mode;
 }
 
+static UINT nulldrv_SetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
+{
+    return DCB_RESET;
+}
+
 static COLORREF nulldrv_SetDCBrushColor( PHYSDEV dev, COLORREF color )
 {
     return color;
@@ -657,24 +666,29 @@ static BOOL nulldrv_UnrealizePalette( HPALETTE palette )
     return FALSE;
 }
 
-static BOOL nulldrv_wglCopyContext( HGLRC ctx_src, HGLRC ctx_dst, UINT mask )
+static BOOL nulldrv_wglCopyContext( struct wgl_context *src, struct wgl_context *dst, UINT mask )
 {
     return FALSE;
 }
 
-static HGLRC nulldrv_wglCreateContext( PHYSDEV dev )
+static struct wgl_context *nulldrv_wglCreateContext( HDC hdc )
 {
     return 0;
 }
 
-static HGLRC nulldrv_wglCreateContextAttribsARB( PHYSDEV dev, HGLRC share_ctx, const int *attribs )
+static struct wgl_context *nulldrv_wglCreateContextAttribsARB( HDC hdc, struct wgl_context *share_ctx,
+                                                               const int *attribs )
 {
     return 0;
 }
 
-static BOOL nulldrv_wglDeleteContext( HGLRC ctx )
+static void nulldrv_wglDeleteContext( struct wgl_context *context )
 {
-    return FALSE;
+}
+
+static HDC nulldrv_wglGetCurrentDC( struct wgl_context *context )
+{
+    return 0;
 }
 
 static PROC nulldrv_wglGetProcAddress( LPCSTR name )
@@ -682,39 +696,29 @@ static PROC nulldrv_wglGetProcAddress( LPCSTR name )
     return NULL;
 }
 
-static HDC nulldrv_wglGetPbufferDCARB( PHYSDEV dev, void *pbuffer )
-{
-    return 0;
-}
-
-static BOOL nulldrv_wglMakeCurrent( PHYSDEV dev, HGLRC ctx )
+static BOOL nulldrv_wglMakeContextCurrentARB( HDC draw_hdc, HDC read_hdc, struct wgl_context *context )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglMakeContextCurrentARB( PHYSDEV dev_draw, PHYSDEV dev_read, HGLRC ctx )
+static BOOL nulldrv_wglMakeCurrent( HDC hdc, struct wgl_context *context )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglSetPixelFormatWINE( PHYSDEV dev, INT format, const PIXELFORMATDESCRIPTOR *descr )
+static BOOL nulldrv_wglShareLists( struct wgl_context *org, struct wgl_context *dst )
 {
     return FALSE;
 }
 
-static BOOL nulldrv_wglShareLists( HGLRC ctx1, HGLRC ctx2 )
+static const struct wgl_funcs *nulldrv_wine_get_wgl_driver( PHYSDEV dev, UINT version )
 {
-    return FALSE;
-}
-
-static BOOL nulldrv_wglUseFontBitmapsA( PHYSDEV dev, DWORD start, DWORD count, DWORD base )
-{
-    return FALSE;
-}
-
-static BOOL nulldrv_wglUseFontBitmapsW( PHYSDEV dev, DWORD start, DWORD count, DWORD base )
-{
-    return FALSE;
+    if (version != WINE_GDI_DRIVER_VERSION)
+    {
+        ERR( "version mismatch, opengl32 wants %u but driver has %u\n", version, WINE_GDI_DRIVER_VERSION );
+        return NULL;
+    }
+    return &null_wgl_driver;
 }
 
 const struct gdi_dc_funcs null_driver =
@@ -727,14 +731,10 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_ArcTo,                      /* pArcTo */
     nulldrv_BeginPath,                  /* pBeginPath */
     nulldrv_BlendImage,                 /* pBlendImage */
-    nulldrv_ChoosePixelFormat,          /* pChoosePixelFormat */
     nulldrv_Chord,                      /* pChord */
     nulldrv_CloseFigure,                /* pCloseFigure */
-    nulldrv_CopyBitmap,                 /* pCopyBitmap */
-    nulldrv_CreateBitmap,               /* pCreateBitmap */
     nulldrv_CreateCompatibleDC,         /* pCreateCompatibleDC */
     nulldrv_CreateDC,                   /* pCreateDC */
-    nulldrv_DeleteBitmap,               /* pDeleteBitmap */
     nulldrv_DeleteDC,                   /* pDeleteDC */
     nulldrv_DeleteObject,               /* pDeleteObject */
     nulldrv_DescribePixelFormat,        /* pDescribePixelFormat */
@@ -758,6 +758,7 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_FrameRgn,                   /* pFrameRgn */
     nulldrv_GdiComment,                 /* pGdiComment */
     nulldrv_GdiRealizationInfo,         /* pGdiRealizationInfo */
+    nulldrv_GetBoundsRect,              /* pGetBoundsRect */
     nulldrv_GetCharABCWidths,           /* pGetCharABCWidths */
     nulldrv_GetCharABCWidthsI,          /* pGetCharABCWidthsI */
     nulldrv_GetCharWidth,               /* pGetCharWidth */
@@ -773,7 +774,6 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_GetNearestColor,            /* pGetNearestColor */
     nulldrv_GetOutlineTextMetrics,      /* pGetOutlineTextMetrics */
     nulldrv_GetPixel,                   /* pGetPixel */
-    nulldrv_GetPixelFormat,             /* pGetPixelFormat */
     nulldrv_GetSystemPaletteEntries,    /* pGetSystemPaletteEntries */
     nulldrv_GetTextCharsetInfo,         /* pGetTextCharsetInfo */
     nulldrv_GetTextExtentExPoint,       /* pGetTextExtentExPoint */
@@ -819,6 +819,7 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_SetArcDirection,            /* pSetArcDirection */
     nulldrv_SetBkColor,                 /* pSetBkColor */
     nulldrv_SetBkMode,                  /* pSetBkMode */
+    nulldrv_SetBoundsRect,              /* pSetBoundsRect */
     nulldrv_SetDCBrushColor,            /* pSetDCBrushColor */
     nulldrv_SetDCPenColor,              /* pSetDCPenColor */
     nulldrv_SetDIBitsToDevice,          /* pSetDIBitsToDevice */
@@ -851,20 +852,24 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_SwapBuffers,                /* pSwapBuffers */
     nulldrv_UnrealizePalette,           /* pUnrealizePalette */
     nulldrv_WidenPath,                  /* pWidenPath */
-    nulldrv_wglCopyContext,             /* pwglCopyContext */
-    nulldrv_wglCreateContext,           /* pwglCreateContext */
-    nulldrv_wglCreateContextAttribsARB, /* pwglCreateContextAttribsARB */
-    nulldrv_wglDeleteContext,           /* pwglDeleteContext */
-    nulldrv_wglGetPbufferDCARB,         /* pwglGetPbufferDCARB */
-    nulldrv_wglGetProcAddress,          /* pwglGetProcAddress */
-    nulldrv_wglMakeContextCurrentARB,   /* pwglMakeContextCurrentARB */
-    nulldrv_wglMakeCurrent,             /* pwglMakeCurrent */
-    nulldrv_wglSetPixelFormatWINE,      /* pwglSetPixelFormatWINE */
-    nulldrv_wglShareLists,              /* pwglShareLists */
-    nulldrv_wglUseFontBitmapsA,         /* pwglUseFontBitmapsA */
-    nulldrv_wglUseFontBitmapsW,         /* pwglUseFontBitmapsW */
+    nulldrv_wine_get_wgl_driver,        /* wine_get_wgl_driver */
+
+    GDI_PRIORITY_NULL_DRV               /* priority */
 };
 
+static const struct wgl_funcs null_wgl_driver =
+{
+    nulldrv_GetPixelFormat,             /* p_GetPixelFormat */
+    nulldrv_wglCopyContext,             /* p_wglCopyContext */
+    nulldrv_wglCreateContext,           /* p_wglCreateContext */
+    nulldrv_wglCreateContextAttribsARB, /* p_wglCreateContextAttribsARB */
+    nulldrv_wglDeleteContext,           /* p_wglDeleteContext */
+    nulldrv_wglGetCurrentDC,            /* p_wglGetCurrentDC */
+    nulldrv_wglGetProcAddress,          /* p_wglGetProcAddress */
+    nulldrv_wglMakeContextCurrentARB,   /* p_wglMakeContextCurrentARB */
+    nulldrv_wglMakeCurrent,             /* p_wglMakeCurrent */
+    nulldrv_wglShareLists,              /* p_wglShareLists */
+};
 
 /*****************************************************************************
  *      DRIVER_GetDriverName
