@@ -44,14 +44,10 @@ static const struct gdi_dc_funcs EMFDRV_Funcs =
     NULL,                            /* pArcTo */
     EMFDRV_BeginPath,                /* pBeginPath */
     NULL,                            /* pBlendImage */
-    NULL,                            /* pChoosePixelFormat */
     EMFDRV_Chord,                    /* pChord */
     EMFDRV_CloseFigure,              /* pCloseFigure */
-    NULL,                            /* pCopyBitmap */
-    NULL,                            /* pCreateBitmap */
     NULL,                            /* pCreateCompatibleDC */
     NULL,                            /* pCreateDC */
-    NULL,                            /* pDeleteBitmap */
     EMFDRV_DeleteDC,                 /* pDeleteDC */
     EMFDRV_DeleteObject,             /* pDeleteObject */
     NULL,                            /* pDescribePixelFormat */
@@ -75,6 +71,7 @@ static const struct gdi_dc_funcs EMFDRV_Funcs =
     EMFDRV_FrameRgn,                 /* pFrameRgn */
     EMFDRV_GdiComment,               /* pGdiComment */
     NULL,                            /* pGdiRealizationInfo */
+    NULL,                            /* pGetBoundsRect */
     NULL,                            /* pGetCharABCWidths */
     NULL,                            /* pGetCharABCWidthsI */
     NULL,                            /* pGetCharWidth */
@@ -90,7 +87,6 @@ static const struct gdi_dc_funcs EMFDRV_Funcs =
     NULL,                            /* pGetNearestColor */
     NULL,                            /* pGetOutlineTextMetrics */
     NULL,                            /* pGetPixel */
-    NULL,                            /* pGetPixelFormat */
     NULL,                            /* pGetSystemPaletteEntries */
     NULL,                            /* pGetTextCharsetInfo */
     NULL,                            /* pGetTextExtentExPoint */
@@ -136,6 +132,7 @@ static const struct gdi_dc_funcs EMFDRV_Funcs =
     EMFDRV_SetArcDirection,          /* pSetArcDirection */
     EMFDRV_SetBkColor,               /* pSetBkColor */
     EMFDRV_SetBkMode,                /* pSetBkMode */
+    NULL,                            /* pSetBoundsRect */
     EMFDRV_SetDCBrushColor,          /* pSetDCBrushColor*/
     EMFDRV_SetDCPenColor,            /* pSetDCPenColor*/
     EMFDRV_SetDIBitsToDevice,        /* pSetDIBitsToDevice */
@@ -167,7 +164,9 @@ static const struct gdi_dc_funcs EMFDRV_Funcs =
     EMFDRV_StrokePath,               /* pStrokePath */
     NULL,                            /* pSwapBuffers */
     NULL,                            /* pUnrealizePalette */
-    EMFDRV_WidenPath                 /* pWidenPath */
+    EMFDRV_WidenPath,                /* pWidenPath */
+    NULL,                            /* wine_get_wgl_driver */
+    GDI_PRIORITY_GRAPHICS_DRV        /* priority */
 };
 
 
@@ -317,8 +316,6 @@ HDC WINAPI CreateEnhMetaFileW(
     static const WCHAR displayW[] = {'D','I','S','P','L','A','Y',0};
     HDC ret;
     DC *dc;
-    HDC hRefDC = hdc ? hdc : CreateDCW(displayW,NULL,NULL,NULL); 
-        /* If no ref, use current display */
     EMFDRV_PDEVICE *physDev;
     HANDLE hFile;
     DWORD size = 0, length = 0;
@@ -354,19 +351,12 @@ HDC WINAPI CreateEnhMetaFileW(
     physDev->hFile = 0;
     physDev->dc_brush = 0;
     physDev->dc_pen = 0;
-    physDev->horzres = GetDeviceCaps(hRefDC, HORZRES);
-    physDev->vertres = GetDeviceCaps(hRefDC, VERTRES);
-    physDev->logpixelsx = GetDeviceCaps(hRefDC, LOGPIXELSX);
-    physDev->logpixelsy = GetDeviceCaps(hRefDC, LOGPIXELSY);
-    physDev->horzsize = GetDeviceCaps(hRefDC, HORZSIZE);
-    physDev->vertsize = GetDeviceCaps(hRefDC, VERTSIZE);
-    physDev->bitspixel = GetDeviceCaps(hRefDC, BITSPIXEL);
-    physDev->textcaps = GetDeviceCaps(hRefDC, TEXTCAPS);
-    physDev->rastercaps = GetDeviceCaps(hRefDC, RASTERCAPS);
-    physDev->technology = GetDeviceCaps(hRefDC, TECHNOLOGY);
-    physDev->planes = GetDeviceCaps(hRefDC, PLANES);
-    physDev->numcolors = GetDeviceCaps(hRefDC, NUMCOLORS);
+    physDev->screen_dc = 0;
     physDev->restoring = 0;
+    if (hdc)  /* if no ref, use current display */
+        physDev->ref_dc = hdc;
+    else
+        physDev->ref_dc = physDev->screen_dc = CreateDCW( displayW, NULL, NULL, NULL );
 
     SetVirtualResolution(physDev->dev.hdc, 0, 0, 0, 0);
 
@@ -400,16 +390,16 @@ HDC WINAPI CreateEnhMetaFileW(
     physDev->emh->nPalEntries = 0; /* I guess this should start at 0 */
 
     /* Size in pixels */
-    physDev->emh->szlDevice.cx = physDev->horzres;
-    physDev->emh->szlDevice.cy = physDev->vertres;
+    physDev->emh->szlDevice.cx = GetDeviceCaps( physDev->ref_dc, HORZRES );
+    physDev->emh->szlDevice.cy = GetDeviceCaps( physDev->ref_dc, VERTRES );
 
     /* Size in millimeters */
-    physDev->emh->szlMillimeters.cx = physDev->horzsize;
-    physDev->emh->szlMillimeters.cy = physDev->vertsize;
+    physDev->emh->szlMillimeters.cx = GetDeviceCaps( physDev->ref_dc, HORZSIZE );
+    physDev->emh->szlMillimeters.cy = GetDeviceCaps( physDev->ref_dc, VERTSIZE );
 
     /* Size in micrometers */
-    physDev->emh->szlMicrometers.cx = physDev->horzsize * 1000;
-    physDev->emh->szlMicrometers.cy = physDev->vertsize * 1000;
+    physDev->emh->szlMicrometers.cx = physDev->emh->szlMillimeters.cx * 1000;
+    physDev->emh->szlMicrometers.cy = physDev->emh->szlMillimeters.cy * 1000;
 
     memcpy((char *)physDev->emh + sizeof(ENHMETAHEADER), description, length);
 
@@ -431,9 +421,6 @@ HDC WINAPI CreateEnhMetaFileW(
     ret = physDev->dev.hdc;
     release_dc_ptr( dc );
 
-    if( !hdc )
-      DeleteDC( hRefDC );
-
     return ret;
 }
 
@@ -469,6 +456,7 @@ HENHMETAFILE WINAPI CloseEnhMetaFile(HDC hdc) /* [in] metafile DC */
 
     if (physDev->dc_brush) DeleteObject( physDev->dc_brush );
     if (physDev->dc_pen) DeleteObject( physDev->dc_pen );
+    if (physDev->screen_dc) DeleteDC( physDev->screen_dc );
 
     emr.emr.iType = EMR_EOF;
     emr.emr.nSize = sizeof(emr);
