@@ -59,6 +59,24 @@ static INT MFDRV_ExtEscape( PHYSDEV dev, INT nEscape, INT cbInput, LPCVOID in_da
 }
 
 
+/******************************************************************
+ *         MFDRV_GetBoundsRect
+ */
+static UINT MFDRV_GetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
+{
+    return 0;
+}
+
+
+/******************************************************************
+ *         MFDRV_SetBoundsRect
+ */
+static UINT MFDRV_SetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
+{
+    return 0;
+}
+
+
 /******************************************************************
  *         MFDRV_GetDeviceCaps
  *
@@ -89,14 +107,10 @@ static const struct gdi_dc_funcs MFDRV_Funcs =
     NULL,                            /* pArcTo */
     MFDRV_BeginPath,                 /* pBeginPath */
     NULL,                            /* pBlendImage */
-    NULL,                            /* pChoosePixelFormat */
     MFDRV_Chord,                     /* pChord */
     MFDRV_CloseFigure,               /* pCloseFigure */
-    NULL,                            /* pCopyBitmap */
-    NULL,                            /* pCreateBitmap */
     MFDRV_CreateCompatibleDC,        /* pCreateCompatibleDC */
     NULL,                            /* pCreateDC */
-    NULL,                            /* pDeleteBitmap */
     MFDRV_DeleteDC,                  /* pDeleteDC */
     MFDRV_DeleteObject,              /* pDeleteObject */
     NULL,                            /* pDescribePixelFormat */
@@ -120,6 +134,7 @@ static const struct gdi_dc_funcs MFDRV_Funcs =
     MFDRV_FrameRgn,                  /* pFrameRgn */
     NULL,                            /* pGdiComment */
     NULL,                            /* pGdiRealizationInfo */
+    MFDRV_GetBoundsRect,             /* pGetBoundsRect */
     NULL,                            /* pGetCharABCWidths */
     NULL,                            /* pGetCharABCWidthsI */
     NULL,                            /* pGetCharWidth */
@@ -135,7 +150,6 @@ static const struct gdi_dc_funcs MFDRV_Funcs =
     NULL,                            /* pGetNearestColor */
     NULL,                            /* pGetOutlineTextMetrics */
     NULL,                            /* pGetPixel */
-    NULL,                            /* pGetPixelFormat */
     NULL,                            /* pGetSystemPaletteEntries */
     NULL,                            /* pGetTextCharsetInfo */
     NULL,                            /* pGetTextExtentExPoint */
@@ -181,6 +195,7 @@ static const struct gdi_dc_funcs MFDRV_Funcs =
     NULL,                            /* pSetArcDirection */
     MFDRV_SetBkColor,                /* pSetBkColor */
     MFDRV_SetBkMode,                 /* pSetBkMode */
+    MFDRV_SetBoundsRect,             /* pSetBoundsRect */
     MFDRV_SetDCBrushColor,           /* pSetDCBrushColor*/
     MFDRV_SetDCPenColor,             /* pSetDCPenColor*/
     MFDRV_SetDIBitsToDevice,         /* pSetDIBitsToDevice */
@@ -212,7 +227,9 @@ static const struct gdi_dc_funcs MFDRV_Funcs =
     MFDRV_StrokePath,                /* pStrokePath */
     NULL,                            /* pSwapBuffers */
     NULL,                            /* pUnrealizePalette */
-    MFDRV_WidenPath                  /* pWidenPath */
+    MFDRV_WidenPath,                 /* pWidenPath */
+    NULL,                            /* wine_get_wgl_driver */
+    GDI_PRIORITY_GRAPHICS_DRV        /* priority */
 };
 
 
