@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2007 Google (Evan Stade)
+ * Copyright (C) 2012 Dmitry Timoshkov
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,42 +34,123 @@ WINE_DEFAULT_DEBUG_CHANNEL (gdiplus);
 #include "gdiplus.h"
 #include "gdiplus_private.h"
 
+/* PANOSE is 10 bytes in size, need to pack the structure properly */
+#include "pshpack2.h"
+typedef struct
+{
+    USHORT version;
+    SHORT xAvgCharWidth;
+    USHORT usWeightClass;
+    USHORT usWidthClass;
+    SHORT fsType;
+    SHORT ySubscriptXSize;
+    SHORT ySubscriptYSize;
+    SHORT ySubscriptXOffset;
+    SHORT ySubscriptYOffset;
+    SHORT ySuperscriptXSize;
+    SHORT ySuperscriptYSize;
+    SHORT ySuperscriptXOffset;
+    SHORT ySuperscriptYOffset;
+    SHORT yStrikeoutSize;
+    SHORT yStrikeoutPosition;
+    SHORT sFamilyClass;
+    PANOSE panose;
+    ULONG ulUnicodeRange1;
+    ULONG ulUnicodeRange2;
+    ULONG ulUnicodeRange3;
+    ULONG ulUnicodeRange4;
+    CHAR achVendID[4];
+    USHORT fsSelection;
+    USHORT usFirstCharIndex;
+    USHORT usLastCharIndex;
+    /* According to the Apple spec, original version didn't have the below fields,
+     * version numbers were taken from the OpenType spec.
+     */
+    /* version 0 (TrueType 1.5) */
+    USHORT sTypoAscender;
+    USHORT sTypoDescender;
+    USHORT sTypoLineGap;
+    USHORT usWinAscent;
+    USHORT usWinDescent;
+    /* version 1 (TrueType 1.66) */
+    ULONG ulCodePageRange1;
+    ULONG ulCodePageRange2;
+    /* version 2 (OpenType 1.2) */
+    SHORT sxHeight;
+    SHORT sCapHeight;
+    USHORT usDefaultChar;
+    USHORT usBreakChar;
+    USHORT usMaxContext;
+} TT_OS2_V2;
+
+typedef struct
+{
+    ULONG Version;
+    SHORT Ascender;
+    SHORT Descender;
+    SHORT LineGap;
+    USHORT advanceWidthMax;
+    SHORT minLeftSideBearing;
+    SHORT minRightSideBearing;
+    SHORT xMaxExtent;
+    SHORT caretSlopeRise;
+    SHORT caretSlopeRun;
+    SHORT caretOffset;
+    SHORT reserved[4];
+    SHORT metricDataFormat;
+    USHORT numberOfHMetrics;
+} TT_HHEA;
+#include "poppack.h"
+
+#ifdef WORDS_BIGENDIAN
+#define GET_BE_WORD(x) (x)
+#define GET_BE_DWORD(x) (x)
+#else
+#define GET_BE_WORD(x) MAKEWORD(HIBYTE(x), LOBYTE(x))
+#define GET_BE_DWORD(x) MAKELONG(GET_BE_WORD(HIWORD(x)), GET_BE_WORD(LOWORD(x)));
+#endif
+
+#define MS_MAKE_TAG(ch0, ch1, ch2, ch3) \
+                    ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8) | \
+                    ((DWORD)(BYTE)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24))
+#define MS_OS2_TAG MS_MAKE_TAG('O','S','/','2')
+#define MS_HHEA_TAG MS_MAKE_TAG('h','h','e','a')
+
+static GpStatus clone_font_family(const GpFontFamily *, GpFontFamily **);
+
 static const REAL mm_per_inch = 25.4;
 static const REAL inch_per_point = 1.0/72.0;
 
 static GpFontCollection installedFontCollection = {0};
 
-static inline REAL get_dpi (void)
+static LONG em_size_to_pixel(REAL em_size, Unit unit, LONG dpi)
 {
-    REAL dpi;
-    GpGraphics *graphics;
-    HDC hdc = GetDC(0);
-    GdipCreateFromHDC (hdc, &graphics);
-    GdipGetDpiX(graphics, &dpi);
-    GdipDeleteGraphics(graphics);
-    ReleaseDC (0, hdc);
+    switch (unit)
+    {
+    default:
+        FIXME("Unhandled unit type: %d\n", unit);
+        return 0;
 
-    return dpi;
-}
-
-static inline REAL point_to_pixel (REAL point)
-{
-    return point * get_dpi() * inch_per_point;
-}
-
-static inline REAL inch_to_pixel (REAL inch)
-{
-    return inch * get_dpi();
-}
-
-static inline REAL document_to_pixel (REAL doc)
-{
-    return doc * (get_dpi() / 300.0); /* Per MSDN */
-}
-
-static inline REAL mm_to_pixel (REAL mm)
-{
-    return mm * (get_dpi() / mm_per_inch);
+    case UnitPixel:
+    case UnitWorld:
+        /* FIXME: Figure out when World != Pixel */
+        return em_size;
+    case UnitDisplay:
+        FIXME("Unknown behavior for UnitDisplay! Please report!\n");
+        /* FIXME: Figure out how this works...
+         * MSDN says that if "DISPLAY" is a monitor, then pixel should be
+         * used. That's not what I got. Tests on Windows revealed no output,
+         * and the tests in tests/font crash windows */
+        return 0;
+    case UnitPoint:
+        return em_size * dpi * inch_per_point;
+    case UnitInch:
+        return em_size * dpi;
+    case UnitDocument:
+        return em_size * dpi / 300.0; /* Per MSDN */
+    case UnitMillimeter:
+        return em_size * dpi / mm_per_inch;
+    }
 }
 
 /*******************************************************************************
@@ -95,68 +177,53 @@ static inline REAL mm_to_pixel (REAL mm)
 GpStatus WINGDIPAPI GdipCreateFont(GDIPCONST GpFontFamily *fontFamily,
                         REAL emSize, INT style, Unit unit, GpFont **font)
 {
-    WCHAR facename[LF_FACESIZE];
-    LOGFONTW* lfw;
-    const NEWTEXTMETRICW* tmw;
+    HFONT hfont;
+    OUTLINETEXTMETRICW otm;
+    LOGFONTW lfw;
+    HDC hdc;
     GpStatus stat;
+    int ret;
 
-    if (!fontFamily || !font)
+    if (!fontFamily || !font || emSize < 0.0)
         return InvalidParameter;
 
     TRACE("%p (%s), %f, %d, %d, %p\n", fontFamily,
             debugstr_w(fontFamily->FamilyName), emSize, style, unit, font);
 
-    stat = GdipGetFamilyName (fontFamily, facename, 0);
+    memset(&lfw, 0, sizeof(lfw));
+
+    stat = GdipGetFamilyName(fontFamily, lfw.lfFaceName, LANG_NEUTRAL);
     if (stat != Ok) return stat;
+
+    lfw.lfHeight = -em_size_to_pixel(emSize, unit, fontFamily->dpi);
+    lfw.lfWeight = style & FontStyleBold ? FW_BOLD : FW_REGULAR;
+    lfw.lfItalic = style & FontStyleItalic;
+    lfw.lfUnderline = style & FontStyleUnderline;
+    lfw.lfStrikeOut = style & FontStyleStrikeout;
+
+    hfont = CreateFontIndirectW(&lfw);
+    hdc = CreateCompatibleDC(0);
+    SelectObject(hdc, hfont);
+    otm.otmSize = sizeof(otm);
+    ret = GetOutlineTextMetricsW(hdc, otm.otmSize, &otm);
+    DeleteDC(hdc);
+    DeleteObject(hfont);
+
+    if (!ret) return NotTrueTypeFont;
+
     *font = GdipAlloc(sizeof(GpFont));
-
-    tmw = &fontFamily->tmw;
-    lfw = &((*font)->lfw);
-    ZeroMemory(&(*lfw), sizeof(*lfw));
-
-    lfw->lfWeight = tmw->tmWeight;
-    lfw->lfItalic = tmw->tmItalic;
-    lfw->lfUnderline = tmw->tmUnderlined;
-    lfw->lfStrikeOut = tmw->tmStruckOut;
-    lfw->lfCharSet = tmw->tmCharSet;
-    lfw->lfPitchAndFamily = tmw->tmPitchAndFamily;
-    lstrcpynW(lfw->lfFaceName, facename, LF_FACESIZE);
-
-    switch (unit)
-    {
-        case UnitWorld:
-            /* FIXME: Figure out when World != Pixel */
-            (*font)->pixel_size = emSize; break;
-        case UnitDisplay:
-            FIXME("Unknown behavior for UnitDisplay! Please report!\n");
-            /* FIXME: Figure out how this works...
-             * MSDN says that if "DISPLAY" is a monitor, then pixel should be
-             * used. That's not what I got. Tests on Windows revealed no output,
-             * and the tests in tests/font crash windows */
-            (*font)->pixel_size = 0; break;
-        case UnitPixel:
-            (*font)->pixel_size = emSize; break;
-        case UnitPoint:
-            (*font)->pixel_size = point_to_pixel(emSize); break;
-        case UnitInch:
-            (*font)->pixel_size = inch_to_pixel(emSize); break;
-        case UnitDocument:
-            (*font)->pixel_size = document_to_pixel(emSize); break;
-        case UnitMillimeter:
-            (*font)->pixel_size = mm_to_pixel(emSize); break;
-    }
-
-    lfw->lfHeight = (*font)->pixel_size * -1;
-
-    lfw->lfWeight = style & FontStyleBold ? 700 : 400;
-    lfw->lfItalic = style & FontStyleItalic;
-    lfw->lfUnderline = style & FontStyleUnderline;
-    lfw->lfStrikeOut = style & FontStyleStrikeout;
+    if (!*font) return OutOfMemory;
 
     (*font)->unit = unit;
     (*font)->emSize = emSize;
-    (*font)->height = tmw->ntmSizeEM;
-    (*font)->line_spacing = tmw->tmAscent + tmw->tmDescent + tmw->tmExternalLeading;
+    (*font)->otm = otm;
+
+    stat = clone_font_family(fontFamily, &(*font)->family);
+    if (stat != Ok)
+    {
+        GdipFree(*font);
+        return stat;
+    }
 
     TRACE("<-- %p\n", *font);
 
@@ -170,43 +237,38 @@ GpStatus WINGDIPAPI GdipCreateFontFromLogfontW(HDC hdc,
     GDIPCONST LOGFONTW *logfont, GpFont **font)
 {
     HFONT hfont, oldfont;
-    TEXTMETRICW textmet;
+    OUTLINETEXTMETRICW otm;
+    GpStatus stat;
+    int ret;
 
     TRACE("(%p, %p, %p)\n", hdc, logfont, font);
 
-    if(!logfont || !font)
+    if (!hdc || !logfont || !font)
         return InvalidParameter;
 
-    if (logfont->lfFaceName[0] == 0)
-        return NotTrueTypeFont;
-
-    *font = GdipAlloc(sizeof(GpFont));
-    if(!*font)  return OutOfMemory;
-
-    memcpy((*font)->lfw.lfFaceName, logfont->lfFaceName, LF_FACESIZE *
-           sizeof(WCHAR));
-    (*font)->lfw.lfHeight = logfont->lfHeight;
-    (*font)->lfw.lfItalic = logfont->lfItalic;
-    (*font)->lfw.lfUnderline = logfont->lfUnderline;
-    (*font)->lfw.lfStrikeOut = logfont->lfStrikeOut;
-
-    (*font)->pixel_size = (*font)->emSize = logfont->lfHeight;
-    (*font)->unit = UnitPixel;
-
-    hfont = CreateFontIndirectW(&(*font)->lfw);
+    hfont = CreateFontIndirectW(logfont);
     oldfont = SelectObject(hdc, hfont);
-    GetTextMetricsW(hdc, &textmet);
-
-    (*font)->lfw.lfHeight = -(textmet.tmHeight-textmet.tmInternalLeading);
-    (*font)->lfw.lfWeight = textmet.tmWeight;
-    (*font)->lfw.lfCharSet = textmet.tmCharSet;
-
-    (*font)->height = 1; /* FIXME: need NEWTEXTMETRIC.ntmSizeEM here */
-    (*font)->line_spacing = textmet.tmAscent + textmet.tmDescent + textmet.tmExternalLeading;
-
+    otm.otmSize = sizeof(otm);
+    ret = GetOutlineTextMetricsW(hdc, otm.otmSize, &otm);
     SelectObject(hdc, oldfont);
     DeleteObject(hfont);
 
+    if (!ret) return NotTrueTypeFont;
+
+    *font = GdipAlloc(sizeof(GpFont));
+    if (!*font) return OutOfMemory;
+
+    (*font)->unit = UnitWorld;
+    (*font)->emSize = otm.otmTextMetrics.tmAscent;
+    (*font)->otm = otm;
+
+    stat = GdipCreateFontFamilyFromName(logfont->lfFaceName, NULL, &(*font)->family);
+    if (stat != Ok)
+    {
+        GdipFree(*font);
+        return NotTrueTypeFont;
+    }
+
     TRACE("<-- %p\n", *font);
 
     return Ok;
@@ -243,6 +305,7 @@ GpStatus WINGDIPAPI GdipDeleteFont(GpFont* font)
     if(!font)
         return InvalidParameter;
 
+    GdipDeleteFontFamily(font->family);
     GdipFree(font);
 
     return Ok;
@@ -291,7 +354,12 @@ GpStatus WINGDIPAPI GdipGetFamily(GpFont *font, GpFontFamily **family)
     if (!(font && family))
         return InvalidParameter;
 
-    return GdipCreateFontFamilyFromName(font->lfw.lfFaceName, NULL, family);
+    return GdipCloneFontFamily(font->family, family);
+}
+
+static REAL get_font_size(const GpFont *font)
+{
+    return font->emSize;
 }
 
 /******************************************************************************
@@ -316,11 +384,30 @@ GpStatus WINGDIPAPI GdipGetFontSize(GpFont *font, REAL *size)
 
     if (!(font && size)) return InvalidParameter;
 
-    *size = font->emSize;
+    *size = get_font_size(font);
+    TRACE("%s,%d => %f\n", debugstr_w(font->family->FamilyName), font->otm.otmTextMetrics.tmHeight, *size);
 
     return Ok;
 }
 
+static INT get_font_style(const GpFont *font)
+{
+    INT style;
+
+    if (font->otm.otmTextMetrics.tmWeight > FW_REGULAR)
+        style = FontStyleBold;
+    else
+        style = FontStyleRegular;
+    if (font->otm.otmTextMetrics.tmItalic)
+        style |= FontStyleItalic;
+    if (font->otm.otmTextMetrics.tmUnderlined)
+        style |= FontStyleUnderline;
+    if (font->otm.otmTextMetrics.tmStruckOut)
+        style |= FontStyleStrikeout;
+
+    return style;
+}
+
 /*******************************************************************************
  * GdipGetFontStyle [GDIPLUS.@]
  *
@@ -341,16 +428,8 @@ GpStatus WINGDIPAPI GdipGetFontStyle(GpFont *font, INT *style)
     if (!(font && style))
         return InvalidParameter;
 
-    if (font->lfw.lfWeight > 400)
-        *style = FontStyleBold;
-    else
-        *style = 0;
-    if (font->lfw.lfItalic)
-        *style |= FontStyleItalic;
-    if (font->lfw.lfUnderline)
-        *style |= FontStyleUnderline;
-    if (font->lfw.lfStrikeOut)
-        *style |= FontStyleStrikeout;
+    *style = get_font_style(font);
+    TRACE("%s,%d => %d\n", debugstr_w(font->family->FamilyName), font->otm.otmTextMetrics.tmHeight, *style);
 
     return Ok;
 }
@@ -373,6 +452,7 @@ GpStatus WINGDIPAPI GdipGetFontUnit(GpFont *font, Unit *unit)
     if (!(font && unit)) return InvalidParameter;
 
     *unit = font->unit;
+    TRACE("%s,%d => %d\n", debugstr_w(font->family->FamilyName), font->otm.otmTextMetrics.tmHeight, *unit);
 
     return Ok;
 }
@@ -400,6 +480,25 @@ GpStatus WINGDIPAPI GdipGetLogFontA(GpFont *font, GpGraphics *graphics,
     return Ok;
 }
 
+void get_log_fontW(const GpFont *font, GpGraphics *graphics, LOGFONTW *lf)
+{
+    /* FIXME: use graphics */
+    lf->lfHeight = -em_size_to_pixel(font->emSize, font->unit, font->family->dpi);
+    lf->lfWidth = 0;
+    lf->lfEscapement = 0;
+    lf->lfOrientation = 0;
+    lf->lfWeight = font->otm.otmTextMetrics.tmWeight;
+    lf->lfItalic = font->otm.otmTextMetrics.tmItalic ? 1 : 0;
+    lf->lfUnderline = font->otm.otmTextMetrics.tmUnderlined ? 1 : 0;
+    lf->lfStrikeOut = font->otm.otmTextMetrics.tmStruckOut ? 1 : 0;
+    lf->lfCharSet = font->otm.otmTextMetrics.tmCharSet;
+    lf->lfOutPrecision = OUT_DEFAULT_PRECIS;
+    lf->lfClipPrecision = CLIP_DEFAULT_PRECIS;
+    lf->lfQuality = DEFAULT_QUALITY;
+    lf->lfPitchAndFamily = 0;
+    strcpyW(lf->lfFaceName, font->family->FamilyName);
+}
+
 /*******************************************************************************
  * GdipGetLogFontW [GDIPLUS.@]
  */
@@ -408,11 +507,11 @@ GpStatus WINGDIPAPI GdipGetLogFontW(GpFont *font, GpGraphics *graphics,
 {
     TRACE("(%p, %p, %p)\n", font, graphics, lfw);
 
-    /* FIXME: use graphics */
     if(!font || !graphics || !lfw)
         return InvalidParameter;
 
-    *lfw = font->lfw;
+    get_log_fontW(font, graphics, lfw);
+    TRACE("=> %s,%d\n", debugstr_w(lfw->lfFaceName), lfw->lfHeight);
 
     return Ok;
 }
@@ -422,6 +521,8 @@ GpStatus WINGDIPAPI GdipGetLogFontW(GpFont *font, GpGraphics *graphics,
  */
 GpStatus WINGDIPAPI GdipCloneFont(GpFont *font, GpFont **cloneFont)
 {
+    GpStatus stat;
+
     TRACE("(%p, %p)\n", font, cloneFont);
 
     if(!font || !cloneFont)
@@ -431,8 +532,10 @@ GpStatus WINGDIPAPI GdipCloneFont(GpFont *font, GpFont **cloneFont)
     if(!*cloneFont)    return OutOfMemory;
 
     **cloneFont = *font;
+    stat = GdipCloneFontFamily(font->family, &(*cloneFont)->family);
+    if (stat != Ok) GdipFree(*cloneFont);
 
-    return Ok;
+    return stat;
 }
 
 /*******************************************************************************
@@ -456,12 +559,15 @@ GpStatus WINGDIPAPI GdipGetFontHeight(GDIPCONST GpFont *font,
 
     TRACE("%p %p %p\n", font, graphics, height);
 
-    stat = GdipGetDpiY((GpGraphics*)graphics, &dpi);
+    if (graphics)
+    {
+        stat = GdipGetDpiY((GpGraphics*)graphics, &dpi);
+        if (stat != Ok) return stat;
+    }
+    else
+        dpi = font->family->dpi;
 
-    if (stat == Ok)
-        stat = GdipGetFontHeightGivenDPI(font, dpi, height);
-
-    return stat;
+    return GdipGetFontHeightGivenDPI(font, dpi, height);
 }
 
 /*******************************************************************************
@@ -481,14 +587,24 @@ GpStatus WINGDIPAPI GdipGetFontHeight(GDIPCONST GpFont *font,
  */
 GpStatus WINGDIPAPI GdipGetFontHeightGivenDPI(GDIPCONST GpFont *font, REAL dpi, REAL *height)
 {
-    REAL font_height;
+    GpStatus stat;
+    INT style;
+    UINT16 line_spacing, em_height;
+    REAL font_height, font_size;
+
+    if (!font || !height) return InvalidParameter;
 
     TRACE("%p (%s), %f, %p\n", font,
-            debugstr_w(font->lfw.lfFaceName), dpi, height);
+            debugstr_w(font->family->FamilyName), dpi, height);
 
-    if (!(font && height)) return InvalidParameter;
+    font_size = get_font_size(font);
+    style = get_font_style(font);
+    stat = GdipGetLineSpacing(font->family, style, &line_spacing);
+    if (stat != Ok) return stat;
+    stat = GdipGetEmHeight(font->family, style, &em_height);
+    if (stat != Ok) return stat;
 
-    font_height = font->line_spacing * (font->emSize / font->height);
+    font_height = (REAL)line_spacing * font_size / (REAL)em_height;
 
     switch (font->unit)
     {
@@ -513,6 +629,9 @@ GpStatus WINGDIPAPI GdipGetFontHeightGivenDPI(GDIPCONST GpFont *font, REAL dpi,
             return NotImplemented;
     }
 
+    TRACE("%s,%d(unit %d) => %f\n",
+          debugstr_w(font->family->FamilyName), font->otm.otmTextMetrics.tmHeight, font->unit, *height);
+
     return Ok;
 }
 
@@ -525,25 +644,87 @@ GpStatus WINGDIPAPI GdipGetFontHeightGivenDPI(GDIPCONST GpFont *font, REAL dpi,
 static INT CALLBACK is_font_installed_proc(const LOGFONTW *elf,
                             const TEXTMETRICW *ntm, DWORD type, LPARAM lParam)
 {
-    if (!ntm || type == RASTER_FONTTYPE)
-    {
+    if (type & RASTER_FONTTYPE)
         return 1;
-    }
 
-    *(NEWTEXTMETRICW*)lParam = *(const NEWTEXTMETRICW*)ntm;
+    *(LOGFONTW *)lParam = *elf;
 
     return 0;
 }
 
-static BOOL find_installed_font(const WCHAR *name, NEWTEXTMETRICW *ntm)
+struct font_metrics
 {
-    HDC hdc = GetDC(0);
-    BOOL ret = FALSE;
+    UINT16 em_height, ascent, descent, line_spacing; /* in font units */
+    int dpi;
+};
 
-    if(!EnumFontFamiliesW(hdc, name, is_font_installed_proc, (LPARAM)ntm))
-        ret = TRUE;
+static BOOL get_font_metrics(HDC hdc, struct font_metrics *fm)
+{
+    OUTLINETEXTMETRICW otm;
+    TT_OS2_V2 tt_os2;
+    TT_HHEA tt_hori;
+    LONG size;
+    UINT16 line_gap;
 
-    ReleaseDC(0, hdc);
+    otm.otmSize = sizeof(otm);
+    if (!GetOutlineTextMetricsW(hdc, otm.otmSize, &otm)) return FALSE;
+
+    fm->em_height = otm.otmEMSquare;
+    fm->dpi = GetDeviceCaps(hdc, LOGPIXELSY);
+
+    memset(&tt_hori, 0, sizeof(tt_hori));
+    if (GetFontData(hdc, MS_HHEA_TAG, 0, &tt_hori, sizeof(tt_hori)) != GDI_ERROR)
+    {
+        fm->ascent = GET_BE_WORD(tt_hori.Ascender);
+        fm->descent = -GET_BE_WORD(tt_hori.Descender);
+        TRACE("hhea: ascent %d, descent %d\n", fm->ascent, fm->descent);
+        line_gap = GET_BE_WORD(tt_hori.LineGap);
+        fm->line_spacing = fm->ascent + fm->descent + line_gap;
+        TRACE("line_gap %u, line_spacing %u\n", line_gap, fm->line_spacing);
+        if (fm->ascent + fm->descent != 0) return TRUE;
+    }
+
+    size = GetFontData(hdc, MS_OS2_TAG, 0, NULL, 0);
+    if (size == GDI_ERROR) return FALSE;
+
+    if (size > sizeof(tt_os2)) size = sizeof(tt_os2);
+
+    memset(&tt_os2, 0, sizeof(tt_os2));
+    if (GetFontData(hdc, MS_OS2_TAG, 0, &tt_os2, size) != size) return FALSE;
+
+    fm->ascent = GET_BE_WORD(tt_os2.usWinAscent);
+    fm->descent = GET_BE_WORD(tt_os2.usWinDescent);
+    TRACE("usWinAscent %u, usWinDescent %u\n", fm->ascent, fm->descent);
+    if (fm->ascent + fm->descent == 0)
+    {
+        fm->ascent = GET_BE_WORD(tt_os2.sTypoAscender);
+        fm->descent = GET_BE_WORD(tt_os2.sTypoDescender);
+        TRACE("sTypoAscender %u, sTypoDescender %u\n", fm->ascent, fm->descent);
+    }
+    line_gap = GET_BE_WORD(tt_os2.sTypoLineGap);
+    fm->line_spacing = fm->ascent + fm->descent + line_gap;
+    TRACE("line_gap %u, line_spacing %u\n", line_gap, fm->line_spacing);
+    return TRUE;
+}
+
+static GpStatus find_installed_font(const WCHAR *name, struct font_metrics *fm)
+{
+    LOGFONTW lf;
+    HDC hdc = CreateCompatibleDC(0);
+    GpStatus ret = FontFamilyNotFound;
+
+    if(!EnumFontFamiliesW(hdc, name, is_font_installed_proc, (LPARAM)&lf))
+    {
+        HFONT hfont, old_font;
+
+        hfont = CreateFontIndirectW(&lf);
+        old_font = SelectObject(hdc, hfont);
+        ret = get_font_metrics(hdc, fm) ? Ok : NotTrueTypeFont;
+        SelectObject(hdc, old_font);
+        DeleteObject(hfont);
+    }
+
+    DeleteDC(hdc);
     return ret;
 }
 
@@ -571,8 +752,9 @@ GpStatus WINGDIPAPI GdipCreateFontFamilyFromName(GDIPCONST WCHAR *name,
                                         GpFontCollection *fontCollection,
                                         GpFontFamily **FontFamily)
 {
+    GpStatus stat;
     GpFontFamily* ffamily;
-    NEWTEXTMETRICW ntm;
+    struct font_metrics fm;
 
     TRACE("%s, %p %p\n", debugstr_w(name), fontCollection, FontFamily);
 
@@ -581,14 +763,18 @@ GpStatus WINGDIPAPI GdipCreateFontFamilyFromName(GDIPCONST WCHAR *name,
     if (fontCollection)
         FIXME("No support for FontCollections yet!\n");
 
-    if (!find_installed_font(name, &ntm))
-        return FontFamilyNotFound;
+    stat = find_installed_font(name, &fm);
+    if (stat != Ok) return stat;
 
     ffamily = GdipAlloc(sizeof (GpFontFamily));
     if (!ffamily) return OutOfMemory;
 
-    ffamily->tmw = ntm;
     lstrcpynW(ffamily->FamilyName, name, LF_FACESIZE);
+    ffamily->em_height = fm.em_height;
+    ffamily->ascent = fm.ascent;
+    ffamily->descent = fm.descent;
+    ffamily->line_spacing = fm.line_spacing;
+    ffamily->dpi = fm.dpi;
 
     *FontFamily = ffamily;
 
@@ -597,6 +783,16 @@ GpStatus WINGDIPAPI GdipCreateFontFamilyFromName(GDIPCONST WCHAR *name,
     return Ok;
 }
 
+static GpStatus clone_font_family(const GpFontFamily *family, GpFontFamily **clone)
+{
+    *clone = GdipAlloc(sizeof(GpFontFamily));
+    if (!*clone) return OutOfMemory;
+
+    **clone = *family;
+
+    return Ok;
+}
+
 /*******************************************************************************
  * GdipCloneFontFamily [GDIPLUS.@]
  *
@@ -611,16 +807,15 @@ GpStatus WINGDIPAPI GdipCreateFontFamilyFromName(GDIPCONST WCHAR *name,
  */
 GpStatus WINGDIPAPI GdipCloneFontFamily(GpFontFamily* FontFamily, GpFontFamily** clonedFontFamily)
 {
+    GpStatus status;
+
     if (!(FontFamily && clonedFontFamily)) return InvalidParameter;
 
-    TRACE("stub: %p (%s), %p\n", FontFamily,
+    TRACE("%p (%s), %p\n", FontFamily,
             debugstr_w(FontFamily->FamilyName), clonedFontFamily);
 
-    *clonedFontFamily = GdipAlloc(sizeof(GpFontFamily));
-    if (!*clonedFontFamily) return OutOfMemory;
-
-    (*clonedFontFamily)->tmw = FontFamily->tmw;
-    lstrcpyW((*clonedFontFamily)->FamilyName, FontFamily->FamilyName);
+    status = clone_font_family(FontFamily, clonedFontFamily);
+    if (status != Ok) return status;
 
     TRACE("<-- %p\n", *clonedFontFamily);
 
@@ -692,7 +887,8 @@ GpStatus WINGDIPAPI GdipGetCellAscent(GDIPCONST GpFontFamily *family,
 {
     if (!(family && CellAscent)) return InvalidParameter;
 
-    *CellAscent = family->tmw.tmAscent;
+    *CellAscent = family->ascent;
+    TRACE("%s => %u\n", debugstr_w(family->FamilyName), *CellAscent);
 
     return Ok;
 }
@@ -704,7 +900,8 @@ GpStatus WINGDIPAPI GdipGetCellDescent(GDIPCONST GpFontFamily *family,
 
     if (!(family && CellDescent)) return InvalidParameter;
 
-    *CellDescent = family->tmw.tmDescent;
+    *CellDescent = family->descent;
+    TRACE("%s => %u\n", debugstr_w(family->FamilyName), *CellDescent);
 
     return Ok;
 }
@@ -729,7 +926,8 @@ GpStatus WINGDIPAPI GdipGetEmHeight(GDIPCONST GpFontFamily *family, INT style, U
 
     TRACE("%p (%s), %d, %p\n", family, debugstr_w(family->FamilyName), style, EmHeight);
 
-    *EmHeight = family->tmw.ntmSizeEM;
+    *EmHeight = family->em_height;
+    TRACE("%s => %u\n", debugstr_w(family->FamilyName), *EmHeight);
 
     return Ok;
 }
@@ -759,7 +957,8 @@ GpStatus WINGDIPAPI GdipGetLineSpacing(GDIPCONST GpFontFamily *family,
 
     if (style) FIXME("ignoring style\n");
 
-    *LineSpacing = family->tmw.tmAscent + family->tmw.tmDescent + family->tmw.tmExternalLeading;
+    *LineSpacing = family->line_spacing;
+    TRACE("%s => %u\n", debugstr_w(family->FamilyName), *LineSpacing);
 
     return Ok;
 }
@@ -767,7 +966,7 @@ GpStatus WINGDIPAPI GdipGetLineSpacing(GDIPCONST GpFontFamily *family,
 static INT CALLBACK font_has_style_proc(const LOGFONTW *elf,
                             const TEXTMETRICW *ntm, DWORD type, LPARAM lParam)
 {
-    INT fontstyle=0;
+    INT fontstyle = FontStyleRegular;
 
     if (!ntm) return 1;
 
