@@ -29,6 +29,7 @@
 
 #include "objbase.h"
 #include "ocidl.h"
+#include "wincodecsdk.h"
 #include "wine/list.h"
 
 #include "gdiplus.h"
@@ -63,8 +64,6 @@ extern void calc_curve_bezier_endp(REAL xend, REAL yend, REAL xadj, REAL yadj,
 
 extern void free_installed_fonts(void) DECLSPEC_HIDDEN;
 
-extern void get_font_hfont(GpGraphics *graphics, GDIPCONST GpFont *font, HFONT *hfont) DECLSPEC_HIDDEN;
-
 extern BOOL lengthen_path(GpPath *path, INT len) DECLSPEC_HIDDEN;
 
 extern GpStatus trace_path(GpGraphics *graphics, GpPath *path) DECLSPEC_HIDDEN;
@@ -121,7 +120,7 @@ extern void convert_32bppARGB_to_32bppPARGB(UINT width, UINT height,
 
 extern GpStatus convert_pixels(INT width, INT height,
     INT dst_stride, BYTE *dst_bits, PixelFormat dst_format,
-    INT src_stride, const BYTE *src_bits, PixelFormat src_format, ARGB *src_palette) DECLSPEC_HIDDEN;
+    INT src_stride, const BYTE *src_bits, PixelFormat src_format, ColorPalette *palette) DECLSPEC_HIDDEN;
 
 struct GpPen{
     UINT style;
@@ -161,6 +160,7 @@ struct GpGraphics{
     UINT textcontrast; /* not used yet. get/set only */
     struct list containers;
     GraphicsContainer contid; /* last-issued container ID */
+    INT origin_x, origin_y;
     /* For giving the caller an HDC when we technically can't: */
     HBITMAP temp_hbitmap;
     int temp_hbitmap_width;
@@ -198,6 +198,10 @@ struct GpPathGradient{
     INT blendcount;
     ARGB *surroundcolors;
     INT surroundcolorcount;
+    ARGB* pblendcolor; /* preset blend colors */
+    REAL* pblendpos; /* preset blend positions */
+    INT pblendcount;
+    GpMatrix *transform;
 };
 
 struct GpLineGradient{
@@ -257,14 +261,13 @@ struct GpAdustableArrowCap{
 };
 
 struct GpImage{
-    IPicture* picture;
+    IPicture *picture;
+    IStream *stream; /* source stream */
     ImageType type;
     GUID format;
     UINT flags;
-    UINT palette_flags;
-    UINT palette_count;
-    UINT palette_size;
-    ARGB *palette_entries;
+    UINT frame_count, current_frame;
+    ColorPalette *palette;
     REAL xres, yres;
 };
 
@@ -304,6 +307,7 @@ struct GpBitmap{
     INT stride; /* stride of bits if this is a DIB */
     BYTE *own_bits; /* image bits that need to be freed with this object */
     INT lockx, locky; /* X and Y coordinates of the rect when a bitmap is locked for writing. */
+    IWICMetadataReader *metadata_reader; /* NULL if there is no metadata */
 };
 
 struct GpCachedBitmap{
@@ -326,7 +330,7 @@ struct color_matrix{
 struct color_remap_table{
     BOOL enabled;
     INT mapsize;
-    GDIPCONST ColorMap *colormap;
+    ColorMap *colormap;
 };
 
 struct GpImageAttributes{
@@ -341,11 +345,9 @@ struct GpImageAttributes{
 };
 
 struct GpFont{
-    LOGFONTW lfw;
-    REAL emSize;
-    REAL pixel_size;
-    UINT height;
-    LONG line_spacing;
+    GpFontFamily *family;
+    OUTLINETEXTMETRICW otm;
+    REAL emSize; /* in font units */
     Unit unit;
 };
 
@@ -372,8 +374,9 @@ struct GpFontCollection{
 };
 
 struct GpFontFamily{
-    NEWTEXTMETRICW tmw;
     WCHAR FamilyName[LF_FACESIZE];
+    UINT16 em_height, ascent, descent, line_spacing; /* in font units */
+    int dpi;
 };
 
 /* internal use */
@@ -424,11 +427,14 @@ struct GpRegion{
 typedef GpStatus (*gdip_format_string_callback)(HDC hdc,
     GDIPCONST WCHAR *string, INT index, INT length, GDIPCONST GpFont *font,
     GDIPCONST RectF *rect, GDIPCONST GpStringFormat *format,
-    INT lineno, const RectF *bounds, void *user_data);
+    INT lineno, const RectF *bounds, INT *underlined_indexes,
+    INT underlined_index_count, void *user_data);
 
 GpStatus gdip_format_string(HDC hdc,
     GDIPCONST WCHAR *string, INT length, GDIPCONST GpFont *font,
     GDIPCONST RectF *rect, GDIPCONST GpStringFormat *format,
     gdip_format_string_callback callback, void *user_data) DECLSPEC_HIDDEN;
 
+void get_log_fontW(const GpFont *, GpGraphics *, LOGFONTW *) DECLSPEC_HIDDEN;
+
 #endif
