@@ -48,8 +48,123 @@ static void ExpandContract(HHInfo *pHHInfo);
 #define TAB_MARGIN  8
 #define EDIT_HEIGHT         20
 
+struct list window_list = LIST_INIT(window_list);
+
 static const WCHAR szEmpty[] = {0};
 
+struct html_encoded_symbol {
+    const char *html_code;
+    char        ansi_symbol;
+};
+
+/*
+ * Table mapping the conversion between HTML encoded symbols and their ANSI code page equivalent.
+ * Note: Add additional entries in proper alphabetical order (a binary search is used on this table).
+ */
+struct html_encoded_symbol html_encoded_symbols[] =
+{
+    {"AElig",  0xC6},
+    {"Aacute", 0xC1},
+    {"Acirc",  0xC2},
+    {"Agrave", 0xC0},
+    {"Aring",  0xC5},
+    {"Atilde", 0xC3},
+    {"Auml",   0xC4},
+    {"Ccedil", 0xC7},
+    {"ETH",    0xD0},
+    {"Eacute", 0xC9},
+    {"Ecirc",  0xCA},
+    {"Egrave", 0xC8},
+    {"Euml",   0xCB},
+    {"Iacute", 0xCD},
+    {"Icirc",  0xCE},
+    {"Igrave", 0xCC},
+    {"Iuml",   0xCF},
+    {"Ntilde", 0xD1},
+    {"Oacute", 0xD3},
+    {"Ocirc",  0xD4},
+    {"Ograve", 0xD2},
+    {"Oslash", 0xD8},
+    {"Otilde", 0xD5},
+    {"Ouml",   0xD6},
+    {"THORN",  0xDE},
+    {"Uacute", 0xDA},
+    {"Ucirc",  0xDB},
+    {"Ugrave", 0xD9},
+    {"Uuml",   0xDC},
+    {"Yacute", 0xDD},
+    {"aacute", 0xE1},
+    {"acirc",  0xE2},
+    {"acute",  0xB4},
+    {"aelig",  0xE6},
+    {"agrave", 0xE0},
+    {"amp",    '&'},
+    {"aring",  0xE5},
+    {"atilde", 0xE3},
+    {"auml",   0xE4},
+    {"brvbar", 0xA6},
+    {"ccedil", 0xE7},
+    {"cedil",  0xB8},
+    {"cent",   0xA2},
+    {"copy",   0xA9},
+    {"curren", 0xA4},
+    {"deg",    0xB0},
+    {"divide", 0xF7},
+    {"eacute", 0xE9},
+    {"ecirc",  0xEA},
+    {"egrave", 0xE8},
+    {"eth",    0xF0},
+    {"euml",   0xEB},
+    {"frac12", 0xBD},
+    {"frac14", 0xBC},
+    {"frac34", 0xBE},
+    {"gt",     '>'},
+    {"iacute", 0xED},
+    {"icirc",  0xEE},
+    {"iexcl",  0xA1},
+    {"igrave", 0xEC},
+    {"iquest", 0xBF},
+    {"iuml",   0xEF},
+    {"laquo",  0xAB},
+    {"lt",     '<'},
+    {"macr",   0xAF},
+    {"micro",  0xB5},
+    {"middot", 0xB7},
+    {"nbsp",   ' '},
+    {"not",    0xAC},
+    {"ntilde", 0xF1},
+    {"oacute", 0xF3},
+    {"ocirc",  0xF4},
+    {"ograve", 0xF2},
+    {"ordf",   0xAA},
+    {"ordm",   0xBA},
+    {"oslash", 0xF8},
+    {"otilde", 0xF5},
+    {"ouml",   0xF6},
+    {"para",   0xB6},
+    {"plusmn", 0xB1},
+    {"pound",  0xA3},
+    {"quot",   '"'},
+    {"raquo",  0xBB},
+    {"reg",    0xAE},
+    {"sect",   0xA7},
+    {"shy",    0xAD},
+    {"sup1",   0xB9},
+    {"sup2",   0xB2},
+    {"sup3",   0xB3},
+    {"szlig",  0xDF},
+    {"thorn",  0xFE},
+    {"times",  0xD7},
+    {"uacute", 0xFA},
+    {"ucirc",  0xFB},
+    {"ugrave", 0xF9},
+    {"uml",    0xA8},
+    {"uuml",   0xFC},
+    {"yacute", 0xFD},
+    {"yen",    0xA5},
+    {"yuml",   0xFF}
+};
+
 /* Loads a string from the resource file */
 static LPWSTR HH_LoadString(DWORD dwID)
 {
@@ -227,7 +342,7 @@ static void SB_OnLButtonDown(HWND hWnd, WPARAM wParam, LPARAM lParam)
 
 static void SB_OnLButtonUp(HWND hWnd, WPARAM wParam, LPARAM lParam)
 {
-    HHInfo *pHHInfo = (HHInfo *)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+    HHInfo *pHHInfo = (HHInfo *)GetWindowLongPtrW(hWnd, 0);
     POINT pt;
 
     pt.x = (short)LOWORD(lParam);
@@ -278,7 +393,7 @@ static void HH_RegisterSizeBarClass(HHInfo *pHHInfo)
     wcex.style          = 0;
     wcex.lpfnWndProc    = SizeBar_WndProc;
     wcex.cbClsExtra     = 0;
-    wcex.cbWndExtra     = 0;
+    wcex.cbWndExtra     = sizeof(LONG_PTR);
     wcex.hInstance      = hhctrl_hinstance;
     wcex.hIcon          = LoadIconW(NULL, (LPCWSTR)IDI_APPLICATION);
     wcex.hCursor        = LoadCursorW(NULL, (LPCWSTR)IDC_SIZEWE);
@@ -324,7 +439,7 @@ static BOOL HH_AddSizeBar(HHInfo *pHHInfo)
         return FALSE;
 
     /* store the pointer to the HH info struct */
-    SetWindowLongPtrW(hWnd, GWLP_USERDATA, (LONG_PTR)pHHInfo);
+    SetWindowLongPtrW(hWnd, 0, (LONG_PTR)pHHInfo);
 
     pHHInfo->hwndSizeBar = hWnd;
     return TRUE;
@@ -428,7 +543,7 @@ static void ResizeTabChild(HHInfo *info, int tab)
 
 static LRESULT Child_OnSize(HWND hwnd)
 {
-    HHInfo *info = (HHInfo*)GetWindowLongPtrW(hwnd, GWLP_USERDATA);
+    HHInfo *info = (HHInfo*)GetWindowLongPtrW(hwnd, 0);
     RECT rect;
 
     if(!info || hwnd != info->WinType.hwndNavigation)
@@ -441,12 +556,14 @@ static LRESULT Child_OnSize(HWND hwnd)
 
     ResizeTabChild(info, TAB_CONTENTS);
     ResizeTabChild(info, TAB_INDEX);
+    ResizeTabChild(info, TAB_SEARCH);
     return 0;
 }
 
 static LRESULT OnTabChange(HWND hwnd)
 {
-    HHInfo *info = (HHInfo*)GetWindowLongPtrW(hwnd, GWLP_USERDATA);
+    HHInfo *info = (HHInfo*)GetWindowLongPtrW(hwnd, 0);
+    int tab_id, tab_index, i;
 
     TRACE("%p\n", hwnd);
 
@@ -456,7 +573,23 @@ static LRESULT OnTabChange(HWND hwnd)
     if(info->tabs[info->current_tab].hwnd)
         ShowWindow(info->tabs[info->current_tab].hwnd, SW_HIDE);
 
-    info->current_tab = SendMessageW(info->hwndTabCtrl, TCM_GETCURSEL, 0, 0);
+    tab_id = (int) SendMessageW(info->hwndTabCtrl, TCM_GETCURSEL, 0, 0);
+    /* convert the ID of the tab to an index in our tab list */
+    tab_index = -1;
+    for (i=0; i<TAB_NUMTABS; i++)
+    {
+        if (info->tabs[i].id == tab_id)
+        {
+            tab_index = i;
+            break;
+        }
+    }
+    if (tab_index == -1)
+    {
+        FIXME("Tab ID %d does not correspond to a valid index in the tab list.\n", tab_id);
+        return 0;
+    }
+    info->current_tab = tab_index;
 
     if(info->tabs[info->current_tab].hwnd)
         ShowWindow(info->tabs[info->current_tab].hwnd, SW_SHOW);
@@ -503,6 +636,8 @@ static LRESULT OnTopicChange(HHInfo *info, void *user_data)
                 IndexSubItem *item = &iiter->items[i];
                 WCHAR *name = iiter->keyword;
 
+                if(!item->name)
+                    item->name = GetDocumentTitle(info->pCHMInfo, item->local);
                 if(item->name)
                     name = item->name;
                 memset(&lvi, 0, sizeof(lvi));
@@ -568,7 +703,7 @@ static LRESULT CALLBACK Child_WndProc(HWND hWnd, UINT message, WPARAM wParam, LP
     case WM_SIZE:
         return Child_OnSize(hWnd);
     case WM_NOTIFY: {
-        HHInfo *info = (HHInfo*)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+        HHInfo *info = (HHInfo*)GetWindowLongPtrW(hWnd, 0);
         NMHDR *nmhdr = (NMHDR*)lParam;
 
         switch(nmhdr->code) {
@@ -576,6 +711,24 @@ static LRESULT CALLBACK Child_WndProc(HWND hWnd, UINT message, WPARAM wParam, LP
             return OnTabChange(hWnd);
         case TVN_SELCHANGEDW:
             return OnTopicChange(info, (void*)((NMTREEVIEWW *)lParam)->itemNew.lParam);
+        case TVN_ITEMEXPANDINGW: {
+            TVITEMW *item = &((NMTREEVIEWW *)lParam)->itemNew;
+            HWND hwndTreeView = info->tabs[TAB_CONTENTS].hwnd;
+
+            item->mask = TVIF_IMAGE|TVIF_SELECTEDIMAGE;
+            if (item->state & TVIS_EXPANDED)
+            {
+                item->iImage = HHTV_FOLDER;
+                item->iSelectedImage = HHTV_FOLDER;
+            }
+            else
+            {
+                item->iImage = HHTV_OPENFOLDER;
+                item->iSelectedImage = HHTV_OPENFOLDER;
+            }
+            SendMessageW(hwndTreeView, TVM_SETITEMW, 0, (LPARAM)item);
+            return 0;
+        }
         case NM_DBLCLK:
             if(!info)
                 return 0;
@@ -651,7 +804,7 @@ static void HH_RegisterChildWndClass(HHInfo *pHHInfo)
     wcex.style          = 0;
     wcex.lpfnWndProc    = Child_WndProc;
     wcex.cbClsExtra     = 0;
-    wcex.cbWndExtra     = 0;
+    wcex.cbWndExtra     = sizeof(LONG_PTR);
     wcex.hInstance      = hhctrl_hinstance;
     wcex.hIcon          = LoadIconW(NULL, (LPCWSTR)IDI_APPLICATION);
     wcex.hCursor        = LoadCursorW(NULL, (LPCWSTR)IDC_ARROW);
@@ -717,7 +870,7 @@ static void DisplayPopupMenu(HHInfo *info)
 
 static void TB_OnClick(HWND hWnd, DWORD dwID)
 {
-    HHInfo *info = (HHInfo *)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+    HHInfo *info = (HHInfo *)GetWindowLongPtrW(hWnd, 0);
 
     switch (dwID)
     {
@@ -749,6 +902,14 @@ static void TB_OnClick(HWND hWnd, DWORD dwID)
         case IDTB_OPTIONS:
             DisplayPopupMenu(info);
             break;
+        case IDTB_NOTES:
+        case IDTB_CONTENTS:
+        case IDTB_INDEX:
+        case IDTB_SEARCH:
+        case IDTB_HISTORY:
+        case IDTB_FAVORITES:
+            /* These are officially unimplemented as of the Windows 7 SDK */
+            break;
         case IDTB_BROWSE_FWD:
         case IDTB_BROWSE_BACK:
         case IDTB_JUMP1:
@@ -776,6 +937,7 @@ static void TB_AddButtonsFromFlags(HHInfo *pHHInfo, TBBUTTON *pButtons, DWORD dw
     int nHistBitmaps = 0, nStdBitmaps = 0, nHHBitmaps = 0;
     HWND hToolbar = pHHInfo->WinType.hwndToolBar;
     TBADDBITMAP tbAB;
+    DWORD unsupported;
 
     /* Common bitmaps */
     tbAB.hInst = HINST_COMMCTRL;
@@ -786,14 +948,22 @@ static void TB_AddButtonsFromFlags(HHInfo *pHHInfo, TBBUTTON *pButtons, DWORD dw
     /* hhctrl.ocx bitmaps */
     tbAB.hInst = hhctrl_hinstance;
     tbAB.nID = IDB_HHTOOLBAR;
-    nHHBitmaps = SendMessageW(hToolbar, TB_ADDBITMAP, 0, (LPARAM)&tbAB);
+    nHHBitmaps = SendMessageW(hToolbar, TB_ADDBITMAP, HHTB_NUMBITMAPS, (LPARAM)&tbAB);
 
     *pdwNumButtons = 0;
 
+    unsupported = dwButtonFlags & (HHWIN_BUTTON_BROWSE_FWD |
+        HHWIN_BUTTON_BROWSE_BCK | HHWIN_BUTTON_NOTES | HHWIN_BUTTON_CONTENTS |
+        HHWIN_BUTTON_INDEX | HHWIN_BUTTON_SEARCH | HHWIN_BUTTON_HISTORY |
+        HHWIN_BUTTON_FAVORITES | HHWIN_BUTTON_JUMP1 | HHWIN_BUTTON_JUMP2 |
+        HHWIN_BUTTON_ZOOM | HHWIN_BUTTON_TOC_NEXT | HHWIN_BUTTON_TOC_PREV);
+    if (unsupported)
+        FIXME("got asked for unsupported buttons: %06x\n", unsupported);
+
     if (dwButtonFlags & HHWIN_BUTTON_EXPAND)
     {
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_EXPAND, nHistBitmaps + HIST_VIEWTREE);
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_CONTRACT, nHistBitmaps + HIST_VIEWTREE);
+        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_EXPAND, nHHBitmaps + HHTB_EXPAND);
+        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_CONTRACT, nHHBitmaps + HHTB_CONTRACT);
 
         if (pHHInfo->WinType.fNotExpanded)
             pButtons[1].fsState |= TBSTATE_HIDDEN;
@@ -808,38 +978,22 @@ static void TB_AddButtonsFromFlags(HHInfo *pHHInfo, TBBUTTON *pButtons, DWORD dw
         TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_FORWARD, nHistBitmaps + HIST_FORWARD);
 
     if (dwButtonFlags & HHWIN_BUTTON_STOP)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_STOP, nHHBitmaps + HH_STOP);
+        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_STOP, nHHBitmaps + HHTB_STOP);
 
     if (dwButtonFlags & HHWIN_BUTTON_REFRESH)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_REFRESH, nHHBitmaps + HH_REFRESH);
+        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_REFRESH, nHHBitmaps + HHTB_REFRESH);
 
     if (dwButtonFlags & HHWIN_BUTTON_HOME)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_HOME, nHHBitmaps + HH_HOME);
+        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_HOME, nHHBitmaps + HHTB_HOME);
 
-    /* FIXME: Load the correct button bitmaps */
     if (dwButtonFlags & HHWIN_BUTTON_SYNC)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_SYNC, nStdBitmaps + STD_PRINT);
+        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_SYNC, nHHBitmaps + HHTB_SYNC);
 
     if (dwButtonFlags & HHWIN_BUTTON_OPTIONS)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_OPTIONS, nStdBitmaps + STD_PRINT);
+        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_OPTIONS, nStdBitmaps + STD_PROPERTIES);
 
     if (dwButtonFlags & HHWIN_BUTTON_PRINT)
         TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_PRINT, nStdBitmaps + STD_PRINT);
-
-    if (dwButtonFlags & HHWIN_BUTTON_JUMP1)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_JUMP1, nStdBitmaps + STD_PRINT);
-
-    if (dwButtonFlags & HHWIN_BUTTON_JUMP2)
-        TB_AddButton(pButtons,(*pdwNumButtons)++, IDTB_JUMP2, nStdBitmaps + STD_PRINT);
-
-    if (dwButtonFlags & HHWIN_BUTTON_ZOOM)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_ZOOM, nStdBitmaps + STD_PRINT);
-
-    if (dwButtonFlags & HHWIN_BUTTON_TOC_NEXT)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_TOC_NEXT, nStdBitmaps + STD_PRINT);
-
-    if (dwButtonFlags & HHWIN_BUTTON_TOC_PREV)
-        TB_AddButton(pButtons, (*pdwNumButtons)++, IDTB_TOC_PREV, nStdBitmaps + STD_PRINT);
 }
 
 static BOOL HH_AddToolbar(HHInfo *pHHInfo)
@@ -948,7 +1102,7 @@ static BOOL HH_AddNavigationPane(HHInfo *info)
     if (!hWnd)
         return FALSE;
 
-    SetWindowLongPtrW(hWnd, GWLP_USERDATA, (LONG_PTR)info);
+    SetWindowLongPtrW(hWnd, 0, (LONG_PTR)info);
 
     hwndTabCtrl = CreateWindowExW(dwExStyles, WC_TABCONTROLW, szEmpty, dwStyles | WS_VISIBLE,
                                   0, TAB_TOP_PADDING,
@@ -1020,7 +1174,7 @@ static BOOL HH_AddHTMLPane(HHInfo *pHHInfo)
         return FALSE;
 
     /* store the pointer to the HH info struct */
-    SetWindowLongPtrW(hWnd, GWLP_USERDATA, (LONG_PTR)pHHInfo);
+    SetWindowLongPtrW(hWnd, 0, (LONG_PTR)pHHInfo);
 
     ShowWindow(hWnd, SW_SHOW);
     UpdateWindow(hWnd);
@@ -1031,18 +1185,29 @@ static BOOL HH_AddHTMLPane(HHInfo *pHHInfo)
 
 static BOOL AddContentTab(HHInfo *info)
 {
+    HIMAGELIST hImageList;
+    HBITMAP hBitmap;
+    HWND hWnd;
+
     if(info->tabs[TAB_CONTENTS].id == -1)
         return TRUE; /* No "Contents" tab */
-    info->tabs[TAB_CONTENTS].hwnd = CreateWindowExW(WS_EX_CLIENTEDGE, WC_TREEVIEWW,
-           szEmpty, WS_CHILD | WS_BORDER | 0x25, 50, 50, 100, 100,
-           info->WinType.hwndNavigation, NULL, hhctrl_hinstance, NULL);
-    if(!info->tabs[TAB_CONTENTS].hwnd) {
+    hWnd = CreateWindowExW(WS_EX_CLIENTEDGE, WC_TREEVIEWW, szEmpty, WS_CHILD | WS_BORDER | TVS_LINESATROOT
+                           | TVS_SHOWSELALWAYS | TVS_HASBUTTONS, 50, 50, 100, 100,
+                           info->WinType.hwndNavigation, NULL, hhctrl_hinstance, NULL);
+    if(!hWnd) {
         ERR("Could not create treeview control\n");
         return FALSE;
     }
 
+    hImageList = ImageList_Create(16, 16, ILC_COLOR32, 0, HHTV_NUMBITMAPS);
+    hBitmap = LoadBitmapW(hhctrl_hinstance, MAKEINTRESOURCEW(IDB_HHTREEVIEW));
+    ImageList_Add(hImageList, hBitmap, NULL);
+    SendMessageW(hWnd, TVM_SETIMAGELIST, TVSIL_NORMAL, (LPARAM)hImageList);
+
+    info->contents.hImageList = hImageList;
+    info->tabs[TAB_CONTENTS].hwnd = hWnd;
     ResizeTabChild(info, TAB_CONTENTS);
-    ShowWindow(info->tabs[TAB_CONTENTS].hwnd, SW_SHOW);
+    ShowWindow(hWnd, SW_SHOW);
 
     return TRUE;
 }
@@ -1132,7 +1297,7 @@ static BOOL AddSearchTab(HHInfo *info)
     info->search.hwndContainer = hwndContainer;
     info->tabs[TAB_SEARCH].hwnd = hwndContainer;
 
-    SetWindowLongPtrW(hwndContainer, GWLP_USERDATA, (LONG_PTR)info);
+    SetWindowLongPtrW(hwndContainer, 0, (LONG_PTR)info);
 
     ResizeTabChild(info, TAB_SEARCH);
 
@@ -1172,7 +1337,7 @@ static void ResizePopupChild(HHInfo *info)
 
 static LRESULT CALLBACK HelpPopup_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
-    HHInfo *info = (HHInfo *)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+    HHInfo *info = (HHInfo *)GetWindowLongPtrW(hWnd, 0);
 
     switch (message)
     {
@@ -1202,7 +1367,7 @@ static LRESULT CALLBACK PopupChild_WndProc(HWND hWnd, UINT message, WPARAM wPara
         switch(nmhdr->code)
         {
         case NM_DBLCLK: {
-            HHInfo *info = (HHInfo*)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+            HHInfo *info = (HHInfo*)GetWindowLongPtrW(hWnd, 0);
             IndexSubItem *iter;
 
             if(info == 0 || lParam == 0)
@@ -1215,7 +1380,7 @@ static LRESULT CALLBACK PopupChild_WndProc(HWND hWnd, UINT message, WPARAM wPara
             return 0;
         }
         case NM_RETURN: {
-            HHInfo *info = (HHInfo*)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+            HHInfo *info = (HHInfo*)GetWindowLongPtrW(hWnd, 0);
             IndexSubItem *iter;
             LVITEMW lvItem;
 
@@ -1257,7 +1422,7 @@ static BOOL AddIndexPopup(HHInfo *info)
     wcex.style          = CS_HREDRAW | CS_VREDRAW;
     wcex.lpfnWndProc    = HelpPopup_WndProc;
     wcex.cbClsExtra     = 0;
-    wcex.cbWndExtra     = 0;
+    wcex.cbWndExtra     = sizeof(LONG_PTR);
     wcex.hInstance      = hhctrl_hinstance;
     wcex.hIcon          = LoadIconW(NULL, (LPCWSTR)IDI_APPLICATION);
     wcex.hCursor        = LoadCursorW(NULL, (LPCWSTR)IDC_ARROW);
@@ -1271,7 +1436,7 @@ static BOOL AddIndexPopup(HHInfo *info)
     wcex.style          = 0;
     wcex.lpfnWndProc    = PopupChild_WndProc;
     wcex.cbClsExtra     = 0;
-    wcex.cbWndExtra     = 0;
+    wcex.cbWndExtra     = sizeof(LONG_PTR);
     wcex.hInstance      = hhctrl_hinstance;
     wcex.hIcon          = LoadIconW(NULL, (LPCWSTR)IDI_APPLICATION);
     wcex.hCursor        = LoadCursorW(NULL, (LPCWSTR)IDC_ARROW);
@@ -1319,8 +1484,8 @@ static BOOL AddIndexPopup(HHInfo *info)
     info->popup.hwndCallback = hwndCallback;
     info->popup.hwndPopup = hwndPopup;
     info->popup.hwndList = hwndList;
-    SetWindowLongPtrW(hwndPopup, GWLP_USERDATA, (LONG_PTR)info);
-    SetWindowLongPtrW(hwndCallback, GWLP_USERDATA, (LONG_PTR)info);
+    SetWindowLongPtrW(hwndPopup, 0, (LONG_PTR)info);
+    SetWindowLongPtrW(hwndCallback, 0, (LONG_PTR)info);
 
     ResizePopupChild(info);
     ShowWindow(hwndList, SW_SHOW);
@@ -1363,7 +1528,7 @@ static void ExpandContract(HHInfo *pHHInfo)
 
 static LRESULT Help_OnSize(HWND hWnd)
 {
-    HHInfo *pHHInfo = (HHInfo *)GetWindowLongPtrW(hWnd, GWLP_USERDATA);
+    HHInfo *pHHInfo = (HHInfo *)GetWindowLongPtrW(hWnd, 0);
     DWORD dwSize;
     RECT rc;
 
@@ -1404,7 +1569,7 @@ static LRESULT CALLBACK Help_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPA
     case WM_SIZE:
         return Help_OnSize(hWnd);
     case WM_CLOSE:
-        ReleaseHelpViewer((HHInfo *)GetWindowLongPtrW(hWnd, GWLP_USERDATA));
+        ReleaseHelpViewer((HHInfo *)GetWindowLongPtrW(hWnd, 0));
         return 0;
     case WM_DESTROY:
         if(hh_process)
@@ -1420,7 +1585,7 @@ static LRESULT CALLBACK Help_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPA
 
 static BOOL HH_CreateHelpWindow(HHInfo *info)
 {
-    HWND hWnd;
+    HWND hWnd, parent = 0;
     RECT winPos = info->WinType.rcWindowPos;
     WNDCLASSEXW wcex;
     DWORD dwStyles, dwExStyles;
@@ -1435,7 +1600,7 @@ static BOOL HH_CreateHelpWindow(HHInfo *info)
     wcex.style          = CS_HREDRAW | CS_VREDRAW;
     wcex.lpfnWndProc    = Help_WndProc;
     wcex.cbClsExtra     = 0;
-    wcex.cbWndExtra     = 0;
+    wcex.cbWndExtra     = sizeof(LONG_PTR);
     wcex.hInstance      = hhctrl_hinstance;
     wcex.hIcon          = LoadIconW(NULL, (LPCWSTR)IDI_APPLICATION);
     wcex.hCursor        = LoadCursorW(NULL, (LPCWSTR)IDC_ARROW);
@@ -1448,7 +1613,11 @@ static BOOL HH_CreateHelpWindow(HHInfo *info)
 
     /* Read in window parameters if available */
     if (info->WinType.fsValidMembers & HHWIN_PARAM_STYLES)
-        dwStyles = info->WinType.dwStyles | WS_OVERLAPPEDWINDOW;
+    {
+        dwStyles = info->WinType.dwStyles;
+        if (!(info->WinType.dwStyles & WS_CHILD))
+            dwStyles |= WS_OVERLAPPEDWINDOW;
+    }
     else
         dwStyles = WS_OVERLAPPEDWINDOW | WS_VISIBLE |
                    WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
@@ -1490,8 +1659,11 @@ static BOOL HH_CreateHelpWindow(HHInfo *info)
     caption = info->WinType.pszCaption;
     if (!*caption) caption = info->pCHMInfo->defTitle;
 
+    if (info->WinType.dwStyles & WS_CHILD)
+        parent = info->WinType.hwndCaller;
+
     hWnd = CreateWindowExW(dwExStyles, windowClassW, caption,
-                           dwStyles, x, y, width, height, NULL, NULL, hhctrl_hinstance, NULL);
+                           dwStyles, x, y, width, height, parent, NULL, hhctrl_hinstance, NULL);
     if (!hWnd)
         return FALSE;
 
@@ -1499,7 +1671,7 @@ static BOOL HH_CreateHelpWindow(HHInfo *info)
     UpdateWindow(hWnd);
 
     /* store the pointer to the HH info struct */
-    SetWindowLongPtrW(hWnd, GWLP_USERDATA, (LONG_PTR)info);
+    SetWindowLongPtrW(hWnd, 0, (LONG_PTR)info);
 
     info->WinType.hwndHelp = hWnd;
     return TRUE;
@@ -1577,6 +1749,8 @@ void ReleaseHelpViewer(HHInfo *info)
     if (!info)
         return;
 
+    list_remove(&info->entry);
+
     /* Free allocated strings */
     heap_free(info->pszType);
     heap_free(info->pszCaption);
@@ -1597,6 +1771,8 @@ void ReleaseHelpViewer(HHInfo *info)
     ReleaseIndex(info);
     ReleaseSearch(info);
 
+    if(info->contents.hImageList)
+        ImageList_Destroy(info->contents.hImageList);
     if(info->WinType.hwndHelp)
         DestroyWindow(info->WinType.hwndHelp);
 
@@ -1604,7 +1780,7 @@ void ReleaseHelpViewer(HHInfo *info)
     OleUninitialize();
 }
 
-HHInfo *CreateHelpViewer(LPCWSTR filename)
+HHInfo *CreateHelpViewer(LPCWSTR filename, HWND caller)
 {
     HHInfo *info = heap_alloc_zero(sizeof(HHInfo));
     int i;
@@ -1627,11 +1803,102 @@ HHInfo *CreateHelpViewer(LPCWSTR filename)
         ReleaseHelpViewer(info);
         return NULL;
     }
+    info->WinType.hwndCaller = caller;
 
     if(!CreateViewer(info)) {
         ReleaseHelpViewer(info);
         return NULL;
     }
 
+    list_add_tail(&window_list, &info->entry);
     return info;
 }
+
+/*
+ * Search the table of HTML entities and return the corresponding ANSI symbol.
+ */
+static char find_html_symbol(const char *entity, int entity_len)
+{
+    int max = sizeof(html_encoded_symbols)/sizeof(html_encoded_symbols[0])-1;
+    int min = 0, dir;
+
+    while(min <= max)
+    {
+        int pos = (min+max)/2;
+        const char *encoded_symbol = html_encoded_symbols[pos].html_code;
+        dir = strncmp(encoded_symbol, entity, entity_len);
+        if(dir == 0 && !encoded_symbol[entity_len]) return html_encoded_symbols[pos].ansi_symbol;
+        if(dir < 0)
+            min = pos+1;
+        else
+            max = pos-1;
+    }
+    return 0;
+}
+
+/*
+ * Decode a string containing HTML encoded characters into a unicode string.
+ */
+WCHAR *decode_html(const char *html_fragment, int html_fragment_len, UINT code_page)
+{
+    const char *h = html_fragment;
+    char *amp, *sem, symbol, *tmp;
+    int len, tmp_len = 0;
+    WCHAR *unicode_text;
+
+    tmp = heap_alloc(html_fragment_len+1);
+    while(1)
+    {
+        symbol = 0;
+        amp = strchr(h, '&');
+        if(!amp) break;
+        len = amp-h;
+        /* Copy the characters prior to the HTML encoded character */
+        memcpy(&tmp[tmp_len], h, len);
+        tmp_len += len;
+        amp++; /* skip ampersand */
+        sem = strchr(amp, ';');
+        /* Require a semicolon after the ampersand */
+        if(!sem)
+        {
+            h = amp;
+            tmp[tmp_len++] = '&';
+            continue;
+        }
+        /* Find the symbol either by using the ANSI character number (prefixed by the pound symbol)
+         * or by searching the HTML entity table */
+        len = sem-amp;
+        if(amp[0] == '#')
+        {
+            char *endnum = NULL;
+            int tmp;
+
+            tmp = (char) strtol(amp, &endnum, 10);
+            if(endnum == sem)
+                symbol = tmp;
+        }
+        else
+            symbol = find_html_symbol(amp, len);
+        if(!symbol)
+        {
+            FIXME("Failed to translate HTML encoded character '&%.*s;'.\n", len, amp);
+            h = amp;
+            tmp[tmp_len++] = '&';
+            continue;
+        }
+        /* Insert the new symbol */
+        h = sem+1;
+        tmp[tmp_len++] = symbol;
+    }
+    /* Convert any remaining characters */
+    len = html_fragment_len-(h-html_fragment);
+    memcpy(&tmp[tmp_len], h, len);
+    tmp_len += len;
+    tmp[tmp_len++] = 0; /* NULL-terminate the string */
+
+    len = MultiByteToWideChar(code_page, 0, tmp, tmp_len, NULL, 0);
+    unicode_text = heap_alloc(len*sizeof(WCHAR));
+    MultiByteToWideChar(code_page, 0, tmp, tmp_len, unicode_text, len);
+    heap_free(tmp);
+    return unicode_text;
+}
