@@ -16,6 +16,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include "config.h"
+#include "wine/port.h"
+
 #include <math.h>
 
 #include "jscript.h"
@@ -94,8 +97,7 @@ static HRESULT set_length(jsdisp_t *obj, jsexcept_t *ei, DWORD length)
         return S_OK;
     }
 
-    V_VT(&var) = VT_I4;
-    V_I4(&var) = length;
+    num_set_int(&var, length);
     return jsdisp_propput_name(obj, lengthW, &var, ei);
 }
 
@@ -114,7 +116,7 @@ static WCHAR *idx_to_str(DWORD idx, WCHAR *ptr)
     return ptr+1;
 }
 
-static HRESULT Array_length(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_length(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     ArrayInstance *This = array_from_vdisp(jsthis);
@@ -123,24 +125,18 @@ static HRESULT Array_length(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISP
 
     switch(flags) {
     case DISPATCH_PROPERTYGET:
-        V_VT(retv) = VT_I4;
-        V_I4(retv) = This->length;
+        num_set_int(retv, This->length);
         break;
     case DISPATCH_PROPERTYPUT: {
-        VARIANT num;
         DOUBLE len = -1;
         DWORD i;
         HRESULT hres;
 
-        hres = to_number(ctx, get_arg(dp, 0), ei, &num);
+        hres = to_number(ctx, argv, ei, &len);
         if(FAILED(hres))
             return hres;
 
-        if(V_VT(&num) == VT_I4)
-            len = V_I4(&num);
-        else
-            len = floor(V_R8(&num));
-
+        len = floor(len);
         if(len!=(DWORD)len)
             return throw_range_error(ctx, ei, JS_E_INVALID_LENGTH, NULL);
 
@@ -205,7 +201,7 @@ static HRESULT concat_obj(jsdisp_t *array, IDispatch *obj, DWORD *len, jsexcept_
     return jsdisp_propput_idx(array, (*len)++, &var, ei);
 }
 
-static HRESULT Array_concat(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_concat(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *ret;
@@ -223,8 +219,8 @@ static HRESULT Array_concat(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISP
         VARIANT *arg;
         DWORD i;
 
-        for(i=0; i < arg_cnt(dp); i++) {
-            arg = get_arg(dp, i);
+        for(i=0; i < argc; i++) {
+            arg = argv+i;
             if(V_VT(arg) == VT_DISPATCH)
                 hres = concat_obj(ret, V_DISPATCH(arg), &len, ei);
             else
@@ -344,7 +340,7 @@ static HRESULT array_join(script_ctx_t *ctx, jsdisp_t *array, DWORD length, cons
 }
 
 /* ECMA-262 3rd Edition    15.4.4.5 */
-static HRESULT Array_join(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_join(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *jsthis;
@@ -357,10 +353,10 @@ static HRESULT Array_join(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPAR
     if(FAILED(hres))
         return hres;
 
-    if(arg_cnt(dp)) {
+    if(argc) {
         BSTR sep;
 
-        hres = to_string(ctx, get_arg(dp,0), ei, &sep);
+        hres = to_string(ctx, argv, ei, &sep);
         if(FAILED(hres))
             return hres;
 
@@ -374,7 +370,7 @@ static HRESULT Array_join(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPAR
     return hres;
 }
 
-static HRESULT Array_pop(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_pop(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *jsthis;
@@ -425,12 +421,12 @@ static HRESULT Array_pop(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARA
 }
 
 /* ECMA-262 3rd Edition    15.4.4.7 */
-static HRESULT Array_push(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_push(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *jsthis;
     DWORD length = 0;
-    int i, n;
+    unsigned i;
     HRESULT hres;
 
     TRACE("\n");
@@ -439,25 +435,22 @@ static HRESULT Array_push(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPAR
     if(FAILED(hres))
         return hres;
 
-    n = arg_cnt(dp);
-    for(i=0; i < n; i++) {
-        hres = jsdisp_propput_idx(jsthis, length+i, get_arg(dp, i), ei);
+    for(i=0; i < argc; i++) {
+        hres = jsdisp_propput_idx(jsthis, length+i, argv+i, ei);
         if(FAILED(hres))
             return hres;
     }
 
-    hres = set_length(jsthis, ei, length+n);
+    hres = set_length(jsthis, ei, length+argc);
     if(FAILED(hres))
         return hres;
 
-    if(retv) {
-        V_VT(retv) = VT_I4;
-        V_I4(retv) = length+n;
-    }
+    if(retv)
+        num_set_int(retv, length+argc);
     return S_OK;
 }
 
-static HRESULT Array_reverse(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_reverse(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *jsthis;
@@ -515,7 +508,7 @@ static HRESULT Array_reverse(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISP
 }
 
 /* ECMA-262 3rd Edition    15.4.4.9 */
-static HRESULT Array_shift(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_shift(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *jsthis;
@@ -569,11 +562,10 @@ static HRESULT Array_shift(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPA
 }
 
 /* ECMA-262 3rd Edition    15.4.4.10 */
-static HRESULT Array_slice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_slice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *arr, *jsthis;
-    VARIANT v;
     DOUBLE range;
     DWORD length, start, end, idx;
     HRESULT hres;
@@ -584,16 +576,12 @@ static HRESULT Array_slice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPA
     if(FAILED(hres))
         return hres;
 
-    if(arg_cnt(dp)) {
-        hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    if(argc) {
+        hres = to_number(ctx, argv, ei, &range);
         if(FAILED(hres))
             return hres;
 
-        if(V_VT(&v) == VT_I4)
-            range = V_I4(&v);
-        else
-            range = floor(V_R8(&v));
-
+        range = floor(range);
         if(-range>length || isnan(range)) start = 0;
         else if(range < 0) start = range+length;
         else if(range <= length) start = range;
@@ -601,16 +589,12 @@ static HRESULT Array_slice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPA
     }
     else start = 0;
 
-    if(arg_cnt(dp)>1) {
-        hres = to_number(ctx, get_arg(dp, 1), ei, &v);
+    if(argc > 1) {
+        hres = to_number(ctx, argv+1, ei, &range);
         if(FAILED(hres))
             return hres;
 
-        if(V_VT(&v) == VT_I4)
-            range = V_I4(&v);
-        else
-            range = floor(V_R8(&v));
-
+        range = floor(range);
         if(-range>length) end = 0;
         else if(range < 0) end = range+length;
         else if(range <= length) end = range;
@@ -623,6 +607,8 @@ static HRESULT Array_slice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPA
         return hres;
 
     for(idx=start; idx<end; idx++) {
+        VARIANT v;
+
         hres = jsdisp_get_idx(jsthis, idx, &v, ei);
         if(hres == DISP_E_UNKNOWNNAME)
             continue;
@@ -652,26 +638,24 @@ static HRESULT sort_cmp(script_ctx_t *ctx, jsdisp_t *cmp_func, VARIANT *v1, VARI
 
     if(cmp_func) {
         VARIANTARG args[2];
-        DISPPARAMS dp = {args, NULL, 2, 0};
-        VARIANT tmp;
+        double n;
         VARIANT res;
 
-        args[0] = *v2;
-        args[1] = *v1;
+        args[0] = *v1;
+        args[1] = *v2;
 
-        hres = jsdisp_call_value(cmp_func, DISPATCH_METHOD, &dp, &res, ei);
+        hres = jsdisp_call_value(cmp_func, NULL, DISPATCH_METHOD, 2, args, &res, ei);
         if(FAILED(hres))
             return hres;
 
-        hres = to_number(ctx, &res, ei, &tmp);
+        hres = to_number(ctx, &res, ei, &n);
         VariantClear(&res);
         if(FAILED(hres))
             return hres;
 
-        if(V_VT(&tmp) == VT_I4)
-            *cmp = V_I4(&tmp);
-        else
-            *cmp = V_R8(&tmp) > 0.0 ? 1 : -1;
+        if(n == 0)
+            *cmp = 0;
+        *cmp = n > 0.0 ? 1 : -1;
     }else if(V_VT(v1) == VT_EMPTY) {
         *cmp = V_VT(v2) == VT_EMPTY ? 0 : 1;
     }else if(V_VT(v2) == VT_EMPTY) {
@@ -703,7 +687,7 @@ static HRESULT sort_cmp(script_ctx_t *ctx, jsdisp_t *cmp_func, VARIANT *v1, VARI
 }
 
 /* ECMA-262 3rd Edition    15.4.4.11 */
-static HRESULT Array_sort(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_sort(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *jsthis, *cmp_func = NULL;
@@ -718,21 +702,19 @@ static HRESULT Array_sort(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPAR
     if(FAILED(hres))
         return hres;
 
-    if(arg_cnt(dp) > 1) {
-        WARN("invalid arg_cnt %d\n", arg_cnt(dp));
+    if(argc > 1) {
+        WARN("invalid arg_cnt %d\n", argc);
         return E_FAIL;
     }
 
-    if(arg_cnt(dp) == 1) {
-        VARIANT *arg = get_arg(dp, 0);
-
-        if(V_VT(arg) != VT_DISPATCH) {
+    if(argc == 1) {
+        if(V_VT(argv) != VT_DISPATCH) {
             WARN("arg is not dispatch\n");
             return E_FAIL;
         }
 
 
-        cmp_func = iface_to_jsdisp((IUnknown*)V_DISPATCH(arg));
+        cmp_func = iface_to_jsdisp((IUnknown*)V_DISPATCH(argv));
         if(!cmp_func || !is_class(cmp_func, JSCLASS_FUNCTION)) {
             WARN("cmp_func is not a function\n");
             if(cmp_func)
@@ -858,12 +840,14 @@ static HRESULT Array_sort(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPAR
 }
 
 /* ECMA-262 3rd Edition    15.4.4.12 */
-static HRESULT Array_splice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_splice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    DWORD length, start=0, delete_cnt=0, argc, i, add_args = 0;
+    DWORD length, start=0, delete_cnt=0, i, add_args = 0;
     jsdisp_t *ret_array = NULL, *jsthis;
     VARIANT v;
+    double d;
+    int n;
     HRESULT hres = S_OK;
 
     TRACE("\n");
@@ -872,31 +856,30 @@ static HRESULT Array_splice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPP
     if(FAILED(hres))
         return hres;
 
-    argc = arg_cnt(dp);
-    if(argc >= 1) {
-        hres = to_integer(ctx, get_arg(dp,0), ei, &v);
+    if(argc) {
+        hres = to_integer(ctx, argv, ei, &d);
         if(FAILED(hres))
             return hres;
 
-        if(V_VT(&v) == VT_I4) {
-            if(V_I4(&v) >= 0)
-                start = min(V_I4(&v), length);
+        if(is_int32(d)) {
+            if((n = d) >= 0)
+                start = min(n, length);
             else
-                start = -V_I4(&v) > length ? 0 : length + V_I4(&v);
+                start = -n > length ? 0 : length + n;
         }else {
-            start = V_R8(&v) < 0.0 ? 0 : length;
+            start = d < 0.0 ? 0 : length;
         }
     }
 
     if(argc >= 2) {
-        hres = to_integer(ctx, get_arg(dp,1), ei, &v);
+        hres = to_integer(ctx, argv+1, ei, &d);
         if(FAILED(hres))
             return hres;
 
-        if(V_VT(&v) == VT_I4) {
-            if(V_I4(&v) > 0)
-                delete_cnt = min(V_I4(&v), length-start);
-        }else if(V_R8(&v) > 0.0) {
+        if(is_int32(d)) {
+            if((n = d) > 0)
+                delete_cnt = min(n, length-start);
+        }else if(d > 0.0) {
             delete_cnt = length-start;
         }
 
@@ -917,9 +900,7 @@ static HRESULT Array_splice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPP
         }
 
         if(SUCCEEDED(hres)) {
-            V_VT(&v) = VT_I4;
-            V_I4(&v) = delete_cnt;
-
+            num_set_int(&v, delete_cnt);
             hres = jsdisp_propput_name(ret_array, lengthW, &v, ei);
         }
     }
@@ -946,11 +927,10 @@ static HRESULT Array_splice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPP
     }
 
     for(i=0; SUCCEEDED(hres) && i < add_args; i++)
-        hres = jsdisp_propput_idx(jsthis, start+i, get_arg(dp,i+2), ei);
+        hres = jsdisp_propput_idx(jsthis, start+i, argv+i+2, ei);
 
     if(SUCCEEDED(hres)) {
-        V_VT(&v) = VT_I4;
-        V_I4(&v) = length-delete_cnt+add_args;
+        num_set_int(&v, length-delete_cnt+add_args);
         hres = jsdisp_propput_name(jsthis, lengthW, &v, ei);
     }
 
@@ -966,7 +946,7 @@ static HRESULT Array_splice(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPP
 }
 
 /* ECMA-262 3rd Edition    15.4.4.2 */
-static HRESULT Array_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     ArrayInstance *array;
@@ -980,7 +960,7 @@ static HRESULT Array_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DI
     return array_join(ctx, &array->dispex, array->length, default_separatorW, retv, ei);
 }
 
-static HRESULT Array_toLocaleString(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_toLocaleString(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     FIXME("\n");
@@ -988,12 +968,12 @@ static HRESULT Array_toLocaleString(script_ctx_t *ctx, vdisp_t *vthis, WORD flag
 }
 
 /* ECMA-262 3rd Edition    15.4.4.13 */
-static HRESULT Array_unshift(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_unshift(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *jsthis;
     WCHAR buf[14], *buf_end, *str;
-    DWORD argc, i, length;
+    DWORD i, length;
     VARIANT var;
     DISPID id;
     HRESULT hres;
@@ -1004,7 +984,6 @@ static HRESULT Array_unshift(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISP
     if(FAILED(hres))
         return hres;
 
-    argc = arg_cnt(dp);
     if(argc) {
         buf_end = buf + sizeof(buf)/sizeof(WCHAR)-1;
         *buf_end-- = 0;
@@ -1031,7 +1010,7 @@ static HRESULT Array_unshift(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISP
     }
 
     for(i=0; i<argc; i++) {
-        hres = jsdisp_propput_idx(jsthis, i, get_arg(dp,i), ei);
+        hres = jsdisp_propput_idx(jsthis, i, argv+i, ei);
         if(FAILED(hres))
             return hres;
     }
@@ -1044,17 +1023,15 @@ static HRESULT Array_unshift(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISP
     }
 
     if(retv) {
-        if(ctx->version < 2) {
+        if(ctx->version < 2)
             V_VT(retv) = VT_EMPTY;
-        }else {
-            V_VT(retv) = VT_I4;
-            V_I4(retv) = length;
-        }
+        else
+            num_set_int(retv, length);
     }
     return S_OK;
 }
 
-static HRESULT Array_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Array_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     TRACE("\n");
@@ -1123,11 +1100,10 @@ static const builtin_info_t Array_info = {
     Array_on_put
 };
 
-static HRESULT ArrayConstr_value(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, DISPPARAMS *dp,
+static HRESULT ArrayConstr_value(script_ctx_t *ctx, vdisp_t *vthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     jsdisp_t *obj;
-    VARIANT *arg_var;
     DWORD i;
     HRESULT hres;
 
@@ -1136,11 +1112,11 @@ static HRESULT ArrayConstr_value(script_ctx_t *ctx, vdisp_t *vthis, WORD flags,
     switch(flags) {
     case DISPATCH_METHOD:
     case DISPATCH_CONSTRUCT: {
-        if(arg_cnt(dp) == 1 && V_VT((arg_var = get_arg(dp, 0))) == VT_I4) {
-            if(V_I4(arg_var) < 0)
+        if(argc == 1 && V_VT(argv) == VT_I4) {
+            if(V_I4(argv) < 0)
                 return throw_range_error(ctx, ei, JS_E_INVALID_LENGTH, NULL);
 
-            hres = create_array(ctx, V_I4(arg_var), &obj);
+            hres = create_array(ctx, V_I4(argv), &obj);
             if(FAILED(hres))
                 return hres;
 
@@ -1148,12 +1124,12 @@ static HRESULT ArrayConstr_value(script_ctx_t *ctx, vdisp_t *vthis, WORD flags,
             return S_OK;
         }
 
-        hres = create_array(ctx, arg_cnt(dp), &obj);
+        hres = create_array(ctx, argc, &obj);
         if(FAILED(hres))
             return hres;
 
-        for(i=0; i < arg_cnt(dp); i++) {
-            hres = jsdisp_propput_idx(obj, i, get_arg(dp, i), ei);
+        for(i=0; i < argc; i++) {
+            hres = jsdisp_propput_idx(obj, i, argv+i, ei);
             if(FAILED(hres))
                 break;
         }
@@ -1207,7 +1183,7 @@ HRESULT create_array_constr(script_ctx_t *ctx, jsdisp_t *object_prototype, jsdis
     if(FAILED(hres))
         return hres;
 
-    hres = create_builtin_function(ctx, ArrayConstr_value, ArrayW, NULL, PROPF_CONSTR|1, &array->dispex, ret);
+    hres = create_builtin_constructor(ctx, ArrayConstr_value, ArrayW, NULL, PROPF_CONSTR|1, &array->dispex, ret);
 
     jsdisp_release(&array->dispex);
     return hres;
