@@ -52,6 +52,13 @@ typedef struct {
     unsigned labels_cnt;
 
     statement_ctx_t *stat_ctx;
+    function_code_t *func;
+
+    variable_declaration_t *var_head;
+    variable_declaration_t *var_tail;
+
+    function_expression_t *func_head;
+    function_expression_t *func_tail;
 } compiler_ctx_t;
 
 static const struct {
@@ -80,9 +87,6 @@ static void dump_instr_arg(instr_arg_type_t type, instr_arg_t *arg)
     case ARG_ADDR:
         TRACE_(jscript_disas)("\t%u", arg->uint);
         break;
-    case ARG_DBL:
-        TRACE_(jscript_disas)("\t%lf", *arg->dbl);
-        break;
     case ARG_FUNC:
     case ARG_NONE:
         break;
@@ -97,8 +101,12 @@ static void dump_code(compiler_ctx_t *ctx, unsigned off)
 
     for(instr = ctx->code->instrs+off; instr < ctx->code->instrs+ctx->code_off; instr++) {
         TRACE_(jscript_disas)("%d:\t%s", (int)(instr-ctx->code->instrs), instr_info[instr->op].op_str);
-        dump_instr_arg(instr_info[instr->op].arg1_type, &instr->arg1);
-        dump_instr_arg(instr_info[instr->op].arg2_type, &instr->arg2);
+        if(instr_info[instr->op].arg1_type == ARG_DBL) {
+            TRACE_(jscript_disas)("\t%lf", instr->u.dbl);
+        }else {
+            dump_instr_arg(instr_info[instr->op].arg1_type, instr->u.arg);
+            dump_instr_arg(instr_info[instr->op].arg2_type, instr->u.arg+1);
+        }
         TRACE_(jscript_disas)("\n");
     }
 }
@@ -181,7 +189,7 @@ static HRESULT push_instr_int(compiler_ctx_t *ctx, jsop_t op, LONG arg)
     if(!instr)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, instr)->arg1.lng = arg;
+    instr_ptr(ctx, instr)->u.arg->lng = arg;
     return S_OK;
 }
 
@@ -198,7 +206,7 @@ static HRESULT push_instr_str(compiler_ctx_t *ctx, jsop_t op, const WCHAR *arg)
     if(!instr)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, instr)->arg1.str = str;
+    instr_ptr(ctx, instr)->u.arg->str = str;
     return S_OK;
 }
 
@@ -215,7 +223,7 @@ static HRESULT push_instr_bstr(compiler_ctx_t *ctx, jsop_t op, const WCHAR *arg)
     if(!instr)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, instr)->arg1.bstr = str;
+    instr_ptr(ctx, instr)->u.arg->bstr = str;
     return S_OK;
 }
 
@@ -232,8 +240,8 @@ static HRESULT push_instr_bstr_uint(compiler_ctx_t *ctx, jsop_t op, const WCHAR
     if(!instr)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, instr)->arg1.bstr = str;
-    instr_ptr(ctx, instr)->arg2.uint = arg2;
+    instr_ptr(ctx, instr)->u.arg[0].bstr = str;
+    instr_ptr(ctx, instr)->u.arg[1].uint = arg2;
     return S_OK;
 }
 
@@ -250,29 +258,28 @@ static HRESULT push_instr_uint_str(compiler_ctx_t *ctx, jsop_t op, unsigned arg1
     if(!instr)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, instr)->arg1.uint = arg1;
-    instr_ptr(ctx, instr)->arg2.str = str;
+    instr_ptr(ctx, instr)->u.arg[0].uint = arg1;
+    instr_ptr(ctx, instr)->u.arg[1].str = str;
     return S_OK;
 }
 
 static HRESULT push_instr_double(compiler_ctx_t *ctx, jsop_t op, double arg)
 {
     unsigned instr;
-    DOUBLE *dbl;
-
-    dbl = compiler_alloc(ctx->code, sizeof(arg));
-    if(!dbl)
-        return E_OUTOFMEMORY;
-    *dbl = arg;
 
     instr = push_instr(ctx, op);
     if(!instr)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, instr)->arg1.dbl = dbl;
+    instr_ptr(ctx, instr)->u.dbl = arg;
     return S_OK;
 }
 
+static inline void set_arg_uint(compiler_ctx_t *ctx, unsigned instr, unsigned arg)
+{
+    instr_ptr(ctx, instr)->u.arg->uint = arg;
+}
+
 static HRESULT push_instr_uint(compiler_ctx_t *ctx, jsop_t op, unsigned arg)
 {
     unsigned instr;
@@ -281,7 +288,7 @@ static HRESULT push_instr_uint(compiler_ctx_t *ctx, jsop_t op, unsigned arg)
     if(!instr)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, instr)->arg1.uint = arg;
+    set_arg_uint(ctx, instr, arg);
     return S_OK;
 }
 
@@ -456,7 +463,7 @@ static HRESULT compile_logical_expression(compiler_ctx_t *ctx, binary_expression
     if(FAILED(hres))
         return hres;
 
-    instr_ptr(ctx, instr)->arg1.uint = ctx->code_off;
+    set_arg_uint(ctx, instr, ctx->code_off);
     return S_OK;
 }
 
@@ -482,7 +489,7 @@ static HRESULT compile_conditional_expression(compiler_ctx_t *ctx, conditional_e
     if(!jmp_end)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, jmp_false)->arg1.uint = ctx->code_off;
+    set_arg_uint(ctx, jmp_false, ctx->code_off);
     if(!push_instr(ctx, OP_pop))
         return E_OUTOFMEMORY;
 
@@ -490,7 +497,7 @@ static HRESULT compile_conditional_expression(compiler_ctx_t *ctx, conditional_e
     if(FAILED(hres))
         return hres;
 
-    instr_ptr(ctx, jmp_end)->arg1.uint = ctx->code_off;
+    set_arg_uint(ctx, jmp_end, ctx->code_off);
     return S_OK;
 }
 
@@ -511,7 +518,7 @@ static HRESULT compile_new_expression(compiler_ctx_t *ctx, call_expression_t *ex
         arg_cnt++;
     }
 
-    return push_instr_int(ctx, OP_new, arg_cnt);
+    return push_instr_uint(ctx, OP_new, arg_cnt);
 }
 
 static HRESULT compile_call_expression(compiler_ctx_t *ctx, call_expression_t *expr, BOOL *no_ret)
@@ -544,8 +551,8 @@ static HRESULT compile_call_expression(compiler_ctx_t *ctx, call_expression_t *e
     if(!instr)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, instr)->arg1.uint = arg_cnt;
-    instr_ptr(ctx, instr)->arg2.lng = no_ret == NULL;
+    instr_ptr(ctx, instr)->u.arg[0].uint = arg_cnt;
+    instr_ptr(ctx, instr)->u.arg[1].lng = no_ret == NULL;
     if(no_ret)
         *no_ret = TRUE;
     return S_OK;
@@ -607,9 +614,43 @@ static HRESULT compile_delete_expression(compiler_ctx_t *ctx, unary_expression_t
 
 static HRESULT compile_assign_expression(compiler_ctx_t *ctx, binary_expression_t *expr, jsop_t op)
 {
+    BOOL use_throw_path = FALSE;
+    unsigned arg_cnt = 0;
     HRESULT hres;
 
-    if(!is_memberid_expr(expr->expression1->type)) {
+    if(expr->expression1->type == EXPR_CALL) {
+        call_expression_t *call_expr = (call_expression_t*)expr->expression1;
+        argument_t *arg;
+
+        if(op != OP_LAST) {
+            FIXME("op %d not supported on parametrized assign expressions\n", op);
+            return E_NOTIMPL;
+        }
+
+        if(is_memberid_expr(call_expr->expression->type) && call_expr->argument_list) {
+            hres = compile_memberid_expression(ctx, call_expr->expression, fdexNameEnsure);
+            if(FAILED(hres))
+                return hres;
+
+            for(arg = call_expr->argument_list; arg; arg = arg->next) {
+                hres = compile_expression(ctx, arg->expr);
+                if(FAILED(hres))
+                    return hres;
+                arg_cnt++;
+            }
+        }else {
+            use_throw_path = TRUE;
+        }
+    }else if(is_memberid_expr(expr->expression1->type)) {
+        hres = compile_memberid_expression(ctx, expr->expression1, fdexNameEnsure);
+        if(FAILED(hres))
+            return hres;
+    }else {
+        use_throw_path = TRUE;
+    }
+
+    if(use_throw_path) {
+        /* Illegal assignment: evaluate and throw */
         hres = compile_expression(ctx, expr->expression1);
         if(FAILED(hres))
             return hres;
@@ -624,10 +665,6 @@ static HRESULT compile_assign_expression(compiler_ctx_t *ctx, binary_expression_
         return push_instr_uint(ctx, OP_throw_ref, JS_E_ILLEGAL_ASSIGN);
     }
 
-    hres = compile_memberid_expression(ctx, expr->expression1, fdexNameEnsure);
-    if(FAILED(hres))
-        return hres;
-
     if(op != OP_LAST && !push_instr(ctx, OP_refval))
         return E_OUTOFMEMORY;
 
@@ -638,6 +675,9 @@ static HRESULT compile_assign_expression(compiler_ctx_t *ctx, binary_expression_
     if(op != OP_LAST && !push_instr(ctx, op))
         return E_OUTOFMEMORY;
 
+    if(arg_cnt)
+        return push_instr_uint(ctx, OP_assign_call, arg_cnt);
+
     if(!push_instr(ctx, OP_assign))
         return E_OUTOFMEMORY;
 
@@ -672,8 +712,6 @@ static HRESULT compile_literal(compiler_ctx_t *ctx, literal_t *literal)
         return push_instr_int(ctx, OP_bool, literal->u.bval);
     case LT_DOUBLE:
         return push_instr_double(ctx, OP_double, literal->u.dval);
-    case LT_INT:
-        return push_instr_int(ctx, OP_int, literal->u.lval);
     case LT_NULL:
         return push_instr(ctx, OP_null) ? S_OK : E_OUTOFMEMORY;
     case LT_STRING:
@@ -692,8 +730,8 @@ static HRESULT compile_literal(compiler_ctx_t *ctx, literal_t *literal)
         if(!instr)
             return E_OUTOFMEMORY;
 
-        instr_ptr(ctx, instr)->arg1.str = str;
-        instr_ptr(ctx, instr)->arg2.lng = literal->u.regexp.flags;
+        instr_ptr(ctx, instr)->u.arg[0].str = str;
+        instr_ptr(ctx, instr)->u.arg[1].uint = literal->u.regexp.flags;
         return S_OK;
     }
     default:
@@ -708,9 +746,6 @@ static HRESULT literal_as_bstr(compiler_ctx_t *ctx, literal_t *literal, BSTR *st
     case LT_STRING:
         *str = compiler_alloc_bstr(ctx, literal->u.wstr);
         break;
-    case LT_INT:
-        *str = int_to_bstr(literal->u.lval);
-        break;
     case LT_DOUBLE:
         return double_to_bstr(literal->u.dval, str);
     default:
@@ -770,7 +805,7 @@ static HRESULT compile_object_literal(compiler_ctx_t *ctx, property_value_expres
         if(!instr)
             return E_OUTOFMEMORY;
 
-        instr_ptr(ctx, instr)->arg1.bstr = name;
+        instr_ptr(ctx, instr)->u.arg->bstr = name;
     }
 
     return S_OK;
@@ -778,18 +813,15 @@ static HRESULT compile_object_literal(compiler_ctx_t *ctx, property_value_expres
 
 static HRESULT compile_function_expression(compiler_ctx_t *ctx, function_expression_t *expr)
 {
-    unsigned instr;
+    ctx->func_tail = ctx->func_tail ? (ctx->func_tail->next = expr) : (ctx->func_head = expr);
 
     /* FIXME: not exactly right */
-    if(expr->identifier)
+    if(expr->identifier) {
+        ctx->func->func_cnt++;
         return push_instr_bstr(ctx, OP_ident, expr->identifier);
+    }
 
-    instr = push_instr(ctx, OP_func);
-    if(!instr)
-        return E_OUTOFMEMORY;
-
-    instr_ptr(ctx, instr)->arg1.func = expr;
-    return S_OK;
+    return push_instr_uint(ctx, OP_func, ctx->func->func_cnt++);
 }
 
 static HRESULT compile_expression_noret(compiler_ctx_t *ctx, expression_t *expr, BOOL *no_ret)
@@ -959,7 +991,19 @@ static HRESULT compile_variable_list(compiler_ctx_t *ctx, variable_declaration_t
     variable_declaration_t *iter;
     HRESULT hres;
 
+    assert(list != NULL);
+
+    if(ctx->var_tail)
+        ctx->var_tail->global_next = list;
+    else
+        ctx->var_head = list;
+
     for(iter = list; iter; iter = iter->next) {
+        ctx->func->var_cnt++;
+        iter->global_next = iter->next;
+        if(!iter->next)
+            ctx->var_tail = iter;
+
         if(!iter->expr)
             continue;
 
@@ -1026,7 +1070,7 @@ static HRESULT compile_if_statement(compiler_ctx_t *ctx, if_statement_t *stat)
     if(!jmp_end)
         return E_OUTOFMEMORY;
 
-    instr_ptr(ctx, jmp_else)->arg1.uint = ctx->code_off;
+    set_arg_uint(ctx, jmp_else, ctx->code_off);
 
     if(stat->else_stat) {
         hres = compile_statement(ctx, NULL, stat->else_stat);
@@ -1038,7 +1082,7 @@ static HRESULT compile_if_statement(compiler_ctx_t *ctx, if_statement_t *stat)
             return E_OUTOFMEMORY;
     }
 
-    instr_ptr(ctx, jmp_end)->arg1.uint = ctx->code_off;
+    set_arg_uint(ctx, jmp_end, ctx->code_off);
     return S_OK;
 }
 
@@ -1490,11 +1534,11 @@ static HRESULT compile_switch_statement(compiler_ctx_t *ctx, switch_statement_t
     i = 0;
     for(iter = stat->case_list; iter; iter = iter->next) {
         while(iter->next && iter->next->stat == iter->stat) {
-            instr_ptr(ctx, iter->expr ? case_jmps[i++] : default_jmp)->arg1.uint = ctx->code_off;
+            set_arg_uint(ctx, iter->expr ? case_jmps[i++] : default_jmp, ctx->code_off);
             iter = iter->next;
         }
 
-        instr_ptr(ctx, iter->expr ? case_jmps[i++] : default_jmp)->arg1.uint = ctx->code_off;
+        set_arg_uint(ctx, iter->expr ? case_jmps[i++] : default_jmp, ctx->code_off);
 
         for(stat_iter = iter->stat; stat_iter && (!iter->next || iter->next->stat != stat_iter); stat_iter = stat_iter->next) {
             hres = compile_statement(ctx, &stat_ctx, stat_iter);
@@ -1515,8 +1559,14 @@ static HRESULT compile_switch_statement(compiler_ctx_t *ctx, switch_statement_t
         return hres;
     assert(i == case_cnt);
 
-    if(!have_default)
-        instr_ptr(ctx, default_jmp)->arg1.uint = ctx->code_off;
+    if(!have_default) {
+        hres = push_instr_uint(ctx, OP_jmp, stat_ctx.break_label);
+        if(FAILED(hres))
+            return hres;
+        set_arg_uint(ctx, default_jmp, ctx->code_off);
+        if(!push_instr(ctx, OP_undefined))
+            return E_OUTOFMEMORY;
+    }
 
     label_set_addr(ctx, stat_ctx.break_label);
     return S_OK;
@@ -1555,7 +1605,7 @@ static HRESULT compile_try_statement(compiler_ctx_t *ctx, try_statement_t *stat)
         ident = NULL;
     }
 
-    instr_ptr(ctx, push_except)->arg2.bstr = ident;
+    instr_ptr(ctx, push_except)->u.arg[1].bstr = ident;
 
     if(!stat->catch_block)
         try_ctx.stack_use = 2;
@@ -1574,7 +1624,7 @@ static HRESULT compile_try_statement(compiler_ctx_t *ctx, try_statement_t *stat)
         if(!jmp_finally)
             return E_OUTOFMEMORY;
 
-        instr_ptr(ctx, push_except)->arg1.uint = ctx->code_off;
+        instr_ptr(ctx, push_except)->u.arg[0].uint = ctx->code_off;
 
         hres = compile_statement(ctx, &catch_ctx, stat->catch_block->statement);
         if(FAILED(hres))
@@ -1583,9 +1633,9 @@ static HRESULT compile_try_statement(compiler_ctx_t *ctx, try_statement_t *stat)
         if(!push_instr(ctx, OP_pop_scope))
             return E_OUTOFMEMORY;
 
-        instr_ptr(ctx, jmp_finally)->arg1.uint = ctx->code_off;
+        set_arg_uint(ctx, jmp_finally, ctx->code_off);
     }else {
-        instr_ptr(ctx, push_except)->arg1.uint = ctx->code_off;
+        set_arg_uint(ctx, push_except, ctx->code_off);
     }
 
     if(stat->finally_statement) {
@@ -1680,9 +1730,9 @@ static void resolve_labels(compiler_ctx_t *ctx, unsigned off)
     instr_t *instr;
 
     for(instr = ctx->code->instrs+off; instr < ctx->code->instrs+ctx->code_off; instr++) {
-        if(instr_info[instr->op].arg1_type == ARG_ADDR && (instr->arg1.uint & LABEL_FLAG)) {
-            assert((instr->arg1.uint & ~LABEL_FLAG) < ctx->labels_cnt);
-            instr->arg1.uint = ctx->labels[instr->arg1.uint & ~LABEL_FLAG];
+        if(instr_info[instr->op].arg1_type == ARG_ADDR && (instr->u.arg->uint & LABEL_FLAG)) {
+            assert((instr->u.arg->uint & ~LABEL_FLAG) < ctx->labels_cnt);
+            instr->u.arg->uint = ctx->labels[instr->u.arg->uint & ~LABEL_FLAG];
         }
         assert(instr_info[instr->op].arg2_type != ARG_ADDR);
     }
@@ -1697,19 +1747,17 @@ void release_bytecode(bytecode_t *code)
     if(--code->ref)
         return;
 
-    if(code->parser)
-        parser_release(code->parser);
-
     for(i=0; i < code->bstr_cnt; i++)
         SysFreeString(code->bstr_pool[i]);
 
+    heap_free(code->source);
     jsheap_free(&code->heap);
     heap_free(code->bstr_pool);
     heap_free(code->instrs);
     heap_free(code);
 }
 
-static HRESULT init_code(compiler_ctx_t *compiler)
+static HRESULT init_code(compiler_ctx_t *compiler, const WCHAR *source)
 {
     compiler->code = heap_alloc_zero(sizeof(bytecode_t));
     if(!compiler->code)
@@ -1718,6 +1766,12 @@ static HRESULT init_code(compiler_ctx_t *compiler)
     compiler->code->ref = 1;
     jsheap_init(&compiler->code->heap);
 
+    compiler->code->source = heap_strdupW(source);
+    if(!compiler->code->source) {
+        release_bytecode(compiler->code);
+        return E_OUTOFMEMORY;
+    }
+
     compiler->code->instrs = heap_alloc(64 * sizeof(instr_t));
     if(!compiler->code->instrs) {
         release_bytecode(compiler->code);
@@ -1729,15 +1783,21 @@ static HRESULT init_code(compiler_ctx_t *compiler)
     return S_OK;
 }
 
-static HRESULT compile_function(compiler_ctx_t *ctx, source_elements_t *source, BOOL from_eval)
+static HRESULT compile_function(compiler_ctx_t *ctx, source_elements_t *source, function_expression_t *func_expr,
+        BOOL from_eval, function_code_t *func)
 {
-    function_declaration_t *iter;
-    unsigned off;
+    variable_declaration_t *var_iter;
+    function_expression_t *iter;
+    unsigned off, i;
     HRESULT hres;
 
     TRACE("\n");
 
+    ctx->var_head = ctx->var_tail = NULL;
+    ctx->func_head = ctx->func_tail = NULL;
+
     off = ctx->code_off;
+    ctx->func = func;
     hres = compile_block_statement(ctx, source->statement);
     if(FAILED(hres))
         return hres;
@@ -1752,36 +1812,88 @@ static HRESULT compile_function(compiler_ctx_t *ctx, source_elements_t *source,
     if(TRACE_ON(jscript_disas))
         dump_code(ctx, off);
 
-    source->instr_off = off;
+    func->instr_off = off;
 
-    for(iter = source->functions; iter; iter = iter->next) {
-        hres = compile_function(ctx, iter->expr->source_elements, FALSE);
+    if(func_expr && func_expr->identifier) {
+        func->name = compiler_alloc_bstr(ctx, func_expr->identifier);
+        if(!func->name)
+            return E_OUTOFMEMORY;
+    }
+
+    if(func_expr) {
+        parameter_t *param_iter;
+
+        func->source = func_expr->src_str;
+        func->source_len = func_expr->src_len;
+
+        for(param_iter = func_expr->parameter_list; param_iter; param_iter = param_iter->next)
+            func->param_cnt++;
+
+        func->params = compiler_alloc(ctx->code, func->param_cnt * sizeof(*func->params));
+        if(!func->params)
+            return E_OUTOFMEMORY;
+
+        for(param_iter = func_expr->parameter_list, i=0; param_iter; param_iter = param_iter->next, i++) {
+            func->params[i] = compiler_alloc_bstr(ctx, param_iter->identifier);
+            if(!func->params[i])
+                return E_OUTOFMEMORY;
+        }
+    }
+
+    func->variables = compiler_alloc(ctx->code, func->var_cnt * sizeof(*func->variables));
+    if(!func->variables)
+        return E_OUTOFMEMORY;
+
+    for(var_iter = ctx->var_head, i=0; var_iter; var_iter = var_iter->global_next, i++) {
+        func->variables[i] = compiler_alloc_bstr(ctx, var_iter->identifier);
+        if(!func->variables[i])
+            return E_OUTOFMEMORY;
+    }
+
+    assert(i == func->var_cnt);
+
+    func->funcs = compiler_alloc(ctx->code, func->func_cnt * sizeof(*func->funcs));
+    if(!func->funcs)
+        return E_OUTOFMEMORY;
+    memset(func->funcs, 0, func->func_cnt * sizeof(*func->funcs));
+
+    for(iter = ctx->func_head, i=0; iter; iter = iter->next, i++) {
+        hres = compile_function(ctx, iter->source_elements, iter, FALSE, func->funcs+i);
         if(FAILED(hres))
             return hres;
     }
 
+    assert(i == func->func_cnt);
+
     return S_OK;
 }
 
-HRESULT compile_script(script_ctx_t *ctx, const WCHAR *code, const WCHAR *delimiter, BOOL from_eval,
+HRESULT compile_script(script_ctx_t *ctx, const WCHAR *code, const WCHAR *delimiter, BOOL from_eval, BOOL use_decode,
         bytecode_t **ret)
 {
     compiler_ctx_t compiler = {0};
     HRESULT hres;
 
-    hres = script_parse(ctx, code, delimiter, from_eval, &compiler.parser);
+    hres = init_code(&compiler, code);
     if(FAILED(hres))
         return hres;
 
-    hres = init_code(&compiler);
+    if(use_decode) {
+        hres = decode_source(compiler.code->source);
+        if(FAILED(hres)) {
+            WARN("Decoding failed\n");
+            return hres;
+        }
+    }
+
+    hres = script_parse(ctx, compiler.code->source, delimiter, from_eval, &compiler.parser);
     if(FAILED(hres)) {
-        parser_release(compiler.parser);
+        release_bytecode(compiler.code);
         return hres;
     }
 
-    compiler.code->parser = compiler.parser;
-
-    hres = compile_function(&compiler, compiler.parser->source, from_eval);
+    hres = compile_function(&compiler, compiler.parser->source, NULL, from_eval, &compiler.code->global_code);
+    parser_release(compiler.parser);
     if(FAILED(hres)) {
         release_bytecode(compiler.code);
         return hres;
