@@ -133,6 +133,14 @@ static inline VARIANT *stack_topn(exec_ctx_t *ctx, unsigned n)
     return ctx->stack + ctx->top-1-n;
 }
 
+static inline VARIANT *stack_args(exec_ctx_t *ctx, unsigned n)
+{
+    if(!n)
+        return NULL;
+    assert(ctx->top > n-1);
+    return ctx->stack + ctx->top-n;
+}
+
 static inline VARIANT *stack_pop(exec_ctx_t *ctx)
 {
     assert(ctx->top);
@@ -145,7 +153,7 @@ static void stack_popn(exec_ctx_t *ctx, unsigned n)
         VariantClear(stack_pop(ctx));
 }
 
-static HRESULT stack_pop_number(exec_ctx_t *ctx, VARIANT *r)
+static HRESULT stack_pop_number(exec_ctx_t *ctx, double *r)
 {
     VARIANT *v;
     HRESULT hres;
@@ -364,7 +372,7 @@ static HRESULT disp_get_id(script_ctx_t *ctx, IDispatch *disp, BSTR name, DWORD
 
     hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
     if(FAILED(hres)) {
-        TRACE("unsing IDispatch\n");
+        TRACE("using IDispatch\n");
 
         *id = 0;
         return IDispatch_GetIDsOfNames(disp, &IID_NULL, &name, 1, 0, id);
@@ -554,10 +562,30 @@ static HRESULT identifier_eval(script_ctx_t *ctx, BSTR identifier, exprval_t *re
     return S_OK;
 }
 
+static inline BSTR get_op_bstr(exec_ctx_t *ctx, int i){
+    return ctx->code->instrs[ctx->ip].u.arg[i].bstr;
+}
+
+static inline unsigned get_op_uint(exec_ctx_t *ctx, int i){
+    return ctx->code->instrs[ctx->ip].u.arg[i].uint;
+}
+
+static inline unsigned get_op_int(exec_ctx_t *ctx, int i){
+    return ctx->code->instrs[ctx->ip].u.arg[i].lng;
+}
+
+static inline const WCHAR *get_op_str(exec_ctx_t *ctx, int i){
+    return ctx->code->instrs[ctx->ip].u.arg[i].str;
+}
+
+static inline double get_op_double(exec_ctx_t *ctx){
+    return ctx->code->instrs[ctx->ip].u.dbl;
+}
+
 /* ECMA-262 3rd Edition    12.2 */
 static HRESULT interp_var_set(exec_ctx_t *ctx)
 {
-    const BSTR name = ctx->code->instrs[ctx->ip].arg1.bstr;
+    const BSTR name = get_op_bstr(ctx, 0);
     VARIANT *v;
     HRESULT hres;
 
@@ -572,7 +600,7 @@ static HRESULT interp_var_set(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    12.6.4 */
 static HRESULT interp_forin(exec_ctx_t *ctx)
 {
-    const HRESULT arg = ctx->code->instrs[ctx->ip].arg1.uint;
+    const HRESULT arg = get_op_uint(ctx, 0);
     IDispatch *var_obj, *obj = NULL;
     IDispatchEx *dispex;
     DISPID id, var_id;
@@ -676,7 +704,7 @@ static HRESULT interp_pop_scope(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    12.13 */
 static HRESULT interp_case(exec_ctx_t *ctx)
 {
-    const unsigned arg = ctx->code->instrs[ctx->ip].arg1.uint;
+    const unsigned arg = get_op_uint(ctx, 0);
     VARIANT *v;
     BOOL b;
     HRESULT hres;
@@ -709,7 +737,7 @@ static HRESULT interp_throw(exec_ctx_t *ctx)
 
 static HRESULT interp_throw_ref(exec_ctx_t *ctx)
 {
-    const HRESULT arg = ctx->code->instrs[ctx->ip].arg1.uint;
+    const HRESULT arg = get_op_uint(ctx, 0);
 
     TRACE("%08x\n", arg);
 
@@ -718,8 +746,8 @@ static HRESULT interp_throw_ref(exec_ctx_t *ctx)
 
 static HRESULT interp_throw_type(exec_ctx_t *ctx)
 {
-    const HRESULT hres = ctx->code->instrs[ctx->ip].arg1.uint;
-    const WCHAR *str = ctx->code->instrs[ctx->ip].arg2.str;
+    const HRESULT hres = get_op_uint(ctx, 0);
+    const WCHAR *str = get_op_str(ctx, 1);
 
     TRACE("%08x %s\n", hres, debugstr_w(str));
 
@@ -729,8 +757,8 @@ static HRESULT interp_throw_type(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    12.14 */
 static HRESULT interp_push_except(exec_ctx_t *ctx)
 {
-    const unsigned arg1 = ctx->code->instrs[ctx->ip].arg1.uint;
-    const BSTR arg2 = ctx->code->instrs[ctx->ip].arg2.bstr;
+    const unsigned arg1 = get_op_uint(ctx, 0);
+    const BSTR arg2 = get_op_bstr(ctx, 1);
     except_frame_t *except;
     unsigned stack_top;
 
@@ -803,15 +831,15 @@ static HRESULT interp_end_finally(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    13 */
 static HRESULT interp_func(exec_ctx_t *ctx)
 {
-    function_expression_t *expr = ctx->code->instrs[ctx->ip].arg1.func;
+    unsigned func_idx = get_op_uint(ctx, 0);
     jsdisp_t *dispex;
     VARIANT v;
     HRESULT hres;
 
-    TRACE("\n");
+    TRACE("%d\n", func_idx);
 
-    hres = create_source_function(ctx->script, ctx->code, expr->parameter_list, expr->source_elements, ctx->scope_chain,
-            expr->src_str, expr->src_len, &dispex);
+    hres = create_source_function(ctx->script, ctx->code, ctx->func_code->funcs+func_idx,
+            ctx->scope_chain, &dispex);
     if(FAILED(hres))
         return hres;
 
@@ -863,7 +891,7 @@ static HRESULT interp_array(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.2.1 */
 static HRESULT interp_member(exec_ctx_t *ctx)
 {
-    const BSTR arg = ctx->code->instrs[ctx->ip].arg1.bstr;
+    const BSTR arg = get_op_bstr(ctx, 0);
     IDispatch *obj;
     VARIANT v;
     DISPID id;
@@ -893,7 +921,7 @@ static HRESULT interp_member(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.2.1 */
 static HRESULT interp_memberid(exec_ctx_t *ctx)
 {
-    const unsigned arg = ctx->code->instrs[ctx->ip].arg1.lng;
+    const unsigned arg = get_op_uint(ctx, 0);
     VARIANT *objv, *namev;
     IDispatch *obj;
     BSTR name;
@@ -952,32 +980,11 @@ static HRESULT interp_refval(exec_ctx_t *ctx)
     return stack_push(ctx, &v);
 }
 
-static void jsstack_to_dp(exec_ctx_t *ctx, unsigned arg_cnt, DISPPARAMS *dp)
-{
-    VARIANT tmp;
-    unsigned i;
-
-    dp->cArgs = arg_cnt;
-    dp->rgdispidNamedArgs = NULL;
-    dp->cNamedArgs = 0;
-
-    assert(ctx->top >= arg_cnt);
-
-    for(i=1; i*2 <= arg_cnt; i++) {
-        tmp = ctx->stack[ctx->top-i];
-        ctx->stack[ctx->top-i] = ctx->stack[ctx->top-arg_cnt+i-1];
-        ctx->stack[ctx->top-arg_cnt+i-1] = tmp;
-    }
-
-    dp->rgvarg = ctx->stack + ctx->top-arg_cnt;
-}
-
 /* ECMA-262 3rd Edition    11.2.2 */
 static HRESULT interp_new(exec_ctx_t *ctx)
 {
-    const LONG arg = ctx->code->instrs[ctx->ip].arg1.lng;
+    const unsigned arg = get_op_uint(ctx, 0);
     VARIANT *constr, v;
-    DISPPARAMS dp;
     HRESULT hres;
 
     TRACE("%d\n", arg);
@@ -993,9 +1000,7 @@ static HRESULT interp_new(exec_ctx_t *ctx)
     else if(!V_DISPATCH(constr))
         return throw_type_error(ctx->script, ctx->ei, JS_E_INVALID_PROPERTY, NULL);
 
-    jsstack_to_dp(ctx, arg, &dp);
-    hres = disp_call(ctx->script, V_DISPATCH(constr), DISPID_VALUE,
-            DISPATCH_CONSTRUCT, &dp, &v, ctx->ei);
+    hres = disp_call_value(ctx->script, V_DISPATCH(constr), NULL, DISPATCH_CONSTRUCT, arg, stack_args(ctx, arg), &v, ctx->ei);
     if(FAILED(hres))
         return hres;
 
@@ -1006,10 +1011,9 @@ static HRESULT interp_new(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.2.3 */
 static HRESULT interp_call(exec_ctx_t *ctx)
 {
-    const unsigned argn = ctx->code->instrs[ctx->ip].arg1.uint;
-    const int do_ret = ctx->code->instrs[ctx->ip].arg2.lng;
+    const unsigned argn = get_op_uint(ctx, 0);
+    const int do_ret = get_op_int(ctx, 1);
     VARIANT v, *objv;
-    DISPPARAMS dp;
     HRESULT hres;
 
     TRACE("%d %d\n", argn, do_ret);
@@ -1018,8 +1022,7 @@ static HRESULT interp_call(exec_ctx_t *ctx)
     if(V_VT(objv) != VT_DISPATCH)
         return throw_type_error(ctx->script, ctx->ei, JS_E_INVALID_PROPERTY, NULL);
 
-    jsstack_to_dp(ctx, argn, &dp);
-    hres = disp_call(ctx->script, V_DISPATCH(objv), DISPID_VALUE, DISPATCH_METHOD, &dp,
+    hres = disp_call_value(ctx->script, V_DISPATCH(objv), NULL, DISPATCH_METHOD, argn, stack_args(ctx, argn),
             do_ret ? &v : NULL, ctx->ei);
     if(FAILED(hres))
         return hres;
@@ -1032,10 +1035,9 @@ static HRESULT interp_call(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.2.3 */
 static HRESULT interp_call_member(exec_ctx_t *ctx)
 {
-    const unsigned argn = ctx->code->instrs[ctx->ip].arg1.uint;
-    const int do_ret = ctx->code->instrs[ctx->ip].arg2.lng;
+    const unsigned argn = get_op_uint(ctx, 0);
+    const int do_ret = get_op_int(ctx, 1);
     IDispatch *obj;
-    DISPPARAMS dp;
     VARIANT v;
     DISPID id;
     HRESULT hres;
@@ -1046,8 +1048,7 @@ static HRESULT interp_call_member(exec_ctx_t *ctx)
     if(!obj)
         return throw_type_error(ctx->script, ctx->ei, id, NULL);
 
-    jsstack_to_dp(ctx, argn, &dp);
-    hres = disp_call(ctx->script, obj, id, DISPATCH_METHOD, &dp, do_ret ? &v : NULL, ctx->ei);
+    hres = disp_call(ctx->script, obj, id, DISPATCH_METHOD, argn, stack_args(ctx, argn), do_ret ? &v : NULL, ctx->ei);
     if(FAILED(hres))
         return hres;
 
@@ -1072,7 +1073,7 @@ static HRESULT interp_this(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    10.1.4 */
 static HRESULT interp_ident(exec_ctx_t *ctx)
 {
-    const BSTR arg = ctx->code->instrs[ctx->ip].arg1.bstr;
+    const BSTR arg = get_op_bstr(ctx, 0);
     exprval_t exprval;
     VARIANT v;
     HRESULT hres;
@@ -1097,8 +1098,8 @@ static HRESULT interp_ident(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    10.1.4 */
 static HRESULT interp_identid(exec_ctx_t *ctx)
 {
-    const BSTR arg = ctx->code->instrs[ctx->ip].arg1.bstr;
-    const unsigned flags = ctx->code->instrs[ctx->ip].arg2.uint;
+    const BSTR arg = get_op_bstr(ctx, 0);
+    const unsigned flags = get_op_uint(ctx, 1);
     exprval_t exprval;
     HRESULT hres;
 
@@ -1141,7 +1142,7 @@ static HRESULT interp_null(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    7.8.2 */
 static HRESULT interp_bool(exec_ctx_t *ctx)
 {
-    const LONG arg = ctx->code->instrs[ctx->ip].arg1.lng;
+    const int arg = get_op_int(ctx, 0);
 
     TRACE("%s\n", arg ? "true" : "false");
 
@@ -1151,7 +1152,7 @@ static HRESULT interp_bool(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    7.8.3 */
 static HRESULT interp_int(exec_ctx_t *ctx)
 {
-    const LONG arg = ctx->code->instrs[ctx->ip].arg1.lng;
+    const int arg = get_op_int(ctx, 0);
     VARIANT v;
 
     TRACE("%d\n", arg);
@@ -1164,20 +1165,17 @@ static HRESULT interp_int(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    7.8.3 */
 static HRESULT interp_double(exec_ctx_t *ctx)
 {
-    const double arg = *ctx->code->instrs[ctx->ip].arg1.dbl;
-    VARIANT v;
+    const double arg = get_op_double(ctx);
 
     TRACE("%lf\n", arg);
 
-    V_VT(&v) = VT_R8;
-    V_R8(&v) = arg;
-    return stack_push(ctx, &v);
+    return stack_push_number(ctx, arg);
 }
 
 /* ECMA-262 3rd Edition    7.8.4 */
 static HRESULT interp_str(exec_ctx_t *ctx)
 {
-    const WCHAR *str = ctx->code->instrs[ctx->ip].arg1.str;
+    const WCHAR *str = get_op_str(ctx, 0);
     VARIANT v;
 
     TRACE("%s\n", debugstr_w(str));
@@ -1193,8 +1191,8 @@ static HRESULT interp_str(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    7.8 */
 static HRESULT interp_regexp(exec_ctx_t *ctx)
 {
-    const WCHAR *source = ctx->code->instrs[ctx->ip].arg1.str;
-    const LONG flags = ctx->code->instrs[ctx->ip].arg2.lng;
+    const WCHAR *source = get_op_str(ctx, 0);
+    const unsigned flags = get_op_uint(ctx, 1);
     jsdisp_t *regexp;
     VARIANT v;
     HRESULT hres;
@@ -1212,7 +1210,7 @@ static HRESULT interp_regexp(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.1.4 */
 static HRESULT interp_carray(exec_ctx_t *ctx)
 {
-    const unsigned arg = ctx->code->instrs[ctx->ip].arg1.uint;
+    const unsigned arg = get_op_uint(ctx, 0);
     jsdisp_t *array;
     VARIANT *v, r;
     unsigned i;
@@ -1259,7 +1257,7 @@ static HRESULT interp_new_obj(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.1.5 */
 static HRESULT interp_obj_prop(exec_ctx_t *ctx)
 {
-    const BSTR name = ctx->code->instrs[ctx->ip].arg1.bstr;
+    const BSTR name = get_op_bstr(ctx, 0);
     jsdisp_t *obj;
     VARIANT *v;
     HRESULT hres;
@@ -1279,7 +1277,7 @@ static HRESULT interp_obj_prop(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.11 */
 static HRESULT interp_cnd_nz(exec_ctx_t *ctx)
 {
-    const unsigned arg = ctx->code->instrs[ctx->ip].arg1.uint;
+    const unsigned arg = get_op_uint(ctx, 0);
     VARIANT_BOOL b;
     HRESULT hres;
 
@@ -1301,7 +1299,7 @@ static HRESULT interp_cnd_nz(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.11 */
 static HRESULT interp_cnd_z(exec_ctx_t *ctx)
 {
-    const unsigned arg = ctx->code->instrs[ctx->ip].arg1.uint;
+    const unsigned arg = get_op_uint(ctx, 0);
     VARIANT_BOOL b;
     HRESULT hres;
 
@@ -1526,13 +1524,13 @@ static HRESULT add_eval(script_ctx_t *ctx, VARIANT *lval, VARIANT *rval, jsexcep
         if(V_VT(&r) != VT_BSTR)
             SysFreeString(rstr);
     }else {
-        VARIANT nl, nr;
+        double nl, nr;
 
         hres = to_number(ctx, &l, ei, &nl);
         if(SUCCEEDED(hres)) {
             hres = to_number(ctx, &r, ei, &nr);
             if(SUCCEEDED(hres))
-                num_set_val(retv, num_val(&nl) + num_val(&nr));
+                num_set_val(retv, nl + nr);
         }
     }
 
@@ -1564,7 +1562,7 @@ static HRESULT interp_add(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.6.2 */
 static HRESULT interp_sub(exec_ctx_t *ctx)
 {
-    VARIANT l, r;
+    double l, r;
     HRESULT hres;
 
     TRACE("\n");
@@ -1577,13 +1575,13 @@ static HRESULT interp_sub(exec_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    return stack_push_number(ctx, num_val(&l)-num_val(&r));
+    return stack_push_number(ctx, l-r);
 }
 
 /* ECMA-262 3rd Edition    11.5.1 */
 static HRESULT interp_mul(exec_ctx_t *ctx)
 {
-    VARIANT l, r;
+    double l, r;
     HRESULT hres;
 
     TRACE("\n");
@@ -1596,13 +1594,13 @@ static HRESULT interp_mul(exec_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    return stack_push_number(ctx, num_val(&l)*num_val(&r));
+    return stack_push_number(ctx, l*r);
 }
 
 /* ECMA-262 3rd Edition    11.5.2 */
 static HRESULT interp_div(exec_ctx_t *ctx)
 {
-    VARIANT l, r;
+    double l, r;
     HRESULT hres;
 
     TRACE("\n");
@@ -1615,13 +1613,13 @@ static HRESULT interp_div(exec_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    return stack_push_number(ctx, num_val(&l)/num_val(&r));
+    return stack_push_number(ctx, l/r);
 }
 
 /* ECMA-262 3rd Edition    11.5.3 */
 static HRESULT interp_mod(exec_ctx_t *ctx)
 {
-    VARIANT l, r;
+    double l, r;
     HRESULT hres;
 
     TRACE("\n");
@@ -1634,7 +1632,7 @@ static HRESULT interp_mod(exec_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    return stack_push_number(ctx, fmod(num_val(&l), num_val(&r)));
+    return stack_push_number(ctx, fmod(l, r));
 }
 
 /* ECMA-262 3rd Edition    11.4.2 */
@@ -1687,7 +1685,7 @@ static HRESULT interp_delete(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.4.2 */
 static HRESULT interp_delete_ident(exec_ctx_t *ctx)
 {
-    const BSTR arg = ctx->code->instrs[ctx->ip].arg1.bstr;
+    const BSTR arg = get_op_bstr(ctx, 0);
     IDispatchEx *dispex;
     exprval_t exprval;
     BOOL ret = FALSE;
@@ -1805,7 +1803,7 @@ static HRESULT interp_typeofid(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.4.3 */
 static HRESULT interp_typeofident(exec_ctx_t *ctx)
 {
-    const BSTR arg = ctx->code->instrs[ctx->ip].arg1.bstr;
+    const BSTR arg = get_op_bstr(ctx, 0);
     exprval_t exprval;
     const WCHAR *ret;
     VARIANT v;
@@ -1857,7 +1855,7 @@ static HRESULT interp_typeof(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.4.7 */
 static HRESULT interp_minus(exec_ctx_t *ctx)
 {
-    VARIANT n;
+    double n;
     HRESULT hres;
 
     TRACE("\n");
@@ -1866,30 +1864,31 @@ static HRESULT interp_minus(exec_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    return stack_push_number(ctx, -num_val(&n));
+    return stack_push_number(ctx, -n);
 }
 
 /* ECMA-262 3rd Edition    11.4.6 */
 static HRESULT interp_tonum(exec_ctx_t *ctx)
 {
-    VARIANT *v, num;
+    VARIANT *v;
+    double n;
     HRESULT hres;
 
     TRACE("\n");
 
     v = stack_pop(ctx);
-    hres = to_number(ctx->script, v, ctx->ei, &num);
+    hres = to_number(ctx->script, v, ctx->ei, &n);
     VariantClear(v);
     if(FAILED(hres))
         return hres;
 
-    return stack_push(ctx, &num);
+    return stack_push_number(ctx, n);
 }
 
 /* ECMA-262 3rd Edition    11.3.1 */
 static HRESULT interp_postinc(exec_ctx_t *ctx)
 {
-    const int arg = ctx->code->instrs[ctx->ip].arg1.lng;
+    const int arg = get_op_int(ctx, 0);
     IDispatch *obj;
     DISPID id;
     VARIANT v;
@@ -1903,11 +1902,12 @@ static HRESULT interp_postinc(exec_ctx_t *ctx)
 
     hres = disp_propget(ctx->script, obj, id, &v, ctx->ei);
     if(SUCCEEDED(hres)) {
-        VARIANT n, inc;
+        VARIANT inc;
+        double n;
 
         hres = to_number(ctx->script, &v, ctx->ei, &n);
         if(SUCCEEDED(hres)) {
-            num_set_val(&inc, num_val(&n)+(double)arg);
+            num_set_val(&inc, n+(double)arg);
             hres = disp_propput(ctx->script, obj, id, &inc, ctx->ei);
         }
         if(FAILED(hres))
@@ -1923,7 +1923,7 @@ static HRESULT interp_postinc(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.4.4, 11.4.5 */
 static HRESULT interp_preinc(exec_ctx_t *ctx)
 {
-    const int arg = ctx->code->instrs[ctx->ip].arg1.lng;
+    const int arg = get_op_int(ctx, 0);
     IDispatch *obj;
     DISPID id;
     VARIANT v;
@@ -1937,12 +1937,12 @@ static HRESULT interp_preinc(exec_ctx_t *ctx)
 
     hres = disp_propget(ctx->script, obj, id, &v, ctx->ei);
     if(SUCCEEDED(hres)) {
-        VARIANT n;
+        double n;
 
         hres = to_number(ctx->script, &v, ctx->ei, &n);
         VariantClear(&v);
         if(SUCCEEDED(hres)) {
-            num_set_val(&v, num_val(&n)+(double)arg);
+            num_set_val(&v, n+(double)arg);
             hres = disp_propput(ctx->script, obj, id, &v, ctx->ei);
         }
     }
@@ -1980,39 +1980,45 @@ static HRESULT equal_values(script_ctx_t *ctx, VARIANT *lval, VARIANT *rval, jse
 
     if(V_VT(lval) == VT_BSTR && is_num_vt(V_VT(rval))) {
         VARIANT v;
+        double n;
         HRESULT hres;
 
-        hres = to_number(ctx, lval, ei, &v);
+        hres = to_number(ctx, lval, ei, &n);
         if(FAILED(hres))
             return hres;
 
+        /* FIXME: optimize */
+        num_set_val(&v, n);
+
         return equal_values(ctx, &v, rval, ei, ret);
     }
 
     if(V_VT(rval) == VT_BSTR && is_num_vt(V_VT(lval))) {
         VARIANT v;
+        double n;
         HRESULT hres;
 
-        hres = to_number(ctx, rval, ei, &v);
+        hres = to_number(ctx, rval, ei, &n);
         if(FAILED(hres))
             return hres;
 
+        /* FIXME: optimize */
+        num_set_val(&v, n);
+
         return equal_values(ctx, lval, &v, ei, ret);
     }
 
     if(V_VT(rval) == VT_BOOL) {
         VARIANT v;
 
-        V_VT(&v) = VT_I4;
-        V_I4(&v) = V_BOOL(rval) ? 1 : 0;
+        num_set_int(&v, V_BOOL(rval) ? 1 : 0);
         return equal_values(ctx, lval, &v, ei, ret);
     }
 
     if(V_VT(lval) == VT_BOOL) {
         VARIANT v;
 
-        V_VT(&v) = VT_I4;
-        V_I4(&v) = V_BOOL(lval) ? 1 : 0;
+        num_set_int(&v, V_BOOL(lval) ? 1 : 0);
         return equal_values(ctx, &v, rval, ei, ret);
     }
 
@@ -2138,7 +2144,8 @@ static HRESULT interp_neq2(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.8.5 */
 static HRESULT less_eval(script_ctx_t *ctx, VARIANT *lval, VARIANT *rval, BOOL greater, jsexcept_t *ei, BOOL *ret)
 {
-    VARIANT l, r, ln, rn;
+    double ln, rn;
+    VARIANT l, r;
     HRESULT hres;
 
     hres = to_primitive(ctx, lval, ei, &l, NO_HINT);
@@ -2166,15 +2173,7 @@ static HRESULT less_eval(script_ctx_t *ctx, VARIANT *lval, VARIANT *rval, BOOL g
     if(FAILED(hres))
         return hres;
 
-    if(V_VT(&ln) == VT_I4 && V_VT(&rn) == VT_I4) {
-        *ret = (V_I4(&ln) < V_I4(&rn)) ^ greater;
-    }else  {
-        DOUBLE ld = num_val(&ln);
-        DOUBLE rd = num_val(&rn);
-
-        *ret = !isnan(ld) && !isnan(rd) && ((ld < rd) ^ greater);
-    }
-
+    *ret = !isnan(ln) && !isnan(rn) && ((ln < rn) ^ greater);
     return S_OK;
 }
 
@@ -2265,7 +2264,7 @@ static HRESULT interp_gteq(exec_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.4.8 */
 static HRESULT interp_bneg(exec_ctx_t *ctx)
 {
-    VARIANT *v, r;
+    VARIANT *v;
     INT i;
     HRESULT hres;
 
@@ -2277,9 +2276,7 @@ static HRESULT interp_bneg(exec_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    V_VT(&r) = VT_I4;
-    V_I4(&r) = ~i;
-    return stack_push(ctx, &r);
+    return stack_push_int(ctx, ~i);
 }
 
 /* ECMA-262 3rd Edition    11.4.9 */
@@ -2379,6 +2376,30 @@ static HRESULT interp_assign(exec_ctx_t *ctx)
     return stack_push(ctx, v);
 }
 
+/* JScript extension */
+static HRESULT interp_assign_call(exec_ctx_t *ctx)
+{
+    const unsigned arg = get_op_uint(ctx, 0);
+    IDispatch *disp;
+    VARIANT *v;
+    DISPID id;
+    HRESULT hres;
+
+    TRACE("%u\n", arg);
+
+    disp = stack_topn_objid(ctx, arg+1, &id);
+    if(!disp)
+        return throw_reference_error(ctx->script, ctx->ei, JS_E_ILLEGAL_ASSIGN, NULL);
+
+    hres = disp_call(ctx->script, disp, id, DISPATCH_PROPERTYPUT, arg+1, stack_args(ctx,arg+1), NULL, ctx->ei);
+    if(FAILED(hres))
+        return hres;
+
+    v = stack_pop(ctx);
+    stack_popn(ctx, arg+2);
+    return stack_push(ctx, v);
+}
+
 static HRESULT interp_undefined(exec_ctx_t *ctx)
 {
     VARIANT v;
@@ -2391,9 +2412,9 @@ static HRESULT interp_undefined(exec_ctx_t *ctx)
 
 static HRESULT interp_jmp(exec_ctx_t *ctx)
 {
-    const unsigned arg = ctx->code->instrs[ctx->ip].arg1.uint;
+    const unsigned arg = get_op_uint(ctx, 0);
 
-    TRACE("\n");
+    TRACE("%u\n", arg);
 
     ctx->ip = arg;
     return S_OK;
@@ -2401,7 +2422,7 @@ static HRESULT interp_jmp(exec_ctx_t *ctx)
 
 static HRESULT interp_jmp_z(exec_ctx_t *ctx)
 {
-    const unsigned arg = ctx->code->instrs[ctx->ip].arg1.uint;
+    const unsigned arg = get_op_uint(ctx, 0);
     VARIANT_BOOL b;
     VARIANT *v;
     HRESULT hres;
@@ -2508,10 +2529,11 @@ static HRESULT unwind_exception(exec_ctx_t *ctx)
     return hres;
 }
 
-static HRESULT enter_bytecode(script_ctx_t *ctx, bytecode_t *code, unsigned ip, jsexcept_t *ei, VARIANT *ret)
+static HRESULT enter_bytecode(script_ctx_t *ctx, bytecode_t *code, function_code_t *func, jsexcept_t *ei, VARIANT *ret)
 {
     exec_ctx_t *exec_ctx = ctx->exec_ctx;
     except_frame_t *prev_except_frame;
+    function_code_t *prev_func;
     unsigned prev_ip, prev_top;
     scope_chain_t *prev_scope;
     bytecode_t *prev_code;
@@ -2527,10 +2549,12 @@ static HRESULT enter_bytecode(script_ctx_t *ctx, bytecode_t *code, unsigned ip,
     prev_ip = exec_ctx->ip;
     prev_ei = exec_ctx->ei;
     prev_code = exec_ctx->code;
-    exec_ctx->ip = ip;
+    prev_func = exec_ctx->func_code;
+    exec_ctx->ip = func->instr_off;
     exec_ctx->ei = ei;
     exec_ctx->except_frame = NULL;
     exec_ctx->code = code;
+    exec_ctx->func_code = func;
 
     while(exec_ctx->ip != -1) {
         op = code->instrs[exec_ctx->ip].op;
@@ -2553,6 +2577,7 @@ static HRESULT enter_bytecode(script_ctx_t *ctx, bytecode_t *code, unsigned ip,
     exec_ctx->ei = prev_ei;
     exec_ctx->except_frame = prev_except_frame;
     exec_ctx->code = prev_code;
+    exec_ctx->func_code = prev_func;
 
     if(FAILED(hres)) {
         while(exec_ctx->scope_chain != prev_scope)
@@ -2571,66 +2596,50 @@ static HRESULT enter_bytecode(script_ctx_t *ctx, bytecode_t *code, unsigned ip,
     return S_OK;
 }
 
-HRESULT exec_source(exec_ctx_t *ctx, bytecode_t *code, source_elements_t *source, BOOL from_eval,
+HRESULT exec_source(exec_ctx_t *ctx, bytecode_t *code, function_code_t *func, BOOL from_eval,
         jsexcept_t *ei, VARIANT *retv)
 {
-    function_declaration_t *func;
-    var_list_t *var;
-    VARIANT val;
     exec_ctx_t *prev_ctx;
+    VARIANT val;
+    unsigned i;
     HRESULT hres = S_OK;
 
-    for(func = source->functions; func; func = func->next) {
+    for(i = 0; i < func->func_cnt; i++) {
         jsdisp_t *func_obj;
         VARIANT var;
 
-        if(!func->expr->identifier)
+        if(!func->funcs[i].name)
             continue;
 
-        hres = create_source_function(ctx->script, code, func->expr->parameter_list, func->expr->source_elements,
-                ctx->scope_chain, func->expr->src_str, func->expr->src_len, &func_obj);
+        hres = create_source_function(ctx->script, code, func->funcs+i, ctx->scope_chain, &func_obj);
         if(FAILED(hres))
             return hres;
 
         var_set_jsdisp(&var, func_obj);
-        hres = jsdisp_propput_name(ctx->var_disp, func->expr->identifier, &var, ei);
+        hres = jsdisp_propput_name(ctx->var_disp, func->funcs[i].name, &var, ei);
         jsdisp_release(func_obj);
         if(FAILED(hres))
             return hres;
     }
 
-    for(var = source->variables; var; var = var->next) {
-        DISPID id = 0;
-        BSTR name;
+    for(i=0; i < func->var_cnt; i++) {
+        if(!ctx->is_global || !lookup_global_members(ctx->script, func->variables[i], NULL)) {
+            DISPID id = 0;
 
-        name = SysAllocString(var->identifier);
-        if(!name)
-            return E_OUTOFMEMORY;
-
-        if(!ctx->is_global || !lookup_global_members(ctx->script, name, NULL))
-            hres = jsdisp_get_id(ctx->var_disp, var->identifier, fdexNameEnsure, &id);
-        SysFreeString(name);
-        if(FAILED(hres))
-            return hres;
+            hres = jsdisp_get_id(ctx->var_disp, func->variables[i], fdexNameEnsure, &id);
+            if(FAILED(hres))
+                return hres;
+        }
     }
 
     prev_ctx = ctx->script->exec_ctx;
     ctx->script->exec_ctx = ctx;
 
-    if(source->statement) {
-        assert(source->instr_off);
-        hres = enter_bytecode(ctx->script, code, source->instr_off, ei, &val);
-    }else {
-        V_VT(&val) = VT_EMPTY;
-    }
-
+    hres = enter_bytecode(ctx->script, code, func, ei, &val);
     assert(ctx->script->exec_ctx == ctx);
     ctx->script->exec_ctx = prev_ctx;
-
-    if(FAILED(hres)) {
-        VariantClear(&val);
+    if(FAILED(hres))
         return hres;
-    }
 
     if(retv)
         *retv = val;
