@@ -126,13 +126,18 @@ void *jsheap_alloc(jsheap_t *heap, DWORD size)
 
 void *jsheap_grow(jsheap_t *heap, void *mem, DWORD size, DWORD inc)
 {
+    void *ret;
+
     if(mem == (BYTE*)heap->blocks[heap->last_block] + heap->offset-size
        && heap->offset+inc < block_size(heap->last_block)) {
         heap->offset += inc;
         return mem;
     }
 
-    return jsheap_alloc(heap, size+inc);
+    ret = jsheap_alloc(heap, size+inc);
+    if(ret) /* FIXME: avoid copying for custom blocks */
+        memcpy(ret, mem, size);
+    return ret;
 }
 
 void jsheap_clear(jsheap_t *heap)
@@ -198,7 +203,6 @@ HRESULT to_primitive(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, VARIANT *ret
     case VT_DISPATCH: {
         jsdisp_t *jsdisp;
         DISPID id;
-        DISPPARAMS dp = {NULL, NULL, 0, 0};
         HRESULT hres;
 
         static const WCHAR toStringW[] = {'t','o','S','t','r','i','n','g',0};
@@ -222,7 +226,7 @@ HRESULT to_primitive(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, VARIANT *ret
 
         hres = jsdisp_get_id(jsdisp, hint == HINT_STRING ? toStringW : valueOfW, 0, &id);
         if(SUCCEEDED(hres)) {
-            hres = jsdisp_call(jsdisp, id, DISPATCH_METHOD, &dp, ret, ei);
+            hres = jsdisp_call(jsdisp, id, DISPATCH_METHOD, 0, NULL, ret, ei);
             if(FAILED(hres)) {
                 WARN("call error - forwarding exception\n");
                 jsdisp_release(jsdisp);
@@ -238,7 +242,7 @@ HRESULT to_primitive(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, VARIANT *ret
 
         hres = jsdisp_get_id(jsdisp, hint == HINT_STRING ? valueOfW : toStringW, 0, &id);
         if(SUCCEEDED(hres)) {
-            hres = jsdisp_call(jsdisp, id, DISPATCH_METHOD, &dp, ret, ei);
+            hres = jsdisp_call(jsdisp, id, DISPATCH_METHOD, 0, NULL, ret, ei);
             if(FAILED(hres)) {
                 WARN("call error - forwarding exception\n");
                 jsdisp_release(jsdisp);
@@ -257,6 +261,9 @@ HRESULT to_primitive(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, VARIANT *ret
         WARN("failed\n");
         return throw_type_error(ctx, ei, JS_E_TO_PRIMITIVE, NULL);
     }
+    case VT_I2:
+    case VT_INT:
+        assert(0);
     default:
         FIXME("Unimplemented for vt %d\n", V_VT(v));
         return E_NOTIMPL;
@@ -312,7 +319,7 @@ static int hex_to_int(WCHAR c)
 }
 
 /* ECMA-262 3rd Edition    9.3.1 */
-static HRESULT str_to_number(BSTR str, VARIANT *ret)
+static HRESULT str_to_number(BSTR str, double *ret)
 {
     const WCHAR *ptr = str;
     BOOL neg = FALSE;
@@ -320,6 +327,11 @@ static HRESULT str_to_number(BSTR str, VARIANT *ret)
 
     static const WCHAR infinityW[] = {'I','n','f','i','n','i','t','y'};
 
+    if(!ptr) {
+        *ret = 0;
+        return S_OK;
+    }
+
     while(isspaceW(*ptr))
         ptr++;
 
@@ -336,9 +348,9 @@ static HRESULT str_to_number(BSTR str, VARIANT *ret)
             ptr++;
 
         if(*ptr)
-            num_set_nan(ret);
+            *ret = NAN;
         else
-            num_set_inf(ret, !neg);
+            *ret = neg ? -INFINITY : INFINITY;
         return S_OK;
     }
 
@@ -351,7 +363,7 @@ static HRESULT str_to_number(BSTR str, VARIANT *ret)
             ptr++;
         }
 
-        num_set_val(ret, d);
+        *ret = d;
         return S_OK;
     }
 
@@ -390,31 +402,32 @@ static HRESULT str_to_number(BSTR str, VARIANT *ret)
         ptr++;
 
     if(*ptr) {
-        num_set_nan(ret);
+        *ret = NAN;
         return S_OK;
     }
 
     if(neg)
         d = -d;
 
-    num_set_val(ret, d);
+    *ret = d;
     return S_OK;
 }
 
 /* ECMA-262 3rd Edition    9.3 */
-HRESULT to_number(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, VARIANT *ret)
+HRESULT to_number(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, double *ret)
 {
     switch(V_VT(v)) {
     case VT_EMPTY:
-        num_set_nan(ret);
+        *ret = NAN;
         break;
     case VT_NULL:
-        V_VT(ret) = VT_I4;
-        V_I4(ret) = 0;
+        *ret = 0;
         break;
     case VT_I4:
+        *ret = V_I4(v);
+        break;
     case VT_R8:
-        *ret = *v;
+        *ret = V_R8(v);
         break;
     case VT_BSTR:
         return str_to_number(V_BSTR(v), ret);
@@ -431,9 +444,11 @@ HRESULT to_number(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, VARIANT *ret)
         return hres;
     }
     case VT_BOOL:
-        V_VT(ret) = VT_I4;
-        V_I4(ret) = V_BOOL(v) ? 1 : 0;
+        *ret = V_BOOL(v) ? 1 : 0;
         break;
+    case VT_I2:
+    case VT_INT:
+        assert(0);
     default:
         FIXME("unimplemented for vt %d\n", V_VT(v));
         return E_NOTIMPL;
@@ -443,62 +458,66 @@ HRESULT to_number(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, VARIANT *ret)
 }
 
 /* ECMA-262 3rd Edition    9.4 */
-HRESULT to_integer(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, VARIANT *ret)
+HRESULT to_integer(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, double *ret)
 {
-    VARIANT num;
+    double n;
     HRESULT hres;
 
-    hres = to_number(ctx, v, ei, &num);
+    if(V_VT(v) == VT_I4) {
+        *ret = V_I4(v);
+        return S_OK;
+    }
+
+    hres = to_number(ctx, v, ei, &n);
     if(FAILED(hres))
         return hres;
 
-    if(V_VT(&num) == VT_I4) {
-        *ret = num;
-    }else if(isnan(V_R8(&num))) {
-        V_VT(ret) = VT_I4;
-        V_I4(ret) = 0;
-    }else {
-        num_set_val(ret, V_R8(&num) >= 0.0 ? floor(V_R8(&num)) : -floor(-V_R8(&num)));
-    }
-
+    if(isnan(n))
+        *ret = 0;
+    else
+        *ret = n >= 0.0 ? floor(n) : -floor(-n);
     return S_OK;
 }
 
 /* ECMA-262 3rd Edition    9.5 */
 HRESULT to_int32(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, INT *ret)
 {
-    VARIANT num;
+    double n;
     HRESULT hres;
 
-    hres = to_number(ctx, v, ei, &num);
+    if(V_VT(v) == VT_I4) {
+        *ret = V_I4(v);
+        return S_OK;
+    }
+
+    hres = to_number(ctx, v, ei, &n);
     if(FAILED(hres))
         return hres;
 
-    if(V_VT(&num) == VT_I4)
-        *ret = V_I4(&num);
-    else
-        *ret = isnan(V_R8(&num)) || isinf(V_R8(&num)) ? 0 : (INT)V_R8(&num);
+    *ret = isnan(n) || isinf(n) ? 0 : n;
     return S_OK;
 }
 
 /* ECMA-262 3rd Edition    9.6 */
 HRESULT to_uint32(script_ctx_t *ctx, VARIANT *v, jsexcept_t *ei, DWORD *ret)
 {
-    VARIANT num;
+    double n;
     HRESULT hres;
 
-    hres = to_number(ctx, v, ei, &num);
+    if(V_VT(v) == VT_I4) {
+        *ret = V_I4(v);
+        return S_OK;
+    }
+
+    hres = to_number(ctx, v, ei, &n);
     if(FAILED(hres))
         return hres;
 
-    if(V_VT(&num) == VT_I4)
-        *ret = V_I4(&num);
-    else
-        *ret = isnan(V_R8(&num)) || isinf(V_R8(&num)) ? 0 : (DWORD)V_R8(&num);
+    *ret = isnan(n) || isinf(n) ? 0 : n;
     return S_OK;
 }
 
-BSTR int_to_bstr(int i)
+static BSTR int_to_bstr(int i)
 {
     WCHAR buf[12], *p;
     BOOL neg = FALSE;
@@ -616,7 +635,7 @@ HRESULT to_object(script_ctx_t *ctx, VARIANT *v, IDispatch **disp)
         break;
     case VT_I4:
     case VT_R8:
-        hres = create_number(ctx, v, &dispex);
+        hres = create_number(ctx, num_val(v), &dispex);
         if(FAILED(hres))
             return hres;
 
@@ -679,17 +698,19 @@ HRESULT variant_change_type(script_ctx_t *ctx, VARIANT *dst, VARIANT *src, VARTY
         }
         break;
     }
-    case VT_R8:
-        hres = to_number(ctx, src, &ei, dst);
-        if(SUCCEEDED(hres) && V_VT(dst) == VT_I4)
-            V_R8(dst) = V_I4(dst);
+    case VT_R8: {
+        double n;
+        hres = to_number(ctx, src, &ei, &n);
+        if(SUCCEEDED(hres))
+            V_R8(dst) = n;
         break;
+    }
     case VT_R4: {
-        VARIANT n;
+        double n;
 
         hres = to_number(ctx, src, &ei, &n);
         if(SUCCEEDED(hres))
-            V_R4(dst) = num_val(&n);
+            V_R4(dst) = n;
         break;
     }
     case VT_BOOL: {
