@@ -58,276 +58,256 @@ static const WCHAR sqrtW[] = {'s','q','r','t',0};
 static const WCHAR tanW[] = {'t','a','n',0};
 
 /* ECMA-262 3rd Edition    15.8.2.12 */
-static HRESULT Math_abs(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_abs(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
-    DOUBLE d;
+    double d;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
+    if(!argc) {
         if(retv)
-            num_set_nan(retv);
+            num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &d);
     if(FAILED(hres))
         return hres;
 
-    d = num_val(&v);
     if(retv)
         num_set_val(retv, d < 0.0 ? -d : d);
     return S_OK;
 }
 
-static HRESULT Math_acos(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_acos(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        if(retv) num_set_nan(retv);
+    if(!argc) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) {
-        DOUBLE x = num_val(&v);
-        if(x < -1.0 || x > 1.0)
-            num_set_nan(retv);
-        else
-            num_set_val(retv, acos(x));
-    }
+    if(retv)
+        num_set_val(retv, x < -1.0 || x > 1.0 ? NAN : acos(x));
     return S_OK;
 }
 
-static HRESULT Math_asin(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_asin(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        if(retv) num_set_nan(retv);
+    if(!argc) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) {
-        DOUBLE x = num_val(&v);
-        if(x < -1.0 || x > 1.0)
-            num_set_nan(retv);
-        else
-            num_set_val(retv, asin(x));
-    }
+    if(retv)
+        num_set_val(retv, x < -1.0 || x > 1.0 ? NAN : asin(x));
     return S_OK;
 }
 
-static HRESULT Math_atan(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_atan(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        if(retv) num_set_nan(retv);
+    if(!argc) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) num_set_val(retv, atan(num_val(&v)));
+    if(retv) num_set_val(retv, atan(x));
     return S_OK;
 }
 
-static HRESULT Math_atan2(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_atan2(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v1, v2;
+    double x, y;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(arg_cnt(dp)<2) {
-        if(retv) num_set_nan(retv);
+    if(argc<2) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v1);
+    hres = to_number(ctx, argv, ei, &y);
     if(FAILED(hres))
         return hres;
 
-    hres = to_number(ctx, get_arg(dp, 1), ei, &v2);
+    hres = to_number(ctx, argv+1, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) num_set_val(retv, atan2(num_val(&v1), num_val(&v2)));
+    if(retv) num_set_val(retv, atan2(y, x));
     return S_OK;
 }
 
 /* ECMA-262 3rd Edition    15.8.2.6 */
-static HRESULT Math_ceil(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_ceil(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
+    if(!argc) {
         if(retv)
-            num_set_nan(retv);
+            num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
     if(retv)
-        num_set_val(retv, ceil(num_val(&v)));
+        num_set_val(retv, ceil(x));
     return S_OK;
 }
 
-static HRESULT Math_cos(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_cos(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        if(retv) num_set_nan(retv);
+    if(!argc) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) num_set_val(retv, cos(num_val(&v)));
+    if(retv) num_set_val(retv, cos(x));
     return S_OK;
 }
 
-static HRESULT Math_exp(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_exp(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        if(retv) num_set_nan(retv);
+    if(!argc) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) num_set_val(retv, exp(num_val(&v)));
+    if(retv) num_set_val(retv, exp(x));
     return S_OK;
 }
 
-static HRESULT Math_floor(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_floor(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
+    if(!argc) {
         if(retv)
-            num_set_nan(retv);
+            num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
     if(retv)
-        num_set_val(retv, floor(num_val(&v)));
+        num_set_val(retv, floor(x));
     return S_OK;
 }
 
-static HRESULT Math_log(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_log(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
+    if(!argc) {
         if(retv)
-            num_set_nan(retv);
+            num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) {
-        DOUBLE x = num_val(&v);
-        if(x < -0.0)
-            num_set_nan(retv);
-        else
-            num_set_val(retv, log(x));
-    }
+    if(retv)
+        num_set_val(retv, x < -0.0 ? NAN : log(x));
     return S_OK;
 }
 
 /* ECMA-262 3rd Edition    15.8.2.11 */
-static HRESULT Math_max(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_max(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     DOUBLE max, d;
-    VARIANT v;
     DWORD i;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
+    if(!argc) {
         if(retv)
-            num_set_inf(retv, FALSE);
+            num_set_val(retv, -INFINITY);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &max);
     if(FAILED(hres))
         return hres;
 
-    max = num_val(&v);
-    for(i=1; i < arg_cnt(dp); i++) {
-        hres = to_number(ctx, get_arg(dp, i), ei, &v);
+    for(i=1; i < argc; i++) {
+        hres = to_number(ctx, argv+i, ei, &d);
         if(FAILED(hres))
             return hres;
 
-        d = num_val(&v);
         if(d > max || isnan(d))
             max = d;
     }
@@ -338,33 +318,30 @@ static HRESULT Math_max(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARA
 }
 
 /* ECMA-262 3rd Edition    15.8.2.12 */
-static HRESULT Math_min(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_min(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     DOUBLE min, d;
-    VARIANT v;
     DWORD i;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
+    if(!argc) {
         if(retv)
-            num_set_inf(retv, TRUE);
+            num_set_val(retv, INFINITY);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &min);
     if(FAILED(hres))
         return hres;
 
-    min = num_val(&v);
-    for(i=1; i < arg_cnt(dp); i++) {
-        hres = to_number(ctx, get_arg(dp, i), ei, &v);
+    for(i=1; i < argc; i++) {
+        hres = to_number(ctx, argv+i, ei, &d);
         if(FAILED(hres))
             return hres;
 
-        d = num_val(&v);
         if(d < min || isnan(d))
             min = d;
     }
@@ -375,34 +352,34 @@ static HRESULT Math_min(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARA
 }
 
 /* ECMA-262 3rd Edition    15.8.2.13 */
-static HRESULT Math_pow(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_pow(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT x, y;
+    double x, y;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(arg_cnt(dp) < 2) {
-        if(retv) num_set_nan(retv);
+    if(argc < 2) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &x);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    hres = to_number(ctx, get_arg(dp, 1), ei, &y);
+    hres = to_number(ctx, argv+1, ei, &y);
     if(FAILED(hres))
         return hres;
 
     if(retv)
-        num_set_val(retv, pow(num_val(&x), num_val(&y)));
+        num_set_val(retv, pow(x, y));
     return S_OK;
 }
 
 /* ECMA-262 3rd Edition    15.8.2.14 */
-static HRESULT Math_random(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_random(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
     UINT r;
@@ -419,88 +396,89 @@ static HRESULT Math_random(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPP
 }
 
 /* ECMA-262 3rd Edition    15.8.2.15 */
-static HRESULT Math_round(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_round(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        num_set_nan(retv);
+    if(!argc) {
+        if(retv)
+            num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
     if(retv)
-        num_set_val(retv, floor(num_val(&v)+0.5));
+        num_set_val(retv, floor(x+0.5));
     return S_OK;
 }
 
-static HRESULT Math_sin(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_sin(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        if(retv) num_set_nan(retv);
+    if(!argc) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) num_set_val(retv, sin(num_val(&v)));
+    if(retv) num_set_val(retv, sin(x));
     return S_OK;
 }
 
-static HRESULT Math_sqrt(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_sqrt(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        if(retv) num_set_nan(retv);
+    if(!argc) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) num_set_val(retv, sqrt(num_val(&v)));
+    if(retv) num_set_val(retv, sqrt(x));
     return S_OK;
 }
 
-static HRESULT Math_tan(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, DISPPARAMS *dp,
+static HRESULT Math_tan(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, VARIANT *argv,
         VARIANT *retv, jsexcept_t *ei)
 {
-    VARIANT v;
+    double x;
     HRESULT hres;
 
     TRACE("\n");
 
-    if(!arg_cnt(dp)) {
-        if(retv) num_set_nan(retv);
+    if(!argc) {
+        if(retv) num_set_val(retv, NAN);
         return S_OK;
     }
 
-    hres = to_number(ctx, get_arg(dp, 0), ei, &v);
+    hres = to_number(ctx, argv, ei, &x);
     if(FAILED(hres))
         return hres;
 
-    if(retv) num_set_val(retv, tan(num_val(&v)));
+    if(retv) num_set_val(retv, tan(x));
     return S_OK;
 }
 
