@@ -45,7 +45,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL( mscoree );
 
-static const WCHAR net_11_subdir[] = {'1','.','0',0};
 static const WCHAR net_20_subdir[] = {'2','.','0',0};
 static const WCHAR net_40_subdir[] = {'4','.','0',0};
 
@@ -54,7 +53,7 @@ static const struct ICLRRuntimeInfoVtbl CLRRuntimeInfoVtbl;
 #define NUM_RUNTIMES 3
 
 static struct CLRRuntimeInfo runtimes[NUM_RUNTIMES] = {
-    {{&CLRRuntimeInfoVtbl}, net_11_subdir, 1, 1, 4322, 0},
+    {{&CLRRuntimeInfoVtbl}, net_20_subdir, 1, 1, 4322, 0},
     {{&CLRRuntimeInfoVtbl}, net_20_subdir, 2, 0, 50727, 0},
     {{&CLRRuntimeInfoVtbl}, net_40_subdir, 4, 0, 30319, 0}
 };
@@ -100,14 +99,32 @@ static void CDECL do_nothing(void)
 {
 }
 
-static void missing_runtime_message(const CLRRuntimeInfo *This)
+static MonoImage* image_open_module_handle_dummy(HMODULE module_handle,
+    char* fname, UINT has_entry_point, MonoImageOpenStatus* status, int abi_version)
 {
-    if (This->major == 1)
-        MESSAGE("wine: Install Mono 2.6 for Windows to run .NET 1.1 applications.\n");
-    else if (This->major == 2)
-        MESSAGE("wine: Install Mono for Windows to run .NET 2.0 applications.\n");
-    else if (This->major == 4)
-        MESSAGE("wine: Install Mono 2.8 or greater for Windows to run .NET 4.0 applications.\n");
+    return loaded_monos[abi_version-1].mono_image_open(fname, status);
+}
+
+static CDECL MonoImage* image_open_module_handle_dummy_1(HMODULE module_handle,
+    char* fname, UINT has_entry_point, MonoImageOpenStatus* status)
+{
+    return image_open_module_handle_dummy(module_handle, fname, has_entry_point, status, 1);
+}
+
+static CDECL MonoImage* image_open_module_handle_dummy_2(HMODULE module_handle,
+    char* fname, UINT has_entry_point, MonoImageOpenStatus* status)
+{
+    return image_open_module_handle_dummy(module_handle, fname, has_entry_point, status, 2);
+}
+
+MonoImage* (CDECL * const image_from_handle_fn[NUM_ABI_VERSIONS])(HMODULE module_handle, char* fname, UINT has_entry_point, MonoImageOpenStatus* status) = {
+    image_open_module_handle_dummy_1,
+    image_open_module_handle_dummy_2
+};
+
+static void missing_runtime_message(void)
+{
+    MESSAGE("wine: Install Mono for Windows to run .NET applications.\n");
 }
 
 static HRESULT load_mono(CLRRuntimeInfo *This, loaded_mono **result)
@@ -124,7 +141,7 @@ static HRESULT load_mono(CLRRuntimeInfo *This, loaded_mono **result)
 
     if (This->mono_abi_version <= 0 || This->mono_abi_version > NUM_ABI_VERSIONS)
     {
-        missing_runtime_message(This);
+        missing_runtime_message();
         return E_FAIL;
     }
 
@@ -165,16 +182,19 @@ static HRESULT load_mono(CLRRuntimeInfo *This, loaded_mono **result)
 } while (0);
 
         LOAD_MONO_FUNCTION(mono_assembly_get_image);
+        LOAD_MONO_FUNCTION(mono_assembly_load_from);
         LOAD_MONO_FUNCTION(mono_assembly_open);
         LOAD_MONO_FUNCTION(mono_config_parse);
         LOAD_MONO_FUNCTION(mono_class_from_mono_type);
         LOAD_MONO_FUNCTION(mono_class_from_name);
         LOAD_MONO_FUNCTION(mono_class_get_method_from_name);
         LOAD_MONO_FUNCTION(mono_domain_assembly_open);
+        LOAD_MONO_FUNCTION(mono_image_open);
         LOAD_MONO_FUNCTION(mono_install_assembly_preload_hook);
         LOAD_MONO_FUNCTION(mono_jit_exec);
         LOAD_MONO_FUNCTION(mono_jit_init);
         LOAD_MONO_FUNCTION(mono_jit_set_trace_options);
+        LOAD_MONO_FUNCTION(mono_marshal_get_vtfixup_ftnptr);
         LOAD_MONO_FUNCTION(mono_object_get_domain);
         LOAD_MONO_FUNCTION(mono_object_new);
         LOAD_MONO_FUNCTION(mono_object_unbox);
@@ -186,6 +206,7 @@ static HRESULT load_mono(CLRRuntimeInfo *This, loaded_mono **result)
         LOAD_MONO_FUNCTION(mono_set_dirs);
         LOAD_MONO_FUNCTION(mono_stringify_assembly_name);
         LOAD_MONO_FUNCTION(mono_string_new);
+        LOAD_MONO_FUNCTION(mono_thread_attach);
 
         /* GLib imports obsoleted by the 2.0 ABI */
         if (This->mono_abi_version == 1)
@@ -203,19 +224,20 @@ static HRESULT load_mono(CLRRuntimeInfo *This, loaded_mono **result)
 
 #undef LOAD_MONO_FUNCTION
 
-#define LOAD_OPT_VOID_MONO_FUNCTION(x) do { \
+#define LOAD_OPT_MONO_FUNCTION(x, default) do { \
     (*result)->x = (void*)GetProcAddress((*result)->mono_handle, #x); \
     if (!(*result)->x) { \
-        (*result)->x = do_nothing; \
+        (*result)->x = default; \
     } \
 } while (0);
 
-        LOAD_OPT_VOID_MONO_FUNCTION(mono_runtime_set_shutting_down);
-        LOAD_OPT_VOID_MONO_FUNCTION(mono_thread_pool_cleanup);
-        LOAD_OPT_VOID_MONO_FUNCTION(mono_thread_suspend_all_other_threads);
-        LOAD_OPT_VOID_MONO_FUNCTION(mono_threads_set_shutting_down);
+        LOAD_OPT_MONO_FUNCTION(mono_image_open_from_module_handle, image_from_handle_fn[This->mono_abi_version-1]);
+        LOAD_OPT_MONO_FUNCTION(mono_runtime_set_shutting_down, do_nothing);
+        LOAD_OPT_MONO_FUNCTION(mono_thread_pool_cleanup, do_nothing);
+        LOAD_OPT_MONO_FUNCTION(mono_thread_suspend_all_other_threads, do_nothing);
+        LOAD_OPT_MONO_FUNCTION(mono_threads_set_shutting_down, do_nothing);
 
-#undef LOAD_OPT_VOID_MONO_FUNCTION
+#undef LOAD_OPT_MONO_FUNCTION
 
         (*result)->mono_profiler_install((MonoProfiler*)*result, mono_shutdown_callback_fn);
 
@@ -564,6 +586,14 @@ HRESULT ICLRRuntimeInfo_GetRuntimeHost(ICLRRuntimeInfo *iface, RuntimeHost **res
     return CLRRuntimeInfo_GetRuntimeHost(This, result);
 }
 
+#ifdef __i386__
+static const WCHAR libmono2_arch_dll[] = {'\\','b','i','n','\\','l','i','b','m','o','n','o','-','2','.','0','-','x','8','6','.','d','l','l',0};
+#elif defined(__x86_64__)
+static const WCHAR libmono2_arch_dll[] = {'\\','b','i','n','\\','l','i','b','m','o','n','o','-','2','.','0','-','x','8','6','_','6','4','.','d','l','l',0};
+#else
+static const WCHAR libmono2_arch_dll[] = {'\\','b','i','n','\\','l','i','b','m','o','n','o','-','2','.','0','.','d','l','l',0};
+#endif
+
 static BOOL find_mono_dll(LPCWSTR path, LPWSTR dll_path, int abi_version)
 {
     static const WCHAR mono_dll[] = {'\\','b','i','n','\\','m','o','n','o','.','d','l','l',0};
@@ -588,9 +618,16 @@ static BOOL find_mono_dll(LPCWSTR path, LPWSTR dll_path, int abi_version)
     else if (abi_version == 2)
     {
         strcpyW(dll_path, path);
-        strcatW(dll_path, mono2_dll);
+        strcatW(dll_path, libmono2_arch_dll);
         attributes = GetFileAttributesW(dll_path);
 
+        if (attributes == INVALID_FILE_ATTRIBUTES)
+        {
+            strcpyW(dll_path, path);
+            strcatW(dll_path, mono2_dll);
+            attributes = GetFileAttributesW(dll_path);
+        }
+
         if (attributes == INVALID_FILE_ATTRIBUTES)
         {
             strcpyW(dll_path, path);
@@ -973,7 +1010,7 @@ static BOOL parse_runtime_version(LPCWSTR version, DWORD *major, DWORD *minor, D
     *minor = 0;
     *build = 0;
 
-    if (version[0] == 'v')
+    if (version[0] == 'v' || version[0] == 'V')
     {
         version++;
         if (!isdigit(*version))
@@ -1035,7 +1072,7 @@ HRESULT WINAPI CLRMetaHost_GetRuntime(ICLRMetaHost* iface,
                         ppRuntime);
             else
             {
-                missing_runtime_message(&runtimes[i]);
+                missing_runtime_message();
                 return CLR_E_SHIM_RUNTIME;
             }
         }
@@ -1347,10 +1384,7 @@ HRESULT get_runtime_info(LPCWSTR exefile, LPCWSTR version, LPCWSTR config_file,
             }
         }
 
-        if (legacy)
-            missing_runtime_message(&runtimes[1]);
-        else
-            missing_runtime_message(&runtimes[NUM_RUNTIMES-1]);
+        missing_runtime_message();
 
         return CLR_E_SHIM_RUNTIME;
     }
