@@ -480,6 +480,19 @@ HRESULT dispex_get_dprop_ref(DispatchEx *This, const WCHAR *name, BOOL alloc, VA
     return S_OK;
 }
 
+HRESULT dispex_get_dynid(DispatchEx *This, const WCHAR *name, DISPID *id)
+{
+    dynamic_prop_t *prop;
+    HRESULT hres;
+
+    hres = get_dynamic_prop(This, name, fdexNameEnsure, &prop);
+    if(FAILED(hres))
+        return hres;
+
+    *id = DISPID_DYNPROP_0 + (prop - This->dynamic_data->props);
+    return S_OK;
+}
+
 static HRESULT dispex_value(DispatchEx *This, LCID lcid, WORD flags, DISPPARAMS *params,
         VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
 {
@@ -633,6 +646,47 @@ static func_disp_t *create_func_disp(DispatchEx *obj, func_info_t *info)
     return ret;
 }
 
+static HRESULT invoke_disp_value(DispatchEx *This, IDispatch *func_disp, LCID lcid, WORD flags, DISPPARAMS *dp,
+        VARIANT *res, EXCEPINFO *ei, IServiceProvider *caller)
+{
+    DISPID named_arg = DISPID_THIS;
+    DISPPARAMS new_dp = {NULL, &named_arg, 0, 1};
+    IDispatchEx *dispex;
+    HRESULT hres;
+
+    if(dp->cNamedArgs) {
+        FIXME("named args not supported\n");
+        return E_NOTIMPL;
+    }
+
+    new_dp.rgvarg = heap_alloc((dp->cArgs+1)*sizeof(VARIANTARG));
+    if(!new_dp.rgvarg)
+        return E_OUTOFMEMORY;
+
+    new_dp.cArgs = dp->cArgs+1;
+    memcpy(new_dp.rgvarg+1, dp->rgvarg, dp->cArgs*sizeof(VARIANTARG));
+
+    V_VT(new_dp.rgvarg) = VT_DISPATCH;
+    V_DISPATCH(new_dp.rgvarg) = (IDispatch*)&This->IDispatchEx_iface;
+
+    hres = IDispatch_QueryInterface(func_disp, &IID_IDispatchEx, (void**)&dispex);
+    TRACE(">>>\n");
+    if(SUCCEEDED(hres)) {
+        hres = IDispatchEx_InvokeEx(dispex, DISPID_VALUE, lcid, flags, &new_dp, res, ei, caller);
+        IDispatchEx_Release(dispex);
+    }else {
+        ULONG err = 0;
+        hres = IDispatch_Invoke(func_disp, DISPID_VALUE, &IID_NULL, lcid, flags, &new_dp, res, ei, &err);
+    }
+    if(SUCCEEDED(hres))
+        TRACE("<<< %s\n", debugstr_variant(res));
+    else
+        WARN("<<< %08x\n", hres);
+
+    heap_free(new_dp.rgvarg);
+    return hres;
+}
+
 static HRESULT function_invoke(DispatchEx *This, func_info_t *func, WORD flags, DISPPARAMS *dp, VARIANT *res,
         EXCEPINFO *ei)
 {
@@ -795,7 +849,7 @@ static HRESULT builtin_propget(DispatchEx *This, func_info_t *func, DISPPARAMS *
     IUnknown *iface;
     HRESULT hres;
 
-    if(dp->cArgs) {
+    if(dp && dp->cArgs) {
         FIXME("cArgs %d\n", dp->cArgs);
         return E_NOTIMPL;
     }
@@ -907,7 +961,25 @@ static HRESULT invoke_builtin_prop(DispatchEx *This, DISPID id, LCID lcid, WORD
         hres = builtin_propget(This, func, dp, res);
         break;
     default:
-        hres = typeinfo_invoke(This, func, flags, dp, res, ei);
+        if(!func->get_vtbl_off) {
+            hres = typeinfo_invoke(This, func, flags, dp, res, ei);
+        }else {
+            VARIANT v;
+
+            hres = builtin_propget(This, func, NULL, &v);
+            if(FAILED(hres))
+                return hres;
+
+            if(V_VT(&v) != VT_DISPATCH) {
+                FIXME("Not a function %s\n", debugstr_variant(&v));
+                VariantClear(&v);
+                return E_FAIL;
+            }
+
+            hres = invoke_disp_value(This, V_DISPATCH(&v), lcid, flags, dp, res, ei, caller);
+
+            IDispatch_Release(V_DISPATCH(&v));
+        }
     }
 
     return hres;
@@ -1095,45 +1167,13 @@ static HRESULT WINAPI DispatchEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
             if(!pvarRes)
                 return E_INVALIDARG;
             /* fall through */
-        case DISPATCH_METHOD: {
-            DISPID named_arg = DISPID_THIS;
-            DISPPARAMS dp = {NULL, &named_arg, 0, 1};
-            IDispatchEx *dispex;
-
+        case DISPATCH_METHOD:
             if(V_VT(&prop->var) != VT_DISPATCH) {
                 FIXME("invoke %s\n", debugstr_variant(&prop->var));
                 return E_NOTIMPL;
             }
 
-            if(pdp->cNamedArgs) {
-                FIXME("named args not supported\n");
-                return E_NOTIMPL;
-            }
-
-            dp.rgvarg = heap_alloc((pdp->cArgs+1)*sizeof(VARIANTARG));
-            if(!dp.rgvarg)
-                return E_OUTOFMEMORY;
-
-            dp.cArgs = pdp->cArgs+1;
-            memcpy(dp.rgvarg+1, pdp->rgvarg, pdp->cArgs*sizeof(VARIANTARG));
-
-            V_VT(dp.rgvarg) = VT_DISPATCH;
-            V_DISPATCH(dp.rgvarg) = (IDispatch*)&This->IDispatchEx_iface;
-
-            hres = IDispatch_QueryInterface(V_DISPATCH(&prop->var), &IID_IDispatchEx, (void**)&dispex);
-            TRACE("%s call\n", debugstr_w(This->dynamic_data->props[idx].name));
-            if(SUCCEEDED(hres)) {
-                hres = IDispatchEx_InvokeEx(dispex, DISPID_VALUE, lcid, wFlags, &dp, pvarRes, pei, pspCaller);
-                IDispatchEx_Release(dispex);
-            }else {
-                ULONG err = 0;
-                hres = IDispatch_Invoke(V_DISPATCH(&prop->var), DISPID_VALUE, &IID_NULL, lcid, wFlags, pdp, pvarRes, pei, &err);
-            }
-            TRACE("%s ret %08x\n", debugstr_w(This->dynamic_data->props[idx].name), hres);
-
-            heap_free(dp.rgvarg);
-            return hres;
-        }
+            return invoke_disp_value(This, V_DISPATCH(&prop->var), lcid, wFlags, pdp, pvarRes, pei, pspCaller);
         case DISPATCH_PROPERTYGET:
             if(prop->flags & DYNPROP_DELETED)
                 return DISP_E_UNKNOWNNAME;
