@@ -18,6 +18,7 @@
 
 #include <stdarg.h>
 #include <stdio.h>
+#include <assert.h>
 
 #define COBJMACROS
 
@@ -28,6 +29,7 @@
 
 #include "mshtml_private.h"
 #include "htmlevent.h"
+#include "binding.h"
 
 #include "wine/debug.h"
 
@@ -41,11 +43,43 @@ typedef struct {
     nsIDOMHTMLAnchorElement *nsanchor;
 } HTMLAnchorElement;
 
+static HRESULT navigate_anchor_window(HTMLAnchorElement *This, const WCHAR *target)
+{
+    nsAString href_str;
+    IUri *uri;
+    nsresult nsres;
+    HRESULT hres;
+
+    nsAString_Init(&href_str, NULL);
+    nsres = nsIDOMHTMLAnchorElement_GetHref(This->nsanchor, &href_str);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *href;
+
+        nsAString_GetData(&href_str, &href);
+        hres = create_relative_uri(This->element.node.doc->basedoc.window, href, &uri);
+    }else {
+        ERR("Could not get anchor href: %08x\n", nsres);
+        hres = E_FAIL;
+    }
+    nsAString_Finish(&href_str);
+    if(FAILED(hres))
+        return hres;
+
+    hres = navigate_new_window(This->element.node.doc->basedoc.window, uri, target, NULL);
+    IUri_Release(uri);
+    return hres;
+}
+
 static HRESULT navigate_anchor(HTMLAnchorElement *This)
 {
     nsAString href_str, target_str;
+    HTMLOuterWindow *window = NULL;
     nsresult nsres;
-    HRESULT hres;
+    HRESULT hres = E_FAIL;
+
+    static const WCHAR _parentW[] = {'p','a','r','e','n','t',0};
+    static const WCHAR _selfW[] = {'_','s','e','l','f',0};
+    static const WCHAR _topW[] = {'_','t','o','p',0};
 
     nsAString_Init(&target_str, NULL);
     nsres = nsIDOMHTMLAnchorElement_GetTarget(This->nsanchor, &target_str);
@@ -53,10 +87,19 @@ static HRESULT navigate_anchor(HTMLAnchorElement *This)
         const PRUnichar *target;
 
         nsAString_GetData(&target_str, &target);
-        if(*target) {
-            FIXME("Navigating to target %s is not implemented\n", debugstr_w(target));
-            nsAString_Finish(&target_str);
-            return S_OK;
+        if(*target && strcmpiW(target, _selfW)) {
+            if(!strcmpiW(target, _topW)) {
+                TRACE("target _top\n");
+                get_top_window(This->element.node.doc->basedoc.window, &window);
+            }else if(!strcmpiW(target, _parentW)) {
+                FIXME("Navigating to target _parent is not implemented\n");
+                nsAString_Finish(&target_str);
+                return S_OK;
+            }else {
+                hres = navigate_anchor_window(This, target);
+                nsAString_Finish(&target_str);
+                return hres;
+            }
         }
     }
     nsAString_Finish(&target_str);
@@ -68,7 +111,8 @@ static HRESULT navigate_anchor(HTMLAnchorElement *This)
 
         nsAString_GetData(&href_str, &href);
         if(*href) {
-            HTMLWindow *window = This->element.node.doc->basedoc.window;
+            if(!window)
+                window = This->element.node.doc->basedoc.window;
             hres = navigate_url(window, href, window->url);
         }else {
             TRACE("empty href\n");
@@ -270,15 +314,32 @@ static HRESULT WINAPI HTMLAnchorElement_get_Methods(IHTMLAnchorElement *iface, B
 static HRESULT WINAPI HTMLAnchorElement_put_name(IHTMLAnchorElement *iface, BSTR v)
 {
     HTMLAnchorElement *This = impl_from_IHTMLAnchorElement(iface);
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
-    return E_NOTIMPL;
+    nsAString nsstr;
+    nsresult nsres;
+
+    TRACE("(%p)->(%s)\n", This, debugstr_w(v));
+
+    nsAString_InitDepend(&nsstr, v);
+    nsres = nsIDOMHTMLAnchorElement_SetName(This->nsanchor, &nsstr);
+    nsAString_Finish(&nsstr);
+    if(NS_FAILED(nsres))
+        return E_FAIL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLAnchorElement_get_name(IHTMLAnchorElement *iface, BSTR *p)
 {
     HTMLAnchorElement *This = impl_from_IHTMLAnchorElement(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsAString name_str;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsAString_Init(&name_str, NULL);
+    nsres = nsIDOMHTMLAnchorElement_GetName(This->nsanchor, &name_str);
+
+    return return_nsstr(nsres, &name_str, p);
 }
 
 static HRESULT WINAPI HTMLAnchorElement_put_host(IHTMLAnchorElement *iface, BSTR v)
@@ -354,7 +415,7 @@ static HRESULT WINAPI HTMLAnchorElement_get_protocol(IHTMLAnchorElement *iface,
 static HRESULT WINAPI HTMLAnchorElement_put_search(IHTMLAnchorElement *iface, BSTR v)
 {
     HTMLAnchorElement *This = impl_from_IHTMLAnchorElement(iface);
-    FIXME("(%p)->(%p)\n", This, debugstr_w(v));
+    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
     return E_NOTIMPL;
 }
 
@@ -571,24 +632,36 @@ static HRESULT HTMLAnchorElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
     return HTMLElement_QI(&This->element.node, riid, ppv);
 }
 
-static void HTMLAnchorElement_destructor(HTMLDOMNode *iface)
-{
-    HTMLAnchorElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nsanchor)
-        nsIDOMHTMLAnchorElement_Release(This->nsanchor);
-
-    HTMLElement_destructor(&This->element.node);
-}
-
-static HRESULT HTMLAnchorElement_handle_event(HTMLDOMNode *iface, eventid_t eid, BOOL *prevent_default)
+static HRESULT HTMLAnchorElement_handle_event(HTMLDOMNode *iface, eventid_t eid, nsIDOMEvent *event, BOOL *prevent_default)
 {
     HTMLAnchorElement *This = impl_from_HTMLDOMNode(iface);
 
     if(eid == EVENTID_CLICK) {
+        nsIDOMMouseEvent *mouse_event;
+        PRUint16 button;
+        nsresult nsres;
+
         TRACE("CLICK\n");
-        *prevent_default = TRUE;
-        return navigate_anchor(This);
+
+        nsres = nsIDOMEvent_QueryInterface(event, &IID_nsIDOMMouseEvent, (void**)&mouse_event);
+        assert(nsres == NS_OK);
+
+        nsres = nsIDOMMouseEvent_GetButton(mouse_event, &button);
+        assert(nsres == NS_OK);
+
+        nsIDOMMouseEvent_Release(mouse_event);
+
+        switch(button) {
+        case 0:
+            *prevent_default = TRUE;
+            return navigate_anchor(This);
+        case 1:
+            *prevent_default = TRUE;
+            return navigate_anchor_window(This, NULL);
+        default:
+            *prevent_default = FALSE;
+            return S_OK;
+        }
     }
 
     return S_OK;
@@ -596,8 +669,9 @@ static HRESULT HTMLAnchorElement_handle_event(HTMLDOMNode *iface, eventid_t eid,
 
 static const NodeImplVtbl HTMLAnchorElementImplVtbl = {
     HTMLAnchorElement_QI,
-    HTMLAnchorElement_destructor,
+    HTMLElement_destructor,
     HTMLElement_clone,
+    HTMLElement_get_attr_col,
     NULL,
     NULL,
     HTMLAnchorElement_handle_event
@@ -630,15 +704,14 @@ HRESULT HTMLAnchorElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nsele
     ret->IHTMLAnchorElement_iface.lpVtbl = &HTMLAnchorElementVtbl;
     ret->element.node.vtbl = &HTMLAnchorElementImplVtbl;
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLAnchorElement, (void**)&ret->nsanchor);
-    if(NS_FAILED(nsres)) {
-        ERR("Could not get nsIDOMHTMLAnchorElement iface: %08x\n", nsres);
-        heap_free(ret);
-        return E_FAIL;
-    }
-
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLAnchorElement_dispex);
 
+    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLAnchorElement, (void**)&ret->nsanchor);
+
+    /* Shere the reference with nsnode */
+    assert(nsres == NS_OK && (nsIDOMNode*)ret->nsanchor == ret->element.node.nsnode);
+    nsIDOMNode_Release(ret->element.node.nsnode);
+
     *elem = &ret->element;
     return S_OK;
 }
