@@ -171,17 +171,23 @@ static BOOL variant_to_nscolor(const VARIANT *v, nsAString *nsstr)
     }
 
     default:
-        FIXME("invalid vt=%d\n", V_VT(v));
+        FIXME("invalid color %s\n", debugstr_variant(v));
     }
 
     return FALSE;
 
 }
 
-static void nscolor_to_variant(const nsAString *nsstr, VARIANT *p)
+static HRESULT return_nscolor(nsresult nsres, nsAString *nsstr, VARIANT *p)
 {
     const PRUnichar *color;
 
+    if(NS_FAILED(nsres)) {
+        ERR("failed: %08x\n", nsres);
+        nsAString_Finish(nsstr);
+        return E_FAIL;
+    }
+
     nsAString_GetData(nsstr, &color);
 
     if(*color == '#') {
@@ -190,7 +196,15 @@ static void nscolor_to_variant(const nsAString *nsstr, VARIANT *p)
     }else {
         V_VT(p) = VT_BSTR;
         V_BSTR(p) = SysAllocString(color);
+        if(!V_BSTR(p)) {
+            nsAString_Finish(nsstr);
+            return E_OUTOFMEMORY;
+        }
     }
+
+    nsAString_Finish(nsstr);
+    TRACE("ret %s\n", debugstr_variant(p));
+    return S_OK;
 }
 
 static inline HTMLBodyElement *impl_from_IHTMLBodyElement(IHTMLBodyElement *iface)
@@ -279,21 +293,8 @@ static HRESULT WINAPI HTMLBodyElement_get_background(IHTMLBodyElement *iface, BS
     TRACE("(%p)->(%p)\n", This, p);
 
     nsAString_Init(&background_str, NULL);
-
     nsres = nsIDOMHTMLBodyElement_GetBackground(This->nsbody, &background_str);
-    if(NS_SUCCEEDED(nsres)) {
-        const PRUnichar *background;
-        nsAString_GetData(&background_str, &background);
-        *p = *background ? SysAllocString(background) : NULL;
-    }else {
-        ERR("GetBackground failed: %08x\n", nsres);
-        *p = NULL;
-    }
-
-    nsAString_Finish(&background_str);
-
-    TRACE("*p = %s\n", debugstr_w(*p));
-    return S_OK;
+    return return_nsstr(nsres, &background_str, p);
 }
 
 static HRESULT WINAPI HTMLBodyElement_put_bgProperties(IHTMLBodyElement *iface, BSTR v)
@@ -313,7 +314,7 @@ static HRESULT WINAPI HTMLBodyElement_get_bgProperties(IHTMLBodyElement *iface,
 static HRESULT WINAPI HTMLBodyElement_put_leftMargin(IHTMLBodyElement *iface, VARIANT v)
 {
     HTMLBodyElement *This = impl_from_IHTMLBodyElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -327,7 +328,7 @@ static HRESULT WINAPI HTMLBodyElement_get_leftMargin(IHTMLBodyElement *iface, VA
 static HRESULT WINAPI HTMLBodyElement_put_topMargin(IHTMLBodyElement *iface, VARIANT v)
 {
     HTMLBodyElement *This = impl_from_IHTMLBodyElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -341,7 +342,7 @@ static HRESULT WINAPI HTMLBodyElement_get_topMargin(IHTMLBodyElement *iface, VAR
 static HRESULT WINAPI HTMLBodyElement_put_rightMargin(IHTMLBodyElement *iface, VARIANT v)
 {
     HTMLBodyElement *This = impl_from_IHTMLBodyElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -355,7 +356,7 @@ static HRESULT WINAPI HTMLBodyElement_get_rightMargin(IHTMLBodyElement *iface, V
 static HRESULT WINAPI HTMLBodyElement_put_bottomMargin(IHTMLBodyElement *iface, VARIANT v)
 {
     HTMLBodyElement *This = impl_from_IHTMLBodyElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -386,7 +387,7 @@ static HRESULT WINAPI HTMLBodyElement_put_bgColor(IHTMLBodyElement *iface, VARIA
     nsAString strColor;
     nsresult nsres;
 
-    TRACE("(%p)->()\n", This);
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     if(!variant_to_nscolor(&v, &strColor))
         return S_OK;
@@ -431,7 +432,7 @@ static HRESULT WINAPI HTMLBodyElement_put_text(IHTMLBodyElement *iface, VARIANT
     nsAString text;
     nsresult nsres;
 
-    TRACE("(%p)->(v%d)\n", This, V_VT(&v));
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     if(!variant_to_nscolor(&v, &text))
         return S_OK;
@@ -479,7 +480,7 @@ static HRESULT WINAPI HTMLBodyElement_put_link(IHTMLBodyElement *iface, VARIANT
     nsAString link_str;
     nsresult nsres;
 
-    TRACE("(%p)->(v%d)\n", This, V_VT(&v));
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     if(!variant_to_nscolor(&v, &link_str))
         return S_OK;
@@ -502,13 +503,7 @@ static HRESULT WINAPI HTMLBodyElement_get_link(IHTMLBodyElement *iface, VARIANT
 
     nsAString_Init(&link_str, NULL);
     nsres = nsIDOMHTMLBodyElement_GetLink(This->nsbody, &link_str);
-    if(NS_FAILED(nsres))
-        ERR("GetLink failed: %08x\n", nsres);
-
-    nscolor_to_variant(&link_str, p);
-    nsAString_Finish(&link_str);
-
-    return S_OK;
+    return return_nscolor(nsres, &link_str, p);
 }
 
 static HRESULT WINAPI HTMLBodyElement_put_vLink(IHTMLBodyElement *iface, VARIANT v)
@@ -517,7 +512,7 @@ static HRESULT WINAPI HTMLBodyElement_put_vLink(IHTMLBodyElement *iface, VARIANT
     nsAString vlink_str;
     nsresult nsres;
 
-    TRACE("(%p)->(v%d)\n", This, V_VT(&v));
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     if(!variant_to_nscolor(&v, &vlink_str))
         return S_OK;
@@ -540,13 +535,7 @@ static HRESULT WINAPI HTMLBodyElement_get_vLink(IHTMLBodyElement *iface, VARIANT
 
     nsAString_Init(&vlink_str, NULL);
     nsres = nsIDOMHTMLBodyElement_GetVLink(This->nsbody, &vlink_str);
-    if(NS_FAILED(nsres))
-        ERR("GetLink failed: %08x\n", nsres);
-
-    nscolor_to_variant(&vlink_str, p);
-    nsAString_Finish(&vlink_str);
-
-    return S_OK;
+    return return_nscolor(nsres, &vlink_str, p);
 }
 
 static HRESULT WINAPI HTMLBodyElement_put_aLink(IHTMLBodyElement *iface, VARIANT v)
@@ -555,7 +544,7 @@ static HRESULT WINAPI HTMLBodyElement_put_aLink(IHTMLBodyElement *iface, VARIANT
     nsAString alink_str;
     nsresult nsres;
 
-    TRACE("(%p)->(v%d)\n", This, V_VT(&v));
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     if(!variant_to_nscolor(&v, &alink_str))
         return S_OK;
@@ -578,19 +567,13 @@ static HRESULT WINAPI HTMLBodyElement_get_aLink(IHTMLBodyElement *iface, VARIANT
 
     nsAString_Init(&alink_str, NULL);
     nsres = nsIDOMHTMLBodyElement_GetALink(This->nsbody, &alink_str);
-    if(NS_FAILED(nsres))
-        ERR("GetALink failed: %08x\n", nsres);
-
-    nscolor_to_variant(&alink_str, p);
-    nsAString_Finish(&alink_str);
-
-    return S_OK;
+    return return_nscolor(nsres, &alink_str, p);
 }
 
 static HRESULT WINAPI HTMLBodyElement_put_onload(IHTMLBodyElement *iface, VARIANT v)
 {
     HTMLBodyElement *This = impl_from_IHTMLBodyElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -604,7 +587,7 @@ static HRESULT WINAPI HTMLBodyElement_get_onload(IHTMLBodyElement *iface, VARIAN
 static HRESULT WINAPI HTMLBodyElement_put_onunload(IHTMLBodyElement *iface, VARIANT v)
 {
     HTMLBodyElement *This = impl_from_IHTMLBodyElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -632,7 +615,7 @@ static HRESULT WINAPI HTMLBodyElement_get_scroll(IHTMLBodyElement *iface, BSTR *
 static HRESULT WINAPI HTMLBodyElement_put_onselect(IHTMLBodyElement *iface, VARIANT v)
 {
     HTMLBodyElement *This = impl_from_IHTMLBodyElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -646,7 +629,7 @@ static HRESULT WINAPI HTMLBodyElement_get_onselect(IHTMLBodyElement *iface, VARI
 static HRESULT WINAPI HTMLBodyElement_put_onbeforeunload(IHTMLBodyElement *iface, VARIANT v)
 {
     HTMLBodyElement *This = impl_from_IHTMLBodyElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
