@@ -16,8 +16,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-
 #include <stdarg.h>
+#include <assert.h>
 
 #define COBJMACROS
 
@@ -45,12 +45,14 @@ static const WCHAR headW[]     = {'H','E','A','D',0};
 static const WCHAR iframeW[]   = {'I','F','R','A','M','E',0};
 static const WCHAR imgW[]      = {'I','M','G',0};
 static const WCHAR inputW[]    = {'I','N','P','U','T',0};
+static const WCHAR metaW[]     = {'M','E','T','A',0};
 static const WCHAR objectW[]   = {'O','B','J','E','C','T',0};
 static const WCHAR optionW[]   = {'O','P','T','I','O','N',0};
 static const WCHAR scriptW[]   = {'S','C','R','I','P','T',0};
 static const WCHAR selectW[]   = {'S','E','L','E','C','T',0};
 static const WCHAR styleW[]    = {'S','T','Y','L','E',0};
 static const WCHAR tableW[]    = {'T','A','B','L','E',0};
+static const WCHAR tdW[]       = {'T','D',0};
 static const WCHAR textareaW[] = {'T','E','X','T','A','R','E','A',0};
 static const WCHAR title_tagW[]= {'T','I','T','L','E',0};
 static const WCHAR trW[]       = {'T','R',0};
@@ -70,12 +72,14 @@ static const tag_desc_t tag_descs[] = {
     {iframeW,    HTMLIFrame_Create},
     {imgW,       HTMLImgElement_Create},
     {inputW,     HTMLInputElement_Create},
+    {metaW,      HTMLMetaElement_Create},
     {objectW,    HTMLObjectElement_Create},
     {optionW,    HTMLOptionElement_Create},
     {scriptW,    HTMLScriptElement_Create},
     {selectW,    HTMLSelectElement_Create},
     {styleW,     HTMLStyleElement_Create},
     {tableW,     HTMLTable_Create},
+    {tdW,        HTMLTableCell_Create},
     {textareaW,  HTMLTextAreaElement_Create},
     {title_tagW, HTMLTitleElement_Create},
     {trW,        HTMLTableRow_Create}
@@ -104,7 +108,6 @@ static const tag_desc_t *get_tag_desc(const WCHAR *tag_name)
 HRESULT replace_node_by_html(nsIDOMHTMLDocument *nsdoc, nsIDOMNode *nsnode, const WCHAR *html)
 {
     nsIDOMDocumentFragment *nsfragment;
-    nsIDOMNSRange *nsrange;
     nsIDOMNode *nsparent;
     nsIDOMRange *range;
     nsAString html_str;
@@ -117,16 +120,9 @@ HRESULT replace_node_by_html(nsIDOMHTMLDocument *nsdoc, nsIDOMNode *nsnode, cons
         return E_FAIL;
     }
 
-    nsres = nsIDOMRange_QueryInterface(range, &IID_nsIDOMNSRange, (void**)&nsrange);
-    nsIDOMRange_Release(range);
-    if(NS_FAILED(nsres)) {
-        ERR("Could not get nsIDOMNSRange: %08x\n", nsres);
-        return E_FAIL;
-    }
-
     nsAString_InitDepend(&html_str, html);
-    nsIDOMNSRange_CreateContextualFragment(nsrange, &html_str, &nsfragment);
-    nsIDOMNSRange_Release(nsrange);
+    nsIDOMRange_CreateContextualFragment(range, &html_str, &nsfragment);
+    nsIDOMRange_Release(range);
     nsAString_Finish(&html_str);
     if(NS_FAILED(nsres)) {
         ERR("CreateContextualFragment failed: %08x\n", nsres);
@@ -265,7 +261,7 @@ static HRESULT WINAPI HTMLElement_setAttribute(IHTMLElement *iface, BSTR strAttr
     DISPPARAMS dispParams;
     EXCEPINFO excep;
 
-    TRACE("(%p)->(%s . %08x)\n", This, debugstr_w(strAttributeName), lFlags);
+    TRACE("(%p)->(%s %s %08x)\n", This, debugstr_w(strAttributeName), debugstr_variant(&AttributeValue), lFlags);
 
     hres = IDispatchEx_GetDispID(&This->node.dispex.IDispatchEx_iface, strAttributeName,
             fdexNameCaseInsensitive | fdexNameEnsure, &dispid);
@@ -348,7 +344,6 @@ static HRESULT WINAPI HTMLElement_get_className(IHTMLElement *iface, BSTR *p)
     HTMLElement *This = impl_from_IHTMLElement(iface);
     nsAString class_str;
     nsresult nsres;
-    HRESULT hres = S_OK;
 
     TRACE("(%p)->(%p)\n", This, p);
 
@@ -359,20 +354,7 @@ static HRESULT WINAPI HTMLElement_get_className(IHTMLElement *iface, BSTR *p)
 
     nsAString_Init(&class_str, NULL);
     nsres = nsIDOMHTMLElement_GetClassName(This->nselem, &class_str);
-
-    if(NS_SUCCEEDED(nsres)) {
-        const PRUnichar *class;
-        nsAString_GetData(&class_str, &class);
-        *p = *class ? SysAllocString(class) : NULL;
-    }else {
-        ERR("GetClassName failed: %08x\n", nsres);
-        hres = E_FAIL;
-    }
-
-    nsAString_Finish(&class_str);
-
-    TRACE("className=%s\n", debugstr_w(*p));
-    return hres;
+    return return_nsstr(nsres, &class_str, p);
 }
 
 static HRESULT WINAPI HTMLElement_put_id(IHTMLElement *iface, BSTR v)
@@ -400,34 +382,24 @@ static HRESULT WINAPI HTMLElement_put_id(IHTMLElement *iface, BSTR v)
 static HRESULT WINAPI HTMLElement_get_id(IHTMLElement *iface, BSTR *p)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    const PRUnichar *id;
     nsAString id_str;
     nsresult nsres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    *p = NULL;
-
-    if(!This->nselem)
+    if(!This->nselem) {
+        *p = NULL;
         return S_OK;
+    }
 
     nsAString_Init(&id_str, NULL);
     nsres = nsIDOMHTMLElement_GetId(This->nselem, &id_str);
-    nsAString_GetData(&id_str, &id);
-
-    if(NS_FAILED(nsres))
-        ERR("GetId failed: %08x\n", nsres);
-    else if(*id)
-        *p = SysAllocString(id);
-
-    nsAString_Finish(&id_str);
-    return S_OK;
+    return return_nsstr(nsres, &id_str, p);
 }
 
 static HRESULT WINAPI HTMLElement_get_tagName(IHTMLElement *iface, BSTR *p)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    const PRUnichar *tag;
     nsAString tag_str;
     nsresult nsres;
 
@@ -439,21 +411,12 @@ static HRESULT WINAPI HTMLElement_get_tagName(IHTMLElement *iface, BSTR *p)
         WARN("NULL nselem, assuming comment\n");
 
         *p = SysAllocString(comment_tagW);
-        return S_OK;
+        return *p ? S_OK : E_OUTOFMEMORY;
     }
 
     nsAString_Init(&tag_str, NULL);
     nsres = nsIDOMHTMLElement_GetTagName(This->nselem, &tag_str);
-    if(NS_SUCCEEDED(nsres)) {
-        nsAString_GetData(&tag_str, &tag);
-        *p = SysAllocString(tag);
-    }else {
-        ERR("GetTagName failed: %08x\n", nsres);
-        *p = NULL;
-    }
-    nsAString_Finish(&tag_str);
-
-    return S_OK;
+    return return_nsstr(nsres, &tag_str, p);
 }
 
 static HRESULT WINAPI HTMLElement_get_parentElement(IHTMLElement *iface, IHTMLElement **p)
@@ -521,7 +484,7 @@ static HRESULT WINAPI HTMLElement_get_style(IHTMLElement *iface, IHTMLStyle **p)
 static HRESULT WINAPI HTMLElement_put_onhelp(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -536,7 +499,7 @@ static HRESULT WINAPI HTMLElement_put_onclick(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
 
-    TRACE("(%p)->()\n", This);
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     return set_node_event(&This->node, EVENTID_CLICK, &v);
 }
@@ -590,7 +553,7 @@ static HRESULT WINAPI HTMLElement_put_onkeyup(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
 
-    TRACE("(%p)->()\n", This);
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     return set_node_event(&This->node, EVENTID_KEYUP, &v);
 }
@@ -605,15 +568,19 @@ static HRESULT WINAPI HTMLElement_get_onkeyup(IHTMLElement *iface, VARIANT *p)
 static HRESULT WINAPI HTMLElement_put_onkeypress(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
+
+    return set_node_event(&This->node, EVENTID_KEYPRESS, &v);
 }
 
 static HRESULT WINAPI HTMLElement_get_onkeypress(IHTMLElement *iface, VARIANT *p)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return get_node_event(&This->node, EVENTID_KEYPRESS, p);
 }
 
 static HRESULT WINAPI HTMLElement_put_onmouseout(IHTMLElement *iface, VARIANT v)
@@ -638,7 +605,7 @@ static HRESULT WINAPI HTMLElement_put_onmouseover(IHTMLElement *iface, VARIANT v
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
 
-    TRACE("(%p)->()\n", This);
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     return set_node_event(&This->node, EVENTID_MOUSEOVER, &v);
 }
@@ -656,7 +623,7 @@ static HRESULT WINAPI HTMLElement_put_onmousemove(IHTMLElement *iface, VARIANT v
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
 
-    TRACE("(%p)->()\n", This);
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     return set_node_event(&This->node, EVENTID_MOUSEMOVE, &v);
 }
@@ -674,7 +641,7 @@ static HRESULT WINAPI HTMLElement_put_onmousedown(IHTMLElement *iface, VARIANT v
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
 
-    TRACE("(%p)->()\n", This);
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     return set_node_event(&This->node, EVENTID_MOUSEDOWN, &v);
 }
@@ -783,17 +750,7 @@ static HRESULT WINAPI HTMLElement_get_title(IHTMLElement *iface, BSTR *p)
 
     nsAString_Init(&title_str, NULL);
     nsres = nsIDOMHTMLElement_GetTitle(This->nselem, &title_str);
-    if(NS_SUCCEEDED(nsres)) {
-        const PRUnichar *title;
-
-        nsAString_GetData(&title_str, &title);
-        *p = *title ? SysAllocString(title) : NULL;
-    }else {
-        ERR("GetTitle failed: %08x\n", nsres);
-        return E_FAIL;
-    }
-
-    return S_OK;
+    return return_nsstr(nsres, &title_str, p);
 }
 
 static HRESULT WINAPI HTMLElement_put_language(IHTMLElement *iface, BSTR v)
@@ -831,7 +788,7 @@ static HRESULT WINAPI HTMLElement_get_onselectstart(IHTMLElement *iface, VARIANT
 static HRESULT WINAPI HTMLElement_scrollIntoView(IHTMLElement *iface, VARIANT varargStart)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&varargStart));
     return E_NOTIMPL;
 }
 
@@ -967,6 +924,7 @@ static HRESULT WINAPI HTMLElement_get_offsetParent(IHTMLElement *iface, IHTMLEle
             return hres;
 
         hres = IHTMLDOMNode_QueryInterface(&node->IHTMLDOMNode_iface, &IID_IHTMLElement, (void**)p);
+        node_release(node);
     }else {
         *p = NULL;
         hres = S_OK;
@@ -1014,18 +972,7 @@ static HRESULT WINAPI HTMLElement_get_innerHTML(IHTMLElement *iface, BSTR *p)
 
     nsAString_Init(&html_str, NULL);
     nsres = nsIDOMHTMLElement_GetInnerHTML(This->nselem, &html_str);
-    if(NS_SUCCEEDED(nsres)) {
-        const PRUnichar *html;
-
-        nsAString_GetData(&html_str, &html);
-        *p = *html ? SysAllocString(html) : NULL;
-    }else {
-        FIXME("SetInnerHtml failed %08x\n", nsres);
-        *p = NULL;
-    }
-
-    nsAString_Finish(&html_str);
-    return S_OK;
+    return return_nsstr(nsres, &html_str, p);
 }
 
 static HRESULT WINAPI HTMLElement_put_innerText(IHTMLElement *iface, BSTR v)
@@ -1200,7 +1147,6 @@ static HRESULT WINAPI HTMLElement_insertAdjacentHTML(IHTMLElement *iface, BSTR w
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
     nsIDOMRange *range;
-    nsIDOMNSRange *nsrange;
     nsIDOMNode *nsnode;
     nsAString ns_html;
     nsresult nsres;
@@ -1222,19 +1168,10 @@ static HRESULT WINAPI HTMLElement_insertAdjacentHTML(IHTMLElement *iface, BSTR w
 
     nsIDOMRange_SetStartBefore(range, This->node.nsnode);
 
-    nsIDOMRange_QueryInterface(range, &IID_nsIDOMNSRange, (void **)&nsrange);
-    nsIDOMRange_Release(range);
-    if(NS_FAILED(nsres))
-    {
-        ERR("getting nsIDOMNSRange failed: %08x\n", nsres);
-        return E_FAIL;
-    }
-
     nsAString_InitDepend(&ns_html, html);
-
-    nsres = nsIDOMNSRange_CreateContextualFragment(nsrange, &ns_html, (nsIDOMDocumentFragment **)&nsnode);
-    nsIDOMNSRange_Release(nsrange);
+    nsres = nsIDOMRange_CreateContextualFragment(range, &ns_html, (nsIDOMDocumentFragment **)&nsnode);
     nsAString_Finish(&ns_html);
+    nsIDOMRange_Release(range);
 
     if(NS_FAILED(nsres) || !nsnode)
     {
@@ -1321,7 +1258,7 @@ static HRESULT WINAPI HTMLElement_get_filters(IHTMLElement *iface,
 static HRESULT WINAPI HTMLElement_put_ondragstart(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1342,7 +1279,7 @@ static HRESULT WINAPI HTMLElement_toString(IHTMLElement *iface, BSTR *String)
 static HRESULT WINAPI HTMLElement_put_onbeforeupdate(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1356,7 +1293,7 @@ static HRESULT WINAPI HTMLElement_get_onbeforeupdate(IHTMLElement *iface, VARIAN
 static HRESULT WINAPI HTMLElement_put_onafterupdate(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1370,7 +1307,7 @@ static HRESULT WINAPI HTMLElement_get_onafterupdate(IHTMLElement *iface, VARIANT
 static HRESULT WINAPI HTMLElement_put_onerrorupdate(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1384,7 +1321,7 @@ static HRESULT WINAPI HTMLElement_get_onerrorupdate(IHTMLElement *iface, VARIANT
 static HRESULT WINAPI HTMLElement_put_onrowexit(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1398,7 +1335,7 @@ static HRESULT WINAPI HTMLElement_get_onrowexit(IHTMLElement *iface, VARIANT *p)
 static HRESULT WINAPI HTMLElement_put_onrowenter(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1412,7 +1349,7 @@ static HRESULT WINAPI HTMLElement_get_onrowenter(IHTMLElement *iface, VARIANT *p
 static HRESULT WINAPI HTMLElement_put_ondatasetchanged(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1426,7 +1363,7 @@ static HRESULT WINAPI HTMLElement_get_ondatasetchanged(IHTMLElement *iface, VARI
 static HRESULT WINAPI HTMLElement_put_ondataavailable(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1440,7 +1377,7 @@ static HRESULT WINAPI HTMLElement_get_ondataavailable(IHTMLElement *iface, VARIA
 static HRESULT WINAPI HTMLElement_put_ondatasetcomplete(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1454,7 +1391,7 @@ static HRESULT WINAPI HTMLElement_get_ondatasetcomplete(IHTMLElement *iface, VAR
 static HRESULT WINAPI HTMLElement_put_onfilterchange(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = impl_from_IHTMLElement(iface);
-    FIXME("(%p)->()\n", This);
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
     return E_NOTIMPL;
 }
 
@@ -1479,8 +1416,7 @@ static HRESULT WINAPI HTMLElement_get_children(IHTMLElement *iface, IDispatch **
         return E_FAIL;
     }
 
-    *p = (IDispatch*)create_collection_from_nodelist(This->node.doc,
-            (IUnknown*)&This->IHTMLElement_iface, nsnode_list);
+    *p = (IDispatch*)create_collection_from_nodelist(This->node.doc, nsnode_list);
 
     nsIDOMNodeList_Release(nsnode_list);
     return S_OK;
@@ -1646,8 +1582,6 @@ void HTMLElement_destructor(HTMLDOMNode *iface)
 
     ConnectionPointContainer_Destroy(&This->cp_container);
 
-    if(This->nselem)
-        nsIDOMHTMLElement_Release(This->nselem);
     if(This->style) {
         This->style->elem = NULL;
         IHTMLStyle_Release(&This->style->IHTMLStyle_iface);
@@ -1677,7 +1611,14 @@ HRESULT HTMLElement_clone(HTMLDOMNode *iface, nsIDOMNode *nsnode, HTMLDOMNode **
     if(FAILED(hres))
         return hres;
 
-    IHTMLElement_AddRef(&new_elem->IHTMLElement_iface);
+    if(This->filter) {
+        new_elem->filter = heap_strdupW(This->filter);
+        if(!new_elem->filter) {
+            IHTMLElement_Release(&This->IHTMLElement_iface);
+            return E_OUTOFMEMORY;
+        }
+    }
+
     *ret = &new_elem->node;
     return S_OK;
 }
@@ -1832,11 +1773,13 @@ void HTMLElement_Init(HTMLElement *This, HTMLDocumentNode *doc, nsIDOMHTMLElemen
     init_dispex(&This->node.dispex, (IUnknown*)&This->IHTMLElement_iface,
             dispex_data ? dispex_data : &HTMLElement_dispex);
 
-    if(nselem)
-        nsIDOMHTMLElement_AddRef(nselem);
-    This->nselem = nselem;
+    if(nselem) {
+        HTMLDOMNode_Init(doc, &This->node, (nsIDOMNode*)nselem);
 
-    HTMLDOMNode_Init(doc, &This->node, (nsIDOMNode*)nselem);
+        /* No AddRef, share reference with HTMLDOMNode */
+        assert((nsIDOMNode*)nselem == This->node.nsnode);
+        This->nselem = nselem;
+    }
 
     ConnectionPointContainer_Init(&This->cp_container, (IUnknown*)&This->IHTMLElement_iface);
 }
@@ -2292,7 +2235,7 @@ static HRESULT WINAPI HTMLAttributeCollection_item(IHTMLAttributeCollection *ifa
         hres = get_attr_dispid_by_name(This, V_BSTR(name), &id);
         break;
     default:
-        FIXME("unsupported vt %x\n", V_VT(name));
+        FIXME("unsupported name %s\n", debugstr_variant(name));
         hres = E_NOTIMPL;
     }
     if(hres == DISP_E_UNKNOWNNAME)
