@@ -68,12 +68,18 @@ static const WCHAR ondragW[] = {'o','n','d','r','a','g',0};
 static const WCHAR dragstartW[] = {'d','r','a','g','s','t','a','r','t',0};
 static const WCHAR ondragstartW[] = {'o','n','d','r','a','g','s','t','a','r','t',0};
 
+static const WCHAR errorW[] = {'e','r','r','o','r',0};
+static const WCHAR onerrorW[] = {'o','n','e','r','r','o','r',0};
+
 static const WCHAR focusW[] = {'f','o','c','u','s',0};
 static const WCHAR onfocusW[] = {'o','n','f','o','c','u','s',0};
 
 static const WCHAR keydownW[] = {'k','e','y','d','o','w','n',0};
 static const WCHAR onkeydownW[] = {'o','n','k','e','y','d','o','w','n',0};
 
+static const WCHAR keypressW[] = {'k','e','y','p','r','e','s','s',0};
+static const WCHAR onkeypressW[] = {'o','n','k','e','y','p','r','e','s','s',0};
+
 static const WCHAR keyupW[] = {'k','e','y','u','p',0};
 static const WCHAR onkeyupW[] = {'o','n','k','e','y','u','p',0};
 
@@ -160,10 +166,14 @@ static const event_info_t event_info[] = {
         EVENT_CANCELABLE},
     {dragstartW,         ondragstartW,         EVENTT_MOUSE,  DISPID_EVMETH_ONDRAGSTART,
         EVENT_CANCELABLE},
+    {errorW,             onerrorW,             EVENTT_NONE,   DISPID_EVMETH_ONERROR,
+        EVENT_NODEHANDLER},
     {focusW,             onfocusW,             EVENTT_HTML,   DISPID_EVMETH_ONFOCUS,
         EVENT_DEFAULTLISTENER},
     {keydownW,           onkeydownW,           EVENTT_KEY,    DISPID_EVMETH_ONKEYDOWN,
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
+    {keypressW,          onkeypressW,          EVENTT_KEY,    DISPID_EVMETH_ONKEYPRESS,
+        EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
     {keyupW,             onkeyupW,             EVENTT_KEY,    DISPID_EVMETH_ONKEYUP,
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE},
     {loadW,              onloadW,              EVENTT_HTML,   DISPID_EVMETH_ONLOAD,
@@ -190,7 +200,7 @@ static const event_info_t event_info[] = {
         EVENT_DEFAULTLISTENER|EVENT_BUBBLE|EVENT_CANCELABLE}
 };
 
-static const eventid_t node_handled_list[] = { EVENTID_LOAD };
+static const eventid_t node_handled_list[] = { EVENTID_ERROR, EVENTID_LOAD };
 
 eventid_t str_to_eid(LPCWSTR str)
 {
@@ -240,6 +250,7 @@ typedef struct {
     const event_info_t *type;
     nsIDOMEvent *nsevent;
     BOOL prevent_default;
+    BOOL cancel_bubble;
 } HTMLEventObj;
 
 static inline HTMLEventObj *impl_from_IHTMLEventObj(IHTMLEventObj *iface)
@@ -439,7 +450,7 @@ static HRESULT WINAPI HTMLEventObj_put_returnValue(IHTMLEventObj *iface, VARIANT
     TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     if(V_VT(&v) != VT_BOOL) {
-        FIXME("unsupported vt %d\n", V_VT(&v));
+        FIXME("unsupported value %s\n", debugstr_variant(&v));
         return DISP_E_BADVARTYPE;
     }
 
@@ -461,17 +472,20 @@ static HRESULT WINAPI HTMLEventObj_get_returnValue(IHTMLEventObj *iface, VARIANT
 static HRESULT WINAPI HTMLEventObj_put_cancelBubble(IHTMLEventObj *iface, VARIANT_BOOL v)
 {
     HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
-    FIXME("(%p)->(%x)\n", This, v);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%x)\n", This, v);
+
+    This->cancel_bubble = !!v;
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLEventObj_get_cancelBubble(IHTMLEventObj *iface, VARIANT_BOOL *p)
 {
     HTMLEventObj *This = impl_from_IHTMLEventObj(iface);
 
-    FIXME("(%p)->(%p)\n", This, p);
+    TRACE("(%p)->(%p)\n", This, p);
 
-    *p = VARIANT_FALSE;
+    *p = This->cancel_bubble ? VARIANT_TRUE : VARIANT_FALSE;
     return S_OK;
 }
 
@@ -975,6 +989,7 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
     IHTMLEventObj *prev_event;
     nsIDOMNode *parent, *nsnode;
     BOOL prevent_default = FALSE;
+    HTMLInnerWindow *window;
     HTMLDOMNode *node;
     PRUint16 node_type;
     nsresult nsres;
@@ -982,16 +997,18 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
 
     TRACE("(%p) %s\n", doc, debugstr_w(event_info[eid].name));
 
-    prev_event = doc->basedoc.window->event;
+    window = doc->basedoc.window->base.inner_window;
+    prev_event = window->event;
     if(set_event) {
         hres = get_node(doc, target, TRUE, &node);
         if(FAILED(hres))
             return;
 
         event_obj = create_event(node, eid, nsevent);
-        doc->basedoc.window->event = &event_obj->IHTMLEventObj_iface;
+        node_release(node);
+        window->event = &event_obj->IHTMLEventObj_iface;
     }else {
-        doc->basedoc.window->event = NULL;
+        window->event = NULL;
     }
 
     nsIDOMNode_GetNodeType(target, &node_type);
@@ -1002,11 +1019,13 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
     case ELEMENT_NODE:
         do {
             hres = get_node(doc, nsnode, FALSE, &node);
-            if(SUCCEEDED(hres) && node)
+            if(SUCCEEDED(hres) && node) {
                 call_event_handlers(doc, event_obj, *get_node_event_target(node),
                         node->cp_container, eid, (IDispatch*)&node->IHTMLDOMNode_iface);
+                node_release(node);
+            }
 
-            if(!(event_info[eid].flags & EVENT_BUBBLE))
+            if(!(event_info[eid].flags & EVENT_BUBBLE) || (event_obj && event_obj->cancel_bubble))
                 break;
 
             nsIDOMNode_GetParentNode(nsnode, &parent);
@@ -1018,7 +1037,7 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
             nsIDOMNode_GetNodeType(nsnode, &node_type);
         }while(node_type == ELEMENT_NODE);
 
-        if(!(event_info[eid].flags & EVENT_BUBBLE))
+        if(!(event_info[eid].flags & EVENT_BUBBLE) || (event_obj && event_obj->cancel_bubble))
             break;
 
     case DOCUMENT_NODE:
@@ -1029,9 +1048,11 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
             nsres = nsIDOMHTMLDocument_GetBody(doc->nsdoc, &nsbody);
             if(NS_SUCCEEDED(nsres) && nsbody) {
                 hres = get_node(doc, (nsIDOMNode*)nsbody, FALSE, &node);
-                if(SUCCEEDED(hres) && node)
+                if(SUCCEEDED(hres) && node) {
                     call_event_handlers(doc, event_obj, *get_node_event_target(node),
                             node->cp_container, eid, (IDispatch*)&node->IHTMLDOMNode_iface);
+                    node_release(node);
+                }
                 nsIDOMHTMLElement_Release(nsbody);
             }else {
                 ERR("Could not get body: %08x\n", nsres);
@@ -1051,7 +1072,7 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
 
     if(event_obj && event_obj->prevent_default)
         prevent_default = TRUE;
-    doc->basedoc.window->event = prev_event;
+    window->event = prev_event;
     if(event_obj)
         IHTMLEventObj_Release(&event_obj->IHTMLEventObj_iface);
 
@@ -1064,9 +1085,11 @@ void fire_event(HTMLDocumentNode *doc, eventid_t eid, BOOL set_event, nsIDOMNode
             if(FAILED(hres))
                 break;
 
-            if(node && node->vtbl->handle_event) {
-                hres = node->vtbl->handle_event(node, eid, nsevent, &prevent_default);
-                if(FAILED(hres) || prevent_default)
+            if(node) {
+                if(node->vtbl->handle_event)
+                    hres = node->vtbl->handle_event(node, eid, nsevent, &prevent_default);
+                node_release(node);
+                if(FAILED(hres) || prevent_default || (event_obj && event_obj->cancel_bubble))
                     break;
             }
 
@@ -1186,15 +1209,16 @@ static HRESULT ensure_nsevent_handler(HTMLDocumentNode *doc, event_target_t *eve
 
 void detach_events(HTMLDocumentNode *doc)
 {
-    int i;
+    if(doc->event_vector) {
+        int i;
 
-    if(!doc->event_vector)
-        return;
-
-    for(i=0; i < EVENTID_LAST; i++) {
-        if(doc->event_vector[i])
-            detach_nsevent(doc, event_info[i].name);
+        for(i=0; i < EVENTID_LAST; i++) {
+            if(doc->event_vector[i])
+                detach_nsevent(doc, event_info[i].name);
+        }
     }
+
+    release_nsevents(doc);
 }
 
 
@@ -1242,7 +1266,7 @@ HRESULT set_event_handler(event_target_t **event_target, nsIDOMNode *nsnode, HTM
         return set_event_handler_disp(event_target, nsnode, doc, eid, V_DISPATCH(var));
 
     default:
-        FIXME("not supported vt=%d\n", V_VT(var));
+        FIXME("not handler %s\n", debugstr_variant(var));
         /* fall through */
     case VT_EMPTY:
         return E_NOTIMPL;
@@ -1326,7 +1350,7 @@ HRESULT detach_event(event_target_t *event_target, HTMLDocument *doc, BSTR name,
     return S_OK;
 }
 
-void update_cp_events(HTMLWindow *window, event_target_t **event_target_ptr, cp_static_data_t *cp, nsIDOMNode *nsnode)
+void update_cp_events(HTMLInnerWindow *window, event_target_t **event_target_ptr, cp_static_data_t *cp, nsIDOMNode *nsnode)
 {
     event_target_t *event_target;
     int i;
@@ -1364,11 +1388,13 @@ void check_event_attr(HTMLDocumentNode *doc, nsIDOMElement *nselem)
 
             TRACE("%p.%s = %s\n", nselem, debugstr_w(event_info[i].attr_name), debugstr_w(attr_value));
 
-            disp = script_parse_event(doc->basedoc.window, attr_value);
+            disp = script_parse_event(doc->basedoc.window->base.inner_window, attr_value);
             if(disp) {
                 hres = get_node(doc, (nsIDOMNode*)nselem, TRUE, &node);
-                if(SUCCEEDED(hres))
+                if(SUCCEEDED(hres)) {
                     set_event_handler_disp(get_node_event_target(node), node->nsnode, node->doc, i, disp);
+                    node_release(node);
+                }
                 IDispatch_Release(disp);
             }
         }
