@@ -17,6 +17,7 @@
  */
 
 #include <stdarg.h>
+#include <assert.h>
 
 #define COBJMACROS
 
@@ -38,7 +39,7 @@ static const WCHAR noW[] = {'n','o',0};
 HRESULT set_frame_doc(HTMLFrameBase *frame, nsIDOMDocument *nsdoc)
 {
     nsIDOMWindow *nswindow;
-    HTMLWindow *window;
+    HTMLOuterWindow *window;
     nsresult nsres;
     HRESULT hres = S_OK;
 
@@ -51,7 +52,7 @@ HRESULT set_frame_doc(HTMLFrameBase *frame, nsIDOMDocument *nsdoc)
 
     window = nswindow_to_window(nswindow);
     if(!window)
-        hres = HTMLWindow_Create(frame->element.node.doc->basedoc.doc_obj, nswindow,
+        hres = HTMLOuterWindow_Create(frame->element.node.doc->basedoc.doc_obj, nswindow,
                 frame->element.node.doc->basedoc.window, &window);
     nsIDOMWindow_Release(nswindow);
     if(FAILED(hres))
@@ -155,41 +156,21 @@ static HRESULT WINAPI HTMLFrameBase_get_name(IHTMLFrameBase *iface, BSTR *p)
 {
     HTMLFrameBase *This = impl_from_IHTMLFrameBase(iface);
     nsAString nsstr;
-    const PRUnichar *strdata;
     nsresult nsres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    if(This->nsframe) {
-        nsAString_Init(&nsstr, NULL);
-        nsres = nsIDOMHTMLFrameElement_GetName(This->nsframe, &nsstr);
-    }else if(This->nsiframe) {
-        nsAString_Init(&nsstr, NULL);
-        nsres = nsIDOMHTMLIFrameElement_GetName(This->nsiframe, &nsstr);
-    }else {
+    if(!This->nsframe && !This->nsiframe) {
         ERR("No attached ns frame object\n");
         return E_UNEXPECTED;
     }
 
-    if(NS_FAILED(nsres)) {
-        ERR("GetName failed: 0x%08x\n", nsres);
-        nsAString_Finish(&nsstr);
-        return E_FAIL;
-    }
-
-    nsAString_GetData(&nsstr, &strdata);
-    if(*strdata) {
-        *p = SysAllocString(strdata);
-        if(!*p) {
-            nsAString_Finish(&nsstr);
-            return E_OUTOFMEMORY;
-        }
-    }else
-        *p = NULL;
-
-    nsAString_Finish(&nsstr);
-
-    return S_OK;
+    nsAString_Init(&nsstr, NULL);
+    if(This->nsframe)
+        nsres = nsIDOMHTMLFrameElement_GetName(This->nsframe, &nsstr);
+    else
+        nsres = nsIDOMHTMLIFrameElement_GetName(This->nsiframe, &nsstr);
+    return return_nsstr(nsres, &nsstr, p);
 }
 
 static HRESULT WINAPI HTMLFrameBase_put_border(IHTMLFrameBase *iface, VARIANT v)
@@ -438,8 +419,8 @@ static HRESULT WINAPI HTMLFrameBase2_get_contentWindow(IHTMLFrameBase2 *iface, I
     TRACE("(%p)->(%p)\n", This, p);
 
     if(This->content_window) {
-        IHTMLWindow2_AddRef(&This->content_window->IHTMLWindow2_iface);
-        *p = &This->content_window->IHTMLWindow2_iface;
+        IHTMLWindow2_AddRef(&This->content_window->base.IHTMLWindow2_iface);
+        *p = &This->content_window->base.IHTMLWindow2_iface;
     }else {
         WARN("NULL content window\n");
         *p = NULL;
@@ -481,12 +462,12 @@ static HRESULT WINAPI HTMLFrameBase2_get_readyState(IHTMLFrameBase2 *iface, BSTR
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    if(!This->content_window || !This->content_window->doc) {
+    if(!This->content_window || !This->content_window->base.inner_window->doc) {
         FIXME("no document associated\n");
         return E_FAIL;
     }
 
-    return IHTMLDocument2_get_readyState(&This->content_window->doc->basedoc.IHTMLDocument2_iface, p);
+    return IHTMLDocument2_get_readyState(&This->content_window->base.inner_window->doc->basedoc.IHTMLDocument2_iface, p);
 }
 
 static HRESULT WINAPI HTMLFrameBase2_put_allowTransparency(IHTMLFrameBase2 *iface, VARIANT_BOOL v)
@@ -542,11 +523,6 @@ void HTMLFrameBase_destructor(HTMLFrameBase *This)
     if(This->content_window)
         This->content_window->frame_element = NULL;
 
-    if(This->nsframe)
-        nsIDOMHTMLFrameElement_Release(This->nsframe);
-    if(This->nsiframe)
-        nsIDOMHTMLIFrameElement_Release(This->nsiframe);
-
     HTMLElement_destructor(&This->element.node);
 }
 
@@ -562,9 +538,14 @@ void HTMLFrameBase_Init(HTMLFrameBase *This, HTMLDocumentNode *doc, nsIDOMHTMLEl
 
     nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLFrameElement, (void**)&This->nsframe);
     if(NS_FAILED(nsres)) {
+        This->nsframe = NULL;
         nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLIFrameElement, (void**)&This->nsiframe);
-        if(NS_FAILED(nsres))
-            ERR("Could not get nsIDOMHTML[I]Frame interface\n");
-    }else
+        assert(nsres == NS_OK && (nsIDOMNode*)This->nsiframe == This->element.node.nsnode);
+    }else {
+        assert((nsIDOMNode*)This->nsframe == This->element.node.nsnode);
         This->nsiframe = NULL;
+    }
+
+    /* Share the reference with nsnode */
+    nsIDOMNode_Release(This->element.node.nsnode);
 }
