@@ -17,6 +17,7 @@
  */
 
 #include <stdarg.h>
+#include <assert.h>
 
 #define COBJMACROS
 
@@ -251,17 +252,7 @@ static HRESULT WINAPI HTMLImgElement_get_alt(IHTMLImgElement *iface, BSTR *p)
 
     nsAString_Init(&alt_str, NULL);
     nsres = nsIDOMHTMLImageElement_GetAlt(This->nsimg, &alt_str);
-    if(NS_SUCCEEDED(nsres)) {
-        const PRUnichar *alt;
-
-        nsAString_GetData(&alt_str, &alt);
-        *p = *alt ? SysAllocString(alt) : NULL;
-    }else {
-        ERR("GetAlt failed: %08x\n", nsres);
-    }
-    nsAString_Finish(&alt_str);
-
-    return NS_SUCCEEDED(nsres) ? S_OK : E_FAIL;
+    return return_nsstr(nsres, &alt_str, p);
 }
 
 static HRESULT WINAPI HTMLImgElement_put_src(IHTMLImgElement *iface, BSTR v)
@@ -420,15 +411,19 @@ static HRESULT WINAPI HTMLImgElement_get_onload(IHTMLImgElement *iface, VARIANT
 static HRESULT WINAPI HTMLImgElement_put_onerror(IHTMLImgElement *iface, VARIANT v)
 {
     HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
-    FIXME("(%p)->()\n", This);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->()\n", This);
+
+    return set_node_event(&This->element.node, EVENTID_ERROR, &v);
 }
 
 static HRESULT WINAPI HTMLImgElement_get_onerror(IHTMLImgElement *iface, VARIANT *p)
 {
     HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return get_node_event(&This->element.node, EVENTID_ERROR, p);
 }
 
 static HRESULT WINAPI HTMLImgElement_put_onabort(IHTMLImgElement *iface, VARIANT v)
@@ -455,24 +450,14 @@ static HRESULT WINAPI HTMLImgElement_put_name(IHTMLImgElement *iface, BSTR v)
 static HRESULT WINAPI HTMLImgElement_get_name(IHTMLImgElement *iface, BSTR *p)
 {
     HTMLImgElement *This = impl_from_IHTMLImgElement(iface);
-    nsAString strName;
+    nsAString name;
     nsresult nsres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    nsAString_Init(&strName, NULL);
-    nsres = nsIDOMHTMLImageElement_GetName(This->nsimg, &strName);
-    if(NS_SUCCEEDED(nsres)) {
-        const PRUnichar *str;
-
-        nsAString_GetData(&strName, &str);
-        *p = *str ? SysAllocString(str) : NULL;
-    }else {
-        ERR("GetName failed: %08x\n", nsres);
-    }
-    nsAString_Finish(&strName);
-
-    return NS_SUCCEEDED(nsres) ? S_OK : E_FAIL;
+    nsAString_Init(&name, NULL);
+    nsres = nsIDOMHTMLImageElement_GetName(This->nsimg, &name);
+    return return_nsstr(nsres, &name, p);
 }
 
 static HRESULT WINAPI HTMLImgElement_put_width(IHTMLImgElement *iface, LONG v)
@@ -637,16 +622,6 @@ static HRESULT HTMLImgElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
     return S_OK;
 }
 
-static void HTMLImgElement_destructor(HTMLDOMNode *iface)
-{
-    HTMLImgElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nsimg)
-        nsIDOMHTMLImageElement_Release(This->nsimg);
-
-    HTMLElement_destructor(&This->element.node);
-}
-
 static HRESULT HTMLImgElement_get_readystate(HTMLDOMNode *iface, BSTR *p)
 {
     HTMLImgElement *This = impl_from_HTMLDOMNode(iface);
@@ -656,7 +631,7 @@ static HRESULT HTMLImgElement_get_readystate(HTMLDOMNode *iface, BSTR *p)
 
 static const NodeImplVtbl HTMLImgElementImplVtbl = {
     HTMLImgElement_QI,
-    HTMLImgElement_destructor,
+    HTMLElement_destructor,
     HTMLElement_clone,
     HTMLElement_get_attr_col,
     NULL,
@@ -692,15 +667,14 @@ HRESULT HTMLImgElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nselem,
     ret->IHTMLImgElement_iface.lpVtbl = &HTMLImgElementVtbl;
     ret->element.node.vtbl = &HTMLImgElementImplVtbl;
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLImageElement, (void**)&ret->nsimg);
-    if(NS_FAILED(nsres)) {
-        ERR("Could not get nsIDOMHTMLImageElement: %08x\n", nsres);
-        heap_free(ret);
-        return E_FAIL;
-    }
-
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLImgElement_dispex);
 
+    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLImageElement, (void**)&ret->nsimg);
+
+    /* Share nsimg reference with nsnode */
+    assert(nsres == NS_OK && (nsIDOMNode*)ret->nsimg == ret->element.node.nsnode);
+    nsIDOMNode_Release(ret->element.node.nsnode);
+
     *elem = &ret->element;
     return S_OK;
 }
@@ -823,6 +797,7 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
         VARIANT width, VARIANT height, IHTMLImgElement **img_elem)
 {
     HTMLImageElementFactory *This = impl_from_IHTMLImageElementFactory(iface);
+    HTMLDocumentNode *doc;
     IHTMLImgElement *img;
     HTMLElement *elem;
     nsIDOMHTMLElement *nselem;
@@ -834,18 +809,20 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
     TRACE("(%p)->(%s %s %p)\n", This, debugstr_variant(&width),
             debugstr_variant(&height), img_elem);
 
-    if(!This->window || !This->window->doc) {
+    if(!This->window || !This->window->base.inner_window->doc) {
         WARN("NULL doc\n");
         return E_UNEXPECTED;
     }
 
+    doc = This->window->base.inner_window->doc;
+
     *img_elem = NULL;
 
-    hres = create_nselem(This->window->doc, imgW, &nselem);
+    hres = create_nselem(doc, imgW, &nselem);
     if(FAILED(hres))
         return hres;
 
-    hres = HTMLElement_Create(This->window->doc, (nsIDOMNode*)nselem, FALSE, &elem);
+    hres = HTMLElement_Create(doc, (nsIDOMNode*)nselem, FALSE, &elem);
     nsIDOMHTMLElement_Release(nselem);
     if(FAILED(hres)) {
         ERR("HTMLElement_Create failed\n");
@@ -854,6 +831,7 @@ static HRESULT WINAPI HTMLImageElementFactory_create(IHTMLImageElementFactory *i
 
     hres = IHTMLElement_QueryInterface(&elem->IHTMLElement_iface, &IID_IHTMLImgElement,
             (void**)&img);
+    IHTMLElement_Release(&elem->IHTMLElement_iface);
     if(FAILED(hres)) {
         ERR("IHTMLElement_QueryInterface failed: 0x%08x\n", hres);
         return hres;
@@ -933,11 +911,13 @@ static dispex_static_data_t HTMLImageElementFactory_dispex = {
     HTMLImageElementFactory_iface_tids
 };
 
-HTMLImageElementFactory *HTMLImageElementFactory_Create(HTMLWindow *window)
+HRESULT HTMLImageElementFactory_Create(HTMLInnerWindow *window, HTMLImageElementFactory **ret_val)
 {
     HTMLImageElementFactory *ret;
 
     ret = heap_alloc(sizeof(HTMLImageElementFactory));
+    if(!ret)
+        return E_OUTOFMEMORY;
 
     ret->IHTMLImageElementFactory_iface.lpVtbl = &HTMLImageElementFactoryVtbl;
     ret->ref = 1;
@@ -946,5 +926,6 @@ HTMLImageElementFactory *HTMLImageElementFactory_Create(HTMLWindow *window)
     init_dispex(&ret->dispex, (IUnknown*)&ret->IHTMLImageElementFactory_iface,
             &HTMLImageElementFactory_dispex);
 
-    return ret;
+    *ret_val = ret;
+    return S_OK;
 }
