@@ -95,6 +95,7 @@ static ULONG WINAPI HTMLDOMChildrenCollection_Release(IHTMLDOMChildrenCollection
     TRACE("(%p) ref=%d\n", This, ref);
 
     if(!ref) {
+        htmldoc_release(&This->doc->basedoc);
         nsIDOMNodeList_Release(This->nslist);
         heap_free(This);
     }
@@ -182,7 +183,6 @@ static HRESULT WINAPI HTMLDOMChildrenCollection_item(IHTMLDOMChildrenCollection
         return hres;
 
     *ppItem = (IDispatch*)&node->IHTMLDOMNode_iface;
-    IDispatch_AddRef(*ppItem);
     return S_OK;
 }
 
@@ -286,6 +286,8 @@ static IHTMLDOMChildrenCollection *create_child_collection(HTMLDocumentNode *doc
 
     nsIDOMNodeList_AddRef(nslist);
     ret->nslist = nslist;
+
+    htmldoc_addref(&doc->basedoc);
     ret->doc = doc;
 
     init_dispex(&ret->dispex, (IUnknown*)&ret->IHTMLDOMChildrenCollection_iface,
@@ -429,7 +431,6 @@ static HRESULT WINAPI HTMLDOMNode_get_parentNode(IHTMLDOMNode *iface, IHTMLDOMNo
         return hres;
 
     *p = &node->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*p);
     return S_OK;
 }
 
@@ -494,11 +495,10 @@ static HRESULT WINAPI HTMLDOMNode_insertBefore(IHTMLDOMNode *iface, IHTMLDOMNode
                                                VARIANT refChild, IHTMLDOMNode **node)
 {
     HTMLDOMNode *This = impl_from_IHTMLDOMNode(iface);
-    nsIDOMNode *nsnode, *nsref = NULL;
-    HTMLDOMNode *new_child;
-    HTMLDOMNode *node_obj;
+    HTMLDOMNode *new_child, *node_obj, *ref_node = NULL;
+    nsIDOMNode *nsnode;
     nsresult nsres;
-    HRESULT hres;
+    HRESULT hres = S_OK;
 
     TRACE("(%p)->(%p %s %p)\n", This, newChild, debugstr_variant(&refChild), node);
 
@@ -512,27 +512,31 @@ static HRESULT WINAPI HTMLDOMNode_insertBefore(IHTMLDOMNode *iface, IHTMLDOMNode
     case VT_NULL:
         break;
     case VT_DISPATCH: {
-        HTMLDOMNode *ref_node;
-
         ref_node = get_node_obj(This->doc, (IUnknown*)V_DISPATCH(&refChild));
         if(!ref_node) {
             ERR("unvalid node\n");
-            return E_FAIL;
+            hres = E_FAIL;
+            break;
         }
-
-        nsref = ref_node->nsnode;
         break;
     }
     default:
-        FIXME("unimplemented vt %d\n", V_VT(&refChild));
-        return E_NOTIMPL;
+        FIXME("unimplemented refChild %s\n", debugstr_variant(&refChild));
+        hres = E_NOTIMPL;
     }
 
-    nsres = nsIDOMNode_InsertBefore(This->nsnode, new_child->nsnode, nsref, &nsnode);
-    if(NS_FAILED(nsres)) {
-        ERR("InsertBefore failed: %08x\n", nsres);
-        return E_FAIL;
+    if(SUCCEEDED(hres)) {
+        nsres = nsIDOMNode_InsertBefore(This->nsnode, new_child->nsnode, ref_node ? ref_node->nsnode : NULL, &nsnode);
+        if(NS_FAILED(nsres)) {
+            ERR("InsertBefore failed: %08x\n", nsres);
+            hres = E_FAIL;
+        }
     }
+    node_release(new_child);
+    if(ref_node)
+        node_release(ref_node);
+    if(FAILED(hres))
+        return hres;
 
     hres = get_node(This->doc, nsnode, TRUE, &node_obj);
     nsIDOMNode_Release(nsnode);
@@ -540,7 +544,6 @@ static HRESULT WINAPI HTMLDOMNode_insertBefore(IHTMLDOMNode *iface, IHTMLDOMNode
         return hres;
 
     *node = &node_obj->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*node);
     return S_OK;
 }
 
@@ -560,6 +563,7 @@ static HRESULT WINAPI HTMLDOMNode_removeChild(IHTMLDOMNode *iface, IHTMLDOMNode
         return E_FAIL;
 
     nsres = nsIDOMNode_RemoveChild(This->nsnode, node_obj->nsnode, &nsnode);
+    node_release(node_obj);
     if(NS_FAILED(nsres)) {
         ERR("RemoveChild failed: %08x\n", nsres);
         return E_FAIL;
@@ -572,7 +576,6 @@ static HRESULT WINAPI HTMLDOMNode_removeChild(IHTMLDOMNode *iface, IHTMLDOMNode
 
     /* FIXME: Make sure that node != newChild */
     *node = &node_obj->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*node);
     return S_OK;
 }
 
@@ -580,8 +583,7 @@ static HRESULT WINAPI HTMLDOMNode_replaceChild(IHTMLDOMNode *iface, IHTMLDOMNode
                                                IHTMLDOMNode *oldChild, IHTMLDOMNode **node)
 {
     HTMLDOMNode *This = impl_from_IHTMLDOMNode(iface);
-    HTMLDOMNode *node_new;
-    HTMLDOMNode *node_old;
+    HTMLDOMNode *node_new, *node_old, *ret_node;
     nsIDOMNode *nsnode;
     nsresult nsres;
     HRESULT hres;
@@ -593,23 +595,23 @@ static HRESULT WINAPI HTMLDOMNode_replaceChild(IHTMLDOMNode *iface, IHTMLDOMNode
         return E_FAIL;
 
     node_old = get_node_obj(This->doc, (IUnknown*)oldChild);
-    if(!node_old)
-        return E_FAIL;
-
-    nsres = nsIDOMNode_ReplaceChild(This->nsnode, node_new->nsnode, node_old->nsnode, &nsnode);
-    if(NS_FAILED(nsres)) {
+    if(!node_old) {
+        node_release(node_new);
         return E_FAIL;
     }
 
-    nsnode = node_new->nsnode;
+    nsres = nsIDOMNode_ReplaceChild(This->nsnode, node_new->nsnode, node_old->nsnode, &nsnode);
+    node_release(node_new);
+    node_release(node_old);
+    if(NS_FAILED(nsres))
+        return E_FAIL;
 
-    hres = get_node(This->doc, nsnode, TRUE, &node_new);
+    hres = get_node(This->doc, nsnode, TRUE, &ret_node);
     nsIDOMNode_Release(nsnode);
     if(FAILED(hres))
         return hres;
 
-    *node = &node_new->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*node);
+    *node = &ret_node->IHTMLDOMNode_iface;
     return S_OK;
 }
 
@@ -624,7 +626,7 @@ static HRESULT WINAPI HTMLDOMNode_cloneNode(IHTMLDOMNode *iface, VARIANT_BOOL fD
 
     TRACE("(%p)->(%x %p)\n", This, fDeep, clonedNode);
 
-    nsres = nsIDOMNode_CloneNode(This->nsnode, fDeep != VARIANT_FALSE, &nsnode);
+    nsres = nsIDOMNode_CloneNode(This->nsnode, fDeep != VARIANT_FALSE, 1, &nsnode);
     if(NS_FAILED(nsres) || !nsnode) {
         ERR("CloneNode failed: %08x\n", nsres);
         return E_FAIL;
@@ -678,9 +680,10 @@ static HRESULT WINAPI HTMLDOMNode_appendChild(IHTMLDOMNode *iface, IHTMLDOMNode
         return E_FAIL;
 
     nsres = nsIDOMNode_AppendChild(This->nsnode, node_obj->nsnode, &nsnode);
+    node_release(node_obj);
     if(NS_FAILED(nsres)) {
-        WARN("AppendChild failed: %08x\n", nsres);
-        nsnode = node_obj->nsnode;
+        ERR("AppendChild failed: %08x\n", nsres);
+        return E_FAIL;
     }
 
     hres = get_node(This->doc, nsnode, TRUE, &node_obj);
@@ -690,51 +693,32 @@ static HRESULT WINAPI HTMLDOMNode_appendChild(IHTMLDOMNode *iface, IHTMLDOMNode
 
     /* FIXME: Make sure that node != newChild */
     *node = &node_obj->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*node);
     return S_OK;
 }
 
 static HRESULT WINAPI HTMLDOMNode_get_nodeName(IHTMLDOMNode *iface, BSTR *p)
 {
     HTMLDOMNode *This = impl_from_IHTMLDOMNode(iface);
+    nsAString name;
+    nsresult nsres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
-    *p = NULL;
-
-    if(This->nsnode) {
-        nsAString name_str;
-        const PRUnichar *name;
-        nsresult nsres;
-
-        nsAString_Init(&name_str, NULL);
-        nsres = nsIDOMNode_GetNodeName(This->nsnode, &name_str);
-
-        if(NS_SUCCEEDED(nsres)) {
-            nsAString_GetData(&name_str, &name);
-            *p = SysAllocString(name);
-        }else {
-            ERR("GetNodeName failed: %08x\n", nsres);
-        }
-
-        nsAString_Finish(&name_str);
-    }
-
-    return S_OK;
+    nsAString_Init(&name, NULL);
+    nsres = nsIDOMNode_GetNodeName(This->nsnode, &name);
+    return return_nsstr(nsres, &name, p);
 }
 
 static HRESULT WINAPI HTMLDOMNode_put_nodeValue(IHTMLDOMNode *iface, VARIANT v)
 {
     HTMLDOMNode *This = impl_from_IHTMLDOMNode(iface);
 
-    TRACE("(%p)->()\n", This);
+    TRACE("(%p)->(%s)\n", This, debugstr_variant(&v));
 
     switch(V_VT(&v)) {
     case VT_BSTR: {
         nsAString val_str;
 
-        TRACE("bstr %s\n", debugstr_w(V_BSTR(&v)));
-
         nsAString_InitDepend(&val_str, V_BSTR(&v));
         nsIDOMNode_SetNodeValue(This->nsnode, &val_str);
         nsAString_Finish(&val_str);
@@ -743,7 +727,7 @@ static HRESULT WINAPI HTMLDOMNode_put_nodeValue(IHTMLDOMNode *iface, VARIANT v)
     }
 
     default:
-        FIXME("unsupported vt %d\n", V_VT(&v));
+        FIXME("unsupported value %s\n", debugstr_variant(&v));
     }
 
     return E_NOTIMPL;
@@ -794,7 +778,6 @@ static HRESULT WINAPI HTMLDOMNode_get_firstChild(IHTMLDOMNode *iface, IHTMLDOMNo
         return hres;
 
     *p = &node->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*p);
     return S_OK;
 }
 
@@ -819,7 +802,6 @@ static HRESULT WINAPI HTMLDOMNode_get_lastChild(IHTMLDOMNode *iface, IHTMLDOMNod
         return hres;
 
     *p = &node->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*p);
     return S_OK;
 }
 
@@ -844,7 +826,6 @@ static HRESULT WINAPI HTMLDOMNode_get_previousSibling(IHTMLDOMNode *iface, IHTML
         return hres;
 
     *p = &node->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*p);
     return S_OK;
 }
 
@@ -869,7 +850,6 @@ static HRESULT WINAPI HTMLDOMNode_get_nextSibling(IHTMLDOMNode *iface, IHTMLDOMN
         return hres;
 
     *p = &node->IHTMLDOMNode_iface;
-    IHTMLDOMNode_AddRef(*p);
     return S_OK;
 }
 
@@ -903,6 +883,15 @@ static const IHTMLDOMNodeVtbl HTMLDOMNodeVtbl = {
     HTMLDOMNode_get_nextSibling
 };
 
+static HTMLDOMNode *get_node_obj(HTMLDocumentNode *This, IUnknown *iface)
+{
+    IHTMLDOMNode *node;
+    HRESULT hres;
+
+    hres = IUnknown_QueryInterface(iface, &IID_IHTMLDOMNode, (void**)&node);
+    return hres == S_OK && node->lpVtbl == &HTMLDOMNodeVtbl ? impl_from_IHTMLDOMNode(node) : NULL;
+}
+
 static inline HTMLDOMNode *impl_from_IHTMLDOMNode2(IHTMLDOMNode2 *iface)
 {
     return CONTAINING_RECORD(iface, HTMLDOMNode, IHTMLDOMNode2_iface);
@@ -1035,14 +1024,7 @@ void HTMLDOMNode_destructor(HTMLDOMNode *This)
 
 static HRESULT HTMLDOMNode_clone(HTMLDOMNode *This, nsIDOMNode *nsnode, HTMLDOMNode **ret)
 {
-    HRESULT hres;
-
-    hres = create_node(This->doc, nsnode, ret);
-    if(FAILED(hres))
-        return hres;
-
-    IHTMLDOMNode_AddRef(&(*ret)->IHTMLDOMNode_iface);
-    return S_OK;
+    return create_node(This->doc, nsnode, ret);
 }
 
 static const NodeImplVtbl HTMLDOMNodeImplVtbl = {
@@ -1055,9 +1037,12 @@ void HTMLDOMNode_Init(HTMLDocumentNode *doc, HTMLDOMNode *node, nsIDOMNode *nsno
 {
     node->IHTMLDOMNode_iface.lpVtbl = &HTMLDOMNodeVtbl;
     node->IHTMLDOMNode2_iface.lpVtbl = &HTMLDOMNode2Vtbl;
-    node->ref = 1;
+    node->ref = 2;
     node->doc = doc;
 
+    if(&doc->node != node)
+        node->ref++; /* one extra for list entry reference */
+
     if(nsnode)
         nsIDOMNode_AddRef(nsnode);
     node->nsnode = nsnode;
@@ -1132,6 +1117,8 @@ HRESULT get_node(HTMLDocumentNode *This, nsIDOMNode *nsnode, BOOL create, HTMLDO
     }
 
     if(iter || !create) {
+        if(iter)
+            IHTMLDOMNode_AddRef(&iter->IHTMLDOMNode_iface);
         *ret = iter;
         return S_OK;
     }
@@ -1139,29 +1126,6 @@ HRESULT get_node(HTMLDocumentNode *This, nsIDOMNode *nsnode, BOOL create, HTMLDO
     return create_node(This, nsnode, ret);
 }
 
-/*
- * FIXME
- * We should use better way for getting node object (like private interface)
- * or avoid it at all.
- */
-static HTMLDOMNode *get_node_obj(HTMLDocumentNode *This, IUnknown *iface)
-{
-    HTMLDOMNode *iter = This->nodes;
-    IHTMLDOMNode *node;
-
-    IUnknown_QueryInterface(iface, &IID_IHTMLDOMNode, (void**)&node);
-    IHTMLDOMNode_Release(node);
-
-    while(iter) {
-        if(&iter->IHTMLDOMNode_iface == node)
-            return iter;
-        iter = iter->next;
-    }
-
-    FIXME("Not found %p\n", iface);
-    return NULL;
-}
-
 void release_nodes(HTMLDocumentNode *This)
 {
     HTMLDOMNode *iter, *next;
