@@ -17,6 +17,7 @@
  */
 
 #include <stdarg.h>
+#include <assert.h>
 
 #define COBJMACROS
 
@@ -37,6 +38,7 @@ typedef struct {
     HTMLPluginContainer plugin_container;
 
     IHTMLObjectElement IHTMLObjectElement_iface;
+    IHTMLObjectElement2 IHTMLObjectElement2_iface;
 
     nsIDOMHTMLObjectElement *nsobject;
 } HTMLObjectElement;
@@ -114,15 +116,19 @@ static HRESULT WINAPI HTMLObjectElement_get_object(IHTMLObjectElement *iface, ID
 static HRESULT WINAPI HTMLObjectElement_get_classid(IHTMLObjectElement *iface, BSTR *p)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return IHTMLObjectElement2_get_classid(&This->IHTMLObjectElement_iface, p);
 }
 
 static HRESULT WINAPI HTMLObjectElement_get_data(IHTMLObjectElement *iface, BSTR *p)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return IHTMLObjectElement2_get_data(&This->IHTMLObjectElement_iface, p);
 }
 
 static HRESULT WINAPI HTMLObjectElement_put_recordset(IHTMLObjectElement *iface, IDispatch *v)
@@ -397,6 +403,117 @@ static const IHTMLObjectElementVtbl HTMLObjectElementVtbl = {
     HTMLObjectElement_get_hspace
 };
 
+static inline HTMLObjectElement *impl_from_IHTMLObjectElement2(IHTMLObjectElement2 *iface)
+{
+    return CONTAINING_RECORD(iface, HTMLObjectElement, IHTMLObjectElement2_iface);
+}
+
+static HRESULT WINAPI HTMLObjectElement2_QueryInterface(IHTMLObjectElement2 *iface,
+        REFIID riid, void **ppv)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+
+    return IHTMLDOMNode_QueryInterface(&This->plugin_container.element.node.IHTMLDOMNode_iface,
+            riid, ppv);
+}
+
+static ULONG WINAPI HTMLObjectElement2_AddRef(IHTMLObjectElement2 *iface)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+
+    return IHTMLDOMNode_AddRef(&This->plugin_container.element.node.IHTMLDOMNode_iface);
+}
+
+static ULONG WINAPI HTMLObjectElement2_Release(IHTMLObjectElement2 *iface)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+
+    return IHTMLDOMNode_Release(&This->plugin_container.element.node.IHTMLDOMNode_iface);
+}
+
+static HRESULT WINAPI HTMLObjectElement2_GetTypeInfoCount(IHTMLObjectElement2 *iface, UINT *pctinfo)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    return IDispatchEx_GetTypeInfoCount(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+            pctinfo);
+}
+
+static HRESULT WINAPI HTMLObjectElement2_GetTypeInfo(IHTMLObjectElement2 *iface, UINT iTInfo,
+                                              LCID lcid, ITypeInfo **ppTInfo)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    return IDispatchEx_GetTypeInfo(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+            iTInfo, lcid, ppTInfo);
+}
+
+static HRESULT WINAPI HTMLObjectElement2_GetIDsOfNames(IHTMLObjectElement2 *iface, REFIID riid,
+        LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    return IDispatchEx_GetIDsOfNames(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+            riid, rgszNames, cNames, lcid, rgDispId);
+}
+
+static HRESULT WINAPI HTMLObjectElement2_Invoke(IHTMLObjectElement2 *iface, DISPID dispIdMember,
+        REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams, VARIANT *pVarResult,
+        EXCEPINFO *pExcepInfo, UINT *puArgErr)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    return IDispatchEx_Invoke(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+            dispIdMember, riid, lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
+}
+
+static HRESULT WINAPI HTMLObjectElement2_namedRecordset(IHTMLObjectElement2 *iface, BSTR dataMember,
+        VARIANT *hierarchy, IDispatch **ppRecordset)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    FIXME("(%p)->(%s %p %p)\n", This, debugstr_w(dataMember), hierarchy, ppRecordset);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI HTMLObjectElement2_put_classid(IHTMLObjectElement2 *iface, BSTR v)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI HTMLObjectElement2_get_classid(IHTMLObjectElement2 *iface, BSTR *p)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    FIXME("(%p)->(%p)\n", This, p);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI HTMLObjectElement2_put_data(IHTMLObjectElement2 *iface, BSTR v)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI HTMLObjectElement2_get_data(IHTMLObjectElement2 *iface, BSTR *p)
+{
+    HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
+    FIXME("(%p)->(%p)\n", This, p);
+    return E_NOTIMPL;
+}
+
+static const IHTMLObjectElement2Vtbl HTMLObjectElement2Vtbl = {
+    HTMLObjectElement2_QueryInterface,
+    HTMLObjectElement2_AddRef,
+    HTMLObjectElement2_Release,
+    HTMLObjectElement2_GetTypeInfoCount,
+    HTMLObjectElement2_GetTypeInfo,
+    HTMLObjectElement2_GetIDsOfNames,
+    HTMLObjectElement2_Invoke,
+    HTMLObjectElement2_namedRecordset,
+    HTMLObjectElement2_put_classid,
+    HTMLObjectElement2_get_classid,
+    HTMLObjectElement2_put_data,
+    HTMLObjectElement2_get_data
+};
+
 static inline HTMLObjectElement *impl_from_HTMLDOMNode(HTMLDOMNode *iface)
 {
     return CONTAINING_RECORD(iface, HTMLObjectElement, plugin_container.element.node);
@@ -415,6 +532,9 @@ static HRESULT HTMLObjectElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
     }else if(IsEqualGUID(&IID_IHTMLObjectElement, riid)) {
         TRACE("(%p)->(IID_IHTMLObjectElement %p)\n", This, ppv);
         *ppv = &This->IHTMLObjectElement_iface;
+    }else if(IsEqualGUID(&IID_IHTMLObjectElement2, riid)) {
+        TRACE("(%p)->(IID_IHTMLObjectElement2 %p)\n", This, ppv);
+        *ppv = &This->IHTMLObjectElement2_iface;
     }else if(IsEqualGUID(&IID_HTMLPluginContainer, riid)) {
         TRACE("(%p)->(IID_HTMLPluginContainer %p)\n", This, ppv);
         *ppv = &This->plugin_container;
@@ -452,8 +572,6 @@ static void HTMLObjectElement_destructor(HTMLDOMNode *iface)
 
     if(This->plugin_container.plugin_host)
         detach_plugin_host(This->plugin_container.plugin_host);
-    if(This->nsobject)
-        nsIDOMHTMLObjectElement_Release(This->nsobject);
 
     HTMLElement_destructor(&This->plugin_container.element.node);
 }
@@ -504,6 +622,7 @@ static const NodeImplVtbl HTMLObjectElementImplVtbl = {
 static const tid_t HTMLObjectElement_iface_tids[] = {
     HTMLELEMENT_TIDS,
     IHTMLObjectElement_tid,
+    IHTMLObjectElement2_tid,
     0
 };
 static dispex_static_data_t HTMLObjectElement_dispex = {
@@ -523,17 +642,17 @@ HRESULT HTMLObjectElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nsele
         return E_OUTOFMEMORY;
 
     ret->IHTMLObjectElement_iface.lpVtbl = &HTMLObjectElementVtbl;
+    ret->IHTMLObjectElement2_iface.lpVtbl = &HTMLObjectElement2Vtbl;
     ret->plugin_container.element.node.vtbl = &HTMLObjectElementImplVtbl;
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLObjectElement, (void**)&ret->nsobject);
-    if(NS_FAILED(nsres)) {
-        ERR("Could not get nsIDOMHTMLObjectElement iface: %08x\n", nsres);
-        heap_free(ret);
-        return E_FAIL;
-    }
-
     HTMLElement_Init(&ret->plugin_container.element, doc, nselem, &HTMLObjectElement_dispex);
 
+    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLObjectElement, (void**)&ret->nsobject);
+
+    /* Share nsobject reference with nsnode */
+    assert(nsres == NS_OK && (nsIDOMNode*)ret->nsobject == ret->plugin_container.element.node.nsnode);
+    nsIDOMNode_Release(ret->plugin_container.element.node.nsnode);
+
     *elem = &ret->plugin_container.element;
     return S_OK;
 }
