@@ -17,6 +17,7 @@
  */
 
 #include <stdarg.h>
+#include <assert.h>
 
 #define COBJMACROS
 
@@ -153,23 +154,13 @@ static HRESULT WINAPI HTMLOptionElement_get_value(IHTMLOptionElement *iface, BST
 {
     HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
     nsAString value_str;
-    const PRUnichar *value;
     nsresult nsres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
     nsAString_Init(&value_str, NULL);
     nsres = nsIDOMHTMLOptionElement_GetValue(This->nsoption, &value_str);
-    if(NS_SUCCEEDED(nsres)) {
-        nsAString_GetData(&value_str, &value);
-        *p = SysAllocString(value);
-    }else {
-        ERR("GetValue failed: %08x\n", nsres);
-        *p = NULL;
-    }
-    nsAString_Finish(&value_str);
-
-    return S_OK;
+    return return_nsstr(nsres, &value_str, p);
 }
 
 static HRESULT WINAPI HTMLOptionElement_put_defaultSelected(IHTMLOptionElement *iface, VARIANT_BOOL v)
@@ -253,23 +244,13 @@ static HRESULT WINAPI HTMLOptionElement_get_text(IHTMLOptionElement *iface, BSTR
 {
     HTMLOptionElement *This = impl_from_IHTMLOptionElement(iface);
     nsAString text_str;
-    const PRUnichar *text;
     nsresult nsres;
 
     TRACE("(%p)->(%p)\n", This, p);
 
     nsAString_Init(&text_str, NULL);
     nsres = nsIDOMHTMLOptionElement_GetText(This->nsoption, &text_str);
-    if(NS_SUCCEEDED(nsres)) {
-        nsAString_GetData(&text_str, &text);
-        *p = SysAllocString(text);
-    }else {
-        ERR("GetText failed: %08x\n", nsres);
-        *p = NULL;
-    }
-    nsAString_Finish(&text_str);
-
-    return S_OK;
+    return return_nsstr(nsres, &text_str, p);
 }
 
 static HRESULT WINAPI HTMLOptionElement_get_form(IHTMLOptionElement *iface, IHTMLFormElement **p)
@@ -330,19 +311,9 @@ static HRESULT HTMLOptionElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
     return HTMLElement_QI(&This->element.node, riid, ppv);
 }
 
-static void HTMLOptionElement_destructor(HTMLDOMNode *iface)
-{
-    HTMLOptionElement *This = impl_from_HTMLDOMNode(iface);
-
-    if(This->nsoption)
-        nsIDOMHTMLOptionElement_Release(This->nsoption);
-
-    HTMLElement_destructor(&This->element.node);
-}
-
 static const NodeImplVtbl HTMLOptionElementImplVtbl = {
     HTMLOptionElement_QI,
-    HTMLOptionElement_destructor,
+    HTMLElement_destructor,
     HTMLElement_clone,
     HTMLElement_get_attr_col
 };
@@ -371,15 +342,14 @@ HRESULT HTMLOptionElement_Create(HTMLDocumentNode *doc, nsIDOMHTMLElement *nsele
     ret->IHTMLOptionElement_iface.lpVtbl = &HTMLOptionElementVtbl;
     ret->element.node.vtbl = &HTMLOptionElementImplVtbl;
 
-    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLOptionElement, (void**)&ret->nsoption);
-    if(NS_FAILED(nsres)) {
-        ERR("Could not get nsIDOMHTMLOptionElement interface: %08x\n", nsres);
-        heap_free(ret);
-        return E_FAIL;
-    }
-
     HTMLElement_Init(&ret->element, doc, nselem, &HTMLOptionElement_dispex);
 
+    nsres = nsIDOMHTMLElement_QueryInterface(nselem, &IID_nsIDOMHTMLOptionElement, (void**)&ret->nsoption);
+
+    /* Share nsoption reference with nsnode */
+    assert(nsres == NS_OK && (nsIDOMNode*)ret->nsoption == ret->element.node.nsnode);
+    nsIDOMNode_Release(ret->element.node.nsnode);
+
     *elem = &ret->element;
     return S_OK;
 }
@@ -488,39 +458,40 @@ static HRESULT WINAPI HTMLOptionElementFactory_create(IHTMLOptionElementFactory
     TRACE("(%p)->(%s %s %s %s %p)\n", This, debugstr_variant(&text), debugstr_variant(&value),
           debugstr_variant(&defaultselected), debugstr_variant(&selected), optelem);
 
-    if(!This->window || !This->window->doc) {
+    if(!This->window || !This->window->base.inner_window->doc) {
         WARN("NULL doc\n");
         return E_UNEXPECTED;
     }
 
     *optelem = NULL;
 
-    hres = create_nselem(This->window->doc, optionW, &nselem);
+    hres = create_nselem(This->window->base.inner_window->doc, optionW, &nselem);
     if(FAILED(hres))
         return hres;
 
-    hres = get_node(This->window->doc, (nsIDOMNode*)nselem, TRUE, &node);
+    hres = get_node(This->window->base.inner_window->doc, (nsIDOMNode*)nselem, TRUE, &node);
     nsIDOMHTMLElement_Release(nselem);
     if(FAILED(hres))
         return hres;
 
     hres = IHTMLDOMNode_QueryInterface(&node->IHTMLDOMNode_iface,
             &IID_IHTMLOptionElement, (void**)optelem);
+    node_release(node);
 
     if(V_VT(&text) == VT_BSTR)
         IHTMLOptionElement_put_text(*optelem, V_BSTR(&text));
     else if(V_VT(&text) != VT_EMPTY)
-        FIXME("Unsupported text vt=%d\n", V_VT(&text));
+        FIXME("Unsupported text %s\n", debugstr_variant(&text));
 
     if(V_VT(&value) == VT_BSTR)
         IHTMLOptionElement_put_value(*optelem, V_BSTR(&value));
     else if(V_VT(&value) != VT_EMPTY)
-        FIXME("Unsupported value vt=%d\n", V_VT(&value));
+        FIXME("Unsupported value %s\n", debugstr_variant(&value));
 
     if(V_VT(&defaultselected) != VT_EMPTY)
-        FIXME("Unsupported defaultselected vt=%d\n", V_VT(&defaultselected));
+        FIXME("Unsupported defaultselected %s\n", debugstr_variant(&defaultselected));
     if(V_VT(&selected) != VT_EMPTY)
-        FIXME("Unsupported selected vt=%d\n", V_VT(&selected));
+        FIXME("Unsupported selected %s\n", debugstr_variant(&selected));
 
     return S_OK;
 }
@@ -536,15 +507,18 @@ static const IHTMLOptionElementFactoryVtbl HTMLOptionElementFactoryVtbl = {
     HTMLOptionElementFactory_create
 };
 
-HTMLOptionElementFactory *HTMLOptionElementFactory_Create(HTMLWindow *window)
+HRESULT HTMLOptionElementFactory_Create(HTMLInnerWindow *window, HTMLOptionElementFactory **ret_ptr)
 {
     HTMLOptionElementFactory *ret;
 
-    ret = heap_alloc(sizeof(HTMLOptionElementFactory));
+    ret = heap_alloc(sizeof(*ret));
+    if(!ret)
+        return E_OUTOFMEMORY;
 
     ret->IHTMLOptionElementFactory_iface.lpVtbl = &HTMLOptionElementFactoryVtbl;
     ret->ref = 1;
     ret->window = window;
 
-    return ret;
+    *ret_ptr = ret;
+    return S_OK;
 }
