@@ -83,6 +83,8 @@ typedef enum {
     STYLEID_MARGIN_TOP,
     STYLEID_MIN_HEIGHT,
     STYLEID_OVERFLOW,
+    STYLEID_OVERFLOW_X,
+    STYLEID_OVERFLOW_Y,
     STYLEID_PADDING,
     STYLEID_PADDING_BOTTOM,
     STYLEID_PADDING_LEFT,
