@@ -1287,6 +1287,7 @@ static nsresult NSAPI nsContextMenuListener_OnShowContextMenu(nsIContextMenuList
         return NS_ERROR_FAILURE;
 
     show_context_menu(This->doc, dwID, &pt, (IDispatch*)&node->IHTMLDOMNode_iface);
+    node_release(node);
     return NS_OK;
 }
 
@@ -1733,22 +1734,6 @@ static const nsISupportsWeakReferenceVtbl nsSupportsWeakReferenceVtbl = {
     nsSupportsWeakReference_GetWeakReference
 };
 
-nsresult create_chrome_window(nsIWebBrowserChrome *parent, nsIWebBrowserChrome **ret)
-{
-    NSContainer *new_container;
-    HRESULT hres;
-
-    if(parent->lpVtbl != &nsWebBrowserChromeVtbl)
-        return NS_ERROR_UNEXPECTED;
-
-    hres = create_nscontainer(NULL, impl_from_nsIWebBrowserChrome(parent), &new_container);
-    if(FAILED(hres))
-        return NS_ERROR_FAILURE;
-
-    *ret = &new_container->nsIWebBrowserChrome_iface;
-    return NS_OK;
-}
-
 static HRESULT init_nscontainer(NSContainer *nscontainer)
 {
     nsIWebBrowserSetup *wbsetup;
@@ -1857,7 +1842,7 @@ static HRESULT init_nscontainer(NSContainer *nscontainer)
     return S_OK;
 }
 
-HRESULT create_nscontainer(HTMLDocumentObj *doc, NSContainer *parent, NSContainer **_ret)
+HRESULT create_nscontainer(HTMLDocumentObj *doc, NSContainer **_ret)
 {
     NSContainer *ret;
     HRESULT hres;
@@ -1880,10 +1865,6 @@ HRESULT create_nscontainer(HTMLDocumentObj *doc, NSContainer *parent, NSContaine
     ret->doc = doc;
     ret->ref = 1;
 
-    if(parent)
-        nsIWebBrowserChrome_AddRef(&parent->nsIWebBrowserChrome_iface);
-    ret->parent = parent;
-
     hres = init_nscontainer(ret);
     if(SUCCEEDED(hres))
         *_ret = ret;
