@@ -33,7 +33,6 @@
 #include "pluginhost.h"
 
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
@@ -186,8 +185,9 @@ static void activate_plugin(PluginHost *host)
 {
     IClientSecurity *client_security;
     IQuickActivate *quick_activate;
+    IOleObject *ole_obj = NULL;
     IOleCommandTarget *cmdtrg;
-    IOleObject *ole_obj;
+    IViewObjectEx *view_obj;
     IDispatchEx *dispex;
     IDispatch *disp;
     RECT rect;
@@ -212,22 +212,56 @@ static void activate_plugin(PluginHost *host)
         QACONTAINER container = {sizeof(container)};
         QACONTROL control = {sizeof(control)};
 
+        TRACE("Using IQuickActivate\n");
+
         container.pClientSite = &host->IOleClientSite_iface;
         container.dwAmbientFlags = QACONTAINER_SUPPORTSMNEMONICS|QACONTAINER_MESSAGEREFLECT|QACONTAINER_USERMODE;
         container.pAdviseSink = &host->IAdviseSinkEx_iface;
         container.pPropertyNotifySink = &host->IPropertyNotifySink_iface;
 
         hres = IQuickActivate_QuickActivate(quick_activate, &container, &control);
+        IQuickActivate_Release(quick_activate);
         if(FAILED(hres))
             FIXME("QuickActivate failed: %08x\n", hres);
     }else {
-        FIXME("No IQuickActivate\n");
-        return;
+        DWORD status = 0;
+
+        hres = IUnknown_QueryInterface(host->plugin_unk, &IID_IOleObject, (void**)&ole_obj);
+        if(SUCCEEDED(hres)) {
+            hres = IOleObject_GetMiscStatus(ole_obj, DVASPECT_CONTENT, &status);
+            TRACE("GetMiscStatus returned %08x %x\n", hres, status);
+
+            hres = IOleObject_SetClientSite(ole_obj, &host->IOleClientSite_iface);
+            IOleObject_Release(ole_obj);
+            if(FAILED(hres)) {
+                FIXME("SetClientSite failed: %08x\n", hres);
+                return;
+            }
+        }else {
+            TRACE("Plugin does not support IOleObject\n");
+        }
     }
 
     load_plugin(host);
 
-    /* NOTE: Native QIs for IViewObjectEx, IActiveScript, an undocumented IID, IOleControl and IRunnableObject */
+    if(ole_obj) {
+        hres = IUnknown_QueryInterface(host->plugin_unk, &IID_IViewObjectEx, (void**)&view_obj);
+        if(SUCCEEDED(hres)) {
+            DWORD view_status = 0;
+
+            hres = IViewObjectEx_SetAdvise(view_obj, DVASPECT_CONTENT, 0, (IAdviseSink*)&host->IAdviseSinkEx_iface);
+            if(FAILED(hres))
+                WARN("SetAdvise failed: %08x\n", hres);
+
+            hres = IViewObjectEx_GetViewStatus(view_obj, &view_status);
+            IViewObjectEx_Release(view_obj);
+            TRACE("GetViewStatus returned %08x %x\n", hres, view_status);
+        }
+    }
+
+    update_readystate(host);
+
+    /* NOTE: Native QIs for IActiveScript, an undocumented IID, IOleControl and IRunnableObject */
 
     hres = IUnknown_QueryInterface(host->plugin_unk, &IID_IDispatchEx, (void**)&dispex);
     if(SUCCEEDED(hres)) {
@@ -253,11 +287,13 @@ static void activate_plugin(PluginHost *host)
         return;
     }
 
-    get_pos_rect(host, &rect);
-    hres = IOleObject_DoVerb(ole_obj, OLEIVERB_INPLACEACTIVATE, NULL, &host->IOleClientSite_iface, 0, host->hwnd, &rect);
-    IOleObject_Release(ole_obj);
-    if(FAILED(hres))
-        WARN("DoVerb failed: %08x\n", hres);
+    if(ole_obj) {
+        get_pos_rect(host, &rect);
+        hres = IOleObject_DoVerb(ole_obj, OLEIVERB_INPLACEACTIVATE, NULL, &host->IOleClientSite_iface, 0, host->hwnd, &rect);
+        IOleObject_Release(ole_obj);
+        if(FAILED(hres))
+            WARN("DoVerb failed: %08x\n", hres);
+    }
 
     if(host->ip_object) {
         HWND hwnd;
@@ -355,7 +391,7 @@ HRESULT get_plugin_dispid(HTMLPluginContainer *plugin_container, WCHAR *name, DI
     }else if(plugin_container->props_len == plugin_container->props_size) {
         DISPID *new_props;
 
-        new_props = heap_realloc(plugin_container->props, plugin_container->props_size*2);
+        new_props = heap_realloc(plugin_container->props, plugin_container->props_size*2*sizeof(DISPID));
         if(!new_props)
             return E_OUTOFMEMORY;
 
@@ -1180,7 +1216,7 @@ static HRESULT WINAPI PHServiceProvider_QueryService(IServiceProvider *iface, RE
         return E_NOINTERFACE;
     }
 
-    return IServiceProvider_QueryService(&This->doc->basedoc.window->IServiceProvider_iface,
+    return IServiceProvider_QueryService(&This->doc->basedoc.window->base.IServiceProvider_iface,
             guidService, riid, ppv);
 }
 
@@ -1203,6 +1239,7 @@ static HRESULT assoc_element(PluginHost *host, HTMLDocumentNode *doc, nsIDOMElem
 
     hres = IHTMLDOMNode_QueryInterface(&node->IHTMLDOMNode_iface, &IID_HTMLPluginContainer,
             (void**)&container_elem);
+    node_release(node);
     if(FAILED(hres)) {
         ERR("Not an object element\n");
         return hres;
@@ -1246,6 +1283,7 @@ void detach_plugin_host(PluginHost *host)
     }
 
     list_remove(&host->entry);
+    list_init(&host->entry);
     host->doc = NULL;
 }
 
