@@ -48,6 +48,8 @@ static const WCHAR emptyW[] = {0};
 
 static const CLSID CLSID_JScript =
     {0xf414c260,0x6ac0,0x11cf,{0xb6,0xd1,0x00,0xaa,0x00,0xbb,0xbb,0x58}};
+static const CLSID CLSID_VBScript =
+    {0xb54f3741,0x5b07,0x11cf,{0xa4,0xb0,0x00,0xaa,0x00,0x4a,0x55,0xe8}};
 
 struct ScriptHost {
     IActiveScriptSite              IActiveScriptSite_iface;
@@ -64,7 +66,7 @@ struct ScriptHost {
 
     SCRIPTSTATE script_state;
 
-    HTMLWindow *window;
+    HTMLInnerWindow *window;
 
     GUID guid;
     struct list entry;
@@ -209,7 +211,7 @@ static void release_script_engine(ScriptHost *This)
     This->script_state = SCRIPTSTATE_UNINITIALIZED;
 }
 
-void connect_scripts(HTMLWindow *window)
+void connect_scripts(HTMLInnerWindow *window)
 {
     ScriptHost *iter;
 
@@ -318,7 +320,7 @@ static HRESULT WINAPI ActiveScriptSite_GetItemInfo(IActiveScriptSite *iface, LPC
         return E_FAIL;
 
     /* FIXME: Return proxy object */
-    *ppiunkItem = (IUnknown*)&This->window->IHTMLWindow2_iface;
+    *ppiunkItem = (IUnknown*)&This->window->base.IHTMLWindow2_iface;
     IUnknown_AddRef(*ppiunkItem);
 
     return S_OK;
@@ -589,7 +591,7 @@ static const IServiceProviderVtbl ASServiceProviderVtbl = {
     ASServiceProvider_QueryService
 };
 
-static ScriptHost *create_script_host(HTMLWindow *window, const GUID *guid)
+static ScriptHost *create_script_host(HTMLInnerWindow *window, const GUID *guid)
 {
     ScriptHost *ret;
     HRESULT hres;
@@ -657,7 +659,7 @@ static void parse_extern_script(ScriptHost *script_host, LPCWSTR src)
     if(FAILED(hres))
         return;
 
-    hres = bind_mon_to_buffer(script_host->window->doc, mon, (void**)&buf, &size);
+    hres = bind_mon_to_buffer(script_host->window, mon, (void**)&buf, &size);
     IMoniker_Release(mon);
     if(FAILED(hres))
         return;
@@ -718,10 +720,14 @@ static BOOL get_guid_from_type(LPCWSTR type, GUID *guid)
 {
     const WCHAR text_javascriptW[] =
         {'t','e','x','t','/','j','a','v','a','s','c','r','i','p','t',0};
+    const WCHAR text_vbscriptW[] =
+        {'t','e','x','t','/','v','b','s','c','r','i','p','t',0};
 
     /* FIXME: Handle more types */
     if(!strcmpiW(type, text_javascriptW)) {
         *guid = CLSID_JScript;
+    }else if(!strcmpiW(type, text_vbscriptW)) {
+        *guid = CLSID_VBScript;
     }else {
         FIXME("Unknown type %s\n", debugstr_w(type));
         return FALSE;
@@ -790,7 +796,7 @@ static BOOL get_script_guid(nsIDOMHTMLScriptElement *nsscript, GUID *guid)
     return ret;
 }
 
-static ScriptHost *get_script_host(HTMLWindow *window, const GUID *guid)
+static ScriptHost *get_script_host(HTMLInnerWindow *window, const GUID *guid)
 {
     ScriptHost *iter;
 
@@ -802,7 +808,7 @@ static ScriptHost *get_script_host(HTMLWindow *window, const GUID *guid)
     return create_script_host(window, guid);
 }
 
-void doc_insert_script(HTMLWindow *window, nsIDOMHTMLScriptElement *nsscript)
+void doc_insert_script(HTMLInnerWindow *window, nsIDOMHTMLScriptElement *nsscript)
 {
     ScriptHost *script_host;
     GUID guid;
@@ -812,7 +818,8 @@ void doc_insert_script(HTMLWindow *window, nsIDOMHTMLScriptElement *nsscript)
         return;
     }
 
-    if(IsEqualGUID(&CLSID_JScript, &guid) && window->scriptmode != SCRIPTMODE_ACTIVESCRIPT) {
+    if(IsEqualGUID(&CLSID_JScript, &guid)
+       && (!window->base.outer_window || window->base.outer_window->scriptmode != SCRIPTMODE_ACTIVESCRIPT)) {
         TRACE("Ignoring JScript\n");
         return;
     }
@@ -825,7 +832,7 @@ void doc_insert_script(HTMLWindow *window, nsIDOMHTMLScriptElement *nsscript)
         parse_script_elem(script_host, nsscript);
 }
 
-IDispatch *script_parse_event(HTMLWindow *window, LPCWSTR text)
+IDispatch *script_parse_event(HTMLInnerWindow *window, LPCWSTR text)
 {
     ScriptHost *script_host;
     GUID guid = CLSID_JScript;
@@ -858,7 +865,8 @@ IDispatch *script_parse_event(HTMLWindow *window, LPCWSTR text)
         ptr = text;
     }
 
-    if(IsEqualGUID(&CLSID_JScript, &guid) && window->scriptmode != SCRIPTMODE_ACTIVESCRIPT) {
+    if(IsEqualGUID(&CLSID_JScript, &guid)
+       && (!window->base.outer_window || window->base.outer_window->scriptmode != SCRIPTMODE_ACTIVESCRIPT)) {
         TRACE("Ignoring JScript\n");
         return NULL;
     }
@@ -879,7 +887,7 @@ IDispatch *script_parse_event(HTMLWindow *window, LPCWSTR text)
     return disp;
 }
 
-HRESULT exec_script(HTMLWindow *window, const WCHAR *code, const WCHAR *lang, VARIANT *ret)
+HRESULT exec_script(HTMLInnerWindow *window, const WCHAR *code, const WCHAR *lang, VARIANT *ret)
 {
     ScriptHost *script_host;
     EXCEPINFO ei;
@@ -930,7 +938,7 @@ IDispatch *get_script_disp(ScriptHost *script_host)
     return disp;
 }
 
-BOOL find_global_prop(HTMLWindow *window, BSTR name, DWORD flags, ScriptHost **ret_host, DISPID *ret_id)
+BOOL find_global_prop(HTMLInnerWindow *window, BSTR name, DWORD flags, ScriptHost **ret_host, DISPID *ret_id)
 {
     IDispatchEx *dispex;
     IDispatch *disp;
@@ -981,7 +989,7 @@ static BOOL is_jscript_available(void)
     return available;
 }
 
-void set_script_mode(HTMLWindow *window, SCRIPTMODE mode)
+void set_script_mode(HTMLOuterWindow *window, SCRIPTMODE mode)
 {
     nsIWebBrowserSetup *setup;
     nsresult nsres;
@@ -1009,7 +1017,7 @@ void set_script_mode(HTMLWindow *window, SCRIPTMODE mode)
         ERR("JavaScript setup failed: %08x\n", nsres);
 }
 
-void release_script_hosts(HTMLWindow *window)
+void release_script_hosts(HTMLInnerWindow *window)
 {
     ScriptHost *iter;
 
