@@ -52,6 +52,16 @@ static BSTR a2bstr(const char *str)
     return ret;
 }
 
+#define test_var_bstr(a,b) _test_var_bstr(__LINE__,a,b)
+static void _test_var_bstr(unsigned line, const VARIANT *v, const char *expect)
+{
+    ok_(__FILE__,line)(V_VT(v) == VT_BSTR, "V_VT(v) = %d\n", V_VT(v));
+    if(expect)
+        ok_(__FILE__,line)(!strcmp_wa(V_BSTR(v), expect), "V_BSTR(v) = %s, expected %s\n", wine_dbgstr_w(V_BSTR(v)), expect);
+    else
+        ok_(__FILE__,line)(!V_BSTR(v), "V_BSTR(v) = %s, expected NULL\n", wine_dbgstr_w(V_BSTR(v)));
+}
+
 #define get_elem2_iface(u) _get_elem2_iface(__LINE__,u)
 static IHTMLElement2 *_get_elem2_iface(unsigned line, IUnknown *unk)
 {
@@ -232,6 +242,19 @@ static void _test_style_set_csstext(unsigned line, IHTMLStyle *style, const char
     SysFreeString(tmp);
 }
 
+#define test_style_remove_attribute(a,b,c) _test_style_remove_attribute(__LINE__,a,b,c)
+static void _test_style_remove_attribute(unsigned line, IHTMLStyle *style, const char *attr, VARIANT_BOOL exb)
+{
+    BSTR str = a2bstr(attr);
+    VARIANT_BOOL b = 100;
+    HRESULT hres;
+
+    hres = IHTMLStyle_removeAttribute(style, str, 1, &b);
+    SysFreeString(str);
+    ok_(__FILE__,line)(hres == S_OK, "removeAttribute failed: %08x\n", hres);
+    ok_(__FILE__,line)(b == exb, "removeAttribute returned %x, expected %x\n", b, exb);
+}
+
 static void test_set_csstext(IHTMLStyle *style)
 {
     VARIANT v;
@@ -305,6 +328,54 @@ static void test_style2(IHTMLStyle2 *style2)
     ok(hres == S_OK, "get_direction failed: %08x\n", hres);
     ok(!strcmp_wa(str, "ltr"), "str = %s\n", wine_dbgstr_w(str));
     SysFreeString(str);
+
+    /* bottom */
+    V_VT(&v) = VT_EMPTY;
+    hres = IHTMLStyle2_get_bottom(style2, &v);
+    ok(hres == S_OK, "get_bottom failed: %08x\n", hres);
+    test_var_bstr(&v, NULL);
+
+    V_VT(&v) = VT_I4;
+    V_I4(&v) = 4;
+    hres = IHTMLStyle2_put_bottom(style2, v);
+    ok(hres == S_OK, "put_bottom failed: %08x\n", hres);
+
+    V_VT(&v) = VT_EMPTY;
+    hres = IHTMLStyle2_get_bottom(style2, &v);
+    ok(hres == S_OK, "get_bottom failed: %08x\n", hres);
+    test_var_bstr(&v, "4px");
+
+    /* overflowX */
+    str = (void*)0xdeadbeef;
+    hres = IHTMLStyle2_get_overflowX(style2, &str);
+    ok(hres == S_OK, "get_overflowX failed: %08x\n", hres);
+    ok(!str, "overflowX = %s\n", wine_dbgstr_w(str));
+
+    str = a2bstr("hidden");
+    hres = IHTMLStyle2_put_overflowX(style2, str);
+    ok(hres == S_OK, "put_overflowX failed: %08x\n", hres);
+    SysFreeString(str);
+
+    str = NULL;
+    hres = IHTMLStyle2_get_overflowX(style2, &str);
+    ok(hres == S_OK, "get_overflowX failed: %08x\n", hres);
+    ok(!strcmp_wa(str, "hidden"), "overflowX = %s\n", wine_dbgstr_w(str));
+
+    /* overflowY */
+    str = (void*)0xdeadbeef;
+    hres = IHTMLStyle2_get_overflowY(style2, &str);
+    ok(hres == S_OK, "get_overflowY failed: %08x\n", hres);
+    ok(!str, "overflowY = %s\n", wine_dbgstr_w(str));
+
+    str = a2bstr("hidden");
+    hres = IHTMLStyle2_put_overflowY(style2, str);
+    ok(hres == S_OK, "put_overflowY failed: %08x\n", hres);
+    SysFreeString(str);
+
+    str = NULL;
+    hres = IHTMLStyle2_get_overflowY(style2, &str);
+    ok(hres == S_OK, "get_overflowY failed: %08x\n", hres);
+    ok(!strcmp_wa(str, "hidden"), "overflowX = %s\n", wine_dbgstr_w(str));
 }
 
 static void test_style3(IHTMLStyle3 *style3)
@@ -555,12 +626,35 @@ static void test_body_style(IHTMLStyle *style)
     ok(V_VT(&v) == VT_BSTR, "V_VT(fontSize) = %d\n", V_VT(&v));
     ok(!V_BSTR(&v), "V_BSTR(fontSize) = %s\n", wine_dbgstr_w(V_BSTR(&v)));
 
+    V_VT(&v) = VT_I4;
+    V_I4(&v) = 12;
+    hres = IHTMLStyle_put_fontSize(style, v);
+    ok(hres == S_OK, "put_fontSize failed: %08x\n", hres);
+
+    V_VT(&v) = VT_NULL;
+    hres = IHTMLStyle_get_fontSize(style, &v);
+    ok(hres == S_OK, "get_fontSize failed: %08x\n", hres);
+    ok(V_VT(&v) == VT_BSTR, "V_VT(fontSize) = %d\n", V_VT(&v));
+    ok(!strcmp_wa(V_BSTR(&v), "12px"), "V_BSTR(fontSize) = %s\n", wine_dbgstr_w(V_BSTR(&v)));
+
     V_VT(&v) = VT_NULL;
     hres = IHTMLStyle_get_color(style, &v);
     ok(hres == S_OK, "get_color failed: %08x\n", hres);
     ok(V_VT(&v) == VT_BSTR, "V_VT(color) = %d\n", V_VT(&v));
     ok(!V_BSTR(&v), "V_BSTR(color) = %s\n", wine_dbgstr_w(V_BSTR(&v)));
 
+    V_VT(&v) = VT_I4;
+    V_I4(&v) = 0xfdfd;
+    hres = IHTMLStyle_put_color(style, v);
+    ok(hres == S_OK, "put_color failed: %08x\n", hres);
+
+    V_VT(&v) = VT_NULL;
+    hres = IHTMLStyle_get_color(style, &v);
+    ok(hres == S_OK, "get_color failed: %08x\n", hres);
+    ok(V_VT(&v) == VT_BSTR, "V_VT(color) = %d\n", V_VT(&v));
+    todo_wine
+    ok(!strcmp_wa(V_BSTR(&v), "#00fdfd"), "V_BSTR(color) = %s\n", wine_dbgstr_w(V_BSTR(&v)));
+
     b = 0xfefe;
     hres = IHTMLStyle_get_textDecorationUnderline(style, &b);
     ok(hres == S_OK, "get_textDecorationUnderline failed: %08x\n", hres);
@@ -568,7 +662,6 @@ static void test_body_style(IHTMLStyle *style)
 
     hres = IHTMLStyle_put_textDecorationUnderline(style, VARIANT_TRUE);
     ok(hres == S_OK, "put_textDecorationUnderline failed: %08x\n", hres);
-    ok(b == VARIANT_FALSE, "textDecorationUnderline = %x\n", b);
 
     hres = IHTMLStyle_get_textDecorationUnderline(style, &b);
     ok(hres == S_OK, "get_textDecorationUnderline failed: %08x\n", hres);
@@ -584,7 +677,6 @@ static void test_body_style(IHTMLStyle *style)
 
     hres = IHTMLStyle_put_textDecorationLineThrough(style, VARIANT_TRUE);
     ok(hres == S_OK, "put_textDecorationLineThrough failed: %08x\n", hres);
-    ok(b == VARIANT_FALSE, "textDecorationLineThrough = %x\n", b);
 
     hres = IHTMLStyle_get_textDecorationLineThrough(style, &b);
     ok(hres == S_OK, "get_textDecorationLineThrough failed: %08x\n", hres);
@@ -600,7 +692,6 @@ static void test_body_style(IHTMLStyle *style)
 
     hres = IHTMLStyle_put_textDecorationNone(style, VARIANT_TRUE);
     ok(hres == S_OK, "put_textDecorationNone failed: %08x\n", hres);
-    ok(b == VARIANT_FALSE, "textDecorationNone = %x\n", b);
 
     hres = IHTMLStyle_get_textDecorationNone(style, &b);
     ok(hres == S_OK, "get_textDecorationNone failed: %08x\n", hres);
@@ -616,7 +707,6 @@ static void test_body_style(IHTMLStyle *style)
 
     hres = IHTMLStyle_put_textDecorationOverline(style, VARIANT_TRUE);
     ok(hres == S_OK, "put_textDecorationOverline failed: %08x\n", hres);
-    ok(b == VARIANT_FALSE, "textDecorationOverline = %x\n", b);
 
     hres = IHTMLStyle_get_textDecorationOverline(style, &b);
     ok(hres == S_OK, "get_textDecorationOverline failed: %08x\n", hres);
@@ -632,7 +722,6 @@ static void test_body_style(IHTMLStyle *style)
 
     hres = IHTMLStyle_put_textDecorationBlink(style, VARIANT_TRUE);
     ok(hres == S_OK, "put_textDecorationBlink failed: %08x\n", hres);
-    ok(b == VARIANT_FALSE, "textDecorationBlink = %x\n", b);
 
     hres = IHTMLStyle_get_textDecorationBlink(style, &b);
     ok(hres == S_OK, "get_textDecorationBlink failed: %08x\n", hres);
@@ -999,6 +1088,19 @@ static void test_body_style(IHTMLStyle *style)
     ok(!strcmp_wa(V_BSTR(&v), "middle"), "V_BSTR(v) = %s\n", wine_dbgstr_w(V_BSTR(&v)));
     VariantClear(&v);
 
+    V_VT(&v) = VT_I4;
+    V_I4(&v) = 100;
+    hres = IHTMLStyle_put_verticalAlign(style, v);
+    ok(hres == S_OK, "put_vertivalAlign failed: %08x\n", hres);
+    VariantClear(&v);
+
+    V_VT(&v) = VT_EMPTY;
+    hres = IHTMLStyle_get_verticalAlign(style, &v);
+    ok(hres == S_OK, "get_verticalAlign failed: %08x\n", hres);
+    ok(V_VT(&v) == VT_BSTR, "V_VT(v)=%d\n", V_VT(&v));
+    ok(!strcmp_wa(V_BSTR(&v), "100px"), "V_BSTR(v) = %s\n", wine_dbgstr_w(V_BSTR(&v)));
+    VariantClear(&v);
+
     str = (void*)0xdeadbeef;
     hres = IHTMLStyle_get_textAlign(style, &str);
     ok(hres == S_OK, "get_textAlign failed: %08x\n", hres);
@@ -1233,6 +1335,18 @@ static void test_body_style(IHTMLStyle *style)
     ok(!V_BSTR(&v), "str=%s\n", wine_dbgstr_w(V_BSTR(&v)));
     VariantClear(&v);
 
+    V_VT(&v) = VT_BSTR;
+    V_BSTR(&v) = a2bstr("red");
+    hres = IHTMLStyle_put_backgroundColor(style, v);
+    ok(hres == S_OK, "put_backgroundColor failed: %08x\n", hres);
+    VariantClear(&v);
+
+    hres = IHTMLStyle_get_backgroundColor(style, &v);
+    ok(hres == S_OK, "get_backgroundColor: %08x\n", hres);
+    ok(V_VT(&v) == VT_BSTR, "type failed: %d\n", V_VT(&v));
+    ok(!strcmp_wa(V_BSTR(&v), "red"), "str=%s\n", wine_dbgstr_w(V_BSTR(&v)));
+    VariantClear(&v);
+
     /* padding */
     hres = IHTMLStyle_get_padding(style, &str);
     ok(hres == S_OK, "get_padding failed: %08x\n", hres);
@@ -1809,6 +1923,13 @@ static void test_body_style(IHTMLStyle *style)
     ok(!strcmp_wa(str, "always"), "pageBreakBefore = %s\n", wine_dbgstr_w(str));
     SysFreeString(str);
 
+    test_style_remove_attribute(style, "pageBreakBefore", VARIANT_TRUE);
+    test_style_remove_attribute(style, "pageBreakBefore", VARIANT_FALSE);
+
+    hres = IHTMLStyle_get_pageBreakBefore(style, &str);
+    ok(hres == S_OK, "get_pageBreakBefore failed: %08x\n", hres);
+    ok(!str, "pageBreakBefore = %s\n", wine_dbgstr_w(str));
+
     hres = IHTMLStyle_QueryInterface(style, &IID_IHTMLStyle2, (void**)&style2);
     ok(hres == S_OK, "Could not get IHTMLStyle2 iface: %08x\n", hres);
     if(SUCCEEDED(hres)) {
@@ -1910,6 +2031,11 @@ static void test_style_filters(IHTMLElement *elem)
     test_style_filter(style, "alpha(opacity=100)");
     set_style_filter(style, "xxx(a,b,c) alpha(opacity=100)");
     set_style_filter(style, NULL);
+    set_style_filter(style, "alpha(opacity=100)");
+    test_style_remove_attribute(style, "filter", VARIANT_TRUE);
+    test_style_remove_attribute(style, "filter", VARIANT_FALSE);
+    test_style_filter(style, NULL);
+
 
     IHTMLCurrentStyle2_Release(current_style2);
     IHTMLStyle_Release(style);
@@ -2064,7 +2190,7 @@ static void test_current_style(IHTMLCurrentStyle *current_style)
     hres = IHTMLCurrentStyle_get_verticalAlign(current_style, &v);
     ok(hres == S_OK, "get_verticalAlign failed: %08x\n", hres);
     ok(V_VT(&v) == VT_BSTR, "V_VT(v) = %d\n", V_VT(&v));
-    ok(!strcmp_wa(V_BSTR(&v), "middle"), "get_verticalAlign returned %s\n", wine_dbgstr_w(V_BSTR(&v)));
+    ok(!strcmp_wa(V_BSTR(&v), "100px"), "get_verticalAlign returned %s\n", wine_dbgstr_w(V_BSTR(&v)));
     VariantClear(&v);
 
     hres = IHTMLCurrentStyle_get_marginRight(current_style, &v);
