@@ -69,7 +69,7 @@ UINT msi_schedule_action( MSIPACKAGE *package, UINT script, const WCHAR *action
     UINT count;
     WCHAR **newbuf = NULL;
 
-    if (script >= TOTAL_SCRIPTS)
+    if (script >= SCRIPT_MAX)
     {
         FIXME("Unknown script requested %u\n", script);
         return ERROR_FUNCTION_FAILED;
@@ -308,6 +308,7 @@ static UINT custom_get_process_return( HANDLE process )
     DWORD rc = 0;
 
     GetExitCodeProcess( process, &rc );
+    TRACE("exit code is %u\n", rc);
     if (rc != 0)
         return ERROR_FUNCTION_FAILED;
     return ERROR_SUCCESS;
@@ -733,7 +734,6 @@ static UINT HANDLE_CustomType23(MSIPACKAGE *package, LPCWSTR source,
     msi_custom_action_info *info;
     WCHAR package_path[MAX_PATH];
     DWORD size;
-    UINT r;
 
     size = MAX_PATH;
     msi_get_property(package->db, szSourceDir, package_path, &size);
@@ -743,10 +743,7 @@ static UINT HANDLE_CustomType23(MSIPACKAGE *package, LPCWSTR source,
     TRACE("Installing package %s concurrently\n", debugstr_w(package_path));
 
     info = do_msidbCAConcurrentInstall(package, type, package_path, target, action);
-
-    r = wait_thread_handle(info);
-    release_custom_action_data( info );
-    return r;
+    return wait_thread_handle(info);
 }
 
 static UINT HANDLE_CustomType1(MSIPACKAGE *package, LPCWSTR source,
@@ -754,7 +751,6 @@ static UINT HANDLE_CustomType1(MSIPACKAGE *package, LPCWSTR source,
 {
     msi_custom_action_info *info;
     MSIBINARY *binary;
-    UINT r;
 
     if (!(binary = get_temp_binary( package, source, TRUE )))
         return ERROR_FUNCTION_FAILED;
@@ -762,10 +758,7 @@ static UINT HANDLE_CustomType1(MSIPACKAGE *package, LPCWSTR source,
     TRACE("Calling function %s from %s\n", debugstr_w(target), debugstr_w(binary->tmpfile));
 
     info = do_msidbCustomActionTypeDll( package, type, binary->tmpfile, target, action );
-
-    r = wait_thread_handle( info );
-    release_custom_action_data( info );
-    return r;
+    return wait_thread_handle( info );
 }
 
 static HANDLE execute_command( const WCHAR *app, WCHAR *arg, const WCHAR *dir )
@@ -859,7 +852,6 @@ static UINT HANDLE_CustomType17(MSIPACKAGE *package, LPCWSTR source,
 {
     msi_custom_action_info *info;
     MSIFILE *file;
-    UINT r;
 
     TRACE("%s %s\n", debugstr_w(source), debugstr_w(target));
 
@@ -871,10 +863,7 @@ static UINT HANDLE_CustomType17(MSIPACKAGE *package, LPCWSTR source,
     }
 
     info = do_msidbCustomActionTypeDll( package, type, file->TargetPath, target, action );
-
-    r = wait_thread_handle( info );
-    release_custom_action_data( info );
-    return r;
+    return wait_thread_handle( info );
 }
 
 static UINT HANDLE_CustomType18(MSIPACKAGE *package, LPCWSTR source,
@@ -914,11 +903,11 @@ static UINT HANDLE_CustomType19(MSIPACKAGE *package, LPCWSTR source,
     if( row )
     {
         LPCWSTR error = MSI_RecordGetString( row, 1 );
-        if ((gUILevel & INSTALLUILEVEL_MASK) != INSTALLUILEVEL_NONE)
+        if ((package->ui_level & INSTALLUILEVEL_MASK) != INSTALLUILEVEL_NONE)
             MessageBoxW( NULL, error, NULL, MB_OK );
         msiobj_release( &row->hdr );
     }
-    else if ((gUILevel & INSTALLUILEVEL_MASK) != INSTALLUILEVEL_NONE)
+    else if ((package->ui_level & INSTALLUILEVEL_MASK) != INSTALLUILEVEL_NONE)
         MessageBoxW( NULL, deformated, NULL, MB_OK );
 
     msi_free( deformated );
@@ -946,13 +935,15 @@ static UINT HANDLE_CustomType50(MSIPACKAGE *package, LPCWSTR source,
 static UINT HANDLE_CustomType34(MSIPACKAGE *package, LPCWSTR source,
                                 LPCWSTR target, const INT type, LPCWSTR action)
 {
-    const WCHAR *workingdir;
+    const WCHAR *workingdir = NULL;
     HANDLE handle;
     WCHAR *cmd;
 
-    workingdir = msi_get_target_folder( package, source );
-    if (!workingdir) return ERROR_FUNCTION_FAILED;
-
+    if (source)
+    {
+        workingdir = msi_get_target_folder( package, source );
+        if (!workingdir) return ERROR_FUNCTION_FAILED;
+    }
     deformat_string( package, target, &cmd );
     if (!cmd) return ERROR_FUNCTION_FAILED;
 
@@ -968,7 +959,7 @@ static DWORD ACTION_CallScript( const GUID *guid )
 {
     msi_custom_action_info *info;
     MSIHANDLE hPackage;
-    UINT r;
+    UINT r = ERROR_FUNCTION_FAILED;
 
     info = find_action_by_guid( guid );
     if (!info)
@@ -990,13 +981,13 @@ static DWORD ACTION_CallScript( const GUID *guid )
         ERR("failed to create handle for %p\n", info->package );
 
     release_custom_action_data( info );
-    return S_OK;
+    return r;
 }
 
 static DWORD WINAPI ScriptThread( LPVOID arg )
 {
     LPGUID guid = arg;
-    DWORD rc = 0;
+    DWORD rc;
 
     TRACE("custom action (%x) started\n", GetCurrentThreadId() );
 
@@ -1045,16 +1036,12 @@ static msi_custom_action_info *do_msidbCustomActionTypeScript(
 static UINT HANDLE_CustomType37_38(MSIPACKAGE *package, LPCWSTR source,
                                LPCWSTR target, const INT type, LPCWSTR action)
 {
-    UINT r;
     msi_custom_action_info *info;
 
     TRACE("%s %s\n", debugstr_w(source), debugstr_w(target));
 
     info = do_msidbCustomActionTypeScript( package, type, target, NULL, action );
-
-    r = wait_thread_handle( info );
-    release_custom_action_data( info );
-    return r;
+    return wait_thread_handle( info );
 }
 
 static UINT HANDLE_CustomType5_6(MSIPACKAGE *package, LPCWSTR source,
@@ -1097,7 +1084,6 @@ static UINT HANDLE_CustomType5_6(MSIPACKAGE *package, LPCWSTR source,
 
     info = do_msidbCustomActionTypeScript( package, type, bufferw, target, action );
     r = wait_thread_handle( info );
-    release_custom_action_data( info );
 
 done:
     msi_free(bufferw);
@@ -1157,7 +1143,6 @@ static UINT HANDLE_CustomType21_22(MSIPACKAGE *package, LPCWSTR source,
     }
     info = do_msidbCustomActionTypeScript( package, type, bufferw, target, action );
     r = wait_thread_handle( info );
-    release_custom_action_data( info );
 
 done:
     msi_free(bufferw);
@@ -1170,7 +1155,6 @@ static UINT HANDLE_CustomType53_54(MSIPACKAGE *package, LPCWSTR source,
 {
     msi_custom_action_info *info;
     WCHAR *prop;
-    UINT r;
 
     TRACE("%s %s\n", debugstr_w(source), debugstr_w(target));
 
@@ -1179,9 +1163,61 @@ static UINT HANDLE_CustomType53_54(MSIPACKAGE *package, LPCWSTR source,
 
     info = do_msidbCustomActionTypeScript( package, type, prop, NULL, action );
     msi_free(prop);
-    r = wait_thread_handle( info );
-    release_custom_action_data( info );
-    return r;
+    return wait_thread_handle( info );
+}
+
+static BOOL action_type_matches_script( MSIPACKAGE *package, UINT type, UINT script )
+{
+    switch (script)
+    {
+    case SCRIPT_NONE:
+    case SCRIPT_INSTALL:
+        return !(type & msidbCustomActionTypeCommit) && !(type & msidbCustomActionTypeRollback);
+    case SCRIPT_COMMIT:
+        return (type & msidbCustomActionTypeCommit);
+    case SCRIPT_ROLLBACK:
+        return (type & msidbCustomActionTypeRollback);
+    default:
+        ERR("unhandled script %u\n", script);
+    }
+    return FALSE;
+}
+
+static UINT defer_custom_action( MSIPACKAGE *package, const WCHAR *action, UINT type )
+{
+    WCHAR *actiondata = msi_dup_property( package->db, action );
+    WCHAR *usersid = msi_dup_property( package->db, szUserSID );
+    WCHAR *prodcode = msi_dup_property( package->db, szProductCode );
+    WCHAR *deferred = msi_get_deferred_action( action, actiondata, usersid, prodcode );
+
+    if (!deferred)
+    {
+        msi_free( actiondata );
+        msi_free( usersid );
+        msi_free( prodcode );
+        return ERROR_OUTOFMEMORY;
+    }
+    if (type & msidbCustomActionTypeCommit)
+    {
+        TRACE("deferring commit action\n");
+        msi_schedule_action( package, SCRIPT_COMMIT, deferred );
+    }
+    else if (type & msidbCustomActionTypeRollback)
+    {
+        TRACE("deferring rollback action\n");
+        msi_schedule_action( package, SCRIPT_ROLLBACK, deferred );
+    }
+    else
+    {
+        TRACE("deferring install action\n");
+        msi_schedule_action( package, SCRIPT_INSTALL, deferred );
+    }
+
+    msi_free( actiondata );
+    msi_free( usersid );
+    msi_free( prodcode );
+    msi_free( deferred );
+    return ERROR_SUCCESS;
 }
 
 UINT ACTION_CustomAction(MSIPACKAGE *package, LPCWSTR action, UINT script, BOOL execute)
@@ -1212,7 +1248,6 @@ UINT ACTION_CustomAction(MSIPACKAGE *package, LPCWSTR action, UINT script, BOOL
     }
 
     type = MSI_RecordGetInteger(row,2);
-
     source = MSI_RecordGetString(row,3);
     target = MSI_RecordGetString(row,4);
 
@@ -1228,34 +1263,9 @@ UINT ACTION_CustomAction(MSIPACKAGE *package, LPCWSTR action, UINT script, BOOL
         if (type & msidbCustomActionTypeNoImpersonate)
             WARN("msidbCustomActionTypeNoImpersonate not handled\n");
 
-        if (!execute)
+        if (!execute || !action_type_matches_script( package, type, script ))
         {
-            LPWSTR actiondata = msi_dup_property(package->db, action);
-            LPWSTR usersid = msi_dup_property(package->db, szUserSID);
-            LPWSTR prodcode = msi_dup_property(package->db, szProductCode);
-            LPWSTR deferred = msi_get_deferred_action(action, actiondata, usersid, prodcode);
-
-            if (type & msidbCustomActionTypeCommit)
-            {
-                TRACE("Deferring commit action\n");
-                msi_schedule_action(package, COMMIT_SCRIPT, deferred);
-            }
-            else if (type & msidbCustomActionTypeRollback)
-            {
-                TRACE("Deferring rollback action\n");
-                msi_schedule_action(package, ROLLBACK_SCRIPT, deferred);
-            }
-            else
-            {
-                TRACE("Deferring action\n");
-                msi_schedule_action(package, INSTALL_SCRIPT, deferred);
-            }
-
-            rc = ERROR_SUCCESS;
-            msi_free(actiondata);
-            msi_free(usersid);
-            msi_free(prodcode);
-            msi_free(deferred);
+            rc = defer_custom_action( package, action, type );
             goto end;
         }
         else
@@ -1403,8 +1413,14 @@ void ACTION_FinishCustomActions(const MSIPACKAGE* package)
         msi_dialog_check_messages( wait_handles[i] );
         CloseHandle( wait_handles[i] );
     }
-
     msi_free( wait_handles );
+
+    EnterCriticalSection( &msi_custom_action_cs );
+    LIST_FOR_EACH_ENTRY_SAFE( info, cursor, &msi_pending_custom_actions, msi_custom_action_info, entry )
+    {
+        if (info->package == package) release_custom_action_data( info );
+    }
+    LeaveCriticalSection( &msi_custom_action_cs );
 }
 
 typedef struct _msi_custom_remote_impl {
