@@ -62,7 +62,7 @@ static HRESULT create_ActiveScriptSite(IUnknown *pUnkOuter, LPVOID *ppObj)
     if( pUnkOuter )
         return CLASS_E_NOAGGREGATION;
 
-    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(MsiActiveScriptSite));
+    object = msi_alloc_zero( sizeof(MsiActiveScriptSite) );
 
     object->lpVtbl.lpVtbl = &ASS_Vtbl;
     object->ref = 1;
@@ -88,10 +88,7 @@ DWORD call_script(MSIHANDLE hPackage, INT type, LPCWSTR script, LPCWSTR function
     DISPID dispid;
     CLSID clsid;
     VARIANT var;
-
-    /* Return success by default (if Windows Script not installed) - not native behavior. This
-     * should be here until we implement wine scripting. */
-    DWORD ret = ERROR_SUCCESS;
+    DWORD ret = ERROR_INSTALL_FAILURE;
 
     CoInitialize(NULL);
 
@@ -126,9 +123,6 @@ DWORD call_script(MSIHANDLE hPackage, INT type, LPCWSTR script, LPCWSTR function
         goto done;
     }
 
-    /* If we got this far, Windows Script is installed, so don't return success by default anymore */
-    ret = ERROR_INSTALL_FAILURE;
-
     /* Get the IActiveScriptParse engine interface */
     hr = IActiveScript_QueryInterface(pActiveScript, &IID_IActiveScriptParse, (void **)&pActiveScriptParse);
     if (FAILED(hr)) goto done;
@@ -138,14 +132,15 @@ DWORD call_script(MSIHANDLE hPackage, INT type, LPCWSTR script, LPCWSTR function
     if (FAILED(hr)) goto done;
 
     /* Initialize the script engine */
-    hr = IActiveScriptParse_InitNew(pActiveScriptParse);
+    hr = IActiveScriptParse64_InitNew(pActiveScriptParse);
     if (FAILED(hr)) goto done;
 
     /* Add the session object */
-    hr = IActiveScript_AddNamedItem(pActiveScript, szSession, SCRIPTITEM_ISVISIBLE);
+    hr = IActiveScript_AddNamedItem(pActiveScript, szSession, SCRIPTITEM_GLOBALMEMBERS);
+    if (FAILED(hr)) goto done;
 
     /* Pass the script to the engine */
-    hr = IActiveScriptParse_ParseScriptText(pActiveScriptParse, script, NULL, NULL, NULL, 0, 0, 0L, NULL, NULL);
+    hr = IActiveScriptParse64_ParseScriptText(pActiveScriptParse, script, NULL, NULL, NULL, 0, 0, 0L, NULL, NULL);
     if (FAILED(hr)) goto done;
 
     /* Start processing the script */
@@ -236,7 +231,7 @@ static ULONG WINAPI MsiActiveScriptSite_Release(IActiveScriptSite* iface)
     TRACE("(%p/%p)\n", iface, This);
 
     if (!ref)
-        HeapFree(GetProcessHeap(), 0, This);
+        msi_free(This);
 
     return ref;
 }
