@@ -61,6 +61,8 @@ typedef struct __cxx_exception_type
 void WINAPI _CxxThrowException(exception*,const cxx_exception_type*);
 
 /* vtables */
+extern const vtable_ptr MSVCP_type_info_vtable;
+extern const vtable_ptr MSVCP_exception_vtable;
 /* ??_7bad_alloc@std@@6B@ */
 extern const vtable_ptr MSVCP_bad_alloc_vtable;
 /* ??_7logic_error@std@@6B@ */
@@ -73,19 +75,39 @@ extern const vtable_ptr MSVCP_invalid_argument_vtable;
 /* ??_7runtime_error@std@@6B@ */
 extern const vtable_ptr MSVCP_runtime_error_vtable;
 
-/* exception class data */
-static type_info exception_type_info = {
-    NULL, /* set by set_exception_vtable */
-    NULL,
-    ".?AVexception@std@@"
-};
+static void MSVCP_type_info_dtor(type_info * _this)
+{
+    free(_this->name);
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_type_info_vector_dtor,8)
+void * __thiscall MSVCP_type_info_vector_dtor(type_info * _this, unsigned int flags)
+{
+    TRACE("(%p %x)\n", _this, flags);
+    if (flags & 2)
+    {
+        /* we have an array, with the number of elements stored before the first object */
+        int i, *ptr = (int *)_this - 1;
+
+        for (i = *ptr - 1; i >= 0; i--) MSVCP_type_info_dtor(_this + i);
+        MSVCRT_operator_delete(ptr);
+    }
+    else
+    {
+        MSVCP_type_info_dtor(_this);
+        if (flags & 1) MSVCRT_operator_delete(_this);
+    }
+    return _this;
+}
+
+DEFINE_RTTI_DATA0( type_info, 0, ".?AVtype_info@@" );
 
 DEFINE_THISCALL_WRAPPER(MSVCP_exception_ctor, 8)
 exception* __thiscall MSVCP_exception_ctor(exception *this, const char *name)
 {
     TRACE("(%p %s)\n", this, name);
 
-    this->vtable = exception_type_info.vtable;
+    this->vtable = &MSVCP_exception_vtable;
     if(name) {
         unsigned int name_len = strlen(name) + 1;
         this->name = malloc(name_len);
@@ -104,7 +126,7 @@ exception* __thiscall MSVCP_exception_copy_ctor(exception *this, const exception
     TRACE("(%p,%p)\n", this, rhs);
 
     if(!rhs->do_free) {
-        this->vtable = exception_type_info.vtable;
+        this->vtable = &MSVCP_exception_vtable;
         this->name = rhs->name;
         this->do_free = FALSE;
     } else
@@ -117,11 +139,33 @@ DEFINE_THISCALL_WRAPPER(MSVCP_exception_dtor,4)
 void __thiscall MSVCP_exception_dtor(exception *this)
 {
     TRACE("(%p)\n", this);
-    this->vtable = exception_type_info.vtable;
+    this->vtable = &MSVCP_exception_vtable;
     if(this->do_free)
         free(this->name);
 }
 
+DEFINE_THISCALL_WRAPPER(MSVCP_exception_vector_dtor, 8)
+void * __thiscall MSVCP_exception_vector_dtor(exception *this, unsigned int flags)
+{
+    TRACE("%p %x\n", this, flags);
+    if(flags & 2) {
+        /* we have an array, with the number of elements stored before the first object */
+        int i, *ptr = (int *)this-1;
+
+        for(i=*ptr-1; i>=0; i--)
+            MSVCP_exception_dtor(this+i);
+        MSVCRT_operator_delete(ptr);
+    } else {
+        MSVCP_exception_dtor(this);
+        if(flags & 1)
+            MSVCRT_operator_delete(this);
+    }
+
+    return this;
+}
+
+DEFINE_RTTI_DATA0(exception, 0, ".?AVexception@std@@");
+
 /* ?_Doraise@bad_alloc@std@@MBEXXZ */
 /* ?_Doraise@bad_alloc@std@@MEBAXXZ */
 /* ?_Doraise@logic_error@std@@MBEXXZ */
@@ -145,13 +189,6 @@ const char* __thiscall MSVCP_exception_what(exception * this)
     return this->name ? this->name : "Unknown exception";
 }
 
-static const rtti_base_descriptor exception_rtti_base_descriptor = {
-    &exception_type_info,
-    0,
-    { 0, -1, 0 },
-    0
-};
-
 static const cxx_type_info exception_cxx_type_info = {
     0,
     &exception_type_info,
@@ -176,12 +213,6 @@ static const cxx_exception_type exception_cxx_type = {
     &exception_cxx_type_table
 };
 
-void set_exception_vtable(void)
-{
-    HMODULE hmod = GetModuleHandleA("msvcrt.dll");
-    exception_type_info.vtable = (void*)GetProcAddress(hmod, "??_7exception@@6B@");
-}
-
 /* bad_alloc class data */
 typedef exception bad_alloc;
 
@@ -253,7 +284,7 @@ bad_alloc* __thiscall MSVCP_bad_alloc_assign(bad_alloc *this, const bad_alloc *a
     return MSVCP_bad_alloc_copy_ctor(this, assign);
 }
 
-DEFINE_RTTI_DATA(bad_alloc, 0, 1, &exception_rtti_base_descriptor, NULL, NULL, ".?AVbad_alloc@std@@");
+DEFINE_RTTI_DATA1(bad_alloc, 0, &exception_rtti_base_descriptor, ".?AVbad_alloc@std@@");
 
 static const cxx_type_info bad_alloc_cxx_type_info = {
     0,
@@ -374,7 +405,7 @@ const char* __thiscall MSVCP_logic_error_what(logic_error *this)
     return MSVCP_basic_string_char_c_str(&this->str);
 }
 
-DEFINE_RTTI_DATA(logic_error, 0, 1, &exception_rtti_base_descriptor, NULL, NULL, ".?AVlogic_error@std@@");
+DEFINE_RTTI_DATA1(logic_error, 0, &exception_rtti_base_descriptor, ".?AVlogic_error@std@@");
 
 static const cxx_type_info logic_error_cxx_type_info = {
     0,
@@ -451,7 +482,7 @@ length_error* __thiscall MSVCP_length_error_assign(length_error *this, const len
     return MSVCP_length_error_copy_ctor(this, assign);
 }
 
-DEFINE_RTTI_DATA(length_error, 0, 2, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, NULL, ".?AVlength_error@std@@");
+DEFINE_RTTI_DATA2(length_error, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVlength_error@std@@");
 
 static const cxx_type_info length_error_cxx_type_info = {
     0,
@@ -528,7 +559,7 @@ out_of_range* __thiscall MSVCP_out_of_range_assign(out_of_range *this, const out
     return MSVCP_out_of_range_copy_ctor(this, assign);
 }
 
-DEFINE_RTTI_DATA(out_of_range, 0, 2, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, NULL, ".?AVout_of_range@std@@");
+DEFINE_RTTI_DATA2(out_of_range, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVout_of_range@std@@");
 
 static const cxx_type_info out_of_range_cxx_type_info = {
     0,
@@ -585,7 +616,7 @@ void* __thiscall MSVCP_invalid_argument_vector_dtor(
     return MSVCP_logic_error_vector_dtor(this, flags);
 }
 
-DEFINE_RTTI_DATA(invalid_argument, 0, 2, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, NULL, ".?AVinvalid_argument@std@@");
+DEFINE_RTTI_DATA2(invalid_argument, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVinvalid_argument@std@@");
 
 static const cxx_type_info invalid_argument_cxx_type_info = {
     0,
@@ -662,7 +693,7 @@ runtime_error* __thiscall MSVCP_runtime_error_assign(runtime_error *this, const
     return MSVCP_runtime_error_copy_ctor(this, assign);
 }
 
-DEFINE_RTTI_DATA(runtime_error, 0, 1, &exception_rtti_base_descriptor, NULL, NULL, ".?AVruntime_error@std@@");
+DEFINE_RTTI_DATA1(runtime_error, 0, &exception_rtti_base_descriptor, ".?AVruntime_error@std@@");
 
 static const cxx_type_info runtime_error_cxx_type_info = {
     0,
@@ -700,6 +731,10 @@ const char* __thiscall MSVCP_runtime_error_what(runtime_error *this)
 #ifndef __GNUC__
 void __asm_dummy_vtables(void) {
 #endif
+    __ASM_VTABLE(type_info,"");
+    __ASM_VTABLE(exception,
+            VTABLE_ADD_FUNC(MSVCP_exception_what)
+            VTABLE_ADD_FUNC(MSVCP_exception__Doraise));
     __ASM_VTABLE(bad_alloc,
             VTABLE_ADD_FUNC(MSVCP_exception_what)
             VTABLE_ADD_FUNC(MSVCP_exception__Doraise));
@@ -761,5 +796,7 @@ void throw_exception(exception_type et, const char *str)
         MSVCP_runtime_error_ctor(&e, str);
         _CxxThrowException((exception*)&e, &runtime_error_cxx_type);
     }
+    default:
+        ERR("exception type not handled: %d\n", et);
     }
 }
