@@ -58,64 +58,52 @@ typedef struct __cxx_exception_type
     const cxx_type_info_table *type_info_table;  /* list of types for this exception object */
 } cxx_exception_type;
 
-void CDECL _CxxThrowException(exception*,const cxx_exception_type*);
+void WINAPI _CxxThrowException(exception*,const cxx_exception_type*);
 
 /* vtables */
-
-#ifdef _WIN64
-
-#define __ASM_VTABLE(name,funcs) \
-    __asm__(".data\n" \
-            "\t.align 8\n" \
-            "\t.quad " __ASM_NAME(#name "_rtti") "\n" \
-            "\t.globl " __ASM_NAME("MSVCP_" #name "_vtable") "\n" \
-            __ASM_NAME("MSVCP_" #name "_vtable") ":\n" \
-            "\t.quad " THISCALL_NAME(MSVCP_ ## name ## _vector_dtor) "\n" \
-            funcs "\n\t.text");
-
-#define __ASM_EXCEPTION_VTABLE(name) \
-    __ASM_VTABLE(name, "\t.quad " THISCALL_NAME(MSVCP_what_exception) )
-
-#define __ASM_EXCEPTION_STRING_VTABLE(name) \
-    __ASM_VTABLE(name, "\t.quad " THISCALL_NAME(MSVCP_logic_error_what) )
-
-#else
-
-#define __ASM_VTABLE(name,funcs) \
-    __asm__(".data\n" \
-            "\t.align 4\n" \
-            "\t.long " __ASM_NAME(#name "_rtti") "\n" \
-            "\t.globl " __ASM_NAME("MSVCP_" #name "_vtable") "\n" \
-            __ASM_NAME("MSVCP_" #name "_vtable") ":\n" \
-            "\t.long " THISCALL_NAME(MSVCP_ ## name ## _vector_dtor) "\n" \
-            funcs "\n\t.text");
-
-#define __ASM_EXCEPTION_VTABLE(name) \
-    __ASM_VTABLE(name, "\t.long " THISCALL_NAME(MSVCP_what_exception) )
-
-#define __ASM_EXCEPTION_STRING_VTABLE(name) \
-    __ASM_VTABLE(name, "\t.long " THISCALL_NAME(MSVCP_logic_error_what) )
-
-#endif /* _WIN64 */
-
+extern const vtable_ptr MSVCP_exception_vtable;
 extern const vtable_ptr MSVCP_bad_alloc_vtable;
 extern const vtable_ptr MSVCP_logic_error_vtable;
 extern const vtable_ptr MSVCP_length_error_vtable;
 extern const vtable_ptr MSVCP_out_of_range_vtable;
+extern const vtable_ptr MSVCP_invalid_argument_vtable;
+extern const vtable_ptr MSVCP_runtime_error_vtable;
+extern const vtable_ptr MSVCP_failure_vtable;
 
-/* exception class data */
-static type_info exception_type_info = {
-    NULL, /* set by set_exception_vtable */
-    NULL,
-    ".?AVexception@std@@"
-};
+static void MSVCP_type_info_dtor(type_info * _this)
+{
+    free(_this->name);
+}
+
+/* Unexported */
+DEFINE_THISCALL_WRAPPER(MSVCP_type_info_vector_dtor,8)
+void * __thiscall MSVCP_type_info_vector_dtor(type_info * _this, unsigned int flags)
+{
+    TRACE("(%p %x)\n", _this, flags);
+    if (flags & 2)
+    {
+        /* we have an array, with the number of elements stored before the first object */
+        int i, *ptr = (int *)_this - 1;
+
+        for (i = *ptr - 1; i >= 0; i--) MSVCP_type_info_dtor(_this + i);
+        MSVCRT_operator_delete(ptr);
+    }
+    else
+    {
+        MSVCP_type_info_dtor(_this);
+        if (flags & 1) MSVCRT_operator_delete(_this);
+    }
+    return _this;
+}
+
+DEFINE_RTTI_DATA0( type_info, 0, ".?AVtype_info@@" );
 
 DEFINE_THISCALL_WRAPPER(MSVCP_exception_ctor, 8)
-exception* __stdcall MSVCP_exception_ctor(exception *this, const char **name)
+exception* __thiscall MSVCP_exception_ctor(exception *this, const char **name)
 {
     TRACE("(%p %s)\n", this, *name);
 
-    this->vtable = exception_type_info.vtable;
+    this->vtable = &MSVCP_exception_vtable;
     if(*name) {
         unsigned int name_len = strlen(*name) + 1;
         this->name = malloc(name_len);
@@ -129,12 +117,12 @@ exception* __stdcall MSVCP_exception_ctor(exception *this, const char **name)
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_exception_copy_ctor,8)
-exception* __stdcall MSVCP_exception_copy_ctor(exception *this, const exception *rhs)
+exception* __thiscall MSVCP_exception_copy_ctor(exception *this, const exception *rhs)
 {
     TRACE("(%p,%p)\n", this, rhs);
 
     if(!rhs->do_free) {
-        this->vtable = exception_type_info.vtable;
+        this->vtable = &MSVCP_exception_vtable;
         this->name = rhs->name;
         this->do_free = FALSE;
     } else
@@ -144,20 +132,35 @@ exception* __stdcall MSVCP_exception_copy_ctor(exception *this, const exception
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_exception_dtor,4)
-void __stdcall MSVCP_exception_dtor(exception *this)
+void __thiscall MSVCP_exception_dtor(exception *this)
 {
     TRACE("(%p)\n", this);
-    this->vtable = exception_type_info.vtable;
+    this->vtable = &MSVCP_exception_vtable;
     if(this->do_free)
         free(this->name);
 }
 
-static const rtti_base_descriptor exception_rtti_base_descriptor = {
-    &exception_type_info,
-    0,
-    { 0, -1, 0 },
-    0
-};
+DEFINE_THISCALL_WRAPPER(MSVCP_exception_vector_dtor, 8)
+void * __thiscall MSVCP_exception_vector_dtor(exception *this, unsigned int flags)
+{
+    TRACE("%p %x\n", this, flags);
+    if(flags & 2) {
+        /* we have an array, with the number of elements stored before the first object */
+        int i, *ptr = (int *)this-1;
+
+        for(i=*ptr-1; i>=0; i--)
+            MSVCP_exception_dtor(this+i);
+        MSVCRT_operator_delete(ptr);
+    } else {
+        MSVCP_exception_dtor(this);
+        if(flags & 1)
+            MSVCRT_operator_delete(this);
+    }
+
+    return this;
+}
+
+DEFINE_RTTI_DATA0(exception, 0, ".?AVexception@std@@");
 
 static const cxx_type_info exception_cxx_type_info = {
     0,
@@ -183,17 +186,11 @@ static const cxx_exception_type exception_cxx_type = {
     &exception_cxx_type_table
 };
 
-void set_exception_vtable(void)
-{
-    HMODULE hmod = GetModuleHandleA("msvcrt.dll");
-    exception_type_info.vtable = (void*)GetProcAddress(hmod, "??_7exception@@6B@");
-}
-
 /* bad_alloc class data */
 typedef exception bad_alloc;
 
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_ctor, 8)
-bad_alloc* __stdcall MSVCP_bad_alloc_ctor(bad_alloc *this, const char **name)
+bad_alloc* __thiscall MSVCP_bad_alloc_ctor(bad_alloc *this, const char **name)
 {
     TRACE("%p %s\n", this, *name);
     MSVCP_exception_ctor(this, name);
@@ -202,7 +199,7 @@ bad_alloc* __stdcall MSVCP_bad_alloc_ctor(bad_alloc *this, const char **name)
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_copy_ctor, 8)
-bad_alloc* __stdcall MSVCP_bad_alloc_copy_ctor(bad_alloc *this, const bad_alloc *rhs)
+bad_alloc* __thiscall MSVCP_bad_alloc_copy_ctor(bad_alloc *this, const bad_alloc *rhs)
 {
     TRACE("%p %p\n", this, rhs);
     MSVCP_exception_copy_ctor(this, rhs);
@@ -211,14 +208,14 @@ bad_alloc* __stdcall MSVCP_bad_alloc_copy_ctor(bad_alloc *this, const bad_alloc
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_dtor, 4)
-void __stdcall MSVCP_bad_alloc_dtor(bad_alloc *this)
+void __thiscall MSVCP_bad_alloc_dtor(bad_alloc *this)
 {
     TRACE("%p\n", this);
     MSVCP_exception_dtor(this);
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_bad_alloc_vector_dtor, 8)
-void * __stdcall MSVCP_bad_alloc_vector_dtor(bad_alloc *this, unsigned int flags)
+void * __thiscall MSVCP_bad_alloc_vector_dtor(bad_alloc *this, unsigned int flags)
 {
     TRACE("%p %x\n", this, flags);
     if(flags & 2) {
@@ -238,47 +235,13 @@ void * __stdcall MSVCP_bad_alloc_vector_dtor(bad_alloc *this, unsigned int flags
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_what_exception,4)
-const char* __stdcall MSVCP_what_exception(exception * this)
+const char* __thiscall MSVCP_what_exception(exception * this)
 {
     TRACE("(%p) returning %s\n", this, this->name);
     return this->name ? this->name : "Unknown exception";
 }
 
-static const type_info bad_alloc_type_info = {
-    &MSVCP_bad_alloc_vtable,
-    NULL,
-    ".?AVbad_alloc@std@@"
-};
-
-static const rtti_base_descriptor bad_alloc_rtti_base_descriptor = {
-    &bad_alloc_type_info,
-    1,
-    { 0, -1, 0 },
-    64
-};
-
-static const rtti_base_array bad_alloc_rtti_base_array = {
-    {
-        &bad_alloc_rtti_base_descriptor,
-        &exception_rtti_base_descriptor,
-        NULL
-    }
-};
-
-static const rtti_object_hierarchy bad_alloc_type_hierarchy = {
-    0,
-    0,
-    2,
-    &bad_alloc_rtti_base_array
-};
-
-const rtti_object_locator bad_alloc_rtti = {
-    0,
-    0,
-    0,
-    &bad_alloc_type_info,
-    &bad_alloc_type_hierarchy
-};
+DEFINE_RTTI_DATA1(bad_alloc, 0, &exception_rtti_base_descriptor, ".?AVbad_alloc@std@@");
 
 static const cxx_type_info bad_alloc_cxx_type_info = {
     0,
@@ -311,7 +274,7 @@ typedef struct _logic_error {
 } logic_error;
 
 DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_ctor, 8)
-logic_error* __stdcall MSVCP_logic_error_ctor(
+logic_error* __thiscall MSVCP_logic_error_ctor(
         logic_error *this, const char **name)
 {
     TRACE("%p %s\n", this, *name);
@@ -323,7 +286,7 @@ logic_error* __stdcall MSVCP_logic_error_ctor(
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_copy_ctor, 8)
-logic_error* __stdcall MSVCP_logic_error_copy_ctor(
+logic_error* __thiscall MSVCP_logic_error_copy_ctor(
         logic_error *this, logic_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
@@ -334,7 +297,7 @@ logic_error* __stdcall MSVCP_logic_error_copy_ctor(
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_dtor, 4)
-void __stdcall MSVCP_logic_error_dtor(logic_error *this)
+void __thiscall MSVCP_logic_error_dtor(logic_error *this)
 {
     TRACE("%p\n", this);
     MSVCP_exception_dtor(&this->e);
@@ -342,7 +305,7 @@ void __stdcall MSVCP_logic_error_dtor(logic_error *this)
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_vector_dtor, 8)
-void* __stdcall MSVCP_logic_error_vector_dtor(
+void* __thiscall MSVCP_logic_error_vector_dtor(
         logic_error *this, unsigned int flags)
 {
     TRACE("%p %x\n", this, flags);
@@ -363,47 +326,13 @@ void* __stdcall MSVCP_logic_error_vector_dtor(
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_logic_error_what, 4)
-const char* __stdcall MSVCP_logic_error_what(logic_error *this)
+const char* __thiscall MSVCP_logic_error_what(logic_error *this)
 {
     TRACE("%p\n", this);
     return MSVCP_basic_string_char_c_str(&this->str);
 }
 
-static const type_info logic_error_type_info = {
-    &MSVCP_logic_error_vtable,
-    NULL,
-    ".?AVlogic_error@std@@"
-};
-
-static const rtti_base_descriptor logic_error_rtti_base_descriptor = {
-    &logic_error_type_info,
-    1,
-    { 0, -1, 0 },
-    64
-};
-
-static const rtti_base_array logic_error_rtti_base_array = {
-    {
-        &logic_error_rtti_base_descriptor,
-        &exception_rtti_base_descriptor,
-        NULL
-    }
-};
-
-static const rtti_object_hierarchy logic_error_type_hierarchy = {
-    0,
-    0,
-    2,
-    &logic_error_rtti_base_array
-};
-
-const rtti_object_locator logic_error_rtti = {
-    0,
-    0,
-    0,
-    &logic_error_type_info,
-    &logic_error_type_hierarchy
-};
+DEFINE_RTTI_DATA1(logic_error, 0, &exception_rtti_base_descriptor, ".?AVlogic_error@std@@");
 
 static const cxx_type_info logic_error_cxx_type_info = {
     0,
@@ -433,7 +362,7 @@ static const cxx_exception_type logic_error_cxx_type = {
 typedef logic_error length_error;
 
 DEFINE_THISCALL_WRAPPER(MSVCP_length_error_ctor, 8)
-length_error* __stdcall MSVCP_length_error_ctor(
+length_error* __thiscall MSVCP_length_error_ctor(
         length_error *this, const char **name)
 {
     TRACE("%p %s\n", this, *name);
@@ -443,7 +372,7 @@ length_error* __stdcall MSVCP_length_error_ctor(
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_length_error_copy_ctor, 8)
-length_error* __stdcall MSVCP_length_error_copy_ctor(
+length_error* __thiscall MSVCP_length_error_copy_ctor(
         length_error *this, length_error *rhs)
 {
     TRACE("%p %p\n", this, rhs);
@@ -453,48 +382,14 @@ length_error* __stdcall MSVCP_length_error_copy_ctor(
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_length_error_vector_dtor, 8)
-void* __stdcall MSVCP_length_error_vector_dtor(
+void* __thiscall MSVCP_length_error_vector_dtor(
         length_error *this, unsigned int flags)
 {
     TRACE("%p %x\n", this, flags);
     return MSVCP_logic_error_vector_dtor(this, flags);
 }
 
-static const type_info length_error_type_info = {
-    &MSVCP_length_error_vtable,
-    NULL,
-    ".?AVlength_error@std@@"
-};
-
-static const rtti_base_descriptor length_error_rtti_base_descriptor = {
-    &length_error_type_info,
-    2,
-    { 0, -1, 0 },
-    64
-};
-
-static const rtti_base_array length_error_rtti_base_array = {
-    {
-        &length_error_rtti_base_descriptor,
-        &logic_error_rtti_base_descriptor,
-        &exception_rtti_base_descriptor
-    }
-};
-
-static const rtti_object_hierarchy length_error_type_hierarchy = {
-    0,
-    0,
-    3,
-    &length_error_rtti_base_array
-};
-
-const rtti_object_locator length_error_rtti = {
-    0,
-    0,
-    0,
-    &length_error_type_info,
-    &length_error_type_hierarchy
-};
+DEFINE_RTTI_DATA2(length_error, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVlength_error@std@@");
 
 static const cxx_type_info length_error_cxx_type_info = {
     0,
@@ -524,7 +419,7 @@ static const cxx_exception_type length_error_cxx_type = {
 typedef logic_error out_of_range;
 
 DEFINE_THISCALL_WRAPPER(MSVCP_out_of_range_ctor, 8)
-out_of_range* __stdcall MSVCP_out_of_range_ctor(
+out_of_range* __thiscall MSVCP_out_of_range_ctor(
         out_of_range *this, const char **name)
 {
     TRACE("%p %s\n", this, *name);
@@ -534,7 +429,7 @@ out_of_range* __stdcall MSVCP_out_of_range_ctor(
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_out_of_range_copy_ctor, 8)
-out_of_range* __stdcall MSVCP_out_of_range_copy_ctor(
+out_of_range* __thiscall MSVCP_out_of_range_copy_ctor(
         out_of_range *this, out_of_range *rhs)
 {
     TRACE("%p %p\n", this, rhs);
@@ -544,48 +439,14 @@ out_of_range* __stdcall MSVCP_out_of_range_copy_ctor(
 }
 
 DEFINE_THISCALL_WRAPPER(MSVCP_out_of_range_vector_dtor, 8)
-void* __stdcall MSVCP_out_of_range_vector_dtor(
+void* __thiscall MSVCP_out_of_range_vector_dtor(
         out_of_range *this, unsigned int flags)
 {
     TRACE("%p %x\n", this, flags);
     return MSVCP_logic_error_vector_dtor(this, flags);
 }
 
-static const type_info out_of_range_type_info = {
-    &MSVCP_out_of_range_vtable,
-    NULL,
-    ".?AVout_of_range@std@@"
-};
-
-static const rtti_base_descriptor out_of_range_rtti_base_descriptor = {
-    &out_of_range_type_info,
-    2,
-    { 0, -1, 0 },
-    64
-};
-
-static const rtti_base_array out_of_range_rtti_base_array = {
-    {
-        &out_of_range_rtti_base_descriptor,
-        &logic_error_rtti_base_descriptor,
-        &exception_rtti_base_descriptor
-    }
-};
-
-static const rtti_object_hierarchy out_of_range_type_hierarchy = {
-    0,
-    0,
-    3,
-    &out_of_range_rtti_base_array
-};
-
-const rtti_object_locator out_of_range_rtti = {
-    0,
-    0,
-    0,
-    &out_of_range_type_info,
-    &out_of_range_type_hierarchy
-};
+DEFINE_RTTI_DATA2(out_of_range, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVout_of_range@std@@");
 
 static const cxx_type_info out_of_range_cxx_type_info = {
     0,
@@ -611,13 +472,237 @@ static const cxx_exception_type out_of_range_cxx_type = {
     &out_of_range_cxx_type_table
 };
 
+/* invalid_argument class data */
+typedef logic_error invalid_argument;
+
+DEFINE_THISCALL_WRAPPER(MSVCP_invalid_argument_ctor, 8)
+invalid_argument* __thiscall MSVCP_invalid_argument_ctor(
+        invalid_argument *this, const char **name)
+{
+    TRACE("%p %s\n", this, *name);
+    MSVCP_logic_error_ctor(this, name);
+    this->e.vtable = &MSVCP_invalid_argument_vtable;
+    return this;
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_invalid_argument_copy_ctor, 8)
+invalid_argument* __thiscall MSVCP_invalid_argument_copy_ctor(
+        invalid_argument *this, invalid_argument *rhs)
+{
+    TRACE("%p %p\n", this, rhs);
+    MSVCP_logic_error_copy_ctor(this, rhs);
+    this->e.vtable = &MSVCP_invalid_argument_vtable;
+    return this;
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_invalid_argument_vector_dtor, 8)
+void* __thiscall MSVCP_invalid_argument_vector_dtor(
+        invalid_argument *this, unsigned int flags)
+{
+    TRACE("%p %x\n", this, flags);
+    return MSVCP_logic_error_vector_dtor(this, flags);
+}
+
+DEFINE_RTTI_DATA2(invalid_argument, 0, &logic_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVinvalid_argument@std@@");
+
+static const cxx_type_info invalid_argument_cxx_type_info = {
+    0,
+    &invalid_argument_type_info,
+    { 0, -1, 0 },
+    sizeof(invalid_argument),
+    (cxx_copy_ctor)THISCALL(MSVCP_invalid_argument_copy_ctor)
+};
+
+static const cxx_type_info_table invalid_argument_cxx_type_table = {
+    3,
+    {
+        &invalid_argument_cxx_type_info,
+        &logic_error_cxx_type_info,
+        &exception_cxx_type_info
+    }
+};
+
+static const cxx_exception_type invalid_argument_cxx_type = {
+    0,
+    (cxx_copy_ctor)THISCALL(MSVCP_logic_error_dtor),
+    NULL,
+    &invalid_argument_cxx_type_table
+};
+
+/* runtime_error class data */
+typedef struct {
+    exception e;
+    basic_string_char str;
+} runtime_error;
+
+DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_ctor, 8)
+runtime_error* __thiscall MSVCP_runtime_error_ctor(
+        runtime_error *this, const char **name)
+{
+    TRACE("%p %s\n", this, *name);
+    this->e.vtable = &MSVCP_runtime_error_vtable;
+    this->e.name = NULL;
+    this->e.do_free = FALSE;
+    MSVCP_basic_string_char_ctor_cstr(&this->str, *name);
+    return this;
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_copy_ctor, 8)
+runtime_error* __thiscall MSVCP_runtime_error_copy_ctor(
+        runtime_error *this, runtime_error *rhs)
+{
+    TRACE("%p %p\n", this, rhs);
+    MSVCP_exception_copy_ctor(&this->e, &rhs->e);
+    MSVCP_basic_string_char_copy_ctor(&this->str, &rhs->str);
+    this->e.vtable = &MSVCP_runtime_error_vtable;
+    return this;
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_dtor, 4)
+void __thiscall MSVCP_runtime_error_dtor(runtime_error *this)
+{
+    TRACE("%p\n", this);
+    MSVCP_exception_dtor(&this->e);
+    MSVCP_basic_string_char_dtor(&this->str);
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_vector_dtor, 8)
+void* __thiscall MSVCP_runtime_error_vector_dtor(
+        runtime_error *this, unsigned int flags)
+{
+    TRACE("%p %x\n", this, flags);
+    if(flags & 2) {
+        /* we have an array, with the number of elements stored before the first object */
+        int i, *ptr = (int *)this-1;
+
+        for(i=*ptr-1; i>=0; i--)
+            MSVCP_runtime_error_dtor(this+i);
+        MSVCRT_operator_delete(ptr);
+    } else {
+        MSVCP_runtime_error_dtor(this);
+        if(flags & 1)
+            MSVCRT_operator_delete(this);
+    }
+
+    return this;
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_runtime_error_what, 4)
+const char* __thiscall MSVCP_runtime_error_what(runtime_error *this)
+{
+    TRACE("%p\n", this);
+    return MSVCP_basic_string_char_c_str(&this->str);
+}
+
+DEFINE_RTTI_DATA1(runtime_error, 0, &exception_rtti_base_descriptor, ".?AVruntime_error@std@@");
+
+static const cxx_type_info runtime_error_cxx_type_info = {
+    0,
+    &runtime_error_type_info,
+    { 0, -1, 0 },
+    sizeof(runtime_error),
+    (cxx_copy_ctor)THISCALL(MSVCP_runtime_error_copy_ctor)
+};
+
+static const cxx_type_info_table runtime_error_cxx_type_table = {
+    2,
+    {
+        &runtime_error_cxx_type_info,
+        &exception_cxx_type_info,
+        NULL
+    }
+};
+
+static const cxx_exception_type runtime_error_cxx_type = {
+    0,
+    (cxx_copy_ctor)THISCALL(MSVCP_runtime_error_dtor),
+    NULL,
+    &runtime_error_cxx_type_table
+};
+
+/* failure class data */
+typedef runtime_error failure;
+
+DEFINE_THISCALL_WRAPPER(MSVCP_failure_ctor, 8)
+failure* __thiscall MSVCP_failure_ctor(
+        failure *this, const char **name)
+{
+    TRACE("%p %s\n", this, *name);
+    MSVCP_runtime_error_ctor(this, name);
+    this->e.vtable = &MSVCP_failure_vtable;
+    return this;
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_failure_copy_ctor, 8)
+failure* __thiscall MSVCP_failure_copy_ctor(
+        failure *this, failure *rhs)
+{
+    TRACE("%p %p\n", this, rhs);
+    MSVCP_runtime_error_copy_ctor(this, rhs);
+    this->e.vtable = &MSVCP_failure_vtable;
+    return this;
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_failure_dtor, 4)
+void __thiscall MSVCP_failure_dtor(failure *this)
+{
+    TRACE("%p\n", this);
+    MSVCP_runtime_error_dtor(this);
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_failure_vector_dtor, 8)
+void* __thiscall MSVCP_failure_vector_dtor(
+        failure *this, unsigned int flags)
+{
+    TRACE("%p %x\n", this, flags);
+    return MSVCP_runtime_error_vector_dtor(this, flags);
+}
+
+DEFINE_THISCALL_WRAPPER(MSVCP_failure_what, 4)
+const char* __thiscall MSVCP_failure_what(failure *this)
+{
+    TRACE("%p\n", this);
+    return MSVCP_runtime_error_what(this);
+}
+
+DEFINE_RTTI_DATA2(failure, 0, &runtime_error_rtti_base_descriptor, &exception_rtti_base_descriptor, ".?AVfailure@std@@");
+
+static const cxx_type_info failure_cxx_type_info = {
+    0,
+    &failure_type_info,
+    { 0, -1, 0 },
+    sizeof(failure),
+    (cxx_copy_ctor)THISCALL(MSVCP_failure_copy_ctor)
+};
+
+static const cxx_type_info_table failure_cxx_type_table = {
+    3,
+    {
+        &failure_cxx_type_info,
+        &runtime_error_cxx_type_info,
+        &exception_cxx_type_info
+    }
+};
+
+static const cxx_exception_type failure_cxx_type = {
+    0,
+    (cxx_copy_ctor)THISCALL(MSVCP_failure_dtor),
+    NULL,
+    &failure_cxx_type_table
+};
+
 #ifndef __GNUC__
 void __asm_dummy_vtables(void) {
 #endif
-    __ASM_EXCEPTION_VTABLE(bad_alloc)
-    __ASM_EXCEPTION_STRING_VTABLE(logic_error)
-    __ASM_EXCEPTION_STRING_VTABLE(length_error)
-    __ASM_EXCEPTION_STRING_VTABLE(out_of_range)
+    __ASM_VTABLE(type_info, "");
+    __ASM_VTABLE(exception, VTABLE_ADD_FUNC(MSVCP_what_exception));
+    __ASM_VTABLE(bad_alloc, VTABLE_ADD_FUNC(MSVCP_what_exception));
+    __ASM_VTABLE(logic_error, VTABLE_ADD_FUNC(MSVCP_logic_error_what));
+    __ASM_VTABLE(length_error, VTABLE_ADD_FUNC(MSVCP_logic_error_what));
+    __ASM_VTABLE(out_of_range, VTABLE_ADD_FUNC(MSVCP_logic_error_what));
+    __ASM_VTABLE(invalid_argument, VTABLE_ADD_FUNC(MSVCP_logic_error_what));
+    __ASM_VTABLE(runtime_error, VTABLE_ADD_FUNC(MSVCP_runtime_error_what));
+    __ASM_VTABLE(failure, VTABLE_ADD_FUNC(MSVCP_failure_what));
 #ifndef __GNUC__
 }
 #endif
@@ -628,34 +713,47 @@ void throw_exception(exception_type et, const char *str)
     const char *addr = str;
 
     switch(et) {
+    case EXCEPTION_RERAISE:
+        _CxxThrowException(NULL, NULL);
     case EXCEPTION: {
         exception e;
         MSVCP_exception_ctor(&e, &addr);
         _CxxThrowException(&e, &exception_cxx_type);
-        return;
     }
     case EXCEPTION_BAD_ALLOC: {
         bad_alloc e;
         MSVCP_bad_alloc_ctor(&e, &addr);
         _CxxThrowException(&e, &bad_alloc_cxx_type);
-        return;
     }
     case EXCEPTION_LOGIC_ERROR: {
         logic_error e;
         MSVCP_logic_error_ctor(&e, &addr);
         _CxxThrowException((exception*)&e, &logic_error_cxx_type);
-        return;
     }
     case EXCEPTION_LENGTH_ERROR: {
         length_error e;
         MSVCP_length_error_ctor(&e, &addr);
         _CxxThrowException((exception*)&e, &length_error_cxx_type);
-        return;
     }
     case EXCEPTION_OUT_OF_RANGE: {
         out_of_range e;
         MSVCP_out_of_range_ctor(&e, &addr);
         _CxxThrowException((exception*)&e, &out_of_range_cxx_type);
     }
+    case EXCEPTION_INVALID_ARGUMENT: {
+        invalid_argument e;
+        MSVCP_invalid_argument_ctor(&e, &addr);
+        _CxxThrowException((exception*)&e, &invalid_argument_cxx_type);
+    }
+    case EXCEPTION_RUNTIME_ERROR: {
+        runtime_error e;
+        MSVCP_runtime_error_ctor(&e, &addr);
+        _CxxThrowException((exception*)&e, &runtime_error_cxx_type);
+    }
+    case EXCEPTION_FAILURE: {
+        failure e;
+        MSVCP_failure_ctor(&e, &addr);
+        _CxxThrowException((exception*)&e, &failure_cxx_type);
+    }
     }
 }
