@@ -89,10 +89,12 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
             return FALSE;    /* prefer native version */
         case DLL_PROCESS_ATTACH:
             init_cxx_funcs();
-            set_exception_vtable();
             init_lockit();
+            init_io();
             break;
         case DLL_PROCESS_DETACH:
+            free_io();
+            free_locale();
             free_lockit();
             break;
     }
