@@ -21,12 +21,14 @@
 #ifndef __MSVCRT_CPPEXCEPT_H
 #define __MSVCRT_CPPEXCEPT_H
 
-#define CXX_FRAME_MAGIC    0x19930520
-#define CXX_EXCEPTION      0xe06d7363
+#define CXX_FRAME_MAGIC_VC6 0x19930520
+#define CXX_FRAME_MAGIC_VC7 0x19930521
+#define CXX_FRAME_MAGIC_VC8 0x19930522
+#define CXX_EXCEPTION       0xe06d7363
 
-typedef void (*vtable_ptr)();
+typedef void (*vtable_ptr)(void);
 
-/* type_info object, see cpp.c for inplementation */
+/* type_info object, see cpp.c for implementation */
 typedef struct __type_info
 {
   const vtable_ptr *vtable;
@@ -42,54 +44,6 @@ typedef struct __exception
   int               do_free; /* Whether to free 'name' in our dtor */
 } exception;
 
-/* the exception frame used by CxxFrameHandler */
-typedef struct __cxx_exception_frame
-{
-    EXCEPTION_REGISTRATION_RECORD  frame;    /* the standard exception frame */
-    int                            trylevel;
-    DWORD                          ebp;
-} cxx_exception_frame;
-
-/* info about a single catch {} block */
-typedef struct __catchblock_info
-{
-    UINT             flags;         /* flags (see below) */
-    const type_info *type_info;     /* C++ type caught by this block */
-    int              offset;        /* stack offset to copy exception object to */
-    void           (*handler)();    /* catch block handler code */
-} catchblock_info;
-#define TYPE_FLAG_CONST      1
-#define TYPE_FLAG_VOLATILE   2
-#define TYPE_FLAG_REFERENCE  8
-
-/* info about a single try {} block */
-typedef struct __tryblock_info
-{
-    int                    start_level;      /* start trylevel of that block */
-    int                    end_level;        /* end trylevel of that block */
-    int                    catch_level;      /* initial trylevel of the catch block */
-    int                    catchblock_count; /* count of catch blocks in array */
-    const catchblock_info *catchblock;       /* array of catch blocks */
-} tryblock_info;
-
-/* info about the unwind handler for a given trylevel */
-typedef struct __unwind_info
-{
-    int    prev;          /* prev trylevel unwind handler, to run after this one */
-    void (*handler)();    /* unwind handler */
-} unwind_info;
-
-/* descriptor of all try blocks of a given function */
-typedef struct __cxx_function_descr
-{
-    UINT                 magic;          /* must be CXX_FRAME_MAGIC */
-    UINT                 unwind_count;   /* number of unwind handlers */
-    const unwind_info   *unwind_table;   /* array of unwind handlers */
-    UINT                 tryblock_count; /* number of try blocks */
-    const tryblock_info *tryblock;       /* array of try blocks */
-    UINT                 unknown[3];
-} cxx_function_descr;
-
 typedef void (*cxx_copy_ctor)(void);
 
 /* offsets for computing the this pointer */
@@ -119,21 +73,25 @@ typedef struct __cxx_type_info_table
     const cxx_type_info *info[3];   /* variable length, we declare it large enough for static RTTI */
 } cxx_type_info_table;
 
-typedef DWORD (*cxx_exc_custom_handler)( PEXCEPTION_RECORD, cxx_exception_frame*,
+struct __cxx_exception_frame;
+struct __cxx_function_descr;
+
+typedef DWORD (*cxx_exc_custom_handler)( PEXCEPTION_RECORD, struct __cxx_exception_frame*,
                                          PCONTEXT, EXCEPTION_REGISTRATION_RECORD**,
-                                         const cxx_function_descr*, int nested_trylevel,
+                                         const struct __cxx_function_descr*, int nested_trylevel,
                                          EXCEPTION_REGISTRATION_RECORD *nested_frame, DWORD unknown3 );
 
 /* type information for an exception object */
 typedef struct __cxx_exception_type
 {
     UINT                       flags;            /* TYPE_FLAG flags */
-    void                     (*destructor)();    /* exception object destructor */
+    void                     (*destructor)(void);/* exception object destructor */
     cxx_exc_custom_handler     custom_handler;   /* custom handler for this exception */
     const cxx_type_info_table *type_info_table;  /* list of types for this exception object */
 } cxx_exception_type;
 
-void _CxxThrowException(exception*,const cxx_exception_type*);
+void WINAPI _CxxThrowException(exception*,const cxx_exception_type*);
+int CDECL _XcptFilter(NTSTATUS, PEXCEPTION_POINTERS);
 
 static inline const char *dbgstr_type_info( const type_info *info )
 {
