@@ -33,7 +33,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(msvcrt);
 /*********************************************************************
  *		_beep (MSVCRT.@)
  */
-void CDECL _beep( unsigned int freq, unsigned int duration)
+void CDECL MSVCRT__beep( unsigned int freq, unsigned int duration)
 {
     TRACE(":Freq %d, Duration %d\n",freq,duration);
     Beep(freq, duration);
@@ -123,6 +123,33 @@ void* CDECL _lsearch(const void* match, void* start,
   return start;
 }
 
+/*********************************************************************
+ *                  bsearch_s (msvcrt.@)
+ */
+void* CDECL MSVCRT_bsearch_s(const void *key, const void *base,
+                             MSVCRT_size_t nmemb, MSVCRT_size_t size,
+                             int (__cdecl *compare)(void *, const void *, const void *), void *ctx)
+{
+    ssize_t min = 0;
+    ssize_t max = nmemb - 1;
+
+    if (!MSVCRT_CHECK_PMT(size != 0) || !MSVCRT_CHECK_PMT(compare != NULL))
+        return NULL;
+
+    while (min <= max)
+    {
+        ssize_t cursor = (min + max) / 2;
+        int ret = compare(ctx, key,(const char *)base+(cursor*size));
+        if (!ret)
+            return (char*)base+(cursor*size);
+        if (ret < 0)
+            max = cursor - 1;
+        else
+            min = cursor + 1;
+    }
+    return NULL;
+}
+
 /*********************************************************************
  *		_chkesp (MSVCRT.@)
  *
@@ -265,7 +292,7 @@ unsigned int CDECL _get_output_format(void)
 /*********************************************************************
  * _resetstkoflw (MSVCRT.@)
  */
-int CDECL _resetstkoflw(void)
+int CDECL MSVCRT__resetstkoflw(void)
 {
     int stack_addr;
 
