@@ -109,18 +109,20 @@ typedef struct MSVCRT_tagLC_ID {
 } MSVCRT_LC_ID, *MSVCRT_LPLC_ID;
 
 typedef struct {
-    /* str table contains following strings:
-     * short weekday names [7];
-     * weekday names [7];
-     * short month names [12];
-     * month names [12];
-     * s1159 (AM);
-     * s2359 (PM);
-     * short date format;
-     * date format;
-     * time format;
-     */
-    char *str[43];
+    union {
+        char *str[43];
+        struct {
+            char *short_wday[7];
+            char *wday[7];
+            char *short_mon[12];
+            char *mon[12];
+            char *am;
+            char *pm;
+            char *short_date;
+            char *date;
+            char *time;
+        } names;
+    } str;
     LCID lcid;
     int  unk[2];
     MSVCRT_wchar_t *wstr[43];
@@ -243,8 +245,8 @@ int __cdecl MSVCRT__set_new_mode(int mode);
 void* __cdecl MSVCRT_operator_new(MSVCRT_size_t);
 void __cdecl MSVCRT_operator_delete(void*);
 
-typedef void* (*__cdecl malloc_func_t)(MSVCRT_size_t);
-typedef void  (*__cdecl free_func_t)(void*);
+typedef void* (__cdecl *malloc_func_t)(MSVCRT_size_t);
+typedef void  (__cdecl *free_func_t)(void*);
 
 extern char* __cdecl __unDName(char *,const char*,int,malloc_func_t,free_func_t,unsigned short int);
 extern char* __cdecl __unDNameEx(char *,const char*,int,malloc_func_t,free_func_t,void *,unsigned short int);
@@ -895,6 +897,7 @@ int            __cdecl MSVCRT_vsnwprintf(MSVCRT_wchar_t *str, MSVCRT_size_t len,
                                        const MSVCRT_wchar_t *format, __ms_va_list valist );
 int            __cdecl MSVCRT__snwprintf(MSVCRT_wchar_t*, unsigned int, const MSVCRT_wchar_t*, ...);
 int            __cdecl MSVCRT_sprintf(char*,const char*,...);
+int            __cdecl MSVCRT__snprintf(char*,unsigned int,const char*,...);
 int            __cdecl MSVCRT__scprintf(const char*,...);
 int            __cdecl MSVCRT_raise(int sig);
 
