@@ -57,7 +57,7 @@ static void StgStreamImpl_Destroy(StgStreamImpl* This)
 
   /*
    * Release the reference we are holding on the parent storage.
-   * IStorage_Release((IStorage*)This->parentStorage);
+   * IStorage_Release(&This->parentStorage->IStorage_iface);
    *
    * No, don't do this. Some apps call IStorage_Release without
    * calling IStream_Release first. If we grab a reference the
@@ -74,9 +74,6 @@ static void StgStreamImpl_Destroy(StgStreamImpl* This)
 
   This->parentStorage = 0;
 
-  /*
-   * Finally, free the memory used-up by the class.
-   */
   HeapFree(GetProcessHeap(), 0, This);
 }
 
@@ -89,41 +86,24 @@ static HRESULT WINAPI StgStreamImpl_QueryInterface(
 		  REFIID         riid,	      /* [in] */
 		  void**         ppvObject)   /* [iid_is][out] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
-  /*
-   * Perform a sanity check on the parameters.
-   */
   if (ppvObject==0)
     return E_INVALIDARG;
 
-  /*
-   * Initialize the return parameter.
-   */
   *ppvObject = 0;
 
-  /*
-   * Compare the riid with the interface IDs implemented by this object.
-   */
   if (IsEqualIID(&IID_IUnknown, riid) ||
       IsEqualIID(&IID_IPersist, riid) ||
       IsEqualIID(&IID_IPersistStream, riid) ||
       IsEqualIID(&IID_ISequentialStream, riid) ||
       IsEqualIID(&IID_IStream, riid))
   {
-    *ppvObject = This;
+    *ppvObject = &This->IStream_iface;
   }
-
-  /*
-   * Check that we obtained an interface.
-   */
-  if ((*ppvObject)==0)
+  else
     return E_NOINTERFACE;
 
-  /*
-   * Query Interface always increases the reference count by one when it is
-   * successful
-   */
   IStream_AddRef(iface);
 
   return S_OK;
@@ -136,7 +116,7 @@ static HRESULT WINAPI StgStreamImpl_QueryInterface(
 static ULONG WINAPI StgStreamImpl_AddRef(
 		IStream* iface)
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
   return InterlockedIncrement(&This->ref);
 }
 
@@ -147,7 +127,7 @@ static ULONG WINAPI StgStreamImpl_AddRef(
 static ULONG WINAPI StgStreamImpl_Release(
 		IStream* iface)
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   ULONG ref;
 
@@ -179,7 +159,7 @@ static HRESULT WINAPI StgStreamImpl_Read(
 		  ULONG          cb,        /* [in] */
 		  ULONG*         pcbRead)   /* [out] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   ULONG bytesReadBuffer;
   HRESULT res;
@@ -235,7 +215,7 @@ static HRESULT WINAPI StgStreamImpl_Write(
 		  ULONG          cb,          /* [in] */
 		  ULONG*         pcbWritten)  /* [out] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   ULONG bytesWritten = 0;
   HRESULT res;
@@ -295,6 +275,9 @@ static HRESULT WINAPI StgStreamImpl_Write(
    */
   This->currentPosition.u.LowPart += *pcbWritten;
 
+  if (SUCCEEDED(res))
+    res = StorageBaseImpl_Flush(This->parentStorage);
+
   TRACE("<-- S_OK, written %u\n", *pcbWritten);
   return res;
 }
@@ -313,7 +296,7 @@ static HRESULT WINAPI StgStreamImpl_Seek(
 		  DWORD           dwOrigin,         /* [in] */
 		  ULARGE_INTEGER* plibNewPosition) /* [out] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   ULARGE_INTEGER newPosition;
   DirEntry currentEntry;
@@ -386,7 +369,7 @@ static HRESULT WINAPI StgStreamImpl_SetSize(
 				     IStream*      iface,
 				     ULARGE_INTEGER  libNewSize)   /* [in] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   HRESULT      hr;
 
@@ -417,6 +400,10 @@ static HRESULT WINAPI StgStreamImpl_SetSize(
   }
 
   hr = StorageBaseImpl_StreamSetSize(This->parentStorage, This->dirEntry, libNewSize);
+
+  if (SUCCEEDED(hr))
+    hr = StorageBaseImpl_Flush(This->parentStorage);
+
   return hr;
 }
 
@@ -434,7 +421,7 @@ static HRESULT WINAPI StgStreamImpl_CopyTo(
 				    ULARGE_INTEGER* pcbRead,      /* [out] */
 				    ULARGE_INTEGER* pcbWritten)   /* [out] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
   HRESULT        hr = S_OK;
   BYTE           tmpBuffer[128];
   ULONG          bytesRead, bytesWritten, copySize;
@@ -509,7 +496,7 @@ static HRESULT WINAPI StgStreamImpl_Commit(
 		  IStream*      iface,
 		  DWORD           grfCommitFlags)  /* [in] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   if (!This->parentStorage)
   {
@@ -540,7 +527,7 @@ static HRESULT WINAPI StgStreamImpl_LockRegion(
 					ULARGE_INTEGER cb,          /* [in] */
 					DWORD          dwLockType)  /* [in] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   if (!This->parentStorage)
   {
@@ -558,7 +545,7 @@ static HRESULT WINAPI StgStreamImpl_UnlockRegion(
 					  ULARGE_INTEGER cb,          /* [in] */
 					  DWORD          dwLockType)  /* [in] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   if (!This->parentStorage)
   {
@@ -583,7 +570,7 @@ static HRESULT WINAPI StgStreamImpl_Stat(
 		  STATSTG*       pstatstg,     /* [out] */
 		  DWORD          grfStatFlag)  /* [in] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
 
   DirEntry     currentEntry;
   HRESULT      hr;
@@ -643,7 +630,7 @@ static HRESULT WINAPI StgStreamImpl_Clone(
 				   IStream*     iface,
 				   IStream**    ppstm) /* [out] */
 {
-  StgStreamImpl* const This=(StgStreamImpl*)iface;
+  StgStreamImpl* This = impl_from_IStream(iface);
   HRESULT hres;
   StgStreamImpl* new_stream;
   LARGE_INTEGER seek_pos;
@@ -665,12 +652,12 @@ static HRESULT WINAPI StgStreamImpl_Clone(
   if (!new_stream)
     return STG_E_INSUFFICIENTMEMORY; /* Currently the only reason for new_stream=0 */
 
-  *ppstm = (IStream*) new_stream;
+  *ppstm = &new_stream->IStream_iface;
   IStream_AddRef(*ppstm);
 
   seek_pos.QuadPart = This->currentPosition.QuadPart;
 
-  hres=StgStreamImpl_Seek (*ppstm, seek_pos, STREAM_SEEK_SET, NULL);
+  hres = IStream_Seek(*ppstm, seek_pos, STREAM_SEEK_SET, NULL);
 
   assert (SUCCEEDED(hres));
 
@@ -680,7 +667,7 @@ static HRESULT WINAPI StgStreamImpl_Clone(
 /*
  * Virtual function table for the StgStreamImpl class.
  */
-static const IStreamVtbl StgStreamImpl_Vtbl =
+static const IStreamVtbl StgStreamVtbl =
 {
     StgStreamImpl_QueryInterface,
     StgStreamImpl_AddRef,
@@ -718,12 +705,12 @@ StgStreamImpl* StgStreamImpl_Construct(
 
   newStream = HeapAlloc(GetProcessHeap(), 0, sizeof(StgStreamImpl));
 
-  if (newStream!=0)
+  if (newStream)
   {
     /*
      * Set-up the virtual function table and reference count.
      */
-    newStream->lpVtbl    = &StgStreamImpl_Vtbl;
+    newStream->IStream_iface.lpVtbl = &StgStreamVtbl;
     newStream->ref       = 0;
 
     newStream->parentStorage = parentStorage;
@@ -732,7 +719,7 @@ StgStreamImpl* StgStreamImpl_Construct(
      * We want to nail-down the reference to the storage in case the
      * stream out-lives the storage in the client application.
      *
-     * -- IStorage_AddRef((IStorage*)newStream->parentStorage);
+     * -- IStorage_AddRef(&newStream->parentStorage->IStorage_iface);
      *
      * No, don't do this. Some apps call IStorage_Release without
      * calling IStream_Release first. If we grab a reference the
