@@ -36,16 +36,6 @@
 #define WINAPI      __stdcall
 #define APIENTRY    WINAPI
 
-/* X11 locking */
-
-extern void (*wine_tsx11_lock_ptr)(void);
-extern void (*wine_tsx11_unlock_ptr)(void);
-
-/* As GLX relies on X, this is needed */
-void enter_gl(void);
-#define ENTER_GL() enter_gl()
-#define LEAVE_GL() wine_tsx11_unlock_ptr()
-
 
 typedef struct {
   const char  *name;     /* name of the extension */
