@@ -37,6 +37,7 @@ const unsigned char * WINAPI glGetString(unsigned int);
 #define GL_VENDOR 0x1F00
 #define GL_RENDERER 0x1F01
 #define GL_VERSION 0x1F02
+#define GL_EXTENSIONS 0x1F03
 
 #define GL_VIEWPORT 0x0ba2
 void WINAPI glGetIntegerv(GLenum pname, GLint *params);
@@ -116,6 +117,33 @@ static void init_functions(void)
 #undef GET_PROC
 }
 
+static BOOL gl_extension_supported(const char *extensions, const char *extension_string)
+{
+    size_t ext_str_len = strlen(extension_string);
+
+    while (*extensions)
+    {
+        const char *start;
+        size_t len;
+
+        while (isspace(*extensions))
+            ++extensions;
+        start = extensions;
+        while (!isspace(*extensions) && *extensions)
+            ++extensions;
+
+        len = extensions - start;
+        if (!len)
+            continue;
+
+        if (len == ext_str_len && !memcmp(start, extension_string, ext_str_len))
+        {
+            return TRUE;
+        }
+    }
+    return FALSE;
+}
+
 static void test_pbuffers(HDC hdc)
 {
     const int iAttribList[] = { WGL_DRAW_TO_PBUFFER_ARB, 1, /* Request pbuffer support */
@@ -360,7 +388,6 @@ static void test_makecurrent(HDC winhdc)
 {
     BOOL ret;
     HGLRC hglrc;
-    DWORD error;
 
     hglrc = wglCreateContext(winhdc);
     ok( hglrc != 0, "wglCreateContext failed\n" );
@@ -383,9 +410,9 @@ static void test_makecurrent(HDC winhdc)
 
     SetLastError( 0xdeadbeef );
     ret = wglMakeCurrent( NULL, NULL );
-    ok( !ret, "wglMakeCurrent succeeded\n" );
-    error = GetLastError();
-    ok( error == ERROR_INVALID_HANDLE, "Expected ERROR_INVALID_HANDLE, got error=%x\n", error);
+    ok( !ret || broken(ret) /* nt4 */, "wglMakeCurrent succeeded\n" );
+    if (!ret) ok( GetLastError() == ERROR_INVALID_HANDLE,
+                  "Expected ERROR_INVALID_HANDLE, got error=%x\n", GetLastError() );
 
     ret = wglMakeCurrent( winhdc, NULL );
     ok( ret, "wglMakeCurrent failed\n" );
@@ -400,9 +427,9 @@ static void test_makecurrent(HDC winhdc)
 
     SetLastError( 0xdeadbeef );
     ret = wglMakeCurrent( NULL, NULL );
-    ok( !ret, "wglMakeCurrent succeeded\n" );
-    error = GetLastError();
-    ok( error == ERROR_INVALID_HANDLE, "Expected ERROR_INVALID_HANDLE, got error=%x\n", error);
+    ok( !ret || broken(ret) /* nt4 */, "wglMakeCurrent succeeded\n" );
+    if (!ret) ok( GetLastError() == ERROR_INVALID_HANDLE,
+                  "Expected ERROR_INVALID_HANDLE, got error=%x\n", GetLastError() );
 
     ret = wglMakeCurrent( winhdc, hglrc );
     ok( ret, "wglMakeCurrent failed\n" );
@@ -515,37 +542,49 @@ static void test_acceleration(HDC hdc)
     }
 }
 
-static void test_bitmap_rendering(void)
+static void test_bitmap_rendering( BOOL use_dib )
 {
     PIXELFORMATDESCRIPTOR pfd;
-    int i, iPixelFormat=0;
+    int i, ret, bpp, iPixelFormat=0;
     unsigned int nFormats;
-    HGLRC hglrc;
+    HGLRC hglrc, hglrc2;
     BITMAPINFO biDst;
-    HBITMAP bmpDst, oldDst;
+    HBITMAP bmpDst, oldDst, bmp2;
     HDC hdcDst, hdcScreen;
-    UINT32 *dstBuffer;
-
-    memset(&biDst, 0, sizeof(BITMAPINFO));
-    biDst.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
-    biDst.bmiHeader.biWidth = 2;
-    biDst.bmiHeader.biHeight = -2;
-    biDst.bmiHeader.biPlanes = 1;
-    biDst.bmiHeader.biBitCount = 32;
-    biDst.bmiHeader.biCompression = BI_RGB;
+    UINT *dstBuffer = NULL;
 
     hdcScreen = CreateCompatibleDC(0);
-    if(GetDeviceCaps(hdcScreen, BITSPIXEL) != 32)
+    hdcDst = CreateCompatibleDC(0);
+
+    if (use_dib)
     {
-        DeleteDC(hdcScreen);
-        trace("Skipping bitmap rendering test\n");
-        return;
+        bpp = 32;
+        memset(&biDst, 0, sizeof(BITMAPINFO));
+        biDst.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
+        biDst.bmiHeader.biWidth = 4;
+        biDst.bmiHeader.biHeight = -4;
+        biDst.bmiHeader.biPlanes = 1;
+        biDst.bmiHeader.biBitCount = 32;
+        biDst.bmiHeader.biCompression = BI_RGB;
+
+        bmpDst = CreateDIBSection(0, &biDst, DIB_RGB_COLORS, (void**)&dstBuffer, NULL, 0);
+
+        biDst.bmiHeader.biWidth = 12;
+        biDst.bmiHeader.biHeight = -12;
+        biDst.bmiHeader.biBitCount = 16;
+        bmp2 = CreateDIBSection(0, &biDst, DIB_RGB_COLORS, NULL, NULL, 0);
+    }
+    else
+    {
+        bpp = GetDeviceCaps( hdcScreen, BITSPIXEL );
+        bmpDst = CreateBitmap( 4, 4, 1, bpp, NULL );
+        bmp2 = CreateBitmap( 12, 12, 1, bpp, NULL );
     }
 
-    hdcDst = CreateCompatibleDC(hdcScreen);
-    bmpDst = CreateDIBSection(hdcDst, &biDst, DIB_RGB_COLORS, (void**)&dstBuffer, NULL, 0);
     oldDst = SelectObject(hdcDst, bmpDst);
 
+    trace( "testing on %s\n", use_dib ? "DIB" : "DDB" );
+
     /* Pick a pixel format by hand because ChoosePixelFormat is unreliable */
     nFormats = DescribePixelFormat(hdcDst, 0, 0, NULL);
     for(i=1; i<=nFormats; i++)
@@ -555,7 +594,7 @@ static void test_bitmap_rendering(void)
 
         if((pfd.dwFlags & PFD_DRAW_TO_BITMAP) &&
            (pfd.dwFlags & PFD_SUPPORT_OPENGL) &&
-           (pfd.cColorBits == 32) &&
+           (pfd.cColorBits == bpp) &&
            (pfd.cAlphaBits == 8) )
         {
             iPixelFormat = i;
@@ -569,55 +608,115 @@ static void test_bitmap_rendering(void)
     }
     else
     {
-        SetPixelFormat(hdcDst, iPixelFormat, &pfd);
+        ret = SetPixelFormat(hdcDst, iPixelFormat, &pfd);
+        ok( ret, "SetPixelFormat failed\n" );
+        ret = GetPixelFormat( hdcDst );
+        ok( ret == iPixelFormat, "GetPixelFormat returned %d/%d\n", ret, iPixelFormat );
+        ret = SetPixelFormat(hdcDst, iPixelFormat + 1, &pfd);
+        ok( !ret, "SetPixelFormat succeeded\n" );
         hglrc = wglCreateContext(hdcDst);
-        todo_wine ok(hglrc != NULL, "Unable to create a context\n");
+        ok(hglrc != NULL, "Unable to create a context\n");
 
         if(hglrc)
         {
+            GLint viewport[4];
             wglMakeCurrent(hdcDst, hglrc);
+            hglrc2 = wglCreateContext(hdcDst);
+            ok(hglrc2 != NULL, "Unable to create a context\n");
 
             /* Note this is RGBA but we read ARGB back */
             glClearColor((float)0x22/0xff, (float)0x33/0xff, (float)0x44/0xff, (float)0x11/0xff);
             glClear(GL_COLOR_BUFFER_BIT);
+            glGetIntegerv( GL_VIEWPORT, viewport );
             glFinish();
 
+            ok( viewport[0] == 0 && viewport[1] == 0 && viewport[2] == 4 && viewport[3] == 4,
+                "wrong viewport %d,%d,%d,%d\n", viewport[0], viewport[1], viewport[2], viewport[3] );
             /* Note apparently the alpha channel is not supported by the software renderer (bitmap only works using software) */
-            ok(dstBuffer[0] == 0x223344, "Expected color=0x223344, received color=%x\n", dstBuffer[0]);
+            if (dstBuffer)
+                for (i = 0; i < 16; i++)
+                    ok(dstBuffer[i] == 0x223344 || dstBuffer[i] == 0x11223344, "Received color=%x at %u\n",
+                       dstBuffer[i], i);
+
+            SelectObject(hdcDst, bmp2);
+            ret = GetPixelFormat( hdcDst );
+            ok( ret == iPixelFormat, "GetPixelFormat returned %d/%d\n", ret, iPixelFormat );
+            ret = SetPixelFormat(hdcDst, iPixelFormat + 1, &pfd);
+            ok( !ret, "SetPixelFormat succeeded\n" );
+
+            /* context still uses the old pixel format and viewport */
+            glClearColor((float)0x44/0xff, (float)0x33/0xff, (float)0x22/0xff, (float)0x11/0xff);
+            glClear(GL_COLOR_BUFFER_BIT);
+            glFinish();
+            glGetIntegerv( GL_VIEWPORT, viewport );
+            ok( viewport[0] == 0 && viewport[1] == 0 && viewport[2] == 4 && viewport[3] == 4,
+                "wrong viewport %d,%d,%d,%d\n", viewport[0], viewport[1], viewport[2], viewport[3] );
 
             wglMakeCurrent(NULL, NULL);
-            wglDeleteContext(hglrc);
+            wglMakeCurrent(hdcDst, hglrc);
+            glClearColor((float)0x44/0xff, (float)0x55/0xff, (float)0x66/0xff, (float)0x11/0xff);
+            glClear(GL_COLOR_BUFFER_BIT);
+            glFinish();
+            glGetIntegerv( GL_VIEWPORT, viewport );
+            ok( viewport[0] == 0 && viewport[1] == 0 && viewport[2] == 4 && viewport[3] == 4,
+                "wrong viewport %d,%d,%d,%d\n", viewport[0], viewport[1], viewport[2], viewport[3] );
+
+            wglMakeCurrent(hdcDst, hglrc2);
+            glGetIntegerv( GL_VIEWPORT, viewport );
+            ok( viewport[0] == 0 && viewport[1] == 0 && viewport[2] == 12 && viewport[3] == 12,
+                "wrong viewport %d,%d,%d,%d\n", viewport[0], viewport[1], viewport[2], viewport[3] );
+
+            wglMakeCurrent(hdcDst, hglrc);
+            glGetIntegerv( GL_VIEWPORT, viewport );
+            ok( viewport[0] == 0 && viewport[1] == 0 && viewport[2] == 4 && viewport[3] == 4,
+                "wrong viewport %d,%d,%d,%d\n", viewport[0], viewport[1], viewport[2], viewport[3] );
+
+            SelectObject(hdcDst, bmpDst);
+            ret = GetPixelFormat( hdcDst );
+            ok( ret == iPixelFormat, "GetPixelFormat returned %d/%d\n", ret, iPixelFormat );
+            ret = SetPixelFormat(hdcDst, iPixelFormat + 1, &pfd);
+            ok( !ret, "SetPixelFormat succeeded\n" );
+            wglMakeCurrent(hdcDst, hglrc2);
+            glGetIntegerv( GL_VIEWPORT, viewport );
+            ok( viewport[0] == 0 && viewport[1] == 0 && viewport[2] == 12 && viewport[3] == 12,
+                "wrong viewport %d,%d,%d,%d\n", viewport[0], viewport[1], viewport[2], viewport[3] );
         }
     }
 
     SelectObject(hdcDst, oldDst);
+    DeleteObject(bmp2);
     DeleteObject(bmpDst);
     DeleteDC(hdcDst);
-
     DeleteDC(hdcScreen);
 }
 
 struct wgl_thread_param
 {
     HANDLE test_finished;
+    HWND hwnd;
     HGLRC hglrc;
-    BOOL hglrc_deleted;
-    DWORD last_error;
+    BOOL make_current;
+    BOOL make_current_error;
+    BOOL deleted;
+    DWORD deleted_error;
 };
 
 static DWORD WINAPI wgl_thread(void *param)
 {
     struct wgl_thread_param *p = param;
+    HDC hdc = GetDC( p->hwnd );
 
     SetLastError(0xdeadbeef);
-    p->hglrc_deleted = wglDeleteContext(p->hglrc);
-    p->last_error = GetLastError();
+    p->make_current = wglMakeCurrent(hdc, p->hglrc);
+    p->make_current_error = GetLastError();
+    p->deleted = wglDeleteContext(p->hglrc);
+    p->deleted_error = GetLastError();
+    ReleaseDC( p->hwnd, hdc );
     SetEvent(p->test_finished);
-
     return 0;
 }
 
-static void test_deletecontext(HDC hdc)
+static void test_deletecontext(HWND hwnd, HDC hdc)
 {
     struct wgl_thread_param thread_params;
     HGLRC hglrc = wglCreateContext(hdc);
@@ -646,14 +745,17 @@ static void test_deletecontext(HDC hdc)
      * This differs from GLX which does allow it but it delays actual deletion until the context becomes not current.
      */
     thread_params.hglrc = hglrc;
+    thread_params.hwnd  = hwnd;
     thread_params.test_finished = CreateEvent(NULL, FALSE, FALSE, NULL);
     thread_handle = CreateThread(NULL, 0, wgl_thread, &thread_params, 0, &tid);
     ok(!!thread_handle, "Failed to create thread, last error %#x.\n", GetLastError());
     if(thread_handle)
     {
         WaitForSingleObject(thread_handle, INFINITE);
-        ok(thread_params.hglrc_deleted == FALSE, "Attempt to delete WGL context from another thread passed but should fail!\n");
-        ok(thread_params.last_error == ERROR_BUSY, "Expected last error to be ERROR_BUSY, got %u\n", thread_params.last_error);
+        ok(!thread_params.make_current, "Attempt to make WGL context from another thread passed\n");
+        ok(thread_params.make_current_error == ERROR_BUSY, "Expected last error to be ERROR_BUSY, got %u\n", thread_params.make_current_error);
+        ok(!thread_params.deleted, "Attempt to delete WGL context from another thread passed\n");
+        ok(thread_params.deleted_error == ERROR_BUSY, "Expected last error to be ERROR_BUSY, got %u\n", thread_params.deleted_error);
     }
     CloseHandle(thread_params.test_finished);
 
@@ -672,6 +774,52 @@ static void test_deletecontext(HDC hdc)
     ok(hglrc == NULL, "A WGL context is active while none was expected\n");
 }
 
+
+static void test_getprocaddress(HDC hdc)
+{
+    const char *extensions = (const char*)glGetString(GL_EXTENSIONS);
+    PROC func = NULL;
+    HGLRC ctx = wglGetCurrentContext();
+
+    if (!extensions)
+    {
+        skip("skipping wglGetProcAddress tests because no GL extensions supported\n");
+        return;
+    }
+
+    /* Core GL 1.0/1.1 functions should not be loadable through wglGetProcAddress.
+     * Try to load the function with and without a context.
+     */
+    func = wglGetProcAddress("glEnable");
+    ok(func == NULL, "Lookup of function glEnable with a context passed, expected a failure\n");
+    wglMakeCurrent(hdc, NULL);
+    func = wglGetProcAddress("glEnable");
+    ok(func == NULL, "Lookup of function glEnable without a context passed, expected a failure\n");
+    wglMakeCurrent(hdc, ctx);
+
+    /* The goal of the test will be to test behavior of wglGetProcAddress when
+     * no WGL context is active. Before the test we pick an extension (GL_ARB_multitexture)
+     * which any GL >=1.2.1 implementation supports. Unfortunately the GDI renderer doesn't
+     * support it. There aren't any extensions we can use for this test which are supported by
+     * both GDI and real drivers.
+     * Note GDI only has GL_EXT_bgra, GL_EXT_paletted_texture and GL_WIN_swap_hint.
+     */
+    if (!gl_extension_supported(extensions, "GL_ARB_multitexture"))
+    {
+        skip("skipping test because lack of GL_ARB_multitexture support\n");
+        return;
+    }
+
+    func = wglGetProcAddress("glActiveTextureARB");
+    ok(func != NULL, "Unable to lookup glActiveTextureARB, last error %#x\n", GetLastError());
+
+    /* Temporarily disable the context, so we can see that we can't retrieve functions now. */
+    wglMakeCurrent(hdc, NULL);
+    func = wglGetProcAddress("glActiveTextureARB");
+    ok(func == NULL, "Function lookup without a context passed, expected a failure; last error %#x\n", GetLastError());
+    wglMakeCurrent(hdc, ctx);
+}
+
 static void test_make_current_read(HDC hdc)
 {
     int res;
@@ -756,7 +904,7 @@ static void test_opengl3(HDC hdc)
         HGLRC gl3Ctx;
         DWORD error;
         gl3Ctx = pwglCreateContextAttribsARB(hdc, (HGLRC)0xdeadbeef, 0);
-        todo_wine ok(gl3Ctx == 0, "pwglCreateContextAttribsARB using an invalid shareList passed\n");
+        ok(gl3Ctx == 0, "pwglCreateContextAttribsARB using an invalid shareList passed\n");
         error = GetLastError();
         /* The Nvidia implementation seems to return hresults instead of win32 error codes */
         todo_wine ok(error == ERROR_INVALID_OPERATION ||
@@ -1305,7 +1453,8 @@ START_TEST(opengl)
         res = SetPixelFormat(hdc, iPixelFormat, &pfd);
         ok(res, "SetPixelformat failed: %x\n", GetLastError());
 
-        test_bitmap_rendering();
+        test_bitmap_rendering( TRUE );
+        test_bitmap_rendering( FALSE );
         test_minimized();
         test_window_dc();
         test_dc(hwnd, hdc);
@@ -1329,6 +1478,10 @@ START_TEST(opengl)
          * any WGL call :( On Wine this would work but not on real Windows because there can be different implementations (software, ICD, MCD).
          */
         init_functions();
+        test_getprocaddress(hdc);
+        test_deletecontext(hwnd, hdc);
+        test_makecurrent(hdc);
+
         /* The lack of wglGetExtensionsStringARB in general means broken software rendering or the lack of decent OpenGL support, skip tests in such cases */
         if (!pwglGetExtensionsStringARB)
         {
@@ -1336,8 +1489,6 @@ START_TEST(opengl)
             return;
         }
 
-        test_deletecontext(hdc);
-        test_makecurrent(hdc);
         test_setpixelformat(hdc);
         test_destroy(hdc);
         test_sharelists(hdc);
