@@ -34,12 +34,10 @@ static DWORD dll_ref = 0;
 /* For the moment, do nothing here. */
 BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpv)
 {
-    if (fdwReason == DLL_PROCESS_DETACH)
-        video_unregister_windowclass();
     return QUARTZ_DllMain( hInstDLL, fdwReason, lpv );
 }
 
-HRESULT SeekingPassThru_create(IUnknown *pUnkOuter, LPVOID *ppObj)
+static HRESULT SeekingPassThru_create(IUnknown *pUnkOuter, LPVOID *ppObj)
 {
     return PosPassThru_Construct(pUnkOuter,ppObj); /* from strmbase */
 }
@@ -73,6 +71,7 @@ static const struct object_creation_info object_creation[] =
     { &CLSID_MPEG1Splitter, MPEGSplitter_create },
     { &CLSID_VideoRenderer, VideoRenderer_create },
     { &CLSID_NullRenderer, NullRenderer_create },
+    { &CLSID_VideoMixingRenderer9, VMR9Impl_create },
     { &CLSID_VideoRendererDefault, VideoRendererDefault_create },
     { &CLSID_DSoundRender, DSoundRender_create },
     { &CLSID_AudioRender, DSoundRender_create },
@@ -125,10 +124,13 @@ static HRESULT WINAPI DSCF_CreateInstance(LPCLASSFACTORY iface, LPUNKNOWN pOuter
     IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
     HRESULT hres;
     LPUNKNOWN punk;
-    
+
     TRACE("(%p)->(%p,%s,%p)\n",This,pOuter,debugstr_guid(riid),ppobj);
 
     *ppobj = NULL;
+    if(pOuter && !IsEqualGUID(&IID_IUnknown, riid))
+        return E_NOINTERFACE;
+
     hres = This->pfnCreateInstance(pOuter, (LPVOID *) &punk);
     if (SUCCEEDED(hres)) {
         hres = IUnknown_QueryInterface(punk, riid, ppobj);
