@@ -19,8 +19,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include <assert.h>
-
 #define COBJMACROS
 #define CONST_VTABLE
 
@@ -1096,6 +1094,7 @@ static HRESULT createtestfilter(const CLSID* pClsid, const TestFilterPinData *pi
     pTestFilter->IBaseFilter_iface.lpVtbl = &TestFilter_Vtbl;
     pTestFilter->refCount = 1;
     InitializeCriticalSection(&pTestFilter->csFilter);
+    pTestFilter->csFilter.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": TestFilterImpl.csFilter");
     pTestFilter->state = State_Stopped;
 
     ZeroMemory(&pTestFilter->filterInfo, sizeof(FILTER_INFO));
@@ -1150,6 +1149,7 @@ static HRESULT createtestfilter(const CLSID* pClsid, const TestFilterPinData *pi
         }
     }
     CoTaskMemFree(pTestFilter->ppPins);
+    pTestFilter->csFilter.DebugInfo->Spare[0] = 0;
     DeleteCriticalSection(&pTestFilter->csFilter);
     CoTaskMemFree(pTestFilter);
 
@@ -1213,6 +1213,7 @@ static ULONG WINAPI TestFilter_Release(IBaseFilter * iface)
 
         CoTaskMemFree(This->ppPins);
 
+        This->csFilter.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&This->csFilter);
 
         CoTaskMemFree(This);
@@ -1797,8 +1798,8 @@ static void test_render_filter_priority(void)
 
     rgf2.dwVersion = 2;
     rgf2.dwMerit = MERIT_UNLIKELY;
-    S1(U(rgf2)).cPins2 = 1;
-    S1(U(rgf2)).rgPins2 = rgPins2;
+    S2(U(rgf2)).cPins2 = 1;
+    S2(U(rgf2)).rgPins2 = rgPins2;
     rgPins2[0].dwFlags = REG_PINFLAG_B_RENDERER;
     rgPins2[0].cInstances = 1;
     rgPins2[0].nMediaTypes = 1;
@@ -1824,7 +1825,7 @@ static void test_render_filter_priority(void)
                     &CLSID_LegacyAmFilterCategory, NULL, &rgf2);
         ok(hr == S_OK, "IFilterMapper2_RegisterFilter failed with %x\n", hr);
 
-        S1(U(rgf2)).cPins2 = 2;
+        S2(U(rgf2)).cPins2 = 2;
         rgPins2[0].dwFlags = 0;
         rgPinType[0].clsMinorType = &mediasubtype1;
 
