@@ -211,7 +211,6 @@ static RETERR16 VCP_VirtnodeCreate(const VCPFILESPEC *vfsSrc, const VCPFILESPEC
 {
     HANDLE heap;
     LPVIRTNODE lpvn;
-    RETERR16 cbres;
 
     while (vn_last < vn_num)
     {
@@ -225,10 +224,10 @@ static RETERR16 VCP_VirtnodeCreate(const VCPFILESPEC *vfsSrc, const VCPFILESPEC
 	vn_num += 20;
 	if (pvnlist)
 	    pvnlist = HeapReAlloc(heap, HEAP_ZERO_MEMORY, pvnlist,
-				sizeof(LPVIRTNODE *) * vn_num);
+				sizeof(*pvnlist) * vn_num);
 	else
 	    pvnlist = HeapAlloc(heap, HEAP_ZERO_MEMORY,
-				sizeof(LPVIRTNODE *) * vn_num);
+				sizeof(*pvnlist) * vn_num);
     }
     pvnlist[vn_last] = HeapAlloc(heap, HEAP_ZERO_MEMORY, sizeof(VIRTNODE));
     lpvn = pvnlist[vn_last];
@@ -248,9 +247,9 @@ static RETERR16 VCP_VirtnodeCreate(const VCPFILESPEC *vfsSrc, const VCPFILESPEC
 
     lpvn->vhstrDstFinalName = 0xffff; /* FIXME: what is this ? */
 
-    cbres = VCP_Callback(lpvn, VCPM_NODECREATE, 0, 0, VCP_MsgRef);
+    VCP_Callback(lpvn, VCPM_NODECREATE, 0, 0, VCP_MsgRef);
     lpvn->fl |= VFNL_CREATED;
-    cbres = VCP_Callback(lpvn, VCPM_NODEACCEPT, 0, 0, VCP_MsgRef);
+    VCP_Callback(lpvn, VCPM_NODEACCEPT, 0, 0, VCP_MsgRef);
 
     return OK;
 }
@@ -420,17 +419,16 @@ static RETERR16 VCP_CheckPaths(void)
 {
     DWORD n;
     LPVIRTNODE lpvn;
-    RETERR16 cbres;
 
-    cbres = VCP_Callback(&vcp_status, VCPM_VSTATPATHCHECKSTART, 0, 0, VCP_MsgRef);
+    VCP_Callback(&vcp_status, VCPM_VSTATPATHCHECKSTART, 0, 0, VCP_MsgRef);
     for (n = 0; n < vn_num; n++)
     {
 	lpvn = pvnlist[n];
 	if (!lpvn) continue;
         /* FIXME: check paths of all VIRTNODEs here ! */
-	cbres = VCP_Callback(&lpvn->vfsDst, VCPM_CHECKPATH, 0, (DWORD)lpvn, VCP_MsgRef);
+	VCP_Callback(&lpvn->vfsDst, VCPM_CHECKPATH, 0, (DWORD)lpvn, VCP_MsgRef);
     }
-    cbres = VCP_Callback(&vcp_status, VCPM_VSTATPATHCHECKEND, 0, 0, VCP_MsgRef);
+    VCP_Callback(&vcp_status, VCPM_VSTATPATHCHECKEND, 0, 0, VCP_MsgRef);
     return OK;
 }
 
@@ -485,7 +483,6 @@ static RETERR16 VCP_CopyFiles(void)
 RETERR16 WINAPI VcpClose16(WORD fl, LPCSTR lpszBackupDest)
 {
     RETERR16 res = OK;
-    WORD cbres = VCPN_PROCEED;
 
     TRACE("(%04x, '%s')\n", fl, lpszBackupDest);
 
@@ -496,7 +493,7 @@ RETERR16 WINAPI VcpClose16(WORD fl, LPCSTR lpszBackupDest)
     memset(&vcp_status, 0, sizeof(VCPSTATUS));
     /* yes, vcp_status.cbSize is 0 ! */
     TRACE("#2\n");
-    cbres = VCP_Callback(&vcp_status, VCPM_VSTATCLOSESTART, 0, 0, VCP_MsgRef);
+    VCP_Callback(&vcp_status, VCPM_VSTATCLOSESTART, 0, 0, VCP_MsgRef);
     TRACE("#3\n");
 
     res = VCP_CheckPaths();
@@ -506,7 +503,7 @@ RETERR16 WINAPI VcpClose16(WORD fl, LPCSTR lpszBackupDest)
     VCP_CopyFiles();
 
     TRACE("#5\n");
-    cbres = VCP_Callback(&vcp_status, VCPM_VSTATCLOSEEND, 0, 0, VCP_MsgRef);
+    VCP_Callback(&vcp_status, VCPM_VSTATCLOSEEND, 0, 0, VCP_MsgRef);
     TRACE("#6\n");
     VCP_Proc = NULL;
     VCP_opened = FALSE;
@@ -565,14 +562,6 @@ VCP_UI_FileCopyWndProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
     if (uMsg != WM_CREATE)
         return DefWindowProcA (hwnd, uMsg, wParam, lParam);
 
-    switch (uMsg)
-    {
-	case WM_CREATE:
-	    return 0;
-	default:
-	    FIXME("%04x: unhandled.\n", uMsg);
-    }
-
     return 0;
 }
 
@@ -602,9 +591,17 @@ static void VCP_UI_RegisterProgressClass(void)
 static RETERR16 VCP_UI_NodeCompare(LPVIRTNODE vn1, LPVIRTNODE vn2)
 {
     LPCSTR file1, file2;
+    int ret;
     file1 = vsmGetStringRawName16(vn1->vfsSrc.vhstrFileName);
     file2 = vsmGetStringRawName16(vn2->vfsSrc.vhstrFileName);
-    return (RETERR16)strcmp(file1, file2);
+
+    ret = strcmp(file1, file2);
+    /* Looks too complicated, but in optimized strcpy we might get
+     * a 32bit wide difference and would truncate it to 16 bit, so
+     * erroneously returning equality. */
+    if (ret < 0) return -1;
+    if (ret > 0) return  1;
+    return 0;
 }
 
 static RETERR16 VCP_UI_CopyStart(void)
