@@ -42,7 +42,6 @@
 
 #include "commctrl.h"
 #include "cpanel.h"
-#include "enumidlist.h"
 #include "pidl.h"
 #include "undocshell.h"
 #include "shell32_main.h"
@@ -59,11 +58,11 @@ WINE_DEFAULT_DEBUG_CHANNEL(shell);
 */
 
 typedef struct {
-    const IShellFolder2Vtbl      *lpVtbl;
-    LONG                   ref;
-    const IPersistFolder2Vtbl    *lpVtblPersistFolder2;
-    const IShellExecuteHookWVtbl *lpVtblShellExecuteHookW;
-    const IShellExecuteHookAVtbl *lpVtblShellExecuteHookA;
+    IShellFolder2      IShellFolder2_iface;
+    IPersistFolder2    IPersistFolder2_iface;
+    IShellExecuteHookW IShellExecuteHookW_iface;
+    IShellExecuteHookA IShellExecuteHookA_iface;
+    LONG               ref;
 
     IUnknown *pUnkOuter;	/* used for aggregation */
 
@@ -77,34 +76,26 @@ static const IPersistFolder2Vtbl vt_PersistFolder2;
 static const IShellExecuteHookWVtbl vt_ShellExecuteHookW;
 static const IShellExecuteHookAVtbl vt_ShellExecuteHookA;
 
-static inline ICPanelImpl *impl_from_IPersistFolder2( IPersistFolder2 *iface )
+static inline ICPanelImpl *impl_from_IShellFolder2(IShellFolder2 *iface)
 {
-    return (ICPanelImpl *)((char*)iface - FIELD_OFFSET(ICPanelImpl, lpVtblPersistFolder2));
+    return CONTAINING_RECORD(iface, ICPanelImpl, IShellFolder2_iface);
 }
 
-static inline ICPanelImpl *impl_from_IShellExecuteHookW( IShellExecuteHookW *iface )
+static inline ICPanelImpl *impl_from_IPersistFolder2(IPersistFolder2 *iface)
 {
-    return (ICPanelImpl *)((char*)iface - FIELD_OFFSET(ICPanelImpl, lpVtblShellExecuteHookW));
+    return CONTAINING_RECORD(iface, ICPanelImpl, IPersistFolder2_iface);
 }
 
-static inline ICPanelImpl *impl_from_IShellExecuteHookA( IShellExecuteHookA *iface )
+static inline ICPanelImpl *impl_from_IShellExecuteHookW(IShellExecuteHookW *iface)
 {
-    return (ICPanelImpl *)((char*)iface - FIELD_OFFSET(ICPanelImpl, lpVtblShellExecuteHookA));
+    return CONTAINING_RECORD(iface, ICPanelImpl, IShellExecuteHookW_iface);
 }
 
+static inline ICPanelImpl *impl_from_IShellExecuteHookA(IShellExecuteHookA *iface)
+{
+    return CONTAINING_RECORD(iface, ICPanelImpl, IShellExecuteHookA_iface);
+}
 
-/*
-  converts This to an interface pointer
-*/
-#define _IUnknown_(This)           ((IUnknown*)&(This)->lpVtbl)
-#define _IShellFolder_(This)       ((IShellFolder*)&(This)->lpVtbl)
-#define _IShellFolder2_(This)      (&(This)->lpVtbl)
-
-#define _IPersist_(This)           (&(This)->lpVtblPersistFolder2)
-#define _IPersistFolder_(This)     (&(This)->lpVtblPersistFolder2)
-#define _IPersistFolder2_(This)    (&(This)->lpVtblPersistFolder2)
-#define _IShellExecuteHookW_(This) (&(This)->lpVtblShellExecuteHookW)
-#define _IShellExecuteHookA_(This) (&(This)->lpVtblShellExecuteHookA)
 
 /***********************************************************************
 *   IShellFolder [ControlPanel] implementation
@@ -136,15 +127,15 @@ HRESULT WINAPI IControlPanel_Constructor(IUnknown* pUnkOuter, REFIID riid, LPVOI
 	return E_OUTOFMEMORY;
 
     sf->ref = 0;
-    sf->lpVtbl = &vt_ShellFolder2;
-    sf->lpVtblPersistFolder2 = &vt_PersistFolder2;
-    sf->lpVtblShellExecuteHookW = &vt_ShellExecuteHookW;
-    sf->lpVtblShellExecuteHookA = &vt_ShellExecuteHookA;
+    sf->IShellFolder2_iface.lpVtbl = &vt_ShellFolder2;
+    sf->IPersistFolder2_iface.lpVtbl = &vt_PersistFolder2;
+    sf->IShellExecuteHookW_iface.lpVtbl = &vt_ShellExecuteHookW;
+    sf->IShellExecuteHookA_iface.lpVtbl = &vt_ShellExecuteHookA;
     sf->pidlRoot = _ILCreateControlPanel();	/* my qualified pidl */
-    sf->pUnkOuter = pUnkOuter ? pUnkOuter : _IUnknown_ (sf);
+    sf->pUnkOuter = pUnkOuter ? pUnkOuter : (IUnknown *)&sf->IShellFolder2_iface;
 
-    if (FAILED(IUnknown_QueryInterface(_IUnknown_(sf), riid, ppv))) {
-	IUnknown_Release(_IUnknown_(sf));
+    if (FAILED(IShellFolder2_QueryInterface(&sf->IShellFolder2_iface, riid, ppv))) {
+        IShellFolder2_Release(&sf->IShellFolder2_iface);
 	return E_NOINTERFACE;
     }
 
@@ -157,9 +148,10 @@ HRESULT WINAPI IControlPanel_Constructor(IUnknown* pUnkOuter, REFIID riid, LPVOI
  *
  * NOTES supports not IPersist/IPersistFolder
  */
-static HRESULT WINAPI ISF_ControlPanel_fnQueryInterface(IShellFolder2 * iface, REFIID riid, LPVOID * ppvObject)
+static HRESULT WINAPI ISF_ControlPanel_fnQueryInterface(IShellFolder2 *iface, REFIID riid,
+        void **ppvObject)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     TRACE("(%p)->(%s,%p)\n", This, shdebugstr_guid(riid), ppvObject);
 
@@ -170,11 +162,11 @@ static HRESULT WINAPI ISF_ControlPanel_fnQueryInterface(IShellFolder2 * iface, R
 	*ppvObject = This;
     else if (IsEqualIID(riid, &IID_IPersist) ||
 	       IsEqualIID(riid, &IID_IPersistFolder) || IsEqualIID(riid, &IID_IPersistFolder2))
-	*ppvObject = _IPersistFolder2_(This);
+        *ppvObject = &This->IPersistFolder2_iface;
     else if (IsEqualIID(riid, &IID_IShellExecuteHookW))
-	*ppvObject = _IShellExecuteHookW_(This);
+        *ppvObject = &This->IShellExecuteHookW_iface;
     else if (IsEqualIID(riid, &IID_IShellExecuteHookA))
-	*ppvObject = _IShellExecuteHookA_(This);
+        *ppvObject = &This->IShellExecuteHookA_iface;
 
     if (*ppvObject) {
 	IUnknown_AddRef((IUnknown *)(*ppvObject));
@@ -185,9 +177,9 @@ static HRESULT WINAPI ISF_ControlPanel_fnQueryInterface(IShellFolder2 * iface, R
     return E_NOINTERFACE;
 }
 
-static ULONG WINAPI ISF_ControlPanel_fnAddRef(IShellFolder2 * iface)
+static ULONG WINAPI ISF_ControlPanel_fnAddRef(IShellFolder2 *iface)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
     ULONG refCount = InterlockedIncrement(&This->ref);
 
     TRACE("(%p)->(count=%u)\n", This, refCount - 1);
@@ -195,9 +187,9 @@ static ULONG WINAPI ISF_ControlPanel_fnAddRef(IShellFolder2 * iface)
     return refCount;
 }
 
-static ULONG WINAPI ISF_ControlPanel_fnRelease(IShellFolder2 * iface)
+static ULONG WINAPI ISF_ControlPanel_fnRelease(IShellFolder2 *iface)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
     ULONG refCount = InterlockedDecrement(&This->ref);
 
     TRACE("(%p)->(count=%u)\n", This, refCount + 1);
@@ -213,14 +205,11 @@ static ULONG WINAPI ISF_ControlPanel_fnRelease(IShellFolder2 * iface)
 /**************************************************************************
 *	ISF_ControlPanel_fnParseDisplayName
 */
-static HRESULT WINAPI
-ISF_ControlPanel_fnParseDisplayName(IShellFolder2 * iface,
-				   HWND hwndOwner,
-				   LPBC pbc,
-				   LPOLESTR lpszDisplayName,
-				   DWORD * pchEaten, LPITEMIDLIST * ppidl, DWORD * pdwAttributes)
+static HRESULT WINAPI ISF_ControlPanel_fnParseDisplayName(IShellFolder2 *iface, HWND hwndOwner,
+        LPBC pbc, LPOLESTR lpszDisplayName, DWORD *pchEaten, LPITEMIDLIST *ppidl,
+        DWORD *pdwAttributes)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     HRESULT hr = E_INVALIDARG;
 
@@ -297,7 +286,7 @@ static PIDLCPanelStruct* _ILGetCPanelPointer(LPCITEMIDLIST pidl)
  /**************************************************************************
  *		ISF_ControlPanel_fnEnumObjects
  */
-static BOOL SHELL_RegisterCPanelApp(IEnumIDList* list, LPCSTR path)
+static BOOL SHELL_RegisterCPanelApp(IEnumIDListImpl *list, LPCSTR path)
 {
     LPITEMIDLIST pidl;
     CPlApplet* applet;
@@ -313,15 +302,15 @@ static BOOL SHELL_RegisterCPanelApp(IEnumIDList* list, LPCSTR path)
 
     MultiByteToWideChar(CP_ACP, 0, path, -1, wpath, MAX_PATH);
 
-    panel.first = NULL;
+    list_init( &panel.applets );
     applet = Control_LoadApplet(0, wpath, &panel);
 
     if (applet)
     {
         for(i=0; i<applet->count; ++i)
         {
-            WideCharToMultiByte(CP_ACP, 0, applet->info[i].szName, -1, displayName, MAX_PATH, 0, 0);
-            WideCharToMultiByte(CP_ACP, 0, applet->info[i].szInfo, -1, comment, MAX_PATH, 0, 0);
+            WideCharToMultiByte(CP_ACP, 0, applet->info[i].name, -1, displayName, MAX_PATH, 0, 0);
+            WideCharToMultiByte(CP_ACP, 0, applet->info[i].info, -1, comment, MAX_PATH, 0, 0);
 
             applet->proc(0, CPL_INQUIRE, i, (LPARAM)&info);
 
@@ -340,7 +329,7 @@ static BOOL SHELL_RegisterCPanelApp(IEnumIDList* list, LPCSTR path)
     return TRUE;
 }
 
-static int SHELL_RegisterRegistryCPanelApps(IEnumIDList* list, HKEY hkey_root, LPCSTR szRepPath)
+static int SHELL_RegisterRegistryCPanelApps(IEnumIDListImpl *list, HKEY hkey_root, LPCSTR szRepPath)
 {
     char name[MAX_PATH];
     char value[MAX_PATH];
@@ -369,7 +358,7 @@ static int SHELL_RegisterRegistryCPanelApps(IEnumIDList* list, HKEY hkey_root, L
     return cnt;
 }
 
-static int SHELL_RegisterCPanelFolders(IEnumIDList* list, HKEY hkey_root, LPCSTR szRepPath)
+static int SHELL_RegisterCPanelFolders(IEnumIDListImpl *list, HKEY hkey_root, LPCSTR szRepPath)
 {
     char name[MAX_PATH];
     HKEY hkey;
@@ -402,19 +391,18 @@ static int SHELL_RegisterCPanelFolders(IEnumIDList* list, HKEY hkey_root, LPCSTR
 /**************************************************************************
  *  CreateCPanelEnumList()
  */
-static BOOL CreateCPanelEnumList(
-    IEnumIDList * iface,
-    DWORD dwFlags)
+static BOOL CreateCPanelEnumList(IEnumIDListImpl *list, DWORD dwFlags)
 {
     CHAR szPath[MAX_PATH];
     WIN32_FIND_DATAA wfd;
     HANDLE hFile;
 
-    TRACE("(%p)->(flags=0x%08x)\n", iface, dwFlags);
+    TRACE("(%p)->(flags=0x%08x)\n", list, dwFlags);
 
     /* enumerate control panel folders */
     if (dwFlags & SHCONTF_FOLDERS)
-        SHELL_RegisterCPanelFolders(iface, HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\ControlPanel\\NameSpace");
+        SHELL_RegisterCPanelFolders(list, HKEY_LOCAL_MACHINE,
+                "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\ControlPanel\\NameSpace");
 
     /* enumerate the control panel applets */
     if (dwFlags & SHCONTF_NONFOLDERS)
@@ -425,7 +413,7 @@ static BOOL CreateCPanelEnumList(
         p = PathAddBackslashA(szPath);
         strcpy(p, "*.cpl");
 
-        TRACE("-- (%p)-> enumerate SHCONTF_NONFOLDERS of %s\n",iface,debugstr_a(szPath));
+        TRACE("-- (%p)-> enumerate SHCONTF_NONFOLDERS of %s\n", list, debugstr_a(szPath));
         hFile = FindFirstFileA(szPath, &wfd);
 
         if (hFile != INVALID_HANDLE_VALUE)
@@ -437,14 +425,16 @@ static BOOL CreateCPanelEnumList(
 
                 if (!(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
                     strcpy(p, wfd.cFileName);
-                    SHELL_RegisterCPanelApp(iface, szPath);
+                    SHELL_RegisterCPanelApp(list, szPath);
                 }
             } while(FindNextFileA(hFile, &wfd));
             FindClose(hFile);
         }
 
-        SHELL_RegisterRegistryCPanelApps(iface, HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Cpls");
-        SHELL_RegisterRegistryCPanelApps(iface, HKEY_CURRENT_USER, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Cpls");
+        SHELL_RegisterRegistryCPanelApps(list, HKEY_LOCAL_MACHINE,
+                "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Cpls");
+        SHELL_RegisterRegistryCPanelApps(list, HKEY_CURRENT_USER,
+                "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Cpls");
     }
     return TRUE;
 }
@@ -452,30 +442,31 @@ static BOOL CreateCPanelEnumList(
 /**************************************************************************
 *		ISF_ControlPanel_fnEnumObjects
 */
-static HRESULT WINAPI
-ISF_ControlPanel_fnEnumObjects(IShellFolder2 * iface, HWND hwndOwner, DWORD dwFlags, LPENUMIDLIST * ppEnumIDList)
+static HRESULT WINAPI ISF_ControlPanel_fnEnumObjects(IShellFolder2 *iface, HWND hwndOwner,
+        DWORD dwFlags, LPENUMIDLIST *ppEnumIDList)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
+    IEnumIDListImpl *list;
 
     TRACE("(%p)->(HWND=%p flags=0x%08x pplist=%p)\n", This, hwndOwner, dwFlags, ppEnumIDList);
 
-    *ppEnumIDList = IEnumIDList_Constructor();
-    if (*ppEnumIDList)
-        CreateCPanelEnumList(*ppEnumIDList, dwFlags);
+    if (!(list = IEnumIDList_Constructor()))
+        return E_OUTOFMEMORY;
+    CreateCPanelEnumList(list, dwFlags);
+    *ppEnumIDList = &list->IEnumIDList_iface;
 
     TRACE("--(%p)->(new ID List: %p)\n", This, *ppEnumIDList);
 
-    return(*ppEnumIDList) ? S_OK : E_OUTOFMEMORY;
+    return S_OK;
 }
 
 /**************************************************************************
 *		ISF_ControlPanel_fnBindToObject
 */
-static HRESULT WINAPI
-ISF_ControlPanel_fnBindToObject(IShellFolder2 * iface, LPCITEMIDLIST pidl,
-			       LPBC pbcReserved, REFIID riid, LPVOID * ppvOut)
+static HRESULT WINAPI ISF_ControlPanel_fnBindToObject(IShellFolder2 *iface, LPCITEMIDLIST pidl,
+        LPBC pbcReserved, REFIID riid, void **ppvOut)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     TRACE("(%p)->(pidl=%p,%p,%s,%p)\n", This, pidl, pbcReserved, shdebugstr_guid(riid), ppvOut);
 
@@ -485,11 +476,10 @@ ISF_ControlPanel_fnBindToObject(IShellFolder2 * iface, LPCITEMIDLIST pidl,
 /**************************************************************************
 *	ISF_ControlPanel_fnBindToStorage
 */
-static HRESULT WINAPI
-ISF_ControlPanel_fnBindToStorage(IShellFolder2 * iface,
-				LPCITEMIDLIST pidl, LPBC pbcReserved, REFIID riid, LPVOID * ppvOut)
+static HRESULT WINAPI ISF_ControlPanel_fnBindToStorage(IShellFolder2 *iface, LPCITEMIDLIST pidl,
+        LPBC pbcReserved, REFIID riid, void **ppvOut)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     FIXME("(%p)->(pidl=%p,%p,%s,%p) stub\n", This, pidl, pbcReserved, shdebugstr_guid(riid), ppvOut);
 
@@ -501,15 +491,15 @@ ISF_ControlPanel_fnBindToStorage(IShellFolder2 * iface,
 * 	ISF_ControlPanel_fnCompareIDs
 */
 
-static HRESULT WINAPI
-ISF_ControlPanel_fnCompareIDs(IShellFolder2 * iface, LPARAM lParam, LPCITEMIDLIST pidl1, LPCITEMIDLIST pidl2)
+static HRESULT WINAPI ISF_ControlPanel_fnCompareIDs(IShellFolder2 *iface, LPARAM lParam,
+        LPCITEMIDLIST pidl1, LPCITEMIDLIST pidl2)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     int nReturn;
 
     TRACE("(%p)->(0x%08lx,pidl1=%p,pidl2=%p)\n", This, lParam, pidl1, pidl2);
-    nReturn = SHELL32_CompareIDs(_IShellFolder_(This), lParam, pidl1, pidl2);
+    nReturn = SHELL32_CompareIDs((IShellFolder *)&This->IShellFolder2_iface, lParam, pidl1, pidl2);
     TRACE("-- %i\n", nReturn);
     return nReturn;
 }
@@ -517,10 +507,10 @@ ISF_ControlPanel_fnCompareIDs(IShellFolder2 * iface, LPARAM lParam, LPCITEMIDLIS
 /**************************************************************************
 *	ISF_ControlPanel_fnCreateViewObject
 */
-static HRESULT WINAPI
-ISF_ControlPanel_fnCreateViewObject(IShellFolder2 * iface, HWND hwndOwner, REFIID riid, LPVOID * ppvOut)
+static HRESULT WINAPI ISF_ControlPanel_fnCreateViewObject(IShellFolder2 *iface, HWND hwndOwner,
+        REFIID riid, void **ppvOut)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     LPSHELLVIEW pShellView;
     HRESULT hr = E_INVALIDARG;
@@ -551,10 +541,10 @@ ISF_ControlPanel_fnCreateViewObject(IShellFolder2 * iface, HWND hwndOwner, REFII
 /**************************************************************************
 *  ISF_ControlPanel_fnGetAttributesOf
 */
-static HRESULT WINAPI
-ISF_ControlPanel_fnGetAttributesOf(IShellFolder2 * iface, UINT cidl, LPCITEMIDLIST * apidl, DWORD * rgfInOut)
+static HRESULT WINAPI ISF_ControlPanel_fnGetAttributesOf(IShellFolder2 *iface, UINT cidl,
+        LPCITEMIDLIST *apidl, DWORD *rgfInOut)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     HRESULT hr = S_OK;
 
@@ -571,7 +561,7 @@ ISF_ControlPanel_fnGetAttributesOf(IShellFolder2 * iface, UINT cidl, LPCITEMIDLI
 
     while(cidl > 0 && *apidl) {
 	pdump(*apidl);
-	SHELL32_GetItemAttributes(_IShellFolder_(This), *apidl, rgfInOut);
+        SHELL32_GetItemAttributes((IShellFolder *)&This->IShellFolder2_iface, *apidl, rgfInOut);
 	apidl++;
 	cidl--;
     }
@@ -594,12 +584,10 @@ ISF_ControlPanel_fnGetAttributesOf(IShellFolder2 * iface, UINT cidl, LPCITEMIDLI
 *  LPVOID*        ppvObject) //[out] Resulting Interface
 *
 */
-static HRESULT WINAPI
-ISF_ControlPanel_fnGetUIObjectOf(IShellFolder2 * iface,
-				HWND hwndOwner,
-				UINT cidl, LPCITEMIDLIST * apidl, REFIID riid, UINT * prgfInOut, LPVOID * ppvOut)
+static HRESULT WINAPI ISF_ControlPanel_fnGetUIObjectOf(IShellFolder2 *iface, HWND hwndOwner,
+        UINT cidl, LPCITEMIDLIST *apidl, REFIID riid, UINT *prgfInOut, void **ppvOut)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     LPITEMIDLIST pidl;
     IUnknown *pObj = NULL;
@@ -612,8 +600,7 @@ ISF_ControlPanel_fnGetUIObjectOf(IShellFolder2 * iface,
 	*ppvOut = NULL;
 
 	if (IsEqualIID(riid, &IID_IContextMenu) &&(cidl >= 1)) {
-	    pObj = (LPUNKNOWN) ISvItemCm_Constructor((IShellFolder *) iface, This->pidlRoot, apidl, cidl);
-	    hr = S_OK;
+	    return ItemMenu_Constructor((IShellFolder*)iface, This->pidlRoot, apidl, cidl, riid, ppvOut);
 	} else if (IsEqualIID(riid, &IID_IDataObject) &&(cidl >= 1)) {
 	    pObj = (LPUNKNOWN) IDataObject_Constructor(hwndOwner, This->pidlRoot, apidl, cidl);
 	    hr = S_OK;
@@ -648,9 +635,10 @@ ISF_ControlPanel_fnGetUIObjectOf(IShellFolder2 * iface,
 /**************************************************************************
 *	ISF_ControlPanel_fnGetDisplayNameOf
 */
-static HRESULT WINAPI ISF_ControlPanel_fnGetDisplayNameOf(IShellFolder2 * iface, LPCITEMIDLIST pidl, DWORD dwFlags, LPSTRRET strRet)
+static HRESULT WINAPI ISF_ControlPanel_fnGetDisplayNameOf(IShellFolder2 *iface, LPCITEMIDLIST pidl,
+        DWORD dwFlags, LPSTRRET strRet)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     CHAR szPath[MAX_PATH];
     WCHAR wszPath[MAX_PATH+1]; /* +1 for potential backslash */
@@ -688,8 +676,7 @@ static HRESULT WINAPI ISF_ControlPanel_fnGetDisplayNameOf(IShellFolder2 * iface,
 	    PathAddBackslashW(wszPath);
 	    len = lstrlenW(wszPath);
 
-	    if (!SUCCEEDED
-	      (SHELL32_GetDisplayNameOfChild(iface, pidl, dwFlags | SHGDN_INFOLDER, wszPath + len, MAX_PATH + 1 - len)))
+            if (FAILED(SHELL32_GetDisplayNameOfChild(iface, pidl, dwFlags | SHGDN_INFOLDER, wszPath + len, MAX_PATH + 1 - len)))
 		return E_OUTOFMEMORY;
 	    if (!WideCharToMultiByte(CP_ACP, 0, wszPath, -1, szPath, MAX_PATH, NULL, NULL))
 		wszPath[0] = '\0';
@@ -715,29 +702,31 @@ static HRESULT WINAPI ISF_ControlPanel_fnGetDisplayNameOf(IShellFolder2 * iface,
 *  DWORD         dwFlags,    //[in ] SHGNO formatting flags
 *  LPITEMIDLIST* ppidlOut)   //[out] simple pidl returned
 */
-static HRESULT WINAPI ISF_ControlPanel_fnSetNameOf(IShellFolder2 * iface, HWND hwndOwner, LPCITEMIDLIST pidl,	/*simple pidl */
-						  LPCOLESTR lpName, DWORD dwFlags, LPITEMIDLIST * pPidlOut)
+static HRESULT WINAPI ISF_ControlPanel_fnSetNameOf(IShellFolder2 *iface, HWND hwndOwner,
+        LPCITEMIDLIST pidl, LPCOLESTR lpName, DWORD dwFlags, LPITEMIDLIST *pPidlOut)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
     FIXME("(%p)->(%p,pidl=%p,%s,%u,%p)\n", This, hwndOwner, pidl, debugstr_w(lpName), dwFlags, pPidlOut);
     return E_FAIL;
 }
 
-static HRESULT WINAPI ISF_ControlPanel_fnGetDefaultSearchGUID(IShellFolder2 * iface, GUID * pguid)
+static HRESULT WINAPI ISF_ControlPanel_fnGetDefaultSearchGUID(IShellFolder2 *iface, GUID *pguid)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
-static HRESULT WINAPI ISF_ControlPanel_fnEnumSearches(IShellFolder2 * iface, IEnumExtraSearch ** ppenum)
+static HRESULT WINAPI ISF_ControlPanel_fnEnumSearches(IShellFolder2 *iface,
+        IEnumExtraSearch **ppenum)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
-static HRESULT WINAPI ISF_ControlPanel_fnGetDefaultColumn(IShellFolder2 * iface, DWORD dwRes, ULONG * pSort, ULONG * pDisplay)
+static HRESULT WINAPI ISF_ControlPanel_fnGetDefaultColumn(IShellFolder2 *iface, DWORD dwRes,
+        ULONG *pSort, ULONG *pDisplay)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     TRACE("(%p)\n", This);
 
@@ -745,9 +734,10 @@ static HRESULT WINAPI ISF_ControlPanel_fnGetDefaultColumn(IShellFolder2 * iface,
     if (pDisplay) *pDisplay = 0;
     return S_OK;
 }
-static HRESULT WINAPI ISF_ControlPanel_fnGetDefaultColumnState(IShellFolder2 * iface, UINT iColumn, DWORD * pcsFlags)
+static HRESULT WINAPI ISF_ControlPanel_fnGetDefaultColumnState(IShellFolder2 *iface, UINT iColumn,
+        DWORD *pcsFlags)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
 
     TRACE("(%p)\n", This);
 
@@ -755,16 +745,18 @@ static HRESULT WINAPI ISF_ControlPanel_fnGetDefaultColumnState(IShellFolder2 * i
     *pcsFlags = ControlPanelSFHeader[iColumn].pcsFlags;
     return S_OK;
 }
-static HRESULT WINAPI ISF_ControlPanel_fnGetDetailsEx(IShellFolder2 * iface, LPCITEMIDLIST pidl, const SHCOLUMNID * pscid, VARIANT * pv)
+static HRESULT WINAPI ISF_ControlPanel_fnGetDetailsEx(IShellFolder2 *iface, LPCITEMIDLIST pidl,
+        const SHCOLUMNID *pscid, VARIANT *pv)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI ISF_ControlPanel_fnGetDetailsOf(IShellFolder2 * iface, LPCITEMIDLIST pidl, UINT iColumn, SHELLDETAILS * psd)
+static HRESULT WINAPI ISF_ControlPanel_fnGetDetailsOf(IShellFolder2 *iface, LPCITEMIDLIST pidl,
+        UINT iColumn, SHELLDETAILS *psd)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
     PIDLCPanelStruct* pcpanel;
     HRESULT hr;
 
@@ -801,9 +793,10 @@ static HRESULT WINAPI ISF_ControlPanel_fnGetDetailsOf(IShellFolder2 * iface, LPC
 
     return hr;
 }
-static HRESULT WINAPI ISF_ControlPanel_fnMapColumnToSCID(IShellFolder2 * iface, UINT column, SHCOLUMNID * pscid)
+static HRESULT WINAPI ISF_ControlPanel_fnMapColumnToSCID(IShellFolder2 *iface, UINT column,
+        SHCOLUMNID *pscid)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellFolder2(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
@@ -844,7 +837,7 @@ static HRESULT WINAPI ICPanel_PersistFolder2_QueryInterface(IPersistFolder2 * if
 
     TRACE("(%p)\n", This);
 
-    return IUnknown_QueryInterface(_IUnknown_(This), iid, ppvObject);
+    return IShellFolder2_QueryInterface(&This->IShellFolder2_iface, iid, ppvObject);
 }
 
 /************************************************************************
@@ -856,7 +849,7 @@ static ULONG WINAPI ICPanel_PersistFolder2_AddRef(IPersistFolder2 * iface)
 
     TRACE("(%p)->(count=%u)\n", This, This->ref);
 
-    return IUnknown_AddRef(_IUnknown_(This));
+    return IShellFolder2_AddRef(&This->IShellFolder2_iface);
 }
 
 /************************************************************************
@@ -868,7 +861,7 @@ static ULONG WINAPI ICPanel_PersistFolder2_Release(IPersistFolder2 * iface)
 
     TRACE("(%p)->(count=%u)\n", This, This->ref);
 
-    return IUnknown_Release(_IUnknown_(This));
+    return IShellFolder2_Release(&This->IShellFolder2_iface);
 }
 
 /************************************************************************
@@ -972,10 +965,11 @@ static ULONG STDMETHODCALLTYPE IShellExecuteHookW_fnRelease(IShellExecuteHookW*
     return IUnknown_Release(This->pUnkOuter);
 }
 
-static HRESULT WINAPI IShellExecuteHookW_fnExecute(IShellExecuteHookW* iface, LPSHELLEXECUTEINFOW psei)
+static HRESULT WINAPI IShellExecuteHookW_fnExecute(IShellExecuteHookW *iface,
+        LPSHELLEXECUTEINFOW psei)
 {
+    ICPanelImpl *This = impl_from_IShellExecuteHookW(iface);
     static const WCHAR wCplopen[] = {'c','p','l','o','p','e','n','\0'};
-    ICPanelImpl *This = (ICPanelImpl *)iface;
 
     SHELLEXECUTEINFOW sei_tmp;
     PIDLCPanelStruct* pcpanel;
@@ -1060,9 +1054,10 @@ static ULONG STDMETHODCALLTYPE IShellExecuteHookA_fnRelease(IShellExecuteHookA*
     return IUnknown_Release(This->pUnkOuter);
 }
 
-static HRESULT WINAPI IShellExecuteHookA_fnExecute(IShellExecuteHookA* iface, LPSHELLEXECUTEINFOA psei)
+static HRESULT WINAPI IShellExecuteHookA_fnExecute(IShellExecuteHookA *iface,
+        LPSHELLEXECUTEINFOA psei)
 {
-    ICPanelImpl *This = (ICPanelImpl *)iface;
+    ICPanelImpl *This = impl_from_IShellExecuteHookA(iface);
 
     SHELLEXECUTEINFOA sei_tmp;
     PIDLCPanelStruct* pcpanel;
