@@ -37,6 +37,7 @@
 #include "winuser.h"
 #include "wingdi.h"
 #include "shlobj.h"
+#include "rpcproxy.h"
 #include "shlwapi.h"
 #include "propsys.h"
 
@@ -45,6 +46,8 @@
 #include "shell32_main.h"
 #include "version.h"
 #include "shresdef.h"
+#include "initguid.h"
+#include "shfldr.h"
 
 #include "wine/debug.h"
 #include "wine/unicode.h"
@@ -89,6 +92,12 @@ LPWSTR* WINAPI CommandLineToArgvW(LPCWSTR lpCmdline, int* numargs)
     LPWSTR cmdline;
     int in_quotes,bcount;
 
+    if(!numargs)
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return NULL;
+    }
+
     if (*lpCmdline==0)
     {
         /* Return the path to the executable */
@@ -110,8 +119,7 @@ LPWSTR* WINAPI CommandLineToArgvW(LPCWSTR lpCmdline, int* numargs)
             LocalFree( argv );
         }
         argv[0]=(LPWSTR)(argv+1);
-        if (numargs)
-            *numargs=1;
+        *numargs=1;
 
         return argv;
     }
@@ -225,8 +233,7 @@ LPWSTR* WINAPI CommandLineToArgvW(LPCWSTR lpCmdline, int* numargs)
         *d='\0';
         argv[argc++]=arg;
     }
-    if (numargs)
-        *numargs=argc;
+    *numargs=argc;
 
     return argv;
 }
@@ -675,7 +682,7 @@ DWORD_PTR WINAPI SHGetFileInfoA(LPCSTR path,DWORD dwFileAttributes,
     INT len;
     LPWSTR temppath = NULL;
     LPCWSTR pathW;
-    DWORD ret;
+    DWORD_PTR ret;
     SHFILEINFOW temppsfi;
 
     if (flags & SHGFI_PIDL)
@@ -925,7 +932,7 @@ HRESULT WINAPI SHLoadInProc (REFCLSID rclsid)
     {
         IUnknown * pUnk = ptr;
         IUnknown_Release(pUnk);
-        return NOERROR;
+        return S_OK;
     }
     return DISP_E_MEMBERNOTFOUND;
 }
@@ -1176,7 +1183,7 @@ HIMAGELIST   ShellBigIconList = 0;
  * SHELL32 DllMain
  *
  * NOTES
- *  calling oleinitialize here breaks sone apps.
+ *  calling oleinitialize here breaks some apps.
  */
 BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID fImpLoad)
 {
@@ -1230,6 +1237,24 @@ HRESULT WINAPI DllCanUnloadNow(void)
     return S_FALSE;
 }
 
+/***********************************************************************
+ *		DllRegisterServer (SHELL32.@)
+ */
+HRESULT WINAPI DllRegisterServer(void)
+{
+    HRESULT hr = __wine_register_resources( shell32_hInstance );
+    if (SUCCEEDED(hr)) hr = SHELL_RegisterShellFolders();
+    return hr;
+}
+
+/***********************************************************************
+ *		DllUnregisterServer (SHELL32.@)
+ */
+HRESULT WINAPI DllUnregisterServer(void)
+{
+    return __wine_unregister_resources( shell32_hInstance );
+}
+
 /***********************************************************************
  *              ExtractVersionResource16W (SHELL32.@)
  */
@@ -1257,8 +1282,29 @@ LRESULT CALLBACK ShellHookProc(DWORD a, DWORD b, DWORD c)
     return 0;
 }
 
+/***********************************************************************
+ *              SHGetLocalizedName (SHELL32.@)
+ */
 HRESULT WINAPI SHGetLocalizedName(LPCWSTR path, LPWSTR module, UINT size, INT *res)
 {
     FIXME("%s %p %u %p: stub\n", debugstr_w(path), module, size, res);
     return E_NOTIMPL;
 }
+
+/***********************************************************************
+ *              SetCurrentProcessExplicitAppUserModelID (SHELL32.@)
+ */
+HRESULT WINAPI SetCurrentProcessExplicitAppUserModelID(PCWSTR appid)
+{
+    FIXME("%s: stub\n", debugstr_w(appid));
+    return E_NOTIMPL;
+}
+
+/***********************************************************************
+ *              SHSetUnreadMailCountW (SHELL32.@)
+ */
+HRESULT WINAPI SHSetUnreadMailCountW(LPCWSTR mailaddress, DWORD count, LPCWSTR executecommand)
+{
+    FIXME("%s %x %s: stub\n", debugstr_w(mailaddress), count, debugstr_w(executecommand));
+    return E_NOTIMPL;
+}
