@@ -92,6 +92,7 @@ typedef struct {
     BOOL use_cache_file;
     DWORD state;
     HRESULT hres;
+    CLSID clsid;
     download_state_t download_state;
     IUnknown *obj;
     IMoniker *mon;
@@ -178,8 +179,8 @@ static void stop_binding(Binding *binding, HRESULT hres, LPCWSTR str)
     if(!(binding->state & BINDING_STOPPED)) {
         binding->state |= BINDING_STOPPED;
 
-        IBindStatusCallback_OnStopBinding(binding->callback, hres, str);
         binding->hres = hres;
+        IBindStatusCallback_OnStopBinding(binding->callback, hres, str);
     }
 }
 
@@ -310,6 +311,7 @@ static void create_object(Binding *binding)
     heap_free(clsid_str);
 
     IBindStatusCallback_OnProgress(binding->callback, 0, 0, BINDSTATUS_ENDSYNCOPERATION, NULL);
+    binding->clsid = CLSID_NULL;
 
     stop_binding(binding, hres, NULL);
     if(FAILED(hres))
@@ -881,8 +883,23 @@ static HRESULT WINAPI Binding_GetBindResult(IBinding *iface, CLSID *pclsidProtoc
         DWORD *pdwResult, LPOLESTR *pszResult, DWORD *pdwReserved)
 {
     Binding *This = impl_from_IBinding(iface);
-    FIXME("(%p)->(%p %p %p %p)\n", This, pclsidProtocol, pdwResult, pszResult, pdwReserved);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p %p %p %p)\n", This, pclsidProtocol, pdwResult, pszResult, pdwReserved);
+
+    if(!pdwResult || !pszResult || pdwReserved)
+        return E_INVALIDARG;
+
+    if(!(This->state & BINDING_STOPPED)) {
+        *pclsidProtocol = CLSID_NULL;
+        *pdwResult = 0;
+        *pszResult = NULL;
+        return S_OK;
+    }
+
+    *pclsidProtocol = This->hres==S_OK ? CLSID_NULL : This->clsid;
+    *pdwResult = This->hres;
+    *pszResult = NULL;
+    return S_OK;
 }
 
 static const IBindingVtbl BindingVtbl = {
@@ -983,6 +1000,7 @@ static HRESULT WINAPI InternetProtocolSink_ReportProgress(IInternetProtocolSink
         on_progress(This, 0, 0, BINDSTATUS_SENDINGREQUEST, szStatusText);
         break;
     case BINDSTATUS_PROTOCOLCLASSID:
+        CLSIDFromString(szStatusText, &This->clsid);
         break;
     case BINDSTATUS_MIMETYPEAVAILABLE:
     case BINDSTATUS_VERIFIEDMIMETYPEAVAILABLE:
@@ -1077,8 +1095,19 @@ static void report_data(Binding *This, DWORD bscf, ULONG progress, ULONG progres
         formatetc.tymed = stgmed.tymed;
         formatetc.cfFormat = This->clipboard_format;
 
-        IBindStatusCallback_OnDataAvailable(This->callback, bscf, progress,
+        hres = IBindStatusCallback_OnDataAvailable(This->callback, bscf, progress,
                 &formatetc, &stgmed);
+        if(hres != S_OK) {
+            if(This->download_state != END_DOWNLOAD) {
+                This->download_state = END_DOWNLOAD;
+                IBindStatusCallback_OnProgress(This->callback, progress, progress_max,
+                        BINDSTATUS_ENDDOWNLOADDATA, This->url);
+            }
+
+            WARN("OnDataAvailable returned %x\n", hres);
+            stop_binding(This, hres, NULL);
+            return;
+        }
 
         if(This->download_state == END_DOWNLOAD)
             stop_binding(This, S_OK, NULL);
@@ -1150,19 +1179,7 @@ static HRESULT WINAPI InternetBindInfo_GetBindInfo(IInternetBindInfo *iface,
     TRACE("(%p)->(%p %p)\n", This, grfBINDF, pbindinfo);
 
     *grfBINDF = This->bindf;
-
-    *pbindinfo = This->bindinfo;
-
-    if(pbindinfo->szExtraInfo || pbindinfo->szCustomVerb)
-        FIXME("copy strings\n");
-
-    if(pbindinfo->stgmedData.pUnkForRelease)
-        IUnknown_AddRef(pbindinfo->stgmedData.pUnkForRelease);
-
-    if(pbindinfo->pUnk)
-        IUnknown_AddRef(pbindinfo->pUnk);
-
-    return S_OK;
+    return CopyBindInfo(&This->bindinfo, pbindinfo);
 }
 
 static HRESULT WINAPI InternetBindInfo_GetBindString(IInternetBindInfo *iface,
@@ -1494,7 +1511,10 @@ static HRESULT start_binding(IMoniker *mon, Binding *binding_ctx, IUri *uri, IBi
     hres = IBindStatusCallback_OnStartBinding(binding->callback, 0, &binding->IBinding_iface);
     if(FAILED(hres)) {
         WARN("OnStartBinding failed: %08x\n", hres);
-        stop_binding(binding, INET_E_DOWNLOAD_FAILURE, NULL);
+        if(hres != E_ABORT && hres != E_NOTIMPL)
+            hres = INET_E_DOWNLOAD_FAILURE;
+
+        stop_binding(binding, hres, NULL);
         IBinding_Release(&binding->IBinding_iface);
         return hres;
     }
