@@ -24,6 +24,8 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(vbscript);
 
+static DISPID propput_dispid = DISPID_PROPERTYPUT;
+
 typedef struct {
     vbscode_t *code;
     instr_t *instr;
@@ -164,16 +166,6 @@ static HRESULT lookup_identifier(exec_ctx_t *ctx, BSTR name, vbdisp_invoke_type_
     }
 
     LIST_FOR_EACH_ENTRY(item, &ctx->script->named_items, named_item_t, entry) {
-        if((item->flags & SCRIPTITEM_GLOBALMEMBERS)) {
-            hres = disp_get_id(item->disp, name, invoke_type, FALSE, &id);
-            if(SUCCEEDED(hres)) {
-                ref->type = REF_DISP;
-                ref->u.d.disp = item->disp;
-                ref->u.d.id = id;
-                return S_OK;
-            }
-        }
-
         if((item->flags & SCRIPTITEM_ISVISIBLE) && !strcmpiW(item->name, name)) {
             if(!item->disp) {
                 IUnknown *unk;
@@ -198,6 +190,18 @@ static HRESULT lookup_identifier(exec_ctx_t *ctx, BSTR name, vbdisp_invoke_type_
         }
     }
 
+    LIST_FOR_EACH_ENTRY(item, &ctx->script->named_items, named_item_t, entry) {
+        if((item->flags & SCRIPTITEM_GLOBALMEMBERS)) {
+            hres = disp_get_id(item->disp, name, invoke_type, FALSE, &id);
+            if(SUCCEEDED(hres)) {
+                ref->type = REF_DISP;
+                ref->u.d.disp = item->disp;
+                ref->u.d.id = id;
+                return S_OK;
+            }
+        }
+    }
+
     ref->type = REF_NONE;
     return S_OK;
 }
@@ -227,6 +231,7 @@ static HRESULT add_dynamic_var(exec_ctx_t *ctx, const WCHAR *name, BOOL is_const
     if(own_val) {
         new_var->v = *val;
     }else {
+        V_VT(&new_var->v) = VT_EMPTY;
         hres = VariantCopy(&new_var->v, val);
         if(FAILED(hres))
             return hres;
@@ -312,12 +317,69 @@ static HRESULT stack_pop_val(exec_ctx_t *ctx, variant_val_t *v)
     return S_OK;
 }
 
+static HRESULT stack_assume_val(exec_ctx_t *ctx, unsigned n)
+{
+    VARIANT *v = stack_top(ctx, n);
+    HRESULT hres;
+
+    if(V_VT(v) == (VT_BYREF|VT_VARIANT)) {
+        VARIANT *ref = V_VARIANTREF(v);
+
+        V_VT(v) = VT_EMPTY;
+        hres = VariantCopy(v, ref);
+        if(FAILED(hres))
+            return hres;
+    }
+
+    if(V_VT(v) == VT_DISPATCH) {
+        DISPPARAMS dp = {0};
+        IDispatch *disp;
+
+        disp = V_DISPATCH(v);
+        V_VT(v) = VT_EMPTY;
+        hres = disp_call(ctx->script, disp, DISPID_VALUE, &dp, v);
+        IDispatch_Release(disp);
+        if(FAILED(hres))
+            return hres;
+    }
+
+    return S_OK;
+}
+
 static inline void release_val(variant_val_t *v)
 {
     if(v->owned)
         VariantClear(v->v);
 }
 
+static int stack_pop_bool(exec_ctx_t *ctx, BOOL *b)
+{
+    variant_val_t val;
+    HRESULT hres;
+
+    hres = stack_pop_val(ctx, &val);
+    if(FAILED(hres))
+        return hres;
+
+    switch (V_VT(val.v))
+    {
+    case VT_BOOL:
+        *b = V_BOOL(val.v);
+        break;
+    case VT_I2:
+        *b = V_I2(val.v);
+        break;
+    case VT_I4:
+        *b = V_I4(val.v);
+        break;
+    default:
+        FIXME("unsupported for %s\n", debugstr_variant(val.v));
+        release_val(&val);
+        return E_NOTIMPL;
+    }
+    return S_OK;
+}
+
 static HRESULT stack_pop_disp(exec_ctx_t *ctx, IDispatch **ret)
 {
     VARIANT *v = stack_pop(ctx);
@@ -345,16 +407,43 @@ static HRESULT stack_pop_disp(exec_ctx_t *ctx, IDispatch **ret)
     return S_OK;
 }
 
+static HRESULT stack_assume_disp(exec_ctx_t *ctx, unsigned n, IDispatch **disp)
+{
+    VARIANT *v = stack_top(ctx, n), *ref;
+
+    if(V_VT(v) != VT_DISPATCH) {
+        if(V_VT(v) != (VT_VARIANT|VT_BYREF)) {
+            FIXME("not supported type: %s\n", debugstr_variant(v));
+            return E_FAIL;
+        }
+
+        ref = V_VARIANTREF(v);
+        if(V_VT(ref) != VT_DISPATCH) {
+            FIXME("not disp %s\n", debugstr_variant(ref));
+            return E_FAIL;
+        }
+
+        V_VT(v) = VT_DISPATCH;
+        V_DISPATCH(v) = V_DISPATCH(ref);
+        if(V_DISPATCH(v))
+            IDispatch_AddRef(V_DISPATCH(v));
+    }
+
+    if(disp)
+        *disp = V_DISPATCH(v);
+    return S_OK;
+}
+
 static inline void instr_jmp(exec_ctx_t *ctx, unsigned addr)
 {
     ctx->instr = ctx->code->instrs + addr;
 }
 
-static void vbstack_to_dp(exec_ctx_t *ctx, unsigned arg_cnt, DISPPARAMS *dp)
+static void vbstack_to_dp(exec_ctx_t *ctx, unsigned arg_cnt, BOOL is_propput, DISPPARAMS *dp)
 {
-    dp->cArgs = arg_cnt;
-    dp->rgdispidNamedArgs = NULL;
-    dp->cNamedArgs = 0;
+    dp->cNamedArgs = is_propput ? 1 : 0;
+    dp->cArgs = arg_cnt + dp->cNamedArgs;
+    dp->rgdispidNamedArgs = is_propput ? &propput_dispid : NULL;
 
     if(arg_cnt) {
         VARIANT tmp;
@@ -368,9 +457,9 @@ static void vbstack_to_dp(exec_ctx_t *ctx, unsigned arg_cnt, DISPPARAMS *dp)
             ctx->stack[ctx->top-arg_cnt+i-1] = tmp;
         }
 
-        dp->rgvarg = ctx->stack + ctx->top-arg_cnt;
+        dp->rgvarg = ctx->stack + ctx->top-dp->cArgs;
     }else {
-        dp->rgvarg = NULL;
+        dp->rgvarg = is_propput ? ctx->stack+ctx->top-1 : NULL;
     }
 }
 
@@ -386,7 +475,7 @@ static HRESULT do_icall(exec_ctx_t *ctx, VARIANT *res)
     if(FAILED(hres))
         return hres;
 
-    vbstack_to_dp(ctx, arg_cnt, &dp);
+    vbstack_to_dp(ctx, arg_cnt, FALSE, &dp);
 
     switch(ref.type) {
     case REF_VAR:
@@ -473,7 +562,7 @@ static HRESULT do_mcall(exec_ctx_t *ctx, VARIANT *res)
         return E_FAIL;
     }
 
-    vbstack_to_dp(ctx, arg_cnt, &dp);
+    vbstack_to_dp(ctx, arg_cnt, FALSE, &dp);
 
     hres = disp_get_id(obj, identifier, VBDISP_CALLGET, FALSE, &id);
     if(SUCCEEDED(hres))
@@ -507,7 +596,7 @@ static HRESULT interp_mcallv(exec_ctx_t *ctx)
     return do_mcall(ctx, NULL);
 }
 
-static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, VARIANT *val, BOOL own_val)
+static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, DISPPARAMS *dp)
 {
     ref_t ref;
     HRESULT hres;
@@ -520,22 +609,19 @@ static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, VARIANT *val, BOOL own_v
     case REF_VAR: {
         VARIANT *v = ref.u.v;
 
+        if(arg_cnt(dp)) {
+            FIXME("arg_cnt %d not supported\n", arg_cnt(dp));
+            return E_NOTIMPL;
+        }
+
         if(V_VT(v) == (VT_VARIANT|VT_BYREF))
             v = V_VARIANTREF(v);
 
-        if(own_val) {
-            VariantClear(v);
-            *v = *val;
-            hres = S_OK;
-        }else {
-            hres = VariantCopy(v, val);
-        }
+        hres = VariantCopy(v, dp->rgvarg);
         break;
     }
     case REF_DISP:
-        hres = disp_propput(ctx->script, ref.u.d.disp, ref.u.d.id, val);
-        if(own_val)
-            VariantClear(val);
+        hres = disp_propput(ctx->script, ref.u.d.disp, ref.u.d.id, dp);
         break;
     case REF_FUNC:
         FIXME("functions not implemented\n");
@@ -551,8 +637,13 @@ static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, VARIANT *val, BOOL own_v
             FIXME("throw exception\n");
             hres = E_FAIL;
         }else {
+            if(arg_cnt(dp)) {
+                FIXME("arg_cnt %d not supported\n", arg_cnt(dp));
+                return E_NOTIMPL;
+            }
+
             TRACE("creating variable %s\n", debugstr_w(name));
-            hres = add_dynamic_var(ctx, name, FALSE, val, own_val);
+            hres = add_dynamic_var(ctx, name, FALSE, dp->rgvarg, FALSE);
         }
     }
 
@@ -563,29 +654,29 @@ static HRESULT interp_assign_ident(exec_ctx_t *ctx)
 {
     const BSTR arg = ctx->instr->arg1.bstr;
     const unsigned arg_cnt = ctx->instr->arg2.uint;
-    variant_val_t v;
+    DISPPARAMS dp;
     HRESULT hres;
 
     TRACE("%s\n", debugstr_w(arg));
 
-    if(arg_cnt) {
-        FIXME("arguments not supported\n");
-        return E_NOTIMPL;
-    }
-
-    hres = stack_pop_val(ctx, &v);
+    hres = stack_assume_val(ctx, arg_cnt);
     if(FAILED(hres))
         return hres;
 
-    return assign_ident(ctx, arg, v.v, v.owned);
+    vbstack_to_dp(ctx, arg_cnt, TRUE, &dp);
+    hres = assign_ident(ctx, arg, &dp);
+    if(FAILED(hres))
+        return hres;
+
+    stack_popn(ctx, arg_cnt+1);
+    return S_OK;
 }
 
 static HRESULT interp_set_ident(exec_ctx_t *ctx)
 {
     const BSTR arg = ctx->instr->arg1.bstr;
     const unsigned arg_cnt = ctx->instr->arg2.uint;
-    IDispatch *disp;
-    VARIANT v;
+    DISPPARAMS dp;
     HRESULT hres;
 
     TRACE("%s\n", debugstr_w(arg));
@@ -595,32 +686,31 @@ static HRESULT interp_set_ident(exec_ctx_t *ctx)
         return E_NOTIMPL;
     }
 
-    hres = stack_pop_disp(ctx, &disp);
+    hres = stack_assume_disp(ctx, 0, NULL);
     if(FAILED(hres))
         return hres;
 
-    V_VT(&v) = VT_DISPATCH;
-    V_DISPATCH(&v) = disp;
-    return assign_ident(ctx, ctx->instr->arg1.bstr, &v, TRUE);
+    vbstack_to_dp(ctx, 0, TRUE, &dp);
+    hres = assign_ident(ctx, ctx->instr->arg1.bstr, &dp);
+    if(FAILED(hres))
+        return hres;
+
+    stack_popn(ctx, 1);
+    return S_OK;
 }
 
 static HRESULT interp_assign_member(exec_ctx_t *ctx)
 {
     BSTR identifier = ctx->instr->arg1.bstr;
     const unsigned arg_cnt = ctx->instr->arg2.uint;
-    variant_val_t val;
     IDispatch *obj;
+    DISPPARAMS dp;
     DISPID id;
     HRESULT hres;
 
     TRACE("%s\n", debugstr_w(identifier));
 
-    if(arg_cnt) {
-        FIXME("arguments not supported\n");
-        return E_NOTIMPL;
-    }
-
-    hres = stack_pop_disp(ctx, &obj);
+    hres = stack_assume_disp(ctx, arg_cnt+1, &obj);
     if(FAILED(hres))
         return hres;
 
@@ -629,26 +719,28 @@ static HRESULT interp_assign_member(exec_ctx_t *ctx)
         return E_FAIL;
     }
 
-    hres = stack_pop_val(ctx, &val);
-    if(FAILED(hres)) {
-        IDispatch_Release(obj);
+    hres = stack_assume_val(ctx, arg_cnt);
+    if(FAILED(hres))
         return hres;
-    }
 
     hres = disp_get_id(obj, identifier, VBDISP_LET, FALSE, &id);
-    if(SUCCEEDED(hres))
-        hres = disp_propput(ctx->script, obj, id, val.v);
+    if(SUCCEEDED(hres)) {
+        vbstack_to_dp(ctx, arg_cnt, TRUE, &dp);
+        hres = disp_propput(ctx->script, obj, id, &dp);
+    }
+    if(FAILED(hres))
+        return hres;
 
-    release_val(&val);
-    IDispatch_Release(obj);
-    return hres;
+    stack_popn(ctx, arg_cnt+2);
+    return S_OK;
 }
 
 static HRESULT interp_set_member(exec_ctx_t *ctx)
 {
     BSTR identifier = ctx->instr->arg1.bstr;
     const unsigned arg_cnt = ctx->instr->arg2.uint;
-    IDispatch *obj, *val;
+    IDispatch *obj;
+    DISPPARAMS dp;
     DISPID id;
     HRESULT hres;
 
@@ -659,7 +751,7 @@ static HRESULT interp_set_member(exec_ctx_t *ctx)
         return E_NOTIMPL;
     }
 
-    hres = stack_pop_disp(ctx, &obj);
+    hres = stack_assume_disp(ctx, 1, &obj);
     if(FAILED(hres))
         return hres;
 
@@ -668,25 +760,20 @@ static HRESULT interp_set_member(exec_ctx_t *ctx)
         return E_FAIL;
     }
 
-    hres = stack_pop_disp(ctx, &val);
-    if(FAILED(hres)) {
-        IDispatch_Release(obj);
+    hres = stack_assume_disp(ctx, 0, NULL);
+    if(FAILED(hres))
         return hres;
-    }
 
     hres = disp_get_id(obj, identifier, VBDISP_SET, FALSE, &id);
     if(SUCCEEDED(hres)) {
-        VARIANT v;
-
-        V_VT(&v) = VT_DISPATCH;
-        V_DISPATCH(&v) = val;
-        hres = disp_propput(ctx->script, obj, id, &v);
+        vbstack_to_dp(ctx, arg_cnt, TRUE, &dp);
+        hres = disp_propput(ctx->script, obj, id, &dp);
     }
+    if(FAILED(hres))
+        return hres;
 
-    if(val)
-        IDispatch_Release(val);
-    IDispatch_Release(obj);
-    return hres;
+    stack_popn(ctx, 2);
+    return S_OK;
 }
 
 static HRESULT interp_const(exec_ctx_t *ctx)
@@ -807,10 +894,92 @@ static HRESULT interp_step(exec_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    if(hres == VARCMP_EQ || hres == (gteq_zero ? VARCMP_LT : VARCMP_GT))
+    if(hres == VARCMP_EQ || hres == (gteq_zero ? VARCMP_LT : VARCMP_GT)) {
         ctx->instr++;
-    else
+    }else {
+        stack_popn(ctx, 2);
         instr_jmp(ctx, ctx->instr->arg1.uint);
+    }
+    return S_OK;
+}
+
+static HRESULT interp_newenum(exec_ctx_t *ctx)
+{
+    VARIANT *v, r;
+    HRESULT hres;
+
+    TRACE("\n");
+
+    v = stack_pop(ctx);
+    switch(V_VT(v)) {
+    case VT_DISPATCH: {
+        IEnumVARIANT *iter;
+        DISPPARAMS dp = {0};
+        VARIANT iterv;
+
+        hres = disp_call(ctx->script, V_DISPATCH(v), DISPID_NEWENUM, &dp, &iterv);
+        VariantClear(v);
+        if(FAILED(hres))
+            return hres;
+
+        if(V_VT(&iterv) != VT_UNKNOWN && V_VT(&iterv) != VT_DISPATCH) {
+            FIXME("Unsupported iterv %s\n", debugstr_variant(&iterv));
+            VariantClear(&iterv);
+            return hres;
+        }
+
+        hres = IUnknown_QueryInterface(V_UNKNOWN(&iterv), &IID_IEnumVARIANT, (void**)&iter);
+        IUnknown_Release(V_UNKNOWN(&iterv));
+        if(FAILED(hres)) {
+            FIXME("Could not get IEnumVARIANT iface: %08x\n", hres);
+            return hres;
+        }
+
+        V_VT(&r) = VT_UNKNOWN;
+        V_UNKNOWN(&r) = (IUnknown*)iter;
+        break;
+    }
+    default:
+        FIXME("Unsupported for %s\n", debugstr_variant(v));
+        VariantClear(v);
+        return E_NOTIMPL;
+    }
+
+    return stack_push(ctx, &r);
+}
+
+static HRESULT interp_enumnext(exec_ctx_t *ctx)
+{
+    const unsigned loop_end = ctx->instr->arg1.uint;
+    const BSTR ident = ctx->instr->arg2.bstr;
+    VARIANT v;
+    DISPPARAMS dp = {&v, &propput_dispid, 1, 1};
+    IEnumVARIANT *iter;
+    BOOL do_continue;
+    HRESULT hres;
+
+    TRACE("\n");
+
+    assert(V_VT(stack_top(ctx, 0)) == VT_UNKNOWN);
+    iter = (IEnumVARIANT*)V_UNKNOWN(stack_top(ctx, 0));
+
+    V_VT(&v) = VT_EMPTY;
+    hres = IEnumVARIANT_Next(iter, 1, &v, NULL);
+    if(FAILED(hres))
+        return hres;
+
+    do_continue = hres == S_OK;
+    hres = assign_ident(ctx, ident, &dp);
+    VariantClear(&v);
+    if(FAILED(hres))
+        return hres;
+
+    if(do_continue) {
+        ctx->instr++;
+    }else {
+        stack_pop(ctx);
+        instr_jmp(ctx, loop_end);
+    }
     return S_OK;
 }
 
@@ -827,22 +996,16 @@ static HRESULT interp_jmp(exec_ctx_t *ctx)
 static HRESULT interp_jmp_false(exec_ctx_t *ctx)
 {
     const unsigned arg = ctx->instr->arg1.uint;
-    variant_val_t val;
     HRESULT hres;
+    BOOL b;
 
     TRACE("%u\n", arg);
 
-    hres = stack_pop_val(ctx, &val);
+    hres = stack_pop_bool(ctx, &b);
     if(FAILED(hres))
         return hres;
 
-    if(V_VT(val.v) != VT_BOOL) {
-        FIXME("unsupported for %s\n", debugstr_variant(val.v));
-        release_val(&val);
-        return E_NOTIMPL;
-    }
-
-    if(V_BOOL(val.v))
+    if(b)
         ctx->instr++;
     else
         instr_jmp(ctx, ctx->instr->arg1.uint);
@@ -852,22 +1015,16 @@ static HRESULT interp_jmp_false(exec_ctx_t *ctx)
 static HRESULT interp_jmp_true(exec_ctx_t *ctx)
 {
     const unsigned arg = ctx->instr->arg1.uint;
-    variant_val_t val;
     HRESULT hres;
+    BOOL b;
 
     TRACE("%u\n", arg);
 
-    hres = stack_pop_val(ctx, &val);
+    hres = stack_pop_bool(ctx, &b);
     if(FAILED(hres))
         return hres;
 
-    if(V_VT(val.v) != VT_BOOL) {
-        FIXME("unsupported for %s\n", debugstr_variant(val.v));
-        release_val(&val);
-        return E_NOTIMPL;
-    }
-
-    if(V_BOOL(val.v))
+    if(b)
         instr_jmp(ctx, ctx->instr->arg1.uint);
     else
         ctx->instr++;
