@@ -16,12 +16,16 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include <assert.h>
+
 #include "vbscript.h"
 
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(vbscript);
 
+#define FDEX_VERSION_MASK 0xf0000000
+
 static inline BOOL is_func_id(vbdisp_t *This, DISPID id)
 {
     return id < This->desc->func_cnt;
@@ -133,6 +137,9 @@ static HRESULT invoke_variant_prop(vbdisp_t *This, VARIANT *v, WORD flags, DISPP
 
 static HRESULT invoke_builtin(vbdisp_t *This, const builtin_prop_t *prop, WORD flags, DISPPARAMS *dp, VARIANT *res)
 {
+    VARIANT *args, arg_buf[8];
+    unsigned argn;
+
     switch(flags) {
     case DISPATCH_PROPERTYGET:
         if(!(prop->flags & (BP_GET|BP_GETPUT))) {
@@ -161,12 +168,38 @@ static HRESULT invoke_builtin(vbdisp_t *This, const builtin_prop_t *prop, WORD f
         return E_NOTIMPL;
     }
 
-    if(arg_cnt(dp) < prop->min_args || arg_cnt(dp) > (prop->max_args ? prop->max_args : prop->min_args)) {
+    argn = arg_cnt(dp);
+
+    if(argn < prop->min_args || argn > (prop->max_args ? prop->max_args : prop->min_args)) {
         FIXME("invalid number of arguments\n");
         return E_FAIL;
     }
 
-    return prop->proc(This, dp->rgvarg, dp->cArgs, res);
+    args = dp->rgvarg;
+
+    if(argn == 1) {
+        if(V_VT(dp->rgvarg) == (VT_BYREF|VT_VARIANT))
+            args = V_VARIANTREF(dp->rgvarg);
+    }else {
+        unsigned i;
+
+        assert(argn < sizeof(arg_buf)/sizeof(*arg_buf));
+
+        for(i=0; i < argn; i++) {
+            if(V_VT(dp->rgvarg+i) == (VT_BYREF|VT_VARIANT)) {
+                for(i=0; i < argn; i++) {
+                    if(V_VT(dp->rgvarg+i) == (VT_BYREF|VT_VARIANT))
+                        arg_buf[i] = *V_VARIANTREF(dp->rgvarg+i);
+                    else
+                        arg_buf[i] = dp->rgvarg[i];
+                }
+                args = arg_buf;
+                break;
+            }
+        }
+    }
+
+    return prop->proc(This, args, dp->cArgs, res);
 }
 
 static BOOL run_terminator(vbdisp_t *This)
@@ -295,6 +328,8 @@ static HRESULT WINAPI DispatchEx_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
 
     TRACE("(%p)->(%s %x %p)\n", This, debugstr_w(bstrName), grfdex, pid);
 
+    grfdex &= ~FDEX_VERSION_MASK;
+
     if(!This->desc)
         return E_UNEXPECTED;
 
@@ -488,10 +523,14 @@ void collect_objects(script_ctx_t *ctx)
     LIST_FOR_EACH_ENTRY_SAFE(iter, iter2, &ctx->objects, vbdisp_t, entry)
         run_terminator(iter);
 
-    LIST_FOR_EACH_ENTRY_SAFE(iter, iter2, &ctx->objects, vbdisp_t, entry) {
+    while(!list_empty(&ctx->objects)) {
+        iter = LIST_ENTRY(list_head(&ctx->objects), vbdisp_t, entry);
+
         IDispatchEx_AddRef(&iter->IDispatchEx_iface);
         clean_props(iter);
         iter->desc = NULL;
+        list_remove(&iter->entry);
+        list_init(&iter->entry);
         IDispatchEx_Release(&iter->IDispatchEx_iface);
     }
 }
@@ -508,7 +547,7 @@ HRESULT disp_get_id(IDispatch *disp, BSTR name, vbdisp_invoke_type_t invoke_type
 
     hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
     if(FAILED(hres)) {
-        TRACE("unsing IDispatch\n");
+        TRACE("using IDispatch\n");
         return IDispatch_GetIDsOfNames(disp, &IID_NULL, &name, 1, 0, id);
     }
 
@@ -541,23 +580,21 @@ HRESULT disp_call(script_ctx_t *ctx, IDispatch *disp, DISPID id, DISPPARAMS *dp,
     return hres;
 }
 
-HRESULT disp_propput(script_ctx_t *ctx, IDispatch *disp, DISPID id, VARIANT *val)
+HRESULT disp_propput(script_ctx_t *ctx, IDispatch *disp, DISPID id, DISPPARAMS *dp)
 {
-    DISPID propput_dispid = DISPID_PROPERTYPUT;
-    DISPPARAMS dp  = {val, &propput_dispid, 1, 1};
     IDispatchEx *dispex;
     EXCEPINFO ei = {0};
     HRESULT hres;
 
     hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
     if(SUCCEEDED(hres)) {
-        hres = IDispatchEx_InvokeEx(dispex, id, ctx->lcid, DISPATCH_PROPERTYPUT, &dp, NULL, &ei, NULL /* FIXME! */);
+        hres = IDispatchEx_InvokeEx(dispex, id, ctx->lcid, DISPATCH_PROPERTYPUT, dp, NULL, &ei, NULL /* FIXME! */);
         IDispatchEx_Release(dispex);
     }else {
         ULONG err = 0;
 
         TRACE("using IDispatch\n");
-        hres = IDispatch_Invoke(disp, id, &IID_NULL, ctx->lcid, DISPATCH_PROPERTYPUT, &dp, NULL, &ei, &err);
+        hres = IDispatch_Invoke(disp, id, &IID_NULL, ctx->lcid, DISPATCH_PROPERTYPUT, dp, NULL, &ei, &err);
     }
 
     return hres;
