@@ -302,8 +302,14 @@ static BOOL buffer_check_attribute(struct wined3d_buffer *This, const struct win
 
         if (!is_ffp_color) FIXME("Test for non-color fixed function WINED3DFMT_B8G8R8A8_UNORM format\n");
     }
-    else if (is_ffp_position && format == WINED3DFMT_R32G32B32A32_FLOAT)
+    else if (is_ffp_position && si->position_transformed)
     {
+        if (format != WINED3DFMT_R32G32B32A32_FLOAT)
+        {
+            FIXME("Unexpected format %s for transformed position.\n", debug_d3dformat(format));
+            return FALSE;
+        }
+
         ret = buffer_process_converted_attribute(This, CONV_POSITIONT, attrib, stride_this_run);
     }
     else if (This->conversion_map)
@@ -470,7 +476,7 @@ void buffer_get_memory(struct wined3d_buffer *buffer, const struct wined3d_gl_in
     data->buffer_object = buffer->buffer_object;
     if (!buffer->buffer_object)
     {
-        if (buffer->flags & WINED3D_BUFFER_CREATEBO)
+        if ((buffer->flags & WINED3D_BUFFER_CREATEBO) && !buffer->resource.map_count)
         {
             buffer_create_buffer_object(buffer, gl_info);
             buffer->flags &= ~WINED3D_BUFFER_CREATEBO;
@@ -598,9 +604,11 @@ static void buffer_sync_apple(struct wined3d_buffer *This, DWORD flags, const st
     enum wined3d_event_query_result ret;
 
     /* No fencing needs to be done if the app promises not to overwrite
-     * existing data */
-    if(flags & WINED3DLOCK_NOOVERWRITE) return;
-    if(flags & WINED3DLOCK_DISCARD)
+     * existing data. */
+    if (flags & WINED3D_MAP_NOOVERWRITE)
+        return;
+
+    if (flags & WINED3D_MAP_DISCARD)
     {
         ENTER_GL();
         GL_EXTCALL(glBufferDataARB(This->buffer_type_hint, This->resource.size, NULL, This->buffer_object_usage));
@@ -692,8 +700,10 @@ static void buffer_direct_upload(struct wined3d_buffer *This, const struct wined
         if (This->flags & WINED3D_BUFFER_APPLESYNC)
         {
             DWORD syncflags = 0;
-            if (flags & WINED3D_BUFFER_DISCARD) syncflags |= WINED3DLOCK_DISCARD;
-            if (flags & WINED3D_BUFFER_NOSYNC) syncflags |= WINED3DLOCK_NOOVERWRITE;
+            if (flags & WINED3D_BUFFER_DISCARD)
+                syncflags |= WINED3D_MAP_DISCARD;
+            if (flags & WINED3D_BUFFER_NOSYNC)
+                syncflags |= WINED3D_MAP_NOOVERWRITE;
             LEAVE_GL();
             buffer_sync_apple(This, syncflags, gl_info);
             ENTER_GL();
@@ -747,6 +757,12 @@ void CDECL wined3d_buffer_preload(struct wined3d_buffer *buffer)
 
     TRACE("buffer %p.\n", buffer);
 
+    if (buffer->resource.map_count)
+    {
+        WARN("Buffer is mapped, skipping preload.\n");
+        return;
+    }
+
     buffer->flags &= ~(WINED3D_BUFFER_NOSYNC | WINED3D_BUFFER_DISCARD);
 
     if (!buffer->buffer_object)
@@ -767,7 +783,7 @@ void CDECL wined3d_buffer_preload(struct wined3d_buffer *buffer)
     }
 
     /* Reading the declaration makes only sense if the stateblock is finalized and the buffer bound to a stream */
-    if (device->isInDraw && buffer->bind_count > 0)
+    if (device->isInDraw && buffer->resource.bind_count > 0)
     {
         decl_changed = buffer_find_decl(buffer);
         buffer->flags |= WINED3D_BUFFER_HASDESC;
@@ -836,7 +852,7 @@ void CDECL wined3d_buffer_preload(struct wined3d_buffer *buffer)
                 FIXME("Too many full buffer conversions, stopping converting.\n");
                 buffer_unload(&buffer->resource);
                 buffer->flags &= ~WINED3D_BUFFER_CREATEBO;
-                if (buffer->bind_count)
+                if (buffer->resource.bind_count)
                     device_invalidate_state(device, STATE_STREAMSRC);
                 return;
             }
@@ -936,29 +952,31 @@ void CDECL wined3d_buffer_preload(struct wined3d_buffer *buffer)
 
 static DWORD buffer_sanitize_flags(const struct wined3d_buffer *buffer, DWORD flags)
 {
-    /* Not all flags make sense together, but Windows never returns an error. Catch the
-     * cases that could cause issues */
-    if(flags & WINED3DLOCK_READONLY)
+    /* Not all flags make sense together, but Windows never returns an error.
+     * Catch the cases that could cause issues. */
+    if (flags & WINED3D_MAP_READONLY)
     {
-        if(flags & WINED3DLOCK_DISCARD)
+        if (flags & WINED3D_MAP_DISCARD)
         {
-            WARN("WINED3DLOCK_READONLY combined with WINED3DLOCK_DISCARD, ignoring flags\n");
+            WARN("WINED3D_MAP_READONLY combined with WINED3D_MAP_DISCARD, ignoring flags.\n");
             return 0;
         }
-        if(flags & WINED3DLOCK_NOOVERWRITE)
+        if (flags & WINED3D_MAP_NOOVERWRITE)
         {
-            WARN("WINED3DLOCK_READONLY combined with WINED3DLOCK_NOOVERWRITE, ignoring flags\n");
+            WARN("WINED3D_MAP_READONLY combined with WINED3D_MAP_NOOVERWRITE, ignoring flags.\n");
             return 0;
         }
     }
-    else if((flags & (WINED3DLOCK_DISCARD | WINED3DLOCK_NOOVERWRITE)) == (WINED3DLOCK_DISCARD | WINED3DLOCK_NOOVERWRITE))
+    else if ((flags & (WINED3D_MAP_DISCARD | WINED3D_MAP_NOOVERWRITE))
+            == (WINED3D_MAP_DISCARD | WINED3D_MAP_NOOVERWRITE))
     {
-        WARN("WINED3DLOCK_DISCARD and WINED3DLOCK_NOOVERWRITE used together, ignoring\n");
+        WARN("WINED3D_MAP_DISCARD and WINED3D_MAP_NOOVERWRITE used together, ignoring.\n");
         return 0;
     }
-    else if (flags & (WINED3DLOCK_DISCARD | WINED3DLOCK_NOOVERWRITE) && !(buffer->resource.usage & WINED3DUSAGE_DYNAMIC))
+    else if (flags & (WINED3D_MAP_DISCARD | WINED3D_MAP_NOOVERWRITE)
+            && !(buffer->resource.usage & WINED3DUSAGE_DYNAMIC))
     {
-        WARN("DISCARD or NOOVERWRITE lock on non-dynamic buffer, ignoring\n");
+        WARN("DISCARD or NOOVERWRITE map on non-dynamic buffer, ignoring.\n");
         return 0;
     }
 
@@ -969,14 +987,14 @@ static GLbitfield buffer_gl_map_flags(DWORD d3d_flags)
 {
     GLbitfield ret = 0;
 
-    if (!(d3d_flags & WINED3DLOCK_READONLY))
+    if (!(d3d_flags & WINED3D_MAP_READONLY))
         ret |= GL_MAP_WRITE_BIT | GL_MAP_FLUSH_EXPLICIT_BIT;
-    if (!(d3d_flags & (WINED3DLOCK_DISCARD | WINED3DLOCK_NOOVERWRITE)))
+    if (!(d3d_flags & (WINED3D_MAP_DISCARD | WINED3D_MAP_NOOVERWRITE)))
         ret |= GL_MAP_READ_BIT;
 
-    if (d3d_flags & WINED3DLOCK_DISCARD)
+    if (d3d_flags & WINED3D_MAP_DISCARD)
         ret |= GL_MAP_INVALIDATE_BUFFER_BIT;
-    if (d3d_flags & WINED3DLOCK_NOOVERWRITE)
+    if (d3d_flags & WINED3D_MAP_NOOVERWRITE)
         ret |= GL_MAP_UNSYNCHRONIZED_BIT;
 
     return ret;
@@ -997,12 +1015,25 @@ HRESULT CDECL wined3d_buffer_map(struct wined3d_buffer *buffer, UINT offset, UIN
     TRACE("buffer %p, offset %u, size %u, data %p, flags %#x\n", buffer, offset, size, data, flags);
 
     flags = buffer_sanitize_flags(buffer, flags);
-    if (!(flags & WINED3DLOCK_READONLY))
+    if (!(flags & WINED3D_MAP_READONLY))
     {
-        if (!buffer_add_dirty_area(buffer, offset, size)) return E_OUTOFMEMORY;
+        if (flags & WINED3D_MAP_DISCARD)
+        {
+            /* DISCARD invalidates the entire buffer, regardless of the
+             * specified offset and size. Some applications also depend on the
+             * entire buffer being uploaded in that case. Two such
+             * applications are Port Royale and Darkstar One. */
+            if (!buffer_add_dirty_area(buffer, 0, 0))
+                return E_OUTOFMEMORY;
+        }
+        else
+        {
+            if (!buffer_add_dirty_area(buffer, offset, size))
+                return E_OUTOFMEMORY;
+        }
     }
 
-    count = InterlockedIncrement(&buffer->lock_count);
+    count = ++buffer->resource.map_count;
 
     if (buffer->buffer_object)
     {
@@ -1063,7 +1094,7 @@ HRESULT CDECL wined3d_buffer_map(struct wined3d_buffer *buffer, UINT offset, UIN
                         TRACE("Dynamic buffer, dropping VBO\n");
                         buffer_unload(&buffer->resource);
                         buffer->flags &= ~WINED3D_BUFFER_CREATEBO;
-                        if (buffer->bind_count)
+                        if (buffer->resource.bind_count)
                             device_invalidate_state(device, STATE_STREAMSRC);
                     }
                     else
@@ -1080,17 +1111,17 @@ HRESULT CDECL wined3d_buffer_map(struct wined3d_buffer *buffer, UINT offset, UIN
         {
             if (dirty)
             {
-                if (buffer->flags & WINED3D_BUFFER_NOSYNC && !(flags & WINED3DLOCK_NOOVERWRITE))
+                if (buffer->flags & WINED3D_BUFFER_NOSYNC && !(flags & WINED3D_MAP_NOOVERWRITE))
                 {
                     buffer->flags &= ~WINED3D_BUFFER_NOSYNC;
                 }
             }
-            else if(flags & WINED3DLOCK_NOOVERWRITE)
+            else if(flags & WINED3D_MAP_NOOVERWRITE)
             {
                 buffer->flags |= WINED3D_BUFFER_NOSYNC;
             }
 
-            if (flags & WINED3DLOCK_DISCARD)
+            if (flags & WINED3D_MAP_DISCARD)
             {
                 buffer->flags |= WINED3D_BUFFER_DISCARD;
             }
@@ -1115,13 +1146,13 @@ void CDECL wined3d_buffer_unmap(struct wined3d_buffer *buffer)
      * number of Map calls, d3d returns always D3D_OK.
      * This is also needed to prevent Map from returning garbage on
      * the next call (this will happen if the lock_count is < 0). */
-    if (!buffer->lock_count)
+    if (!buffer->resource.map_count)
     {
         WARN("Unmap called without a previous map call.\n");
         return;
     }
 
-    if (InterlockedDecrement(&buffer->lock_count))
+    if (--buffer->resource.map_count)
     {
         /* Delay loading the buffer until everything is unlocked */
         TRACE("Ignoring unmap.\n");
