@@ -25,6 +25,8 @@
  */
 
 #include "config.h"
+#include "wine/port.h"
+
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d_draw);
@@ -65,7 +67,7 @@ static void drawStridedFast(const struct wined3d_gl_info *gl_info, GLenum primit
  */
 
 /* GL locking is done by the caller */
-static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_context *context,
+static void drawStridedSlow(const struct wined3d_device *device, const struct wined3d_context *context,
         const struct wined3d_stream_info *si, UINT NumVertexes, GLenum glPrimType,
         const void *idxData, UINT idxSize, UINT startIdx)
 {
@@ -143,10 +145,10 @@ static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_
         specular = element->data.addr;
 
         /* special case where the fog density is stored in the specular alpha channel */
-        if (state->render_states[WINED3DRS_FOGENABLE]
-                && (state->render_states[WINED3DRS_FOGVERTEXMODE] == WINED3DFOG_NONE
+        if (state->render_states[WINED3D_RS_FOGENABLE]
+                && (state->render_states[WINED3D_RS_FOGVERTEXMODE] == WINED3D_FOG_NONE
                     || si->elements[WINED3D_FFP_POSITION].format->id == WINED3DFMT_R32G32B32A32_FLOAT)
-                && state->render_states[WINED3DRS_FOGTABLEMODE] == WINED3DFOG_NONE)
+                && state->render_states[WINED3D_RS_FOGTABLEMODE] == WINED3D_FOG_NONE)
         {
             if (gl_info->supported[EXT_FOG_COORD])
             {
@@ -173,7 +175,7 @@ static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_
 
     for (textureNo = 0; textureNo < texture_stages; ++textureNo)
     {
-        int coordIdx = state->texture_states[textureNo][WINED3DTSS_TEXCOORDINDEX];
+        int coordIdx = state->texture_states[textureNo][WINED3D_TSS_TEXCOORD_INDEX];
         DWORD texture_idx = device->texUnitMap[textureNo];
 
         if (!gl_info->supported[ARB_MULTITEXTURE] && textureNo > 0)
@@ -243,7 +245,7 @@ static void drawStridedSlow(struct wined3d_device *device, const struct wined3d_
 
             if (!(tmp_tex_mask & 1)) continue;
 
-            coord_idx = state->texture_states[texture][WINED3DTSS_TEXCOORDINDEX];
+            coord_idx = state->texture_states[texture][WINED3D_TSS_TEXCOORD_INDEX];
             ptr = texCoords[coord_idx] + (SkipnStrides * si->elements[WINED3D_FFP_TEXCOORD0 + coord_idx].stride);
 
             texture_idx = device->texUnitMap[texture];
@@ -384,7 +386,7 @@ static inline void send_attribute(const struct wined3d_gl_info *gl_info,
             /* Are those 16 bit floats. C doesn't have a 16 bit float type. I could read the single bits and calculate a 4
              * byte float according to the IEEE standard
              */
-            if (gl_info->supported[NV_HALF_FLOAT])
+            if (gl_info->supported[NV_HALF_FLOAT] && gl_info->supported[NV_VERTEX_PROGRAM])
             {
                 /* Not supported by GL_ARB_half_float_vertex */
                 GL_EXTCALL(glVertexAttrib2hvNV(index, ptr));
@@ -397,7 +399,7 @@ static inline void send_attribute(const struct wined3d_gl_info *gl_info,
             }
             break;
         case WINED3DFMT_R16G16B16A16_FLOAT:
-            if (gl_info->supported[NV_HALF_FLOAT])
+            if (gl_info->supported[NV_HALF_FLOAT] && gl_info->supported[NV_VERTEX_PROGRAM])
             {
                 /* Not supported by GL_ARB_half_float_vertex */
                 GL_EXTCALL(glVertexAttrib4hvNV(index, ptr));
@@ -575,7 +577,7 @@ static void remove_vbos(const struct wined3d_gl_info *gl_info,
 }
 
 /* Routine common to the draw primitive and draw indexed primitive routines */
-void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartIdx, UINT idxSize, const void *idxData)
+void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartIdx, BOOL indexed, const void *idxData)
 {
     const struct wined3d_state *state = &device->stateBlock->state;
     struct wined3d_context *context;
@@ -583,7 +585,7 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
 
     if (!index_count) return;
 
-    if (state->render_states[WINED3DRS_COLORWRITEENABLE])
+    if (state->render_states[WINED3D_RS_COLORWRITEENABLE])
     {
         /* Invalidate the back buffer memory so LockRect will read it the next time */
         for (i = 0; i < device->adapter->gl_info.limits.buffers; ++i)
@@ -591,8 +593,8 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
             struct wined3d_surface *target = device->fb.render_targets[i];
             if (target)
             {
-                surface_load_location(target, SFLAG_INDRAWABLE, NULL);
-                surface_modify_location(target, SFLAG_INDRAWABLE, TRUE);
+                surface_load_location(target, target->draw_binding, NULL);
+                surface_modify_location(target, target->draw_binding, TRUE);
             }
         }
     }
@@ -615,13 +617,13 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
          * Z-compare function into account, but we could skip loading the
          * depthstencil for D3DCMP_NEVER and D3DCMP_ALWAYS as well. Also note
          * that we never copy the stencil data.*/
-        DWORD location = context->render_offscreen ? SFLAG_DS_OFFSCREEN : SFLAG_DS_ONSCREEN;
-        if (state->render_states[WINED3DRS_ZWRITEENABLE] || state->render_states[WINED3DRS_ZENABLE])
+        DWORD location = context->render_offscreen ? device->fb.depth_stencil->draw_binding : SFLAG_INDRAWABLE;
+        if (state->render_states[WINED3D_RS_ZWRITEENABLE] || state->render_states[WINED3D_RS_ZENABLE])
         {
             struct wined3d_surface *ds = device->fb.depth_stencil;
             RECT current_rect, draw_rect, r;
 
-            if (location == SFLAG_DS_ONSCREEN && ds != device->onscreen_depth_stencil)
+            if (!context->render_offscreen && ds != device->onscreen_depth_stencil)
                 device_switch_onscreen_ds(device, context, ds);
 
             if (ds->flags & location)
@@ -634,12 +636,6 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
             IntersectRect(&r, &draw_rect, &current_rect);
             if (!EqualRect(&r, &draw_rect))
                 surface_load_ds_location(ds, context, location);
-
-            if (state->render_states[WINED3DRS_ZWRITEENABLE])
-            {
-                surface_modify_ds_location(ds, location, ds->ds_current_size.cx, ds->ds_current_size.cy);
-                surface_modify_location(ds, SFLAG_INDRAWABLE, TRUE);
-            }
         }
     }
 
@@ -650,10 +646,18 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
         return;
     }
 
+    if (device->fb.depth_stencil && state->render_states[WINED3D_RS_ZWRITEENABLE])
+    {
+        struct wined3d_surface *ds = device->fb.depth_stencil;
+        DWORD location = context->render_offscreen ? ds->draw_binding : SFLAG_INDRAWABLE;
+
+        surface_modify_ds_location(ds, location, ds->ds_current_size.cx, ds->ds_current_size.cy);
+    }
+
     if ((!context->gl_info->supported[WINED3D_GL_VERSION_2_0]
             || (!glPointParameteri && !context->gl_info->supported[NV_POINT_SPRITE]))
             && context->render_offscreen
-            && state->render_states[WINED3DRS_POINTSPRITEENABLE]
+            && state->render_states[WINED3D_RS_POINTSPRITEENABLE]
             && state->gl_primitive_type == GL_POINTS)
     {
         FIXME("Point sprite coordinate origin switching not supported.\n");
@@ -667,11 +671,29 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
         BOOL emulation = FALSE;
         const struct wined3d_stream_info *stream_info = &device->strided_streams;
         struct wined3d_stream_info stridedlcl;
+        UINT idx_size = 0;
+
+        if (indexed)
+        {
+            if (!state->user_stream)
+            {
+                struct wined3d_buffer *index_buffer = state->index_buffer;
+                if (!index_buffer->buffer_object || !stream_info->all_vbo)
+                    idxData = index_buffer->resource.allocatedMemory;
+                else
+                    idxData = NULL;
+            }
+
+            if (state->index_format == WINED3DFMT_R16_UINT)
+                idx_size = 2;
+            else
+                idx_size = 4;
+        }
 
         if (!use_vs(state))
         {
             if (!stream_info->position_transformed && context->num_untracked_materials
-                    && state->render_states[WINED3DRS_LIGHTING])
+                    && state->render_states[WINED3D_RS_LIGHTING])
             {
                 static BOOL warned;
                 if (!warned) {
@@ -682,7 +704,7 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
                 }
                 emulation = TRUE;
             }
-            else if (context->fog_coord && state->render_states[WINED3DRS_FOGENABLE])
+            else if (context->fog_coord && state->render_states[WINED3D_RS_FOGENABLE])
             {
                 /* Either write a pipeline replacement shader or convert the specular alpha from unsigned byte
                  * to a float in the vertex buffer
@@ -717,23 +739,23 @@ void drawPrimitive(struct wined3d_device *device, UINT index_count, UINT StartId
                     TRACE("Using immediate mode with vertex shaders for half float emulation\n");
                 }
                 drawStridedSlowVs(context->gl_info, state, stream_info,
-                        index_count, glPrimType, idxData, idxSize, StartIdx);
+                        index_count, glPrimType, idxData, idx_size, StartIdx);
             }
             else
             {
                 drawStridedSlow(device, context, stream_info, index_count,
-                        glPrimType, idxData, idxSize, StartIdx);
+                        glPrimType, idxData, idx_size, StartIdx);
             }
         }
         else if (device->instancedDraw)
         {
             /* Instancing emulation with mixing immediate mode and arrays */
             drawStridedInstanced(context->gl_info, state, stream_info,
-                    index_count, glPrimType, idxData, idxSize, StartIdx, base_vertex_index);
+                    index_count, glPrimType, idxData, idx_size, StartIdx, base_vertex_index);
         }
         else
         {
-            drawStridedFast(context->gl_info, glPrimType, index_count, idxSize, idxData, StartIdx, base_vertex_index);
+            drawStridedFast(context->gl_info, glPrimType, index_count, idx_size, idxData, StartIdx, base_vertex_index);
         }
     }
 
@@ -786,9 +808,10 @@ static void normalize_normal(float *n) {
  * responsible of taking care that either the gl states are restored, or the context activated
  * for drawing to reset the lastWasBlit flag.
  */
-HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch *patch)
+HRESULT tesselate_rectpatch(struct wined3d_device *This, struct wined3d_rect_patch *patch)
 {
     unsigned int i, j, num_quads, out_vertex_size, buffer_size, d3d_out_vertex_size;
+    const struct wined3d_rect_patch_info *info = &patch->rect_patch_info;
     float max_x = 0.0f, max_y = 0.0f, max_z = 0.0f, neg_z = 0.0f;
     struct wined3d_state *state = &This->stateBlock->state;
     struct wined3d_stream_info stream_info;
@@ -796,7 +819,6 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
     struct wined3d_context *context;
     struct wined3d_shader *vs;
     const BYTE *data;
-    const WINED3DRECTPATCH_INFO *info = &patch->RectPatchInfo;
     DWORD vtxStride;
     GLenum feedback_type;
     GLfloat *feedbuffer;
@@ -811,7 +833,7 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
      * the stateblock. Beware of VBOs. */
     vs = state->vertex_shader;
     state->vertex_shader = NULL;
-    device_stream_info_from_declaration(This, &stream_info, NULL);
+    device_stream_info_from_declaration(This, &stream_info);
     state->vertex_shader = vs;
 
     e = &stream_info.elements[WINED3D_FFP_POSITION];
@@ -821,9 +843,9 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
         e->data.addr = (BYTE *)((ULONG_PTR)e->data.addr + (ULONG_PTR)buffer_get_sysmem(vb, context->gl_info));
     }
     vtxStride = e->stride;
-    data = e->data.addr +
-           vtxStride * info->Stride * info->StartVertexOffsetHeight +
-           vtxStride * info->StartVertexOffsetWidth;
+    data = e->data.addr
+            + vtxStride * info->stride * info->start_vertex_offset_height
+            + vtxStride * info->start_vertex_offset_width;
 
     /* Not entirely sure about what happens with transformed vertices */
     if (stream_info.position_transformed) FIXME("Transformed position in rectpatch generation\n");
@@ -835,17 +857,17 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
          */
         ERR("Vertex stride is not a multiple of sizeof(GLfloat)\n");
     }
-    if(info->Basis != WINED3DBASIS_BEZIER) {
-        FIXME("Basis is %s, how to handle this?\n", debug_d3dbasis(info->Basis));
-    }
-    if(info->Degree != WINED3DDEGREE_CUBIC) {
-        FIXME("Degree is %s, how to handle this?\n", debug_d3ddegree(info->Degree));
-    }
+    if (info->basis != WINED3D_BASIS_BEZIER)
+        FIXME("Basis is %s, how to handle this?\n", debug_d3dbasis(info->basis));
+    if (info->degree != WINED3D_DEGREE_CUBIC)
+        FIXME("Degree is %s, how to handle this?\n", debug_d3ddegree(info->degree));
 
     /* First, get the boundary cube of the input data */
-    for(j = 0; j < info->Height; j++) {
-        for(i = 0; i < info->Width; i++) {
-            const float *v = (const float *)(data + vtxStride * i + vtxStride * info->Stride * j);
+    for (j = 0; j < info->height; ++j)
+    {
+        for (i = 0; i < info->width; ++i)
+        {
+            const float *v = (const float *)(data + vtxStride * i + vtxStride * info->stride * j);
             if(fabs(v[0]) > max_x) max_x = fabsf(v[0]);
             if(fabs(v[1]) > max_y) max_y = fabsf(v[1]);
             if(fabs(v[2]) > max_z) max_z = fabsf(v[2]);
@@ -875,7 +897,7 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
      */
     glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
     checkGLcall("glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)");
-    context_invalidate_state(context, STATE_RENDER(WINED3DRS_FILLMODE));
+    context_invalidate_state(context, STATE_RENDER(WINED3D_RS_FILLMODE));
     if (patch->has_normals)
     {
         static const GLfloat black[] = {0.0f, 0.0f, 0.0f, 0.0f};
@@ -887,7 +909,7 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
         checkGLcall("glEnable(GL_LIGHTING)");
         glLightModelfv(GL_LIGHT_MODEL_AMBIENT, black);
         checkGLcall("glLightModel for MODEL_AMBIENT");
-        context_invalidate_state(context, STATE_RENDER(WINED3DRS_AMBIENT));
+        context_invalidate_state(context, STATE_RENDER(WINED3D_RS_AMBIENT));
 
         for (i = 3; i < context->gl_info->limits.lights; ++i)
         {
@@ -919,7 +941,7 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
         checkGLcall("Setting up light 3");
 
         context_invalidate_state(context, STATE_MATERIAL);
-        context_invalidate_state(context, STATE_RENDER(WINED3DRS_COLORVERTEX));
+        context_invalidate_state(context, STATE_RENDER(WINED3D_RS_COLORVERTEX));
         glDisable(GL_COLOR_MATERIAL);
         glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, black);
         glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, black);
@@ -964,14 +986,15 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
     feedbuffer = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, buffer_size * sizeof(float) * 8);
 
     glMap2f(GL_MAP2_VERTEX_3,
-            0.0f, 1.0f, vtxStride / sizeof(float), info->Width,
-            0.0f, 1.0f, info->Stride * vtxStride / sizeof(float), info->Height,
+            0.0f, 1.0f, vtxStride / sizeof(float), info->width,
+            0.0f, 1.0f, info->stride * vtxStride / sizeof(float), info->height,
             (const GLfloat *)data);
     checkGLcall("glMap2f");
-    if(patch->has_texcoords) {
+    if (patch->has_texcoords)
+    {
         glMap2f(GL_MAP2_TEXTURE_COORD_4,
-                0.0f, 1.0f, vtxStride / sizeof(float), info->Width,
-                0.0f, 1.0f, info->Stride * vtxStride / sizeof(float), info->Height,
+                0.0f, 1.0f, vtxStride / sizeof(float), info->width,
+                0.0f, 1.0f, info->stride * vtxStride / sizeof(float), info->height,
                 (const GLfloat *)data);
         checkGLcall("glMap2f");
     }
@@ -1124,21 +1147,22 @@ HRESULT tesselate_rectpatch(struct wined3d_device *This, struct WineD3DRectPatch
     }
     memset(&patch->strided, 0, sizeof(patch->strided));
     patch->strided.position.format = WINED3DFMT_R32G32B32_FLOAT;
-    patch->strided.position.lpData = (BYTE *) patch->mem;
-    patch->strided.position.dwStride = vtxStride;
+    patch->strided.position.data = (BYTE *)patch->mem;
+    patch->strided.position.stride = vtxStride;
 
-    if(patch->has_normals) {
+    if (patch->has_normals)
+    {
         patch->strided.normal.format = WINED3DFMT_R32G32B32_FLOAT;
-        patch->strided.normal.lpData = (BYTE *) patch->mem + 3 * sizeof(float) /* pos */;
-        patch->strided.normal.dwStride = vtxStride;
+        patch->strided.normal.data = (BYTE *)patch->mem + 3 * sizeof(float) /* pos */;
+        patch->strided.normal.stride = vtxStride;
     }
-    if(patch->has_texcoords) {
-        patch->strided.texCoords[0].format = WINED3DFMT_R32G32B32A32_FLOAT;
-        patch->strided.texCoords[0].lpData = (BYTE *) patch->mem + 3 * sizeof(float) /* pos */;
-        if(patch->has_normals) {
-            patch->strided.texCoords[0].lpData += 3 * sizeof(float);
-        }
-        patch->strided.texCoords[0].dwStride = vtxStride;
+    if (patch->has_texcoords)
+    {
+        patch->strided.tex_coords[0].format = WINED3DFMT_R32G32B32A32_FLOAT;
+        patch->strided.tex_coords[0].data = (BYTE *)patch->mem + 3 * sizeof(float) /* pos */;
+        if (patch->has_normals)
+            patch->strided.tex_coords[0].data += 3 * sizeof(float);
+        patch->strided.tex_coords[0].stride = vtxStride;
     }
 
     return WINED3D_OK;
