@@ -23,6 +23,7 @@
  */
 
 #include "config.h"
+#include "wine/port.h"
 
 #include <math.h>
 #include <stdio.h>
@@ -64,10 +65,12 @@ static const char * const shader_opcode_names[] =
     /* WINED3DSIH_ENDIF         */ "endif",
     /* WINED3DSIH_ENDLOOP       */ "endloop",
     /* WINED3DSIH_ENDREP        */ "endrep",
+    /* WINED3DSIH_EQ            */ "eq",
     /* WINED3DSIH_EXP           */ "exp",
     /* WINED3DSIH_EXPP          */ "expp",
     /* WINED3DSIH_FRC           */ "frc",
     /* WINED3DSIH_FTOI          */ "ftoi",
+    /* WINED3DSIH_GE            */ "ge",
     /* WINED3DSIH_IADD          */ "iadd",
     /* WINED3DSIH_IEQ           */ "ieq",
     /* WINED3DSIH_IF            */ "if",
@@ -102,6 +105,7 @@ static const char * const shader_opcode_names[] =
     /* WINED3DSIH_RCP           */ "rcp",
     /* WINED3DSIH_REP           */ "rep",
     /* WINED3DSIH_RET           */ "ret",
+    /* WINED3DSIH_ROUND_NI      */ "round_ni",
     /* WINED3DSIH_RSQ           */ "rsq",
     /* WINED3DSIH_SAMPLE        */ "sample",
     /* WINED3DSIH_SAMPLE_GRAD   */ "sample_d",
@@ -135,28 +139,31 @@ static const char * const shader_opcode_names[] =
     /* WINED3DSIH_TEXREG2AR     */ "texreg2ar",
     /* WINED3DSIH_TEXREG2GB     */ "texreg2gb",
     /* WINED3DSIH_TEXREG2RGB    */ "texreg2rgb",
+    /* WINED3DSIH_UDIV          */ "udiv",
+    /* WINED3DSIH_USHR          */ "ushr",
     /* WINED3DSIH_UTOF          */ "utof",
+    /* WINED3DSIH_XOR           */ "xor",
 };
 
 static const char * const semantic_names[] =
 {
-    /* WINED3DDECLUSAGE_POSITION        */ "SV_POSITION",
-    /* WINED3DDECLUSAGE_BLENDWEIGHT     */ "BLENDWEIGHT",
-    /* WINED3DDECLUSAGE_BLENDINDICES    */ "BLENDINDICES",
-    /* WINED3DDECLUSAGE_NORMAL          */ "NORMAL",
-    /* WINED3DDECLUSAGE_PSIZE           */ "PSIZE",
-    /* WINED3DDECLUSAGE_TEXCOORD        */ "TEXCOORD",
-    /* WINED3DDECLUSAGE_TANGENT         */ "TANGENT",
-    /* WINED3DDECLUSAGE_BINORMAL        */ "BINORMAL",
-    /* WINED3DDECLUSAGE_TESSFACTOR      */ "TESSFACTOR",
-    /* WINED3DDECLUSAGE_POSITIONT       */ "POSITIONT",
-    /* WINED3DDECLUSAGE_COLOR           */ "COLOR",
-    /* WINED3DDECLUSAGE_FOG             */ "FOG",
-    /* WINED3DDECLUSAGE_DEPTH           */ "DEPTH",
-    /* WINED3DDECLUSAGE_SAMPLE          */ "SAMPLE",
+    /* WINED3D_DECL_USAGE_POSITION      */ "SV_POSITION",
+    /* WINED3D_DECL_USAGE_BLEND_WEIGHT  */ "BLENDWEIGHT",
+    /* WINED3D_DECL_USAGE_BLEND_INDICES */ "BLENDINDICES",
+    /* WINED3D_DECL_USAGE_NORMAL        */ "NORMAL",
+    /* WINED3D_DECL_USAGE_PSIZE         */ "PSIZE",
+    /* WINED3D_DECL_USAGE_TEXCOORD      */ "TEXCOORD",
+    /* WINED3D_DECL_USAGE_TANGENT       */ "TANGENT",
+    /* WINED3D_DECL_USAGE_BINORMAL      */ "BINORMAL",
+    /* WINED3D_DECL_USAGE_TESS_FACTOR   */ "TESSFACTOR",
+    /* WINED3D_DECL_USAGE_POSITIONT     */ "POSITIONT",
+    /* WINED3D_DECL_USAGE_COLOR         */ "COLOR",
+    /* WINED3D_DECL_USAGE_FOG           */ "FOG",
+    /* WINED3D_DECL_USAGE_DEPTH         */ "DEPTH",
+    /* WINED3D_DECL_USAGE_SAMPLE        */ "SAMPLE",
 };
 
-static const char *shader_semantic_name_from_usage(WINED3DDECLUSAGE usage)
+static const char *shader_semantic_name_from_usage(enum wined3d_decl_usage usage)
 {
     if (usage >= sizeof(semantic_names) / sizeof(*semantic_names))
     {
@@ -167,7 +174,7 @@ static const char *shader_semantic_name_from_usage(WINED3DDECLUSAGE usage)
     return semantic_names[usage];
 }
 
-static WINED3DDECLUSAGE shader_usage_from_semantic_name(const char *name)
+static enum wined3d_decl_usage shader_usage_from_semantic_name(const char *name)
 {
     unsigned int i;
 
@@ -179,7 +186,7 @@ static WINED3DDECLUSAGE shader_usage_from_semantic_name(const char *name)
     return ~0U;
 }
 
-BOOL shader_match_semantic(const char *semantic_name, WINED3DDECLUSAGE usage)
+BOOL shader_match_semantic(const char *semantic_name, enum wined3d_decl_usage usage)
 {
     return !strcmp(semantic_name, shader_semantic_name_from_usage(usage));
 }
@@ -196,7 +203,7 @@ static void shader_signature_from_semantic(struct wined3d_shader_signature_eleme
 }
 
 static void shader_signature_from_usage(struct wined3d_shader_signature_element *e,
-        WINED3DDECLUSAGE usage, UINT usage_idx, UINT reg_idx, DWORD write_mask)
+        enum wined3d_decl_usage usage, UINT usage_idx, UINT reg_idx, DWORD write_mask)
 {
     e->semantic_name = shader_semantic_name_from_usage(usage);
     e->semantic_idx = usage_idx;
@@ -311,7 +318,7 @@ static void shader_init(struct wined3d_shader *shader, struct wined3d_device *de
 
 /* Convert floating point offset relative to a register file to an absolute
  * offset for float constants. */
-static unsigned int shader_get_float_offset(WINED3DSHADER_PARAM_REGISTER_TYPE register_type, UINT register_idx)
+static unsigned int shader_get_float_offset(enum wined3d_shader_register_type register_type, UINT register_idx)
 {
     switch (register_type)
     {
@@ -327,13 +334,13 @@ static unsigned int shader_get_float_offset(WINED3DSHADER_PARAM_REGISTER_TYPE re
 
 static void shader_delete_constant_list(struct list *clist)
 {
-    struct local_constant *constant;
+    struct wined3d_shader_lconst *constant;
     struct list *ptr;
 
     ptr = list_head(clist);
     while (ptr)
     {
-        constant = LIST_ENTRY(ptr, struct local_constant, entry);
+        constant = LIST_ENTRY(ptr, struct wined3d_shader_lconst, entry);
         ptr = list_next(clist, ptr);
         HeapFree(GetProcessHeap(), 0, constant);
     }
@@ -512,7 +519,8 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
                 case WINED3DSPR_OUTPUT:
                     reg_maps->output_registers |= 1 << semantic.reg.reg.idx;
                     shader_signature_from_semantic(&output_signature[semantic.reg.reg.idx], &semantic);
-                    if (semantic.usage == WINED3DDECLUSAGE_FOG) reg_maps->fog = 1;
+                    if (semantic.usage == WINED3D_DECL_USAGE_FOG)
+                        reg_maps->fog = 1;
                     break;
 
                 /* Save sampler usage token. */
@@ -530,7 +538,7 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
             struct wined3d_shader_src_param rel_addr;
             struct wined3d_shader_dst_param dst;
 
-            local_constant *lconst = HeapAlloc(GetProcessHeap(), 0, sizeof(local_constant));
+            struct wined3d_shader_lconst *lconst = HeapAlloc(GetProcessHeap(), 0, sizeof(*lconst));
             if (!lconst) return E_OUTOFMEMORY;
 
             fe->shader_read_dst_param(fe_data, &ptr, &dst, &rel_addr);
@@ -560,7 +568,7 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
             struct wined3d_shader_src_param rel_addr;
             struct wined3d_shader_dst_param dst;
 
-            local_constant *lconst = HeapAlloc(GetProcessHeap(), 0, sizeof(local_constant));
+            struct wined3d_shader_lconst *lconst = HeapAlloc(GetProcessHeap(), 0, sizeof(*lconst));
             if (!lconst) return E_OUTOFMEMORY;
 
             fe->shader_read_dst_param(fe_data, &ptr, &dst, &rel_addr);
@@ -577,7 +585,7 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
             struct wined3d_shader_src_param rel_addr;
             struct wined3d_shader_dst_param dst;
 
-            local_constant *lconst = HeapAlloc(GetProcessHeap(), 0, sizeof(local_constant));
+            struct wined3d_shader_lconst *lconst = HeapAlloc(GetProcessHeap(), 0, sizeof(*lconst));
             if (!lconst) return E_OUTOFMEMORY;
 
             fe->shader_read_dst_param(fe_data, &ptr, &dst, &rel_addr);
@@ -589,33 +597,6 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
             list_add_head(&shader->constantsB, &lconst->entry);
             reg_maps->local_bool_consts |= (1 << dst.reg.idx);
         }
-        /* If there's a loop in the shader. */
-        else if (ins.handler_idx == WINED3DSIH_LOOP
-                || ins.handler_idx == WINED3DSIH_REP)
-        {
-            struct wined3d_shader_src_param src, rel_addr;
-
-            fe->shader_read_src_param(fe_data, &ptr, &src, &rel_addr);
-
-            /* Rep and Loop always use an integer constant for the control parameters. */
-            if (ins.handler_idx == WINED3DSIH_REP)
-            {
-                reg_maps->integer_constants |= 1 << src.reg.idx;
-            }
-            else
-            {
-                fe->shader_read_src_param(fe_data, &ptr, &src, &rel_addr);
-                reg_maps->integer_constants |= 1 << src.reg.idx;
-            }
-
-            cur_loop_depth++;
-            if (cur_loop_depth > max_loop_depth) max_loop_depth = cur_loop_depth;
-        }
-        else if (ins.handler_idx == WINED3DSIH_ENDLOOP
-                || ins.handler_idx == WINED3DSIH_ENDREP)
-        {
-            cur_loop_depth--;
-        }
         /* For subroutine prototypes. */
         else if (ins.handler_idx == WINED3DSIH_LABEL)
         {
@@ -660,19 +641,19 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
                                 case 0: /* oPos */
                                     reg_maps->output_registers |= 1 << 10;
                                     shader_signature_from_usage(&output_signature[10],
-                                            WINED3DDECLUSAGE_POSITION, 0, 10, WINED3DSP_WRITEMASK_ALL);
+                                            WINED3D_DECL_USAGE_POSITION, 0, 10, WINED3DSP_WRITEMASK_ALL);
                                     break;
 
                                 case 1: /* oFog */
                                     reg_maps->output_registers |= 1 << 11;
                                     shader_signature_from_usage(&output_signature[11],
-                                            WINED3DDECLUSAGE_FOG, 0, 11, WINED3DSP_WRITEMASK_0);
+                                            WINED3D_DECL_USAGE_FOG, 0, 11, WINED3DSP_WRITEMASK_0);
                                     break;
 
                                 case 2: /* oPts */
                                     reg_maps->output_registers |= 1 << 11;
                                     shader_signature_from_usage(&output_signature[11],
-                                            WINED3DDECLUSAGE_PSIZE, 0, 11, WINED3DSP_WRITEMASK_1);
+                                            WINED3D_DECL_USAGE_PSIZE, 0, 11, WINED3DSP_WRITEMASK_1);
                                     break;
                             }
                             break;
@@ -689,7 +670,7 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
                                 {
                                     reg_maps->output_registers |= 1 << idx;
                                     shader_signature_from_usage(&output_signature[idx],
-                                            WINED3DDECLUSAGE_COLOR, idx - 8, idx, dst_param.write_mask);
+                                            WINED3D_DECL_USAGE_COLOR, idx - 8, idx, dst_param.write_mask);
                                 }
                             }
                             break;
@@ -705,7 +686,7 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
                             {
                                 reg_maps->output_registers |= 1 << idx;
                                 shader_signature_from_usage(&output_signature[idx],
-                                        WINED3DDECLUSAGE_TEXCOORD, idx, idx, dst_param.write_mask);
+                                        WINED3D_DECL_USAGE_TEXCOORD, idx, idx, dst_param.write_mask);
                             }
                             break;
 
@@ -789,6 +770,16 @@ static HRESULT shader_get_registers_used(struct wined3d_shader *shader, const st
             else if (ins.handler_idx == WINED3DSIH_IFC) reg_maps->usesifc = 1;
             else if (ins.handler_idx == WINED3DSIH_CALL) reg_maps->usescall = 1;
             else if (ins.handler_idx == WINED3DSIH_POW) reg_maps->usespow = 1;
+            else if (ins.handler_idx == WINED3DSIH_LOOP
+                    || ins.handler_idx == WINED3DSIH_REP)
+            {
+                ++cur_loop_depth;
+                if (cur_loop_depth > max_loop_depth)
+                    max_loop_depth = cur_loop_depth;
+            }
+            else if (ins.handler_idx == WINED3DSIH_ENDLOOP
+                    || ins.handler_idx == WINED3DSIH_ENDREP)
+                --cur_loop_depth;
 
             limit = ins.src_count + (ins.predicate ? 1 : 0);
             for (i = 0; i < limit; ++i)
@@ -863,60 +854,60 @@ static void shader_dump_decl_usage(const struct wined3d_shader_semantic *semanti
 
         switch (semantic->usage)
         {
-            case WINED3DDECLUSAGE_POSITION:
+            case WINED3D_DECL_USAGE_POSITION:
                 TRACE("position%u", semantic->usage_idx);
                 break;
 
-            case WINED3DDECLUSAGE_BLENDINDICES:
+            case WINED3D_DECL_USAGE_BLEND_INDICES:
                 TRACE("blend");
                 break;
 
-            case WINED3DDECLUSAGE_BLENDWEIGHT:
+            case WINED3D_DECL_USAGE_BLEND_WEIGHT:
                 TRACE("weight");
                 break;
 
-            case WINED3DDECLUSAGE_NORMAL:
+            case WINED3D_DECL_USAGE_NORMAL:
                 TRACE("normal%u", semantic->usage_idx);
                 break;
 
-            case WINED3DDECLUSAGE_PSIZE:
+            case WINED3D_DECL_USAGE_PSIZE:
                 TRACE("psize");
                 break;
 
-            case WINED3DDECLUSAGE_COLOR:
+            case WINED3D_DECL_USAGE_COLOR:
                 if (!semantic->usage_idx) TRACE("color");
                 else TRACE("specular%u", (semantic->usage_idx - 1));
                 break;
 
-            case WINED3DDECLUSAGE_TEXCOORD:
+            case WINED3D_DECL_USAGE_TEXCOORD:
                 TRACE("texture%u", semantic->usage_idx);
                 break;
 
-            case WINED3DDECLUSAGE_TANGENT:
+            case WINED3D_DECL_USAGE_TANGENT:
                 TRACE("tangent");
                 break;
 
-            case WINED3DDECLUSAGE_BINORMAL:
+            case WINED3D_DECL_USAGE_BINORMAL:
                 TRACE("binormal");
                 break;
 
-            case WINED3DDECLUSAGE_TESSFACTOR:
+            case WINED3D_DECL_USAGE_TESS_FACTOR:
                 TRACE("tessfactor");
                 break;
 
-            case WINED3DDECLUSAGE_POSITIONT:
+            case WINED3D_DECL_USAGE_POSITIONT:
                 TRACE("positionT%u", semantic->usage_idx);
                 break;
 
-            case WINED3DDECLUSAGE_FOG:
+            case WINED3D_DECL_USAGE_FOG:
                 TRACE("fog");
                 break;
 
-            case WINED3DDECLUSAGE_DEPTH:
+            case WINED3D_DECL_USAGE_DEPTH:
                 TRACE("depth");
                 break;
 
-            case WINED3DDECLUSAGE_SAMPLE:
+            case WINED3D_DECL_USAGE_SAMPLE:
                 TRACE("sample");
                 break;
 
@@ -1099,7 +1090,7 @@ void shader_dump_dst_param(const struct wined3d_shader_dst_param *param,
 void shader_dump_src_param(const struct wined3d_shader_src_param *param,
         const struct wined3d_shader_version *shader_version)
 {
-    DWORD src_modifier = param->modifiers;
+    enum wined3d_shader_src_modifier src_modifier = param->modifiers;
     DWORD swizzle = param->swizzle;
 
     if (src_modifier == WINED3DSPSM_NEG
@@ -1439,12 +1430,12 @@ static void shader_trace_init(const struct wined3d_shader_frontend *fe, void *fe
             {
                 switch (ins.flags)
                 {
-                    case COMPARISON_GT: TRACE("_gt"); break;
-                    case COMPARISON_EQ: TRACE("_eq"); break;
-                    case COMPARISON_GE: TRACE("_ge"); break;
-                    case COMPARISON_LT: TRACE("_lt"); break;
-                    case COMPARISON_NE: TRACE("_ne"); break;
-                    case COMPARISON_LE: TRACE("_le"); break;
+                    case WINED3D_SHADER_REL_OP_GT: TRACE("_gt"); break;
+                    case WINED3D_SHADER_REL_OP_EQ: TRACE("_eq"); break;
+                    case WINED3D_SHADER_REL_OP_GE: TRACE("_ge"); break;
+                    case WINED3D_SHADER_REL_OP_LT: TRACE("_lt"); break;
+                    case WINED3D_SHADER_REL_OP_NE: TRACE("_ne"); break;
+                    case WINED3D_SHADER_REL_OP_LE: TRACE("_le"); break;
                     default: TRACE("_(%u)", ins.flags);
                 }
             }
@@ -1502,7 +1493,7 @@ static void shader_none_load_np2fixup_constants(void *shader_priv,
 static void shader_none_destroy(struct wined3d_shader *shader) {}
 static HRESULT shader_none_alloc(struct wined3d_device *device) {return WINED3D_OK;}
 static void shader_none_free(struct wined3d_device *device) {}
-static BOOL shader_none_dirty_const(void) {return FALSE;}
+static void shader_none_context_destroyed(void *shader_priv, const struct wined3d_context *context) {}
 
 static void shader_none_get_caps(const struct wined3d_gl_info *gl_info, struct shader_caps *caps)
 {
@@ -1547,7 +1538,7 @@ const struct wined3d_shader_backend_ops none_shader_backend =
     shader_none_destroy,
     shader_none_alloc,
     shader_none_free,
-    shader_none_dirty_const,
+    shader_none_context_destroyed,
     shader_none_get_caps,
     shader_none_color_fixup_supported,
 };
@@ -1559,6 +1550,7 @@ static HRESULT shader_set_function(struct wined3d_shader *shader, const DWORD *b
     struct wined3d_shader_reg_maps *reg_maps = &shader->reg_maps;
     const struct wined3d_shader_frontend *fe;
     HRESULT hr;
+    unsigned int backend_version;
 
     TRACE("shader %p, byte_code %p, output_signature %p, float_const_count %u.\n",
             shader, byte_code, output_signature, float_const_count);
@@ -1602,6 +1594,24 @@ static HRESULT shader_set_function(struct wined3d_shader *shader, const DWORD *b
         WARN("Shader version %d not supported by this D3D API version.\n", reg_maps->shader_version.major);
         return WINED3DERR_INVALIDCALL;
     }
+    switch (type)
+    {
+        case WINED3D_SHADER_TYPE_VERTEX:
+            backend_version = shader->device->vshader_version;
+            break;
+        case WINED3D_SHADER_TYPE_PIXEL:
+            backend_version = shader->device->pshader_version;
+            break;
+        default:
+            FIXME("No backend version-checking for this shader type\n");
+            backend_version = 0;
+    }
+    if (reg_maps->shader_version.major > backend_version)
+    {
+        WARN("Shader version %d.%d not supported by your GPU with the current shader backend.\n",
+                reg_maps->shader_version.major, reg_maps->shader_version.minor);
+        return WINED3DERR_INVALIDCALL;
+    }
 
     shader->function = HeapAlloc(GetProcessHeap(), 0, shader->functionLength);
     if (!shader->function)
@@ -1686,7 +1696,7 @@ HRESULT CDECL wined3d_shader_set_local_constants_float(struct wined3d_shader *sh
 
     for (i = start_idx; i < end_idx; ++i)
     {
-        struct local_constant *lconst = HeapAlloc(GetProcessHeap(), 0, sizeof(local_constant));
+        struct wined3d_shader_lconst *lconst = HeapAlloc(GetProcessHeap(), 0, sizeof(*lconst));
         if (!lconst)
             return E_OUTOFMEMORY;
 
@@ -1701,19 +1711,23 @@ HRESULT CDECL wined3d_shader_set_local_constants_float(struct wined3d_shader *sh
 void find_vs_compile_args(const struct wined3d_state *state,
         const struct wined3d_shader *shader, struct vs_compile_args *args)
 {
-    args->fog_src = state->render_states[WINED3DRS_FOGTABLEMODE]
-            == WINED3DFOG_NONE ? VS_FOG_COORD : VS_FOG_Z;
-    args->clip_enabled = state->render_states[WINED3DRS_CLIPPING]
-            && state->render_states[WINED3DRS_CLIPPLANEENABLE];
+    args->fog_src = state->render_states[WINED3D_RS_FOGTABLEMODE]
+            == WINED3D_FOG_NONE ? VS_FOG_COORD : VS_FOG_Z;
+    args->clip_enabled = state->render_states[WINED3D_RS_CLIPPING]
+            && state->render_states[WINED3D_RS_CLIPPLANEENABLE];
     args->swizzle_map = shader->device->strided_streams.swizzle_map;
 }
 
 static BOOL match_usage(BYTE usage1, BYTE usage_idx1, BYTE usage2, BYTE usage_idx2)
 {
-    if (usage_idx1 != usage_idx2) return FALSE;
-    if (usage1 == usage2) return TRUE;
-    if (usage1 == WINED3DDECLUSAGE_POSITION && usage2 == WINED3DDECLUSAGE_POSITIONT) return TRUE;
-    if (usage2 == WINED3DDECLUSAGE_POSITION && usage1 == WINED3DDECLUSAGE_POSITIONT) return TRUE;
+    if (usage_idx1 != usage_idx2)
+        return FALSE;
+    if (usage1 == usage2)
+        return TRUE;
+    if (usage1 == WINED3D_DECL_USAGE_POSITION && usage2 == WINED3D_DECL_USAGE_POSITIONT)
+        return TRUE;
+    if (usage2 == WINED3D_DECL_USAGE_POSITION && usage1 == WINED3D_DECL_USAGE_POSITIONT)
+        return TRUE;
 
     return FALSE;
 }
@@ -1891,7 +1905,7 @@ void find_ps_compile_args(const struct wined3d_state *state,
     UINT i;
 
     memset(args, 0, sizeof(*args)); /* FIXME: Make sure all bits are set. */
-    if (state->render_states[WINED3DRS_SRGBWRITEENABLE])
+    if (state->render_states[WINED3D_RS_SRGBWRITEENABLE])
     {
         const struct wined3d_surface *rt = state->fb->render_targets[0];
         if (rt->resource.format->flags & WINED3DFMT_FLAG_SRGB_WRITE) args->srgb_correction = 1;
@@ -1902,11 +1916,53 @@ void find_ps_compile_args(const struct wined3d_state *state,
     {
         for (i = 0; i < 4; ++i)
         {
-            DWORD flags = state->texture_states[i][WINED3DTSS_TEXTURETRANSFORMFLAGS];
-            DWORD tex_transform = flags & ~WINED3DTTFF_PROJECTED;
-            if (flags & WINED3DTTFF_PROJECTED)
-                tex_transform |= WINED3D_PSARGS_PROJECTED;
-            args->tex_transform |= tex_transform << i * WINED3D_PSARGS_TEXTRANSFORM_SHIFT;
+            DWORD flags = state->texture_states[i][WINED3D_TSS_TEXTURE_TRANSFORM_FLAGS];
+
+            if (flags & WINED3D_TTFF_PROJECTED)
+            {
+                DWORD tex_transform = flags & ~WINED3D_TTFF_PROJECTED;
+
+                if (!state->vertex_shader)
+                {
+                    unsigned int j;
+                    unsigned int index = state->texture_states[i][WINED3D_TSS_TEXCOORD_INDEX];
+                    DWORD max_valid = WINED3D_TTFF_COUNT4;
+                    enum wined3d_sampler_texture_type sampler_type = shader->reg_maps.sampler_type[i];
+
+                    for (j = 0; j < state->vertex_declaration->element_count; ++j)
+                    {
+                        struct wined3d_vertex_declaration_element *element =
+                                &state->vertex_declaration->elements[j];
+
+                        if (element->usage == WINED3D_DECL_USAGE_TEXCOORD
+                                && element->usage_idx == index)
+                        {
+                            max_valid = element->format->component_count;
+                            break;
+                        }
+                    }
+                    if (!tex_transform || tex_transform > max_valid)
+                    {
+                        WARN("Fixing up projected texture transform flags from %#x to %#x.\n",
+                                tex_transform, max_valid);
+                        tex_transform = max_valid;
+                    }
+                    if ((sampler_type == WINED3DSTT_1D && tex_transform > WINED3D_TTFF_COUNT1)
+                            || (sampler_type == WINED3DSTT_2D && tex_transform > WINED3D_TTFF_COUNT2)
+                            || (sampler_type == WINED3DSTT_VOLUME && tex_transform > WINED3D_TTFF_COUNT3))
+                        tex_transform |= WINED3D_PSARGS_PROJECTED;
+                    else
+                    {
+                        WARN("Application requested projected texture with unsuitable texture coordinates.\n");
+                        WARN("(texture unit %u, transform flags %#x, sampler type %u).\n",
+                                i, tex_transform, sampler_type);
+                    }
+                }
+                else
+                    tex_transform = WINED3D_TTFF_COUNT4 | WINED3D_PSARGS_PROJECTED;
+
+                args->tex_transform |= tex_transform << i * WINED3D_PSARGS_TEXTRANSFORM_SHIFT;
+            }
         }
     }
 
@@ -1949,29 +2005,29 @@ void find_ps_compile_args(const struct wined3d_state *state,
     else
     {
         args->vp_mode = vertexshader;
-        if (state->render_states[WINED3DRS_FOGENABLE])
+        if (state->render_states[WINED3D_RS_FOGENABLE])
         {
-            switch (state->render_states[WINED3DRS_FOGTABLEMODE])
+            switch (state->render_states[WINED3D_RS_FOGTABLEMODE])
             {
-                case WINED3DFOG_NONE:
+                case WINED3D_FOG_NONE:
                     if (device->strided_streams.position_transformed || use_vs(state))
                     {
                         args->fog = FOG_LINEAR;
                         break;
                     }
 
-                    switch (state->render_states[WINED3DRS_FOGVERTEXMODE])
+                    switch (state->render_states[WINED3D_RS_FOGVERTEXMODE])
                     {
-                        case WINED3DFOG_NONE: /* Fall through. */
-                        case WINED3DFOG_LINEAR: args->fog = FOG_LINEAR; break;
-                        case WINED3DFOG_EXP:    args->fog = FOG_EXP;    break;
-                        case WINED3DFOG_EXP2:   args->fog = FOG_EXP2;   break;
+                        case WINED3D_FOG_NONE: /* Fall through. */
+                        case WINED3D_FOG_LINEAR: args->fog = FOG_LINEAR; break;
+                        case WINED3D_FOG_EXP:    args->fog = FOG_EXP;    break;
+                        case WINED3D_FOG_EXP2:   args->fog = FOG_EXP2;   break;
                     }
                     break;
 
-                case WINED3DFOG_LINEAR: args->fog = FOG_LINEAR; break;
-                case WINED3DFOG_EXP:    args->fog = FOG_EXP;    break;
-                case WINED3DFOG_EXP2:   args->fog = FOG_EXP2;   break;
+                case WINED3D_FOG_LINEAR: args->fog = FOG_LINEAR; break;
+                case WINED3D_FOG_EXP:    args->fog = FOG_EXP;    break;
+                case WINED3D_FOG_EXP2:   args->fog = FOG_EXP2;   break;
             }
         }
         else
@@ -2138,7 +2194,7 @@ static HRESULT pixelshader_init(struct wined3d_shader *shader, struct wined3d_de
 
 void pixelshader_update_samplers(struct wined3d_shader_reg_maps *reg_maps, struct wined3d_texture * const *textures)
 {
-    WINED3DSAMPLER_TEXTURE_TYPE *sampler_type = reg_maps->sampler_type;
+    enum wined3d_sampler_texture_type *sampler_type = reg_maps->sampler_type;
     unsigned int i;
 
     if (reg_maps->shader_version.major != 1) return;
