@@ -23,6 +23,7 @@
  */
 
 #include "config.h"
+#include "wine/port.h"
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
@@ -828,10 +829,10 @@ HRESULT CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock)
     {
         if (!(map & 1)) continue;
 
-        if (memcmp(src_state->clip_planes[i], stateblock->state.clip_planes[i], sizeof(*stateblock->state.clip_planes)))
+        if (memcmp(&stateblock->state.clip_planes[i], &src_state->clip_planes[i], sizeof(src_state->clip_planes[i])))
         {
             TRACE("Updating clipplane %u.\n", i);
-            memcpy(stateblock->state.clip_planes[i], src_state->clip_planes[i], sizeof(*stateblock->state.clip_planes));
+            stateblock->state.clip_planes[i] = src_state->clip_planes[i];
         }
     }
 
@@ -1057,15 +1058,9 @@ HRESULT CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblo
     map = stateblock->changed.clipplane;
     for (i = 0; map; map >>= 1, ++i)
     {
-        float clip[4];
-
         if (!(map & 1)) continue;
 
-        clip[0] = (float) stateblock->state.clip_planes[i][0];
-        clip[1] = (float) stateblock->state.clip_planes[i][1];
-        clip[2] = (float) stateblock->state.clip_planes[i][2];
-        clip[3] = (float) stateblock->state.clip_planes[i][3];
-        wined3d_device_set_clip_plane(device, i, clip);
+        wined3d_device_set_clip_plane(device, i, &stateblock->state.clip_planes[i]);
     }
 
     stateblock->device->stateBlock->state.lowest_disabled_stage = MAX_TEXTURES - 1;
@@ -1108,11 +1103,11 @@ void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
     memset(stateblock->changed.vertexShaderConstantsF, 0, device->d3d_vshader_constantF * sizeof(BOOL));
 
     /* Set some of the defaults for lights, transforms etc */
-    memcpy(&state->transforms[WINED3D_TS_PROJECTION], identity, sizeof(identity));
-    memcpy(&state->transforms[WINED3D_TS_VIEW], identity, sizeof(identity));
+    memcpy(&state->transforms[WINED3D_TS_PROJECTION], &identity, sizeof(identity));
+    memcpy(&state->transforms[WINED3D_TS_VIEW], &identity, sizeof(identity));
     for (i = 0; i < 256; ++i)
     {
-        memcpy(&state->transforms[WINED3D_TS_WORLD_MATRIX(i)], identity, sizeof(identity));
+        memcpy(&state->transforms[WINED3D_TS_WORLD_MATRIX(i)], &identity, sizeof(identity));
     }
 
     state->fb = &device->fb;
@@ -1253,7 +1248,7 @@ void stateblock_init_default_state(struct wined3d_stateblock *stateblock)
     for (i = 0; i < MAX_TEXTURES; ++i)
     {
         TRACE("Setting up default texture states for texture Stage %u.\n", i);
-        memcpy(&state->transforms[WINED3D_TS_TEXTURE0 + i], identity, sizeof(identity));
+        memcpy(&state->transforms[WINED3D_TS_TEXTURE0 + i], &identity, sizeof(identity));
         state->texture_states[i][WINED3D_TSS_COLOR_OP] = i ? WINED3D_TOP_DISABLE : WINED3D_TOP_MODULATE;
         state->texture_states[i][WINED3D_TSS_COLOR_ARG1] = WINED3DTA_TEXTURE;
         state->texture_states[i][WINED3D_TSS_COLOR_ARG2] = WINED3DTA_CURRENT;
