@@ -21,6 +21,7 @@
  */
 
 #include "config.h"
+#include "wine/port.h"
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
@@ -30,6 +31,7 @@ WINE_DECLARE_DEBUG_CHANNEL(fps);
 static void swapchain_cleanup(struct wined3d_swapchain *swapchain)
 {
     struct wined3d_display_mode mode;
+    HRESULT hr;
     UINT i;
 
     TRACE("Destroying swapchain %p.\n", swapchain);
@@ -78,7 +80,10 @@ static void swapchain_cleanup(struct wined3d_swapchain *swapchain)
         mode.height = swapchain->orig_height;
         mode.refresh_rate = 0;
         mode.format_id = swapchain->orig_fmt;
-        wined3d_device_set_display_mode(swapchain->device, 0, &mode);
+        mode.scanline_ordering = WINED3D_SCANLINE_ORDERING_UNKNOWN;
+        if (FAILED(hr = wined3d_set_adapter_display_mode(swapchain->device->wined3d,
+                swapchain->device->adapter->ordinal, &mode)))
+            ERR("Failed to restore display mode, hr %#x.\n", hr);
     }
 
     if (swapchain->backup_dc)
@@ -145,10 +150,20 @@ HRESULT CDECL wined3d_swapchain_present(struct wined3d_swapchain *swapchain,
             swapchain, wine_dbgstr_rect(src_rect), wine_dbgstr_rect(dst_rect),
             dst_window_override, dirty_region, flags);
 
+    if (flags)
+        FIXME("Ignoring flags %#x.\n", flags);
+
+    if (!swapchain->back_buffers)
+    {
+        WARN("Swapchain doesn't have a backbuffer, returning WINED3DERR_INVALIDCALL\n");
+        return WINED3DERR_INVALIDCALL;
+    }
+
     wined3d_swapchain_set_window(swapchain, dst_window_override);
 
-    return swapchain->swapchain_ops->swapchain_present(swapchain,
-            src_rect, dst_rect, dirty_region, flags);
+    swapchain->swapchain_ops->swapchain_present(swapchain, src_rect, dst_rect, dirty_region, flags);
+
+    return WINED3D_OK;
 }
 
 HRESULT CDECL wined3d_swapchain_get_front_buffer_data(const struct wined3d_swapchain *swapchain,
@@ -227,7 +242,7 @@ HRESULT CDECL wined3d_swapchain_get_raster_status(const struct wined3d_swapchain
     if (!QueryPerformanceCounter(&counter) || !QueryPerformanceFrequency(&freq_per_sec))
         return WINED3DERR_INVALIDCALL;
 
-    if (FAILED(wined3d_swapchain_get_display_mode(swapchain, &mode)))
+    if (FAILED(wined3d_swapchain_get_display_mode(swapchain, &mode, NULL)))
         return WINED3DERR_INVALIDCALL;
     if (mode.refresh_rate == DEFAULT_REFRESH_RATE)
         mode.refresh_rate = 60;
@@ -249,13 +264,14 @@ HRESULT CDECL wined3d_swapchain_get_raster_status(const struct wined3d_swapchain
 }
 
 HRESULT CDECL wined3d_swapchain_get_display_mode(const struct wined3d_swapchain *swapchain,
-        struct wined3d_display_mode *mode)
+        struct wined3d_display_mode *mode, enum wined3d_display_rotation *rotation)
 {
     HRESULT hr;
 
-    TRACE("swapchain %p, mode %p.\n", swapchain, mode);
+    TRACE("swapchain %p, mode %p, rotation %p.\n", swapchain, mode, rotation);
 
-    hr = wined3d_get_adapter_display_mode(swapchain->device->wined3d, swapchain->device->adapter->ordinal, mode);
+    hr = wined3d_get_adapter_display_mode(swapchain->device->wined3d,
+            swapchain->device->adapter->ordinal, mode, rotation);
 
     TRACE("Returning w %u, h %u, refresh rate %u, format %s.\n",
             mode->width, mode->height, mode->refresh_rate, debug_d3dformat(mode->format_id));
@@ -446,21 +462,22 @@ static void swapchain_blit(const struct wined3d_swapchain *swapchain,
     }
 }
 
-static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const RECT *src_rect_in,
+static void swapchain_gl_present(struct wined3d_swapchain *swapchain, const RECT *src_rect_in,
         const RECT *dst_rect_in, const RGNDATA *dirty_region, DWORD flags)
 {
+    struct wined3d_surface *back_buffer = swapchain->back_buffers[0];
     const struct wined3d_fb_state *fb = &swapchain->device->fb;
     const struct wined3d_gl_info *gl_info;
     struct wined3d_context *context;
     RECT src_rect, dst_rect;
     BOOL render_to_fbo;
 
-    context = context_acquire(swapchain->device, swapchain->back_buffers[0]);
+    context = context_acquire(swapchain->device, back_buffer);
     if (!context->valid)
     {
         context_release(context);
         WARN("Invalid context, skipping present.\n");
-        return WINED3D_OK;
+        return;
     }
 
     gl_info = context->gl_info;
@@ -481,7 +498,7 @@ static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const R
         TRACE("Rendering the cursor. Creating fake surface at %p\n", &cursor);
         /* Build a fake surface to call the Blitting code. It is not possible to use the interface passed by
          * the application because we are only supposed to copy the information out. Using a fake surface
-         * allows to use the Blitting engine and avoid copying the whole texture -> render target blitting code.
+         * allows us to use the Blitting engine and avoid copying the whole texture -> render target blitting code.
          */
         memset(&cursor, 0, sizeof(cursor));
         cursor.resource.ref = 1;
@@ -504,8 +521,8 @@ static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const R
          */
         if (swapchain->desc.windowed)
             MapWindowPoints(NULL, swapchain->win_handle, (POINT *)&destRect, 2);
-        wined3d_surface_blt(swapchain->back_buffers[0], &destRect,
-                &cursor, NULL, WINEDDBLT_KEYSRC, NULL, WINED3D_TEXF_POINT);
+        wined3d_surface_blt(back_buffer, &destRect, &cursor, NULL, WINEDDBLT_KEYSRC,
+                NULL, WINED3D_TEXF_POINT);
     }
 
     if (swapchain->device->logo_surface)
@@ -514,8 +531,8 @@ static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const R
         RECT rect = {0, 0, src_surface->resource.width, src_surface->resource.height};
 
         /* Blit the logo into the upper left corner of the drawable. */
-        wined3d_surface_blt(swapchain->back_buffers[0], &rect, src_surface, &rect,
-                 WINEDDBLT_KEYSRC, NULL, WINED3D_TEXF_POINT);
+        wined3d_surface_blt(back_buffer, &rect, src_surface, &rect, WINEDDBLT_KEYSRC,
+                NULL, WINED3D_TEXF_POINT);
     }
 
     TRACE("Presenting HDC %p.\n", context->hdc);
@@ -559,11 +576,15 @@ static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const R
      */
     if (!swapchain->render_to_fbo && render_to_fbo && wined3d_settings.offscreen_rendering_mode == ORM_FBO)
     {
-        surface_load_location(swapchain->back_buffers[0], SFLAG_INTEXTURE, NULL);
-        surface_modify_location(swapchain->back_buffers[0], SFLAG_INDRAWABLE, FALSE);
+        surface_load_location(back_buffer, SFLAG_INTEXTURE, NULL);
+        surface_modify_location(back_buffer, SFLAG_INDRAWABLE, FALSE);
         swapchain->render_to_fbo = TRUE;
         swapchain_update_draw_bindings(swapchain);
     }
+    else
+    {
+        surface_load_location(back_buffer, back_buffer->draw_binding, NULL);
+    }
 
     if (swapchain->render_to_fbo)
     {
@@ -626,17 +647,17 @@ static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const R
     }
 
     if (!swapchain->render_to_fbo && ((swapchain->front_buffer->flags & SFLAG_INSYSMEM)
-            || (swapchain->back_buffers[0]->flags & SFLAG_INSYSMEM)))
+            || (back_buffer->flags & SFLAG_INSYSMEM)))
     {
         /* Both memory copies of the surfaces are ok, flip them around too instead of dirtifying
          * Doesn't work with render_to_fbo because we're not flipping
          */
         struct wined3d_surface *front = swapchain->front_buffer;
-        struct wined3d_surface *back = swapchain->back_buffers[0];
 
-        if(front->resource.size == back->resource.size) {
+        if (front->resource.size == back_buffer->resource.size)
+        {
             DWORD fbflags;
-            flip_surface(front, back);
+            flip_surface(front, back_buffer);
 
             /* Tell the front buffer surface that is has been modified. However,
              * the other locations were preserved during that, so keep the flags.
@@ -648,7 +669,7 @@ static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const R
         else
         {
             surface_modify_location(front, SFLAG_INDRAWABLE, TRUE);
-            surface_modify_location(back, SFLAG_INDRAWABLE, TRUE);
+            surface_modify_location(back_buffer, SFLAG_INDRAWABLE, TRUE);
         }
     }
     else
@@ -660,7 +681,7 @@ static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const R
          * the texture / sysmem copy needs to be reloaded from the drawable
          */
         if (swapchain->desc.swap_effect == WINED3D_SWAP_EFFECT_FLIP)
-            surface_modify_location(swapchain->back_buffers[0], swapchain->back_buffers[0]->draw_binding, TRUE);
+            surface_modify_location(back_buffer, back_buffer->draw_binding, TRUE);
     }
 
     if (fb->depth_stencil)
@@ -680,9 +701,6 @@ static HRESULT swapchain_gl_present(struct wined3d_swapchain *swapchain, const R
     }
 
     context_release(context);
-
-    TRACE("returning\n");
-    return WINED3D_OK;
 }
 
 static const struct wined3d_swapchain_ops swapchain_gl_ops =
@@ -705,7 +723,7 @@ void x11_copy_to_screen(const struct wined3d_swapchain *swapchain, const RECT *r
     if (!(front->resource.usage & WINED3DUSAGE_RENDERTARGET))
         return;
 
-    if (front->flags & SFLAG_LOCKED)
+    if (front->resource.map_count)
         ERR("Trying to blit a mapped surface.\n");
 
     TRACE("Copying surface %p to screen.\n", front);
@@ -735,16 +753,11 @@ void x11_copy_to_screen(const struct wined3d_swapchain *swapchain, const RECT *r
     ReleaseDC(window, dst_dc);
 }
 
-static HRESULT swapchain_gdi_present(struct wined3d_swapchain *swapchain, const RECT *src_rect_in,
+static void swapchain_gdi_present(struct wined3d_swapchain *swapchain, const RECT *src_rect_in,
         const RECT *dst_rect_in, const RGNDATA *dirty_region, DWORD flags)
 {
     struct wined3d_surface *front, *back;
 
-    if (!swapchain->back_buffers)
-    {
-        WARN("Swapchain doesn't have a backbuffer, returning WINED3DERR_INVALIDCALL\n");
-        return WINED3DERR_INVALIDCALL;
-    }
     front = swapchain->front_buffer;
     back = swapchain->back_buffers[0];
 
@@ -801,8 +814,6 @@ static HRESULT swapchain_gdi_present(struct wined3d_swapchain *swapchain, const
     }
 
     x11_copy_to_screen(swapchain, NULL);
-
-    return WINED3D_OK;
 }
 
 static const struct wined3d_swapchain_ops swapchain_gdi_ops =
@@ -898,7 +909,7 @@ static HRESULT swapchain_init(struct wined3d_swapchain *swapchain, enum wined3d_
     swapchain->win_handle = window;
     swapchain->device_window = window;
 
-    wined3d_get_adapter_display_mode(device->wined3d, adapter->ordinal, &mode);
+    wined3d_get_adapter_display_mode(device->wined3d, adapter->ordinal, &mode, NULL);
     swapchain->orig_width = mode.width;
     swapchain->orig_height = mode.height;
     swapchain->orig_fmt = mode.format_id;
@@ -932,12 +943,11 @@ static HRESULT swapchain_init(struct wined3d_swapchain *swapchain, enum wined3d_
     swapchain_update_render_to_fbo(swapchain);
 
     TRACE("Creating front buffer.\n");
-    hr = device->device_parent->ops->create_rendertarget(device->device_parent, parent,
+    if (FAILED(hr = device->device_parent->ops->create_swapchain_surface(device->device_parent, parent,
             swapchain->desc.backbuffer_width, swapchain->desc.backbuffer_height,
-            swapchain->desc.backbuffer_format, swapchain->desc.multisample_type,
-            swapchain->desc.multisample_quality, TRUE /* Lockable */,
-            &swapchain->front_buffer);
-    if (FAILED(hr))
+            swapchain->desc.backbuffer_format, WINED3DUSAGE_RENDERTARGET,
+            swapchain->desc.multisample_type, swapchain->desc.multisample_quality,
+            &swapchain->front_buffer)))
     {
         WARN("Failed to create front buffer, hr %#x.\n", hr);
         goto err;
@@ -960,9 +970,9 @@ static HRESULT swapchain_init(struct wined3d_swapchain *swapchain, enum wined3d_
         mode.height = desc->backbuffer_height;
         mode.format_id = desc->backbuffer_format;
         mode.refresh_rate = desc->refresh_rate;
+        mode.scanline_ordering = WINED3D_SCANLINE_ORDERING_UNKNOWN;
 
-        hr = wined3d_device_set_display_mode(device, 0, &mode);
-        if (FAILED(hr))
+        if (FAILED(hr = wined3d_set_adapter_display_mode(device->wined3d, device->adapter->ordinal, &mode)))
         {
             WARN("Failed to set display mode, hr %#x.\n", hr);
             goto err;
@@ -1042,12 +1052,11 @@ static HRESULT swapchain_init(struct wined3d_swapchain *swapchain, enum wined3d_
         for (i = 0; i < swapchain->desc.backbuffer_count; ++i)
         {
             TRACE("Creating back buffer %u.\n", i);
-            hr = device->device_parent->ops->create_rendertarget(device->device_parent, parent,
+            if (FAILED(hr = device->device_parent->ops->create_swapchain_surface(device->device_parent, parent,
                     swapchain->desc.backbuffer_width, swapchain->desc.backbuffer_height,
-                    swapchain->desc.backbuffer_format, swapchain->desc.multisample_type,
-                    swapchain->desc.multisample_quality, TRUE /* Lockable */,
-                    &swapchain->back_buffers[i]);
-            if (FAILED(hr))
+                    swapchain->desc.backbuffer_format, WINED3DUSAGE_RENDERTARGET,
+                    swapchain->desc.multisample_type, swapchain->desc.multisample_quality,
+                    &swapchain->back_buffers[i])))
             {
                 WARN("Failed to create back buffer %u, hr %#x.\n", i, hr);
                 goto err;
@@ -1063,12 +1072,11 @@ static HRESULT swapchain_init(struct wined3d_swapchain *swapchain, enum wined3d_
         TRACE("Creating depth/stencil buffer.\n");
         if (!device->auto_depth_stencil)
         {
-            hr = device->device_parent->ops->create_depth_stencil(device->device_parent,
-                    swapchain->desc.backbuffer_width, swapchain->desc.backbuffer_height,
-                    swapchain->desc.auto_depth_stencil_format, swapchain->desc.multisample_type,
-                    swapchain->desc.multisample_quality, FALSE /* FIXME: Discard */,
-                    &device->auto_depth_stencil);
-            if (FAILED(hr))
+            if (FAILED(hr = device->device_parent->ops->create_swapchain_surface(device->device_parent,
+                    device->device_parent, swapchain->desc.backbuffer_width, swapchain->desc.backbuffer_height,
+                    swapchain->desc.auto_depth_stencil_format, WINED3DUSAGE_DEPTHSTENCIL,
+                    swapchain->desc.multisample_type, swapchain->desc.multisample_quality,
+                    &device->auto_depth_stencil)))
             {
                 WARN("Failed to create the auto depth stencil, hr %#x.\n", hr);
                 goto err;
