@@ -21,6 +21,7 @@
  */
 
 #include "config.h"
+#include "wine/port.h"
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d_texture);
@@ -856,9 +857,8 @@ static HRESULT cubetexture_init(struct wined3d_texture *texture, UINT edge_lengt
             UINT idx = j * texture->level_count + i;
             struct wined3d_surface *surface;
 
-            hr = device->device_parent->ops->create_surface(device->device_parent, parent, tmp_w, tmp_w,
-                    format_id, usage, pool, i /* Level */, j, &surface);
-            if (FAILED(hr))
+            if (FAILED(hr = device->device_parent->ops->create_texture_surface(device->device_parent, parent,
+                    tmp_w, tmp_w, format_id, usage, pool, i /* Level */, j, &surface)))
             {
                 FIXME("(%p) Failed to create surface, hr %#x.\n", texture, hr);
                 wined3d_texture_cleanup(texture);
@@ -1012,9 +1012,8 @@ static HRESULT texture_init(struct wined3d_texture *texture, UINT width, UINT he
         struct wined3d_surface *surface;
 
         /* Use the callback to create the texture surface. */
-        hr = device->device_parent->ops->create_surface(device->device_parent, parent, tmp_w, tmp_h,
-                format->id, usage, pool, i, 0, &surface);
-        if (FAILED(hr))
+        if (FAILED(hr = device->device_parent->ops->create_texture_surface(device->device_parent, parent,
+                tmp_w, tmp_h, format->id, usage, pool, i, 0, &surface)))
         {
             FIXME("Failed to create surface %p, hr %#x\n", texture, hr);
             wined3d_texture_cleanup(texture);
