@@ -23,6 +23,7 @@
  */
 
 #include "config.h"
+#include "wine/port.h"
 
 #include "initguid.h"
 #include "wined3d_private.h"
@@ -90,19 +91,19 @@ struct wined3d_settings wined3d_settings =
 };
 
 /* Do not call while under the GL lock. */
-struct wined3d * CDECL wined3d_create(UINT version, DWORD flags, void *parent)
+struct wined3d * CDECL wined3d_create(UINT version, DWORD flags)
 {
     struct wined3d *object;
     HRESULT hr;
 
-    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
+    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, FIELD_OFFSET(struct wined3d, adapters[1]));
     if (!object)
     {
         ERR("Failed to allocate wined3d object memory.\n");
         return NULL;
     }
 
-    hr = wined3d_init(object, version, flags, parent);
+    hr = wined3d_init(object, version, flags);
     if (FAILED(hr))
     {
         WARN("Failed to initialize wined3d object, hr %#x.\n", hr);
@@ -260,12 +261,7 @@ static BOOL wined3d_dll_init(HINSTANCE hInstDLL)
         }
         if ( !get_config_key( hkey, appkey, "RenderTargetLockMode", buffer, size) )
         {
-            if (!strcmp(buffer,"disabled"))
-            {
-                TRACE("Disabling render target locking\n");
-                wined3d_settings.rendertargetlock_mode = RTL_DISABLE;
-            }
-            else if (!strcmp(buffer,"readdraw"))
+            if (!strcmp(buffer,"readdraw"))
             {
                 TRACE("Using glReadPixels for render target reading and glDrawPixels for writing\n");
                 wined3d_settings.rendertargetlock_mode = RTL_READDRAW;
@@ -387,6 +383,8 @@ static BOOL wined3d_dll_destroy(HINSTANCE hInstDLL)
     HeapFree(GetProcessHeap(), 0, wined3d_settings.logo);
     UnregisterClassA(WINED3D_OPENGL_WINDOW_CLASS_NAME, hInstDLL);
 
+    DeleteCriticalSection(&wined3d_wndproc_cs);
+    DeleteCriticalSection(&wined3d_cs);
     return TRUE;
 }
 
@@ -447,7 +445,11 @@ static LRESULT CALLBACK wined3d_wndproc(HWND window, UINT message, WPARAM wparam
     proc = entry->proc;
     wined3d_wndproc_mutex_unlock();
 
-    return device_process_message(device, window, unicode, message, wparam, lparam, proc);
+    if (device)
+        return device_process_message(device, window, unicode, message, wparam, lparam, proc);
+    if (unicode)
+        return CallWindowProcW(proc, window, message, wparam, lparam);
+    return CallWindowProcA(proc, window, message, wparam, lparam);
 }
 
 BOOL wined3d_register_window(HWND window, struct wined3d_device *device)
@@ -518,6 +520,7 @@ void wined3d_unregister_window(HWND window)
         proc = GetWindowLongPtrW(window, GWLP_WNDPROC);
         if (proc != (LONG_PTR)wined3d_wndproc)
         {
+            entry->device = NULL;
             wined3d_wndproc_mutex_unlock();
             WARN("Not unregistering window %p, window proc %#lx doesn't match wined3d window proc %p.\n",
                     window, proc, wined3d_wndproc);
@@ -531,6 +534,7 @@ void wined3d_unregister_window(HWND window)
         proc = GetWindowLongPtrA(window, GWLP_WNDPROC);
         if (proc != (LONG_PTR)wined3d_wndproc)
         {
+            entry->device = NULL;
             wined3d_wndproc_mutex_unlock();
             WARN("Not unregistering window %p, window proc %#lx doesn't match wined3d window proc %p.\n",
                     window, proc, wined3d_wndproc);
