@@ -60,16 +60,7 @@
 #ifdef HAVE_SYS_POLL_H
 #include <sys/poll.h>
 #endif
-#if defined(HAVE_SYS_SOUNDCARD_H)
-# include <sys/soundcard.h>
-#elif defined(HAVE_MACHINE_SOUNDCARD_H)
-# include <machine/soundcard.h>
-#elif defined(HAVE_SOUNDCARD_H)
-# include <soundcard.h>
-#endif
-#ifdef HAVE_SYS_ERRNO_H
-#include <sys/errno.h>
-#endif
+#include <sys/soundcard.h>
 
 #include "windef.h"
 #include "winbase.h"
@@ -161,7 +152,7 @@ static	int 	MIDI_UnixToWindowsDeviceType(int type)
      * MOD_SQSYNTH      square wave internal synth
      * MOD_FMSYNTH      FM internal synth
      * MOD_MAPPER       MIDI mapper
-     * MOD_WAVETABLE    hardware watetable internal synth
+     * MOD_WAVETABLE    hardware wavetable internal synth
      * MOD_SWSYNTH      software internal synth
      */
 
@@ -1543,10 +1534,10 @@ static DWORD modLongData(WORD wDevID, LPMIDIHDR lpMidiHdr, DWORD dwSize)
     /* FIXME: MS doc is not 100% clear. Will lpData only contain system exclusive
      * data, or can it also contain raw MIDI data, to be split up and sent to
      * modShortData() ?
-     * If the latest is true, then the following WARNing will fire up
+     * If the latter is true, then the following WARNing will fire up
      */
     if (lpData[0] != 0xF0 || lpData[lpMidiHdr->dwBufferLength - 1] != 0xF7) {
-	WARN("Alledged system exclusive buffer is not correct\n\tPlease report with MIDI file\n");
+	WARN("The allegedly system exclusive buffer is not correct\n\tPlease report with MIDI file\n");
     }
 
     TRACE("dwBufferLength=%u !\n", lpMidiHdr->dwBufferLength);
@@ -1781,4 +1772,29 @@ DWORD WINAPI OSS_modMessage(UINT wDevID, UINT wMsg, DWORD_PTR dwUser,
     return MMSYSERR_NOTSUPPORTED;
 }
 
-/*-----------------------------------------------------------------------*/
+/**************************************************************************
+ * 				DriverProc (WINEOSS.1)
+ */
+LRESULT CALLBACK OSS_DriverProc(DWORD_PTR dwDevID, HDRVR hDriv, UINT wMsg,
+                                LPARAM dwParam1, LPARAM dwParam2)
+{
+     TRACE("(%08lX, %p, %08X, %08lX, %08lX)\n",
+           dwDevID, hDriv, wMsg, dwParam1, dwParam2);
+
+    switch(wMsg) {
+    case DRV_LOAD:
+    case DRV_FREE:
+    case DRV_OPEN:
+    case DRV_CLOSE:
+    case DRV_ENABLE:
+    case DRV_DISABLE:
+    case DRV_QUERYCONFIGURE:
+    case DRV_CONFIGURE:
+        return 1;
+    case DRV_INSTALL:
+    case DRV_REMOVE:
+        return DRV_SUCCESS;
+    default:
+	return 0;
+    }
+}
