@@ -27,7 +27,6 @@
 #include "winbase.h"
 #include "winreg.h"
 #include "x11drv.h"
-#include "x11font.h"
 #include "ddrawi.h"
 #include "wine/debug.h"
 
@@ -42,11 +41,8 @@ static int horz_size;     /* horz. size of screen in millimeters */
 static int vert_size;     /* vert. size of screen in millimeters */
 static int palette_size;
 static int device_init_done;
-unsigned int text_caps = (TC_OP_CHARACTER | TC_OP_STROKE | TC_CP_STROKE |
-                          TC_CR_ANY | TC_SA_DOUBLE | TC_SA_INTEGER |
-                          TC_SA_CONTIN | TC_UA_ABLE | TC_SO_ABLE | TC_RA_ABLE);
-                          /* X11R6 adds TC_SF_X_YINDEP, Xrender adds TC_VA_ABLE */
 
+static Pixmap stock_bitmap_pixmap;  /* phys bitmap for the default stock bitmap */
 
 static const WCHAR dpi_key_name[] = {'S','o','f','t','w','a','r','e','\\','F','o','n','t','s','\0'};
 static const WCHAR dpi_value_name[] = {'L','o','g','P','i','x','e','l','s','\0'};
@@ -96,15 +92,14 @@ static void device_init(void)
 
     palette_size = X11DRV_PALETTE_Init();
 
-    X11DRV_BITMAP_Init();
+    wine_tsx11_lock();
+    stock_bitmap_pixmap = XCreatePixmap( gdi_display, root_window, 1, 1, 1 );
+    wine_tsx11_unlock();
 
     /* Initialize device caps */
     log_pixels_x = log_pixels_y = get_dpi();
     horz_size = MulDiv( screen_width, 254, log_pixels_x * 10 );
     vert_size = MulDiv( screen_height, 254, log_pixels_y * 10 );
-
-    /* Initialize fonts and text caps */
-    X11DRV_FONT_Init(log_pixels_x, log_pixels_y);
 }
 
 /**********************************************************************
@@ -126,12 +121,6 @@ static X11DRV_PDEVICE *create_x11_physdev( Drawable drawable )
 
     if (!(physDev = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*physDev) ))) return NULL;
 
-    if (!(physDev->region = CreateRectRgn( 0, 0, 0, 0 )))
-    {
-        HeapFree( GetProcessHeap(), 0, physDev );
-        return NULL;
-    }
-
     wine_tsx11_lock();
     physDev->drawable = drawable;
     physDev->gc = XCreateGC( gdi_display, drawable, 0, NULL );
@@ -148,18 +137,19 @@ static X11DRV_PDEVICE *create_x11_physdev( Drawable drawable )
 static BOOL X11DRV_CreateDC( PHYSDEV *pdev, LPCWSTR driver, LPCWSTR device,
                              LPCWSTR output, const DEVMODEW* initData )
 {
+    const struct gdi_dc_funcs *glx_funcs = get_glx_driver();
     X11DRV_PDEVICE *physDev = create_x11_physdev( root_window );
 
     if (!physDev) return FALSE;
 
     physDev->depth         = screen_depth;
     physDev->color_shifts  = &X11DRV_PALETTE_default_shifts;
-    physDev->drawable_rect = virtual_screen_rect;
     SetRect( &physDev->dc_rect, 0, 0, virtual_screen_rect.right - virtual_screen_rect.left,
              virtual_screen_rect.bottom - virtual_screen_rect.top );
     push_dc_driver( pdev, &physDev->dev, &x11drv_funcs );
-    if (!xrender_funcs) return TRUE;
-    return xrender_funcs->pCreateDC( pdev, driver, device, output, initData );
+    if (xrender_funcs && !xrender_funcs->pCreateDC( pdev, driver, device, output, initData )) return FALSE;
+    if (glx_funcs && !glx_funcs->pCreateDC( pdev, driver, device, output, initData )) return FALSE;
+    return TRUE;
 }
 
 
@@ -168,21 +158,18 @@ static BOOL X11DRV_CreateDC( PHYSDEV *pdev, LPCWSTR driver, LPCWSTR device,
  */
 static BOOL X11DRV_CreateCompatibleDC( PHYSDEV orig, PHYSDEV *pdev )
 {
-    X11DRV_PDEVICE *physDev = create_x11_physdev( BITMAP_stock_phys_bitmap.pixmap );
+    const struct gdi_dc_funcs *glx_funcs = get_glx_driver();
+    X11DRV_PDEVICE *physDev = create_x11_physdev( stock_bitmap_pixmap );
 
     if (!physDev) return FALSE;
 
-    if (!BITMAP_stock_phys_bitmap.hbitmap)
-        BITMAP_stock_phys_bitmap.hbitmap = GetCurrentObject( (*pdev)->hdc, OBJ_BITMAP );
-
-    physDev->bitmap = &BITMAP_stock_phys_bitmap;
     physDev->depth  = 1;
-    SetRect( &physDev->drawable_rect, 0, 0, 1, 1 );
-    physDev->dc_rect = physDev->drawable_rect;
+    SetRect( &physDev->dc_rect, 0, 0, 1, 1 );
     push_dc_driver( pdev, &physDev->dev, &x11drv_funcs );
     if (orig) return TRUE;  /* we already went through Xrender if we have an orig device */
-    if (!xrender_funcs) return TRUE;
-    return xrender_funcs->pCreateCompatibleDC( NULL, pdev );
+    if (xrender_funcs && !xrender_funcs->pCreateCompatibleDC( NULL, pdev )) return FALSE;
+    if (glx_funcs && !glx_funcs->pCreateCompatibleDC( NULL, pdev )) return FALSE;
+    return TRUE;
 }
 
 
@@ -193,7 +180,6 @@ static BOOL X11DRV_DeleteDC( PHYSDEV dev )
 {
     X11DRV_PDEVICE *physDev = get_x11drv_dev( dev );
 
-    DeleteObject( physDev->region );
     wine_tsx11_lock();
     XFreeGC( gdi_display, physDev->gc );
     wine_tsx11_unlock();
@@ -202,6 +188,31 @@ static BOOL X11DRV_DeleteDC( PHYSDEV dev )
 }
 
 
+void add_device_bounds( X11DRV_PDEVICE *dev, const RECT *rect )
+{
+    RECT rc;
+
+    if (!dev->bounds) return;
+    if (dev->region && GetRgnBox( dev->region, &rc ))
+    {
+        if (IntersectRect( &rc, &rc, rect )) add_bounds_rect( dev->bounds, &rc );
+    }
+    else add_bounds_rect( dev->bounds, rect );
+}
+
+/***********************************************************************
+ *           X11DRV_SetBoundsRect
+ */
+static UINT X11DRV_SetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
+{
+    X11DRV_PDEVICE *pdev = get_x11drv_dev( dev );
+
+    if (flags & DCB_DISABLE) pdev->bounds = NULL;
+    else if (flags & DCB_ENABLE) pdev->bounds = rect;
+    return DCB_RESET;  /* we don't have device-specific bounds */
+}
+
+
 /***********************************************************************
  *           GetDeviceCaps    (X11DRV.@)
  */
@@ -254,7 +265,9 @@ static INT X11DRV_GetDeviceCaps( PHYSDEV dev, INT cap )
         return (PC_POLYGON | PC_RECTANGLE | PC_WINDPOLYGON | PC_SCANLINE |
                 PC_WIDE | PC_STYLED | PC_WIDESTYLED | PC_INTERIORS);
     case TEXTCAPS:
-        return text_caps;
+        return (TC_OP_CHARACTER | TC_OP_STROKE | TC_CP_STROKE |
+                TC_CR_ANY | TC_SF_X_YINDEP | TC_SA_DOUBLE | TC_SA_INTEGER |
+                TC_SA_CONTIN | TC_UA_ABLE | TC_SO_ABLE | TC_RA_ABLE | TC_VA_ABLE);
     case CLIPCAPS:
         return CP_REGION;
     case COLORRES:
@@ -331,46 +344,25 @@ static INT X11DRV_ExtEscape( PHYSDEV dev, INT escape, INT in_count, LPCVOID in_d
         {
             switch(*(const enum x11drv_escape_codes *)in_data)
             {
-            case X11DRV_GET_DISPLAY:
-                if (out_count >= sizeof(Display *))
-                {
-                    *(Display **)out_data = gdi_display;
-                    return TRUE;
-                }
-                break;
-            case X11DRV_GET_DRAWABLE:
-                if (out_count >= sizeof(Drawable))
-                {
-                    *(Drawable *)out_data = physDev->drawable;
-                    return TRUE;
-                }
-                break;
-            case X11DRV_GET_FONT:
-                if (out_count >= sizeof(Font))
-                {
-                    fontObject* pfo = XFONT_GetFontObject( physDev->font );
-		    if (pfo == NULL) return FALSE;
-                    *(Font *)out_data = pfo->fs->fid;
-                    return TRUE;
-                }
-                break;
             case X11DRV_SET_DRAWABLE:
                 if (in_count >= sizeof(struct x11drv_escape_set_drawable))
                 {
                     const struct x11drv_escape_set_drawable *data = in_data;
                     physDev->dc_rect = data->dc_rect;
                     physDev->drawable = data->drawable;
-                    physDev->drawable_rect = data->drawable_rect;
-                    physDev->current_pf = pixelformat_from_fbconfig_id( data->fbconfig_id );
-                    physDev->gl_drawable = data->gl_drawable;
-                    physDev->pixmap = data->pixmap;
-                    physDev->gl_copy = data->gl_copy;
                     wine_tsx11_lock();
                     XSetSubwindowMode( gdi_display, physDev->gc, data->mode );
                     wine_tsx11_unlock();
-                    TRACE( "SET_DRAWABLE hdc %p drawable %lx gl_drawable %lx pf %u dc_rect %s drawable_rect %s\n",
-                           dev->hdc, physDev->drawable, physDev->gl_drawable, physDev->current_pf,
-                           wine_dbgstr_rect(&physDev->dc_rect), wine_dbgstr_rect(&physDev->drawable_rect) );
+                    TRACE( "SET_DRAWABLE hdc %p drawable %lx dc_rect %s\n",
+                           dev->hdc, physDev->drawable, wine_dbgstr_rect(&physDev->dc_rect) );
+                    return TRUE;
+                }
+                break;
+            case X11DRV_GET_DRAWABLE:
+                if (out_count >= sizeof(struct x11drv_escape_get_drawable))
+                {
+                    struct x11drv_escape_get_drawable *data = out_data;
+                    data->drawable = physDev->drawable;
                     return TRUE;
                 }
                 break;
@@ -434,27 +426,8 @@ static INT X11DRV_ExtEscape( PHYSDEV dev, INT escape, INT in_count, LPCVOID in_d
                     return TRUE;
                 }
                 break;
-            case X11DRV_GET_DCE:
-            case X11DRV_SET_DCE:
-                FIXME( "%x escape no longer supported\n", *(const enum x11drv_escape_codes *)in_data );
+            default:
                 break;
-            case X11DRV_GET_GLX_DRAWABLE:
-                if (out_count >= sizeof(Drawable))
-                {
-                    *(Drawable *)out_data = get_glxdrawable(physDev);
-                    return TRUE;
-                }
-                break;
-            case X11DRV_SYNC_PIXMAP:
-                if(physDev->bitmap)
-                {
-                    X11DRV_CoerceDIBSection(physDev, DIB_Status_GdiMod);
-                    return TRUE;
-                }
-                return FALSE;
-            case X11DRV_FLUSH_GL_DRAWABLE:
-                flush_gl_drawable(physDev);
-                return TRUE;
             }
         }
         break;
@@ -463,6 +436,31 @@ static INT X11DRV_ExtEscape( PHYSDEV dev, INT escape, INT in_count, LPCVOID in_d
 }
 
 
+static inline void opengl_error(void)
+{
+    static int warned;
+    if (!warned++) ERR("No OpenGL support compiled in.\n");
+}
+
+/***********************************************************************
+ *		X11DRV_DescribePixelFormat
+ */
+static int X11DRV_DescribePixelFormat( PHYSDEV dev, int fmt, UINT size, PIXELFORMATDESCRIPTOR *ppfd )
+{
+    opengl_error();
+    return 0;
+}
+
+/***********************************************************************
+ *		X11DRV_SetPixelFormat
+ */
+static BOOL X11DRV_SetPixelFormat( PHYSDEV dev, int fmt, const PIXELFORMATDESCRIPTOR *ppfd )
+{
+    opengl_error();
+    return FALSE;
+}
+
+
 static const struct gdi_dc_funcs x11drv_funcs =
 {
     NULL,                               /* pAbortDoc */
@@ -473,14 +471,10 @@ static const struct gdi_dc_funcs x11drv_funcs =
     NULL,                               /* pArcTo */
     NULL,                               /* pBeginPath */
     NULL,                               /* pBlendImage */
-    X11DRV_ChoosePixelFormat,           /* pChoosePixelFormat */
     X11DRV_Chord,                       /* pChord */
     NULL,                               /* pCloseFigure */
-    X11DRV_CreateBitmap,                /* pCreateBitmap */
     X11DRV_CreateCompatibleDC,          /* pCreateCompatibleDC */
     X11DRV_CreateDC,                    /* pCreateDC */
-    X11DRV_CreateDIBSection,            /* pCreateDIBSection */
-    X11DRV_DeleteBitmap,                /* pDeleteBitmap */
     X11DRV_DeleteDC,                    /* pDeleteDC */
     NULL,                               /* pDeleteObject */
     X11DRV_DescribePixelFormat,         /* pDescribePixelFormat */
@@ -489,30 +483,44 @@ static const struct gdi_dc_funcs x11drv_funcs =
     NULL,                               /* pEndDoc */
     NULL,                               /* pEndPage */
     NULL,                               /* pEndPath */
-    X11DRV_EnumDeviceFonts,             /* pEnumDeviceFonts */
+    NULL,                               /* pEnumFonts */
     X11DRV_EnumICMProfiles,             /* pEnumICMProfiles */
     NULL,                               /* pExcludeClipRect */
     NULL,                               /* pExtDeviceMode */
     X11DRV_ExtEscape,                   /* pExtEscape */
     X11DRV_ExtFloodFill,                /* pExtFloodFill */
     NULL,                               /* pExtSelectClipRgn */
-    X11DRV_ExtTextOut,                  /* pExtTextOut */
+    NULL,                               /* pExtTextOut */
     NULL,                               /* pFillPath */
     NULL,                               /* pFillRgn */
     NULL,                               /* pFlattenPath */
+    NULL,                               /* pFontIsLinked */
     NULL,                               /* pFrameRgn */
     NULL,                               /* pGdiComment */
-    X11DRV_GetCharWidth,                /* pGetCharWidth */
+    NULL,                               /* pGdiRealizationInfo */
+    NULL,                               /* pGetBoundsRect */
+    NULL,                               /* pGetCharABCWidths */
+    NULL,                               /* pGetCharABCWidthsI */
+    NULL,                               /* pGetCharWidth */
     X11DRV_GetDeviceCaps,               /* pGetDeviceCaps */
     X11DRV_GetDeviceGammaRamp,          /* pGetDeviceGammaRamp */
+    NULL,                               /* pGetFontData */
+    NULL,                               /* pGetFontUnicodeRanges */
+    NULL,                               /* pGetGlyphIndices */
+    NULL,                               /* pGetGlyphOutline */
     X11DRV_GetICMProfile,               /* pGetICMProfile */
     X11DRV_GetImage,                    /* pGetImage */
+    NULL,                               /* pGetKerningPairs */
     X11DRV_GetNearestColor,             /* pGetNearestColor */
-    X11DRV_GetPixel,                    /* pGetPixel */
-    X11DRV_GetPixelFormat,              /* pGetPixelFormat */
+    NULL,                               /* pGetOutlineTextMetrics */
+    NULL,                               /* pGetPixel */
     X11DRV_GetSystemPaletteEntries,     /* pGetSystemPaletteEntries */
-    X11DRV_GetTextExtentExPoint,        /* pGetTextExtentExPoint */
-    X11DRV_GetTextMetrics,              /* pGetTextMetrics */
+    NULL,                               /* pGetTextCharsetInfo */
+    NULL,                               /* pGetTextExtentExPoint */
+    NULL,                               /* pGetTextExtentExPointI */
+    NULL,                               /* pGetTextFace */
+    NULL,                               /* pGetTextMetrics */
+    X11DRV_GradientFill,                /* pGradientFill */
     NULL,                               /* pIntersectClipRect */
     NULL,                               /* pInvertRgn */
     X11DRV_LineTo,                      /* pLineTo */
@@ -530,7 +538,7 @@ static const struct gdi_dc_funcs x11drv_funcs =
     X11DRV_PolyPolygon,                 /* pPolyPolygon */
     X11DRV_PolyPolyline,                /* pPolyPolyline */
     X11DRV_Polygon,                     /* pPolygon */
-    X11DRV_Polyline,                    /* pPolyline */
+    NULL,                               /* pPolyline */
     NULL,                               /* pPolylineTo */
     X11DRV_PutImage,                    /* pPutImage */
     X11DRV_RealizeDefaultPalette,       /* pRealizeDefaultPalette */
@@ -542,18 +550,18 @@ static const struct gdi_dc_funcs x11drv_funcs =
     NULL,                               /* pSaveDC */
     NULL,                               /* pScaleViewportExt */
     NULL,                               /* pScaleWindowExt */
-    X11DRV_SelectBitmap,                /* pSelectBitmap */
+    NULL,                               /* pSelectBitmap */
     X11DRV_SelectBrush,                 /* pSelectBrush */
     NULL,                               /* pSelectClipPath */
-    X11DRV_SelectFont,                  /* pSelectFont */
+    NULL,                               /* pSelectFont */
     NULL,                               /* pSelectPalette */
     X11DRV_SelectPen,                   /* pSelectPen */
     NULL,                               /* pSetArcDirection */
-    X11DRV_SetBkColor,                  /* pSetBkColor */
+    NULL,                               /* pSetBkColor */
     NULL,                               /* pSetBkMode */
+    X11DRV_SetBoundsRect,               /* pSetBoundsRect */
     X11DRV_SetDCBrushColor,             /* pSetDCBrushColor */
     X11DRV_SetDCPenColor,               /* pSetDCPenColor */
-    X11DRV_SetDIBColorTable,            /* pSetDIBColorTable */
     NULL,                               /* pSetDIBitsToDevice */
     X11DRV_SetDeviceClipping,           /* pSetDeviceClipping */
     X11DRV_SetDeviceGammaRamp,          /* pSetDeviceGammaRamp */
@@ -568,7 +576,7 @@ static const struct gdi_dc_funcs x11drv_funcs =
     NULL,                               /* pSetStretchBltMode */
     NULL,                               /* pSetTextAlign */
     NULL,                               /* pSetTextCharacterExtra */
-    X11DRV_SetTextColor,                /* pSetTextColor */
+    NULL,                               /* pSetTextColor */
     NULL,                               /* pSetTextJustification */
     NULL,                               /* pSetViewportExt */
     NULL,                               /* pSetViewportOrg */
@@ -581,21 +589,11 @@ static const struct gdi_dc_funcs x11drv_funcs =
     NULL,                               /* pStretchDIBits */
     NULL,                               /* pStrokeAndFillPath */
     NULL,                               /* pStrokePath */
-    X11DRV_SwapBuffers,                 /* pSwapBuffers */
+    NULL,                               /* pSwapBuffers */
     X11DRV_UnrealizePalette,            /* pUnrealizePalette */
     NULL,                               /* pWidenPath */
-    X11DRV_wglCopyContext,              /* pwglCopyContext */
-    X11DRV_wglCreateContext,            /* pwglCreateContext */
-    X11DRV_wglCreateContextAttribsARB,  /* pwglCreateContextAttribsARB */
-    X11DRV_wglDeleteContext,            /* pwglDeleteContext */
-    X11DRV_wglGetPbufferDCARB,          /* pwglGetPbufferDCARB */
-    X11DRV_wglGetProcAddress,           /* pwglGetProcAddress */
-    X11DRV_wglMakeContextCurrentARB,    /* pwglMakeContextCurrentARB */
-    X11DRV_wglMakeCurrent,              /* pwglMakeCurrent */
-    X11DRV_wglSetPixelFormatWINE,       /* pwglSetPixelFormatWINE */
-    X11DRV_wglShareLists,               /* pwglShareLists */
-    X11DRV_wglUseFontBitmapsA,          /* pwglUseFontBitmapsA */
-    X11DRV_wglUseFontBitmapsW,          /* pwglUseFontBitmapsW */
+    NULL,                               /* wine_get_wgl_driver */
+    GDI_PRIORITY_GRAPHICS_DRV           /* priority */
 };
 
 
