@@ -47,8 +47,6 @@
 #include "winreg.h"
 
 #include "x11drv.h"
-#include "xvidmode.h"
-#include "xrandr.h"
 #include "xcomposite.h"
 #include "wine/server.h"
 #include "wine/debug.h"
@@ -90,7 +88,6 @@ int managed_mode = 1;
 int decorated_mode = 1;
 int private_color_map = 0;
 int primary_monitor = 0;
-int client_side_with_core = 1;
 int client_side_with_render = 1;
 int client_side_antialias_with_core = 1;
 int client_side_antialias_with_render = 1;
@@ -423,9 +420,6 @@ static void setup_options(void)
     if (!get_config_key( hkey, appkey, "ScreenDepth", buffer, sizeof(buffer) ))
         screen_depth = atoi(buffer);
 
-    if (!get_config_key( hkey, appkey, "ClientSideWithCore", buffer, sizeof(buffer) ))
-        client_side_with_core = IS_OPTION_TRUE( buffer[0] );
-
     if (!get_config_key( hkey, appkey, "ClientSideWithRender", buffer, sizeof(buffer) ))
         client_side_with_render = IS_OPTION_TRUE( buffer[0] );
 
@@ -543,6 +537,7 @@ static BOOL process_attach(void)
 
     /* Open display */
 
+    if (!XInitThreads()) ERR( "XInitThreads failed, trouble ahead\n" );
     if (!(display = XOpenDisplay( NULL ))) return FALSE;
 
     fcntl( ConnectionNumber(display), F_SETFD, 1 ); /* set close on exec flag */
@@ -578,14 +573,10 @@ static BOOL process_attach(void)
     xinerama_init( WidthOfScreen(screen), HeightOfScreen(screen) );
     X11DRV_Settings_Init();
 
-#ifdef SONAME_LIBXXF86VM
     /* initialize XVidMode */
     X11DRV_XF86VM_Init();
-#endif
-#ifdef SONAME_LIBXRANDR
     /* initialize XRandR */
     X11DRV_XRandR_Init();
-#endif
 #ifdef SONAME_LIBXCOMPOSITE
     X11DRV_XComposite_Init();
 #endif
@@ -628,15 +619,12 @@ static void thread_detach(void)
 static void process_detach(void)
 {
     X11DRV_Clipboard_Cleanup();
-#ifdef SONAME_LIBXXF86VM
     /* cleanup XVidMode */
     X11DRV_XF86VM_Cleanup();
-#endif
     X11DRV_XRender_Finalize();
 
     /* cleanup GDI */
     X11DRV_GDI_Finalize();
-    X11DRV_OpenGL_Cleanup();
 
     IME_UnregisterClasses();
     DeleteCriticalSection( &X11DRV_CritSection );
