@@ -343,7 +343,7 @@ static void test_crack_url(const crack_url_test_t *test)
     ok(!strcmp(scheme, test->exp_scheme), "[%s] Got wrong scheme, expected: %s, got: %s\n",
             test->url, test->exp_scheme, scheme);
 
-    ok(url.nScheme == test->scheme, "[%s] Got wrong nScheme, expected: %d, got: %d",
+    ok(url.nScheme == test->scheme, "[%s] Got wrong nScheme, expected: %d, got: %d\n",
             test->url, test->scheme, url.nScheme);
 
     ok(url.dwHostNameLength == strlen(test->exp_hostname), "[%s] Got wrong hostname length: %d\n",
@@ -398,7 +398,7 @@ static void test_crack_url(const crack_url_test_t *test)
     ok(!lstrcmpW((WCHAR*)scheme, w_str_of(test->exp_scheme)), "[%s] Got wrong scheme, expected: %s, got: %s\n",
             test->url, test->exp_scheme, wine_dbgstr_w((WCHAR*)scheme));
 
-    ok(urlw.nScheme == test->scheme, "[%s] Got wrong nScheme, expected: %d, got: %d",
+    ok(urlw.nScheme == test->scheme, "[%s] Got wrong nScheme, expected: %d, got: %d\n",
             test->url, test->scheme, urlw.nScheme);
 
     ok(urlw.dwHostNameLength == strlen(test->exp_hostname), "[%s] Got wrong hostname length: %d\n",
@@ -465,21 +465,21 @@ static void InternetCrackUrl_test(void)
   ret = InternetCrackUrlA(TEST_URL3, 0, ICU_DECODE, &urlComponents);
   GLE = GetLastError();
   ok(ret==firstret && (GLE==firstGLE), "InternetCrackUrl returned %d with GLE=%d (expected to return %d)\n",
-    ret, GetLastError(), firstret);
+    ret, GLE, firstret);
 
   copy_compsA(&urlSrc, &urlComponents, 32, 1024, 0, 1024, 2048, 1024);
   SetLastError(0xdeadbeef);
   ret = InternetCrackUrlA(TEST_URL3, 0, ICU_DECODE, &urlComponents);
   GLE = GetLastError();
   ok(ret==firstret && (GLE==firstGLE), "InternetCrackUrl returned %d with GLE=%d (expected to return %d)\n",
-    ret, GetLastError(), firstret);
+    ret, GLE, firstret);
 
   copy_compsA(&urlSrc, &urlComponents, 32, 1024, 1024, 0, 2048, 1024);
   SetLastError(0xdeadbeef);
   ret = InternetCrackUrlA(TEST_URL3, 0, ICU_DECODE, &urlComponents);
   GLE = GetLastError();
   ok(ret==firstret && (GLE==firstGLE), "InternetCrackUrl returned %d with GLE=%d (expected to return %d)\n",
-    ret, GetLastError(), firstret);
+    ret, GLE, firstret);
 
   copy_compsA(&urlSrc, &urlComponents, 32, 1024, 1024, 1024, 0, 1024);
   SetLastError(0xdeadbeef);
@@ -792,7 +792,6 @@ static void InternetCreateUrlA_test(void)
 	ok(!ret, "Expected failure\n");
 	ok(GetLastError() == ERROR_INVALID_PARAMETER,
 		"Expected ERROR_INVALID_PARAMETER, got %d\n", GetLastError());
-	ok(len == -1, "Expected len -1, got %d\n", len);
 
 	/* test valid lpUrlComponents, empty szUrl
 	 * lpdwUrlLength is size of buffer required on exit, including
