@@ -87,7 +87,7 @@ extern "C" {
 #define SIO_SET_COMPATIBILITY_MODE _WSAIOW(IOC_VENDOR,300)
 #else
 #define WS_SIO_UDP_CONNRESET _WSAIOW(WS_IOC_VENDOR,12)
-#define WS_SIO_SET_COMPATIBILITY_MODE _WSAIOW(IOC_VENDOR,300)
+#define WS_SIO_SET_COMPATIBILITY_MODE _WSAIOW(WS_IOC_VENDOR,300)
 #endif
 
 #define DE_REUSE_SOCKET TF_REUSE_SOCKET
