@@ -747,8 +747,8 @@ HRESULT WINAPI ParseURLW(LPCWSTR pszUrl, PARSEDURLW *ppu);
 #ifndef NO_SHLWAPI_STRFCNS
 
 /* StrToIntEx flags */
-#define STIF_DEFAULT     0x0L
-#define STIF_SUPPORT_HEX 0x1L
+#define STIF_DEFAULT     __MSABI_LONG(0x0)
+#define STIF_SUPPORT_HEX __MSABI_LONG(0x1)
 
 BOOL WINAPI ChrCmpIA (WORD,WORD);
 BOOL WINAPI ChrCmpIW (WCHAR,WCHAR);
