@@ -22,6 +22,7 @@
 #define __WINE_WINE_GDI_DRIVER_H
 
 struct gdi_dc_funcs;
+struct wgl_funcs;
 
 typedef struct gdi_physdev
 {
@@ -54,7 +55,6 @@ struct gdi_image_bits
 
 struct brush_pattern
 {
-    HBITMAP               bitmap;   /* bitmap handle for DDB patterns */
     BITMAPINFO           *info;     /* DIB info */
     struct gdi_image_bits bits;     /* DIB bits */
     UINT                  usage;    /* color usage for DIB info */
@@ -70,14 +70,10 @@ struct gdi_dc_funcs
     BOOL     (*pArcTo)(PHYSDEV,INT,INT,INT,INT,INT,INT,INT,INT);
     BOOL     (*pBeginPath)(PHYSDEV);
     DWORD    (*pBlendImage)(PHYSDEV,BITMAPINFO*,const struct gdi_image_bits*,struct bitblt_coords*,struct bitblt_coords*,BLENDFUNCTION);
-    INT      (*pChoosePixelFormat)(PHYSDEV,const PIXELFORMATDESCRIPTOR *);
     BOOL     (*pChord)(PHYSDEV,INT,INT,INT,INT,INT,INT,INT,INT);
     BOOL     (*pCloseFigure)(PHYSDEV);
-    BOOL     (*pCopyBitmap)(HBITMAP,HBITMAP);
-    BOOL     (*pCreateBitmap)(PHYSDEV,HBITMAP);
     BOOL     (*pCreateCompatibleDC)(PHYSDEV,PHYSDEV*);
     BOOL     (*pCreateDC)(PHYSDEV*,LPCWSTR,LPCWSTR,LPCWSTR,const DEVMODEW*);
-    BOOL     (*pDeleteBitmap)(HBITMAP);
     BOOL     (*pDeleteDC)(PHYSDEV);
     BOOL     (*pDeleteObject)(PHYSDEV,HGDIOBJ);
     INT      (*pDescribePixelFormat)(PHYSDEV,INT,UINT,PIXELFORMATDESCRIPTOR *);
@@ -101,6 +97,7 @@ struct gdi_dc_funcs
     BOOL     (*pFrameRgn)(PHYSDEV,HRGN,HBRUSH,INT,INT);
     BOOL     (*pGdiComment)(PHYSDEV,UINT,CONST BYTE*);
     BOOL     (*pGdiRealizationInfo)(PHYSDEV,void*);
+    UINT     (*pGetBoundsRect)(PHYSDEV,RECT*,UINT);
     BOOL     (*pGetCharABCWidths)(PHYSDEV,UINT,UINT,LPABC);
     BOOL     (*pGetCharABCWidthsI)(PHYSDEV,UINT,UINT,WORD*,LPABC);
     BOOL     (*pGetCharWidth)(PHYSDEV,UINT,UINT,LPINT);
@@ -111,12 +108,11 @@ struct gdi_dc_funcs
     DWORD    (*pGetGlyphIndices)(PHYSDEV,LPCWSTR,INT,LPWORD,DWORD);
     DWORD    (*pGetGlyphOutline)(PHYSDEV,UINT,UINT,LPGLYPHMETRICS,DWORD,LPVOID,const MAT2*);
     BOOL     (*pGetICMProfile)(PHYSDEV,LPDWORD,LPWSTR);
-    DWORD    (*pGetImage)(PHYSDEV,HBITMAP,BITMAPINFO*,struct gdi_image_bits*,struct bitblt_coords*);
+    DWORD    (*pGetImage)(PHYSDEV,BITMAPINFO*,struct gdi_image_bits*,struct bitblt_coords*);
     DWORD    (*pGetKerningPairs)(PHYSDEV,DWORD,LPKERNINGPAIR);
     COLORREF (*pGetNearestColor)(PHYSDEV,COLORREF);
     UINT     (*pGetOutlineTextMetrics)(PHYSDEV,UINT,LPOUTLINETEXTMETRICW);
     COLORREF (*pGetPixel)(PHYSDEV,INT,INT);
-    INT      (*pGetPixelFormat)(PHYSDEV);
     UINT     (*pGetSystemPaletteEntries)(PHYSDEV,UINT,UINT,LPPALETTEENTRY);
     UINT     (*pGetTextCharsetInfo)(PHYSDEV,LPFONTSIGNATURE,DWORD);
     BOOL     (*pGetTextExtentExPoint)(PHYSDEV,LPCWSTR,INT,INT,LPINT,LPINT,LPSIZE);
@@ -143,7 +139,7 @@ struct gdi_dc_funcs
     BOOL     (*pPolygon)(PHYSDEV,const POINT*,INT);
     BOOL     (*pPolyline)(PHYSDEV,const POINT*,INT);
     BOOL     (*pPolylineTo)(PHYSDEV,const POINT*,INT);
-    DWORD    (*pPutImage)(PHYSDEV,HBITMAP,HRGN,BITMAPINFO*,const struct gdi_image_bits*,struct bitblt_coords*,struct bitblt_coords*,DWORD);
+    DWORD    (*pPutImage)(PHYSDEV,HRGN,BITMAPINFO*,const struct gdi_image_bits*,struct bitblt_coords*,struct bitblt_coords*,DWORD);
     UINT     (*pRealizeDefaultPalette)(PHYSDEV);
     UINT     (*pRealizePalette)(PHYSDEV,HPALETTE,BOOL);
     BOOL     (*pRectangle)(PHYSDEV,INT,INT,INT,INT);
@@ -162,6 +158,7 @@ struct gdi_dc_funcs
     INT      (*pSetArcDirection)(PHYSDEV,INT);
     COLORREF (*pSetBkColor)(PHYSDEV,COLORREF);
     INT      (*pSetBkMode)(PHYSDEV,INT);
+    UINT     (*pSetBoundsRect)(PHYSDEV,RECT*,UINT);
     COLORREF (*pSetDCBrushColor)(PHYSDEV, COLORREF);
     COLORREF (*pSetDCPenColor)(PHYSDEV, COLORREF);
     INT      (*pSetDIBitsToDevice)(PHYSDEV,INT,INT,DWORD,DWORD,INT,INT,UINT,UINT,LPCVOID,BITMAPINFO*,UINT);
@@ -194,24 +191,20 @@ struct gdi_dc_funcs
     BOOL     (*pSwapBuffers)(PHYSDEV);
     BOOL     (*pUnrealizePalette)(HPALETTE);
     BOOL     (*pWidenPath)(PHYSDEV);
+    const struct wgl_funcs * (*wine_get_wgl_driver)(PHYSDEV,UINT);
 
-    /* OpenGL32 */
-    BOOL     (*pwglCopyContext)(HGLRC,HGLRC,UINT);
-    HGLRC    (*pwglCreateContext)(PHYSDEV);
-    HGLRC    (*pwglCreateContextAttribsARB)(PHYSDEV,HGLRC,const int*);
-    BOOL     (*pwglDeleteContext)(HGLRC);
-    HDC      (*pwglGetPbufferDCARB)(PHYSDEV,void*);
-    PROC     (*pwglGetProcAddress)(LPCSTR);
-    BOOL     (*pwglMakeContextCurrentARB)(PHYSDEV,PHYSDEV,HGLRC);
-    BOOL     (*pwglMakeCurrent)(PHYSDEV,HGLRC);
-    BOOL     (*pwglSetPixelFormatWINE)(PHYSDEV,INT,const PIXELFORMATDESCRIPTOR*);
-    BOOL     (*pwglShareLists)(HGLRC,HGLRC);
-    BOOL     (*pwglUseFontBitmapsA)(PHYSDEV,DWORD,DWORD,DWORD);
-    BOOL     (*pwglUseFontBitmapsW)(PHYSDEV,DWORD,DWORD,DWORD);
+    /* priority order for the driver on the stack */
+    UINT       priority;
 };
 
 /* increment this when you change the DC function table */
-#define WINE_GDI_DRIVER_VERSION 24
+#define WINE_GDI_DRIVER_VERSION 41
+
+#define GDI_PRIORITY_NULL_DRV        0  /* null driver */
+#define GDI_PRIORITY_FONT_DRV      100  /* any font driver */
+#define GDI_PRIORITY_GRAPHICS_DRV  200  /* any graphics driver */
+#define GDI_PRIORITY_DIB_DRV       300  /* the DIB driver */
+#define GDI_PRIORITY_PATH_DRV      400  /* the path driver */
 
 static inline PHYSDEV get_physdev_entry_point( PHYSDEV dev, size_t offset )
 {
@@ -224,17 +217,46 @@ static inline PHYSDEV get_physdev_entry_point( PHYSDEV dev, size_t offset )
 
 static inline void push_dc_driver( PHYSDEV *dev, PHYSDEV physdev, const struct gdi_dc_funcs *funcs )
 {
+    while ((*dev)->funcs->priority > funcs->priority) dev = &(*dev)->next;
     physdev->funcs = funcs;
     physdev->next = *dev;
     physdev->hdc = (*dev)->hdc;
     *dev = physdev;
 }
 
-static inline PHYSDEV pop_dc_driver( PHYSDEV *dev )
+/* OpenGL support */
+
+struct wgl_context;
+
+struct wgl_funcs
 {
-    PHYSDEV ret = *dev;
-    *dev = ret->next;
-    return ret;
-}
+    INT                 (*p_GetPixelFormat)(HDC);
+    BOOL                (*p_wglCopyContext)(struct wgl_context*,struct wgl_context*,UINT);
+    struct wgl_context* (*p_wglCreateContext)(HDC);
+    struct wgl_context* (*p_wglCreateContextAttribsARB)(HDC,struct wgl_context*,const int*);
+    void                (*p_wglDeleteContext)(struct wgl_context*);
+    HDC                 (*p_wglGetCurrentDC)(struct wgl_context*);
+    PROC                (*p_wglGetProcAddress)(LPCSTR);
+    BOOL                (*p_wglMakeContextCurrentARB)(HDC,HDC,struct wgl_context*);
+    BOOL                (*p_wglMakeCurrent)(HDC,struct wgl_context*);
+    BOOL                (*p_wglShareLists)(struct wgl_context*,struct wgl_context*);
+};
+
+/* the DC hook support is only exported on Win16, the 32-bit version is a Wine extension */
+
+#define DCHC_INVALIDVISRGN      0x0001
+#define DCHC_DELETEDC           0x0002
+#define DCHF_INVALIDATEVISRGN   0x0001
+#define DCHF_VALIDATEVISRGN     0x0002
+
+typedef BOOL (CALLBACK *DCHOOKPROC)(HDC,WORD,DWORD_PTR,LPARAM);
+
+WINGDIAPI DWORD_PTR WINAPI GetDCHook(HDC,DCHOOKPROC*);
+WINGDIAPI BOOL      WINAPI SetDCHook(HDC,DCHOOKPROC,DWORD_PTR);
+WINGDIAPI WORD      WINAPI SetHookFlags(HDC,WORD);
+
+extern void CDECL __wine_make_gdi_object_system( HGDIOBJ handle, BOOL set );
+extern void CDECL __wine_set_visible_region( HDC hdc, HRGN hrgn, const RECT *vis_rect );
+extern const struct wgl_funcs * CDECL __wine_get_wgl_driver( HDC hdc, UINT version );
 
 #endif /* __WINE_WINE_GDI_DRIVER_H */
