@@ -174,7 +174,7 @@ static int pre_exec(void)
     return 1;  /* we have a preloader on x86-64 */
 }
 
-#elif defined(__FreeBSD__) && defined(__i386__)
+#elif (defined(__FreeBSD__) || defined (__FreeBSD_kernel__) || defined(__DragonFly__)) && defined(__i386__)
 
 static int pre_exec(void)
 {
@@ -219,7 +219,9 @@ int main( int argc, char *argv[] )
         }
     }
 
+#ifndef __APPLE__
     if (wine_main_preload_info)
+#endif
     {
         for (i = 0; wine_main_preload_info[i].size; i++)
             reserve_area( wine_main_preload_info[i].addr, wine_main_preload_info[i].size );
