@@ -24,10 +24,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(cmd);
 
-extern WCHAR quals[MAX_PATH], param1[MAX_PATH], param2[MAX_PATH];
-extern BATCH_CONTEXT *context;
-extern DWORD errorlevel;
-
 /****************************************************************************
  * WCMD_batch
  *
@@ -44,8 +40,8 @@ extern DWORD errorlevel;
  * a label to goto once opened.
  */
 
-void WCMD_batch (WCHAR *file, WCHAR *command, int called, WCHAR *startLabel, HANDLE pgmHandle) {
-
+void WCMD_batch (WCHAR *file, WCHAR *command, BOOL called, WCHAR *startLabel, HANDLE pgmHandle)
+{
   HANDLE h = INVALID_HANDLE_VALUE;
   BATCH_CONTEXT *prev_context;
 
@@ -118,32 +114,29 @@ void WCMD_batch (WCHAR *file, WCHAR *command, int called, WCHAR *startLabel, HAN
  *
  * PARAMS
  *  s     [I] input string, non NULL
- *  n     [I] # of the (possibly double quotes-delimited) parameter to return
- *            Starts at 0
- *  where [O] if non NULL, pointer to the start of the nth parameter in s,
- *            potentially a " character
- *  end   [O] if non NULL, pointer to the last char of
- *            the nth parameter in s, potentially a " character
+ *  n     [I] # of the parameter to return, counted from 0
+ *  start [O] Optional. Pointer to the first char of param n in s
+ *  end   [O] Optional. Pointer to the last char of param n in s
  *
  * RETURNS
- *  Success: Returns the nth delimited parameter found in s.
- *           *where points to the start of the param, possibly a starting
- *           double quotes character
- *  Failure: Returns an empty string if the param is not found.
- *           *where is set to NULL
+ *  Success: The nth delimited parameter found in s, with any surrounding quotes removed
+ *           if start != NULL, *start points to the start of the param
+ *           if end != NULL, *end points to the end of the param
+ *  Failure: An empty string if the param is not found.
+ *           *start == *end == NULL
  *
  * NOTES
- *  Return value is stored in static storage, hence is overwritten
- *  after each call.
- *  Doesn't include any potentially delimiting double quotes
+ *  Return value is stored in static storage (i.e. overwritten after each call).
+ *  Specify 'start' and/or 'end' to include delimiting double quotes as well, if any.
  */
-WCHAR *WCMD_parameter (WCHAR *s, int n, WCHAR **where, WCHAR **end) {
+WCHAR *WCMD_parameter (WCHAR *s, int n, WCHAR **start, WCHAR **end)
+{
     int curParamNb = 0;
     static WCHAR param[MAX_PATH];
     WCHAR *p = s, *q;
     BOOL quotesDelimited;
 
-    if (where != NULL) *where = NULL;
+    if (start != NULL) *start = NULL;
     if (end != NULL) *end = NULL;
     param[0] = '\0';
     while (TRUE) {
@@ -152,7 +145,7 @@ WCHAR *WCMD_parameter (WCHAR *s, int n, WCHAR **where, WCHAR **end) {
         if (*p == '\0') return param;
 
         quotesDelimited = (*p == '"');
-        if (where != NULL && curParamNb == n) *where = p;
+        if (start != NULL && curParamNb == n) *start = p;
 
         if (quotesDelimited) {
             q = ++p;
@@ -186,12 +179,12 @@ WCHAR *WCMD_parameter (WCHAR *s, int n, WCHAR **where, WCHAR **end) {
  *       NULL on error or EOF
  */
 
-WCHAR *WCMD_fgets(WCHAR *buf, int noChars, HANDLE h)
+WCHAR *WCMD_fgets(WCHAR *buf, DWORD noChars, HANDLE h)
 {
   DWORD charsRead;
   BOOL status;
   LARGE_INTEGER filepos;
-  int i;
+  DWORD i;
 
   /* We can't use the native f* functions because of the filename syntax differences
      between DOS and Unix. Also need to lose the LF (or CRLF) from the line. */
@@ -317,7 +310,6 @@ void WCMD_HandleTildaModifiers(WCHAR **start, const WCHAR *forVariable,
   static const WCHAR validmodifiers[NUMMODIFIERS] = {
         '~', 'f', 'd', 'p', 'n', 'x', 's', 'a', 't', 'z', '$'
   };
-  static const WCHAR space[] = {' ', '\0'};
 
   WIN32_FILE_ATTRIBUTE_DATA fileInfo;
   WCHAR  outputparam[MAX_PATH];
@@ -480,13 +472,13 @@ void WCMD_HandleTildaModifiers(WCHAR **start, const WCHAR *forVariable,
       int datelen;
 
       doneModifier = TRUE;
-      if (finaloutput[0] != 0x00) strcatW(finaloutput, space);
+      if (finaloutput[0] != 0x00) strcatW(finaloutput, spaceW);
 
       /* Format the time */
       FileTimeToSystemTime(&fileInfo.ftLastWriteTime, &systime);
       GetDateFormatW(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &systime,
                         NULL, thisoutput, MAX_PATH);
-      strcatW(thisoutput, space);
+      strcatW(thisoutput, spaceW);
       datelen = strlenW(thisoutput);
       GetTimeFormatW(LOCALE_USER_DEFAULT, TIME_NOSECONDS, &systime,
                         NULL, (thisoutput+datelen), MAX_PATH-datelen);
@@ -502,14 +494,14 @@ void WCMD_HandleTildaModifiers(WCHAR **start, const WCHAR *forVariable,
       static const WCHAR fmt[] = {'%','u','\0'};
 
       doneModifier = TRUE;
-      if (finaloutput[0] != 0x00) strcatW(finaloutput, space);
+      if (finaloutput[0] != 0x00) strcatW(finaloutput, spaceW);
       wsprintfW(thisoutput, fmt, fullsize);
       strcatW(finaloutput, thisoutput);
     }
 
     /* 4. Handle 's' : Use short paths (File doesn't have to exist) */
     if (memchrW(firstModifier, 's', modifierLen) != NULL) {
-      if (finaloutput[0] != 0x00) strcatW(finaloutput, space);
+      if (finaloutput[0] != 0x00) strcatW(finaloutput, spaceW);
       /* Don't flag as doneModifier - %~s on its own is processed later */
       GetShortPathNameW(outputparam, outputparam, sizeof(outputparam)/sizeof(outputparam[0]));
     }
@@ -518,7 +510,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, const WCHAR *forVariable,
     /*      Note this overrides d,p,n,x                                 */
     if (memchrW(firstModifier, 'f', modifierLen) != NULL) {
       doneModifier = TRUE;
-      if (finaloutput[0] != 0x00) strcatW(finaloutput, space);
+      if (finaloutput[0] != 0x00) strcatW(finaloutput, spaceW);
       strcatW(finaloutput, fullfilename);
     } else {
 
@@ -528,7 +520,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, const WCHAR *forVariable,
       WCHAR ext[MAX_PATH];
       BOOL doneFileModifier = FALSE;
 
-      if (finaloutput[0] != 0x00) strcatW(finaloutput, space);
+      if (finaloutput[0] != 0x00) strcatW(finaloutput, spaceW);
 
       /* Split into components */
       WCMD_splitpath(fullfilename, drive, dir, fname, ext);
@@ -565,7 +557,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, const WCHAR *forVariable,
       if (!doneFileModifier &&
           memchrW(firstModifier, 's', modifierLen) != NULL) {
         doneModifier = TRUE;
-        if (finaloutput[0] != 0x00) strcatW(finaloutput, space);
+        if (finaloutput[0] != 0x00) strcatW(finaloutput, spaceW);
         strcatW(finaloutput, outputparam);
       }
     }
@@ -588,7 +580,7 @@ void WCMD_call (WCHAR *command) {
 
   /* Run other program if no leading ':' */
   if (*command != ':') {
-    WCMD_run_program(command, 1);
+    WCMD_run_program(command, TRUE);
   } else {
 
     WCHAR gotoLabel[MAX_PATH];
@@ -605,7 +597,7 @@ void WCMD_call (WCHAR *command) {
       li.u.LowPart = SetFilePointer(context -> h, li.u.LowPart,
                      &li.u.HighPart, FILE_CURRENT);
 
-      WCMD_batch (param1, command, 1, gotoLabel, context->h);
+      WCMD_batch (param1, command, TRUE, gotoLabel, context->h);
 
       SetFilePointer(context -> h, li.u.LowPart,
                      &li.u.HighPart, FILE_BEGIN);
