@@ -26,81 +26,38 @@
  */
 
 #include "config.h"
+#include <time.h>
 #include "wcmd.h"
+#include "shellapi.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(cmd);
 
-const WCHAR inbuilt[][10] = {
-        {'C','A','L','L','\0'},
-        {'C','D','\0'},
-        {'C','H','D','I','R','\0'},
-        {'C','L','S','\0'},
-        {'C','O','P','Y','\0'},
-        {'C','T','T','Y','\0'},
-        {'D','A','T','E','\0'},
-        {'D','E','L','\0'},
-        {'D','I','R','\0'},
-        {'E','C','H','O','\0'},
-        {'E','R','A','S','E','\0'},
-        {'F','O','R','\0'},
-        {'G','O','T','O','\0'},
-        {'H','E','L','P','\0'},
-        {'I','F','\0'},
-        {'L','A','B','E','L','\0'},
-        {'M','D','\0'},
-        {'M','K','D','I','R','\0'},
-        {'M','O','V','E','\0'},
-        {'P','A','T','H','\0'},
-        {'P','A','U','S','E','\0'},
-        {'P','R','O','M','P','T','\0'},
-        {'R','E','M','\0'},
-        {'R','E','N','\0'},
-        {'R','E','N','A','M','E','\0'},
-        {'R','D','\0'},
-        {'R','M','D','I','R','\0'},
-        {'S','E','T','\0'},
-        {'S','H','I','F','T','\0'},
-        {'T','I','M','E','\0'},
-        {'T','I','T','L','E','\0'},
-        {'T','Y','P','E','\0'},
-        {'V','E','R','I','F','Y','\0'},
-        {'V','E','R','\0'},
-        {'V','O','L','\0'},
-        {'E','N','D','L','O','C','A','L','\0'},
-        {'S','E','T','L','O','C','A','L','\0'},
-        {'P','U','S','H','D','\0'},
-        {'P','O','P','D','\0'},
-        {'A','S','S','O','C','\0'},
-        {'C','O','L','O','R','\0'},
-        {'F','T','Y','P','E','\0'},
-        {'M','O','R','E','\0'},
-        {'C','H','O','I','C','E','\0'},
-        {'E','X','I','T','\0'}
-};
+extern const WCHAR inbuilt[][10];
+extern struct env_stack *pushd_directories;
 
-const WCHAR externals[NUM_EXTERNALS][10] = {
-        {'A','T','T','R','I','B','\0'},
-        {'X','C','O','P','Y','\0'}
-};
-
-HINSTANCE hinst;
+BATCH_CONTEXT *context = NULL;
 DWORD errorlevel;
+WCHAR quals[MAX_PATH], param1[MAXSTRING], param2[MAXSTRING];
+
 int defaultColor = 7;
 BOOL echo_mode = TRUE;
-static int opt_c, opt_k, opt_s;
-const WCHAR newline[] = {'\r','\n','\0'};
-static const WCHAR equalsW[] = {'=','\0'};
-static const WCHAR closeBW[] = {')','\0'};
-WCHAR anykey[100];
-WCHAR version_string[100];
-WCHAR quals[MAX_PATH], param1[MAXSTRING], param2[MAXSTRING];
-BATCH_CONTEXT *context = NULL;
-extern struct env_stack *pushd_directories;
+
+WCHAR anykey[100], version_string[100];
+const WCHAR newlineW[] = {'\r','\n','\0'};
+const WCHAR spaceW[]   = {' ','\0'};
+
+static BOOL opt_c, opt_k, opt_s, unicodeOutput = FALSE;
+
+/* Variables pertaining to paging */
+static BOOL paged_mode;
 static const WCHAR *pagedMessage = NULL;
-static char  *output_bufA = NULL;
+static int line_count;
+static int max_height;
+static int max_width;
+static int numChars;
+
 #define MAX_WRITECONSOLE_SIZE 65535
-static BOOL unicodePipes = FALSE;
 
 /*
  * Returns a buffer for reading from/writing to file
@@ -108,6 +65,7 @@ static BOOL unicodePipes = FALSE;
  */
 static char *get_file_buffer(void)
 {
+    static char *output_bufA = NULL;
     if (!output_bufA) {
         output_bufA = HeapAlloc(GetProcessHeap(), 0, MAX_WRITECONSOLE_SIZE);
         if (!output_bufA)
@@ -123,8 +81,8 @@ static char *get_file_buffer(void)
  *  and hence required WriteConsoleW to output it, however if file i/o is
  *  redirected, it needs to be WriteFile'd using OEM (not ANSI) format
  */
-static void WCMD_output_asis_len(const WCHAR *message, int len, HANDLE device) {
-
+static void WCMD_output_asis_len(const WCHAR *message, DWORD len, HANDLE device)
+{
     DWORD   nOut= 0;
     DWORD   res = 0;
 
@@ -141,7 +99,7 @@ static void WCMD_output_asis_len(const WCHAR *message, int len, HANDLE device) {
       DWORD convertedChars;
       char *buffer;
 
-      if (!unicodePipes) {
+      if (!unicodeOutput) {
 
         if (!(buffer = get_file_buffer()))
             return;
@@ -165,21 +123,25 @@ static void WCMD_output_asis_len(const WCHAR *message, int len, HANDLE device) {
  *
  */
 
-void WCMD_output (const WCHAR *format, ...) {
+void CDECL WCMD_output (const WCHAR *format, ...) {
 
-  va_list ap;
-  WCHAR string[1024];
-  int ret;
+  __ms_va_list ap;
+  WCHAR* string;
+  DWORD len;
 
-  va_start(ap,format);
-  ret = vsnprintfW(string, sizeof(string)/sizeof(WCHAR), format, ap);
-  if( ret >= (sizeof(string)/sizeof(WCHAR))) {
-       WINE_ERR("Output truncated\n" );
-       ret = (sizeof(string)/sizeof(WCHAR)) - 1;
-       string[ret] = '\0';
+  __ms_va_start(ap,format);
+  SetLastError(NO_ERROR);
+  string = NULL;
+  len = FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ALLOCATE_BUFFER,
+                       format, 0, 0, (LPWSTR)&string, 0, &ap);
+  __ms_va_end(ap);
+  if (len == 0 && GetLastError() != NO_ERROR)
+    WINE_FIXME("Could not format string: le=%u, fmt=%s\n", GetLastError(), wine_dbgstr_w(format));
+  else
+  {
+    WCMD_output_asis_len(string, len, GetStdHandle(STD_OUTPUT_HANDLE));
+    LocalFree(string);
   }
-  va_end(ap);
-  WCMD_output_asis_len(string, ret, GetStdHandle(STD_OUTPUT_HANDLE));
 }
 
 /*******************************************************************
@@ -187,28 +149,50 @@ void WCMD_output (const WCHAR *format, ...) {
  *
  */
 
-void WCMD_output_stderr (const WCHAR *format, ...) {
+void CDECL WCMD_output_stderr (const WCHAR *format, ...) {
 
-  va_list ap;
-  WCHAR string[1024];
-  int ret;
+  __ms_va_list ap;
+  WCHAR* string;
+  DWORD len;
 
-  va_start(ap,format);
-  ret = vsnprintfW(string, sizeof(string)/sizeof(WCHAR), format, ap);
-  if( ret >= (sizeof(string)/sizeof(WCHAR))) {
-       WINE_ERR("Output truncated\n" );
-       ret = (sizeof(string)/sizeof(WCHAR)) - 1;
-       string[ret] = '\0';
+  __ms_va_start(ap,format);
+  SetLastError(NO_ERROR);
+  string = NULL;
+  len = FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ALLOCATE_BUFFER,
+                       format, 0, 0, (LPWSTR)&string, 0, &ap);
+  __ms_va_end(ap);
+  if (len == 0 && GetLastError() != NO_ERROR)
+    WINE_FIXME("Could not format string: le=%u, fmt=%s\n", GetLastError(), wine_dbgstr_w(format));
+  else
+  {
+    WCMD_output_asis_len(string, len, GetStdHandle(STD_ERROR_HANDLE));
+    LocalFree(string);
   }
-  va_end(ap);
-  WCMD_output_asis_len(string, ret, GetStdHandle(STD_ERROR_HANDLE));
 }
 
-static int line_count;
-static int max_height;
-static int max_width;
-static BOOL paged_mode;
-static int numChars;
+/*******************************************************************
+ * WCMD_format_string - allocate a buffer and format a string
+ *
+ */
+
+WCHAR* CDECL WCMD_format_string (const WCHAR *format, ...) {
+
+  __ms_va_list ap;
+  WCHAR* string;
+  DWORD len;
+
+  __ms_va_start(ap,format);
+  SetLastError(NO_ERROR);
+  len = FormatMessageW(FORMAT_MESSAGE_FROM_STRING|FORMAT_MESSAGE_ALLOCATE_BUFFER,
+                       format, 0, 0, (LPWSTR)&string, 0, &ap);
+  __ms_va_end(ap);
+  if (len == 0 && GetLastError() != NO_ERROR) {
+    WINE_FIXME("Could not format string: le=%u, fmt=%s\n", GetLastError(), wine_dbgstr_w(format));
+    string = (WCHAR*)LocalAlloc(LMEM_FIXED, 2);
+    *string = 0;
+  }
+  return string;
+}
 
 void WCMD_enter_paged_mode(const WCHAR *msg)
 {
@@ -278,14 +262,12 @@ static void WCMD_output_asis_handle (DWORD std_handle, const WCHAR *message) {
         ptr++;
       };
       if (*ptr == '\n') ptr++;
-      WCMD_output_asis_len(message, (ptr) ? ptr - message : strlenW(message), handle);
-      if (ptr) {
-        numChars = 0;
-        if (++line_count >= max_height - 1) {
-          line_count = 0;
-          WCMD_output_asis_len(pagedMessage, strlenW(pagedMessage), handle);
-          WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), string, sizeof(string)/sizeof(WCHAR), &count);
-        }
+      WCMD_output_asis_len(message, ptr - message, handle);
+      numChars = 0;
+      if (++line_count >= max_height - 1) {
+        line_count = 0;
+        WCMD_output_asis_len(pagedMessage, strlenW(pagedMessage), handle);
+        WCMD_ReadFile(GetStdHandle(STD_INPUT_HANDLE), string, sizeof(string)/sizeof(WCHAR), &count);
       }
     } while (((message = ptr) != NULL) && (*ptr));
   } else {
@@ -336,7 +318,7 @@ void WCMD_print_error (void) {
   WCMD_output_asis_len(lpMsgBuf, lstrlenW(lpMsgBuf),
                        GetStdHandle(STD_ERROR_HANDLE));
   LocalFree (lpMsgBuf);
-  WCMD_output_asis_len (newline, lstrlenW(newline),
+  WCMD_output_asis_len (newlineW, lstrlenW(newlineW),
                         GetStdHandle(STD_ERROR_HANDLE));
   return;
 }
@@ -506,11 +488,11 @@ BOOL WCMD_keyword_ws_found(const WCHAR *keyword, int len, const WCHAR *ptr) {
 }
 
 /*************************************************************************
- * WCMD_opt_s_strip_quotes
+ * WCMD_strip_quotes
  *
  *	Remove first and last quote WCHARacters, preserving all other text
  */
-void WCMD_opt_s_strip_quotes(WCHAR *cmd) {
+void WCMD_strip_quotes(WCHAR *cmd) {
   WCHAR *src = cmd + 1, *dest = cmd, *lastq = NULL;
   while((*dest=*src) != '\0') {
       if (*src=='\"')
@@ -745,11 +727,9 @@ static WCHAR *WCMD_expand_envvar(WCHAR *start,
         WCMD_strsubstW(start, endOfVar + 1, startCopy, substrlength);
       }
 
-      return start;
-
     /* search and replace manipulation */
     } else {
-      WCHAR *equalspos = strstrW(colonpos, equalsW);
+      WCHAR *equalspos = strstrW(colonpos, equalW);
       WCHAR *replacewith = equalspos+1;
       WCHAR *found       = NULL;
       WCHAR *searchIn;
@@ -806,9 +786,8 @@ static WCHAR *WCMD_expand_envvar(WCHAR *start,
       HeapFree(GetProcessHeap(), 0, s);
       HeapFree(GetProcessHeap(), 0, searchIn);
       HeapFree(GetProcessHeap(), 0, searchFor);
-      return start;
     }
-    return start+1;
+    return start;
 }
 
 /*****************************************************************************
@@ -858,7 +837,7 @@ static void handleExpansion(WCHAR *cmd, BOOL justFors,
     /* Replace use of %* if in batch program*/
     } else if (!justFors && context && *(p+1)=='*') {
       WCHAR *startOfParms = NULL;
-      t = WCMD_parameter(context -> command, 1, &startOfParms, NULL);
+      WCMD_parameter(context -> command, 1, &startOfParms, NULL);
       if (startOfParms != NULL)
         WCMD_strsubstW(p, p+2, startOfParms, -1);
       else
@@ -1014,8 +993,8 @@ static void init_msvcrt_io_block(STARTUPINFOW* st)
  *          findexecutable to achieve this which is left untouched.
  */
 
-void WCMD_run_program (WCHAR *command, int called) {
-
+void WCMD_run_program (WCHAR *command, BOOL called)
+{
   WCHAR  temp[MAX_PATH];
   WCHAR  pathtosearch[MAXSTRING];
   WCHAR *pathposn;
@@ -1032,7 +1011,12 @@ void WCMD_run_program (WCHAR *command, int called) {
   static const WCHAR envPathExt[] = {'P','A','T','H','E','X','T','\0'};
   static const WCHAR delims[] = {'/','\\',':','\0'};
 
-  WCMD_parse (command, quals, param1, param2);	/* Quick way to get the filename */
+  /* Quick way to get the filename
+   * (but handle leading / as part of program name, not qualifier)
+   */
+  for (len = 0; command[len] == '/'; len++) param1[len] = '/';
+  WCMD_parse (command + len, quals, param1 + len, param2);
+
   if (!(*param1) && !(*param2))
     return;
 
@@ -1086,7 +1070,6 @@ void WCMD_run_program (WCHAR *command, int called) {
     WCHAR  thisDir[MAX_PATH] = {'\0'};
     WCHAR *pos               = NULL;
     BOOL  found             = FALSE;
-    static const WCHAR slashW[] = {'\\','\0'};
 
     /* Work on the first directory on the search path */
     pos = strchrW(pathposn, ';');
@@ -1177,10 +1160,7 @@ void WCMD_run_program (WCHAR *command, int called) {
       launched = TRUE;
 
       /* Special case BAT and CMD */
-      if (ext && !strcmpiW(ext, batExt)) {
-        WCMD_batch (thisDir, command, called, NULL, INVALID_HANDLE_VALUE);
-        return;
-      } else if (ext && !strcmpiW(ext, cmdExt)) {
+      if (ext && (!strcmpiW(ext, batExt) || !strcmpiW(ext, cmdExt))) {
         WCMD_batch (thisDir, command, called, NULL, INVALID_HANDLE_VALUE);
         return;
       } else {
@@ -1204,8 +1184,8 @@ void WCMD_run_program (WCHAR *command, int called) {
         if ((opt_c || opt_k) && !opt_s && !status
             && GetLastError()==ERROR_FILE_NOT_FOUND && command[0]=='\"') {
           /* strip first and last quote WCHARacters and try again */
-          WCMD_opt_s_strip_quotes(command);
-          opt_s=1;
+          WCMD_strip_quotes(command);
+          opt_s = TRUE;
           WCMD_run_program(command, called);
           return;
         }
@@ -1334,7 +1314,7 @@ void WCMD_execute (const WCHAR *command, const WCHAR *redirects,
       /* According to MSDN CreateProcess docs, special env vars record
          the current directory on each drive, in the form =C:
          so see if one specified, and if so go back to it             */
-      strcpyW(envvar, equalsW);
+      strcpyW(envvar, equalW);
       strcatW(envvar, cmd);
       if (GetEnvironmentVariableW(envvar, dir, MAX_PATH) == 0) {
         static const WCHAR fmt[] = {'%','s','\\','\0'};
@@ -1556,6 +1536,9 @@ void WCMD_execute (const WCHAR *command, const WCHAR *redirects,
       case WCMD_SHIFT:
         WCMD_shift (p);
         break;
+      case WCMD_START:
+        WCMD_start (p);
+        break;
       case WCMD_TIME:
         WCMD_setshow_time ();
         break;
@@ -1567,7 +1550,7 @@ void WCMD_execute (const WCHAR *command, const WCHAR *redirects,
         WCMD_type (p);
 	break;
       case WCMD_VER:
-        WCMD_output(newline);
+        WCMD_output_asis(newlineW);
         WCMD_version ();
         break;
       case WCMD_VERIFY:
@@ -1602,7 +1585,7 @@ void WCMD_execute (const WCHAR *command, const WCHAR *redirects,
         break;
       default:
         prev_echo_mode = echo_mode;
-        WCMD_run_program (whichcmd, 0);
+        WCMD_run_program (whichcmd, FALSE);
         echo_mode = prev_echo_mode;
     }
     HeapFree( GetProcessHeap(), 0, cmd );
@@ -1830,14 +1813,13 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
     if (strlenW(extraSpace) == MAXSTRING -1) {
         WCMD_output_asis_stderr(WCMD_LoadMessage(WCMD_TRUNCATEDLINE));
         WCMD_output_asis_stderr(extraSpace);
-        WCMD_output_asis_stderr(newline);
+        WCMD_output_asis_stderr(newlineW);
     }
 
     /* Replace env vars if in a batch context */
     if (context) handleExpansion(extraSpace, FALSE, NULL, NULL);
     /* Show prompt before batch line IF echo is on and in batch program */
     if (context && echo_mode && extraSpace[0] && (extraSpace[0] != '@')) {
-      static const WCHAR spc[]={' ','\0'};
       static const WCHAR echoDot[] = {'e','c','h','o','.'};
       static const WCHAR echoCol[] = {'e','c','h','o',':'};
       const DWORD len = sizeof(echoDot)/sizeof(echoDot[0]);
@@ -1852,9 +1834,9 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
           && CompareStringW(LOCALE_SYSTEM_DEFAULT, NORM_IGNORECASE,
                          extraSpace, min_len, echoCol, len) != CSTR_EQUAL)
       {
-          WCMD_output_asis(spc);
+          WCMD_output_asis(spaceW);
       }
-      WCMD_output_asis(newline);
+      WCMD_output_asis(newlineW);
     }
 
     /* Start with an empty string, copying to the command string */
@@ -2190,7 +2172,7 @@ WCHAR *WCMD_ReadAndParseLine(const WCHAR *optionalcmd, CMD_LIST **output, HANDLE
         /* Continue to echo commands IF echo is on and in batch program */
         if (context && echo_mode && extraSpace[0] && (extraSpace[0] != '@')) {
           WCMD_output_asis(extraSpace);
-          WCMD_output_asis(newline);
+          WCMD_output_asis(newlineW);
         }
       }
     }
@@ -2270,26 +2252,26 @@ void WCMD_free_commands(CMD_LIST *cmds) {
 int wmain (int argc, WCHAR *argvW[])
 {
   int     args;
-  WCHAR  *cmd   = NULL;
+  WCHAR  *cmd;
   WCHAR string[1024];
   WCHAR envvar[4];
-  int opt_q;
+  BOOL opt_q;
   int opt_t = 0;
   static const WCHAR promptW[] = {'P','R','O','M','P','T','\0'};
   static const WCHAR defaultpromptW[] = {'$','P','$','G','\0'};
-  char ansiVersion[100];
   CMD_LIST *toExecute = NULL;         /* Commands left to be executed */
 
   srand(time(NULL));
 
   /* Pre initialize some messages */
-  strcpy(ansiVersion, PACKAGE_VERSION);
-  MultiByteToWideChar(CP_ACP, 0, ansiVersion, -1, string, 1024);
-  wsprintfW(version_string, WCMD_LoadMessage(WCMD_VERSION), string);
   strcpyW(anykey, WCMD_LoadMessage(WCMD_ANYKEY));
+  cmd = WCMD_format_string(WCMD_LoadMessage(WCMD_VERSION), PACKAGE_VERSION);
+  strcpyW(version_string, cmd);
+  LocalFree(cmd);
+  cmd = NULL;
 
   args  = argc;
-  opt_c=opt_k=opt_q=opt_s=0;
+  opt_c = opt_k = opt_q = opt_s = FALSE;
   while (args > 0)
   {
       WCHAR c;
@@ -2302,17 +2284,17 @@ int wmain (int argc, WCHAR *argvW[])
 
       c=(*argvW)[1];
       if (tolowerW(c)=='c') {
-          opt_c=1;
+          opt_c = TRUE;
       } else if (tolowerW(c)=='q') {
-          opt_q=1;
+          opt_q = TRUE;
       } else if (tolowerW(c)=='k') {
-          opt_k=1;
+          opt_k = TRUE;
       } else if (tolowerW(c)=='s') {
-          opt_s=1;
+          opt_s = TRUE;
       } else if (tolowerW(c)=='a') {
-          unicodePipes=FALSE;
+          unicodeOutput = FALSE;
       } else if (tolowerW(c)=='u') {
-          unicodePipes=TRUE;
+          unicodeOutput = TRUE;
       } else if (tolowerW(c)=='t' && (*argvW)[2]==':') {
           opt_t=strtoulW(&(*argvW)[3], NULL, 16);
       } else if (tolowerW(c)=='x' || tolowerW(c)=='y') {
@@ -2354,19 +2336,19 @@ int wmain (int argc, WCHAR *argvW[])
       argsLeft = args;
       for (arg = argvW; argsLeft>0; arg++,argsLeft--)
       {
-          int has_space,bcount;
+          BOOL has_space = FALSE;
+          int bcount;
           WCHAR* a;
 
-          has_space=0;
           bcount=0;
           a=*arg;
-          if( !*a ) has_space=1;
+          if( !*a ) has_space = TRUE;
           while (*a!='\0') {
               if (*a=='\\') {
                   bcount++;
               } else {
                   if (*a==' ' || *a=='\t') {
-                      has_space=1;
+                      has_space = TRUE;
                   } else if (*a=='"') {
                       /* doubling of '\' preceding a '"',
                        * plus escaping of said '"'
@@ -2387,20 +2369,20 @@ int wmain (int argc, WCHAR *argvW[])
       }
 
       if (qcount!=2)
-          opt_s=1;
+          opt_s = TRUE;
 
       /* check argvW[0] for a space and invalid characters */
       if (!opt_s) {
-          opt_s=1;
+          opt_s = TRUE;
           p=*argvW;
           while (*p!='\0') {
               if (*p=='&' || *p=='<' || *p=='>' || *p=='(' || *p==')'
                   || *p=='@' || *p=='^' || *p=='|') {
-                  opt_s=1;
+                  opt_s = TRUE;
                   break;
               }
               if (*p==' ')
-                  opt_s=0;
+                  opt_s = FALSE;
               p++;
           }
       }
@@ -2413,20 +2395,19 @@ int wmain (int argc, WCHAR *argvW[])
       argsLeft = args;
       for (arg = argvW; argsLeft>0; arg++,argsLeft--)
       {
-          int has_space,has_quote;
+          BOOL has_space = FALSE, has_quote = FALSE;
           WCHAR* a;
 
           /* Check for quotes and spaces in this argument */
-          has_space=has_quote=0;
           a=*arg;
-          if( !*a ) has_space=1;
+          if( !*a ) has_space = TRUE;
           while (*a!='\0') {
               if (*a==' ' || *a=='\t') {
-                  has_space=1;
+                  has_space = TRUE;
                   if (has_quote)
                       break;
               } else if (*a=='"') {
-                  has_quote=1;
+                  has_quote = TRUE;
                   if (has_space)
                       break;
               }
@@ -2478,7 +2459,7 @@ int wmain (int argc, WCHAR *argvW[])
       /* strip first and last quote characters if opt_s; check for invalid
        * executable is done later */
       if (opt_s && *cmd=='\"')
-          WCMD_opt_s_strip_quotes(cmd);
+          WCMD_strip_quotes(cmd);
   }
 
   if (opt_c) {
