@@ -34,8 +34,8 @@ UINT g_uiDefaultCharset;
 const COLORREF WCUSER_ColorMap[16] =
 {
     RGB(0x00, 0x00, 0x00), RGB(0x00, 0x00, 0x80), RGB(0x00, 0x80, 0x00), RGB(0x00, 0x80, 0x80),
-    RGB(0x80, 0x00, 0x00), RGB(0x80, 0x00, 0x80), RGB(0x80, 0x80, 0x00), RGB(0x80, 0x80, 0x80),
-    RGB(0xC0, 0xC0, 0xC0), RGB(0x00, 0x00, 0xFF), RGB(0x00, 0xFF, 0x00), RGB(0x00, 0xFF, 0xFF),
+    RGB(0x80, 0x00, 0x00), RGB(0x80, 0x00, 0x80), RGB(0x80, 0x80, 0x00), RGB(0xC0, 0xC0, 0xC0),
+    RGB(0x80, 0x80, 0x80), RGB(0x00, 0x00, 0xFF), RGB(0x00, 0xFF, 0x00), RGB(0x00, 0xFF, 0xFF),
     RGB(0xFF, 0x00, 0x00), RGB(0xFF, 0x00, 0xFF), RGB(0xFF, 0xFF, 0x00), RGB(0xFF, 0xFF, 0xFF),
 };
 
@@ -447,7 +447,7 @@ HFONT WCUSER_CopyFont(struct config_data* config, HWND hWnd, const LOGFONTW* lf,
      *  - the width of all characters in the font
      * This isn't true in Wine. As a temporary workaround, we get as the width of the
      * cell, the width of the first character in the font, after checking that all
-     * characters in the font have the same width (I hear paranoïa coming)
+     * characters in the font have the same width (I hear paranoia coming)
      * when this gets fixed, the code should be using tm.tmAveCharWidth
      * or tm.tmMaxCharWidth as the cell width.
      */
@@ -1045,7 +1045,7 @@ static void    WCUSER_GenerateKeyInputRecord(struct inner_data* data, BOOL down,
         {
         case 2:
             /* FIXME... should generate two events... */
-            /* fall thru */
+            /* fall through */
         case 1:
             last = buf[0];
             break;
