@@ -322,7 +322,7 @@ struct thread *create_process( int fd, struct thread *parent_thread, int inherit
     process->priority        = PROCESS_PRIOCLASS_NORMAL;
     process->suspend         = 0;
     process->is_system       = 0;
-    process->create_flags    = 0;
+    process->debug_children  = 0;
     process->console         = NULL;
     process->startup_state   = STARTUP_IN_PROGRESS;
     process->startup_info    = NULL;
@@ -847,6 +847,7 @@ struct process_snapshot *process_snap( int *count )
         ptr->count    = process->obj.refcount;
         ptr->priority = process->priority;
         ptr->handles  = get_handle_table_count(process);
+        ptr->unix_pid = process->unix_pid;
         grab_object( process );
         ptr++;
     }
@@ -886,6 +887,12 @@ DECL_HANDLER(new_process)
         return;
     }
 
+    if (!req->info_size)  /* create an orphaned process */
+    {
+        create_process( socket_fd, NULL, 0 );
+        return;
+    }
+
     /* build the startup info for a new process */
     if (!(info = alloc_object( &startup_info_ops ))) return;
     info->exe_file = NULL;
@@ -931,7 +938,7 @@ DECL_HANDLER(new_process)
 
     if (!(thread = create_process( socket_fd, current, req->inherit_all ))) goto done;
     process = thread->process;
-    process->create_flags = req->create_flags;
+    process->debug_children = !(req->create_flags & DEBUG_ONLY_THIS_PROCESS);
     process->startup_info = (struct startup_info *)grab_object( info );
 
     /* connect to the window station */
@@ -966,7 +973,7 @@ DECL_HANDLER(new_process)
     /* attach to the debugger if requested */
     if (req->create_flags & (DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS))
         set_process_debugger( process, current );
-    else if (parent->debugger && !(parent->create_flags & DEBUG_ONLY_THIS_PROCESS))
+    else if (parent->debugger && parent->debug_children)
         set_process_debugger( process, parent->debugger );
 
     if (!(req->create_flags & CREATE_NEW_PROCESS_GROUP))
@@ -1045,7 +1052,7 @@ DECL_HANDLER(init_process_done)
     set_process_startup_state( process, STARTUP_DONE );
 
     if (req->gui) process->idle_event = create_event( NULL, NULL, 0, 1, 0, NULL );
-    if (current->suspend + process->suspend > 0) stop_thread( current );
+    stop_thread_if_suspended( current );
     if (process->debugger) set_process_debug_flag( process, 1 );
 }
 
